#ifndef __MPARSER_ANTLRGRAMMARPARSEBEHAVIOR_H__
#define __MPARSER_ANTLRGRAMMARPARSEBEHAVIOR_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MParser/BitSet.h"
#include "MParser/Token.h"

class ANTLRGrammarParseBehavior;
typedef class ANTLRGrammarParseBehavior *ANTLRGrammarParseBehavior_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/ANTLRGrammarParseBehavior.java#1 $
 */

class ANTLRGrammarParseBehavior : public MLANG_Refcountable<ACE_SYNCH_MUTEX>

{
protected:

    ANTLRGrammarParseBehavior();

public:

    virtual ~ANTLRGrammarParseBehavior();

public:

   static bool is_nil (ANTLRGrammarParseBehavior_ptr behavior)
   { return behavior == 0; }

   static ANTLRGrammarParseBehavior_ptr _nil()
   { return 0; }

   static ANTLRGrammarParseBehavior_ptr _duplicate(ANTLRGrammarParseBehavior_ptr behavior)
   { return ::acquire(behavior); }
public:

    virtual void abortGrammar() = 0;

    virtual void beginAlt(bool doAST_) = 0;

    virtual void beginChildList() = 0;

    // Exception handling
    virtual void beginExceptionGroup() = 0;

    virtual void beginExceptionSpec(const MParser_Token_var& label) = 0;

    virtual void beginSubRule(const MParser_Token_var& label, const MParser_Token_var& start, bool no) = 0;

    // Trees
    virtual void beginTree(const MParser_Token_var& tok) = 0;

    virtual void defineRuleName(const MParser_Token_var& r, const MLANG_String& access, bool ruleAST, const MLANG_String& docComment) = 0;

    virtual void defineToken(const MParser_Token_var& tokname, const MParser_Token_var& tokliteral) = 0;

    virtual void endAlt() = 0;

    virtual void endChildList() = 0;

    virtual void endExceptionGroup() = 0;

    virtual void endExceptionSpec() = 0;

    virtual void endGrammar() = 0;

    virtual void endOptions() = 0;

    virtual void endRule(const MLANG_String& r) = 0;

    virtual void endSubRule() = 0;

    virtual void endTree() = 0;

    virtual void hasError() = 0;

    virtual void noASTSubRule() = 0;

    virtual void oneOrMoreSubRule() = 0;

    virtual void optionalSubRule() = 0;

    virtual void refAction(const MParser_Token_var& action) = 0;

    virtual void refArgAction(const MParser_Token_var& action) = 0;

    virtual void setUserExceptions(const MLANG_String& thr) = 0;

    virtual void refCharLiteral(const MParser_Token_var& lit, const MParser_Token_var& label, bool inverted, int autoGenType, bool lastInRule) = 0;

    virtual void refCharRange(const MParser_Token_var& t1, const MParser_Token_var& t2, const MParser_Token_var& label, int autoGenType, bool lastInRule) = 0;

    virtual void refElementOption(const MParser_Token_var& option, const MParser_Token_var& value) = 0;

    virtual void refTokensSpecElementOption(const MParser_Token_var& tok, const MParser_Token_var& option, const MParser_Token_var& value) = 0;

    virtual void refExceptionHandler(const MParser_Token_var& exTypeAndName, const MParser_Token_var& action) = 0;

    virtual void refHeaderAction(const MParser_Token_var& name, const MParser_Token_var& act) = 0;

    virtual void refInitAction(const MParser_Token_var& action) = 0;

    virtual void refMemberAction(const MParser_Token_var& act) = 0;

    virtual void refPreambleAction(const MParser_Token_var& act) = 0;

    virtual void refReturnAction(const MParser_Token_var& returnAction) = 0;

    virtual void refRule(const MParser_Token_var& idAssign, const MParser_Token_var& r, const MParser_Token_var& label, const MParser_Token_var& arg, int autoGenType) = 0;

    virtual void refSemPred(const MParser_Token_var& pred) = 0;

    virtual void refStringLiteral(const MParser_Token_var& lit, const MParser_Token_var& label, int autoGenType, bool lastInRule) = 0;

    virtual void refToken(const MParser_Token_var& assignId, const MParser_Token_var& t, const MParser_Token_var& label, const MParser_Token_var& args,
                         bool inverted, int autoGenType, bool lastInRule) = 0;

    virtual void refTokenRange(const MParser_Token_var& t1, const MParser_Token_var& t2, const MParser_Token_var& label, int autoGenType, bool lastInRule) = 0;

    // Tree specifiers
    virtual void refTreeSpecifier(const MParser_Token_var& treeSpec) = 0;

    virtual void refWildcard(const MParser_Token_var& t, const MParser_Token_var& label, int autoGenType) = 0;

    virtual void setArgOfRuleRef(const MParser_Token_var& argaction) = 0;

    virtual void setCharVocabulary(const MParser_BitSet& b) = 0;

    // Options
    virtual void setFileOption(const MParser_Token_var& key, const MParser_Token_var& value, const MLANG_String& filename) = 0;

    virtual void setGrammarOption(const MParser_Token_var& key, const MParser_Token_var& value) = 0;

    virtual void setRuleOption(const MParser_Token_var& key, const MParser_Token_var& value) = 0;

    virtual void setSubruleOption(const MParser_Token_var& key, const MParser_Token_var& value) = 0;

    virtual void startLexer(const MLANG_String& file, const MParser_Token_var& name, const MLANG_String& superClass, const MLANG_String& doc) = 0;

    // Flow control for grammars
    virtual void startParser(const MLANG_String& file, const MParser_Token_var& name, const MLANG_String& superClass, const MLANG_String& doc) = 0;

    virtual void startTreeWalker(const MLANG_String& file, const MParser_Token_var& name, const MLANG_String& superClass, const MLANG_String& doc) = 0;

    virtual void synPred() = 0;

    virtual void zeroOrMoreSubRule() = 0;
};

typedef MLANG_SmartPtr<ANTLRGrammarParseBehavior> ANTLRGrammarParseBehavior_var;

#endif /* __MPARSER_ANTLRGRAMMARPARSEBEHAVIOR_H__ */
