#ifndef __MGRAMMAR_ANTLRLEXER_H__
#define __MGRAMMAR_ANTLRLEXER_H__
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MIO/Reader.h"
#include "MIO/InputStream.h"
#include "MParser/BitSet.h"
#include "MParser/InputBuffer.h"
#include "MParser/LexerSharedInputState.h"
#include "MParser/TokenStream.h"
#include "MParser/CharScanner.h"
#include "MGrammar/ANTLRTokenTypes.h"

class ANTLRLexer;
typedef class ANTLRLexer *ANTLRLexer_ptr;

class ANTLRLexer : public MParser_CharScanner,
                   public ANTLRTokenTypes
{
public:
   typedef MParser_CharScanner super;
   /**Convert 'c' to an integer char value. */
public:
   static int escapeCharValue(const MLANG_String& cs);

public:
   static int tokenTypeForCharLiteral(const MLANG_String& lit);

public:
   ANTLRLexer(const MIO_InputStream_var& in);

public:
   ANTLRLexer(const MIO_Reader_var& in);

public:
   ANTLRLexer(const MParser_InputBuffer_var& ib);

public:
   ANTLRLexer(const MParser_LexerSharedInputState_var& state);

public:
   virtual ~ANTLRLexer();

public:
   static bool is_nil (ANTLRLexer_ptr lexer)
   { return lexer == 0; }

   static ANTLRLexer_ptr _nil()
   { return 0; }

   static ANTLRLexer_ptr _duplicate(ANTLRLexer_ptr lexer)
   { return ::MLANG_acquire(lexer); }

public:
   MParser_Token_var nextToken();

public:
   void mWS(bool createToken);
	
public:
   void mCOMMENT(bool createToken);

protected:
   void mSL_COMMENT(bool createToken);
	
protected:
   void mML_COMMENT(bool createToken);
	
public:
   void mOPEN_ELEMENT_OPTION(bool createToken);
	
public:
   void mCLOSE_ELEMENT_OPTION(bool createToken);
	
public:
   void mCOMMA(bool createToken);
	
public:
   void mQUESTION(bool createToken);
	
public:
   void mTREE_BEGIN(bool createToken);
	
public:
   void mLPAREN(bool createToken);
	
public:
   void mRPAREN(bool createToken);
	
public:
   void mCOLON(bool createToken);
	
public:
   void mSTAR(bool createToken);
	
public:
   void mPLUS(bool createToken);
	
public:
   void mASSIGN(bool createToken);
	
public:
   void mIMPLIES(bool createToken);
	
public:
   void mSEMI(bool createToken);
	
public:
   void mCARET(bool createToken);
	
public:
   void mBANG(bool createToken);
	
public:
   void mOR(bool createToken);
	
public:
   void mWILDCARD(bool createToken);
	
public:
   void mRANGE(bool createToken);
	
public:
   void mNOT_OP(bool createToken);
	
public:
   void mRCURLY(bool createToken);
	
public:
   void mCHAR_LITERAL(bool createToken);
	
protected:
   void mESC(bool createToken);
	
public:
   void mSTRING_LITERAL(bool createToken);
	
protected:
   void mXDIGIT(bool createToken);
	
protected:
   void mDIGIT(bool createToken);
	
public:
   void mINT(bool createToken);
	
public:
   void mARG_ACTION(bool createToken);
	
protected:
   void mNESTED_ARG_ACTION(bool createToken);
	
public:
   void mACTION(bool createToken);
	
protected:
   void mNESTED_ACTION(bool createToken);
	
public:
   void mTOKEN_REF(bool createToken);
	
public:
   void mRULE_REF(bool createToken);
	
protected:
   int  mINTERNAL_RULE_REF(bool createToken);
	
protected:
   void mWS_LOOP(bool createToken);
	
protected:
   void mWS_OPT(bool createToken);

protected:
   void init();
	
private:
   static longArray mk_tokenSet_0();
public:
   static MParser_BitSet _tokenSet_0;
private:
   static longArray mk_tokenSet_1();
public:
   static MParser_BitSet _tokenSet_1;
private:
   static longArray mk_tokenSet_2();
public:
   static MParser_BitSet _tokenSet_2;
private:
   static longArray mk_tokenSet_3();
public:
   static MParser_BitSet _tokenSet_3;
private:
   static longArray mk_tokenSet_4();
public:
   static MParser_BitSet _tokenSet_4;
private:
   static longArray mk_tokenSet_5();
public:
   static MParser_BitSet _tokenSet_5;
};

typedef MLANG_SmartPtr<ANTLRLexer> ANTLRLexer_var;

#endif /* __MGRAMMAR_ANTLRLEXER_H__ */
