#include "MLANG/String.h"
#include "MParser/StringUtils.h"
#include "MLANG/StringBuffer.h"
#include "MGrammar/Tool.h"
#include "MParser/CommonToken.h"
#include "MParser/NoViableAltException.h"
#include "MParser/CharScanner.h"
#include "MGrammar/ANTLRLexer.h"
#include "MGrammar/GrammarParseBehavior.h"
#include "MGrammar/ANTLRParser.h"

// $ANTLR 2.7.4: "antlr.g" -> "ANTLRParser.java"$

bool ANTLRParser::DEBUG_PARSER = false;

const char *ANTLRParser::_tokenNames[] = {
   "<0>",
   "EOF",
   "<2>",
   "NULL_TREE_LOOKAHEAD",
   "\"tokens\"",
   "\"header\"",
   "STRING_LITERAL",
   "ACTION",
   "DOC_COMMENT",
   "\"lexclass\"",
   "\"class\"",
   "\"extends\"",
   "\"Lexer\"",
   "\"TreeParser\"",
   "OPTIONS",
   "ASSIGN",
   "SEMI",
   "RCURLY",
   "\"charVocabulary\"",
   "CHAR_LITERAL",
   "INT",
   "OR",
   "RANGE",
   "TOKENS",
   "TOKEN_REF",
   "OPEN_ELEMENT_OPTION",
   "CLOSE_ELEMENT_OPTION",
   "LPAREN",
   "RPAREN",
   "\"Parser\"",
   "\"protected\"",
   "\"public\"",
   "\"private\"",
   "BANG",
   "ARG_ACTION",
   "\"returns\"",
   "COLON",
   "\"throws\"",
   "COMMA",
   "\"exception\"",
   "\"catch\"",
   "RULE_REF",
   "NOT_OP",
   "SEMPRED",
   "TREE_BEGIN",
   "QUESTION",
   "STAR",
   "PLUS",
   "IMPLIES",
   "CARET",
   "WILDCARD",
   "\"options\"",
   "WS",
   "COMMENT",
   "SL_COMMENT",
   "ML_COMMENT",
   "ESC",
   "DIGIT",
   "XDIGIT",
   "NESTED_ARG_ACTION",
   "NESTED_ACTION",
   "WS_LOOP",
   "INTERNAL_RULE_REF",
   "WS_OPT"
};

ANTLRParser::ANTLRParser (
   const MParser_TokenBuffer_var& tokenBuf,
   const MGrammar_GrammarParseBehavior_var& behavior,
   MGrammar_Tool_ptr tool
) : super(tokenBuf, 1),
    blockNesting_(-1)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
   behavior_ = behavior;
   antlrTool_ = tool;
}

void ANTLRParser::reportError(const MLANG_String& s)

{
   antlrTool_->error(s, getFilename(), -1, -1);
}

void ANTLRParser::reportError (
   MParser_RecognitionException& e
)

{
   reportError(e, e.getErrorMessage().c_str());
}

void ANTLRParser::reportError (
   MParser_RecognitionException& e,
   const MLANG_String& s
)

{
   antlrTool_->error(s, e.getFilename(), e.getLine(), e.getColumn());
}

void ANTLRParser::reportWarning(const MLANG_String& s)

{
   antlrTool_->warning(s, getFilename(), -1, -1);
}

bool ANTLRParser::lastInRule()

{
   if ( blockNesting_==0 && (LA(1)==SEMI || LA(1)==LITERAL_exception || LA(1)==OR) ) {
   return true;
   }
   return false;
}

void ANTLRParser::checkForMissingEndRule(const MParser_Token_var& label) {
   if ( label->getColumn()==1 ) {
   antlrTool_->warning("did you forget to terminate previous rule?", getFilename(), label->getLine(), label->getColumn());
   }
}

ANTLRParser::ANTLRParser (
   const MParser_TokenBuffer_var& tokenBuf,
   int k
)
  : super(tokenBuf,k)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

ANTLRParser::ANTLRParser (
   const MParser_TokenStream_var& lexer,
   int k
)
  : super(lexer,k)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

ANTLRParser::ANTLRParser (const MParser_ParserSharedInputState_var& state)
  : super(state,2)

{ 
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

void ANTLRParser::grammar() {
   
   MParser_Token_var  n = MParser_Token::_nil();
   MParser_Token_var  h = MParser_Token::_nil();
   
   try {      // for error handling
   {
   LABEL(_loop4)
   do {
   	if ((LA(1)==LITERAL_header)) {
   		if ( inputState_->guessing()==0 ) {
   		
   				n = MParser_Token::_nil();	// RK: prevent certain orders of header actions
   							// overwriting eachother.
   				
   		}
   		match(LITERAL_header);
   		{
   		switch ( LA(1)) {
   		case STRING_LITERAL:
   		{
   		n = LT(1);
   		match(STRING_LITERAL);
   		break;
   		}
   		case ACTION:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		h = LT(1);
   		match(ACTION);
   		if ( inputState_->guessing()==0 ) {
   		
   				// store the header action
   				// FIXME: 'n' should be checked for validity
   				behavior_->refHeaderAction(n,h);
   				
   		}
   	}
   	else {
   		BREAK(_loop4);
   	}
   	
   } while (true);
   }
   {
   switch ( LA(1)) {
   case OPTIONS:
   {
   	fileOptionsSpec();
   	break;
   }
   case EOF:
   case ACTION:
   case DOC_COMMENT:
   case LITERAL_lexclass:
   case LITERAL_class:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   LABEL(_loop7)
   do {
   	if (((LA(1) >= ACTION && LA(1) <= LITERAL_class))) {
   		classDef();
   	}
   	else {
   		BREAK(_loop7);
   	}
   	
   } while (true);
   }
   match(MParser_Token::EOF_TYPE);
   }
   catch (MParser_RecognitionException& ex) {
   if (inputState_->guessing()==0) {
   	
   	reportError(ex, MLANG_String("rule grammar trapped:\n")+ex.toString().c_str());
   		consumeUntil(EOF);
   		
   } else {
   	throw ex;
   }
   }
}

void ANTLRParser::fileOptionsSpec() {
   
   MParser_Token_var idTok; MParser_Token_var value;
   
   match(OPTIONS);
   {
   LABEL(_loop18)
   do {
   if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF)) {
   	idTok=id();
   	match(ASSIGN);
   	value=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->setFileOption(idTok, value, getInputState()->filename());
   	}
   	match(SEMI);
   }
   else {
   	BREAK(_loop18);
   }
   
   } while (true);
   }
   match(RCURLY);
}

void ANTLRParser::classDef() {
   
   MParser_Token_var  a = MParser_Token::_nil();
   MParser_Token_var  d = MParser_Token::_nil();
   MLANG_String doc;
   
   try {      // for error handling
   {
   switch ( LA(1)) {
   case ACTION:
   {
   	a = LT(1);
   	match(ACTION);
   	if ( inputState_->guessing()==0 ) {
   		behavior_->refPreambleAction(a);
   	}
   	break;
   }
   case DOC_COMMENT:
   case LITERAL_lexclass:
   case LITERAL_class:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case DOC_COMMENT:
   {
   	d = LT(1);
   	match(DOC_COMMENT);
   	if ( inputState_->guessing()==0 ) {
   		doc=d->getText();
   	}
   	break;
   }
   case LITERAL_lexclass:
   case LITERAL_class:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   bool synPredMatched13 = false;
   if (((LA(1)==LITERAL_lexclass||LA(1)==LITERAL_class) && (LA(2)==TOKEN_REF||LA(2)==RULE_REF))) {
   	int _m13 = mark();
   	synPredMatched13 = true;
   	inputState_->guessing(inputState_->guessing() + 1);
   	try {
   		{
   		switch ( LA(1)) {
   		case LITERAL_lexclass:
   		{
   		match(LITERAL_lexclass);
   		break;
   		}
   		case LITERAL_class:
   		{
   		match(LITERAL_class);
   		id();
   		match(LITERAL_extends);
   		match(LITERAL_Lexer);
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   	}
   	catch (MParser_RecognitionException& pe) {
   		synPredMatched13 = false;
   	}
   	rewind(_m13);
   	inputState_->guessing(inputState_->guessing() - 1);
   }
   if ( synPredMatched13 )
    {
      lexerSpec(doc);
    }
   else
    {
      bool synPredMatched15 = false;
      if (((LA(1)==LITERAL_class) && (LA(2)==TOKEN_REF||LA(2)==RULE_REF)))
       {
   		int _m15 = mark();
   		synPredMatched15 = true;
   		inputState_->guessing(inputState_->guessing() + 1);
   		try {
   		  {
   		     match(LITERAL_class);
   		     id();
   		     match(LITERAL_extends);
   		     match(LITERAL_TreeParser);
   		  }
   		 }
   		catch (MParser_RecognitionException& pe)
                 {
   		   synPredMatched15 = false;
   		 }
   		rewind(_m15);
   		inputState_->guessing(inputState_->guessing() - 1);
   	}
       if (synPredMatched15 )
        {
          treeParserSpec(doc);
   	}
       else if ((LA(1)==LITERAL_class) && (LA(2)==TOKEN_REF||LA(2)==RULE_REF))
        {
          parserSpec(doc);
   	}
       else
        {
          throw MParser_NoViableAltException(LT(1), getFilename());
   	}
       }
   }
   	rules();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->endGrammar();
   	}
    }
   catch (MParser_RecognitionException& ex)
    {
      if (inputState_->guessing()==0)
       {
         if (ACE_dynamic_cast (MParser_NoViableAltException*, &ex))
          {
             MParser_NoViableAltException& e = (MParser_NoViableAltException&)ex;
             // RK: These probably generate inconsequent error messages...
             // have to see how this comes out..
             if ( e.token_->getType()==DOC_COMMENT )
              {
                reportError(ex, "JAVADOC comments may only prefix rules and grammars");
              }
             else
              {
                reportError(ex, MLANG_String("rule classDef trapped:\n")+ex.toString().c_str());
              }
           }
          else
           {
              reportError(ex, MLANG_String("rule classDef trapped:\n")+ex.toString().c_str());
           }
   			behavior_->abortGrammar();
   			bool consuming = true;
   			// consume everything until the next class definition or EOF
   			while (consuming) {
   				consume();
   				switch(LA(1)) {
   				case LITERAL_class:
   				case LITERAL_lexclass:
   				case EOF:
   				consuming = false;
   				break;
   				}
   			}
   		
   	} else {
   		throw ex;
   	}
   }
}
   
MParser_Token_var  ANTLRParser::id()

{
   MParser_Token_var idTok ;
   
   MParser_Token_var  a = MParser_Token::_nil();
   MParser_Token_var  b = MParser_Token::_nil();
   idTok = MParser_Token::_nil();
   
   switch ( LA(1)) {
   case TOKEN_REF:
   {
   a = LT(1);
   match(TOKEN_REF);
   if ( inputState_->guessing()==0 ) {
   	idTok = a;
   }
   break;
   }
   case RULE_REF:
   {
   b = LT(1);
   match(RULE_REF);
   if ( inputState_->guessing()==0 ) {
   	idTok = b;
   }
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   return idTok ;
}

void ANTLRParser::lexerSpec(
   const MLANG_String& doc
) {
   
   MParser_Token_var  lc = MParser_Token::_nil();
   MParser_Token_var  a = MParser_Token::_nil();
   
   MParser_Token_var idTok;
   MLANG_String sup;
   
   {
      switch ( LA(1))
      {
      case LITERAL_lexclass:
           {
             lc = LT(1);
             match(LITERAL_lexclass);
             idTok=id();
             if ( inputState_->guessing()==0 )
              {
   	
                antlrTool_->warning("lexclass' is deprecated; use 'class X extends Lexer'",
   		   getFilename(), lc->getLine(), lc->getColumn());
   	        //			System.out.println("warning: line " + lc.getLine() + ": 'lexclass' is deprecated; use 'class X extends Lexer'");
   			
              }
             break;
           }
      case LITERAL_class:
           {
             match(LITERAL_class);
             idTok=id();
             match(LITERAL_extends);
             match(LITERAL_Lexer);
             {
                switch ( LA(1))
                {
                case LPAREN:
                     {
   	                sup=superClass();
   	                break;
                     }
                case SEMI:
                     {
   	                break;
                     }
                default:
                     {
   	                throw MParser_NoViableAltException(LT(1), getFilename());
                     }
                }
             }
             break;
           }
      default:
           {
             throw MParser_NoViableAltException(LT(1), getFilename());
           }
      }
   }
   if ( inputState_->guessing()==0 )
    {
      behavior_->startLexer(getFilename(), idTok,sup,doc);
    }
   match(SEMI);
   {
      switch ( LA(1))
      {
      case OPTIONS:
           {
              lexerOptionsSpec();
              break;
           }
      case ACTION:
      case DOC_COMMENT:
      case TOKENS:
      case TOKEN_REF:
      case LITERAL_protected:
      case LITERAL_public:
      case LITERAL_private:
      case RULE_REF:
           {
              break;
           }
      default:
           {
              throw MParser_NoViableAltException(LT(1), getFilename());
           }
      }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endOptions();
   }
   {
   switch ( LA(1)) {
   case TOKENS:
   {
   tokensSpec();
   break;
   }
   case ACTION:
   case DOC_COMMENT:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case ACTION:
   {
   a = LT(1);
   match(ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refMemberAction(a);
   }
   break;
   }
   case DOC_COMMENT:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
}

void ANTLRParser::treeParserSpec(
   const MLANG_String& doc
)

{
   
   MParser_Token_var  a = MParser_Token::_nil();
   
   MParser_Token_var idTok;
   MLANG_String sup;
   
   
   match(LITERAL_class);
   idTok=id();
   match(LITERAL_extends);
   match(LITERAL_TreeParser);
   {
   switch ( LA(1)) {
   case LPAREN:
   {
   sup=superClass();
   break;
   }
   case SEMI:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->startTreeWalker(getFilename(), idTok,sup,doc);
   }
   match(SEMI);
   {
   switch ( LA(1)) {
   case OPTIONS:
   {
   treeParserOptionsSpec();
   break;
   }
   case ACTION:
   case DOC_COMMENT:
   case TOKENS:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endOptions();
   }
   {
   switch ( LA(1)) {
   case TOKENS:
   {
   tokensSpec();
   break;
   }
   case ACTION:
   case DOC_COMMENT:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case ACTION:
   {
   a = LT(1);
   match(ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refMemberAction(a);
   }
   break;
   }
   case DOC_COMMENT:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
}

void ANTLRParser::parserSpec(
   const MLANG_String& doc
)

{
   
   MParser_Token_var  a = MParser_Token::_nil();
   
   MParser_Token_var idTok;
   MLANG_String sup;
   
   
   match(LITERAL_class);
   idTok=id();
   {
      switch ( LA(1)) {
      case LITERAL_extends:
           {
              match(LITERAL_extends);
              match(LITERAL_Parser);
              {
                switch ( LA(1)) {
                case LPAREN:
                     {
   	               sup=superClass();
   	               break;
                     }
                case SEMI:
                     {
   	               break;
                     }
                default:
                     {
   	               throw MParser_NoViableAltException(LT(1), getFilename());
                     }
                }
              }
              break;
           }
      case SEMI:
           {
              if ( inputState_->guessing()==0 )
               {
                 antlrTool_->warning("use 'class X extends Parser'",
                       getFilename(), idTok->getLine(), idTok->getColumn());
   	//		System.out.println("warning: line " +
   	//			idTok.getLine() + ": use 'class X extends Parser'");
   			
               }
              break;
           }
      default:
           {
              throw MParser_NoViableAltException(LT(1), getFilename());
           }
      }
   }
   if ( inputState_->guessing()==0 )
    {
      behavior_->startParser(getFilename(), idTok, sup, doc);
    }
   match(SEMI);
   {
   switch ( LA(1)) {
   case OPTIONS:
   {
   parserOptionsSpec();
   break;
   }
   case ACTION:
   case DOC_COMMENT:
   case TOKENS:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endOptions();
   }
   {
   switch ( LA(1)) {
   case TOKENS:
   {
   tokensSpec();
   break;
   }
   case ACTION:
   case DOC_COMMENT:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case ACTION:
   {
   a = LT(1);
   match(ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refMemberAction(a);
   }
   break;
   }
   case DOC_COMMENT:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
}

void ANTLRParser::rules() {
   
{
   int _cnt68=0;
   LABEL(_loop68)
   do {
   if ((_tokenSet_0.member(LA(1))) && (_tokenSet_1.member(LA(2)))) {
   	rule();
   }
   else {
   	if ( _cnt68>=1 ) { BREAK(_loop68); } else {throw MParser_NoViableAltException(LT(1), getFilename());}
   }
   
   _cnt68++;
   } while (true);
   }
}

MParser_Token_var  ANTLRParser::optionValue()

{
   MParser_Token_var retval ;
   
   MParser_Token_var  sl = MParser_Token::_nil();
   MParser_Token_var  cl = MParser_Token::_nil();
   MParser_Token_var  il = MParser_Token::_nil();
   retval = MParser_Token::_nil();
   
   switch ( LA(1)) {
   case TOKEN_REF:
   case RULE_REF:
   {
   retval=qualifiedID();
   break;
   }
   case STRING_LITERAL:
   {
   sl = LT(1);
   match(STRING_LITERAL);
   if ( inputState_->guessing()==0 ) {
   	retval = sl;
   }
   break;
   }
   case CHAR_LITERAL:
   {
   cl = LT(1);
   match(CHAR_LITERAL);
   if ( inputState_->guessing()==0 ) {
   	retval = cl;
   }
   break;
   }
   case INT:
   {
   il = LT(1);
   match(INT);
   if ( inputState_->guessing()==0 ) {
   	retval = il;
   }
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   return retval ;
}

void ANTLRParser::parserOptionsSpec() {
   
   MParser_Token_var idTok; MParser_Token_var value;
   
   match(OPTIONS);
   {
   LABEL(_loop21)
   do {
   if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF)) {
   	idTok=id();
   	match(ASSIGN);
   	value=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->setGrammarOption(idTok, value);
   	}
   	match(SEMI);
   }
   else {
   	BREAK(_loop21);
   }
   
   } while (true);
   }
   match(RCURLY);
}

void ANTLRParser::treeParserOptionsSpec() {
   
   MParser_Token_var idTok; MParser_Token_var value;
   
   match(OPTIONS);
   {
   LABEL(_loop24)
   do {
   if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF)) {
   	idTok=id();
   	match(ASSIGN);
   	value=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->setGrammarOption(idTok, value);
   	}
   	match(SEMI);
   }
   else {
   	BREAK(_loop24);
   }
   
   } while (true);
   }
   match(RCURLY);
}

void ANTLRParser::lexerOptionsSpec()

{
   
   MParser_Token_var idTok; MParser_Token_var value;
   MParser_BitSet b;
   
   match(OPTIONS);
   {
   LABEL(_loop27)
   do {
   switch ( LA(1)) {
   case LITERAL_charVocabulary:
   {
   	match(LITERAL_charVocabulary);
   	match(ASSIGN);
   	b=charSet();
   	match(SEMI);
   	if ( inputState_->guessing()==0 ) {
   		behavior_->setCharVocabulary(b);
   	}
   	break;
   }
   case TOKEN_REF:
   case RULE_REF:
   {
   	idTok=id();
   	match(ASSIGN);
   	value=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->setGrammarOption(idTok, value);
   	}
   	match(SEMI);
   	break;
   }
   default:
   {
   	BREAK(_loop27);
   }
   }
   } while (true);
   }
   match(RCURLY);
}

MParser_BitSet  ANTLRParser::charSet()

{
   MParser_BitSet b ;
   
   
   b = MParser_BitSet();
   MParser_BitSet tmpSet = MParser_BitSet();
   
   
   b=setBlockElement();
   {
   LABEL(_loop34)
   do {
   if ((LA(1)==OR)) {
   	match(OR);
   	tmpSet=setBlockElement();
   	if ( inputState_->guessing()==0 ) {
   		b |= tmpSet;
   	}
   }
   else {
   	BREAK(_loop34);
   }
   
   } while (true);
   }
   return b ;
}

void ANTLRParser::subruleOptionsSpec()

{
   
   MParser_Token_var idTok; MParser_Token_var value;
   
   match(OPTIONS);
   {
   LABEL(_loop30)
   do {
   if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF)) {
   	idTok=id();
   	match(ASSIGN);
   	value=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->setSubruleOption(idTok, value);
   	}
   	match(SEMI);
   }
   else {
   	BREAK(_loop30);
   }
   
   } while (true);
   }
   match(RCURLY);
}

/** Match a.b.c.d qualified ids; WILDCARD here is overloaded as
 *  id separator; that is, I need a reference to the '.' token.
 */
MParser_Token_var  ANTLRParser::qualifiedID()

{
   MParser_Token_var qidTok=MParser_Token::_nil();
   
   
   MLANG_StringBuffer buf;
   MParser_Token_var a;
   
   
   a=id();
   if ( inputState_->guessing()==0 ) {
   buf.append(a->getText());
   }
   {
   LABEL(_loop144)
   do {
   if ((LA(1)==WILDCARD)) {
   	match(WILDCARD);
   	a=id();
   	if ( inputState_->guessing()==0 ) {
   		buf.append('.'); buf.append(a->getText());
   	}
   }
   else {
   	BREAK(_loop144);
   }
   
   } while (true);
   }
   if ( inputState_->guessing()==0 ) {
   
   		 // can use either TOKEN_REF or RULE_REF; should
   		 // really create a QID or something instead.
   		 qidTok = new MParser_CommonToken(TOKEN_REF, buf.toString());
   		 qidTok->setLine(a->getLine());
   		
   }
   return qidTok;
}

MParser_BitSet  ANTLRParser::setBlockElement()

{
   MParser_BitSet b ;
   
   MParser_Token_var  c1 = MParser_Token::_nil();
   MParser_Token_var  c2 = MParser_Token::_nil();
   
   b = MParser_BitSet();
   int rangeMin = 0;
   
   
   c1 = LT(1);
   match(CHAR_LITERAL);
   if ( inputState_->guessing()==0 )
    {
   
      rangeMin = ANTLRLexer::tokenTypeForCharLiteral(c1->getText());
      b = MParser_BitSet::of(rangeMin);
   	
    }
   {
   switch ( LA(1)) {
   case RANGE:
   {
   match(RANGE);
   c2 = LT(1);
   match(CHAR_LITERAL);
   if ( inputState_->guessing()==0 ) {
   	
      int rangeMax = ANTLRLexer::tokenTypeForCharLiteral(c2->getText());
      if (rangeMax < rangeMin)
       {
         antlrTool_->error("Malformed range line ", getFilename(), c1->getLine(), c1->getColumn());
       }
      for (int i = rangeMin+1; i <= rangeMax; i++) {
   				b.add(i);
   			}
   		
   }
   break;
   }
   case SEMI:
   case OR:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   return b ;
}

void ANTLRParser::tokensSpec()

{
   MParser_Token_var  t1 = MParser_Token::_nil();
   MParser_Token_var  s1 = MParser_Token::_nil();
   MParser_Token_var  s3 = MParser_Token::_nil();
   
   match(TOKENS);
   {
   int _cnt43=0;
   LABEL(_loop43)
   do {
   if ((LA(1)==STRING_LITERAL||LA(1)==TOKEN_REF)) {
   	{
   	switch ( LA(1)) {
   	case TOKEN_REF:
   	{
   		if ( inputState_->guessing()==0 ) {
   		s1=MParser_Token::_nil();
   		}
   		t1 = LT(1);
   		match(TOKEN_REF);
   		{
   		switch ( LA(1)) {
   		case ASSIGN:
   		{
   		match(ASSIGN);
   		s1 = LT(1);
   		match(STRING_LITERAL);
   		break;
   		}
   		case SEMI:
   		case OPEN_ELEMENT_OPTION:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		if ( inputState_->guessing()==0 ) {
   		behavior_->defineToken(t1, s1);
   		}
   		{
   		switch ( LA(1)) {
   		case OPEN_ELEMENT_OPTION:
   		{
   		tokensSpecOptions(t1);
   		break;
   		}
   		case SEMI:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		break;
   	}
   	case STRING_LITERAL:
   	{
   		s3 = LT(1);
   		match(STRING_LITERAL);
   		if ( inputState_->guessing()==0 ) {
   		behavior_->defineToken(MParser_Token::_nil(), s3);
   		}
   		{
   		switch ( LA(1)) {
   		case OPEN_ELEMENT_OPTION:
   		{
   		tokensSpecOptions(s3);
   		break;
   		}
   		case SEMI:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		break;
   	}
   	default:
   	{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   	}
   	}
   	}
   	match(SEMI);
   }
   else {
   	if ( _cnt43>=1 ) { BREAK(_loop43); } else {throw MParser_NoViableAltException(LT(1), getFilename());}
   }
   
   _cnt43++;
   } while (true);
   }
   match(RCURLY);
}

void ANTLRParser::tokensSpecOptions(
   const MParser_Token_var& t
)

{
   
   
   MParser_Token_var o, v;
   
   
   match(OPEN_ELEMENT_OPTION);
   o=id();
   match(ASSIGN);
   v=optionValue();
   if ( inputState_->guessing()==0 ) {
   behavior_->refTokensSpecElementOption(t,o,v);
   }
   {
   LABEL(_loop46)
   do {
   if ((LA(1)==SEMI)) {
   	match(SEMI);
   	o=id();
   	match(ASSIGN);
   	v=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->refTokensSpecElementOption(t,o,v);
   	}
   }
   else {
   	BREAK(_loop46);
   }
   
   } while (true);
   }
   match(CLOSE_ELEMENT_OPTION);
}

MLANG_String  ANTLRParser::superClass()

{
   MLANG_String sup;
   
   match(LPAREN);
   if ( inputState_->guessing()==0 ) {
   
   		sup = LT(1)->getText();
   		sup = MParser_StringUtils::stripFrontBack(sup, "\"", "\"");
   		
   }
   {
   match(STRING_LITERAL);
   }
   match(RPAREN);
   return sup;
}

void ANTLRParser::rule()

{
   
   MParser_Token_var  d = MParser_Token::_nil();
   MParser_Token_var  p1 = MParser_Token::_nil();
   MParser_Token_var  p2 = MParser_Token::_nil();
   MParser_Token_var  p3 = MParser_Token::_nil();
   MParser_Token_var  aa = MParser_Token::_nil();
   MParser_Token_var  rt = MParser_Token::_nil();
   MParser_Token_var  a = MParser_Token::_nil();
  
   MLANG_String access="public";
   MParser_Token_var idTok;
   MLANG_String doc;
   bool ruleAutoGen = true;
   blockNesting_ = -1;	// block increments, so -1 to make rule at level 0
   
   
   {
   switch ( LA(1)) {
   case DOC_COMMENT:
   {
   d = LT(1);
   match(DOC_COMMENT);
   if ( inputState_->guessing()==0 ) {
   	doc=d->getText();
   }
   break;
   }
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case LITERAL_protected:
   {
   p1 = LT(1);
   match(LITERAL_protected);
   if ( inputState_->guessing()==0 ) {
   	access=p1->getText();
   }
   break;
   }
   case LITERAL_public:
   {
   p2 = LT(1);
   match(LITERAL_public);
   if ( inputState_->guessing()==0 ) {
   	access=p2->getText();
   }
   break;
   }
   case LITERAL_private:
   {
   p3 = LT(1);
   match(LITERAL_private);
   if ( inputState_->guessing()==0 ) {
   	access=p3->getText();
   }
   break;
   }
   case TOKEN_REF:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   idTok=id();
   {
   switch ( LA(1)) {
   case BANG:
   {
   match(BANG);
   if ( inputState_->guessing()==0 ) {
   	ruleAutoGen = false;
   }
   break;
   }
   case ACTION:
   case OPTIONS:
   case ARG_ACTION:
   case LITERAL_returns:
   case COLON:
   case LITERAL_throws:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   
   		behavior_->defineRuleName(idTok, access, ruleAutoGen, doc);
   	
   }
   {
   switch ( LA(1)) {
   case ARG_ACTION:
   {
   aa = LT(1);
   match(ARG_ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refArgAction(aa);
   }
   break;
   }
   case ACTION:
   case OPTIONS:
   case LITERAL_returns:
   case COLON:
   case LITERAL_throws:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case LITERAL_returns:
   {
   match(LITERAL_returns);
   rt = LT(1);
   match(ARG_ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refReturnAction(rt);
   }
   break;
   }
   case ACTION:
   case OPTIONS:
   case COLON:
   case LITERAL_throws:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case LITERAL_throws:
   {
   throwsSpec();
   break;
   }
   case ACTION:
   case OPTIONS:
   case COLON:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case OPTIONS:
   {
   ruleOptionsSpec();
   break;
   }
   case ACTION:
   case COLON:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case ACTION:
   {
   a = LT(1);
   match(ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refInitAction(a);
   }
   break;
   }
   case COLON:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   match(COLON);
   block();
   match(SEMI);
   {
   switch ( LA(1)) {
   case LITERAL_exception:
   {
   exceptionGroup();
   break;
   }
   case EOF:
   case ACTION:
   case DOC_COMMENT:
   case LITERAL_lexclass:
   case LITERAL_class:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endRule(idTok->getText());
   }
}

void ANTLRParser::throwsSpec()

{
   MLANG_String t;
   MParser_Token_var a,b;
   
   
   match(LITERAL_throws);
   a=id();
   if ( inputState_->guessing()==0 ) {
   t=a->getText();
   }
   {
   LABEL(_loop84)
   do {
   if ((LA(1)==COMMA)) {
   	match(COMMA);
   	b=id();
   	if ( inputState_->guessing()==0 ) {
   		t+=","+b->getText();
   	}
   }
   else {
   	BREAK(_loop84);
   }
   
   } while (true);
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->setUserExceptions(t);	
   }
}

void ANTLRParser::ruleOptionsSpec() {
   
   MParser_Token_var idTok; MParser_Token_var value;
   
   match(OPTIONS);
   {
   LABEL(_loop81)
   do {
   if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF)) {
   	idTok=id();
   	match(ASSIGN);
   	value=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->setRuleOption(idTok, value);
   	}
   	match(SEMI);
   }
   else {
   	BREAK(_loop81);
   }
   
   } while (true);
   }
   match(RCURLY);
}

void ANTLRParser::block()

{
   
   
   if ( inputState_->guessing()==0 ) {
   blockNesting_++;
   }
   alternative();
   {
   LABEL(_loop87)
   do {
   if ((LA(1)==OR)) {
   	match(OR);
   	alternative();
   }
   else {
   	BREAK(_loop87);
   }
   
   } while (true);
   }
   if ( inputState_->guessing()==0 ) {
   blockNesting_--;
   }
}

void ANTLRParser::exceptionGroup() {
   
   
   if ( inputState_->guessing()==0 ) {
   behavior_->beginExceptionGroup();
   }
   {
   int _cnt95=0;
   LABEL(_loop95)
   do {
   if ((LA(1)==LITERAL_exception)) {
   	exceptionSpec();
   }
   else {
   	if ( _cnt95>=1 ) { BREAK(_loop95); } else {throw MParser_NoViableAltException(LT(1), getFilename());}
   }
   
   _cnt95++;
   } while (true);
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endExceptionGroup();
   }
}

void ANTLRParser::alternative()

{
   
   bool altAutoGen = true;
   
   {
   switch ( LA(1)) {
   case BANG:
   {
   match(BANG);
   if ( inputState_->guessing()==0 ) {
   	altAutoGen=false;
   }
   break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case LPAREN:
   case RPAREN:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->beginAlt(altAutoGen);
   }
   {
   LABEL(_loop91)
   do {
   if ((_tokenSet_2.member(LA(1)))) {
   	element();
   }
   else {
   	BREAK(_loop91);
   }
   
   } while (true);
   }
   {
   switch ( LA(1)) {
   case LITERAL_exception:
   {
   exceptionSpecNoLabel();
   break;
   }
   case SEMI:
   case OR:
   case RPAREN:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endAlt();
   }
}

void ANTLRParser::element()

{
   
   
   elementNoOptionSpec();
   {
   switch ( LA(1)) {
   case OPEN_ELEMENT_OPTION:
   {
   elementOptionSpec();
   break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case LPAREN:
   case RPAREN:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
}

void ANTLRParser::exceptionSpecNoLabel()

{
   
   
   match(LITERAL_exception);
   if ( inputState_->guessing()==0 ) {
   behavior_->beginExceptionSpec(MParser_Token::_nil());
   }
   {
   LABEL(_loop102)
   do {
   if ((LA(1)==LITERAL_catch)) {
   	exceptionHandler();
   }
   else {
   	BREAK(_loop102);
   }
   
   } while (true);
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endExceptionSpec();
   }
}

void ANTLRParser::exceptionSpec() {
   
   MParser_Token_var  aa = MParser_Token::_nil();
   MParser_Token_var labelAction = MParser_Token::_nil();
   
   match(LITERAL_exception);
   {
   switch ( LA(1)) {
   case ARG_ACTION:
   {
   aa = LT(1);
   match(ARG_ACTION);
   if ( inputState_->guessing()==0 ) {
   	labelAction = aa;
   }
   break;
   }
   case EOF:
   case ACTION:
   case DOC_COMMENT:
   case LITERAL_lexclass:
   case LITERAL_class:
   case TOKEN_REF:
   case LITERAL_protected:
   case LITERAL_public:
   case LITERAL_private:
   case LITERAL_exception:
   case LITERAL_catch:
   case RULE_REF:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->beginExceptionSpec(labelAction);
   }
   {
   LABEL(_loop99)
   do {
   if ((LA(1)==LITERAL_catch)) {
   	exceptionHandler();
   }
   else {
   	BREAK(_loop99);
   }
   
   } while (true);
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endExceptionSpec();
   }
}

void ANTLRParser::exceptionHandler()

{
   MParser_Token_var  a1 = MParser_Token::_nil();
   MParser_Token_var  a2 = MParser_Token::_nil();
   MParser_Token_var exType; MParser_Token_var exName;
   
   match(LITERAL_catch);
   a1 = LT(1);
   match(ARG_ACTION);
   a2 = LT(1);
   match(ACTION);
   if ( inputState_->guessing()==0 ) {
   behavior_->refExceptionHandler(a1, a2);
   }
}

void ANTLRParser::elementNoOptionSpec()

{
   MParser_Token_var  rr = MParser_Token::_nil();
   MParser_Token_var  aa = MParser_Token::_nil();
   MParser_Token_var  tr = MParser_Token::_nil();
   MParser_Token_var  aa2 = MParser_Token::_nil();
   MParser_Token_var  r2 = MParser_Token::_nil();
   MParser_Token_var  aa3 = MParser_Token::_nil();
   MParser_Token_var  a = MParser_Token::_nil();
   MParser_Token_var  p = MParser_Token::_nil();
   
   MParser_Token_var label = MParser_Token::_nil();
   MParser_Token_var assignId = MParser_Token::_nil();
   MParser_Token_var args = MParser_Token::_nil();
   int autoGen = MGrammar_GrammarElement::AUTO_GEN_NONE;
   
   
   switch ( LA(1)) {
   case ACTION:
   {
   a = LT(1);
   match(ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refAction(a);
   }
   break;
   }
   case SEMPRED:
   {
   p = LT(1);
   match(SEMPRED);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refSemPred(p);
   }
   break;
   }
   case TREE_BEGIN:
   {
   tree();
   break;
   }
   default:
   if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF) && (LA(2)==ASSIGN)) {
   	assignId=id();
   	match(ASSIGN);
   	{
   	if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF) && (LA(2)==COLON)) {
   		label=id();
   		match(COLON);
   		if ( inputState_->guessing()==0 ) {
   		checkForMissingEndRule(label);
   		}
   	}
   	else if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF) && (_tokenSet_3.member(LA(2)))) {
   	}
   	else {
   		throw MParser_NoViableAltException(LT(1), getFilename());
   	}
   	
   	}
   	{
   	switch ( LA(1)) {
   	case RULE_REF:
   	{
   		rr = LT(1);
   		match(RULE_REF);
   		{
   		switch ( LA(1)) {
   		case ARG_ACTION:
   		{
   		aa = LT(1);
   		match(ARG_ACTION);
   		if ( inputState_->guessing()==0 ) {
   			args=aa;
   		}
   		break;
   		}
   		case STRING_LITERAL:
   		case ACTION:
   		case SEMI:
   		case CHAR_LITERAL:
   		case OR:
   		case TOKEN_REF:
   		case OPEN_ELEMENT_OPTION:
   		case LPAREN:
   		case RPAREN:
   		case BANG:
   		case LITERAL_exception:
   		case RULE_REF:
   		case NOT_OP:
   		case SEMPRED:
   		case TREE_BEGIN:
   		case WILDCARD:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		{
   		switch ( LA(1)) {
   		case BANG:
   		{
   		match(BANG);
   		if ( inputState_->guessing()==0 ) {
   			autoGen = MGrammar_GrammarElement::AUTO_GEN_BANG;
   		}
   		break;
   		}
   		case STRING_LITERAL:
   		case ACTION:
   		case SEMI:
   		case CHAR_LITERAL:
   		case OR:
   		case TOKEN_REF:
   		case OPEN_ELEMENT_OPTION:
   		case LPAREN:
   		case RPAREN:
   		case LITERAL_exception:
   		case RULE_REF:
   		case NOT_OP:
   		case SEMPRED:
   		case TREE_BEGIN:
   		case WILDCARD:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		if ( inputState_->guessing()==0 ) {
   		behavior_->refRule(assignId, rr, label, args, autoGen);
   		}
   		break;
   	}
   	case TOKEN_REF:
   	{
   		tr = LT(1);
   		match(TOKEN_REF);
   		{
   		switch ( LA(1)) {
   		case ARG_ACTION:
   		{
   		aa2 = LT(1);
   		match(ARG_ACTION);
   		if ( inputState_->guessing()==0 ) {
   			args=aa2;
   		}
   		break;
   		}
   		case STRING_LITERAL:
   		case ACTION:
   		case SEMI:
   		case CHAR_LITERAL:
   		case OR:
   		case TOKEN_REF:
   		case OPEN_ELEMENT_OPTION:
   		case LPAREN:
   		case RPAREN:
   		case LITERAL_exception:
   		case RULE_REF:
   		case NOT_OP:
   		case SEMPRED:
   		case TREE_BEGIN:
   		case WILDCARD:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		if ( inputState_->guessing()==0 ) {
   		behavior_->refToken(assignId, tr, label, args, false, autoGen, lastInRule());
   		}
   		break;
   	}
   	default:
   	{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   	}
   	}
   	}
   }
   else if ((_tokenSet_4.member(LA(1))) && (_tokenSet_5.member(LA(2)))) {
   	{
   	if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF) && (LA(2)==COLON)) {
   		label=id();
   		match(COLON);
   		if ( inputState_->guessing()==0 ) {
   		checkForMissingEndRule(label);
   		}
   	}
   	else if ((_tokenSet_4.member(LA(1))) && (_tokenSet_6.member(LA(2)))) {
   	}
   	else {
   		throw MParser_NoViableAltException(LT(1), getFilename());
   	}
   	
   	}
   	{
   	switch ( LA(1)) {
   	case RULE_REF:
   	{
   		r2 = LT(1);
   		match(RULE_REF);
   		{
   		switch ( LA(1)) {
   		case ARG_ACTION:
   		{
   		aa3 = LT(1);
   		match(ARG_ACTION);
   		if ( inputState_->guessing()==0 ) {
   			args=aa3;
   		}
   		break;
   		}
   		case STRING_LITERAL:
   		case ACTION:
   		case SEMI:
   		case CHAR_LITERAL:
   		case OR:
   		case TOKEN_REF:
   		case OPEN_ELEMENT_OPTION:
   		case LPAREN:
   		case RPAREN:
   		case BANG:
   		case LITERAL_exception:
   		case RULE_REF:
   		case NOT_OP:
   		case SEMPRED:
   		case TREE_BEGIN:
   		case WILDCARD:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		{
   		switch ( LA(1)) {
   		case BANG:
   		{
   		match(BANG);
   		if ( inputState_->guessing()==0 ) {
   			autoGen = MGrammar_GrammarElement::AUTO_GEN_BANG;
   		}
   		break;
   		}
   		case STRING_LITERAL:
   		case ACTION:
   		case SEMI:
   		case CHAR_LITERAL:
   		case OR:
   		case TOKEN_REF:
   		case OPEN_ELEMENT_OPTION:
   		case LPAREN:
   		case RPAREN:
   		case LITERAL_exception:
   		case RULE_REF:
   		case NOT_OP:
   		case SEMPRED:
   		case TREE_BEGIN:
   		case WILDCARD:
   		{
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		if ( inputState_->guessing()==0 ) {
   		behavior_->refRule(assignId, r2, label, args, autoGen);
   		}
   		break;
   	}
   	case NOT_OP:
   	{
   		match(NOT_OP);
   		{
   		switch ( LA(1)) {
   		case CHAR_LITERAL:
   		case TOKEN_REF:
   		{
   		notTerminal(label);
   		break;
   		}
   		case LPAREN:
   		{
   		ebnf(label,true);
   		break;
   		}
   		default:
   		{
   		throw MParser_NoViableAltException(LT(1), getFilename());
   		}
   		}
   		}
   		break;
   	}
   	case LPAREN:
   	{
   		ebnf(label,false);
   		break;
   	}
   	default:
   		if ((LA(1)==STRING_LITERAL||LA(1)==CHAR_LITERAL||LA(1)==TOKEN_REF) && (LA(2)==RANGE)) {
   		range(label);
   		}
   		else if ((_tokenSet_7.member(LA(1))) && (_tokenSet_8.member(LA(2)))) {
   		terminal(label);
   		}
   	else {
   		throw MParser_NoViableAltException(LT(1), getFilename());
   	}
   	}
   	}
   }
   else {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
}

void ANTLRParser::elementOptionSpec() {
   
   
   MParser_Token_var o, v;
   
   
   match(OPEN_ELEMENT_OPTION);
   o=id();
   match(ASSIGN);
   v=optionValue();
   if ( inputState_->guessing()==0 ) {
   behavior_->refElementOption(o,v);
   }
   {
   LABEL(_loop108)
   do {
   if ((LA(1)==SEMI)) {
   	match(SEMI);
   	o=id();
   	match(ASSIGN);
   	v=optionValue();
   	if ( inputState_->guessing()==0 ) {
   		behavior_->refElementOption(o,v);
   	}
   }
   else {
   	BREAK(_loop108);
   }
   
   } while (true);
   }
   match(CLOSE_ELEMENT_OPTION);
}

void ANTLRParser::range(
    const MParser_Token_var& label 
) {
   
   MParser_Token_var  crLeft = MParser_Token::_nil();
   MParser_Token_var  crRight = MParser_Token::_nil();
   MParser_Token_var  t = MParser_Token::_nil();
   MParser_Token_var  u = MParser_Token::_nil();
   MParser_Token_var  v = MParser_Token::_nil();
   MParser_Token_var  w = MParser_Token::_nil();
   
   MParser_Token_var trLeft;
   MParser_Token_var trRight;
   int autoGen=MGrammar_GrammarElement::AUTO_GEN_NONE;
   
   
   switch ( LA(1)) {
   case CHAR_LITERAL:
   {
   crLeft = LT(1);
   match(CHAR_LITERAL);
   match(RANGE);
   crRight = LT(1);
   match(CHAR_LITERAL);
   {
   switch ( LA(1)) {
   case BANG:
   {
   	match(BANG);
   	if ( inputState_->guessing()==0 ) {
   		autoGen = MGrammar_GrammarElement::AUTO_GEN_BANG;
   	}
   	break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   	behavior_->refCharRange(crLeft, crRight, label, autoGen, lastInRule());
   }
   break;
   }
   case STRING_LITERAL:
   case TOKEN_REF:
   {
   {
   switch ( LA(1)) {
   case TOKEN_REF:
   {
   	t = LT(1);
   	match(TOKEN_REF);
   	if ( inputState_->guessing()==0 ) {
   		trLeft=t;
   	}
   	break;
   }
   case STRING_LITERAL:
   {
   	u = LT(1);
   	match(STRING_LITERAL);
   	if ( inputState_->guessing()==0 ) {
   		trLeft=u;
   	}
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   match(RANGE);
   {
   switch ( LA(1)) {
   case TOKEN_REF:
   {
   	v = LT(1);
   	match(TOKEN_REF);
   	if ( inputState_->guessing()==0 ) {
   		trRight=v;
   	}
   	break;
   }
   case STRING_LITERAL:
   {
   	w = LT(1);
   	match(STRING_LITERAL);
   	if ( inputState_->guessing()==0 ) {
   		trRight=w;
   	}
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   autoGen=ast_type_spec();
   if ( inputState_->guessing()==0 ) {
   	behavior_->refTokenRange(trLeft, trRight, label, autoGen, lastInRule());
   }
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
}

void ANTLRParser::terminal(
    const MParser_Token_var& label 
) {
   
   MParser_Token_var  cl = MParser_Token::_nil();
   MParser_Token_var  tr = MParser_Token::_nil();
   MParser_Token_var  aa = MParser_Token::_nil();
   MParser_Token_var  sl = MParser_Token::_nil();
   MParser_Token_var  wi = MParser_Token::_nil();
   
   int autoGen=MGrammar_GrammarElement::AUTO_GEN_NONE;
   MParser_Token_var args;
   
   
   switch ( LA(1)) {
   case CHAR_LITERAL:
   {
   cl = LT(1);
   match(CHAR_LITERAL);
   {
   switch ( LA(1)) {
   case BANG:
   {
   	match(BANG);
   	if ( inputState_->guessing()==0 ) {
   		autoGen = MGrammar_GrammarElement::AUTO_GEN_BANG;
   	}
   	break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   	behavior_->refCharLiteral(cl, label, false, autoGen, lastInRule());
   }
   break;
   }
   case TOKEN_REF:
   {
   tr = LT(1);
   match(TOKEN_REF);
   autoGen=ast_type_spec();
   {
   switch ( LA(1)) {
   case ARG_ACTION:
   {
   	aa = LT(1);
   	match(ARG_ACTION);
   	if ( inputState_->guessing()==0 ) {
   		args=aa;
   	}
   	break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   	behavior_->refToken(MParser_Token::_nil(), tr, label, args, false, autoGen, lastInRule());
   }
   break;
   }
   case STRING_LITERAL:
   {
   sl = LT(1);
   match(STRING_LITERAL);
   autoGen=ast_type_spec();
   if ( inputState_->guessing()==0 ) {
   	behavior_->refStringLiteral(sl, label, autoGen, lastInRule());
   }
   break;
   }
   case WILDCARD:
   {
   wi = LT(1);
   match(WILDCARD);
   autoGen=ast_type_spec();
   if ( inputState_->guessing()==0 ) {
   	behavior_->refWildcard(wi, label, autoGen);
   }
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
}

void ANTLRParser::notTerminal(
    const MParser_Token_var& label 
)

{
   MParser_Token_var  cl = MParser_Token::_nil();
   MParser_Token_var  tr = MParser_Token::_nil();
   int autoGen=MGrammar_GrammarElement::AUTO_GEN_NONE;
   
   switch ( LA(1)) {
   case CHAR_LITERAL:
   {
   cl = LT(1);
   match(CHAR_LITERAL);
   {
   switch ( LA(1)) {
   case BANG:
   {
   	match(BANG);
   	if ( inputState_->guessing()==0 ) {
   		autoGen = MGrammar_GrammarElement::AUTO_GEN_BANG;
   	}
   	break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   	behavior_->refCharLiteral(cl, label, true, autoGen, lastInRule());
   }
   break;
   }
   case TOKEN_REF:
   {
   tr = LT(1);
   match(TOKEN_REF);
   autoGen=ast_type_spec();
   if ( inputState_->guessing()==0 ) {
   	behavior_->refToken(MParser_Token::_nil(), tr, label, MParser_Token::_nil(), true, autoGen, lastInRule());
   }
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
}

void ANTLRParser::ebnf(
    const MParser_Token_var& label,
    bool no 
)

{
   
   MParser_Token_var  lp = MParser_Token::_nil();
   MParser_Token_var  aa = MParser_Token::_nil();
   MParser_Token_var  ab = MParser_Token::_nil();
   
   lp = LT(1);
   match(LPAREN);
   if ( inputState_->guessing()==0 ) {
   behavior_->beginSubRule(label, lp, no);
   }
   {
   if ((LA(1)==OPTIONS)) {
   subruleOptionsSpec();
   {
   switch ( LA(1)) {
   case ACTION:
   {
   	aa = LT(1);
   	match(ACTION);
   	if ( inputState_->guessing()==0 ) {
   		behavior_->refInitAction(aa);
   	}
   	break;
   }
   case COLON:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   match(COLON);
   }
   else if ((LA(1)==ACTION) && (LA(2)==COLON)) {
   ab = LT(1);
   match(ACTION);
   if ( inputState_->guessing()==0 ) {
   	behavior_->refInitAction(ab);
   }
   match(COLON);
   }
   else if ((_tokenSet_9.member(LA(1))) && (_tokenSet_10.member(LA(2)))) {
   }
   else {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   
   }
   block();
   match(RPAREN);
   {
   switch ( LA(1)) {
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case BANG:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case QUESTION:
   case STAR:
   case PLUS:
   case WILDCARD:
   {
   {
   switch ( LA(1)) {
   case QUESTION:
   {
   	match(QUESTION);
   	if ( inputState_->guessing()==0 ) {
   		behavior_->optionalSubRule();
   	}
   	break;
   }
   case STAR:
   {
   	match(STAR);
   	if ( inputState_->guessing()==0 ) {
   		behavior_->zeroOrMoreSubRule();
   	}
   	break;
   }
   case PLUS:
   {
   	match(PLUS);
   	if ( inputState_->guessing()==0 ) {
   		behavior_->oneOrMoreSubRule();
   	}
   	break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case BANG:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   {
   switch ( LA(1)) {
   case BANG:
   {
   	match(BANG);
   	if ( inputState_->guessing()==0 ) {
   		behavior_->noASTSubRule();
   	}
   	break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   	break;
   }
   default:
   {
   	throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   break;
   }
   case IMPLIES:
   {
   match(IMPLIES);
   if ( inputState_->guessing()==0 ) {
   	behavior_->synPred();
   }
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endSubRule();
   }
}

void ANTLRParser::tree() {
   
   MParser_Token_var  lp = MParser_Token::_nil();
   
   lp = LT(1);
   match(TREE_BEGIN);
   if ( inputState_->guessing()==0 ) {
   behavior_->beginTree(lp);
   }
   rootNode();
   if ( inputState_->guessing()==0 ) {
   behavior_->beginChildList();
   }
   {
   int _cnt122=0;
   LABEL(_loop122)
   do {
   if ((_tokenSet_2.member(LA(1)))) {
   	element();
   }
   else {
   	if ( _cnt122>=1 ) { BREAK(_loop122); } else {throw MParser_NoViableAltException(LT(1), getFilename());}
   }
   
   _cnt122++;
   } while (true);
   }
   if ( inputState_->guessing()==0 ) {
   behavior_->endChildList();
   }
   match(RPAREN);
   if ( inputState_->guessing()==0 ) {
   behavior_->endTree();
   }
}

void ANTLRParser::rootNode()

{
   
   MParser_Token_var label = MParser_Token::_nil();
   
   {
   if ((LA(1)==TOKEN_REF||LA(1)==RULE_REF) && (LA(2)==COLON)) {
   label=id();
   match(COLON);
   if ( inputState_->guessing()==0 ) {
   	checkForMissingEndRule(label);
   }
   }
   else if ((_tokenSet_7.member(LA(1))) && (_tokenSet_11.member(LA(2)))) {
   }
   else {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   
   }
   terminal(label);
}

int  ANTLRParser::ast_type_spec() {
    int autoGen ;
   
   autoGen = MGrammar_GrammarElement::AUTO_GEN_NONE;
   
   {
   switch ( LA(1)) {
   case CARET:
   {
   match(CARET);
   if ( inputState_->guessing()==0 ) {
   	autoGen = MGrammar_GrammarElement::AUTO_GEN_CARET;
   }
   break;
   }
   case BANG:
   {
   match(BANG);
   if ( inputState_->guessing()==0 ) {
   	autoGen = MGrammar_GrammarElement::AUTO_GEN_BANG;
   }
   break;
   }
   case STRING_LITERAL:
   case ACTION:
   case SEMI:
   case CHAR_LITERAL:
   case OR:
   case TOKEN_REF:
   case OPEN_ELEMENT_OPTION:
   case LPAREN:
   case RPAREN:
   case ARG_ACTION:
   case LITERAL_exception:
   case RULE_REF:
   case NOT_OP:
   case SEMPRED:
   case TREE_BEGIN:
   case WILDCARD:
   {
   break;
   }
   default:
   {
   throw MParser_NoViableAltException(LT(1), getFilename());
   }
   }
   }
   return autoGen ;
}


static longArray mk_tokenSet_0() {
   ACE_UINT64 data[] = { 2206556225792L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_0(mk_tokenSet_0());
static longArray mk_tokenSet_1() {
   ACE_UINT64 data[] = { 2472844214400L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_1(mk_tokenSet_1());
static longArray mk_tokenSet_2() {
   ACE_UINT64 data[] = { 1158885407195328L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_2(mk_tokenSet_2());
static longArray mk_tokenSet_3() {
   ACE_UINT64 data[] = { 1159461236965568L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_3(mk_tokenSet_3());
static longArray mk_tokenSet_4() {
   ACE_UINT64 data[] = { 1132497128128576L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_4(mk_tokenSet_4());
static longArray mk_tokenSet_5() {
   ACE_UINT64 data[] = { 1722479914074304L, 0L};
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_5(mk_tokenSet_5());
static longArray mk_tokenSet_6() {
   ACE_UINT64 data[] = { 1722411194597568L, 0L};
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_6(mk_tokenSet_6());
static longArray mk_tokenSet_7() {
   ACE_UINT64 data[] = { 1125899924144192L, 0L};
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_7(mk_tokenSet_7());
static longArray mk_tokenSet_8() {
   ACE_UINT64 data[] = { 1722411190386880L, 0L};
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_8(mk_tokenSet_8());
static longArray mk_tokenSet_9() {
   ACE_UINT64 data[] = { 1159444023476416L, 0L};
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_9(mk_tokenSet_9());
static longArray mk_tokenSet_10() {
   ACE_UINT64 data[] = { 2251345007067328L, 0L};
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_10(mk_tokenSet_10());
static longArray mk_tokenSet_11() {
   static ACE_UINT64 data[] = { 1721861130420416L, 0L};
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRParser::_tokenSet_11(mk_tokenSet_11());

ANTLRParser::~ANTLRParser ()

{
}
