#ifndef __MGRAMMAR_ANTLRPARSER_H__
#define __MGRAMMAR_ANTLRPARSER_H__
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MParser/RecognitionException.h"
#include "MParser/TokenBuffer.h"
#include "MParser/TokenStream.h"
#include "MGrammar/GrammarParseBehavior.h"
#include "MParser/LLkParser.h"
#include "MGrammar/ANTLRTokenTypes.h"

class ANTLRParser;
typedef class ANTLRParser *ANTLRParser_ptr;

class ANTLRParser : public MParser_LLkParser,
                    public ANTLRTokenTypes
{
public:
   typedef MParser_LLkParser super;
protected:
   static bool DEBUG_PARSER;
   MGrammar_GrammarParseBehavior_var behavior_;
   MGrammar_Tool_ptr antlrTool_;
   int blockNesting_;
public:

   ANTLRParser (
      const MParser_TokenBuffer_var& tokenBuf,
      const MGrammar_GrammarParseBehavior_var& behavior,
      MGrammar_Tool_ptr tool
    );

   virtual ~ANTLRParser ();

public:
   static bool is_nil (ANTLRParser_ptr parser)
   { return parser == 0; }

   static ANTLRParser_ptr _nil()
   { return 0; }

   static ANTLRParser_ptr _duplicate(ANTLRParser_ptr parser)
   { return ::MLANG_acquire(parser); }

public:

    void reportError(const MLANG_String& s);

    void reportError(MParser_RecognitionException& e);

    void reportError(MParser_RecognitionException& e, const MLANG_String& s);

    void reportWarning(const MLANG_String& s);

private:

    bool lastInRule();

    void checkForMissingEndRule(const MParser_Token_var& label);

public: 

   ANTLRParser (const MParser_TokenBuffer_var& tokenBuf, int k = 2);

public:

   ANTLRParser(const MParser_TokenStream_var& lexer, int k = 2);

   ANTLRParser(const MParser_ParserSharedInputState_var& state);

public:

   void grammar();

   void fileOptionsSpec();

   void classDef();

   MParser_Token_var  id();

   void lexerSpec (
      const MLANG_String& doc
    );

   void treeParserSpec (
      const MLANG_String& doc
    );

   void parserSpec (
     const MLANG_String& doc
    );

   void rules();

   MParser_Token_var  optionValue();

   void parserOptionsSpec();

   void treeParserOptionsSpec();

   void lexerOptionsSpec();

   MParser_BitSet  charSet();

   void subruleOptionsSpec();

   /** Match a.b.c.d qualified ids; WILDCARD here is overloaded as
    *  id separator; that is, I need a reference to the '.' token.
    */
   MParser_Token_var  qualifiedID();

   MParser_BitSet  setBlockElement();

   void tokensSpec();

   void tokensSpecOptions(
      const MParser_Token_var& t
     );

   MLANG_String  superClass();

   void rule();

   void throwsSpec();

   void ruleOptionsSpec();

   void block();

   void exceptionGroup();

   void alternative();

   void element();

   void exceptionSpecNoLabel();

   void exceptionSpec();

   void exceptionHandler();

   void elementNoOptionSpec();

   void elementOptionSpec();

   void range (
      const MParser_Token_var& label 
    ); 

   void terminal (
      const MParser_Token_var& label 
     );

    void notTerminal (
      const MParser_Token_var& label 
     );

    void ebnf (
      const MParser_Token_var& label,
      bool no
     );

    void tree();

    void rootNode();

    int  ast_type_spec();
protected:
    static const char * _tokenNames[];
public:
    static MParser_BitSet _tokenSet_0;
    static MParser_BitSet _tokenSet_1;
    static MParser_BitSet _tokenSet_2;
    static MParser_BitSet _tokenSet_3;
    static MParser_BitSet _tokenSet_4;
    static MParser_BitSet _tokenSet_5;
    static MParser_BitSet _tokenSet_6;
    static MParser_BitSet _tokenSet_7;
    static MParser_BitSet _tokenSet_8;
    static MParser_BitSet _tokenSet_9;
    static MParser_BitSet _tokenSet_10;
    static MParser_BitSet _tokenSet_11;
};

typedef MLANG_SmartPtr<ANTLRParser> ANTLRParser_var;

#endif /* __MGRAMMAR_ANTLRPARSER_H__ */
