#include "MParser/CharStreamException.h"
#include "MParser/CharStreamIOException.h"
#include "MParser/NoViableAltForCharException.h"
#include "MParser/TokenStreamIOException.h"
#include "MParser/TokenStreamRecognitionException.h"
#include "MParser/StringUtils.h"
#include "MParser/ByteBuffer.h"
#include "MParser/CharBuffer.h"
#include "MGrammar/ANTLRTokdefLexer.h"


ANTLRTokdefLexer::ANTLRTokdefLexer(const MIO_InputStream_var& in)
  : super(new MParser_ByteBuffer(in))

{
   caseSensitiveLiterals_ = true;
   setCaseSensitive(true);
   literals_.clear();
}

ANTLRTokdefLexer::ANTLRTokdefLexer(const MIO_Reader_var& in)
  : super(new MParser_CharBuffer(in))

{
   caseSensitiveLiterals_ = true;
   setCaseSensitive(true);
   literals_.clear();
}

ANTLRTokdefLexer::ANTLRTokdefLexer(const MParser_InputBuffer_var& ib)
  : super(new MParser_LexerSharedInputState(ib))

{
   caseSensitiveLiterals_ = true;
   setCaseSensitive(true);
   literals_.clear();
}

ANTLRTokdefLexer::ANTLRTokdefLexer(const MParser_LexerSharedInputState_var& state)
  : super(state)

{
   caseSensitiveLiterals_ = true;
   setCaseSensitive(true);
   literals_.clear();
}

MParser_Token_var ANTLRTokdefLexer::nextToken()

{
   MParser_Token_var theRetToken;
tryAgain:
   for (;;) {
   MParser_Token_var _token;
   int _ttype = MParser_Token::INVALID_TYPE;
   resetText();
   try {   // for char stream error handling
      try {   // for lexical error handling
      	switch ( LA(1)) {
      	case '\t':  case '\n':  case '\r':  case ' ':
      	{
         mWS(true);
         theRetToken=returnToken_;
         break;
      }
      	case '(':
      	{
         mLPAREN(true);
         theRetToken=returnToken_;
         break;
      }
      	case ')':
      	{
         mRPAREN(true);
         theRetToken=returnToken_;
         break;
      }
      	case '=':
      	{
         mASSIGN(true);
         theRetToken=returnToken_;
         break;
      }
      	case '"':
      	{
         mSTRING(true);
         theRetToken=returnToken_;
         break;
      }
      	case 'A':  case 'B':  case 'C':  case 'D':
      	case 'E':  case 'F':  case 'G':  case 'H':
      	case 'I':  case 'J':  case 'K':  case 'L':
      	case 'M':  case 'N':  case 'O':  case 'P':
      	case 'Q':  case 'R':  case 'S':  case 'T':
      	case 'U':  case 'V':  case 'W':  case 'X':
      	case 'Y':  case 'Z':  case 'a':  case 'b':
      	case 'c':  case 'd':  case 'e':  case 'f':
      	case 'g':  case 'h':  case 'i':  case 'j':
      	case 'k':  case 'l':  case 'm':  case 'n':
      	case 'o':  case 'p':  case 'q':  case 'r':
      	case 's':  case 't':  case 'u':  case 'v':
      	case 'w':  case 'x':  case 'y':  case 'z':
      	{
         mID(true);
         theRetToken=returnToken_;
         break;
      }
      	case '0':  case '1':  case '2':  case '3':
      	case '4':  case '5':  case '6':  case '7':
      	case '8':  case '9':
      	{
         mINT(true);
         theRetToken=returnToken_;
         break;
      }
      	default:
         if ((LA(1)=='/') && (LA(2)=='/')) {
            mSL_COMMENT(true);
            theRetToken=returnToken_;
      	}
         else if ((LA(1)=='/') && (LA(2)=='*')) {
            mML_COMMENT(true);
            theRetToken=returnToken_;
      	}
      	else {
         if (LA(1)==EOF_CHAR) {uponEOF(); returnToken_ = makeToken(MParser_Token::EOF_TYPE);}
      	else {throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
      }
      }
      	if ( MParser_Token::is_nil(returnToken_) ) goto tryAgain; // found SKIP token
      	_ttype = returnToken_->getType();
      	returnToken_->setType(_ttype);
      	return returnToken_;
   }
      catch (MParser_RecognitionException& e) {
      	throw MParser_TokenStreamRecognitionException(e);
   }
	}
   catch (MParser_CharStreamException& cse) {
      if ( ACE_dynamic_cast(MParser_CharStreamIOException*, &cse)) {
      	throw MParser_TokenStreamIOException(((MParser_CharStreamIOException&)cse).io);
   }
      else {
      	throw MParser_TokenStreamException(cse.getMessage().c_str());
   }
	}
}
}

void ANTLRTokdefLexer:: mWS(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = WS;
   int _saveIndex;
   
   {
   switch ( LA(1)) {
   case ' ':
   {
      match(' ');
      break;
	}
   case '\t':
   {
      match('\t');
      break;
	}
   case '\r':
   {
      match('\r');
      {
      if ((LA(1)=='\n')) {
      	match('\n');
   }
      else {
   }
      
   }
      newline();
      break;
	}
   case '\n':
   {
      match('\n');
      newline();
      break;
	}
   default:
   {
      throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
	}
	}
	}
   _ttype = MParser_Token::SKIP;
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mSL_COMMENT(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = SL_COMMENT;
   int _saveIndex;
   
   match("//");
   {
   LABEL(_loop12)
   do {
      if ((_tokenSet_0.member(LA(1)))) {
      	{
      	match(_tokenSet_0);
      }
   }
      else {
      	BREAK(_loop12);
   }
      
	} while (true);
	}
   {
   switch ( LA(1)) {
   case '\n':
   {
      match('\n');
      break;
	}
   case '\r':
   {
      match('\r');
      {
      if ((LA(1)=='\n')) {
      	match('\n');
   }
      else {
   }
      
   }
      break;
	}
   default:
   {
      throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
	}
	}
	}
   _ttype = MParser_Token::SKIP; newline();
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mML_COMMENT(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = ML_COMMENT;
   int _saveIndex;
   
   match("/*");
   {
   LABEL(_loop17)
   do {
      if ((LA(1)=='*') && (_tokenSet_1.member(LA(2)))) {
      	match('*');
      	matchNot('/');
   }
      else if ((LA(1)=='\n')) {
      	match('\n');
      	newline();
   }
      else if ((_tokenSet_2.member(LA(1)))) {
      	matchNot('*');
   }
      else {
      	BREAK(_loop17);
   }
      
	} while (true);
	}
   match("*/");
   _ttype = MParser_Token::SKIP;
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mLPAREN(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = LPAREN;
   int _saveIndex;
   
   match('(');
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mRPAREN(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = RPAREN;
   int _saveIndex;
   
   match(')');
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mASSIGN(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = ASSIGN;
   int _saveIndex;
   
   match('=');
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mSTRING(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = STRING;
   int _saveIndex;
   
   match('"');
   {
   LABEL(_loop23)
   do {
      if ((LA(1)=='\\')) {
      	mESC(false);
   }
      else if ((_tokenSet_3.member(LA(1)))) {
      	matchNot('"');
   }
      else {
      	BREAK(_loop23);
   }
      
	} while (true);
	}
   match('"');
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mESC(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = ESC;
   int _saveIndex;
   
   match('\\');
   {
   switch ( LA(1)) {
   case 'n':
   {
      match('n');
      break;
	}
   case 'r':
   {
      match('r');
      break;
	}
   case 't':
   {
      match('t');
      break;
	}
   case 'b':
   {
      match('b');
      break;
	}
   case 'f':
   {
      match('f');
      break;
	}
   case '"':
   {
      match('"');
      break;
	}
   case '\'':
   {
      match('\'');
      break;
	}
   case '\\':
   {
      match('\\');
      break;
	}
   case '0':  case '1':  case '2':  case '3':
   {
      {
      matchRange('0','3');
   }
      {
      if (((LA(1) >= '0' && LA(1) <= '9')) && ((LA(2) >= char(3) && LA(2) <= int(0xff)))) {
      	mDIGIT(false);
      	{
      	if (((LA(1) >= '0' && LA(1) <= '9')) && ((LA(2) >= char(3) && LA(2) <= int(0xff)))) {
         mDIGIT(false);
      }
      	else if (((LA(1) >= char(3) && LA(1) <= int(0xff))) && (true)) {
      }
      	else {
         throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
      }
      	
      }
   }
      else if (((LA(1) >= char(3) && LA(1) <= int(0xff))) && (true)) {
   }
      else {
      	throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
   }
      
   }
      break;
	}
   case '4':  case '5':  case '6':  case '7':
   {
      {
      matchRange('4','7');
   }
      {
      if (((LA(1) >= '0' && LA(1) <= '9')) && ((LA(2) >= char(3) && LA(2) <= int(0xff)))) {
      	mDIGIT(false);
   }
      else if (((LA(1) >= char(3) && LA(1) <= int(0xff))) && (true)) {
   }
      else {
      	throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
   }
      
   }
      break;
	}
   case 'u':
   {
      match('u');
      mXDIGIT(false);
      mXDIGIT(false);
      mXDIGIT(false);
      mXDIGIT(false);
      break;
	}
   default:
   {
      throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
	}
	}
	}
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mDIGIT(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = DIGIT;
   int _saveIndex;
   
   matchRange('0','9');
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mXDIGIT(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = XDIGIT;
   int _saveIndex;
   
   switch ( LA(1)) {
   case '0':  case '1':  case '2':  case '3':
   case '4':  case '5':  case '6':  case '7':
   case '8':  case '9':
   {
      matchRange('0','9');
      break;
	}
   case 'a':  case 'b':  case 'c':  case 'd':
   case 'e':  case 'f':
   {
      matchRange('a','f');
      break;
	}
   case 'A':  case 'B':  case 'C':  case 'D':
   case 'E':  case 'F':
   {
      matchRange('A','F');
      break;
	}
   default:
   {
      throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
	}
	}
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mID(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = ID;
   int _saveIndex;
   
   {
   switch ( LA(1)) {
   case 'a':  case 'b':  case 'c':  case 'd':
   case 'e':  case 'f':  case 'g':  case 'h':
   case 'i':  case 'j':  case 'k':  case 'l':
   case 'm':  case 'n':  case 'o':  case 'p':
   case 'q':  case 'r':  case 's':  case 't':
   case 'u':  case 'v':  case 'w':  case 'x':
   case 'y':  case 'z':
   {
      matchRange('a','z');
      break;
	}
   case 'A':  case 'B':  case 'C':  case 'D':
   case 'E':  case 'F':  case 'G':  case 'H':
   case 'I':  case 'J':  case 'K':  case 'L':
   case 'M':  case 'N':  case 'O':  case 'P':
   case 'Q':  case 'R':  case 'S':  case 'T':
   case 'U':  case 'V':  case 'W':  case 'X':
   case 'Y':  case 'Z':
   {
      matchRange('A','Z');
      break;
	}
   default:
   {
      throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
	}
	}
	}
   {
   LABEL(_loop36)
   do {
      switch ( LA(1)) {
      case 'a':  case 'b':  case 'c':  case 'd':
      case 'e':  case 'f':  case 'g':  case 'h':
      case 'i':  case 'j':  case 'k':  case 'l':
      case 'm':  case 'n':  case 'o':  case 'p':
      case 'q':  case 'r':  case 's':  case 't':
      case 'u':  case 'v':  case 'w':  case 'x':
      case 'y':  case 'z':
      {
      	matchRange('a','z');
      	break;
   }
      case 'A':  case 'B':  case 'C':  case 'D':
      case 'E':  case 'F':  case 'G':  case 'H':
      case 'I':  case 'J':  case 'K':  case 'L':
      case 'M':  case 'N':  case 'O':  case 'P':
      case 'Q':  case 'R':  case 'S':  case 'T':
      case 'U':  case 'V':  case 'W':  case 'X':
      case 'Y':  case 'Z':
      {
      	matchRange('A','Z');
      	break;
   }
      case '_':
      {
      	match('_');
      	break;
   }
      case '0':  case '1':  case '2':  case '3':
      case '4':  case '5':  case '6':  case '7':
      case '8':  case '9':
      {
      	matchRange('0','9');
      	break;
   }
      default:
      {
      	BREAK(_loop36);
   }
   }
	} while (true);
	}
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
void ANTLRTokdefLexer:: mINT(bool _createToken)

{
   int _ttype; MParser_Token_var _token; int _begin=text_.length();
   _ttype = INT;
   int _saveIndex;
   
   {
   int _cnt39=0;
   LABEL(_loop39)
   do {
      if (((LA(1) >= '0' && LA(1) <= '9'))) {
      	mDIGIT(false);
   }
      else {
      	if ( _cnt39>=1 ) { BREAK(_loop39); } else {throw MParser_NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
   }
      
      _cnt39++;
	} while (true);
	}
   if ( _createToken && MParser_Token::is_nil(_token) && _ttype!=MParser_Token::SKIP ) {
      _token = makeToken(_ttype);
      _token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));
	}
   returnToken_ = _token;
}
	
	
longArray ANTLRTokdefLexer::mk_tokenSet_0() {
   static ACE_UINT64 data[8];
   data[0]=-9224L;
   for (int i = 1; i<=3; i++) { data[i]=-1L; }
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRTokdefLexer::_tokenSet_0(ANTLRTokdefLexer::mk_tokenSet_0());
longArray ANTLRTokdefLexer::mk_tokenSet_1() {
   static ACE_UINT64 data[8];
   data[0]=-140737488355336L;
   for (int i = 1; i<=3; i++) { data[i]=-1L; }
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRTokdefLexer::_tokenSet_1(ANTLRTokdefLexer::mk_tokenSet_1());
longArray ANTLRTokdefLexer::mk_tokenSet_2() {
   static ACE_UINT64 data[8];
   data[0]=-4398046512136L;
   for (int i = 1; i<=3; i++) { data[i]=-1L; }
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRTokdefLexer::_tokenSet_2(ANTLRTokdefLexer::mk_tokenSet_2());
longArray ANTLRTokdefLexer::mk_tokenSet_3() {
   static ACE_UINT64 data[8];
   data[0]=-17179869192L;
   data[1]=-268435457L;
   for (int i = 2; i<=3; i++) { data[i]=-1L; }
   return longArray(data, sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRTokdefLexer::_tokenSet_3(ANTLRTokdefLexer::mk_tokenSet_3());

ANTLRTokdefLexer::~ANTLRTokdefLexer()

{
}
