#include "MParser/TokenBuffer.h"
#include "MGrammar/StringLiteralSymbol.h"
#include "MGrammar/TokenSymbol.h"
#include "MParser/CharScanner.h"
#include "MParser/TokenStreamException.h"
#include "MParser/TokenStreamIOException.h"
#include "MParser/Exception.h"
#include "MParser/LLkParser.h"
#include "MParser/Token.h"
#include "MParser/TokenStream.h"
#include "MParser/RecognitionException.h"
#include "MParser/NoViableAltException.h"
#include "MParser/MismatchedTokenException.h"
#include "MParser/SemanticException.h"
#include "MParser/ParserSharedInputState.h"
#include "MParser/BitSet.h"
#include "MGrammar/ANTLRTokdefParser.h"

/** In order to make it so existing subclasses don't break, we won't require
 * that the antlr.Tool instance be passed as a constructor element. Instead,
 * the antlr.Tool instance should register itself via {@link #initTool(antlr.Tool)}
 * @throws IllegalStateException if a tool has already been registered
 * @since 2.7.2
 */
void ANTLRTokdefParser::setTool(MGrammar_Tool_ptr tool)

{
   if (antlrTool_ == 0)
    {
      antlrTool_ = tool;
    }
   else
    {
    #if 0
      throw MParser_IllegalStateException("antlr.Tool already registered");
      #else
      assert(0);
      #endif
    }
}

/** Delegates the error message to the tool if any was registered via
 *  {@link #initTool(antlr.Tool)}
 *  @since 2.7.2
 */
void ANTLRTokdefParser::reportError (const MLANG_String& s)

{
   if (getTool() != 0) {
      getTool()->error(s, getFilename(), -1, -1);
    }
   else
    {
      super::reportError(s);
    }
}

/** Delegates the error message to the tool if any was registered via
 *  {@link #initTool(antlr.Tool)}
 *  @since 2.7.2
 */
void ANTLRTokdefParser::reportError(MParser_RecognitionException& e)

{
  if (getTool() != 0)
   {
     getTool()->error(e.getErrorMessage().c_str(), e.getFilename(), e.getLine(), e.getColumn());
   }
  else
   {
     super::reportError(e);
   }
}

/** Delegates the warning message to the tool if any was registered via
 *  {@link #initTool(antlr.Tool)}
 *  @since 2.7.2
 */
void ANTLRTokdefParser::reportWarning (const MLANG_String& s)

{
  if (getTool() != 0)
   {
     getTool()->warning(s, getFilename(), -1, -1);
   }
  else
   {
     super::reportWarning(s);
   }
}

const char* ANTLRTokdefParser::_tokenNames[] = {
   "<0>",
   "EOF",
   "<2>",
   "NULL_TREE_LOOKAHEAD",
   "ID",
   "STRING",
   "ASSIGN",
   "LPAREN",
   "RPAREN",
   "INT",
   "WS",
   "SL_COMMENT",
   "ML_COMMENT",
   "ESC",
   "DIGIT",
   "XDIGIT"
};
	
ANTLRTokdefParser::ANTLRTokdefParser (
    const MParser_TokenBuffer_var& tokenBuf,
    int k
)
  : super(tokenBuf,k)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

ANTLRTokdefParser::ANTLRTokdefParser (
   const MParser_TokenStream_var& lexer,
   int k
)
 : super(lexer,k)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

ANTLRTokdefParser::ANTLRTokdefParser (
   const MParser_ParserSharedInputState_var& state
)
 : super(state,3)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

void ANTLRTokdefParser::file (
   MGrammar_ImportVocabTokenManager_ptr tm
)

{
   
   MParser_Token_var  name;
   
   try {   // for error handling
            name = LT(1);
            match(ID);
            {
            LABEL(_loop3)
            do {
            	if ((LA(1)==ID||LA(1)==STRING)) {
               line(tm);
            }
            	else {
               BREAK(_loop3);
            }
            	
   } while (true);
   }
	}
   catch (MParser_RecognitionException& ex) {
            reportError(ex);
            consume();
            consumeUntil(_tokenSet_0);
	}
}
	
void ANTLRTokdefParser::line (
   MGrammar_ImportVocabTokenManager_ptr tm
)	

{
   
   MParser_Token_var  s1;
   MParser_Token_var  lab;
   MParser_Token_var  s2;
   MParser_Token_var  id;
   MParser_Token_var  para;
   MParser_Token_var  id2;
   MParser_Token_var  i;
   MParser_Token_var t; MParser_Token_var s;
   
   try
    {  // for error handling
       {
          if ((LA(1)==STRING))
	   {
             s1 = LT(1);
             match(STRING);
             s = s1;
           }
          else if ((LA(1)==ID) && (LA(2)==ASSIGN) && (LA(3)==STRING))
	   {
             lab = LT(1);
             match(ID);
             t = lab;
             match(ASSIGN);
             s2 = LT(1);
             match(STRING);
             s = s2;
           }
          else if ((LA(1)==ID) && (LA(2)==LPAREN))
	   {
             id = LT(1);
             match(ID);
             t=id;
             match(LPAREN);
             para = LT(1);
             match(STRING);
             match(RPAREN);
           }
          else if ((LA(1)==ID) && (LA(2)==ASSIGN) && (LA(3)==INT))
	   {
             id2 = LT(1);
             match(ID);
             t=id2;
           }
          else
	   {
             throw MParser_NoViableAltException(LT(1), getFilename());
           }
            
       }
            match(ASSIGN);
       i = LT(1);
       match(INT);
       
       int value = atoi(i->getText().c_str());
       // if literal found, define as a string literal
       if ( !MParser_Token::is_nil(s) )
	{
          tm->define(s->getText(), value);
          // if label, then label the string and map label to token symbol also
          if ( !MParser_Token::is_nil(t))
	   {
             MGrammar_StringLiteralSymbol_var sl =
               MGrammar_StringLiteralSymbol::_narrow(tm->getTokenSymbol(s->getText()));
             sl->setLabel(t->getText());
             tm->mapToTokenSymbol(t->getText(), sl);
           }
        }
       // define token (not a literal)
       else if ( !MParser_Token::is_nil(t) )
	{
          tm->define(t->getText(), value);
          if ( !MParser_Token::is_nil(para) )
	   {
             MGrammar_TokenSymbol_var ts = tm->getTokenSymbol(t->getText());
             ts->setParaphrase (
                para->getText()
                 );
           }
        }
               
    }
   catch (MParser_RecognitionException& ex)
    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_1);
    }
}
	
	
longArray ANTLRTokdefParser::mk_tokenSet_0() {
   static ACE_UINT64 data[] = { 2L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRTokdefParser::_tokenSet_0(ANTLRTokdefParser::mk_tokenSet_0());
longArray ANTLRTokdefParser::mk_tokenSet_1() {
   static ACE_UINT64 data[] = { 50L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet ANTLRTokdefParser::_tokenSet_1(ANTLRTokdefParser::mk_tokenSet_1());

ANTLRTokdefParser::~ANTLRTokdefParser()

{
}
