#ifndef __MGRAMMAR_ALTERNATIVE_H__
#define __MGRAMMAR_ALTERNATIVE_H__
#include "MLANG/Vector_T.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MParser/Token.h"
#include "MGrammar/Lookahead.h"
#include "MGrammar/ExceptionSpec.h"
#include "MGrammar/AlternativeElement.h"
#include "MGrammar/SynPredBlock.h"

class MGrammar_Alternative;
typedef class MGrammar_Alternative *MGrammar_Alternative_ptr;
typedef class MGrammar_Alternative *MGrammar_Alternative_var;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/Alternative.java#1 $
 */

/** Intermediate data class holds information about an alternative */
class MGrammar_Alternative
{
public:
    // Tracking alternative linked list
    MGrammar_AlternativeElement_var head_;   // head of alt element list
    MGrammar_AlternativeElement_var tail_;  // last element added

    // Syntactic predicate block if non-null
    MGrammar_SynPredBlock_var synPred_;
    // Semantic predicate action if non-null
    MLANG_String semPred_;
    // Exception specification if non-null
    MGrammar_ExceptionSpec_var exceptionSpec_;
    // Init action if non-null;
    MLANG_Vector<MGrammar_Lookahead> cache_;	// lookahead for alt.  Filled in by
    // deterministic() only!!!!!!!  Used for
    // code gen after calls to deterministic()
    // and used by deterministic for (...)*, (..)+,
    // and (..)? blocks.  1..k
    int lookaheadDepth_;// each alt has different look depth possibly.
    // depth can be NONDETERMINISTIC too.
    // 0..n-1
// If non-null, Tree specification ala -> A B C (not implemented)
    MParser_Token_var treeSpecifier_;
    // True of AST generation is on for this alt
    bool doAutoGen_;

public:

    MGrammar_Alternative()
      : doAutoGen_(false)
    {
    }

    MGrammar_Alternative(const MGrammar_AlternativeElement_var& firstElement)
      : doAutoGen_(false)
    {
        addElement(firstElement);
    }

    MGrammar_Alternative(const MGrammar_Alternative& b);

    ~MGrammar_Alternative();

public:

    MGrammar_Alternative& operator=(const MGrammar_Alternative& b);

public:

    void addElement(const MGrammar_AlternativeElement_var& e);

    bool atStart() const
    {
       return !head_;
    }

    bool getAutoGen() const
    {
        // Don't build an AST if there is a tree-rewrite-specifier
       return doAutoGen_ && !treeSpecifier_;
    }

    const MParser_Token_var& getTreeSpecifier() const
    {
        return treeSpecifier_;
    }

    void setAutoGen(bool doAutoGen)
    {
        doAutoGen_ = doAutoGen;
    }
};

#endif /* __MGRAMMAR_ALTERNATIVE_H__ */
