#ifndef __MGRAMMAR_ALTERNATIVEBLOCK_H__
#define __MGRAMMAR_ALTERNATIVEBLOCK_H__
#include "ace/SString.h"
#include "ace/Vector_T.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MParser/Token.h"
#include "MGrammar/AlternativeElement.h"

class MGrammar_Alternative;
typedef class MGrammar_Alternative *MGrammar_Alternative_ptr;

class MGrammar_AlternativeBlock;
typedef class MGrammar_AlternativeBlock *MGrammar_AlternativeBlock_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/AlternativeBlock.java#1 $
 */

typedef ACE_Vector<MGrammar_Alternative_ptr> ALTVEC;

/**A list of alternatives */
class MGrammar_AlternativeBlock : public MGrammar_AlternativeElement

{
public:
    MLANG_String initAction_;
    ALTVEC alternatives_;	// Contains Alternatives

    MLANG_String label_;			// can label a looping block to break out of it.

    int alti, altj;		// which alts are being compared at the moment with
    // deterministic()?
    int analysisAlt;		// which alt are we computing look on?  Must be alti or altj

    bool hasAnAction_;	// does any alt have an action?
    bool hasASynPred_;	// does any alt have a syntactic predicate?

    int ID_;				// used to generate unique variables
    static int nblks_;	// how many blocks have we allocated?
    bool no_;				// true if block is inverted.

    bool greedy_;			// Blocks are greedy by default
    bool greedySet_;		// but, if not explicitly greedy, warning might be generated

    bool doAutoGen_;	// false if no AST (or text) to be generated for block

    bool warnWhenFollowAmbig_; // warn when an empty path or exit path

    bool generateAmbigWarnings_;  // the general warning "shut-up" mechanism
    // conflicts with alt of subrule.
    // Turning this off will suppress stuff
    // like the if-then-else ambig.

public:

    MGrammar_AlternativeBlock (
        MGrammar_Grammar_ptr g
     );

    MGrammar_AlternativeBlock (
       MGrammar_Grammar_ptr g, 
       const MParser_Token_var& start, 
       bool no
     );

    virtual ~MGrammar_AlternativeBlock();

public:

   static bool is_nil (MGrammar_AlternativeBlock_ptr a)
   { return a == 0; }

   static MGrammar_AlternativeBlock_ptr _nil()
   { return 0; }

   static MGrammar_AlternativeBlock_ptr _duplicate(MGrammar_AlternativeBlock_ptr block)
   { return ::MLANG_acquire(block); }

   static MGrammar_AlternativeBlock_ptr _narrow(MGrammar_AlternativeElement_ptr e)
   { return ::MLANG_acquire(ACE_dynamic_cast(MGrammar_AlternativeBlock_ptr, e)); }

public:

    bool hasAnAction() const
    { return hasAnAction_; }

    bool hasASynPred() const
    { return hasASynPred_; }

    int ID() const
    { return ID_; }

    bool no() const
    { return no_; }

    bool greedy() const
    { return greedy_; }

    bool greedySet() const
    { return greedySet_; }

    void addAlternative(MGrammar_Alternative_ptr alt);

    MGrammar_Alternative_ptr getAlternativeAt(size_t i) const
    { return alternatives_[i]; }

    const ALTVEC& getAlternatives() const 
    {
        return alternatives_;
    }

    bool getAutoGen() const
    {
        return doAutoGen_;
    }

    const MLANG_String& getInitAction() const
    {
        return initAction_;
    }

    const MLANG_String& getLabel() const
    {
        return label_;
    }

public:

    virtual void generate();

    MGrammar_Lookahead look(int k);

    virtual void prepareForAnalysis();

    /**Walk the syntactic predicate and, for a rule ref R, remove
     * the ref from the list of FOLLOW references for R (stored
     * in the symbol table.
     */
    virtual void removeTrackingOfRuleRefs(MGrammar_Grammar_ptr g);

    void setAlternatives(const ALTVEC& v)
    {
        alternatives_ = v;
    }

    void setAutoGen(bool doAutoGen)
    {
        doAutoGen = doAutoGen_;
    }

    void setInitAction (const MLANG_String& initAction)
    {
        initAction_ = initAction;
    }

    void setLabel(const MLANG_String& label)
    {
        label_ = label;
    }

    void setOption (
       const MParser_Token_var& key, 
       const MParser_Token_var& value
     );

    MLANG_String toString() const;
};

typedef MLANG_SmartPtr<MGrammar_AlternativeBlock> MGrammar_AlternativeBlock_var;

#endif /* __MGRAMMAR_ALTERNATIVEBLOCK_H__ */
