#ifndef __MPARSE_BLOCKCONTEXT_H__
#define __MPARSE_BLOCKCONTEXT_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MGrammar/Alternative.h"
#include "MGrammar/AlternativeBlock.h"
#include "MGrammar/BlockEndElement.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/BlockContext.java#1 $
 */

class MGrammar_BlockContext;
typedef class MGrammar_BlockContext *MGrammar_BlockContext_ptr;

/**BlockContext stores the information needed when creating an
 * alternative (list of elements).  Entering a subrule requires
 * that we save this state as each block of alternatives
 * requires state such as "tail of current alternative."
 */
class MGrammar_BlockContext : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
public:
    MGrammar_AlternativeBlock_var block_; // current block of alternatives
    int altNum_;  // which alt are we accepting 0..n-1
    MGrammar_BlockEndElement_var blockEnd_; // used if nested

public:

    MGrammar_BlockContext() : altNum_(0) {}

    virtual ~MGrammar_BlockContext() {}

public:

   static bool is_nil (MGrammar_BlockContext_ptr bc)
   { return bc == 0; }

   static MGrammar_BlockContext_ptr _nil()
   { return 0; }

   static MGrammar_BlockContext_ptr _duplicate(MGrammar_BlockContext_ptr bc)
   { return MLANG_acquire(bc); }

public:
    virtual void addAlternativeElement(const MGrammar_AlternativeElement_var& e)
    {
        currentAlt()->addElement(e);
    }

    const MGrammar_Alternative_var& currentAlt() 
    {
        return block_->alternatives_[altNum_];
    }

    const MGrammar_AlternativeElement_var& currentElement()
    {
        return currentAlt()->tail_;
    }
};

typedef MLANG_SmartPtr<MGrammar_BlockContext> MGrammar_BlockContext_var;

#endif /* __MPARSE_BLOCKCONTEXT_H__ */
