#include "MGrammar/ANTLRLexer.h"
#include "MGrammar/CodeGenerator.h"
#include "MGrammar/LexerGrammar.h"
#include "MGrammar/CharRangeElement.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/CharRangeElement.java#1 $
 */


MGrammar_CharRangeElement::MGrammar_CharRangeElement (
   MGrammar_LexerGrammar_ptr g, 
   const MParser_Token_var& t1, 
   const MParser_Token_var& t2, 
   int autoGenType
) : MGrammar_AlternativeElement(g),
    begin_(0),
    end_(0)

{
   begin_ = (char)ANTLRLexer::tokenTypeForCharLiteral(t1->getText());
   beginText_ = t1->getText();
   end_ = (char)ANTLRLexer::tokenTypeForCharLiteral(t2->getText());
   endText_ = t2->getText();
   line_ = t1->getLine();
   // track which characters are referenced in the grammar
   for (int i = begin_; i <= end_; i++)
     {
       g->charVocabulary_.add(i);
     }
   this->autoGenType_ = autoGenType;
}

void MGrammar_CharRangeElement::generate()

{ 
   grammar_->generator_->gen(this);
}

MGrammar_Lookahead MGrammar_CharRangeElement::look(int k)

{
   return grammar_->theLLkAnalyzer_->look(k, this);
}

void MGrammar_CharRangeElement::setLabel(const MLANG_String& label)

{
   label_ = label;
}

MLANG_String MGrammar_CharRangeElement::toString() const

{
   if (label_.length() != 0)
      return " " + label_ + ":" + beginText_ + ".." + endText_;
   else
      return " " + beginText_ + ".." + endText_;
}

MGrammar_CharRangeElement::~MGrammar_CharRangeElement ()

{
}
