#ifndef __MGRAMMAR_CODEGENERATOR_H__
#define __MGRAMMAR_CODEGENERATOR_H__
#include "MLANG/Vector_T.h"
#include "MLANG/String.h"
#include "MLANG/Object.h"
#include "MLANG/Class.h"
#include "MIO/PrintWriter.h"
#include "MParser/CharFormatter.h"
#include "MParser/Token.h"
#include "MGrammar/TokenRefElement.h"
#include "MGrammar/ActionTransInfo.h"
#include "MGrammar/ActionElement.h"
#include "MGrammar/CharLiteralElement.h"
#include "MGrammar/CharRangeElement.h"
#include "MGrammar/StringLiteralElement.h"
#include "MGrammar/TokenRangeElement.h"
#include "MGrammar/TreeElement.h"
#include "MGrammar/RuleRefElement.h"
#include "MGrammar/WildcardElement.h"
#include "MGrammar/OneOrMoreBlock.h"
#include "MGrammar/ZeroOrMoreBlock.h"
#include "MGrammar/LexerGrammar.h"
#include "MGrammar/ParserGrammar.h"
#include "MGrammar/TreeWalkerGrammar.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/CodeGenerator.java#1 $
 */

class MGrammar_Tool;
typedef class MGrammar_Tool *MGrammar_Tool_ptr;
class MGrammar_Grammar;
typedef class MGrammar_Grammar *MGrammar_Grammar_ptr;
//class MGrammar_LexerGrammar;
//typedef class MGrammar_LexerGrammar *MGrammar_LexerGrammar_ptr;
//class MGrammar_ParserGrammar;
//typedef class MGrammar_ParserGrammar *MGrammar_ParserGrammar_ptr;
class MGrammar_TreeWalkerGrammar;
typedef class MGrammar_TreeWalkerGrammar *MGrammar_TreeWalkerGrammar_ptr;

class MGrammar_CodeGenerator;
typedef class MGrammar_CodeGenerator *MGrammar_CodeGenerator_ptr;
typedef class MGrammar_CodeGenerator *MGrammar_CodeGenerator_var;

#include "MGrammar/DefineGrammarSymbols.h"
#include "MGrammar/LLkGrammarAnalyzer.h"

/**A generic ANTLR code generator.  All code generators
 * Derive from this class.
 *
 * <p>
 * A CodeGenerator knows about a Grammar data structure and
 * a grammar analyzer.  The Grammar is walked to generate the
 * appropriate code for both a parser and lexer (if present).
 * This interface may change slightly so that the lexer is
 * itself living inside of a Grammar object (in which case,
 * this class generates only one recognizer).  The main method
 * to call is <tt>gen()</tt>, which initiates all code gen.
 *
 * <p>
 * The interaction of the code generator with the analyzer is
 * simple: each subrule block calls deterministic() before generating
 * code for the block.  Method deterministic() sets lookahead caches
 * in each Alternative object.  Technically, a code generator
 * doesn't need the grammar analyzer if all lookahead analysis
 * is done at runtime, but this would result in a slower parser.
 *
 * <p>
 * This class provides a set of support utilities to handle argument
 * list parsing and so on.
 *
 * @author  Terence Parr, John Lilley
 * @version 2.00a
 * @see     antlr.JavaCodeGenerator
 * @see     antlr.DiagnosticCodeGenerator
 * @see     antlr.LLkAnalyzer
 * @see     antlr.Grammar
 * @see     antlr.AlternativeElement
 * @see     antlr.Lookahead
 */
class MGrammar_CodeGenerator : public MLANG_Object

{
protected:
    MGrammar_Tool_ptr antlrTool_;

    /** Current tab indentation for code output */
    int tabs_;

    /** Current output Stream */
    MIO_PrintWriter_var currentOutput_; // SAS: for proper text i/o

    /** The grammar for which we generate code */
    MGrammar_Grammar_ptr grammar_;

    /** List of all bitsets that must be dumped.  These are Vectors of BitSet. */
    MLANG_Vector<MParser_BitSet> bitsetsUsed_;

    /** The grammar behavior */
    MGrammar_DefineGrammarSymbols_var behavior_;

    /** The LLk analyzer */
    MGrammar_LLkGrammarAnalyzer_var analyzer_;

    /** Object used to format characters in the target language.
     * subclass must initialize this to the language-specific formatter
     */
    MParser_CharFormatter_var charFormatter_;

    /** Use option "codeGenDebug" to generate debugging output */
    bool DEBUG_CODE_GENERATOR;

    /** Default values for code-generation thresholds */
    enum
    {
       DEFAULT_MAKE_SWITCH_THRESHOLD = 2,
       DEFAULT_BITSET_TEST_THRESHOLD = 4,

    /** If there are more than 8 long words to init in a bitset,
     *  try to optimize it; e.g., detect runs of -1L and 0L.
     */
       BITSET_OPTIMIZE_INIT_THRESHOLD = 8
    };

    /** This is a hint for the language-specific code generator.
     * A switch() or language-specific equivalent will be generated instead
     * of a series of if/else statements for blocks with number of alternates
     * greater than or equal to this number of non-predicated LL(1) alternates.
     * This is modified by the grammar option "codeGenMakeSwitchThreshold"
     */
    int makeSwitchThreshold_;

    /** This is a hint for the language-specific code generator.
     * A bitset membership test will be generated instead of an
     * ORed series of LA(k) comparisions for lookahead sets with
     * degree greater than or equal to this value.
     * This is modified by the grammar option "codeGenBitsetTestThreshold"
     */
    int bitsetTestThreshold_;

    static bool OLD_ACTION_TRANSLATOR_;

public:

    static MLANG_String TokenTypesFileSuffix_;
    static MLANG_String TokenTypesFileExt_;

public:

    /** Construct code generator base class */
    MGrammar_CodeGenerator();

    virtual ~MGrammar_CodeGenerator();

public:
    
    DECLARE_CLASS_INFO();

    /** Output a String to the currentOutput stream.
     * Ignored if string is null.
     * @param s The string to output
     */
    virtual void _print(const MLANG_String& s);

    /** Print an action without leading tabs, attempting to
     * preserve the current indentation level for multi-line actions
     * Ignored if string is null.
     * @param s The action string to output
     */
    virtual void _printAction(const MLANG_String& s);

    /** Output a String followed by newline, to the currentOutput stream.
     * Ignored if string is null.
     * @param s The string to output
     */
    virtual void _println(const MLANG_String& s);

    /** Test if a set element array represents a contiguous range.
     * @param elems The array of elements representing the set, usually from BitSet.toArray().
     * @return true if the elements are a contiguous range (with two or more).
     */
    static bool elementsAreRange(const MLANG_Vector<size_t>& elems);

    /** Get the identifier portion of an argument-action token.
     * The ID of an action is assumed to be a trailing identifier.
     * Specific code-generators may want to override this
     * if the language has unusual declaration syntax.
     * @param t The action token
     * @return A string containing the text of the identifier
     */
protected:
    virtual MLANG_String extractIdOfAction(MParser_Token_ptr t);

    /** Get the identifier portion of an argument-action.
     * The ID of an action is assumed to be a trailing identifier.
     * Specific code-generators may want to override this
     * if the language has unusual declaration syntax.
     * @param s The action text
     * @param line Line used for error reporting.
     * @param column Line used for error reporting.
     * @return A string containing the text of the identifier
     */
    virtual MLANG_String extractIdOfAction(const MLANG_String& s, int line, int column);

    /** Get the type string out of an argument-action token.
     * The type of an action is assumed to precede a trailing identifier
     * Specific code-generators may want to override this
     * if the language has unusual declaration syntax.
     * @param t The action token
     * @return A string containing the text of the type
     */
    virtual MLANG_String extractTypeOfAction(MParser_Token_ptr t);

    /** Get the type portion of an argument-action.
     * The type of an action is assumed to precede a trailing identifier
     * Specific code-generators may want to override this
     * if the language has unusual declaration syntax.
     * @param s The action text
     * @param line Line used for error reporting.
     * @return A string containing the text of the type
     */
    virtual MLANG_String extractTypeOfAction(const MLANG_String& s, int line, int column);

public:

    /** Generate the code for all grammars
     */
    virtual void gen() = 0;

    /** Generate code for the given grammar element.
     * @param action The {...} action to generate
     */
    virtual void gen(MGrammar_ActionElement_ptr action) = 0;

    /** Generate code for the given grammar element.
     * @param blk The "x|y|z|..." block to generate
     */
    virtual void gen(MGrammar_AlternativeBlock_ptr blk) = 0;

    /** Generate code for the given grammar element.
     * @param end The block-end element to generate.  Block-end
     * elements are synthesized by the grammar parser to represent
     * the end of a block.
     */
    virtual void gen(MGrammar_BlockEndElement_ptr end) = 0;

    /** Generate code for the given grammar element.
     * @param atom The character literal reference to generate
     */
    virtual void gen(MGrammar_CharLiteralElement_ptr atom) = 0;

    /** Generate code for the given grammar element.
     * @param r The character-range reference to generate
     */
    virtual void gen(MGrammar_CharRangeElement_ptr r) = 0;

    /** Generate the code for a parser */
    virtual void gen(MGrammar_LexerGrammar_ptr g) = 0;

    /** Generate code for the given grammar element.
     * @param blk The (...)+ block to generate
     */
    virtual void gen(MGrammar_OneOrMoreBlock_ptr blk) = 0;

    /** Generate the code for a parser */
    virtual void gen(MGrammar_ParserGrammar_ptr g) = 0;

    /** Generate code for the given grammar element.
     * @param rr The rule-reference to generate
     */
    virtual void gen(MGrammar_RuleRefElement_ptr rr) = 0;

    /** Generate code for the given grammar element.
     * @param atom The string-literal reference to generate
     */
    virtual void gen(MGrammar_StringLiteralElement_ptr atom) = 0;

    /** Generate code for the given grammar element.
     * @param r The token-range reference to generate
     */
    virtual void gen(MGrammar_TokenRangeElement_ptr r) = 0;

    /** Generate code for the given grammar element.
     * @param atom The token-reference to generate
     */
    virtual void gen(MGrammar_TokenRefElement_ptr atom) = 0;

    /** Generate code for the given grammar element.
     * @param blk The tree to generate code for.
     */
    virtual void gen(MGrammar_TreeElement_ptr t) = 0;

    /** Generate the code for a parser */
    virtual void gen(MGrammar_TreeWalkerGrammar_ptr g) = 0;

    /** Generate code for the given grammar element.
     * @param wc The wildcard element to generate
     */
    virtual void gen(MGrammar_WildcardElement_ptr wc) = 0;

    /** Generate code for the given grammar element.
     * @param blk The (...)* block to generate
     */
    virtual void gen(MGrammar_ZeroOrMoreBlock_ptr blk) = 0;

    /** Generate the token types as a text file for persistence across shared lexer/parser */
protected:

    virtual void genTokenInterchange(MGrammar_TokenManager_ptr tm);

public:
    /** Process a string for an simple expression for use in xx/action.g
     * it is used to cast simple tokens/references to the right type for
     * the generated language.
     * @param str A String.
     */
    virtual MLANG_String processStringForASTConstructor(const MLANG_String& str) const
    {
        return str;
    }

    /** Get a string for an expression to generate creation of an AST subtree.
     * @param v A Vector of String, where each element is an expression in the target language yielding an AST node.
     */

    virtual MLANG_String getASTCreateString(const MLANG_Vector<MLANG_String>& v) const = 0;

    /** Get a string for an expression to generate creating of an AST node
     * @param str The text of the arguments to the AST construction
     */
    virtual MLANG_String getASTCreateString (
         MGrammar_GrammarAtom_ptr atom, 
         const MLANG_String& str
      ) = 0;

protected:
    /** Given the index of a bitset in the bitset list, generate a unique name.
     * Specific code-generators may want to override this
     * if the language does not allow '_' or numerals in identifiers.
     * @param index  The index of the bitset in the bitset list.
     */
    MLANG_String getBitsetName(size_t index) const;

public:

    static MLANG_String encodeLexerRuleName(const MLANG_String& id);

    static MLANG_String decodeLexerRuleName(const MLANG_String& id);

    /** Map an identifier to it's corresponding tree-node variable.
     * This is context-sensitive, depending on the rule and alternative
     * being generated
     * @param id The identifier name to map
     * @param forInput true if the input tree node variable is to be returned, otherwise the output variable is returned.
     * @return The mapped id (which may be the same as the input), or null if the mapping is invalid due to duplicates
     */
    virtual MLANG_String mapTreeId(const MLANG_String& id, MGrammar_ActionTransInfo_ptr tInfo) = 0;

protected:
    /** Add a bitset to the list of bitsets to be generated.
     * if the bitset is already in the list, ignore the request.
     * Always adds the bitset to the end of the list, so the
     * caller can rely on the position of bitsets in the list.
     * The returned position can be used to format the bitset
     * name, since it is invariant.
     * @param p Bit set to mark for code generation
     * @param forParser true if the bitset is used for the parser, false for the lexer
     * @return The position of the bitset in the list.
     */
    virtual int markBitsetForGen(const MParser_BitSet& p);

    /** Output tab indent followed by a String, to the currentOutput stream.
     * Ignored if string is null.
     * @param s The string to output.
     */
    virtual void print(const MLANG_String& s);

    /** Print an action with leading tabs, attempting to
     * preserve the current indentation level for multi-line actions
     * Ignored if string is null.
     * @param s The action string to output
     */
    virtual void printAction(const MLANG_String& s);

    /** Output tab indent followed by a String followed by newline,
     * to the currentOutput stream.  Ignored if string is null.
     * @param s The string to output
     */
    virtual void println(const MLANG_String& s);

    /** Output the current tab indentation.  This outputs the number of tabs
     * indicated by the "tabs" variable to the currentOutput stream.
     */
    virtual void printTabs();

    /** Lexically process $ and # references within the action.
     *  This will replace #id and #(...) with the appropriate
     *  function calls and/or variables etc...
     */
    virtual MLANG_String processActionForSpecialSymbols (
        const MLANG_String& actionStr,
        int line,
        MGrammar_RuleBlock_ptr currentRule,
        MGrammar_ActionTransInfo_ptr tInfo
      ) = 0;

public:

     MLANG_String getFOLLOWBitSet(const MLANG_String& ruleName, int k);

     MLANG_String getFIRSTBitSet(const MLANG_String& ruleName, int k);

protected:

    /**
     * Remove the assignment portion of a declaration, if any.
     * @param d the declaration
     * @return the declaration without any assignment portion
     */
    MLANG_String removeAssignmentFromDeclaration(const MLANG_String& d);

private:
    /** Set all fields back like one just created */
    void reset();

public:

    static MLANG_String reverseLexerRuleName(const MLANG_String& id);

    virtual void setAnalyzer(const MGrammar_LLkGrammarAnalyzer_var& analyzer);

    virtual void setBehavior(const MGrammar_DefineGrammarSymbols_var& behavior);

protected:
    /** Set a grammar for the code generator to use */
    void setGrammar(MGrammar_Grammar_ptr g);

public:
   friend class __MGrammar_CodeGenerator_dict;
    void setTool(MGrammar_Tool_ptr tool);
};

#endif /* __MGRAMMAR_CODEGENERATOR_H__ */
