#ifndef __MGRAMMAR_CPPACTIONLEXER_H__
#define __MGRAMMAR_CPPACTIONLEXER_H__
#include "MLANG/Vector_T.h"
#include "MIO/InputStream.h"
#include "MIO/Reader.h"
#include "MParser/TokenStreamException.h"
#include "MParser/TokenStreamIOException.h"
#include "MParser/TokenStreamRecognitionException.h"
#include "MParser/CharStreamException.h"
#include "MParser/CharStreamIOException.h"
#include "MParser/Exception.h"
#include "MParser/CharScanner.h"
#include "MParser/InputBuffer.h"
#include "MParser/ByteBuffer.h"
#include "MParser/CharBuffer.h"
#include "MParser/Token.h"
#include "MParser/CommonToken.h"
#include "MParser/RecognitionException.h"
#include "MParser/NoViableAltForCharException.h"
#include "MParser/MismatchedCharException.h"
#include "MParser/TokenStream.h"
#include "MParser/LexerSharedInputState.h"
#include "MParser/BitSet.h"
#include "MParser/SemanticException.h"
#include "MGrammar/Tool.h"
#include "MGrammar/CodeGenerator.h"
#include "MGrammar/ActionTransInfo.h"
#include "MGrammar/CppActionLexerTokenTypes.h"

class MGrammar_CppActionLexer;
typedef MGrammar_CppActionLexer *MGrammar_CppActionLexer_ptr;
typedef MLANG_SmartPtr<MGrammar_CppActionLexer> MGrammar_CppActionLexer_var;

/** Perform the following translations:

    AST related translations

	##				-> currentRule_AST
	#(x,y,z)		-> codeGenerator.getASTCreateString(vector-of(x,y,z))
	#[x]			-> codeGenerator.getASTCreateString(x)
	#x				-> codeGenerator.mapTreeId(x)

	Inside context of #(...), you can ref (x,y,z), [x], and x as shortcuts.

    Text related translations

	$append(x)	  -> text.append(x)
	$setText(x)	  -> text.setLength(_begin); text.append(x)
	$getText		  -> new String(text.getBuffer(),_begin,text.length()-_begin)
	$setToken(x)  -> _token = x
	$setType(x)	  -> _ttype = x
   $FOLLOW(r)    -> FOLLOW set name for rule r (optional arg)
   $FIRST(r)     -> FIRST set name for rule r (optional arg)
 */
class MGrammar_CppActionLexer : public MParser_CharScanner, 
                                public MGrammar_CppActionLexerTokenTypes

{
public:
   typedef MParser_CharScanner super;
protected:
   MGrammar_RuleBlock_var currentRule_;
protected:
   MGrammar_CodeGenerator_ptr generator_;
protected:
   int lineOffset_;
private:
   MGrammar_Tool_ptr antlrTool_;	// The ANTLR tool
   MGrammar_ActionTransInfo *transInfo_;

public:
   MGrammar_CppActionLexer (
      const MLANG_String& s, 
      const MGrammar_RuleBlock_var& currentRule,
      MGrammar_CodeGenerator_ptr generator,
      MGrammar_ActionTransInfo *transInfo
    );

public:
   void setLineOffset(int lineOffset);

public:
   void setTool(MGrammar_Tool_ptr tool);
public:
   void reportError(MParser_RecognitionException& e);
public:
   void reportError(const MLANG_String& s);
public:
   void reportWarning(const MLANG_String& s);
public:
   MGrammar_CppActionLexer(const MIO_InputStream_var& in);
public:
   MGrammar_CppActionLexer(const MIO_Reader_var& in);
public:
   MGrammar_CppActionLexer(const MParser_InputBuffer_var& ib);
public:
   MGrammar_CppActionLexer(const MParser_LexerSharedInputState_var& state);

public:

   static bool is_nil (MGrammar_CppActionLexer_ptr lexer)
   { return lexer == 0; }

   static MGrammar_CppActionLexer_ptr _nil()
   { return 0; }

   static MGrammar_CppActionLexer_ptr _duplicate(MGrammar_CppActionLexer_ptr lexer)
   { return MLANG_acquire(lexer); }

public:
   MParser_Token_var nextToken();
public:
   void mACTION(bool _createToken);
/** stuff in between #(...) and #id items
 * Allow the escaping of the # for C preprocessor stuff.
 */
protected:
   void mSTUFF(bool _createToken);
protected:
   void mAST_ITEM(bool _createToken);
protected:
   void mTEXT_ITEM(bool _createToken);
protected:
   void mCOMMENT(bool _createToken);
protected:
   void mSTRING(bool _createToken);
protected:
   void mCHAR(bool _createToken);
protected:
   void mTREE(bool _createToken);
protected:
   void mWS(bool _createToken);
protected:
   void mID(bool _createToken);
protected:
   void mVAR_ASSIGN(bool _createToken);
protected:
   void mAST_CONSTRUCTOR(bool _createToken);
protected:
   void mTEXT_ARG(bool _createToken);
protected:
   void mTREE_ELEMENT(bool _createToken);
/** An ID_ELEMENT can be a func call, array ref, simple var,
 *  or AST label ref.
 */
protected:
   bool  mID_ELEMENT(bool _createToken);
/** The arguments of a #[...] constructor are text, token type,
 *  or a tree.
 */
protected:
   void mAST_CTOR_ELEMENT(bool _createToken);
protected:
   void mINT(bool _createToken);
protected:
   void mARG(bool _createToken);
protected:
   void mTEXT_ARG_ELEMENT(bool _createToken);
protected:
   void mTEXT_ARG_ID_ELEMENT(bool _createToken);
protected:
   void mINT_OR_FLOAT(bool _createToken);
protected:
   void mSL_COMMENT(bool _createToken);
protected:
   void mML_COMMENT(bool _createToken);
protected:
   void mESC(bool _createToken);
protected:
   void mDIGIT(bool _createToken);
private:
   static longArray mk_tokenSet_0();
public:
   static MParser_BitSet _tokenSet_0;
private:
   static longArray mk_tokenSet_1();
public:
   static MParser_BitSet _tokenSet_1;
private:
   static longArray mk_tokenSet_2();
public:
   static MParser_BitSet _tokenSet_2;
private:
   static longArray mk_tokenSet_3();
public:
   static MParser_BitSet _tokenSet_3;
private:
   static longArray mk_tokenSet_4();
public:
   static MParser_BitSet _tokenSet_4;
private:
   static longArray mk_tokenSet_5();
public:
   static MParser_BitSet _tokenSet_5;
private:
   static longArray mk_tokenSet_6();
public:
   static MParser_BitSet _tokenSet_6;
private:
   static longArray mk_tokenSet_7();
public:
   static MParser_BitSet _tokenSet_7;
private:
   static longArray mk_tokenSet_8();
public:
   static MParser_BitSet _tokenSet_8;
private:
   static longArray mk_tokenSet_9();
public:
   static MParser_BitSet _tokenSet_9;
private:
   static longArray mk_tokenSet_10();
public:
   static MParser_BitSet _tokenSet_10;
private:
   static longArray mk_tokenSet_11();
public:
   static MParser_BitSet _tokenSet_11;
private:
   static longArray mk_tokenSet_12();
public:
   static MParser_BitSet _tokenSet_12;
private:
   static longArray mk_tokenSet_13();
public:
   static MParser_BitSet _tokenSet_13;
private:
   static longArray mk_tokenSet_14();
public:
   static MParser_BitSet _tokenSet_14;
private:
   static longArray mk_tokenSet_15();
public:
   static MParser_BitSet _tokenSet_15;
private:
   static longArray mk_tokenSet_16();
public:
   static MParser_BitSet _tokenSet_16;
private:
   static longArray mk_tokenSet_17();
public:
   static MParser_BitSet _tokenSet_17;
private:
   static longArray mk_tokenSet_18();
public:
   static MParser_BitSet _tokenSet_18;
private:
   static longArray mk_tokenSet_19();
public:
   static MParser_BitSet _tokenSet_19;
private:
   static longArray mk_tokenSet_20();
public:
   static MParser_BitSet _tokenSet_20;
private:
   static longArray mk_tokenSet_21();
public:
   static MParser_BitSet _tokenSet_21;
private:
   static longArray mk_tokenSet_22();
public:
   static MParser_BitSet _tokenSet_22;
private:
   static longArray mk_tokenSet_23();
public:
   static MParser_BitSet _tokenSet_23;
private:
   static longArray mk_tokenSet_24();
public:
   static MParser_BitSet _tokenSet_24;
private:
   static longArray mk_tokenSet_25();
public:
   static MParser_BitSet _tokenSet_25;
private:
   static longArray mk_tokenSet_26();
public:
   static MParser_BitSet _tokenSet_26;
};

#endif /* __MGRAMMAR_CPPACTIONLEXER_H__ */
