#include "MLANG/Character.h"
#include "MLANG/Integer.h"
#include "MLANG/StringBuffer.h"
#include "MParser/StringUtils.h"
#include "MParser/System.h"
#include "MParser/CharStreamException.h"
#include "MParser/TokenStreamException.h"
#include "MParser/RecognitionException.h"
#include "MGrammar/ANTLRTokenTypes.h"
#include "MGrammar/GrammarSymbol.h"
#include "MGrammar/StringLiteralSymbol.h"
#include "MGrammar/MakeGrammar.h"
#include "MGrammar/CppActionLexer.h"
#include "MGrammar/CppCharFormatter.h"
#include "MGrammar/CppCodeGenerator.h"

#define INDENT_BLOCK 3

#define BEGIN_SWITCH_BLOCK(expr) \
   println(MLANG_String("switch(") + expr + ")"); \
   tabs_ += 1; \
   println("{");

#define END_SWITCH_BLOCK \
   println("}"); \
   tabs_ -= 1

#define BEGIN_CASE_BLOCK \
   tabs_ += 5; \
   println("{"); \
   tabs_ += 3;

#define END_CASE_BLOCK \
   tabs_ -= 3; \
   println ("}"); \
   tabs_ -= 5

#define BEGIN_DEFAULT_BLOCK \
   tabs_ += 5

#define END_DEFAULT_BLOCK \
   tabs_ -= 5

#define BEGIN_IF(c) \
   println(MLANG_String("if (") + c + ")"); \
   tabs_ += 1; \
   println("{"); \
   tabs_ += 3

#define BEGIN_ELSE_IF(c) \
   tabs_ -= 3; \
   println ("}"); \
   tabs_ -= 1; \
   println(MLANG_String("else if (") + c + ")"); \
   tabs_ += 1; \
   println("{"); \
   tabs_ += 3

#define END_IF \
   tabs_ -= 3; \
   println ("}"); \
   tabs_ -= 1;

#define BEGIN_FOR_BLOCK \
   tabs_ += 1; \
   println ("{"); \
   tabs_ += 3

#define END_FOR_BLOCK(comment) \
   tabs_ -= 3; \
   println (MLANG_String("}") + comment); \
   tabs_ -= 1;

#define BEGIN_ELSE_BLOCK \
   tabs_ -= 3; \
   println ("}"); \
   tabs_ -= 1; \
   println ("else"); \
   tabs_ += 1; \
   println ("{"); \
   tabs_ += 3;

#define END_ELSE_BLOCK \
   tabs_ -= 3; \
   println ("}"); \
   tabs_ -= 1;

#define BEGIN_BLOCK \
    println ("{"); \
    tabs_ += 3

#define END_BLOCK \
    tabs_ -= 3; \
    println ("}")

#define BEGIN_METHOD_BLOCK \
    println ("{"); \
    tabs_ += 3

#define END_METHOD_BLOCK \
    tabs_ -= 3; \
    println ("}");

#define BEGIN_TRY_BLOCK(comment) \
    println("try"); \
    tabs_ += 1; \
    println (MLANG_String("{") + comment); \
    tabs_ += 3;

#define END_TRY_BLOCK \
    tabs_ -= 3; \
    println ("}"); \
    tabs_ -= 1;

#define BEGIN_CATCH_BLOCK(exc) \
    tabs_ -= 3; \
    println ("}"); \
    tabs_ -= 1; \
    println("catch (" + exc + ")"); \
    tabs_ += 1; \
    println ("{"); \
    tabs_ += 3

#define BEGIN_METHOD_DECL(attribute) \
    tabs_ = 0; \
    print(attribute); \
    print(":"); \
    tabs_ += 3; \
    println("");

#define END_METHOD_DECL \
    tabs_ -= 3

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/CppCodeGenerator.java#2 $
 */

DEFINE_CLASS_INFO(MGrammar_CppCodeGenerator)

// C++ code generator by Pete Wells: pete@yamuna.demon.co.uk
// #line generation contributed by: Ric Klaren <klaren@cs.utwente.nl>

/** Generate MyParser.cpp, MyParser.h, MyLexer.cpp, MyLexer.h
 * and MyParserTokenTypes.h
 */
MLANG_String MGrammar_CppCodeGenerator::NONUNIQUE;

int
  MGrammar_CppCodeGenerator::caseSizeThreshold = 127;	// ascii is max

MLANG_String
  MGrammar_CppCodeGenerator::namespaceStd = "MLANG_USE_NAMESPACE(std)";
MLANG_String
//  MGrammar_CppCodeGenerator::namespaceAntlr = "MLANG_USE_NAMESPACE(MParser)";
  MGrammar_CppCodeGenerator::namespaceAntlr = "MParser_";

MGrammar_NameSpace_var
  MGrammar_CppCodeGenerator::nameSpace;

MLANG_String MGrammar_CppCodeGenerator::preIncludeCpp = "pre_include_cpp";
MLANG_String MGrammar_CppCodeGenerator::preIncludeHpp = "pre_include_hpp";
MLANG_String MGrammar_CppCodeGenerator::postIncludeCpp = "post_include_cpp";
MLANG_String MGrammar_CppCodeGenerator::postIncludeHpp = "post_include_hpp";

	/** Create a C++ code-generator using the given Grammar.
	 * The caller must still call setTool, setBehavior, and setAnalyzer
	 * before generating code.
	 */
MGrammar_CppCodeGenerator::MGrammar_CppCodeGenerator ()
  : super (),
    DEBUG_CPP_CODE_GENERATOR (false),
    syntacticPredLevel (0),
    genAST (false),
    saveText (false),
    genHashLines (true),
    noConstructors (false), usingCustomAST (false), astVarNumber (1)
{
  charFormatter_ = new MGrammar_CppCharFormatter ();
}

/** Adds a semantic predicate string to the sem pred vector
    These strings will be used to build an array of sem pred names
    when building a debugging parser.  This method should only be
    called when the debug option is specified
 */
int
MGrammar_CppCodeGenerator::addSemPred (const MLANG_String & predicate)
{
  semPreds.appendElement (predicate);
  return semPreds.size () - 1;
}

void
MGrammar_CppCodeGenerator::exitIfError ()
{
  if (antlrTool_->hasError ())
    {
      antlrTool_->fatalError ("Exiting due to errors.");
    }
}

int
MGrammar_CppCodeGenerator::countLines (const MLANG_String & s)
{
  int lines = 0;
  for (size_t i = 0; i < s.length (); i++)
    {
      if (s.charAt (i) == '\n')
	lines++;
    }
  return lines;
}

/** Output a MLANG_String to the currentOutput stream.
 * Ignored if string is null.
 * @param s The string to output
 */
void
MGrammar_CppCodeGenerator::_print (const MLANG_String & s)
{
#if 0
  if (s.length () != 0)
#endif
    {
      outputLine += countLines (s);
      currentOutput_->print (s);
    }
}

/** Print an action without leading tabs, attempting to
 * preserve the current indentation level for multi-line actions
 * Ignored if string is null.
 * @param s The action string to output
 */
void
MGrammar_CppCodeGenerator::_printAction (const MLANG_String & s)
{
#if 0
  if (s.length () != 0)
#endif
    {
      outputLine += countLines (s) + 1;
      super::_printAction (s);
    }
}

	/** Print an action stored in a token surrounded by #line stuff */
void
MGrammar_CppCodeGenerator::printAction (MParser_Token_ptr t)
{
  if (!MParser_Token::is_nil (t))
    {
      genLineNo (t->getLine ());
      printTabs ();
      super::_printAction (processActionForSpecialSymbols
		    (t->getText (), t->getLine (), 0, 0));
      genLineNo2 ();
    }
}

/** Print a header action by #line stuff also process any tree construction
 * @param name The name of the header part
 */
void
MGrammar_CppCodeGenerator::printHeaderAction (MLANG_String name)
{
  MParser_Token_var a =
    (MParser_Token_var) behavior_->headerActions_.get (name);
  if (!a.is_null ())
    {
      genLineNo (a->getLine ());
      println (processActionForSpecialSymbols (a->getText (), a->getLine (),
					       0, 0));
      genLineNo2 ();
    }
}

/** Output a MLANG_String followed by newline, to the currentOutput stream.
 * Ignored if string is null.
 * @param s The string to output
 */
void
MGrammar_CppCodeGenerator::_println (const MLANG_String & s)
{
#if 0
  if (s.length () != 0)
#endif
    {
      outputLine += countLines (s) + 1;
      currentOutput_->println (s);
    }
}

/** Output tab indent followed by a MLANG_String followed by newline,
 * to the currentOutput stream.  Ignored if string is null.
 * @param s The string to output
 */
void
MGrammar_CppCodeGenerator::println (const MLANG_String & s)
{
#if 0
  if (s.length () != 0)
#endif
    {
      printTabs ();
      outputLine += countLines (s) + 1;
      currentOutput_->println (s);
    }
}

	/** Generate a #line or // line depending on options */
void
MGrammar_CppCodeGenerator::genLineNo (int line)
{
  if (line == 0)
    {
      line++;
    }
  if (genHashLines)
    _println (MLANG_String ("#line ") + MLANG_String::valueOf (line) + " \"" +
	      antlrTool_->fileMinusPath (antlrTool_->grammarFile ()) + "\"");
}

/** Generate a #line or // line depending on options */
void
MGrammar_CppCodeGenerator::genLineNo (MGrammar_GrammarElement_ptr el)
{
  if (el != 0)
    genLineNo (el->getLine ());
}

/** Generate a #line or // line depending on options */
void
MGrammar_CppCodeGenerator::genLineNo (MParser_Token_ptr t)
{
  if (t != 0)
    genLineNo (t->getLine ());
}

/** Generate a #line or // line depending on options */
void
MGrammar_CppCodeGenerator::genLineNo2 ()
{
  if (genHashLines)
    {
      _println ("#line " + MLANG_String::valueOf (outputLine + 1) + " \"" +
		outputFile + "\"");
    }
}

	/// Bound safe isDigit
bool MGrammar_CppCodeGenerator::charIsDigit (const MLANG_String & s, int i)
{
  return (i < (int) s.length ()) && MLANG_Character::isDigit (s.charAt (i));
}

/** Normalize a string coming from antlr's lexer. E.g. translate java
 * escapes to values. Check their size (multibyte) bomb out if they are
 * multibyte (bit crude). Then reescape to C++ style things.
 * Used to generate strings for match() and matchRange()
 * @param lit the literal string
 * @param isCharLiteral if it's for a character literal
 *                       (enforced to be one length) and enclosed in '
 * FIXME: bombing out on mb chars. Should be done in Lexer.
 * FIXME: this is another horrible hack.
 * FIXME: life would be soooo much easier if the stuff from the lexer was
 * normalized in some way.
 */
MLANG_String
  MGrammar_CppCodeGenerator::convertJavaToCppString (const MLANG_String & lit,
						     bool isCharLiteral)
{
  // SYSTEM->out()->println("convertJavaToCppLiteral: "+lit);
  MLANG_String ret;
  MLANG_String s = lit;

  int i = 0;
  int val = 0;

  if (isCharLiteral)		// verify & strip off quotes
    {
      if (!lit.startsWith ("'") || !lit.endsWith ("'"))
	antlrTool_->error ("Invalid character literal: '" + lit + "'");

      s = lit.substring (1, lit.length () - 1);
    }

  while (i < (int) s.length ())
    {
      if (s.charAt (i) == '\\')
	{
	  if ((int) s.length () == i + 1)
	    antlrTool_->error ("Invalid escape in char literal: '" + lit +
			       "' looking at '" + s.substring (i) + "'");

	  // deal with escaped junk
	  switch (s.charAt (i + 1))
	    {
	    case 'a':
	      val = 7;
	      i += 2;
	      break;
	    case 'b':
	      val = 8;
	      i += 2;
	      break;
	    case 't':
	      val = 9;
	      i += 2;
	      break;
	    case 'n':
	      val = 10;
	      i += 2;
	      break;
	    case 'f':
	      val = 12;
	      i += 2;
	      break;
	    case 'r':
	      val = 13;
	      i += 2;
	      break;
	    case '"':
	    case '\'':
	    case '\\':
	      val = s.charAt (i + 1);
	      i += 2;
	      break;

	    case 'u':
	      // Unicode char \u1234
	      if (i + 5 < (int) s.length ())
		{
		  val =
		    MLANG_Character::digit (s.charAt (i + 2),
					    16) * 16 * 16 * 16 +
		    MLANG_Character::digit (s.charAt (i + 3),
					    16) * 16 * 16 +
		    MLANG_Character::digit (s.charAt (i + 4),
					    16) * 16 +
		    MLANG_Character::digit (s.charAt (i + 5), 16);
		  i += 6;
		}
	      else
		antlrTool_->error ("Invalid escape in char literal: '" + lit +
				   "' looking at '" + s.substring (i) + "'");
	      break;

	    case '0':		// \123
	    case '1':
	    case '2':
	    case '3':
	      if (charIsDigit (s, i + 2))
		{
		  if (charIsDigit (s, i + 3))
		    {
		      val =
			(s.charAt (i + 1) - '0') * 8 * 8 + (s.charAt (i + 2) -
							    '0') * 8 +
			(s.charAt (i + 3) - '0');
		      i += 4;
		    }
		  else
		    {
		      val =
			(s.charAt (i + 1) - '0') * 8 + (s.charAt (i + 2) -
							'0');
		      i += 3;
		    }
		}
	      else
		{
		  val = s.charAt (i + 1) - '0';
		  i += 2;
		}
	      break;

	    case '4':
	    case '5':
	    case '6':
	    case '7':
	      if (charIsDigit (s, i + 2))
		{
		  val =
		    (s.charAt (i + 1) - '0') * 8 + (s.charAt (i + 2) - '0');
		  i += 3;
		}
	      else
		{
		  val = s.charAt (i + 1) - '0';
		  i += 2;
		}
	    default:
	      antlrTool_->error ("Unhandled escape in char literal: '" + lit +
				 "' looking at '" + s.substring (i) + "'");
	      val = 0;
	    }
	}
      else
	val = s.charAt (i++);

      if (isCharLiteral)
	{
	  // we should be at end of char literal here..
	  if (i != (int) s.length ())
	    antlrTool_->error ("Invalid char literal: '" + lit + "'");

//                              if( val >= ' ' && val <= 126 )  // just concat if printable
//                                      ret = "'"+(char)val+"'";
//                              else
	  if (val > 255)	// abort if multibyte
	    antlrTool_->
	      error ("Multibyte character found in char literal: '" + lit +
		     "'");
	  else if (val == 255)
	    // the joys of sign extension in the support lib *cough*
	    ret = "static_cast<unsigned char>(255)";
	  else
	    ret = "'" + charFormatter_->escapeChar (val, true) + "'";
	}
      else
	{
	  if (val >= ' ' && val <= 126)	// just concat if printable
	    ret += (char) val;
	  else if (val > 255)	// abort if multibyte
	    antlrTool_->
	      error ("Multibyte character found in string constant: '" + s +
		     "'");
	  else
	    ret += charFormatter_->escapeChar (val, true);
	}
    }
  // SYSTEM->out()->println("convertJavaToCppLiteral: "+lit+" -> "+ret);
  return ret;
}

/** Generate the parser, lexer, treeparser, and token types in C++
 */
void
MGrammar_CppCodeGenerator::gen ()
{
  // Do the code generation
  try
  {
    // Loop over all grammars
    MGrammar_DefineGrammarSymbols::GRAMMARS::const_iterator grammarIter =
      behavior_->grammars ().begin ();
    while (grammarIter != behavior_->grammars ().end ())
      {
	MGrammar_Grammar_var g = grammarIter->second;
	if (g->debuggingOutput ())
	  {
	    antlrTool_->error (g->getFilename () +
			       ": C++ mode does not support -debug");
	  }
	// Connect all the components to each other
	g->setGrammarAnalyzer (analyzer_);
	g->setCodeGenerator (this);
	analyzer_->setGrammar (g);
	// To get right overloading behavior across hetrogeneous grammars
	setupGrammarParameters (g);
	g->generate ();
	exitIfError ();
        grammarIter++;
      }

    // Loop over all token managers (some of which are lexers)
    MGrammar_DefineGrammarSymbols::TOKENMANAGERS::const_iterator tmIter =
      behavior_->tokenManagers ().begin ();
    while (tmIter != behavior_->tokenManagers ().end ())
      {
	MGrammar_TokenManager_var tm = tmIter->second;
	if (!tm->isReadOnly ())
	  {
	    // Write the token manager tokens as C++
	    // this must appear before genTokenInterchange so that
	    // labels are set on string literals
	    genTokenTypes (tm);
	    // Write the token manager tokens as plain text
	    genTokenInterchange (tm);
	  }
	exitIfError ();
        tmIter++;
      }
  }
  catch (MIO_IOException & e)
  {
    antlrTool_->reportException (e, "");
  }
}

/** Generate code for the given grammar element.
 * @param blk The {...} action to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_ActionElement_ptr action)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genAction(" + action->toString () + ")");
  if (action->isSemPred ())
    {
      genSemPred (action->actionText (), action->getLine ());
    }
  else
    {
      if (grammar_->hasSyntacticPredicate ())
	{
	  BEGIN_IF("inputState_->guessing_==0");
	}

      MGrammar_ActionTransInfo_var tInfo = new MGrammar_ActionTransInfo ();
      MLANG_String actionStr =
	processActionForSpecialSymbols (action->actionText (),
					action->getLine (),
					currentRule, tInfo);

      if (tInfo->refRuleRoot_.length () != 0)
	{
	  // Somebody referenced "#rule", make sure translated var is valid
	  // assignment to #rule is left as a ref also, meaning that assignments
	  // with no other refs like "#rule = foo();" still forces this code to be
	  // generated (unnecessarily).
	  println (tInfo->refRuleRoot_ + " = " + labeledElementASTType +
		   "(currentAST->root);");
	}

      // dump the translated action
      genLineNo (action);
      super::printAction (actionStr);
      genLineNo2 ();

      if (tInfo->assignToRoot_)
	{
	  // Somebody did a "#rule=", reset internal currentAST->root
	  println ("currentAST->root = " + tInfo->refRuleRoot_ + ";");
	  // reset the child pointer too to be last sibling in sibling list
	  // now use if else in stead of x ? y : z to shut CC 4.2 up.
	  BEGIN_IF(tInfo->refRuleRoot_ + "!=" + labeledElementASTInit + " &&" + tInfo->refRuleRoot_ + "->getFirstChild() != " + labeledElementASTInit);
	  println ("  currentAST.child = " + tInfo->refRuleRoot_ +
		   "->getFirstChild();");
	  BEGIN_ELSE_BLOCK;
	  println ("currentAST.child = " + tInfo->refRuleRoot_ + ";");
	  END_IF;
	  println ("currentAST.advanceChildToEnd();");
	}

      if (grammar_->hasSyntacticPredicate ())
	{
          END_IF;
	}
    }
}

/** Generate code for the given grammar element.
 * @param blk The "x|y|z|..." block to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_AlternativeBlock_ptr blk)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("gen(" + blk->toString () + ")");
  BEGIN_BLOCK;
  genBlockPreamble (blk);
  genBlockInitAction (blk);

  // Tell AST generation to build subrule result
  MLANG_String saveCurrentASTResult = currentASTResult;
  if (blk->getLabel ().length () != 0)
    {
      currentASTResult = blk->getLabel ();
    }

//  bool ok = grammar_->theLLkAnalyzer_->deterministic (blk);
  grammar_->theLLkAnalyzer_->deterministic (blk);

  MGrammar_CppBlockFinishingInfo howToFinish = genCommonBlock (blk, true);
  beginBlockFinish (howToFinish);
  if (howToFinish.viableAction())
   {
     println (throwNoViable);
   }
  endBlockFinish (howToFinish);

  END_BLOCK;

  // Restore previous AST generation
  currentASTResult = saveCurrentASTResult;
}

/** Generate code for the given grammar element.
 * @param blk The block-end element to generate.  Block-end
 * elements are synthesized by the grammar parser to represent
 * the end of a block.
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_BlockEndElement_ptr end)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genRuleEnd(" + end->toString () + ")");
}

/** Generate code for the given grammar element.
 * Only called from lexer grammars.
 * @param blk The character literal reference to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_CharLiteralElement_ptr atom)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genChar(" + atom->toString () + ")");

  if (!grammar_->isLexer ())
    antlrTool_->error ("cannot ref character literals in grammar: " +
		       atom->toString ());

  if (atom->getLabel ().length () != 0)
    {
      println (atom->getLabel () + " = " + lt1Value + ";");
    }

  bool oldsaveText = saveText;
  saveText = saveText
    && atom->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_NONE;

  // if in lexer and ! on element, save buffer index to kill later
  if (!saveText
      || atom->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG)
    println ("_saveIndex = text_.length();");

  print (atom->getNo ()? "matchNot(" : "match(");
  _print (convertJavaToCppString (atom->getText (), true));
  _println (");");

  if (!saveText
      || atom->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG)
    println ("text_.erase(_saveIndex);");	// kill text atom put in buffer

  saveText = oldsaveText;
}

/** Generate code for the given grammar element.
 * Only called from lexer grammars.
 * @param blk The character-range reference to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_CharRangeElement_ptr r)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genCharRangeElement(" + r->beginText () + ".." +
			     r->endText () + ")");

  if (!grammar_->isLexer ())
    antlrTool_->error ("cannot ref character range in grammar: " +
		       r->toString ());

  if (r->getLabel ().length () != 0 && syntacticPredLevel == 0)
    {
      println (r->getLabel () + " = " + lt1Value + ";");
    }
  // Correctly take care of saveIndex stuff...
  bool save = (grammar_->isLexer () &&
	       (!saveText ||
		r->getAutoGenType () ==
		MGrammar_GrammarElement::AUTO_GEN_BANG));
  if (save)
    println ("_saveIndex=text_.length();");

  println ("matchRange(" + convertJavaToCppString (r->beginText (), true) +
	   "," + convertJavaToCppString (r->endText (), true) + ");");

  if (save)
    println ("text_.setLength(_saveIndex);");
}

/** Generate the lexer C++ files */
void
MGrammar_CppCodeGenerator::gen (MGrammar_LexerGrammar_ptr g)
{
  // If debugging, create a new sempred vector for this grammar
  if (g->debuggingOutput ())
    semPreds = SEMPREDS ();

  if (g->charVocabulary ().size () > 256)
    antlrTool_->warning (g->getFilename () +
			 ": C++ mode does not support more than 8 bit characters (vocabulary size now: "
			 + g->charVocabulary ().size () + ")");

  setGrammar (g);
  if (!(grammar_->isLexer ()))
    {
      antlrTool_->panic ("Internal error generating lexer");
    }

  genBody (g);
  genInclude (g);
}

/** Generate code for the given grammar element.
 * @param blk The (...)+ block to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_OneOrMoreBlock_ptr blk)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("gen+(" + blk->toString () + ")");
  MLANG_String label;
  MLANG_String cnt;
#if 0
  println ("{ // ( ... )+");
#endif
  genBlockPreamble (blk);
  if (blk->getLabel ().length () != 0)
    {
      cnt = "_cnt_" + blk->getLabel ();
    }
  else
    {
      cnt = "_cnt" + MLANG_String::valueOf (blk->ID ());
    }
  println ("int " + cnt + "=0;");
  if (blk->getLabel ().length () != 0)
    {
      label = blk->getLabel ();
    }
  else
    {
      label = "_loop" + MLANG_String::valueOf (blk->ID ());
    }

  println ("for (;;)");
  BEGIN_FOR_BLOCK;
  // generate the init action for ()+ ()* inside the loop
  // this allows us to do usefull EOF checking...
  genBlockInitAction (blk);

  // Tell AST generation to build subrule result
  MLANG_String saveCurrentASTResult = currentASTResult;
  if (blk->getLabel ().length () != 0)
    {
      currentASTResult = blk->getLabel ();
    }

  //bool ok = grammar_->theLLkAnalyzer_->deterministic (blk);
   grammar_->theLLkAnalyzer_->deterministic (blk);

  // generate exit test if greedy set to false
  // and an alt is ambiguous with exit branch
  // or when lookahead derived purely from end-of-file
  // Lookahead analysis stops when end-of-file is hit,
  // returning set {epsilon}.  Since {epsilon} is not
  // ambig with any real tokens, no error is reported
  // by deterministic() routines and we have to check
  // for the case where the lookahead depth didn't get
  // set to NONDETERMINISTIC (this only happens when the
  // FOLLOW contains real atoms + epsilon).
  bool generateNonGreedyExitPath = false;
  int nonGreedyExitDepth = grammar_->maxk ();

  if (!blk->greedy () &&
      blk->exitLookaheadDepth () <= grammar_->maxk () &&
      blk->exitCache ()[blk->exitLookaheadDepth ()].containsEpsilon ())
    {
      generateNonGreedyExitPath = true;
      nonGreedyExitDepth = blk->exitLookaheadDepth ();
    }
  else if (!blk->greedy () &&
	   blk->exitLookaheadDepth () ==
	   MGrammar_LLkGrammarAnalyzer::NONDETERMINISTIC)
    {
      generateNonGreedyExitPath = true;
    }

  // generate exit test if greedy set to false
  // and an alt is ambiguous with exit branch
  if (generateNonGreedyExitPath)
    {
      if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
	{
	  SYSTEM->out ()->println ("nongreedy (...)+ loop; exit depth is " +
				   MLANG_String::valueOf (blk->
							  exitLookaheadDepth
							  ()));
	}
      MLANG_String predictExit =
	getLookaheadTestExpression (blk->exitCache (),
				    nonGreedyExitDepth);
      println ("// nongreedy exit test");
      println ("if ( " + cnt + ">=1 && " + predictExit + ") goto " + label +
	       ";");
    }

  MGrammar_CppBlockFinishingInfo howToFinish = genCommonBlock (blk, false);
  beginBlockFinish (howToFinish);
  if (howToFinish.viableAction())
   {
     BEGIN_IF(cnt + ">=1");
       println ("goto " + label + ";");
     BEGIN_ELSE_BLOCK;
       println(throwNoViable);
     END_IF;
   }
  endBlockFinish (howToFinish);

  println (cnt + "++;");
  END_FOR_BLOCK (" // ( ... )+");
  println (label + ":;");

  // Restore previous AST generation
  currentASTResult = saveCurrentASTResult;
}

	/** Generate the parser C++ file */
void
MGrammar_CppCodeGenerator::gen (MGrammar_ParserGrammar_ptr g)
{

  // if debugging, set up a new vector to keep track of sempred
  //   strings for this grammar
  if (g->debuggingOutput ())
    semPreds = SEMPREDS ();
  setGrammar (g);
  if (!(grammar_->isParser ()))
    {
      antlrTool_->panic ("Internal error generating parser");
    }

  genBody (g);
  genInclude (g);
}

/** Generate code for the given grammar element.
 * @param blk The rule-reference to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_RuleRefElement_ptr rr)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genRR(" + rr->toString () + ")");
  MGrammar_RuleSymbol_var rs =
    Dynamic_Cast<MGrammar_RuleSymbol>(grammar_->getSymbol (rr->targetRule ()));
  if (rs.is_null () || !rs->isDefined ())
    {
      // Is this redundant???
      antlrTool_->error ("Rule '" + rr->targetRule () + "' is not defined",
			 grammar_->getFilename (), rr->getLine (),
			 rr->getColumn ());
      return;
    }
  if (!(instanceof<MGrammar_RuleSymbol>(rs)))
    {
      // Is this redundant???
      antlrTool_->error ("'" + rr->targetRule () +
			 "' does not name a grammar rule",
			 grammar_->getFilename (), rr->getLine (),
			 rr->getColumn ());
      return;
    }

  genErrorTryForElement (rr);

  // AST value for labeled rule refs in tree walker.
  // This is not AST construction;  it is just the input tree node value.
  if ((grammar_->isTreeWalker ()) &&
      rr->getLabel ().length () != 0 && syntacticPredLevel == 0)
    {
      println (rr->getLabel () + " = (_t == ASTNULL) ? " +
	       labeledElementASTInit + " : " + lt1Value + ";");
    }

  // if in lexer and ! on rule ref or alt or rule, save buffer index to
  // kill later
  if ((grammar_->isLexer ())
      && (!saveText
	  || rr->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG))
    {
      println ("_saveIndex = text_.length();");
    }

  // Process return value assignment if any
  printTabs ();
  if (rr->getIdAssign ().length () != 0)
    {
      // Warn if the rule has no return type
      if (rs->getBlock()->returnAction().length() == 0)
	{
	  antlrTool_->warning ("Rule '" + rr->targetRule () +
			       "' has no return type",
			       grammar_->getFilename (), rr->getLine (),
			       rr->getColumn ());
	}
      _print (rr->getIdAssign() + "=");
    }
  else
    {
      // Warn about return value if any, but not inside syntactic predicate
      if (!((grammar_->isLexer())) && syntacticPredLevel == 0
	  && rs->getBlock()->returnAction().length() != 0)
	{
	  antlrTool_->warning ("Rule '" + rr->targetRule() +
			       "' returns a value", grammar_->getFilename (),
			       rr->getLine (), rr->getColumn ());
	}
    }

  // Call the rule
  GenRuleInvocation (rr);

  // if in lexer and ! on element or alt or rule, save buffer index to kill later
  if ((grammar_->isLexer())
      && (!saveText || rr->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG))
    {
      println ("text_.erase(_saveIndex);");
    }

  // if not in a syntactic predicate
  if (syntacticPredLevel == 0)
    {
      bool doNoGuessTest = (grammar_->hasSyntacticPredicate() &&
			    (grammar_->buildAST() && rr->getLabel ().length() != 0 ||
			     (genAST
			      && rr->getAutoGenType () ==
			      MGrammar_GrammarElement::AUTO_GEN_NONE)));

      if (doNoGuessTest)
	{
	  BEGIN_IF("inputState_->guessing_==0");
	}

      if (grammar_->buildAST() && rr->getLabel ().length () != 0)
	{
	  // always gen variable for rule return on labeled rules
	  // RK: hmm do I know here if the returnAST needs a cast ?
	  println (rr->getLabel () + "_AST = returnAST;");
	}

      if (genAST)
	{
	  switch (rr->getAutoGenType ())
	    {
	    case MGrammar_GrammarElement::AUTO_GEN_NONE:
	      if (usingCustomAST)
		println ("astFactory->addASTChild(currentAST, " +
			 namespaceAntlr + "AST_var(returnAST));");
	      else
		println ("astFactory->addASTChild( currentAST, returnAST );");
	      break;
	    case MGrammar_GrammarElement::AUTO_GEN_CARET:
	      // FIXME: RK: I'm not so sure this should be an error..
	      // I think it might actually work and be usefull at times.
	      antlrTool_->
		error ("Internal: encountered ^ after rule reference");
	      break;
	    default:
	      break;
	    }
	}

      // if a lexer and labeled, Token label defined at rule level, just set it here
      if ((grammar_->isLexer()) && rr->getLabel().length() != 0)
	{
	  println (rr->getLabel () + "=returnToken_;");
	}

      if (doNoGuessTest)
	{
          END_IF;
	}
    }
  genErrorCatchForElement (rr);
}

/** Generate code for the given grammar element.
 * @param blk The string-literal reference to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_StringLiteralElement_ptr atom)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genString(" + atom->toString () + ")");

  // Variable declarations for labeled elements
  if (atom->getLabel ().length () != 0 && syntacticPredLevel == 0)
    {
      println (atom->getLabel () + " = " + lt1Value + ";");
    }

  // AST
  genElementAST (atom);

  // is there a bang on the literal?
  bool oldsaveText = saveText;
  saveText = saveText
    && atom->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_NONE;

  // matching
  genMatch (atom);

  saveText = oldsaveText;

  // tack on tree cursor motion if doing a tree walker
  if (grammar_->isTreeWalker ())
    {
      println ("_t = _t->getNextSibling();");
    }
}

	/** Generate code for the given grammar element.
	 * @param blk The token-range reference to generate
	 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_TokenRangeElement_ptr r)
{
  genErrorTryForElement (r);
  if (r->getLabel().length() != 0 && syntacticPredLevel == 0)
    {
      println (r->getLabel () + " = " + lt1Value + ";");
    }

  // AST
  genElementAST (r);

  // match
  println ("matchRange(" + r->beginText () + "," + r->endText () + ");");
  genErrorCatchForElement (r);
}

/** Generate code for the given grammar element.
 * @param blk The token-reference to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_TokenRefElement_ptr atom)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genTokenRef(" + atom->toString() + ")");
  if (grammar_->isLexer ())
    {
      antlrTool_->panic ("Token reference found in lexer");
    }
  genErrorTryForElement (atom);
  // Assign Token value to token label variable
  if (atom->getLabel().length() != 0 && syntacticPredLevel == 0)
    {
      println (atom->getLabel () + " = " + lt1Value + ";");
    }

  // AST
  genElementAST (atom);
  // matching
  genMatch (atom);
  genErrorCatchForElement (atom);

  // tack on tree cursor motion if doing a tree walker
  if ((grammar_->isTreeWalker()))
    {
      println ("_t = _t->getNextSibling();");
    }
}
void
MGrammar_CppCodeGenerator::gen (MGrammar_TreeElement_ptr t)
{
  // save AST cursor
  println (labeledElementType + " __t" + MLANG_String::valueOf (t->ID ()) +
	   " = _t;");

  // If there is a label on the root, then assign that to the variable
  if (t->root()->getLabel().length() != 0)
    {
      println (t->root()->getLabel () + " = (_t == ASTNULL) ? " +
	       labeledElementASTInit + " : _t;");
    }

  // check for invalid modifiers ! and ^ on tree element roots
  if (t->root()->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG)
    {
      antlrTool_->error ("Suffixing a root node with '!' is not implemented",
			 grammar_->getFilename (), t->getLine (),
			 t->getColumn ());
      t->root()->setAutoGenType (MGrammar_GrammarElement::AUTO_GEN_NONE);
    }
  if (t->root()->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_CARET)
    {
      antlrTool_->
	warning
	("Suffixing a root node with '^' is redundant; already a root",
	 grammar_->getFilename (), t->getLine (), t->getColumn ());
      t->root()->setAutoGenType (MGrammar_GrammarElement::AUTO_GEN_NONE);
    }

  // Generate AST variables
  genElementAST (t->root());
  if (grammar_->buildAST())
    {
      // Save the AST construction state
      println (namespaceAntlr + "ASTPair __currentAST" + MLANG_String::valueOf(t->ID()) +
	       " = currentAST;");
      // Make the next item added a child of the TreeElement root
      println ("currentAST->root = currentAST.child;");
      println ("currentAST.child = " + labeledElementASTInit + ";");
    }

  // match root
  if (instanceof<MGrammar_WildcardElement>(t->root()))
    {
      println ("if ( _t == ASTNULL ) throw " + namespaceAntlr +
	       "MismatchedTokenException();");
    }
  else
    {
      genMatch (t->root());
    }
  // move to list of children
  println ("_t = _t->getFirstChild();");

  // walk list of children, generating code for each
  for (size_t i = 0; i < t->getAlternatives ().size (); i++)
    {
      MGrammar_Alternative_var a = t->getAlternativeAt (i);
      MGrammar_AlternativeElement_var e = a->head_;
      while (e)
	{
	  e->generate ();
	  e = e->next_;
	}
    }

  if (grammar_->buildAST())
    {
      // restore the AST construction state to that just after the
      // tree root was added
      println ("currentAST = __currentAST" + MLANG_String::valueOf(t->ID()) + ";");
    }
  // restore AST cursor
  println ("_t = __t" + MLANG_String::valueOf (t->ID ()) + ";");
  // move cursor to sibling of tree just parsed
  println ("_t = _t->getNextSibling();");
}

	/** Generate the tree-parser C++ files */
void
MGrammar_CppCodeGenerator::gen (MGrammar_TreeWalkerGrammar_ptr g)
{
  setGrammar (g);
  if (!(grammar_->isTreeWalker ()))
    {
      antlrTool_->panic ("Internal error generating tree-walker");
    }

  genBody (g);
  genInclude (g);
}

/** Generate code for the given grammar element.
 * @param wc The wildcard element to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_WildcardElement_ptr wc)
{
  // Variable assignment for labeled elements
  if (wc->getLabel().length() != 0 && syntacticPredLevel == 0)
    {
      println (wc->getLabel () + " = " + lt1Value + ";");
    }

  // AST
  genElementAST (wc);
  // Match anything but EOF
  if ((grammar_->isTreeWalker()))
    {
      println ("if ( _t == " + labeledElementASTInit + " ) throw " +
	       namespaceAntlr + "MismatchedTokenException();");
    }
  else if ((grammar_->isLexer()))
    {
      if ((grammar_->isLexer()) &&
	  (!saveText || wc->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG))
	{
	  println ("_saveIndex = text_.length();");
	}
      println ("matchNot(EOF/*_CHAR*/);");
      if ((grammar_->isLexer()) &&
	  (!saveText || wc->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG))
	{
	  println ("text_.erase(_saveIndex);");	// kill text atom put in buffer
	}
    }
  else
    {
      println ("matchNot(" + getValueString (MParser_Token::EOF_TYPE) +
	       ");");
    }

  // tack on tree cursor motion if doing a tree walker
  if ((grammar_->isTreeWalker()))
    {
      println ("_t = _t->getNextSibling();");
    }
}

/** Generate code for the given grammar element.
 * @param blk The (...)* block to generate
 */
void
MGrammar_CppCodeGenerator::gen (MGrammar_ZeroOrMoreBlock_ptr blk)

{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("gen*(" + blk->toString() + ")");
#if 0
  println ("{ // ( ... )*");
#endif
  genBlockPreamble (blk);
  MLANG_String label;
  if (blk->getLabel().length() != 0)
    {
      label = blk->getLabel ();
    }
  else
    {
      label = "_loop" + MLANG_String::valueOf(blk->ID());
    }
  println ("for (;;)");
  BEGIN_FOR_BLOCK;
  // generate the init action for ()+ ()* inside the loop
  // this allows us to do usefull EOF checking...
  genBlockInitAction (blk);

  // Tell AST generation to build subrule result
  MLANG_String saveCurrentASTResult = currentASTResult;
  if (blk->getLabel().length() != 0)
    {
      currentASTResult = blk->getLabel ();
    }

//  bool ok = grammar_->theLLkAnalyzer_->deterministic (blk);
  grammar_->theLLkAnalyzer_->deterministic (blk);

  // generate exit test if greedy set to false
  // and an alt is ambiguous with exit branch
  // or when lookahead derived purely from end-of-file
  // Lookahead analysis stops when end-of-file is hit,
  // returning set {epsilon}.  Since {epsilon} is not
  // ambig with any real tokens, no error is reported
  // by deterministic() routines and we have to check
  // for the case where the lookahead depth didn't get
  // set to NONDETERMINISTIC (this only happens when the
  // FOLLOW contains real atoms + epsilon).
  bool generateNonGreedyExitPath = false;
  int nonGreedyExitDepth = grammar_->maxk();

  if (!blk->greedy() &&
      blk->exitLookaheadDepth() <= grammar_->maxk() &&
      blk->exitCache()[blk->exitLookaheadDepth()].containsEpsilon ())
    {
      generateNonGreedyExitPath = true;
      nonGreedyExitDepth = blk->exitLookaheadDepth();
    }
  else if (!blk->greedy() &&
	   blk->exitLookaheadDepth() == MGrammar_LLkGrammarAnalyzer::NONDETERMINISTIC)
    {
      generateNonGreedyExitPath = true;
    }
  if (generateNonGreedyExitPath)
    {
      if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
	{
	  SYSTEM->out ()->println ("nongreedy (...)* loop; exit depth is " +
				   blk->exitLookaheadDepth());
	}
      MLANG_String predictExit = getLookaheadTestExpression (blk->exitCache(),
							     nonGreedyExitDepth);
      println ("// nongreedy exit test");
      println ("if (" + predictExit + ") goto " + label + ";");
    }

  MGrammar_CppBlockFinishingInfo howToFinish = genCommonBlock (blk, false);
  beginBlockFinish (howToFinish);
  if (howToFinish.viableAction())
   {
     println ("goto " + label + ";");
   }
  endBlockFinish (howToFinish);

  END_FOR_BLOCK(" // ( ... )*");
  println (label + ":;");

  // Restore previous AST generation
  currentASTResult = saveCurrentASTResult;
}

/** Generate an alternative.
 * @param alt  The alternative to generate
 * @param blk The block to which the alternative belongs
 */
void
MGrammar_CppCodeGenerator::genAlt (
   MGrammar_Alternative_ptr alt,
   MGrammar_AlternativeBlock_ptr blk
)

{
  // Save the AST generation state, and set it to that of the alt
  bool savegenAST = genAST;
  genAST = genAST && alt->getAutoGen ();

  bool oldsaveTest = saveText;
  saveText = saveText && alt->getAutoGen ();

  // Reset the variable name map for the alternative
  TREEVARIABLEMAP saveMap = treeVariableMap;
  treeVariableMap = TREEVARIABLEMAP ();

  // Generate try block around the alt for  error handling
  if (alt->exceptionSpec_)
    {
      BEGIN_TRY_BLOCK("// for error handling");
    }

  MGrammar_AlternativeElement_var elem = alt->head_;
  while (!(instanceof<MGrammar_BlockEndElement>(elem)))
    {
      elem->generate ();		// alt can begin with anything. Ask target to gen.
      elem = elem->next_;
    }

  if (genAST)
    {
      if (instanceof<MGrammar_RuleBlock>(blk))
	{
	  // Set the AST return value for the rule
	  MGrammar_RuleBlock_var rblk = Dynamic_Cast<MGrammar_RuleBlock>(blk);
	  if (usingCustomAST)
	    println (rblk->getRuleName () + "_AST = " + labeledElementASTType +
		     "(currentAST->root);");
	  else
	    println (rblk->getRuleName () + "_AST = currentAST->root;");
	}
      else if (blk->getLabel().length() != 0)
	{
	  // ### future: also set AST value for labeled subrules.
	  // println(blk->getLabel() + "_AST = "+labeledElementASTType+"(currentAST->root);");
	  antlrTool_->warning ("Labeled subrules are not implemented",
			       grammar_->getFilename (), blk->getLine (),
			       blk->getColumn ());
	}
    }

  if (alt->exceptionSpec_)
    {
      genErrorHandler (alt->exceptionSpec_);
      // close try block
      END_TRY_BLOCK;
    }

  genAST = savegenAST;
  saveText = oldsaveTest;

  treeVariableMap = saveMap;
}

/** Generate all the bitsets to be used in the parser or lexer
 * Generate the raw bitset data like "long _tokenSet1_data[] = {...};"
 * and the BitSet object declarations like
 * "BitSet _tokenSet1 = new BitSet(_tokenSet1_data);"
 * Note that most languages do not support object initialization inside a
 * class definition, so other code-generators may have to separate the
 * bitset declarations from the initializations (e.g., put the
 * initializations in the generated constructor instead).
 * @param bitsetList The list of bitsets to generate.
 * @param maxVocabulary Ensure that each generated bitset can contain at
 *        least this value.
 * @param prefix string glued in from of bitset names used for namespace
 *        qualifications.
 */
void
MGrammar_CppCodeGenerator::genBitsets (
   const MLANG_Vector<MParser_BitSet>& bitsetList,
   int maxVocabulary, MLANG_String prefix
)

{
  MGrammar_TokenManager_var tm = grammar_->tokenManager_;

  println ("");

  for (size_t i = 0; i < bitsetList.size (); i++)
    {
      MParser_BitSet p = bitsetList[i];
      // Ensure that generated BitSet is large enough for vocabulary
      p.growToInclude (maxVocabulary);

      // initialization data
      println ("const ACE_UINT64 " + prefix + getBitsetName (i) +
	       "_data_" + "[] = { " + p.toStringOfWords () + " };");

      // Dump the contents of the bitset in readable format...
      MLANG_String t = "// ";

      for (size_t j = 0; j < tm->getVocabulary ().size (); j++)
	{
	  if (p.member (j))
	    {
	      if (((grammar_->isLexer())))
		{
		  // only dump out for pure printable ascii.
		  if ((0x20 <= j) && (j < 0x7F))
		    t += charFormatter_->escapeChar (j, true) + " ";
		  else
		    t += "0x" + MLANG_Integer::toString (j, 16) + " ";
		}
	      else
		t += tm->getTokenStringAt (j) + " ";

	      if (t.length () > 70)
		{
		  println (t);
		  t = "// ";
		}
	    }
	}
      if (t != "// ")
	println (t);

      // BitSet object
      println ("const " + namespaceAntlr + "BitSet " + prefix +
	       getBitsetName (i) + "(longArray(" + getBitsetName (i) + "_data_," +
	       "sizeof(" + getBitsetName(i) + "_data_" + ")" + "/" +
	       "sizeof(" + getBitsetName(i) + "_data_[0]" + ")" +
               "));");
    }
}
void
MGrammar_CppCodeGenerator::genBitsetsHeader (
   const MLANG_Vector<MParser_BitSet>& bitsetList,
   int maxVocabulary
)

{
  println ("");
  for (size_t i = 0; i < bitsetList.size (); i++)
    {
      MParser_BitSet p = bitsetList[i];
      // Ensure that generated BitSet is large enough for vocabulary
      p.growToInclude (maxVocabulary);
      // initialization data
      println ("static const ACE_UINT64 " + getBitsetName (i) + "_data_" +
	       "[];");
      // BitSet object
      println ("static const " + namespaceAntlr + "BitSet " +
	       getBitsetName (i) + ";");
    }
}

/** Generate the finish of a block, using a combination of the info
 * returned from genCommonBlock() and the action to perform when
 * no alts were taken
 * @param howToFinish The return of genCommonBlock()
 * @param noViableAction What to generate when no alt is taken
*/
void
MGrammar_CppCodeGenerator::beginBlockFinish (
   const MGrammar_CppBlockFinishingInfo& howToFinish
)

{
  if (howToFinish.needAnErrorClause &&
      (howToFinish.generatedAnIf || howToFinish.generatedSwitch))
    {
      if (howToFinish.generatedAnIf)
	{
	  BEGIN_ELSE_BLOCK;
	}
       else if (howToFinish.generatedSwitch)
        {
          BEGIN_BLOCK;
        }
    }
}

/** Generate the finish of a block, using a combination of the info
 * returned from genCommonBlock() and the action to perform when
 * no alts were taken
 * @param howToFinish The return of genCommonBlock()
 * @param noViableAction What to generate when no alt is taken
*/
void
MGrammar_CppCodeGenerator::endBlockFinish (
   const MGrammar_CppBlockFinishingInfo& howToFinish
)

{
  if (howToFinish.needAnErrorClause &&
      (howToFinish.generatedAnIf || howToFinish.generatedSwitch))
    {
      if (howToFinish.generatedAnIf)
       {
          END_IF;
       }
      else if (howToFinish.generatedSwitch)
       {
          END_BLOCK;
       }
    }

   for (int i = 1; i <= howToFinish.closingBracesOfIFSequence; i++)
    {
       END_IF;
    }

   if (howToFinish.generatedSwitch)
    {
      END_DEFAULT_BLOCK;
      END_SWITCH_BLOCK;
    }

}

/** Generate the finish of a block, using a combination of the info
 * returned from genCommonBlock() and the action to perform when
 * no alts were taken
 * @param howToFinish The return of genCommonBlock()
 * @param noViableAction What to generate when no alt is taken
*/
void
MGrammar_CppCodeGenerator::genBlockFinish (
   const MGrammar_CppBlockFinishingInfo& howToFinish,
   MLANG_String noViableAction
)

{
  if (howToFinish.needAnErrorClause &&
      (howToFinish.generatedAnIf || howToFinish.generatedSwitch))
    {
      if (howToFinish.generatedAnIf)
	{
	  BEGIN_ELSE_BLOCK;
          println (noViableAction);
          END_IF;
	}
      else
	{
	  BEGIN_BLOCK;
          println (noViableAction);
          END_BLOCK;
	}
//      tabs_+=INDENT_BLOCK;
    }

  if (howToFinish.postscript.length() != 0)
    {
      println (howToFinish.postscript);
    }
}

/** Generate the initaction for a block, which may be a RuleBlock or a
 * plain AlternativeBLock.
 * @blk The block for which the preamble is to be generated.
 */
void
MGrammar_CppCodeGenerator::
genBlockInitAction (MGrammar_AlternativeBlock_ptr blk)
{
  // dump out init action
  if (blk->getInitAction().length() != 0)
    {
      genLineNo (blk);
      super::printAction (processActionForSpecialSymbols (blk->getInitAction(), blk->getLine(),
						   currentRule, 0));
      genLineNo2 ();
    }
}

/** Generate the header for a block, which may be a RuleBlock or a
 * plain AlternativeBlock. This generates any variable declarations
 * and syntactic-predicate-testing variables.
 * @blk The block for which the preamble is to be generated.
 */
void
MGrammar_CppCodeGenerator::
genBlockPreamble (MGrammar_AlternativeBlock_ptr blk)
{
  // define labels for rule blocks.
  if (instanceof<MGrammar_RuleBlock>(blk))
    {
      MGrammar_RuleBlock_var rblk = Dynamic_Cast<MGrammar_RuleBlock>(blk);
      if (rblk->labeledElements().size() != 0)
	{
	  for (size_t i = 0; i < rblk->labeledElements().size (); i++)
	    {

	      MGrammar_AlternativeElement_var a = rblk->labeledElements()[i];
	      //SYSTEM->out()->println("looking at labeled element: "+a);
	      // Variables for labeled rule refs and subrules are different than
	      // variables for grammar atoms.  This test is a little tricky because
	      // we want to get all rule refs and ebnf, but not rule blocks or
	      // syntactic predicates
	      if (instanceof<MGrammar_RuleRefElement>(a) ||
		  instanceof<MGrammar_AlternativeBlock>(a) &&
		  !(instanceof<MGrammar_RuleBlock>(a)) && !(instanceof<MGrammar_SynPredBlock>(a)))
		{
		  if (!(instanceof<MGrammar_RuleRefElement>(a)) &&
		      Dynamic_Cast<MGrammar_AlternativeBlock>(a)->no() &&
		      analyzer_->subruleCanBeInverted (Dynamic_Cast<MGrammar_AlternativeBlock>(a),
						     grammar_->isLexer()))
		    {
		      // Special case for inverted subrules that will be
		      // inlined. Treat these like token or char literal
		      // references
		      println (labeledElementType + " " + a->getLabel () +
			       " = " + labeledElementInit + ";");
		      if (grammar_->buildAST())
			{
			  genASTDeclaration (a);
			}
		    }
		  else
		    {
		      if (grammar_->buildAST())
			{
			  // Always gen AST variables for labeled elements,
			  // even if the element itself is marked with !
			  genASTDeclaration (a);
			}
		      if ((grammar_->isLexer()))
			println (namespaceAntlr + "Token_var " +
				 a->getLabel () + ";");

		      if ((grammar_->isTreeWalker()))
			{
			  // always generate rule-ref variables for tree walker
			  println (labeledElementType + " " + a->getLabel () +
				   " = " + labeledElementInit + ";");
			}
		    }
		}
	      else
		{
		  // It is a token or literal reference.  Generate the
		  // correct variable type for this grammar
		  println (labeledElementType + " " + a->getLabel () + " = " +
			   labeledElementInit + ";");
		  // In addition, generate *_AST variables if building ASTs
		  if (grammar_->buildAST())
		    {
		      if (instanceof<MGrammar_GrammarAtom>(a) &&
			  (Dynamic_Cast<MGrammar_GrammarAtom>(a))->getASTNodeType ().length() != 0)
			{
			  MGrammar_GrammarAtom_var ga = Dynamic_Cast<MGrammar_GrammarAtom>(a);
			  genASTDeclaration (a, "Ref" + ga->getASTNodeType ());
			}
		      else
			{
			  genASTDeclaration (a);
			}
		    }
		}
	    }
	}
    }
}

void
MGrammar_CppCodeGenerator::genBody (MGrammar_LexerGrammar_ptr g)
{
  outputFile = grammar_->getClassName () + ".cpp";
  outputLine = 1;
  currentOutput_ = antlrTool_->openOutputFile (outputFile);
  //SAS: changed for proper text file io

  genAST = false;		// no way to gen trees.
  saveText = true;		// save consumed characters.

  tabs_ = 0;

  // Generate header common to all C++ output files
  genHeader (outputFile);

  printHeaderAction (preIncludeCpp);
  // Generate header specific to lexer C++ file
  println ("#include \"" + grammar_->getClassName () + ".h\"");
  println ("#include <MParser/CharBuffer.h>");
  println ("#include <MParser/ByteBuffer.h>");
  println ("#include <MParser/TokenStreamException.h>");
  println ("#include <MParser/TokenStreamIOException.h>");
  println ("#include <MParser/TokenStreamRecognitionException.h>");
  println ("#include <MParser/CharStreamException.h>");
  println ("#include <MParser/CharStreamIOException.h>");
  println ("#include <MParser/NoViableAltForCharException.h>");
  if (grammar_->debuggingOutput())
    println ("#include <MParser/DebuggingInputBuffer.h>");
  println ("");
  printHeaderAction (postIncludeCpp);

  if (nameSpace)
    nameSpace->emitDeclarations (currentOutput_);

  // Generate user-defined lexer file preamble
  printAction (grammar_->preambleAction());

  // Generate lexer class definition
  MLANG_String sup;
  if (grammar_->superClass().length() != 0)
    {
      sup = grammar_->superClass();
    }
  else
    {
      sup = grammar_->getSuperClass ();
      if (sup.lastIndexOf ('.') != -1)
	sup = sup.substring (sup.lastIndexOf ('.') + 1);
      sup = namespaceAntlr + sup;
    }

  if (noConstructors)
    {
      println ("#if 0");
      println
	("// constructor creation turned of with 'noConstructor' option");
    }
  //
  // Generate the constructor from InputStream
  //
  println (grammar_->getClassName () + "::" + grammar_->getClassName () +
	   "(const MIO_InputStream_var& in)");
  tabs_+=INDENT_BLOCK;
  // if debugging, wrap the input buffer in a debugger
  if (grammar_->debuggingOutput())
    println (": " + sup + "(new " + namespaceAntlr +
	     "DebuggingInputBuffer(new " + namespaceAntlr +
	     "ByteBuffer(in))," + MLANG_String::valueOf(g->caseSensitive()) + ")");
  else
    println (": " + sup + "(new " + namespaceAntlr + "ByteBuffer(in))");
  tabs_-=INDENT_BLOCK;
  BEGIN_BLOCK;

  // if debugging, set up array variables and call user-overridable
  //   debugging setup method
  if (grammar_->debuggingOutput())
    {
      println ("setRuleNames(_ruleNames);");
      println ("setSemPredNames(_semPredNames);");
      println ("setupDebugging();");
    }

  println("setCaseSensitive("+MLANG_String::valueOf(g->caseSensitive())+");");
  println ("initLiterals();");
  END_BLOCK;
  println ("");

  // Generate the constructor from InputBuffer
  println (grammar_->getClassName () + "::" + grammar_->getClassName () +
	   "(const " + namespaceAntlr + "InputBuffer_var& ib)");
  tabs_+=INDENT_BLOCK;
  // if debugging, wrap the input buffer in a debugger
  if (grammar_->debuggingOutput())
    println (": " + sup + "(new " + namespaceAntlr +
	     "DebuggingInputBuffer(ib)," + MLANG_String::valueOf(g->caseSensitive()) + ")");
  else
    println (": " + sup + "(ib)");
  tabs_-=INDENT_BLOCK;
  BEGIN_BLOCK;

  // if debugging, set up array variables and call user-overridable
  //   debugging setup method
  if (grammar_->debuggingOutput())
    {
      println ("setRuleNames(_ruleNames);");
      println ("setSemPredNames(_semPredNames);");
      println ("setupDebugging();");
    }

  println("setCaseSensitive("+MLANG_String::valueOf(g->caseSensitive())+");");
  println ("initLiterals();");
  END_BLOCK;
  println ("");

  // Generate the constructor from LexerSharedInputState
  println (grammar_->getClassName () + "::" + grammar_->getClassName () +
	   "(const " + namespaceAntlr + "LexerSharedInputState_var& state)");
  tabs_+=INDENT_BLOCK;
  println (": " + sup + "(state)");
  tabs_-=INDENT_BLOCK;
  BEGIN_BLOCK;

  // if debugging, set up array variables and call user-overridable
  //   debugging setup method
  if (grammar_->debuggingOutput())
    {
      println ("setRuleNames(_ruleNames);");
      println ("setSemPredNames(_semPredNames);");
      println ("setupDebugging();");
    }

  println("setCaseSensitive("+MLANG_String::valueOf(g->caseSensitive())+");");
  println ("initLiterals();");
  END_BLOCK;
  println ("");

  if (noConstructors)
    {
      println
	("// constructor creation turned of with 'noConstructor' option");
      println ("#endif");
    }

  println ("void " + grammar_->getClassName () + "::initLiterals()");
  BEGIN_BLOCK;
  println("setCaseSensitive("+MLANG_String::valueOf(g->caseSensitive())+");");
  // Generate the initialization of the map
  // containing the string literals used in the lexer
  // The literals variable itself is in CharScanner
  MGrammar_TokenManager::KEYS keys = grammar_->tokenManager_->getTokenSymbolKeys ();
  for (size_t i=0; i<keys.size(); i++)
    {
      MLANG_String key = keys[i];
      if (key.charAt (0) != '"')
	{
	  continue;
	}
      MGrammar_TokenSymbol_var sym = grammar_->tokenManager_->getTokenSymbol (key);
      if (instanceof<MGrammar_StringLiteralSymbol>(sym))
	{
	  MGrammar_StringLiteralSymbol_var s = Dynamic_Cast<MGrammar_StringLiteralSymbol>(sym);
	  println ("literals_.put(" + s->getId () + "," +
		   MLANG_String::valueOf(s->getTokenType ()) + ");");
	}
    }

  // Generate the setting of various generated options.
  END_BLOCK;

  MGrammar_Grammar::RULES ids;
  // generate the rule name array for debugging
  if (grammar_->debuggingOutput())
    {
      println ("const char* " + grammar_->getClassName () +
	       "::_ruleNames[] = {");
      tabs_++;

      ids = grammar_->rules();
//      int ruleNum = 0;
      for (size_t i=0; i<ids.size(); i++)
	{
	  MGrammar_GrammarSymbol_var sym = ids[i];
	  if (instanceof<MGrammar_RuleSymbol>(sym))
	    println ("\"" + (Dynamic_Cast<MGrammar_RuleSymbol>(sym))->getId () + "\",");
	}
      println ("0");
      tabs_--;
      println ("};");
    }

  // Generate nextToken() rule.
  // nextToken() is a synthetic lexer rule that is the implicit OR of all
  // user-defined lexer rules.
  genNextToken ();

  // Generate code for each rule in the lexer
  ids = grammar_->rules();
  int ruleNum = 0;
  for (size_t i=0; i<ids.size(); i++)
    {
      MGrammar_RuleSymbol_var sym = ids[i];
      // Don't generate the synthetic rules
      if (!sym->getId ().equals ("mnextToken"))
	{
	  genRule (sym, false, ruleNum++, grammar_->getClassName () + "::");
	}
      exitIfError ();
    }

  // Generate the semantic predicate map for debugging
  if (grammar_->debuggingOutput())
    genSemPredMap (grammar_->getClassName () + "::");

  // Generate the bitsets used throughout the lexer
  genBitsets (bitsetsUsed_,
	      Dynamic_Cast<MGrammar_LexerGrammar>(grammar_)->charVocabulary().size (),
	      grammar_->getClassName () + "::");

  println ("");
  if (nameSpace)
    nameSpace->emitClosures (currentOutput_);

  // Close the lexer output stream
  currentOutput_->close ();
  currentOutput_ = 0;
}

void
MGrammar_CppCodeGenerator::genInitFactory (MGrammar_Grammar_ptr g)
{
  // Generate the method to initialize an ASTFactory when we're
  // building AST's
  MLANG_String param_name = "factory ";
  if (!g->buildAST())
    param_name = "";

  println ("void " + g->getClassName () + "::initializeASTFactory( " +
	   namespaceAntlr + "ASTFactory& " + param_name + ")");
  BEGIN_BLOCK;

  if (g->buildAST())
    {
      // sort out custom AST types... synchronize token manager with token
      // specs on rules (and other stuff we were able to see from
      // action.g) (imperfect of course)
      MGrammar_TokenManager_var tm = grammar_->tokenManager_;
      MGrammar_TokenManager::KEYS tokens = tm->getTokenSymbolKeys ();
      for (size_t i=0; i<tokens.size (); i++)
	{
	  MLANG_String tok = tokens[i];
	  MGrammar_TokenSymbol_var ts = tm->getTokenSymbol (tok);
	  // if we have a custom type and there's not a more local override
	  // of the tokentype then mark this as the type for the tokentype
	  if (ts->getASTNodeType ().length() != 0)
	    {
	      // ensure capacity with this pseudo vector...
	      astTypes.ensureCapacity (ts->getTokenType ());
	      MLANG_String type = astTypes[ts->getTokenType ()];
	      if (type.length() == 0)
		astTypes.setElementAt (ts->getASTNodeType (),
				       ts->getTokenType ());
	      else
		{
		  // give a warning over action taken if the types are unequal
		  if (!ts->getASTNodeType ().equals (type))
		    {
		      antlrTool_->warning ("Token " + tok +
					   " taking most specific AST type",
					   grammar_->getFilename (), 1, 1);
		      antlrTool_->warning ("  using " + type + " ignoring " +
					   ts->getASTNodeType (),
					   grammar_->getFilename (), 1, 1);
		    }
		}
	    }
	}
      // now actually write out all the registered types. (except the default
      // type.
      for (size_t i = 0; i < astTypes.size (); i++)
	{
	  MLANG_String type = astTypes[i];
	  if (type.length() != 0)
	    {
	      println ("factory.registerFactory(" + MLANG_String::valueOf(i)
		       + ", \"" + type + "\", " + type + "::factory);");
	    }
	}
      println ("factory.setMaxNodeType(" +
	       MLANG_String::valueOf(grammar_->tokenManager_->maxTokenType ()) + ");");
    }
  END_BLOCK;
}

// FIXME: and so why are we passing here a g param while inside
// we merrily use the global grammar_->
void
MGrammar_CppCodeGenerator::genBody (MGrammar_ParserGrammar_ptr g)
{
  // Open the output stream for the parser and set the currentOutput
  outputFile = grammar_->getClassName () + ".cpp";
  outputLine = 1;
  currentOutput_ = antlrTool_->openOutputFile (outputFile);

  genAST = grammar_->buildAST();

  tabs_ = 0;

  // Generate the header common to all output files.
  genHeader (outputFile);

  printHeaderAction (preIncludeCpp);

  // Generate header for the parser
  println ("#include \"" + grammar_->getClassName () + ".h\"");
  println ("#include <MParser/NoViableAltException.h>");
  println ("#include <MParser/SemanticException.h>");
  println ("#include <MParser/ASTFactory.h>");

  printHeaderAction (postIncludeCpp);

  if (nameSpace)
    nameSpace->emitDeclarations (currentOutput_);

  // Output the user-defined parser preamble
  printAction (grammar_->preambleAction());

  MLANG_String sup;
  if (grammar_->superClass().length() != 0)
    sup = grammar_->superClass();
  else
    {
      sup = grammar_->getSuperClass ();
      if (sup.lastIndexOf ('.') != -1)
	sup = sup.substring (sup.lastIndexOf ('.') + 1);
      sup = namespaceAntlr + sup;
    }

  // set up an array of all the rule names so the debugger can
  // keep track of them only by number -- less to store in tree...
  if (grammar_->debuggingOutput())
    {
      println ("const char* " + grammar_->getClassName () +
	       "::_ruleNames[] = {");
      tabs_++;

//      int ruleNum = 0;
      MGrammar_Grammar::RULES ids = grammar_->rules();
      for (size_t i=0; i<ids.size(); i++)
	{
	  MGrammar_GrammarSymbol_var sym = ids[i];
	  if (instanceof<MGrammar_RuleSymbol>(sym))
	    println ("\"" + Dynamic_Cast<MGrammar_RuleSymbol>(sym)->getId () + "\",");
	}
      println ("0");
      tabs_--;
      println ("};");
    }

  // Generate _initialize function
  // disabled since it isn't used anymore..

//              println("void " + grammar_->getClassName() + "::_initialize(void)");
//              println("{");
//              tabs_++;

  // if debugging, set up arrays and call the user-overridable
  //   debugging setup method
//              if ( grammar_->debuggingOutput ) {
//                      println("setRuleNames(_ruleNames);");
//                      println("setSemPredNames(_semPredNames);");
//                      println("setupDebugging();");
//              }
//              tabs_--;
//              println("}");
  if (noConstructors)
    {
      println ("#if 0");
      println
	("// constructor creation turned of with 'noConstructor' option");
    }

  // Generate parser class constructor from TokenBuffer
  print (grammar_->getClassName () + "::" + grammar_->getClassName ());
  println ("(const " + namespaceAntlr + "TokenBuffer_var& tokenBuf, int k)");
  println (": " + sup + "(tokenBuf,k)");
  println ("{");
//              tabs_++;
//              println("_initialize();");
//              tabs_--;
  println ("}");
  println ("");

  print (grammar_->getClassName () + "::" + grammar_->getClassName ());
  println ("(const " + namespaceAntlr + "TokenBuffer_var& tokenBuf)");
  println (": " + sup + "(tokenBuf," + MLANG_String::valueOf(grammar_->maxk()) + ")");
  println ("{");
//              tabs_++;
//              println("_initialize();");
//              tabs_--;
  println ("}");
  println ("");

  // Generate parser class constructor from TokenStream
  print (grammar_->getClassName () + "::" + grammar_->getClassName ());
  println ("(const " + namespaceAntlr + "TokenStream_var& lexer, int k)");
  println (": " + sup + "(lexer,k)");
  println ("{");
//              tabs_++;
//              println("_initialize();");
//              tabs_--;
  println ("}");
  println ("");

  print (grammar_->getClassName () + "::" + grammar_->getClassName ());
  println ("(const " + namespaceAntlr + "TokenStream_var& lexer)");
  println (": " + sup + "(lexer," + MLANG_String::valueOf(grammar_->maxk()) + ")");
  println ("{");
//              tabs_++;
//              println("_initialize();");
//              tabs_--;
  println ("}");
  println ("");

  print (grammar_->getClassName () + "::" + grammar_->getClassName ());
  println ("(const " + namespaceAntlr + "ParserSharedInputState_var& state)");
  println (": " + sup + "(state," + MLANG_String::valueOf(grammar_->maxk()) + ")");
  println ("{");
//              tabs_++;
//              println("_initialize();");
//              tabs_--;
  println ("}");
  println ("");

  if (noConstructors)
    {
      println
	("// constructor creation turned of with 'noConstructor' option");
      println ("#endif");
    }

  astTypes = MLANG_Vector<MLANG_String> ();

  // Generate code for each rule in the grammar
  MGrammar_Grammar::RULES ids = grammar_->rules();
  int ruleNum = 0;
  for (size_t i=0; i<ids.size(); i++)
    {
      MGrammar_GrammarSymbol_var sym = ids[i];
      if (instanceof<MGrammar_RuleSymbol>(sym))
	{
	  MGrammar_RuleSymbol_var rs = Dynamic_Cast<MGrammar_RuleSymbol>(sym);
	  genRule (rs, rs->references().size () == 0, ruleNum++,
		   grammar_->getClassName () + "::");
	}
      exitIfError ();
    }

  genInitFactory (g);

  // Generate the token names
  genTokenStrings (grammar_->getClassName () + "::");

  // Generate the bitsets used throughout the grammar
  genBitsets (bitsetsUsed_, grammar_->tokenManager_->maxTokenType (),
	      grammar_->getClassName () + "::");

  // Generate the semantic predicate map for debugging
  if (grammar_->debuggingOutput ())
    genSemPredMap (grammar_->getClassName () + "::");

  // Close class definition
  println ("");
  println ("");
  if (nameSpace)
    nameSpace->emitClosures (currentOutput_);

  // Close the parser output stream
  currentOutput_->close ();
  currentOutput_ = 0;
}

void
MGrammar_CppCodeGenerator::genBody (MGrammar_TreeWalkerGrammar_ptr g)

{
  ACE_UNUSED_ARG(g);
  // Open the output stream for the parser and set the currentOutput
  outputFile = grammar_->getClassName () + ".cpp";
  outputLine = 1;
  currentOutput_ = antlrTool_->openOutputFile (outputFile);
  //SAS: changed for proper text file io

  genAST = grammar_->buildAST();
  tabs_ = 0;

  // Generate the header common to all output files.
  genHeader (outputFile);

  printHeaderAction (preIncludeCpp);

  // Generate header for the parser
  println ("#include \"" + grammar_->getClassName () + ".h\"");
  println ("#include <MParser/Token.h>");
  println ("#include <MParser/AST.h>");
  println ("#include <MParser/NoViableAltException.h>");
  println ("#include <MParser/MismatchedTokenException.h>");
  println ("#include <MParser/SemanticException.h>");
  println ("#include <MParser/BitSet.h>");

  printHeaderAction (postIncludeCpp);

  if (nameSpace)
    nameSpace->emitDeclarations (currentOutput_);

  // Output the user-defined parser premamble
  printAction (grammar_->preambleAction());

  // Generate parser class definition
  MLANG_String sup;
  if (grammar_->superClass().length() != 0)
    {
      sup = grammar_->superClass();
    }
  else
    {
      sup = grammar_->getSuperClass ();
      if (sup.lastIndexOf ('.') != -1)
	sup = sup.substring (sup.lastIndexOf ('.') + 1);
      sup = namespaceAntlr + sup;
    }
  if (noConstructors)
    {
      println ("#if 0");
      println
	("// constructor creation turned of with 'noConstructor' option");
    }

  // Generate default parser class constructor
  println (grammar_->getClassName () + "::" + grammar_->getClassName () +
	   "()");
  println ("\t: " + namespaceAntlr + "TreeParser() {");
  BEGIN_BLOCK;
//              println("setTokenNames(_tokenNames);");
  END_BLOCK;

  if (noConstructors)
    {
      println
	("// constructor creation turned of with 'noConstructor' option");
      println ("#endif");
    }
  println ("");

  astTypes = MLANG_Vector<MLANG_String> ();

  // Generate code for each rule in the grammar
  MGrammar_Grammar::RULES ids = grammar_->rules();
  int ruleNum = 0;
  MLANG_String ruleNameInits = "";
  for (size_t i=0; i<ids.size(); i++)
    {
      MGrammar_GrammarSymbol_var sym = ids[i];
      if (instanceof<MGrammar_RuleSymbol>(sym))
	{
	  MGrammar_RuleSymbol_var rs = Dynamic_Cast<MGrammar_RuleSymbol>(sym);
	  genRule (rs, rs->references().size () == 0, ruleNum++,
		   grammar_->getClassName () + "::");
	}
      exitIfError ();
    }

  // Generate the ASTFactory initialization function
  genInitFactory (grammar_);
  // Generate the token names
  genTokenStrings (grammar_->getClassName () + "::");

  // Generate the bitsets used throughout the grammar
  genBitsets (bitsetsUsed_, grammar_->tokenManager_->maxTokenType (),
	      grammar_->getClassName () + "::");

  // Close class definition
  println ("");
  println ("");

  if (nameSpace)
    nameSpace->emitClosures (currentOutput_);

  // Close the parser output stream
  currentOutput_->close ();
  currentOutput_ = 0;
}

/** Generate a series of case statements that implement a BitSet test.
 * @param p The Bitset for which cases are to be generated
 */
void
MGrammar_CppCodeGenerator::genCases (const MParser_BitSet & p)
{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genCases(" + p.toString () + ")");
  MLANG_Vector < size_t >elems;

  elems = p.toArray ();
  // Wrap cases four-per-line for lexer, one-per-line for parser
  int wrap = 1;			//(instanceof<LexerGrammar>(grammar)) ? 4 : 1;
  int j = 1;
  bool startOfLine = true;
  for (size_t i = 0; i < elems.size(); i++)
    {
      if (j == 1)
	{
	  print ("");
	}
      else
	{
	  _print ("  ");
	}
      _print ("case " + getValueString (elems[i]) + ":");

      if (j == wrap)
	{
	  _println ("");
	  startOfLine = true;
	  j = 1;
	}
      else
	{
	  j++;
	  startOfLine = false;
	}
    }
  if (!startOfLine)
    {
      _println ("");
    }
}

/** Generate common code for a block of alternatives; return a postscript
 * that needs to be generated at the end of the block.  Other routines
 * may append else-clauses and such for error checking before the postfix
 * is generated.
 * If the grammar is a lexer, then generate alternatives in an order where
 * alternatives requiring deeper lookahead are generated first, and
 * EOF in the lookahead set reduces the depth of the lookahead.
 * @param blk The block to generate
 * @param noTestForSingle If true, then it does not generate a test for a single alternative.
 */
MGrammar_CppBlockFinishingInfo
  MGrammar_CppCodeGenerator::genCommonBlock (
   MGrammar_AlternativeBlock_ptr blk,
   bool noTestForSingle
)

{
  int nIF = 0;
  bool createdLL1Switch = false;
  int closingBracesOfIFSequence = 0;
  MGrammar_CppBlockFinishingInfo finishingInfo;
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genCommonBlk(" + blk->toString() + ")");

  // Save the AST generation state, and set it to that of the block
  bool savegenAST = genAST;
  genAST = genAST && blk->getAutoGen ();

  bool oldsaveTest = saveText;
  saveText = saveText && blk->getAutoGen ();

  // Is this block inverted?  If so, generate special-case code
  if (blk->no() &&
      analyzer_->subruleCanBeInverted (blk,
				     (grammar_->isLexer())))
    {
      MGrammar_Lookahead p = analyzer_->look (1, blk);
      // Variable assignment for labeled elements
      if (blk->getLabel().length() != 0 && syntacticPredLevel == 0)
	{
	  println (blk->getLabel () + " = " + lt1Value + ";");
	}

      // AST
      genElementAST (blk);

      MLANG_String astArgs = "";
      if ((grammar_->isTreeWalker()))
	{
	  if (usingCustomAST)
	    astArgs = namespaceAntlr + "AST_var" + "(_t),";
	  else
	    astArgs = "_t,";
	}

      // match the bitset for the alternative
      println ("match(" + astArgs +
	       getBitsetName (markBitsetForGen (p.fset())) + ");");

      // tack on tree cursor motion if doing a tree walker
      if ((grammar_->isTreeWalker()))
	{
	  println ("_t = _t->getNextSibling();");
	}
      return finishingInfo;
    }

  // Special handling for single alt
  if (blk->getAlternatives ().size () == 1)
    {
      MGrammar_Alternative_var alt = blk->getAlternativeAt (0);
      // Generate a warning if there is a synPred for single alt->
      if (!MGrammar_SynPredBlock::is_nil(alt->synPred_))
	{
	  antlrTool_->
	    warning ("Syntactic predicate superfluous for single alternative",
		     grammar_->getFilename (),
		     blk->getAlternativeAt (0)->synPred_->getLine (),
		     blk->getAlternativeAt (0)->synPred_->getColumn ());
	}
      if (noTestForSingle)
	{
	  if (alt->semPred_.length())
	    {
	      // Generate validating predicate
	      genSemPred (alt->semPred_, blk->getLine());
	    }
	  genAlt (alt, blk);
//          finishingInfo.needAnErrorClause = false;
	  return finishingInfo;
	}
    }

  // count number of simple LL(1) cases; only do switch for
  // many LL(1) cases (no preds, no end of token refs)
  // We don't care about exit paths for (...)*, (...)+
  // because we don't explicitly have a test for them
  // as an alt in the loop.
  //
  // Also, we now count how many unicode lookahead sets
  // there are--they must be moved to DEFAULT or ELSE
  // clause.

  int nLL1 = 0;
  for (size_t i = 0; i < blk->getAlternatives ().size (); i++)
    {
      MGrammar_Alternative_var a = blk->getAlternativeAt (i);
      if (suitableForCaseExpression (a))
	nLL1++;
    }

  // do LL(1) cases
  if (nLL1 >= makeSwitchThreshold_)
    {
      // Determine the name of the item to be compared
      MLANG_String testExpr = lookaheadString (1);
      createdLL1Switch = true;
      // when parsing trees, convert null to valid tree node with NULL lookahead
      if ((grammar_->isTreeWalker()))
	{
	  BEGIN_IF("_t == " + labeledElementASTInit);
	    println ("_t = ASTNULL;");
	  END_IF;
	}
      BEGIN_SWITCH_BLOCK(testExpr);
      for (size_t i = 0; i < blk->getAlternatives().size (); i++)
	{
	  MGrammar_Alternative_var alt = blk->getAlternativeAt (i);
	  // ignore any non-LL(1) alts, predicated alts or end-of-token alts
	  // or end-of-token alts for case expressions
	  if (!suitableForCaseExpression (alt))
	    {
	      continue;
	    }
	  MGrammar_Lookahead p = alt->cache_[1];
	  if (p.fset().degree () == 0 && !p.containsEpsilon ())
//	  if (p.fset().degree () == 0)
	    {
	      antlrTool_->
		warning ("Alternate omitted due to empty prediction set p=" + p.toString(),
			 grammar_->getFilename (), alt->head_->getLine (),
			 alt->head_->getColumn ());
	    }
	  else
	    {
	      genCases (p.fset());
              BEGIN_CASE_BLOCK;
	      genAlt (alt, blk);
	      println ("break;");
              END_CASE_BLOCK;
	    }
	}
      println ("default:");
      BEGIN_DEFAULT_BLOCK;
    }

  // do non-LL(1) and nondeterministic cases
  // This is tricky in the lexer, because of cases like:
  //     STAR : '*' ;
  //     ASSIGN_STAR : "*=";
  // Since nextToken is generated without a loop, then the STAR will
  // have end-of-token as it's lookahead set for LA(2).  So, we must generate the
  // alternatives containing trailing end-of-token in their lookahead sets *after*
  // the alternatives without end-of-token.  This implements the usual
  // lexer convention that longer matches come before shorter ones, e.g.
  // "*=" matches ASSIGN_STAR not STAR
  //
  // For non-lexer grammars, this does not sort the alternates by depth
  // Note that alts whose lookahead is purely end-of-token at k=1 end up
  // as default or else clauses.
  int startDepth =
    ((grammar_->isLexer())) ? grammar_->maxk() : 0;
  for (int altDepth = startDepth; altDepth >= 0; altDepth--)
    {
      if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
	SYSTEM->out ()->println ("checking depth " + altDepth);
      for (size_t i = 0; i < blk->getAlternatives().size (); i++)
	{
	  MGrammar_Alternative_var alt = blk->getAlternativeAt (i);
	  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
	    SYSTEM->out ()->println ("genAlt: " + MLANG_String::valueOf(i));
	  // if we made a switch above, ignore what we already took care
	  // of.  Specifically, LL(1) alts with no preds
	  // that do not have end-of-token in their prediction set
	  if (createdLL1Switch && suitableForCaseExpression (alt))
	    {
	      if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
		SYSTEM->out ()->
		  println ("ignoring alt because it was in the switch");
	      continue;
	    }
	  MLANG_String e;

	  bool unpredicted = false;

	  if ((grammar_->isLexer()))
	    {
	      // Calculate the "effective depth" of the alt, which is the max
	      // depth at which cache[depth]!=end-of-token
	      int effectiveDepth = alt->lookaheadDepth_;
	      if (effectiveDepth == MGrammar_GrammarAnalyzer::NONDETERMINISTIC)
		{
		  // use maximum lookahead
		  effectiveDepth = grammar_->maxk();
		}
	      while (effectiveDepth >= 1 &&
		     alt->cache_[effectiveDepth].containsEpsilon ())
		{
		  effectiveDepth--;
		}
	      // Ignore alts whose effective depth is other than the ones we
	      // are generating for this iteration.
	      if (effectiveDepth != altDepth)
		{
		  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
		    SYSTEM->out ()->
		      println
		      ("ignoring alt because effectiveDepth!=altDepth;" +
		       MLANG_String::valueOf(effectiveDepth) + "!=" + MLANG_String::valueOf(altDepth));
		  continue;
		}
	      unpredicted = lookaheadIsEmpty (alt, effectiveDepth);
	      e = getLookaheadTestExpression (alt, effectiveDepth);
	    }
	  else
	    {
	      unpredicted = lookaheadIsEmpty (alt, grammar_->maxk());
	      e = getLookaheadTestExpression (alt, grammar_->maxk());
	    }

	  // Was it a big unicode range that forced unsuitability
	  // for a case expression?
	  if ((int)alt->cache_[1].fset().degree () > caseSizeThreshold &&
	      suitableForCaseExpression (alt))
	    {
	      if (nIF == 0)
		{
		  // generate this only for the first if the elseif's
		  // are covered by this one
		  if ((grammar_->isTreeWalker()))
		    {
		      BEGIN_IF("_t == " + labeledElementASTInit);
		        println ("_t = ASTNULL;");
		      END_IF;
		    }
                  BEGIN_IF(e);
		}
	      else
                {
		   BEGIN_ELSE_IF (e);
                }
	    }
	  else if (unpredicted 
          &&      alt->semPred_.length() == 0 
          &&      MGrammar_SynPredBlock::is_nil(alt->synPred_))
	    {
	      // The alt has empty prediction set and no
	      // predicate to help out.  if we have not
	      // generated a previous if, just put {...} around
	      // the end-of-token clause
	      if (nIF == 0)
		{
		  BEGIN_BLOCK;
                  println ("// este bloque debe ser terminado");
		}
	      else
		{
		  BEGIN_ELSE_BLOCK;
                  println ("// este else bloque debe ser terminado");
		}
              closingBracesOfIFSequence++;
	      finishingInfo.needAnErrorClause = false;
	    }
	  else
	    {
	      // check for sem and syn preds
	      // Add any semantic predicate expression to the lookahead test
	      if (alt->semPred_.length() != 0)
		{
		  // if debugging, wrap the evaluation of the predicate in a method
		  //
		  // translate $ and # references
		  MGrammar_ActionTransInfo_var tInfo = new MGrammar_ActionTransInfo ();
		  MLANG_String actionStr =
		    processActionForSpecialSymbols (alt->semPred_,
						    blk->getLine(),
						    currentRule,
						    tInfo);
		  // ignore translation info...we don't need to do anything with it.

		  // call that will inform SemanticPredicateListeners of the
		  // result
		  if (grammar_->debuggingOutput() &&
		      (((grammar_->isParser()))
		       || ((grammar_->isLexer()))))
		    e = "(" + e + "&& fireSemanticPredicateEvaluated(antlr.debug.SemanticPredicateEvent.PREDICTING," +	//FIXME
		      MLANG_String::valueOf(addSemPred (charFormatter_->
				  escapeString (actionStr))) + "," +
		      actionStr + "))";
		  else
		    e = "(" + e + "&&(" + actionStr + "))";
		}

	      // Generate any syntactic predicates
	      if (nIF > 0)
		{
		  if (!MGrammar_SynPredBlock::is_nil(alt->synPred_))
		    {
		      BEGIN_ELSE_BLOCK;
		      genSynPred (alt->synPred_, e);
		      closingBracesOfIFSequence++;
		    }
		  else
		    {
		      BEGIN_ELSE_IF(e);
		    }
		}
	      else
		{
		  if (!MGrammar_SynPredBlock::is_nil(alt->synPred_))
		    {
		      genSynPred (alt->synPred_, e);
		    }
		  else
		    {
		      // when parsing trees, convert null to valid tree node
		      // with NULL lookahead.
		      if ((grammar_->isTreeWalker()))
			{
			  BEGIN_IF ("_t == " + labeledElementASTInit);
			  println ("_t = ASTNULL;");
			  END_IF;
			}
		      BEGIN_IF(e);
		    }
		}

	    }

	  nIF++;
//	  BEGIN_BLOCK;
	    genAlt (alt, blk);
//          END_BLOCK;
	}
    }
  // Restore the AST generation state
  genAST = savegenAST;

  // restore save text state
  saveText = oldsaveTest;

  // Return the finishing info.
  if (createdLL1Switch)
    {
      finishingInfo.generatedSwitch = true;
      finishingInfo.closingBracesOfIFSequence = closingBracesOfIFSequence;
      finishingInfo.generatedAnIf = nIF > 0;
      //return new CppBlockFinishingInfo(ps+"}",true,nIF>0); // close up switch statement

    }
  else
    {
      finishingInfo.generatedSwitch = false;
      finishingInfo.closingBracesOfIFSequence = closingBracesOfIFSequence;
      finishingInfo.generatedAnIf = nIF > 0;
      //return new CppBlockFinishingInfo(ps, false,nIF>0);
    }
  return finishingInfo;
}

bool
MGrammar_CppCodeGenerator::suitableForCaseExpression (MGrammar_Alternative_ptr a)
{
  return a->lookaheadDepth_ == 1 &&
    a->semPred_.length() == 0 &&
    !a->cache_[1].containsEpsilon () &&
    (int)a->cache_[1].fset().degree () <= caseSizeThreshold;
}

/** Generate code to link an element reference into the AST
 */
void
MGrammar_CppCodeGenerator::genElementAST (MGrammar_AlternativeElement_ptr el)
{

  // handle case where you're not building trees, but are in tree walker.
  // Just need to get labels set up.
  if ((grammar_->isTreeWalker()) && !grammar_->buildAST())
    {
      MLANG_String elementRef;
      MLANG_String astName;

      // Generate names and declarations of the AST variable(s)
      if (el->getLabel ().length() == 0)
	{
	  elementRef = lt1Value;
	  // Generate AST variables for unlabeled stuff
	  astName = "tmp" + MLANG_String::valueOf(astVarNumber) + "_AST";
	  astVarNumber++;
	  // Map the generated AST variable in the alternate
	  mapTreeVariable (el, astName);
	  // Generate an "input" AST variable also
	  println (labeledElementASTType + " " + astName + "_in = " +
		   elementRef + ";");
	}
      return;
    }

  if (grammar_->buildAST() && syntacticPredLevel == 0)
    {
      bool needASTDecl =
	(genAST && (el->getLabel().length() != 0 ||
		    el->getAutoGenType () != MGrammar_GrammarElement::AUTO_GEN_BANG));

      // RK: if we have a grammar element always generate the decl
      // since some guy can access it from an action and we can't
      // peek ahead (well not without making a mess).
      // I'd prefer taking this out.
      if (el->getAutoGenType () != MGrammar_GrammarElement::AUTO_GEN_BANG &&
	  (instanceof<MGrammar_TokenRefElement>(el)))
	needASTDecl = true;

      bool doNoGuessTest = (grammar_->hasSyntacticPredicate() && needASTDecl);

      MLANG_String elementRef;
      MLANG_String astNameBase;

      // Generate names and declarations of the AST variable(s)
      if (el->getLabel().length() != 0)
	{
	  // if the element is labeled use that name...
	  elementRef = el->getLabel ();
	  astNameBase = el->getLabel ();
	}
      else
	{
	  // else generate a temporary name...
	  elementRef = lt1Value;
	  // Generate AST variables for unlabeled stuff
	  astNameBase = "tmp" + MLANG_String::valueOf(astVarNumber);
	  astVarNumber++;
	}

      // Generate the declaration if required.
      if (needASTDecl)
	{
	  if (instanceof<MGrammar_GrammarAtom>(el))
	    {
	      MGrammar_GrammarAtom_var ga = Dynamic_Cast<MGrammar_GrammarAtom>(el);
	      if (ga->getASTNodeType ().length())
		{
		  genASTDeclaration (el, astNameBase,
				     "Ref" + ga->getASTNodeType ());
//                                              println("Ref"+ga.getASTNodeType()+" " + astName + ";");
		}
	      else
		{
		  genASTDeclaration (el, astNameBase, labeledElementASTType);
//                                              println(labeledElementASTType+" " + astName + " = "+labeledElementASTInit+";");
		}
	    }
	  else
	    {
	      genASTDeclaration (el, astNameBase, labeledElementASTType);
//                                      println(labeledElementASTType+" " + astName + " = "+labeledElementASTInit+";");
	    }
	}

      // for convenience..
      MLANG_String astName = astNameBase + "_AST";

      // Map the generated AST variable in the alternate
      mapTreeVariable (el, astName);
      if ((grammar_->isTreeWalker()))
	{
	  // Generate an "input" AST variable also
	  println (labeledElementASTType + " " + astName + "_in = " +
		   labeledElementASTInit + ";");
	}

      // Enclose actions with !guessing
      if (doNoGuessTest)
	{
	  BEGIN_IF("inputState_->guessing_ == 0");
	}

      // if something has a label assume it will be used
      // so we must initialize the AST_var
      if (el->getLabel().length() != 0)
	{
	  if (instanceof<MGrammar_GrammarAtom>(el))
	    {
	      println (astName + " = " +
		       getASTCreateString (Dynamic_Cast<MGrammar_GrammarAtom>(el),
						 elementRef) + ";");
	    }
	  else
	    {
	      println (astName + " = " +
		       getASTCreateString (elementRef) + ";");
	    }
	}

      // if it has no label but a declaration exists initialize it.
      if (el->getLabel ().length() == 0 && needASTDecl)
	{
	  elementRef = lt1Value;
	  if (instanceof<MGrammar_GrammarAtom>(el))
	    {
	      println (astName + " = " +
		       getASTCreateString (Dynamic_Cast<MGrammar_GrammarAtom>(el),
						 elementRef) + ";");
	    }
	  else
	    {
	      println (astName + " = " +
		       getASTCreateString (elementRef) + ";");
	    }
	  // Map the generated AST variable in the alternate
	  if ((grammar_->isTreeWalker()))
	    {
	      // set "input" AST variable also
	      println (astName + "_in = " + elementRef + ";");
	    }
	}

      if (genAST)
	{
	  switch (el->getAutoGenType ())
	    {
	    case MGrammar_GrammarElement::AUTO_GEN_NONE:
	      if (usingCustomAST ||
		  (instanceof<MGrammar_GrammarAtom>(el) &&
		   (Dynamic_Cast<MGrammar_GrammarAtom>(el))->getASTNodeType ().length() != 0))
		println ("astFactory->addASTChild(currentAST, " +
			 namespaceAntlr + "AST_var(" + astName + "));");
	      else
		println ("astFactory->addASTChild(currentAST, " + astName +
			 ");");
	      //                                              println("astFactory.addASTChild(currentAST, "+namespaceAntlr+"AST_var(" + astName + "));");
	      break;
	    case MGrammar_GrammarElement::AUTO_GEN_CARET:
	      if (usingCustomAST ||
		  (instanceof<MGrammar_GrammarAtom>(el) &&
		   (Dynamic_Cast<MGrammar_GrammarAtom>(el))->getASTNodeType ().length() != 0))
		println ("astFactory->makeASTRoot(currentAST, " +
			 namespaceAntlr + "AST_var(" + astName + "));");
	      else
		println ("astFactory->makeASTRoot(currentAST, " + astName +
			 ");");
	      break;
	    default:
	      break;
	    }
	}
      if (doNoGuessTest)
	{
          END_BLOCK;
	}
    }
}

/** Close the try block and generate catch phrases
 * if the element has a labeled handler in the rule
 */
void
MGrammar_CppCodeGenerator::genErrorCatchForElement (MGrammar_AlternativeElement_ptr el)
{
  if (el->getLabel ().length() == 0)
    return;
  MLANG_String r = el->enclosingRuleName();
  if ((grammar_->isLexer()))
    {
      r = MGrammar_CodeGenerator::encodeLexerRuleName (el->enclosingRuleName());
    }
  MGrammar_RuleSymbol_var rs = Dynamic_Cast<MGrammar_RuleSymbol>(grammar_->getSymbol (r));
  if (!rs)
    {
      antlrTool_->panic ("Enclosing rule not found!");
    }
  MGrammar_ExceptionSpec_var ex = rs->getBlock()->findExceptionSpec (el->getLabel ());
  if (ex)
    {
      genErrorHandler (ex);
//      END_TRY_BLOCK;
    }
}

/** Generate the catch phrases for a user-specified error handler */
void
MGrammar_CppCodeGenerator::genErrorHandler (MGrammar_ExceptionSpec_ptr ex)
{
  // Each ExceptionHandler in the ExceptionSpec is a separate catch
  for (size_t i = 0; i < ex->handlers().size (); i++)
    {
      MGrammar_ExceptionHandler_var handler = ex->handlers()[i];
      // Generate catch phrase
      BEGIN_CATCH_BLOCK (handler->exceptionTypeAndName()->getText ());
      if (grammar_->hasSyntacticPredicate())
	{
	  BEGIN_IF("inputState_->guessing_==0");
	}

      // When not guessing, execute user handler action
      MGrammar_ActionTransInfo_var tInfo = new MGrammar_ActionTransInfo ();
      genLineNo (handler->action().in());
      super::printAction (processActionForSpecialSymbols (handler->action()->getText (),
						   handler->action()->getLine (),
						   currentRule, tInfo));
      genLineNo2 ();

      if (grammar_->hasSyntacticPredicate())
	{
          BEGIN_ELSE_BLOCK;
	    // When guessing, rethrow exception
	    println ("throw;");
          END_IF;
	}
    }
}

/** Generate a try { opening if the element has a labeled handler in the rule */
void
MGrammar_CppCodeGenerator::genErrorTryForElement (MGrammar_AlternativeElement_ptr el)
{
  if (el->getLabel ().length() == 0)
    return;
  MLANG_String r = el->enclosingRuleName();
  if ((grammar_->isLexer()))
    {
      r = MGrammar_CodeGenerator::encodeLexerRuleName (el->enclosingRuleName());
    }
  MGrammar_RuleSymbol_var rs = Dynamic_Cast<MGrammar_RuleSymbol>(grammar_->getSymbol (r));
  if (!rs)
    {
      antlrTool_->panic ("Enclosing rule not found!");
    }
  MGrammar_ExceptionSpec_var ex = rs->getBlock()->findExceptionSpec (el->getLabel ());
  if (ex)
    {
      BEGIN_TRY_BLOCK ("// for error handling");
    }
}

/** Generate a header that is common to all C++ files */
void
MGrammar_CppCodeGenerator::genHeader (MLANG_String fileName)
{
#if 0
  println ("/* $ANTLR " + antlrTool_->version + ": " +
	   "\"" + antlrTool_->fileMinusPath (antlrTool_->grammarFile()) + "\"" +
	   " -> " + "\"" + fileName + "\"$ */");
#else
  println (MLANG_String("/* $ANTLR ") + ": " +
	   "\"" + antlrTool_->fileMinusPath (antlrTool_->grammarFile()) + "\"" +
	   " -> " + "\"" + fileName + "\"$ */");
#endif
}

// these are unique to C++ mode
void
MGrammar_CppCodeGenerator::genInclude (MGrammar_LexerGrammar_ptr g)
{
  outputFile = grammar_->getClassName () + ".h";
  outputLine = 1;
  currentOutput_ = antlrTool_->openOutputFile (outputFile);
  //SAS: changed for proper text file io

  genAST = false;		// no way to gen trees.
  saveText = true;		// save consumed characters.

  tabs_ = 0;

  // Generate a guard wrapper
  println ("#ifndef INC_" + grammar_->getClassName () + "_hpp_");
  println ("#define INC_" + grammar_->getClassName () + "_hpp_");
  println ("");

  printHeaderAction (preIncludeHpp);

//  println ("#include <MParser/config.h>");

  // Generate header common to all C++ output files
  genHeader (outputFile);

  // Generate header specific to lexer header file
  println ("#include <MLANG/SmartPtr.h>");
  println ("#include <MIO/InputStream.h>");
  println ("#include <MParser/CommonToken.h>");
  println ("#include <MParser/InputBuffer.h>");
  println ("#include <MParser/BitSet.h>");

  // Find the name of the super class
  MLANG_String sup;
  if (grammar_->superClass().length() != 0)
    {
      sup = grammar_->superClass();

      println
	("\n// Include correct superclass header with a header statement for example:");
      println ("// header \"post_include_hpp\" {");
      println ("// #include \"" + sup + ".h\"");
      println ("// }");
      println ("// Or....");
      println ("// header {");
      println ("// #include \"" + sup + ".h\"");
      println ("// }\n");
    }
  else
    {
      sup = grammar_->getSuperClass ();
      if (sup.lastIndexOf ('.') != -1)
	sup = sup.substring (sup.lastIndexOf ('.') + 1);
      println ("#include <MParser/" + sup + ".h>");
      sup = namespaceAntlr + sup;
    }
  println ("#include \"" + grammar_->tokenManager_->getName () +
	   TokenTypesFileSuffix_ + ".h\"");

  // Do not use printAction because we assume tabs==0
  printHeaderAction (postIncludeHpp);

  if (nameSpace)
    nameSpace->emitDeclarations (currentOutput_);

  printHeaderAction ("");

  // print javadoc comment if any
  if (grammar_->comment().length() != 0)
    {
      _println (grammar_->comment());
    }

  // Generate lexer class definition
  println ("");
  println ("class " + grammar_->getClassName () + ";");
  println ("typedef class " + grammar_->getClassName () + " *" + grammar_->getClassName()+ "_ptr" + ";");
  println ("typedef MLANG_SmartPtr<" + grammar_->getClassName () + "> " + grammar_->getClassName()+ "_var" + ";");
  println ("");
  print ("class " + grammar_->getClassName () + " : public " + sup);
  println (", public " + grammar_->tokenManager_->getName () + TokenTypesFileSuffix_);

  MParser_Token_var tsuffix = grammar_->options().get ("classHeaderSuffix");
  if (tsuffix)
    {
      MLANG_String suffix =
	MParser_StringUtils::stripFrontBack (tsuffix->getText (), "\"",
					   "\"");
      if (suffix.length() != 0)
	{
	  print (", " + suffix);	// must be an interface name for Java
	}
    }
  println ("{");

  // Generate user-defined lexer class members
  if (grammar_->classMemberAction())
    {
      genLineNo (grammar_->classMemberAction().in());
      print (processActionForSpecialSymbols
	     (grammar_->classMemberAction()->getText (),
	      grammar_->classMemberAction()->getLine (), currentRule, 0));
      genLineNo2 ();
    }

  // Generate initLiterals() method
  tabs_ = 0;
  println ("private:");
  tabs_ = 1;
  println ("void initLiterals();");

  // Generate getCaseSensitiveLiterals() method
  BEGIN_METHOD_DECL("public");
    println ("bool getCaseSensitiveLiterals() const");
    BEGIN_METHOD_BLOCK;
    println ("return " + 
             MLANG_String::valueOf(g->caseSensitiveLiterals()) + 
             ";");
    END_METHOD_BLOCK;
  END_METHOD_DECL;

  // Make constructors public
  BEGIN_METHOD_DECL ("public");

  if (noConstructors)
    {
      println ("#if 0");
      println
	("// constructor creation turned of with 'noConstructor' option");
    }

#if 0
  // Generate the constructor from std::istream
  println (grammar_->getClassName () + "(" + namespaceStd + "istream& in);");
#endif

  // Generate the constructor from InputStream
  println (grammar_->getClassName () + "(const " + "MIO_" +
	   "InputStream_var& istream);");

  // Generate the constructor from InputBuffer
  println (grammar_->getClassName () + "(const " + namespaceAntlr +
	   "InputBuffer_var& ib);");

  // Generate the constructor from InputBuffer
  println (grammar_->getClassName () + "(const " + namespaceAntlr +
	   "LexerSharedInputState_var& state);");
  if (noConstructors)
    {
      println
	("// constructor creation turned of with 'noConstructor' option");
      println ("#endif");
    }

  // Generate nextToken() rule.
  // nextToken() is a synthetic lexer rule that is the implicit OR of all
  // user-defined lexer rules.
  println (namespaceAntlr + "Token_var nextToken();");
  END_METHOD_DECL;

  // Generate code for each rule in the lexer
  MGrammar_Grammar::RULES ids = grammar_->rules();
  for (size_t i=0; i<ids.size(); i++)
    {
      MGrammar_RuleSymbol_var sym = ids[i];
      // Don't generate the synthetic rules
      if (!sym->getId ().equals ("mnextToken"))
	{
	  genRuleHeader (sym, false);
	}
      exitIfError ();
    }

  // Generate SmartPtr methods
  BEGIN_METHOD_DECL("public");

    println ("static " + grammar_->getClassName () + "_ptr " + "_nil()");
    BEGIN_METHOD_BLOCK;
      println ("return 0;");
    END_METHOD_BLOCK;

    println ("static bool is_nil(" + grammar_->getClassName () + "_ptr p)");
    BEGIN_METHOD_BLOCK;
      println ("return p==0;");
    END_METHOD_BLOCK;

    println ("static " + grammar_->getClassName () + "_ptr " + "_duplicate(" + grammar_->getClassName() + "_ptr p)");
    BEGIN_METHOD_BLOCK;
      println ("return MLANG_acquire(p);");
    END_METHOD_BLOCK;

  END_METHOD_DECL;

  // Make the rest private
  tabs_ = 0;
  println ("private:");
  tabs_ = 1;

  // generate the rule name array for debugging
  if (grammar_->debuggingOutput())
    {
      println ("static const char* _ruleNames[];");
    }

  // Generate the semantic predicate map for debugging
  if (grammar_->debuggingOutput())
    println ("static const char* _semPredNames[];");

  // Generate the bitsets used throughout the lexer
  genBitsetsHeader (bitsetsUsed_,
		    Dynamic_Cast<MGrammar_LexerGrammar>(grammar_)->charVocabulary().size ());

  tabs_ = 0;
  println ("};");
  println ("");
  if (nameSpace)
    nameSpace->emitClosures (currentOutput_);

  // Generate a guard wrapper
  println ("#endif /*INC_" + grammar_->getClassName () + "_hpp_*/");

  // Close the lexer output stream
  currentOutput_->close ();
  currentOutput_ = 0;
}

void
MGrammar_CppCodeGenerator::genInclude (MGrammar_ParserGrammar_ptr g)
{
  ACE_UNUSED_ARG(g);
  // Open the output stream for the parser and set the currentOutput
  outputFile = grammar_->getClassName () + ".h";
  outputLine = 1;
  currentOutput_ = antlrTool_->openOutputFile (outputFile);
  //SAS: changed for proper text file io

  genAST = grammar_->buildAST();

  tabs_ = 0;

  // Generate a guard wrapper
  println ("#ifndef INC_" + grammar_->getClassName () + "_hpp_");
  println ("#define INC_" + grammar_->getClassName () + "_hpp_");
  println ("");
  printHeaderAction (preIncludeHpp);
//  println ("#include <MParser/config.h>");

  // Generate the header common to all output files.
  genHeader (outputFile);

  // Generate header for the parser
  println ("#include <MParser/TokenStream.h>");
  println ("#include <MParser/TokenBuffer.h>");

  // Generate parser class definition
  MLANG_String sup;
  if (grammar_->superClass().length() != 0)
    {
      sup = grammar_->superClass();
      println
	("\n// Include correct superclass header with a header statement for example:");
      println ("// header \"post_include_hpp\" {");
      println ("// #include \"" + sup + ".h\"");
      println ("// }");
      println ("// Or....");
      println ("// header {");
      println ("// #include \"" + sup + ".h\"");
      println ("// }\n");
    }
  else
    {
      sup = grammar_->getSuperClass ();
      if (sup.lastIndexOf ('.') != -1)
	sup = sup.substring (sup.lastIndexOf ('.') + 1);
      println ("#include <MParser/" + sup + ".h>");
      sup = namespaceAntlr + sup;
    }
  println ("#include \"" + grammar_->tokenManager_->getName () +
	   TokenTypesFileSuffix_ + ".h\"");
  println ("");

  // Do not use printAction because we assume tabs==0
  printHeaderAction (postIncludeHpp);

  if (nameSpace)
    nameSpace->emitDeclarations (currentOutput_);

  printHeaderAction ("");

  // print javadoc comment if any
  if (grammar_->comment().length() != 0)
    {
      _println (grammar_->comment());
    }

  // generate the actual class definition
  println ("");
  println ("class " + grammar_->getClassName () + ";");
  println ("typedef class " + grammar_->getClassName () + " *" + grammar_->getClassName()+ "_ptr" + ";");
  println ("typedef MLANG_SmartPtr<" + grammar_->getClassName () + "> " + grammar_->getClassName()+ "_var" + ";");
  println ("");
  print ("class " + grammar_->getClassName () + " : public " + sup);
  println (", public " + grammar_->tokenManager_->getName () + TokenTypesFileSuffix_);

  MParser_Token_var tsuffix = grammar_->options().get ("classHeaderSuffix");
  if (tsuffix)
    {
      MLANG_String suffix =
	MParser_StringUtils::stripFrontBack (tsuffix->getText (), "\"",
					   "\"");
      if (suffix.length() != 0)
	print (", " + suffix);	// must be an interface name for Java
    }
  println ("{");

  // set up an array of all the rule names so the debugger can
  // keep track of them only by number -- less to store in tree...
  if (grammar_->debuggingOutput())
    {
      println ("public: static const char* _ruleNames[];");
    }
  // Generate user-defined parser class members
  if (grammar_->classMemberAction())
    {
      genLineNo (grammar_->classMemberAction()->getLine ());
      print (processActionForSpecialSymbols
	     (grammar_->classMemberAction()->getText (),
	      grammar_->classMemberAction()->getLine (), currentRule, 0));
      genLineNo2 ();
    }
  println ("public:");
  tabs_ = 1;
  println ("void initializeASTFactory( " + namespaceAntlr +
	   "ASTFactory& factory );");
//              println("// called from constructors");
//              println("void _initialize( void );");

  // Generate parser class constructor from TokenBuffer
  tabs_ = 0;
  if (noConstructors)
    {
      println ("#if 0");
      println
	("// constructor creation turned of with 'noConstructor' option");
    }
  BEGIN_METHOD_DECL("protected");
  println (grammar_->getClassName () + "(const " + namespaceAntlr +
	   "TokenBuffer_var& tokenBuf, int k);");
  END_METHOD_DECL;
  BEGIN_METHOD_DECL ("public");
  println (grammar_->getClassName () + "(const " + namespaceAntlr +
	   "TokenBuffer_var& tokenBuf);");

  // Generate parser class constructor from TokenStream
  END_METHOD_DECL;
  BEGIN_METHOD_DECL ("protected");
  println (grammar_->getClassName () + "(const " + namespaceAntlr +
	   "TokenStream_var& lexer, int k);");
  END_METHOD_DECL;
  BEGIN_METHOD_DECL ("public");
  println (grammar_->getClassName () + "(const " + namespaceAntlr +
	   "TokenStream_var& lexer);");
  END_METHOD_DECL;
  println (grammar_->getClassName () + "(const " + namespaceAntlr +
	   "ParserSharedInputState_var& state);");
  if (noConstructors)
    {
      tabs_ = 0;
      println
	("// constructor creation turned of with 'noConstructor' option");
      println ("#endif");
      tabs_ = 1;
    }

  BEGIN_METHOD_DECL("public");
     println ("int getNumTokens() const");
     BEGIN_METHOD_BLOCK;
     println ("return " + grammar_->getClassName () + "::NUM_TOKENS;");
  END_METHOD_BLOCK;

#if 0
  println ("const char* getTokenName( int type ) const");
  BEGIN_METHOD_BLOCK;
     println ("if( type > getNumTokens() ) return 0;");
     println ("return " + grammar_->getClassName () + "::tokenNames[type];");
  END_METHOD_BLOCK;
#endif

#if 0
  println ("const char* const* getTokenNames() const");
  BEGIN_METHOD_BLOCK;
     println ("return " + grammar_->getClassName () + "::tokenNames;");
  END_METHOD_BLOCK;
#endif

  END_METHOD_DECL;

  // Generate code for each rule in the grammar
  MGrammar_Grammar::RULES ids = grammar_->rules();
  for (size_t i=0; i<ids.size(); i++)
    {
      MGrammar_GrammarSymbol_var sym = ids[i];
      if (instanceof<MGrammar_RuleSymbol>(sym))
	{
	  MGrammar_RuleSymbol_var rs = Dynamic_Cast<MGrammar_RuleSymbol>(sym);
	  genRuleHeader (rs, rs->references().size () == 0);
	}
      exitIfError ();
    }

  // Generate SmartPtr methods
  BEGIN_METHOD_DECL("public");

    println ("static " + grammar_->getClassName () + "_ptr " + "_nil()");
    BEGIN_METHOD_BLOCK;
      println ("return 0;");
    END_METHOD_BLOCK;

    println ("static bool is_nil(" + grammar_->getClassName () + "_ptr p)");
    BEGIN_METHOD_BLOCK;
      println ("return p==0;");
    END_METHOD_BLOCK;

    println ("static " + grammar_->getClassName () + "_ptr " + "_duplicate(" + grammar_->getClassName() + "_ptr p)");
    BEGIN_METHOD_BLOCK;
      println ("return MLANG_acquire(p);");
    END_METHOD_BLOCK;

  END_METHOD_DECL;

  // RK: when we are using a custom ast override Parser::getAST to return
  // the custom AST type. Ok, this does not work anymore with newer
  // compilers gcc 3.2.x and up. The reference counter is probably
  // getting in the way.
  // So now we just patch the return type back to AST_var
  tabs_ = 0;
  println ("public:");
  tabs_ = 1;
  println ("const " + namespaceAntlr + "AST_var& getAST()");
  println ("{");
  if (usingCustomAST)
    {
      tabs_+=INDENT_BLOCK;
      println ("return " + namespaceAntlr + "AST_var(returnAST);");
      tabs_-=INDENT_BLOCK;
    }
  else
    {
      tabs_++;
      println ("return returnAST;");
      tabs_-=INDENT_BLOCK;
    }
  println ("}");
  println ("");

  tabs_ = 0;
  println ("protected:");
  tabs_ = 1;
  println (labeledElementASTType + " returnAST;");

  // Make the rest private
  tabs_ = 0;
  println ("private:");
  tabs_ = 1;

  // Generate the token names
  println ("static const char* tokenNames[];");
  // and how many there are of them
  _println ("#ifndef NO_STATIC_CONSTS");
  println ("static const int NUM_TOKENS = " +
	   MLANG_String::valueOf(grammar_->tokenManager_->getVocabulary ().size ()) + ";");
  _println ("#else");
  println ("enum {");
  println ("\tNUM_TOKENS = " +
	   MLANG_String::valueOf(grammar_->tokenManager_->getVocabulary ().size ()));
  println ("};");
  _println ("#endif");

  // Generate the bitsets used throughout the grammar
  genBitsetsHeader (bitsetsUsed_, grammar_->tokenManager_->maxTokenType ());

  // Generate the semantic predicate map for debugging
  if (grammar_->debuggingOutput())
    println ("static const char* _semPredNames[];");

  // Close class definition
  tabs_ = 0;
  println ("};");
  println ("");
  if (nameSpace)
    nameSpace->emitClosures (currentOutput_);

  // Generate a guard wrapper
  println ("#endif /*INC_" + grammar_->getClassName () + "_hpp_*/");

  // Close the parser output stream
  currentOutput_->close ();
  currentOutput_ = 0;
}

void
MGrammar_CppCodeGenerator::genInclude (MGrammar_TreeWalkerGrammar_ptr g)
{ 
  ACE_UNUSED_ARG(g);
  // Open the output stream for the parser and set the currentOutput
  outputFile = grammar_->getClassName () + ".h";
  outputLine = 1;
  currentOutput_ = antlrTool_->openOutputFile (outputFile);
  //SAS: changed for proper text file io

  genAST = grammar_->buildAST();
  tabs_ = 0;

  // Generate a guard wrapper
  println ("#ifndef INC_" + grammar_->getClassName () + "_hpp_");
  println ("#define INC_" + grammar_->getClassName () + "_hpp_");
  println ("");
  printHeaderAction (preIncludeHpp);
//  println ("#include <MParser/config.h>");
  println ("#include \"" + grammar_->tokenManager_->getName () +
	   TokenTypesFileSuffix_ + ".h\"");

  // Generate the header common to all output files.
  genHeader (outputFile);

  // Find the name of the super class
  MLANG_String sup;
  if (grammar_->superClass().length() != 0)
    {
      sup = grammar_->superClass();
      println
	("\n// Include correct superclass header with a header statement for example:");
      println ("// header \"post_include_hpp\" {");
      println ("// #include \"" + sup + ".h\"");
      println ("// }");
      println ("// Or....");
      println ("// header {");
      println ("// #include \"" + sup + ".h\"");
      println ("// }\n");
    }
  else
    {
      sup = grammar_->getSuperClass ();
      if (sup.lastIndexOf ('.') != -1)
	sup = sup.substring (sup.lastIndexOf ('.') + 1);
      println ("#include <MParser/" + sup + ".h>");
      sup = namespaceAntlr + sup;
    }
  println ("");

  // Generate header for the parser
  //
  // Do not use printAction because we assume tabs==0
  printHeaderAction (postIncludeHpp);

  if (nameSpace)
    nameSpace->emitDeclarations (currentOutput_);

  printHeaderAction ("");

  // print javadoc comment if any
  if (grammar_->comment().length() != 0)
    {
      _println (grammar_->comment());
    }

  // Generate parser class definition
  print ("class CUSTOM_API " + grammar_->getClassName () + " : " + sup);
  println (", " + grammar_->tokenManager_->getName () + TokenTypesFileSuffix_);

  MParser_Token_var tsuffix = grammar_->options().get ("classHeaderSuffix");
  if (tsuffix)
    {
      MLANG_String suffix =
	MParser_StringUtils::stripFrontBack (tsuffix->getText (), "\"",
					   "\"");
      if (suffix.length() != 0)
	{
	  print (", " + suffix);	// must be an interface name for Java
	}
    }
  println ("{");

  // Generate user-defined parser class members
  if (grammar_->classMemberAction())
    {
      genLineNo (grammar_->classMemberAction()->getLine ());
      print (processActionForSpecialSymbols
	     (grammar_->classMemberAction()->getText (),
	      grammar_->classMemberAction()->getLine (), currentRule, 0));
      genLineNo2 ();
    }

  // Generate default parser class constructor
  tabs_ = 0;
  println ("public:");

  if (noConstructors)
    {
      println ("#if 0");
      println
	("// constructor creation turned of with 'noConstructor' option");
    }
  tabs_ = 1;
  println (grammar_->getClassName () + "();");
  if (noConstructors)
    {
      tabs_ = 0;
      println ("#endif");
      tabs_ = 1;
    }

  // Generate declaration for the initializeFactory method
  println ("static void initializeASTFactory( " + namespaceAntlr +
	   "ASTFactory& factory );");

  println ("int getNumTokens() const");
  BEGIN_BLOCK;
  println ("return " + grammar_->getClassName () + "::NUM_TOKENS;");
  END_BLOCK;
  println ("const char* getTokenName( int type ) const");
  BEGIN_BLOCK;
  println ("if( type > getNumTokens() ) return 0;");
  println ("return " + grammar_->getClassName () + "::tokenNames[type];");
  END_BLOCK;
  println ("const char* const* getTokenNames() const");
  BEGIN_BLOCK;
  println ("return " + grammar_->getClassName () + "::tokenNames;");
  END_BLOCK;

  // Generate code for each rule in the grammar
  MGrammar_Grammar::RULES ids = grammar_->rules();
  MLANG_String ruleNameInits = "";
  for (size_t i=0; i<ids.size(); i++)
    {
      MGrammar_GrammarSymbol_var sym = ids[i];
      if (instanceof<MGrammar_RuleSymbol>(sym))
	{
	  MGrammar_RuleSymbol_var rs = Dynamic_Cast<MGrammar_RuleSymbol>(sym);
	  genRuleHeader (rs, rs->references().size () == 0);
	}
      exitIfError ();
    }
  tabs_ = 0;
  println ("public:");
  tabs_ = 1;
  println ("const " + namespaceAntlr + "AST_var& getAST()");
  println ("{");
  if (usingCustomAST)
    {
      BEGIN_BLOCK;
      println ("return " + namespaceAntlr + "AST_var(returnAST);");
      END_BLOCK;
    }
  else
    {
      BEGIN_BLOCK;
      println ("return returnAST;");
      END_BLOCK;
    }
  println ("}");
  println ("");

  tabs_ = 0;
  println ("protected:");
  tabs_ = 1;
  println (labeledElementASTType + " returnAST;");
  println (labeledElementASTType + " _retTree;");

  // Make the rest private
  tabs_ = 0;
  println ("private:");
  tabs_ = 1;

  // Generate the token names
  println ("static const char* tokenNames[];");
  // and how many there are of them
  _println ("#ifndef NO_STATIC_CONSTS");
  println ("static const int NUM_TOKENS = " +
	   MLANG_String::valueOf(grammar_->tokenManager_->getVocabulary ().size ()) + ";");
  _println ("#else");
  println ("enum {");
  println ("\tNUM_TOKENS = " +
	   MLANG_String::valueOf(grammar_->tokenManager_->getVocabulary ().size ()));
  println ("};");
  _println ("#endif");

  // Generate the bitsets used throughout the grammar
  genBitsetsHeader (bitsetsUsed_, grammar_->tokenManager_->maxTokenType ());

  // Close class definition
  tabs_ = 0;
  println ("};");
  println ("");
  if (nameSpace)
    nameSpace->emitClosures (currentOutput_);

  // Generate a guard wrapper
  println ("#endif /*INC_" + grammar_->getClassName () + "_hpp_*/");

  // Close the parser output stream
  currentOutput_->close ();
  currentOutput_ = 0;
}

	/// for convenience
void
MGrammar_CppCodeGenerator::genASTDeclaration (
   MGrammar_AlternativeElement_ptr el
)

{
  genASTDeclaration (el, labeledElementASTType);
}

	/// for convenience
void
MGrammar_CppCodeGenerator::genASTDeclaration (
   MGrammar_AlternativeElement_ptr el,
   MLANG_String node_type
)

{
  genASTDeclaration (el, el->getLabel (), node_type);
}

/// Generate (if not already done) a declaration for the AST for el.
void
MGrammar_CppCodeGenerator::genASTDeclaration (
   MGrammar_AlternativeElement_ptr el,
   MLANG_String var_name,
   MLANG_String node_type
)
{
  // already declared?
  if (declaredASTVariables.contains (el))
    return;

  MLANG_String init = labeledElementASTInit;

  if (instanceof<MGrammar_GrammarAtom>(el) &&
      (Dynamic_Cast<MGrammar_GrammarAtom>(el))->getASTNodeType ().length() != 0)
    init =
      "Ref" + (Dynamic_Cast<MGrammar_GrammarAtom>(el))->getASTNodeType () + "(" +
      labeledElementASTInit + ")";

  // emit code
  println (node_type + " " + var_name + "_AST = " + init + ";");

  // mark as declared
  declaredASTVariables.put (el, el);
}

void
MGrammar_CppCodeGenerator::genLiteralsTest ()
{
  println ("_ttype = testLiteralsTable(_ttype);");
}

void
MGrammar_CppCodeGenerator::genLiteralsTestForPartialToken ()
{
  println
    ("_ttype = testLiteralsTable(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin),_ttype);");
}

void
MGrammar_CppCodeGenerator::genMatch (const MParser_BitSet& b)
{
   ACE_UNUSED_ARG(b);
}

void
MGrammar_CppCodeGenerator::genMatch (MGrammar_GrammarAtom_ptr atom)
{
  if (instanceof<MGrammar_StringLiteralElement>(atom))
    {
      if ((grammar_->isLexer()))
	{
	  genMatchUsingAtomText (atom);
	}
      else
	{
	  genMatchUsingAtomTokenType (atom);
	}
    }
  else if (instanceof<MGrammar_CharLiteralElement>(atom))
    {
      // Lexer case is handled in the gen( CharLiteralElement x )
      antlrTool_->error ("cannot ref character literals in grammar: " + atom->toString());
    }
  else if (instanceof<MGrammar_TokenRefElement>(atom))
    {
      genMatchUsingAtomTokenType (atom);
    }
  else if (instanceof<MGrammar_WildcardElement>(atom))
    {
      gen (Dynamic_Cast<MGrammar_WildcardElement>(atom));
    }
}
void
MGrammar_CppCodeGenerator::genMatchUsingAtomText (MGrammar_GrammarAtom_ptr atom)
{
  // match() for trees needs the _t cursor
  MLANG_String astArgs = "";
  if ((grammar_->isTreeWalker()))
    {
      if (usingCustomAST)
	astArgs = namespaceAntlr + "AST_var" + "(_t),";
      else
	astArgs = "_t,";
    }

  // if in lexer and ! on element, save buffer index to kill later
  if ((grammar_->isLexer())
      && (!saveText
	  || atom->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG))
    {
      println ("_saveIndex = text_.length();");
    }

  print (atom->getNo() ? "matchNot(" : "match(");
  _print (astArgs);

  // print out what to match
  if (atom->getText().equals ("EOF"))
    {
      // horrible hack to handle EOF case
      _print (namespaceAntlr + "Token::EOF_TYPE");
    }
  else
    {
      if ((grammar_->isLexer()))	// lexer needs special handling
	{
	  MLANG_String cppstring =
	    convertJavaToCppString (atom->getText(), false);
	  _print (cppstring);
	}
      else
	_print (atom->getText());
    }

  _println (");");

  if ((grammar_->isLexer())
      && (!saveText
	  || atom->getAutoGenType () == MGrammar_GrammarElement::AUTO_GEN_BANG))
    {
      println ("text_.erase(_saveIndex);");	// kill text atom put in buffer
    }
}

void
MGrammar_CppCodeGenerator::genMatchUsingAtomTokenType (
   MGrammar_GrammarAtom_ptr atom
)

{
  // match() for trees needs the _t cursor
  MLANG_String astArgs = "";
  if (instanceof <MGrammar_TreeWalkerGrammar> (grammar_))
    {
      if (usingCustomAST)
	astArgs = namespaceAntlr + "AST_var" + "(_t),";
      else
	astArgs = "_t,";
    }

  // If the literal can be mangled, generate the symbolic constant instead
  MLANG_String s = astArgs + getValueString (atom->getType ());

  // matching
  println ((atom->getNo() ? "matchNot(" : "match(") + s + ");");
}

/** Generate the nextToken() rule.
 * nextToken() is a synthetic lexer rule that is the implicit OR of all
 * user-defined lexer rules.
 * @param RuleBlock
 */
void
MGrammar_CppCodeGenerator::genNextToken ()
{
  // Are there any rules?  If not, then just generate a
  // fake nextToken().
  bool hasPublicRules = false;
  for (size_t i = 0; i < grammar_->rules().size (); i++)
    {
      MGrammar_RuleSymbol_var rs = grammar_->rules()[i];
      if (rs->isDefined () && rs->access().equals ("public"))
	{
	  hasPublicRules = true;
	  break;
	}
    }
  if (!hasPublicRules)
    {
      println ("");
      println (namespaceAntlr + "Token_var " + grammar_->getClassName () +
	       "::nextToken() { return " + namespaceAntlr + "(new " +
	       namespaceAntlr + "CommonToken(" + namespaceAntlr +
	       "Token::EOF_TYPE, \"\")); }");
      println ("");
      return;
    }

  // Create the synthesized nextToken() rule
  MGrammar_RuleBlock_var nextTokenBlk =
    MGrammar_MakeGrammar::createNextTokenRule (
          MGrammar_Grammar::_duplicate(grammar_), grammar_->rules(), "nextToken");
  // Define the nextToken rule symbol
  MGrammar_RuleSymbol_var nextTokenRs = new MGrammar_RuleSymbol ("mnextToken");
  nextTokenRs->setDefined ();
  nextTokenRs->setBlock (nextTokenBlk);
  nextTokenRs->access ("private");
  grammar_->define (nextTokenRs);
  // Analyze the nextToken rule
//  bool ok = grammar_->theLLkAnalyzer_->deterministic (nextTokenBlk);
  grammar_->theLLkAnalyzer_->deterministic (nextTokenBlk);

  // Generate the next token rule
  MLANG_String filterRule;
  if ((Dynamic_Cast<MGrammar_LexerGrammar>(grammar_))->filterMode())
    {
      filterRule = (Dynamic_Cast<MGrammar_LexerGrammar>(grammar_))->filterRule();
    }

  println ("");
  println (namespaceAntlr + "Token_var " + grammar_->getClassName () +
	   "::nextToken()");
  BEGIN_BLOCK;
  println (namespaceAntlr + "Token_var theRetToken;");
  println ("for (;;)");
  BEGIN_FOR_BLOCK;
  println (namespaceAntlr + "Token_var theRetToken;");
  println ("int _ttype = " + namespaceAntlr + "Token::INVALID_TYPE;");
  if ((Dynamic_Cast<MGrammar_LexerGrammar>(grammar_))->filterMode())
    {
      println ("setCommitToPath(false);");
      if (filterRule.length())
	{
	  // Here's a good place to ensure that the filter rule actually exists
	  if (!grammar_->
	      isDefined (MGrammar_CodeGenerator::encodeLexerRuleName (filterRule)))
	    {
	      grammar_->antlrTool_->error ("Filter rule " + filterRule +
					   " does not exist in this lexer");
	    }
	  else
	    {
	      MGrammar_RuleSymbol_var rs =
		Dynamic_Cast<MGrammar_RuleSymbol>(grammar_->getSymbol (MGrammar_CodeGenerator::
						  encodeLexerRuleName
						  (filterRule)));
	      if (!rs->isDefined ())
		{
		  grammar_->antlrTool_->error ("Filter rule " + filterRule +
					       " does not exist in this lexer");
		}
	      else if (rs->access().equals ("public"))
		{
		  grammar_->antlrTool_->error ("Filter rule " + filterRule +
					       " must be protected");
		}
	    }
	  println ("int _m;");
	  println ("_m = mark();");
	}
    }
  println ("resetText();");

  // Generate try around whole thing to trap scanner errors
  BEGIN_TRY_BLOCK ("// for lexical and char stream error handling");

  // Test for lexical rules with empty paths
  for (size_t i = 0; i < nextTokenBlk->getAlternatives ().size (); i++)
    {
      MGrammar_Alternative_var a = nextTokenBlk->getAlternativeAt (i);
      if (a->cache_[1].containsEpsilon ())
	{
	  antlrTool_->warning ("found optional path in nextToken()");
	}
    }

  // Generate the block
  MLANG_String newline = SYSTEM->getProperty ("line.separator");
  MGrammar_CppBlockFinishingInfo howToFinish = genCommonBlock (nextTokenBlk, false);
  beginBlockFinish (howToFinish);
  if (howToFinish.viableAction())
   {
     BEGIN_IF("LA(1)==EOF_CHAR");
       println("uponEOF();");
       println("returnToken_ = makeToken(" + namespaceAntlr + "Token::EOF_TYPE);");
     if ((Dynamic_Cast<MGrammar_LexerGrammar>(grammar_))->filterMode())
       {
         if (filterRule.length() == 0)
	   {
             BEGIN_ELSE_BLOCK;
	       println ("consume(); goto tryAgain;");
             END_IF;
	   }
         else
	   {
             BEGIN_ELSE_BLOCK;
               println("commit();");
               BEGIN_TRY_BLOCK("");
	         println("m" + filterRule + "(false);");
               BEGIN_CATCH_BLOCK(namespaceAntlr + "RecognitionException& e");
	         println("// catastrophic failure");
	         println("reportError(e);");
	         println("consume();");
               END_TRY_BLOCK;
	       println("goto tryAgain;");
             END_IF;
	   }
       }
     else
       {
         BEGIN_ELSE_BLOCK;
           println (throwNoViable);
         END_IF;
       }
   }
  endBlockFinish (howToFinish);

  // at this point a valid token has been matched, undo "mark" that was done
  if ((Dynamic_Cast<MGrammar_LexerGrammar>(grammar_))->filterMode() && filterRule.length() != 0)
    {
      println ("commit();");
    }

  // Generate literals test if desired
  // make sure _ttype is set first; note returnToken_ must be
  // non-null as the rule was required to create it.
  BEGIN_IF("!returnToken_");
    println("goto tryAgain; // found SKIP token");
  END_IF;
  println ("_ttype = returnToken_->getType();");
  if ((Dynamic_Cast<MGrammar_LexerGrammar>(grammar_))->getTestLiterals ())
    {
      genLiteralsTest ();
    }

  // return token created by rule reference in switch
  println ("returnToken_->setType(_ttype);");
  println ("return returnToken_;");

  // Close try block
  BEGIN_CATCH_BLOCK (namespaceAntlr + "RecognitionException& e");
  if ((Dynamic_Cast<MGrammar_LexerGrammar>(grammar_))->filterMode())
    {
      if (filterRule.length() == 0)
	{
          BEGIN_IF("!getCommitToPath()");
	    println ("consume();");
	    println ("goto tryAgain;");
	  END_IF;
	}
      else
	{
	  BEGIN_IF("!getCommitToPath()");
	    println ("rewind(_m);");
	    println ("resetText();");
	    BEGIN_TRY_BLOCK ("");
              println("m" + filterRule + "(false);");
	    BEGIN_CATCH_BLOCK (namespaceAntlr + "RecognitionException& ee");
	      println ("	// horrendous failure: error in filter rule");
	      println ("	reportError(ee);");
	      println ("	consume();");
	    END_TRY_BLOCK;
	    // println("goto tryAgain;");
          END_IF;
	  println ("else");
	}
    }
  if (nextTokenBlk->getDefaultErrorHandler ())
    {
      BEGIN_BLOCK;
        println ("reportError(e);");
        println ("consume();");
      END_BLOCK;
    }
  else
    {
      // pass on to invoking routine
      BEGIN_BLOCK;
        println ("throw " + namespaceAntlr +
	         "TokenStreamRecognitionException(e);");
      END_BLOCK;
    }

  // close CharStreamException try
  BEGIN_CATCH_BLOCK (namespaceAntlr + "CharStreamIOException& csie");
    println ("throw " + namespaceAntlr + "TokenStreamIOException(csie.io);");
  BEGIN_CATCH_BLOCK (namespaceAntlr + "CharStreamException& cse");
    println ("throw " + namespaceAntlr +
	     "TokenStreamException(cse.getMessage());");
  END_TRY_BLOCK;

  // close for-loop
  _println ("tryAgain:;");
  END_FOR_BLOCK("");

  // close method nextToken
  END_BLOCK;
  println ("");
}

/** Gen a named rule block.
 * ASTs are generated for each element of an alternative unless
 * the rule or the alternative have a '!' modifier.
 *
 * If an alternative defeats the default tree construction, it
 * must set <rule>_AST to the root of the returned AST.
 *
 * Each alternative that does automatic tree construction, builds
 * up root and child list pointers in an ASTPair structure.
 *
 * A rule finishes by setting the returnAST variable from the
 * ASTPair.
 *
 * @param rule The name of the rule to generate
 * @param startSymbol true if the rule is a start symbol (i.e., not referenced elsewhere)
 */
void
MGrammar_CppCodeGenerator::genRule (
   MGrammar_RuleSymbol_ptr s, bool startSymbol,
   int ruleNum, 
   const MLANG_String& prefix
)

{
  ACE_UNUSED_ARG(startSymbol);
//              tabs=1; // JavaCodeGenerator needs this
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genRule(" + s->getId () + ")");
  if (!s->isDefined ())
    {
      antlrTool_->error ("undefined rule: " + s->getId ());
      return;
    }

  // Generate rule return type, name, arguments
  MGrammar_RuleBlock_var rblk = s->getBlock ();

  currentRule = rblk;
  currentASTResult = s->getId ();

  // clear list of declared ast variables..
  declaredASTVariables.clear ();

  // Save the AST generation state, and set it to that of the rule
  bool savegenAST = genAST;
  genAST = genAST && rblk->getAutoGen ();

  // bool oldsaveTest = saveText;
  saveText = rblk->getAutoGen ();

  // print javadoc comment if any
  if (s->comment().length() != 0)
    {
      _println (s->comment());
    }

  // Gen method return type (note lexer return action set at rule creation)
  if (rblk->returnAction().length())
    {
      // Has specified return value
      _print (extractTypeOfAction
	      (rblk->returnAction(), rblk->getLine (), rblk->getColumn ()) + " ");
    }
  else
    {
      // No specified return value
      _print ("void ");
    }

  // Gen method name
  _print (prefix + s->getId () + "(");

  // Additional rule parameters common to all rules for this grammar
  _print (commonExtraParams);
  if (commonExtraParams.length () != 0 && rblk->argAction().length() != 0)
    {
      _print (",");
    }

  // Gen arguments
  if (rblk->argAction().length() != 0)
    {
      // Has specified arguments
      _println ("");
// FIXME: make argAction also a token? Hmmmmm
//                      genLineNo(rblk);
      tabs_++;

      // Process arguments for default arguments
      // newer gcc's don't accept these in two places (header/cpp)
      //
      // Old appraoch with MLANG_StringBuffer gave trouble with gcj.
      //
      // RK: Actually this breaks with string default arguments containing
      // a comma's or equal signs. Then again the old MLANG_StringBuffer method
      // suffered from the same.
      MLANG_String oldarg = rblk->argAction();
      MLANG_String newarg = "";

      MLANG_String comma = "";
      int eqpos = oldarg.indexOf ('=');
      if (eqpos != -1)
	{
	  int cmpos = 0;
	  while (cmpos != -1 && eqpos != -1)
	    {
	      newarg = newarg + comma + oldarg.substring (0, eqpos).trim ();
	      comma = ", ";
	      cmpos = oldarg.indexOf (',', eqpos);
	      if (cmpos != -1)
		{
		  // cut off part we just handled
		  oldarg = oldarg.substring (cmpos + 1).trim ();
		  eqpos = oldarg.indexOf ('=');
                  if (eqpos == -1 )
                     newarg = newarg + comma + oldarg;
		}
	    }
	}
      else
	newarg = oldarg;

      println (newarg);

//                      println(rblk->argAction);
      tabs_--;
      print (") ");
//                      genLineNo2();   // gcc gives error on the brace... hope it works for the others too
    }
  else
    {
      // No specified arguments
      _print (") ");
    }
  println("");
  BEGIN_BLOCK;

  if (grammar_->traceRules())
    {
      if (instanceof < MGrammar_TreeWalkerGrammar > (grammar_))
	{
	  if (usingCustomAST)
	    println ("Tracer traceInOut(this,\"" + s->getId () + "\"," +
		     namespaceAntlr + "AST_var" + "(_t));");
	  else
	    println ("Tracer traceInOut(this,\"" + s->getId () + "\",_t);");
	}
      else
	{
	  println ("Tracer traceInOut(this, \"" + s->getId () + "\");");
	}
    }

  // Convert return action to variable declaration
  if (rblk->returnAction().length())
    {
      genLineNo (rblk.in());
      println (rblk->returnAction() + ";");
      genLineNo2 ();
    }

  // print out definitions needed by rules for various grammar types
  if (!commonLocalVars.equals (""))
    println (commonLocalVars);

  if (instanceof <MGrammar_LexerGrammar > (grammar_))
    {
      // RK: why is this here? It seems not supported in the rest of the
      // tool.
      // lexer rule default return value is the rule's token name
      // This is a horrible hack to support the built-in EOF lexer rule.
      if (s->getId ().equals ("mEOF"))
	println ("_ttype = " + namespaceAntlr + "Token::EOF_TYPE;");
      else
	println ("_ttype = " + s->getId ().substring (1) + ";");
      println ("int _saveIndex;");	// used for element! (so we can kill text matched for element)
/*
			println("bool old_saveConsumedInput=saveConsumedInput;");
			if ( !rblk->getAutoGen() ) {      // turn off "save input" if ! on rule
				println("saveConsumedInput=false;");
			}
*/
    }

  // if debugging, write code to mark entry to the rule
  if (grammar_->debuggingOutput())
    if (instanceof < MGrammar_ParserGrammar > (grammar_))
      println ("fireEnterRule(" + MLANG_String::valueOf(ruleNum) + ",0);");
    else if (instanceof < MGrammar_LexerGrammar > (grammar_))
      println ("fireEnterRule(" + MLANG_String::valueOf(ruleNum) + ",_ttype);");

  // Generate trace code if desired
//              if ( grammar_->debuggingOutput || grammar_->traceRules) {
//                      println("try { // debugging");
//                      tabs_++;
//              }

  // Initialize AST variables
  if (instanceof < MGrammar_TreeWalkerGrammar > (grammar_))
    {
      // "Input" value for rule
//                      println(labeledElementASTType+" " + s->getId() + "_AST_in = "+labeledElementASTType+"(_t);");
      println (labeledElementASTType + " " + s->getId () +
	       "_AST_in = (_t == ASTNULL) ? " + labeledElementASTInit +
	       " : _t;");
    }
  if (grammar_->buildAST())
    {
      // Parser member used to pass AST returns from rule invocations
      println ("returnAST = " + labeledElementASTInit + ";");
      // Tracks AST construction
      println (namespaceAntlr + "ASTPair currentAST;");	// = new ASTPair();");
      // User-settable return value for rule.
      println (labeledElementASTType + " " + s->getId () + "_AST = " +
	       labeledElementASTInit + ";");
    }

  genBlockPreamble (rblk);
  genBlockInitAction (rblk);
  println ("");

  // Search for an unlabeled exception specification attached to the rule
  MGrammar_ExceptionSpec_var unlabeledUserSpec = rblk->findExceptionSpec ("");

  // Generate try block around the entire rule for  error handling
  if (unlabeledUserSpec || rblk->getDefaultErrorHandler ())
    {
      BEGIN_TRY_BLOCK ("// for error handling");
    }

  // Generate the alternatives
  if (rblk->getAlternatives().size () == 1)
    {
      // One alternative -- use simple form
      MGrammar_Alternative_var alt = rblk->getAlternativeAt (0);
      MLANG_String pred = alt->semPred_;
      if (pred.length() != 0)
	genSemPred (pred, currentRule->getLine());
      if (!MGrammar_SynPredBlock::is_nil(alt->synPred_))
	{
	  antlrTool_->
	    warning ("Syntactic predicate ignored for single alternative",
		     grammar_->getFilename (), alt->synPred_->getLine (),
		     alt->synPred_->getColumn ());
	}
      genAlt (alt, rblk);
    }
  else
    {
      // Multiple alternatives -- generate complex form
//      bool ok = grammar_->theLLkAnalyzer_->deterministic (rblk);
      grammar_->theLLkAnalyzer_->deterministic (rblk);

      MGrammar_CppBlockFinishingInfo howToFinish = genCommonBlock (rblk, false);
      beginBlockFinish (howToFinish);
      if (howToFinish.viableAction())
       {
         println (throwNoViable);
       }
      endBlockFinish (howToFinish);
    }

#if 0
  // Generate catch phrase for error handling
  if (unlabeledUserSpec || rblk->getDefaultErrorHandler ())
   {
//     BEGIN_TRY_BLOCK("");
   }
#endif

  // Generate user-defined or default catch phrases
  if (unlabeledUserSpec)
    {
      genErrorHandler (unlabeledUserSpec);
      // Close the try block
      END_TRY_BLOCK;
    }
  else if (rblk->getDefaultErrorHandler ())
    {
      // Generate default catch phrase
      BEGIN_CATCH_BLOCK (exceptionThrown + "& ex");
      // Generate code to handle error if not guessing
      if (grammar_->hasSyntacticPredicate())
	{
	  BEGIN_IF("inputState_->guessing_ == 0");
	}
      println ("reportError(ex);");
      if (!(instanceof < MGrammar_TreeWalkerGrammar > (grammar_)))
	{
	  // Generate code to consume until token in k==1 follow set
	  MGrammar_Lookahead follow =
	    grammar_->theLLkAnalyzer_->FOLLOW (1, rblk->endNode());
	  MLANG_String followSetName =
	    getBitsetName (markBitsetForGen (follow.fset()));
	  println ("consume();");
	  println ("consumeUntil(" + followSetName + ");");
	}
      else
	{
	  // Just consume one token
	  BEGIN_IF("_t != " + labeledElementASTInit);
	    println ("_t = _t->getNextSibling();");
	  END_IF;
	}
      if (grammar_->hasSyntacticPredicate())
	{
	  // When guessing, rethrow exception
          BEGIN_ELSE_BLOCK;
	    println ("throw;");
	  END_IF;
	}
      // Close the try block
      END_TRY_BLOCK;
    }

  // Squirrel away the AST "return" value
  if (grammar_->buildAST())
    {
      println ("returnAST = " + s->getId () + "_AST;");
    }

  // Set return tree value for tree walkers
  if (instanceof < MGrammar_TreeWalkerGrammar > (grammar_))
    {
      println ("_retTree = _t;");
    }

  // Generate literals test for lexer rules so marked
  if (rblk->getTestLiterals ())
    {
      if (s->access().equals ("protected"))
	{
	  genLiteralsTestForPartialToken ();
	}
      else
	{
	  genLiteralsTest ();
	}
    }

  // if doing a lexer rule, dump code to create token if necessary
  if (instanceof < MGrammar_LexerGrammar > (grammar_))
    {
      BEGIN_IF ("_createToken && " + namespaceAntlr + "Token::is_nil(_token) && " +
	        " _ttype!=" + namespaceAntlr + "Token::SKIP");
        println ("_token = makeToken(_ttype);");
        println ("_token->setText(MLANG_String(text_.getBuffer() + _begin, text_.length()-_begin));");
      END_IF;
      println ("returnToken_ = _token;");
      // It should be easy for an optimizing compiler to realize this does nothing
      // but it avoids the warning about the variable being unused.
      println ("_saveIndex=0;");
    }

  // Gen the return statement if there is one (lexer has hard-wired return action)
  if (rblk->returnAction().length())
    {
      println ("return " +
	       extractIdOfAction (rblk->returnAction(), rblk->getLine (),
				  rblk->getColumn ()) + ";");
    }

//              if ( grammar_->debuggingOutput || grammar_->traceRules) {
////                    tabs_--;
////                    println("} finally { // debugging");
////                    tabs_++;
//
//                      // Generate trace code if desired
//                      if ( grammar_->debuggingOutput)
//                              if (instanceof<ParserGrammar>(grammar))
//                                      println("fireExitRule(" + ruleNum + ",0);");
//                              else if (instanceof<LexerGrammar>(grammar))
//                                      println("fireExitRule(" + ruleNum + ",_ttype);");
//
////                    if (grammar_->traceRules) {
////                            if ( instanceof<TreeWalkerGrammar>(grammar) ) {
////                                    println("traceOut(\""+ s->getId() +"\",_t);");
////                            }
////                            else {
////                                    println("traceOut(\""+ s->getId() +"\");");
////                            }
////                    }
////
////                    tabs_--;
////                    println("}");
//              }

  END_BLOCK;
  println ("");

  // Restore the AST generation state
  genAST = savegenAST;

  // restore char save state
  // saveText = oldsaveTest;
}

void
MGrammar_CppCodeGenerator::genRuleHeader (
   MGrammar_RuleSymbol_ptr s, bool startSymbol)
{
  ACE_UNUSED_ARG(startSymbol);
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("genRuleHeader(" + s->getId () + ")");
  if (!s->isDefined ())
    {
      antlrTool_->error ("undefined rule: " + s->getId ());
      return;
    }

  // Generate rule return type, name, arguments
  MGrammar_RuleBlock_var rblk = s->getBlock ();
  currentRule = rblk;
  currentASTResult = s->getId ();

  // Save the AST generation state, and set it to that of the rule
  bool savegenAST = genAST;
  genAST = genAST && rblk->getAutoGen ();

  // bool oldsaveTest = saveText;
  saveText = rblk->getAutoGen ();

  // Gen method access
  BEGIN_METHOD_DECL(s->access());

  // Gen method return type (note lexer return action set at rule creation)
  if (rblk->returnAction().length())
    {
      // Has specified return value
      print (extractTypeOfAction
	      (rblk->returnAction(), rblk->getLine (), rblk->getColumn ()) + " ");
    }
  else
    {
      // No specified return value
      print ("void ");
    }

  // Gen method name
  _print (s->getId () + "(");

  // Additional rule parameters common to all rules for this grammar
  _print (commonExtraParams);
  if (commonExtraParams.length () != 0 && rblk->argAction().length() != 0)
    {
      _print (",");
    }

  // Gen arguments
  if (rblk->argAction().length() != 0)
    {
      // Has specified arguments
      _println ("");
      tabs_++;
      println (rblk->argAction());
      tabs_--;
      print (")");
    }
  else
    {
      // No specified arguments
      _print (")");
    }
  _println (";");

  END_METHOD_DECL;

  // Restore the AST generation state
  genAST = savegenAST;

  // restore char save state
  // saveText = oldsaveTest;
}

void
MGrammar_CppCodeGenerator::GenRuleInvocation (
   MGrammar_RuleRefElement_ptr rr
)

{
  // dump rule name
  _print (rr->targetRule() + "(");

  // lexers must tell rule if it should set returnToken_
  if ((grammar_->isLexer()))
    {
      // if labeled, could access Token, so tell rule to create
      if (rr->getLabel().length() != 0)
	{
	  _print ("true");
	}
      else
	{
	  _print ("false");
	}
      if (commonExtraArgs.length () != 0 || rr->getArgs().length() != 0)
	{
	  _print (",");
	}
    }

  // Extra arguments common to all rules for this grammar
  _print (commonExtraArgs);
  if (commonExtraArgs.length () != 0 && rr->getArgs().length() != 0)
    {
      _print (",");
    }

  // Process arguments to method, if any
  MGrammar_RuleSymbol_var rs = Dynamic_Cast<MGrammar_RuleSymbol>(grammar_->getSymbol (rr->targetRule()));
  if (rr->getArgs().length() != 0)
    {
      // When not guessing, execute user arg action
      MGrammar_ActionTransInfo_var tInfo = new MGrammar_ActionTransInfo ();
      // FIXME: fix line number passed to processActionForTreeSpecifiers here..
      // this one might be a bit off..
      MLANG_String args = processActionForSpecialSymbols (rr->getArgs(), rr->getLine(),
							  currentRule, tInfo);
      if (tInfo->assignToRoot_ || tInfo->refRuleRoot_.length() != 0)
	{
	  antlrTool_->error ("Arguments of rule reference '" +
			     rr->targetRule() + "' cannot set or ref #" +
			     currentRule->getRuleName () + " on line " +
			     MLANG_String::valueOf(rr->getLine ()));
	}
      _print (args);

      // Warn if the rule accepts no arguments
      if (rs->getBlock()->argAction().length() == 0)
	{
	  antlrTool_->warning ("Rule '" + rr->targetRule() +
			       "' accepts no arguments",
			       grammar_->getFilename (), rr->getLine (),
			       rr->getColumn ());
	}
    }
  else
    {
      // For C++, no warning if rule has parameters, because there may be default
      // values for all of the parameters
      //if (rs->getBlock().argAction != null) {
      //      tool.warning("Missing parameters on reference to rule "+rr->targetRule, rr->getLine());
      //}
    }
  _println (");");

  // move down to the first child while parsing
  if ((grammar_->isTreeWalker ()))
    {
      println ("_t = _retTree;");
    }
}
void
MGrammar_CppCodeGenerator::genSemPred (MLANG_String pred, int line)
{
  // translate $ and # references
  MGrammar_ActionTransInfo_var tInfo = new MGrammar_ActionTransInfo ();
  pred = processActionForSpecialSymbols (pred, line, currentRule, tInfo);
  // ignore translation info...we don't need to do anything with it.
  MLANG_String escapedPred = charFormatter_->escapeString (pred);

  // if debugging, wrap the semantic predicate evaluation in a method
  // that can tell SemanticPredicateListeners the result
  if (grammar_->debuggingOutput()
      && ((instanceof < MGrammar_ParserGrammar > (grammar_))
	  || (instanceof < MGrammar_LexerGrammar > (grammar_))))
    pred = "fireSemanticPredicateEvaluated(antlr.debug.SemanticPredicateEvent.VALIDATING,"	//FIXME
      + MLANG_String::valueOf(addSemPred (escapedPred)) + "," + pred + ")";
  BEGIN_IF("!(" + pred + ")");
  println ("throw " + namespaceAntlr + "SemanticException(\"" + escapedPred +
	   "\");");
  END_IF;
}

/** Write an array of Strings which are the semantic predicate
 *  expressions.  The debugger will reference them by number only
 */
void
MGrammar_CppCodeGenerator::genSemPredMap (MLANG_String prefix)
{
  println ("const char* " + prefix + "_semPredNames[] = {");
  tabs_++;
  for (size_t i=0; i<semPreds.size (); i++)
    println ("\"" + semPreds[i] + "\",");
  println ("0");
  tabs_--;
  println ("};");
}

void
MGrammar_CppCodeGenerator::genSynPred (
   MGrammar_SynPredBlock_ptr blk,
   const MLANG_String& lookaheadExpr
)

{
  if (DEBUG_CODE_GENERATOR || DEBUG_CPP_CODE_GENERATOR)
    SYSTEM->out ()->println ("gen=>(" + blk->toString() + ")");

  // Dump synpred result variable
  println ("bool synPredMatched" + MLANG_String::valueOf(blk->ID()) + " = false;");
  // Gen normal lookahead test
  BEGIN_IF(lookaheadExpr);

  // Save input state
  if (instanceof < MGrammar_TreeWalkerGrammar > (grammar_))
    {
      println (labeledElementType + " __t" + MLANG_String::valueOf(blk->ID()) + " = _t;");
    }
  else
    {
      println ("int _m" + MLANG_String::valueOf(blk->ID()) + " = mark();");
    }

  // Once inside the try, assume synpred works unless exception caught
  println ("synPredMatched" + MLANG_String::valueOf(blk->ID()) + " = true;");
  println ("inputState_->guessing_++;");

  // if debugging, tell listeners that a synpred has started
  if (grammar_->debuggingOutput()
      && ((instanceof < MGrammar_ParserGrammar > (grammar_))
	  || (instanceof < MGrammar_LexerGrammar > (grammar_))))
    {
      println ("fireSyntacticPredicateStarted();");
    }

  syntacticPredLevel++;
  BEGIN_TRY_BLOCK("");
  gen (Dynamic_Cast<MGrammar_AlternativeBlock>(blk));	// gen code to test predicate
  //println("SYSTEM->out()->println(\"pred "+blk+" succeeded\");");
  BEGIN_CATCH_BLOCK (exceptionThrown + "& pe");
    println ("synPredMatched" + MLANG_String::valueOf(blk->ID()) + " = false;");
  //println("SYSTEM->out()->println(\"pred "+blk+" failed\");");
  END_TRY_BLOCK;

  // Restore input state
  if (instanceof < MGrammar_TreeWalkerGrammar > (grammar_))
    {
      println ("_t = __t" + MLANG_String::valueOf(blk->ID()) + ";");
    }
  else
    {
      println ("rewind(_m" + MLANG_String::valueOf(blk->ID()) + ");");
    }

  println ("inputState_->guessing_--;");

  // if debugging, tell listeners how the synpred turned out
  if (grammar_->debuggingOutput()
      && ((instanceof < MGrammar_ParserGrammar > (grammar_))
	  || (instanceof < MGrammar_LexerGrammar > (grammar_))))
    {
      println ("if (synPredMatched" + MLANG_String::valueOf(blk->ID()) + ")");
      println ("  fireSyntacticPredicateSucceeded();");
      println ("else");
      println ("  fireSyntacticPredicateFailed();");
    }

  syntacticPredLevel--;
  // Close lookahead test
  END_IF;

  // Test synpred result
  BEGIN_IF ("synPredMatched" + MLANG_String::valueOf(blk->ID()));
}

/** Generate a static array containing the names of the tokens,
 * indexed by the token type values.  This static array is used
 * to format error messages so that the token identifers or literal
 * strings are displayed instead of the token numbers.
 *
 * If a lexical rule has a paraphrase, use it rather than the
 * token label.
 */
void
MGrammar_CppCodeGenerator::genTokenStrings (const MLANG_String& prefix)
{
  // Generate a string for each token.  This creates a static
  // array of MLANG_Strings indexed by token type.
//              println("");
  println ("const char* " + prefix + "tokenNames[] = {");
  tabs_++;

  // Walk the token vocabulary and generate a Vector of strings
  // from the tokens.
  MGrammar_TokenManager::VOCABULARY v = grammar_->tokenManager_->getVocabulary ();
  for (size_t i = 0; i < v.size (); i++)
    {
      MLANG_String s = v [i];
      if (s.length() == 0)
	{
	  s = "<" + MLANG_String::valueOf (i) + ">";
	}
      if (!s.startsWith ("\"") && !s.startsWith ("<"))
	{
	  MGrammar_TokenSymbol_var ts =
	    grammar_->tokenManager_->getTokenSymbol (s);
	  if (ts && ts->getParaphrase ().length() != 0)
	    {
	      s =
		MParser_StringUtils::stripFrontBack (ts->getParaphrase (), "\"",
						   "\"");
	    }
	}
      print (charFormatter_->literalString (s));
      _println (",");
    }
  println ("0");

  // Close the string array initailizer
  tabs_--;
  println ("};");
}

/** Generate the token types C++ file */
void
MGrammar_CppCodeGenerator::genTokenTypes (MGrammar_TokenManager_ptr tm)
{
  // Open the token output header file and set the currentOutput stream
  outputFile = tm->getName () + TokenTypesFileSuffix_ + ".h";
  outputLine = 1;
  currentOutput_ = antlrTool_->openOutputFile (outputFile);
  //SAS: changed for proper text file io

  tabs_ = 0;

  // Generate a guard wrapper
  println ("#ifndef INC_" + tm->getName () + TokenTypesFileSuffix_ + "_hpp_");
  println ("#define INC_" + tm->getName () + TokenTypesFileSuffix_ + "_hpp_");
  println ("");

  if (nameSpace)
    nameSpace->emitDeclarations (currentOutput_);

  // Generate the header common to all C++ files
  genHeader (outputFile);

  // Encapsulate the definitions in an interface.  This can be done
  // because they are all constants.
  println ("");
  println ("#ifndef CUSTOM_API");
  println ("# define CUSTOM_API");
  println ("#endif");
  println ("");
  // In the case that the .h is included from C source (flexLexer!)
  // we just turn things into a plain enum
  println ("#ifdef __cplusplus");
  println ("struct CUSTOM_API " + tm->getName () + TokenTypesFileSuffix_ +
	   " {");
  println ("#endif");
  tabs_++;
  println ("enum {");
  tabs_++;

  // Generate a definition for each token type
  MGrammar_TokenManager::VOCABULARY v = tm->getVocabulary ();

  // Do special tokens manually
  println ("EOF_ = " + MLANG_String::valueOf(MParser_Token::EOF_TYPE) + ",");

  // Move the other special token to the end, so we can solve
  // the superfluous comma problem easily

  for (size_t i = MParser_Token::MIN_USER_TYPE; i < v.size (); i++)
    {
      MLANG_String s = v[i];
      if (s.length() != 0)
	{
	  if (s.startsWith ("\""))
	    {
	      // a string literal
	      MGrammar_StringLiteralSymbol_var sl =
		Dynamic_Cast<MGrammar_StringLiteralSymbol>(tm->getTokenSymbol (s));
	      if (!sl)
		{
		  antlrTool_->panic ("MLANG_String literal " + s +
				     " not in symbol table");
		}
	      else if (sl->getLabel().length() != 0)
		{
		  println (sl->getLabel() + " = " + MLANG_String::valueOf(i) + ",");
		}
	      else
		{
		  MLANG_String mangledName = mangleLiteral (s);
		  if (mangledName.length() != 0)
		    {
		      // We were able to create a meaningful mangled token name
		      println (mangledName + " = " + MLANG_String::valueOf(i) + ",");
		      // if no label specified, make the label equal to the mangled name
		      sl->setLabel(mangledName);
		    }
		  else
		    {
		      println ("// " + s + " = " + MLANG_String::valueOf(i));
		    }
		}
	    }
	  else if (!s.startsWith ("<"))
	    {
	      println (s + " = " + MLANG_String::valueOf(i) + ",");
	    }
	}
    }

  // Moved from above
  println ("NULL_TREE_LOOKAHEAD = " + Integer::toString(MParser_Token::NULL_TREE_LOOKAHEAD));

  // Close the enum
  tabs_--;
  println ("};");

  // Close the interface
  tabs_--;
  println ("#ifdef __cplusplus");
  println ("};");
  println ("#endif");

  if (nameSpace)
    nameSpace->emitClosures (currentOutput_);

  // Generate a guard wrapper
  println ("#endif /*INC_" + tm->getName () + TokenTypesFileSuffix_ +
	   "_hpp_*/");

  // Close the tokens output file
  currentOutput_->close ();
  currentOutput_ = 0;
  exitIfError ();
}

/** Process a string for an simple expression for use in xx/action.g
 * it is used to cast simple tokens/references to the right type for
 * the generated language. Basically called for every element in
 * the vector to getASTCreateString(vector V)
 * @param str A MLANG_String.
 */
MLANG_String
MGrammar_CppCodeGenerator::processStringForASTConstructor (const MLANG_String& str)
{
  if (usingCustomAST &&
      (((grammar_->isTreeWalker ())) ||
       ((grammar_->isParser ()))) &&
      !(grammar_->tokenManager_->tokenDefined (str)))
    {
//                      SYSTEM->out()->println("processMLANG_StringForASTConstructor: "+str+" with cast");
      return namespaceAntlr + "AST_var(" + str + ")";
    }
  else
    {
//                      SYSTEM->out()->println("processMLANG_StringForASTConstructor: "+str);
      return str;
    }
}

/** Get a string for an expression to generate creation of an AST subtree.
 * @param v A Vector of String, where each element is an expression
 * in the target language yielding an AST node.
 */
MLANG_String
MGrammar_CppCodeGenerator::getASTCreateString (
   const MLANG_Vector<MLANG_String>& v
) const

{
  if (v.size () == 0)
    {
      return "";
    }
  MLANG_StringBuffer buf;
  // the labeledElementASTType here can probably be a cast or nothing
  // in the case of ! usingCustomAST
  buf.append (labeledElementASTType +
	      "(astFactory->make((new " + namespaceAntlr +
	      "ASTArray(" + MLANG_String::valueOf(v.size ()) + "))");
  for (size_t i = 0; i < v.size (); i++)
    {
      buf.append ("->add(" + v [i] + ")");
    }
  buf.append ("))");
  return buf.toString ();
}

/** Get a string for an expression to generate creating of an AST node
 * @param str The arguments to the AST constructor
 */
MLANG_String
MGrammar_CppCodeGenerator::getASTCreateString (
   MGrammar_GrammarAtom_ptr atom,
   const MLANG_String& str
)

{
  if (atom && atom->getASTNodeType ().length() != 0)
    {

      // this atom is using a heterogeneous AST type. (and maybe a local
      // override we can't see at the TokenManager level)
      // make note of the factory needed to generate it..
      // later this is inserted into the initializeFactory method.
      astTypes.ensureCapacity (atom->getType ());
      astTypes.setElementAt (atom->getASTNodeType (), atom->getType ());
      // after above init the factory knows what to generate...
      return "astFactory->create(" + str + ")";
    }
  else
    {
      // FIXME: This is *SO* ugly! but it will have to do for now...
      // 2.7.2 will have better I hope
      // this is due to the usage of getASTCreateString from inside
      // actions/cpp/action.g
      bool is_constructor = false;
      if (str.indexOf (',') != -1)
	is_constructor =
	  grammar_->tokenManager_->tokenDefined (str.
					       substring (0,
							  str.indexOf (',')));

//                      SYSTEM->out()->println("getAstCreateMLANG_String(as): "+str+" "+grammar_->tokenManager.tokenDefined(str));
      if (usingCustomAST &&
	  ((grammar_->isTreeWalker())) &&
	  !(grammar_->tokenManager_->tokenDefined (str)) && !is_constructor)
	return "astFactory->create(" + namespaceAntlr + "AST_var(" + str +
	  "))";
      else
	return "astFactory->create(" + str + ")";
    }
}

/** Get a string for an expression to generate creating of an AST node
 * @param str The arguments to the AST constructor
 */
MLANG_String
MGrammar_CppCodeGenerator::getASTCreateString (
   const MLANG_String& str
) const

{
//              SYSTEM->out()->println("getAstCreateMLANG_String(str): "+str+" "+grammar_->tokenManager.tokenDefined(str));
  if (usingCustomAST)
    return labeledElementASTType + "(astFactory->create(" + namespaceAntlr +
      "AST_var(" + str + ")))";
  else
    return "astFactory->create(" + str + ")";
}

MLANG_String
MGrammar_CppCodeGenerator::getLookaheadTestExpression (
   const MLANG_Vector<MGrammar_Lookahead>& look,
   int k
)

{
  MLANG_StringBuffer e;
  bool first = true;

  e.append ("(");
  for (int i = 1; i <= k; i++)
    {
      MParser_BitSet p = look[i].fset();
      if (!first)
	{
	  e.append (") && (");
	}
      first = false;

      // Syn preds can yield <end-of-syn-pred> (epsilon) lookahead.
      // There is no way to predict what that token would be.  Just
      // allow anything instead.
      if (look[i].containsEpsilon ())
	{
	  e.append ("true");
	}
      else
	{
	  e.append (getLookaheadTestTerm (i, p));
	}
    }
  e.append (")");

  return e.toString ();
}

/** Generate a lookahead test expression for an alternate.  This
 * will be a series of tests joined by '&&' and enclosed by '()',
 * the number of such tests being determined by the depth of the lookahead.
 */
MLANG_String
MGrammar_CppCodeGenerator::getLookaheadTestExpression (
   MGrammar_Alternative_ptr alt, int maxDepth)
{
  int depth = alt->lookaheadDepth_;
  if (depth == MGrammar_GrammarAnalyzer::NONDETERMINISTIC)
    {
      // if the decision is nondeterministic, do the best we can: LL(k)
      // any predicates that are around will be generated later.
      depth = grammar_->maxk();
    }

  if (maxDepth == 0)
    {
      // empty lookahead can result from alt with sem pred
      // that can see end of token.  E.g., A : {pred}? ('a')? ;
      return "true";
    }

/*
bool first = true;
		for (int i=1; i<=depth && i<=maxDepth; i++) {
			BitSet p = alt->cache[i].fset;
			if (!first) {
				e.append(") && (");
			}
			first = false;

			// Syn preds can yield <end-of-syn-pred> (epsilon) lookahead.
			// There is no way to predict what that token would be.  Just
			// allow anything instead.
			if ( alt->cache[i].containsEpsilon() ) {
				e.append("true");
			}
			else {
				e.append(getLookaheadTestTerm(i, p));
			}
		}

		e.append(")");
*/

  return "(" + getLookaheadTestExpression (alt->cache_, depth) + ")";
}

/**Generate a depth==1 lookahead test expression given the BitSet.
 * This may be one of:
 * 1) a series of 'x==X||' tests
 * 2) a range test using >= && <= where possible,
 * 3) a bitset membership test for complex comparisons
 * @param k The lookahead level
 * @param p The lookahead set for level k
 */
MLANG_String
MGrammar_CppCodeGenerator::getLookaheadTestTerm (int k, const MParser_BitSet& p)
{
  // Determine the name of the item to be compared
  MLANG_String ts = lookaheadString (k);

  // Generate a range expression if possible
  MLANG_Vector<size_t> elems = p.toArray ();
  if (elementsAreRange (elems))
    {
      return getRangeExpression (k, elems);
    }

  // Generate a bitset membership test if possible
  MLANG_StringBuffer e;
  int degree = p.degree ();
  if (degree == 0)
    {
      return "true";
    }

  if (degree >= bitsetTestThreshold_)
    {
      int bitsetIdx = markBitsetForGen (p);
      return getBitsetName (bitsetIdx) + ".member(" + ts + ")";
    }

  // Otherwise, generate the long-winded series of "x==X||" tests
  e = MLANG_StringBuffer ();
  for (size_t i = 0; i < elems.size(); i++)
    {
      // Get the compared-to item (token or character value)
      MLANG_String cs = getValueString (elems[i]);

      // Generate the element comparison
      if (i > 0)
	e.append (" || ");
      e.append (ts);
      e.append (" == ");
      e.append (cs);
    }
  return e.toString ();
}

/** Return an expression for testing a contiguous renage of elements
 * @param k The lookahead level
 * @param elems The elements representing the set, usually from BitSet.toArray().
 * @return MLANG_String containing test expression.
 */
MLANG_String
MGrammar_CppCodeGenerator::getRangeExpression (int k, const MLANG_Vector<size_t>& elems)
{
  if (!elementsAreRange (elems))
    {
      antlrTool_->panic ("getRangeExpression called with non-range");
    }
  int begin = elems[0];
  int end = elems[elems.size() - 1];
  return
    "(" + lookaheadString (k) + " >= " + getValueString (begin) +
    " && " + lookaheadString (k) + " <= " + getValueString (end) +
    ")";
}

/** getValueMLANG_String: get a string representation of a token or char value
 * @param value The token or char value
 */
MLANG_String
MGrammar_CppCodeGenerator::getValueString (int value)
{
  MLANG_String cs;
  if ((grammar_->isLexer()))
    {
      cs = charFormatter_->literalChar (value);
    }
  else
    {
      MGrammar_TokenSymbol_var ts = grammar_->tokenManager_->getTokenSymbolAt (value);
      if (!ts)
	{
	  return Integer::toString(value);	// return token type as string
	  // tool.panic("vocabulary for token type " + value + " is null");
	}
      MLANG_String tId = ts->getId ();
      if (instanceof<MGrammar_StringLiteralSymbol>(ts))
	{
	  // if string literal, use predefined label if any
	  // if no predefined, try to mangle into LITERAL_xxx.
	  // if can't mangle, use int value as last resort
	  MGrammar_StringLiteralSymbol_var sl = Dynamic_Cast<MGrammar_StringLiteralSymbol>(ts);
	  MLANG_String label = sl->getLabel ();
	  if (label.length() != 0)
	    {
	      cs = label;
	    }
	  else
	    {
	      cs = mangleLiteral (tId);
	      if (cs.length() == 0)
		{
		  cs = MLANG_String::valueOf (value);
		}
	    }
	}
      else
	{
	  if (tId.equals ("EOF"))
	    cs = namespaceAntlr + "Token::EOF_TYPE";
	  else
	    cs = tId;
	}
    }
  return cs;
}

/**Is the lookahead for this alt empty? */
bool
MGrammar_CppCodeGenerator::lookaheadIsEmpty (
   MGrammar_Alternative_ptr alt, int maxDepth)
{
  int depth = alt->lookaheadDepth_;
  if (depth == MGrammar_GrammarAnalyzer::NONDETERMINISTIC)
    {
      depth = grammar_->maxk();
    }
  for (int i = 1; i <= depth && i <= maxDepth; i++)
    {
      const MParser_BitSet& p = alt->cache_[i].fset();
      if (p.degree () != 0)
	{
	  return false;
	}
    }
  return true;
}

MLANG_String
MGrammar_CppCodeGenerator::lookaheadString (int k)
{
  if ((grammar_->isTreeWalker()))
    {
      return "_t->getType()";
    }
  return "LA(" + MLANG_String::valueOf(k) + ")";
}

/** Mangle a string literal into a meaningful token name.  This is
 * only possible for literals that are all characters.  The resulting
 * mangled literal name is literalsPrefix with the text of the literal
 * appended.
 * @return A string representing the mangled literal, or null if not possible.
 */
MLANG_String
MGrammar_CppCodeGenerator::mangleLiteral (const MLANG_String& s)
{
  MLANG_String mangled = antlrTool_->getLiteralsPrefix();
  for (size_t i = 1; i < s.length () - 1; i++)
    {
      if (!MLANG_Character::isLetter (s.charAt (i)) && s.charAt (i) != '_')
	{
	  return "";
	}
      mangled += s.charAt (i);
    }
  if (antlrTool_->getUpperCaseMangledLiterals())
    {
      mangled = mangled.toUpperCase ();
    }
  return mangled;
}

/** Map an identifier to it's corresponding tree-node variable.
 * This is context-sensitive, depending on the rule and alternative
 * being generated
 * @param idParam The identifier name to map
 * @return The mapped id (which may be the same as the input), or null if the mapping is invalid due to duplicates
 */
MLANG_String
MGrammar_CppCodeGenerator::mapTreeId (
   const MLANG_String& idParam, 
   MGrammar_ActionTransInfo_ptr transInfo
)

{
  // if not in an action of a rule, nothing to map.
  if (!currentRule)
    return idParam;
//              SYSTEM->out()->print("mapTreeId: "+idParam+" "+currentRule.getRuleName()+" ");

  bool in_var = false;
  MLANG_String id = idParam;
  if ((grammar_->isTreeWalker()))
    {
//                      RK: hmmm this seems odd. If buildAST is false it translates
//                      #rulename_in to 'rulename_in' else to 'rulename_AST_in' which indeed
//                      exists. disabling for now.. and hope it doesn't blow up somewhere.
      if (!grammar_->buildAST())
	{
	  in_var = true;
//                              SYSTEM->out()->println("in_var1");
	}
      // If the id ends with "_in", then map it to the input variable
//                      else
      if (id.length () > 3 && id.lastIndexOf ("_in") == (int)id.length () - 3)
	{
	  // Strip off the "_in"
	  id = id.substring (0, id.length () - 3);
	  in_var = true;
//                              SYSTEM->out()->println("in_var2");
	}
    }
//              SYSTEM->out()->print(in_var+"\t");

  // Check the rule labels.  If id is a label, then the output
  // variable is label_AST, and the input variable is plain label.
  for (size_t i = 0; i < currentRule->labeledElements().size (); i++)
    {
      MGrammar_AlternativeElement_var elt =
	 currentRule->labeledElements() [i];
      if (elt->getLabel ().equals (id))
	{
//                              if( in_var )
//                                      SYSTEM->out()->println("returning (vec) "+(in_var ? id : id + "_AST"));
	  return in_var ? id : id + "_AST";
	}
    }

  // Failing that, check the id-to-variable map for the alternative.
  // If the id is in the map, then output variable is the name in the
  // map, and input variable is name_in
  MLANG_String s = treeVariableMap.get (id);
  if (s.length() != 0)
    {
      if (s == NONUNIQUE)
	{
//                              if( in_var )
//                                      SYSTEM->out()->println("returning null (nonunique)");
	  // There is more than one element with this id
	  antlrTool_->error ("Ambiguous reference to AST element " + id +
			     " in rule " + currentRule->getRuleName ());
	  return "";
	}
      else if (s.equals (currentRule->getRuleName ()))
	{
	  // a recursive call to the enclosing rule is
	  // ambiguous with the rule itself.
//                              if( in_var )
//                                      SYSTEM->out()->println("returning null (rulename)");
	  antlrTool_->error ("Ambiguous reference to AST element " + id +
			     " in rule " + currentRule->getRuleName ());
	  return "";
	}
      else
	{
//                              if( in_var )
//                              SYSTEM->out()->println("returning "+(in_var?s+"_in":s));
	  return in_var ? s + "_in" : s;
	}
    }

//              SYSTEM->out()->println("Last check: "+id+" == "+currentRule.getRuleName());
  // Failing that, check the rule name itself.  Output variable
  // is rule_AST; input variable is rule_AST_in (treeparsers).
  if (id.equals (currentRule->getRuleName ()))
    {
      MLANG_String r = in_var ? id + "_AST_in" : id + "_AST";
      if (transInfo)
	{
	  if (!in_var)
	    {
	      transInfo->refRuleRoot_ = r;
	    }
	}
//                      if( in_var )
//                              SYSTEM->out()->println("returning (r) "+r);
      return r;
    }
  else
    {
//                      if( in_var )
//                      SYSTEM->out()->println("returning (last) "+id);
      // id does not map to anything -- return itself.
      return id;
    }
}

/** Given an element and the name of an associated AST variable,
 * create a mapping between the element "name" and the variable name.
 */
void
MGrammar_CppCodeGenerator::mapTreeVariable (
   MGrammar_AlternativeElement_ptr e,
   const MLANG_String& name
)

{
  // For tree elements, defer to the root
  if (instanceof<MGrammar_TreeElement>(e))
    {
      mapTreeVariable ((Dynamic_Cast<MGrammar_TreeElement>(e))->root(), name);
      return;
    }

  // Determine the name of the element, if any, for mapping purposes
  MLANG_String elName;

  // Don't map labeled items
  if (e->getLabel ().length() == 0)
    {
      if (instanceof<MGrammar_TokenRefElement>(e))
	{
	  // use the token id
	  elName = (Dynamic_Cast<MGrammar_TokenRefElement>(e))->getText();
	}
      else if (instanceof<MGrammar_RuleRefElement>(e))
	{
	  // use the rule name
	  elName = (Dynamic_Cast<MGrammar_RuleRefElement>(e))->targetRule();
	}
    }
  // Add the element to the tree variable map if it has a name
  if (elName.length())
    {
      if (treeVariableMap.get (elName).length())
	{
	  // Name is already in the map -- mark it as duplicate
	  treeVariableMap.remove (elName);
	  treeVariableMap.put (elName, NONUNIQUE);
	}
      else
	{
	  treeVariableMap.put (elName, name);
	}
    }
}

/** Lexically process tree-specifiers in the action.
 * This will replace #id and #(...) with the appropriate
 * function calls and/or variables.
 */
MLANG_String
MGrammar_CppCodeGenerator::
processActionForSpecialSymbols (
   const MLANG_String& action,
   int line,
   MGrammar_RuleBlock_ptr currentRule,
   MGrammar_ActionTransInfo_ptr tInfo
)

{
  MLANG_String actionStr = action;
  fprintf(stderr,"processActionForSpecialSymbols: action=%s\n", actionStr.c_str());
  if (actionStr.length () == 0)
    return "";

  // The action trans info tells us (at the moment) whether an
  // assignment was done to the rule's tree root()->
  if (grammar_ == 0)
    return actionStr;

  if ((grammar_->buildAST() && actionStr.indexOf ('#') != -1) ||
      (grammar_->isTreeWalker ()) ||
      (((grammar_->isLexer ()) ||
	(grammar_->isParser ())) && actionStr.indexOf ('$') != -1))
    {
      // Create a lexer to read an action and return the translated version
      MGrammar_CppActionLexer_var lexer =
	new MGrammar_CppActionLexer (actionStr, 
                                     MGrammar_RuleBlock::_duplicate(currentRule),
                                     this,
				     tInfo);
      lexer->setLineOffset (line);
      lexer->setFilename (grammar_->getFilename ());
      lexer->setTool (antlrTool_);

      try
      {
	lexer->mACTION (true);
	actionStr = lexer->getTokenObject ()->getText ();
	SYSTEM->out()->println("action translated: "+actionStr);
	SYSTEM->out()->println("trans info is "+tInfo->toString());
      }
      catch (MParser_RecognitionException & ex)
      {
	lexer->reportError (ex);
	return actionStr;
      }
      catch (MParser_TokenStreamException& tex)
      {
	antlrTool_->panic ("Error reading action:" + actionStr);
	return actionStr;
      }
      catch (MParser_CharStreamException& io)
      {
	antlrTool_->panic ("Error reading action:" + actionStr);
	return actionStr;
      }
    }
  return actionStr;
}

MLANG_String
MGrammar_CppCodeGenerator::fixNameSpaceOption (const MLANG_String& ns)
{
  MLANG_String result = MParser_StringUtils::stripFrontBack (ns, "\"", "\"");
  if (result.length () > 2 &&
      !result.substring (result.length () - 2, result.length ()).equals ("::"))
    result += "::";
  return result;
}

void
MGrammar_CppCodeGenerator::setupGrammarParameters (MGrammar_Grammar_ptr g)
{
  if ((g->isParser()) ||
      (g->isLexer()) ||
      (g->isTreeWalker()))
    {
      /* RK: options also have to be added to Grammar.java and for options
       * on the file level entries have to be defined in
       * DefineGrammarSymbols.java and passed around via 'globals' in
       * antlrTool_->java
       */
      if (antlrTool_->getNameSpace())
	nameSpace = antlrTool_->getNameSpace();

      if (antlrTool_->getNamespaceStd().length() != 0)
	namespaceStd = fixNameSpaceOption (antlrTool_->getNamespaceStd());

      if (antlrTool_->getNamespaceAntlr().length() != 0)
	namespaceAntlr = fixNameSpaceOption (antlrTool_->getNamespaceAntlr());

      genHashLines = antlrTool_->getGenHashLines();

      /* let grammar level options override filelevel ones...
       */
      if (g->hasOption ("namespace"))
	{
	  MParser_Token_var t = g->getOption ("namespace");
	  if (t)
	    {
	      nameSpace = new MGrammar_NameSpace (t->getText ());
	    }
	}
      if (g->hasOption ("namespaceAntlr"))
	{
	  MParser_Token_var t = g->getOption ("namespaceAntlr");
	  if (t)
	    {
	      MLANG_String ns =
		MParser_StringUtils::stripFrontBack (t->getText (), "\"", "\"");
	      if (ns.length() != 0)
		{
		  if (ns.length () > 2 &&
		      !ns.substring (ns.length () - 2,
				     ns.length ()).equals ("::"))
		    ns += "::";
		  namespaceAntlr = ns;
		}
	    }
	}
      if (g->hasOption ("namespaceStd"))
	{
	  MParser_Token_var t = g->getOption ("namespaceStd");
	  if (t)
	    {
	      MLANG_String ns =
		MParser_StringUtils::stripFrontBack (t->getText (), "\"", "\"");
	      if (ns.length() != 0)
		{
		  if (ns.length () > 2 &&
		      !ns.substring (ns.length () - 2,
				     ns.length ()).equals ("::"))
		    ns += "::";
		  namespaceStd = ns;
		}
	    }
	}
      if (g->hasOption ("genHashLines"))
	{
	  MParser_Token_var t = g->getOption ("genHashLines");
	  if (t)
	    {
	      MLANG_String val =
		MParser_StringUtils::stripFrontBack (t->getText (), "\"", "\"");
	      genHashLines = val.equals ("true");
	    }
	}
      noConstructors = antlrTool_->noConstructors();	// get the default
      if (g->hasOption ("noConstructors"))
	{
	  MParser_Token_var t = g->getOption ("noConstructors");
	  if ((t)
	      && !(t->getText ().equals ("true")
		   || t->getText ().equals ("false")))
	    antlrTool_->error ("noConstructors option must be true or false",
			       antlrTool_->grammarFile (), t->getLine (),
			       t->getColumn ());
	  noConstructors = t->getText ().equals ("true");
	}
    }
  if ((g->isParser ()))
    {
      labeledElementASTType = namespaceAntlr + "AST_var";
      labeledElementASTInit = namespaceAntlr + "nullAST";
      if (g->hasOption ("ASTLabelType"))
	{
	  MParser_Token_var tsuffix = g->getOption ("ASTLabelType");
	  if (tsuffix)
	    {
	      MLANG_String suffix =
		MParser_StringUtils::stripFrontBack (tsuffix->getText (), "\"",
						   "\"");
	      if (suffix.length () != 0)
		{
		  usingCustomAST = true;
		  labeledElementASTType = suffix;
		  labeledElementASTInit =
		    suffix + "(" + namespaceAntlr + "nullAST)";
		}
	    }
	}
      labeledElementType = namespaceAntlr + "Token_var ";
      labeledElementInit = namespaceAntlr + "nullToken";
      commonExtraArgs = "";
      commonExtraParams = "";
      commonLocalVars = "";
      lt1Value = "LT(1)";
      exceptionThrown = namespaceAntlr + "RecognitionException";
      throwNoViable =
	"throw " + namespaceAntlr +
	"NoViableAltException(LT(1), getFilename());";
    }
  else if ((g->isLexer ()))
    {
      labeledElementType = "char ";
      labeledElementInit = "'\\0'";
      commonExtraArgs = "";
      commonExtraParams = "bool _createToken";
      commonLocalVars =
	"int _ttype; " + namespaceAntlr +
	"Token_var _token; int _begin=text_.length();";
      lt1Value = "LA(1)";
      exceptionThrown = namespaceAntlr + "RecognitionException";
      throwNoViable =
	"throw " + namespaceAntlr +
	"NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());";
    }
  else if ((g->isTreeWalker ()))
    {
      labeledElementInit = namespaceAntlr + "nullAST";
      labeledElementASTInit = namespaceAntlr + "nullAST";
      labeledElementASTType = namespaceAntlr + "AST_var";
      labeledElementType = namespaceAntlr + "AST_var";
      commonExtraParams = namespaceAntlr + "AST_var _t";
      throwNoViable = "throw " + namespaceAntlr + "NoViableAltException(_t);";
      lt1Value = "_t";
      if (g->hasOption ("ASTLabelType"))
	{
	  MParser_Token_var tsuffix = g->getOption ("ASTLabelType");
	  if (tsuffix)
	    {
	      MLANG_String suffix =
		MParser_StringUtils::stripFrontBack (tsuffix->getText (), "\"",
						   "\"");
	      if (suffix.length () != 0)
		{
		  usingCustomAST = true;
		  labeledElementASTType = suffix;
		  labeledElementType = suffix;
		  labeledElementInit =
		    suffix + "(" + namespaceAntlr + "nullAST)";
		  labeledElementASTInit = labeledElementInit;
		  commonExtraParams = suffix + " _t";
		  throwNoViable =
		    "throw " + namespaceAntlr + "NoViableAltException(" +
		    namespaceAntlr + "AST_var(_t));";
		  lt1Value = "_t";
		}
	    }
	}
      if (!g->hasOption ("ASTLabelType"))
	{
	  g->setOption ("ASTLabelType",
			new MParser_Token (ANTLRTokenTypes::STRING_LITERAL,
					   namespaceAntlr + "AST_var"));
	}
      commonExtraArgs = "_t";
      commonLocalVars = "";
      exceptionThrown = namespaceAntlr + "RecognitionException";
    }
  else
    {
      antlrTool_->panic ("Unknown grammar type");
    }
}

MGrammar_CppCodeGenerator::~MGrammar_CppCodeGenerator ()
{
}
