#ifndef __MGRAMMAR_CPPCODEGENERATOR_H__
#define __MGRAMMAR_CPPCODEGENERATOR_H__
#include "MLANG/String.h"
#include "MLANG/HashMap.h"
#include "MLANG/Vector_T.h"
#include "MParser/BitSet.h"
#include "MParser/Token.h"
#include "MIO/PrintWriter.h"
#include "MIO/IOException.h"
#include "MIO/FileWriter.h"
#include "MGrammar/Lookahead.h"
#include "MGrammar/NameSpace.h"
#include "MGrammar/TokenManager.h"
#include "MGrammar/CodeGenerator.h"
#include "MGrammar/CppBlockFinishingInfo.h"

class MGrammar_CppCodeGenerator;
typedef MGrammar_CppCodeGenerator *MGrammar_CppCodeGenerator_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/CppCodeGenerator.java#2 $
 */

// C++ code generator by Pete Wells: pete@yamuna.demon.co.uk
// #line generation contributed by: Ric Klaren <klaren@cs.utwente.nl>

/** Generate MyParser.cpp, MyParser.hpp, MyLexer.cpp, MyLexer.hpp
 * and MyParserTokenTypes.hpp
 */
class MGrammar_CppCodeGenerator : public MGrammar_CodeGenerator
{
   typedef MGrammar_CodeGenerator super;
public:
   bool DEBUG_CPP_CODE_GENERATOR;
   // non-zero if inside syntactic predicate generation
protected:
   int syntacticPredLevel;

   // Are we generating ASTs (for parsers and tree parsers); right now?
protected:
   bool genAST;

   // Are we saving the text consumed (for lexers); right now?
protected:
   bool saveText;

   // Generate #line's
protected:
   bool genHashLines;
   // Generate constructors or not
protected:
   bool noConstructors;

   // Used to keep track of lineno in output
protected:
   int outputLine;
protected:
   MLANG_String outputFile;

   // Grammar parameters set up to handle different grammar classes.
   // These are used to get instanceof tests out of code generation
   bool usingCustomAST;
   MLANG_String labeledElementType;
   MLANG_String labeledElementASTType; // mostly the same as labeledElementType except in parsers
   MLANG_String labeledElementASTInit;
   MLANG_String labeledElementInit;
   MLANG_String commonExtraArgs;
   MLANG_String commonExtraParams;
   MLANG_String commonLocalVars;
   MLANG_String lt1Value;
   MLANG_String exceptionThrown;
   MLANG_String throwNoViable;

   // Tracks the rule being generated.  Used for mapTreeId
   MGrammar_RuleBlock_ptr currentRule;
   // Tracks the rule or labeled subrule being generated.  Used for AST generation.
   MLANG_String currentASTResult;
   // Mapping between the ids used in the current alt, and the
   // names of variables used to represent their AST values.
   typedef MLANG_HashMap<MLANG_String, MLANG_String> TREEVARIABLEMAP;
   TREEVARIABLEMAP treeVariableMap;

   /** Used to keep track of which AST variables have been defined in a rule
    * (except for the #rule_name and #rule_name_in var's
    */
   typedef MLANG_HashMap<MGrammar_AlternativeElement_ptr, MGrammar_AlternativeElement_ptr> DECLAREDASTVARIABLES;
   DECLAREDASTVARIABLES declaredASTVariables;

   // Count of unnamed generated variables
   int astVarNumber;
   // Special value used to mark duplicate in treeVariableMap
protected:
   static MLANG_String NONUNIQUE;

public:
   static int caseSizeThreshold; // ascii is max

private:
   typedef MLANG_Vector<MLANG_String> SEMPREDS;
   SEMPREDS semPreds;

   // Used to keep track of which (heterogeneous AST types are used);
   // which need to be set in the ASTFactory of the generated parser
private:
   typedef MLANG_Vector<MLANG_String> ASTTYPES;
   ASTTYPES astTypes;

private:
   static MLANG_String namespaceStd;
private:
   static MLANG_String namespaceAntlr;
private:
   static MGrammar_NameSpace_var nameSpace;
private:
   static MLANG_String preIncludeCpp;
private:
   static MLANG_String preIncludeHpp;
private:
   static MLANG_String postIncludeCpp;
private:
   static MLANG_String postIncludeHpp;

   /** Create a C++ code-generator using the given Grammar.
    * The caller must still call setTool, setBehavior, and setAnalyzer
    * before generating code.
    */
public:
   MGrammar_CppCodeGenerator();

   virtual ~MGrammar_CppCodeGenerator();

public:

   static bool is_nil (MGrammar_CppCodeGenerator_ptr gen)
   { return gen == 0; }

   static MGrammar_CppCodeGenerator_ptr _nil()
   { return 0; }

   static MGrammar_CppCodeGenerator_ptr _duplicate(MGrammar_CppCodeGenerator_ptr gen)
   { return MLANG_acquire(gen); }

public:

   DECLARE_CLASS_INFO();

protected:
   /** Adds a semantic predicate string to the sem pred vector
       These strings will be used to build an array of sem pred names
       when building a debugging parser.  This method should only be
       called when the debug option is specified
    */
   int addSemPred(const MLANG_String& predicate);

public:
   void exitIfError();
protected:
   int countLines( const MLANG_String& s );
protected:
   /** Output a MLANG_String to the currentOutput stream.
    * Ignored if string is null.
    * @param s The string to output
    */
   void _print(const MLANG_String& s);
protected:
   /** Print an action without leading tabs, attempting to
    * preserve the current indentation level for multi-line actions
    * Ignored if string is null.
    * @param s The action string to output
    */
public:
   void _printAction(const MLANG_String& s);
   /** Print an action stored in a token surrounded by #line stuff */
public:
   void printAction(MParser_Token_ptr t);
   /** Print a header action by #line stuff also process any tree construction
    * @param name The name of the header part
    */
   void printHeaderAction(MLANG_String name);
protected:
   /** Output a MLANG_String followed by newline, to the currentOutput stream.
    * Ignored if string is null.
    * @param s The string to output
    */
   void _println(const MLANG_String& s);

protected:
   /** Output tab indent followed by a MLANG_String followed by newline,
    * to the currentOutput stream.  Ignored if string is null.
    * @param s The string to output
    */
   void println(const MLANG_String& s);

public:
   /** Generate a #line or // line depending on options */
   void genLineNo(int line);

public:
   /** Generate a #line or // line depending on options */
   void genLineNo(MGrammar_GrammarElement_ptr el);
public:
   /** Generate a #line or // line depending on options */
   void genLineNo(MParser_Token_ptr t);
public:
   /** Generate a #line or // line depending on options */
   void genLineNo2();
private:
   /// Bound safe isDigit
   bool charIsDigit( const MLANG_String& s, int i );
private:
   /** Normalize a string coming from antlr's lexer. E.g. translate java
    * escapes to values. Check their size (multibyte); bomb out if they are
    * multibyte (bit crude);. Then reescape to C++ style things.
    * Used to generate strings for match(); and matchRange();
    * @param lit the literal string
    * @param isCharLiteral if it's for a character literal
    *                       (enforced to be one length); and enclosed in '
    * FIXME: bombing out on mb chars. Should be done in Lexer.
    * FIXME: this is another horrible hack.
    * FIXME: life would be soooo much easier if the stuff from the lexer was
    * normalized in some way.
    */
   MLANG_String convertJavaToCppString( 
      const MLANG_String& lit,
      bool isCharLiteral
    );
public:
   /** Generate the parser, lexer, treeparser, and token types in C++
    */
   void gen();

public:
   /** Generate code for the given grammar element.
    * @param blk The {...} action to generate
    */
   void gen(MGrammar_ActionElement_ptr action);

   /** Generate code for the given grammar element.
    * @param blk The "x|y|z|..." block to generate
    */
public:
   void gen(MGrammar_AlternativeBlock_ptr blk);

   /** Generate code for the given grammar element.
    * @param blk The block-end element to generate.  Block-end
    * elements are synthesized by the grammar parser to represent
    * the end of a block.
    */
public:
   void gen(MGrammar_BlockEndElement_ptr end);

   /** Generate code for the given grammar element.
    * Only called from lexer grammars.
    * @param blk The character literal reference to generate
    */
public:
   void gen(MGrammar_CharLiteralElement_ptr atom);

   /** Generate code for the given grammar element.
    * Only called from lexer grammars.
    * @param blk The character-range reference to generate
    */
public:
   void gen(MGrammar_CharRangeElement_ptr r);

   /** Generate the lexer C++ files */
public:
    void gen(MGrammar_LexerGrammar_ptr g);
   
   /** Generate code for the given grammar element.
    * @param blk The (...);+ block to generate
    */
public:
   void gen(MGrammar_OneOrMoreBlock_ptr blk);


public:
   /** Generate the parser C++ file */
   void gen(MGrammar_ParserGrammar_ptr g);
public:
   /** Generate code for the given grammar element.
    * @param blk The rule-reference to generate
    */
   void gen(MGrammar_RuleRefElement_ptr rr);
public:
   /** Generate code for the given grammar element.
    * @param blk The string-literal reference to generate
    */
   void gen(MGrammar_StringLiteralElement_ptr atom);
public:
   /** Generate code for the given grammar element.
    * @param blk The token-range reference to generate
    */
   void gen(MGrammar_TokenRangeElement_ptr r);
public:
   /** Generate code for the given grammar element.
    * @param blk The token-reference to generate
    */
   void gen(MGrammar_TokenRefElement_ptr atom);
public:
   void gen(MGrammar_TreeElement_ptr t);
   /** Generate the tree-parser C++ files */
public:
   void gen(MGrammar_TreeWalkerGrammar_ptr g);
   /** Generate code for the given grammar element.
    * @param wc The wildcard element to generate
    */
public:
   void gen(MGrammar_WildcardElement_ptr wc);
   /** Generate code for the given grammar element.
    * @param blk The (...);* block to generate
    */
public:
   void gen(MGrammar_ZeroOrMoreBlock_ptr blk);
   /** Generate an alternative.
     * @param alt  The alternative to generate
     * @param blk The block to which the alternative belongs
     */
protected:
   void genAlt(MGrammar_Alternative_ptr alt, MGrammar_AlternativeBlock_ptr blk);
   /** Generate all the bitsets to be used in the parser or lexer
    * Generate the raw bitset data like "long _tokenSet1_data[] = {...};"
    * and the BitSet object declarations like
    * "BitSet _tokenSet1 = new BitSet(_tokenSet1_data);;"
    * Note that most languages do not support object initialization inside a
    * class definition, so other code-generators may have to separate the
    * bitset declarations from the initializations (e.g., put the
    * initializations in the generated constructor instead);.
    * @param bitsetList The list of bitsets to generate.
    * @param maxVocabulary Ensure that each generated bitset can contain at
    *        least this value.
    * @param prefix string glued in from of bitset names used for namespace
    *        qualifications.
    */
protected:
   void genBitsets(
      const MLANG_Vector<MParser_BitSet>& bitsetList,
      int maxVocabulary,
      MLANG_String prefix
   );
protected:
   void genBitsetsHeader(
      const MLANG_Vector<MParser_BitSet>& bitsetList,
      int maxVocabulary
   );
private:
   /** Generate the finish of a block, using a combination of the info
    * returned from genCommonBlock(); and the action to perform when
    * no alts were taken
    * @param howToFinish The return of genCommonBlock();
    */
   void beginBlockFinish (
      const MGrammar_CppBlockFinishingInfo& howToFinish
    );
   /** Generate the finish of a block, using a combination of the info
    * returned from genCommonBlock(); and the action to perform when
    * no alts were taken
    * @param howToFinish The return of genCommonBlock();
    * @param noViableAction What to generate when no alt is taken
    */
   void endBlockFinish (
      const MGrammar_CppBlockFinishingInfo& howToFinish
    );
   /** Generate the finish of a block, using a combination of the info
    * returned from genCommonBlock(); and the action to perform when
    * no alts were taken
    * @param howToFinish The return of genCommonBlock();
    * @param noViableAction What to generate when no alt is taken
    */
   void genBlockFinish (
      const MGrammar_CppBlockFinishingInfo& howToFinish,
      MLANG_String noViableAction
    );
protected:
   /** Generate the initaction for a block, which may be a RuleBlock or a
    * plain AlternativeBLock.
    * @blk The block for which the preamble is to be generated.
    */
   void genBlockInitAction( MGrammar_AlternativeBlock_ptr blk );
protected:
   /** Generate the header for a block, which may be a RuleBlock or a
    * plain AlternativeBlock. This generates any variable declarations
    * and syntactic-predicate-testing variables.
    * @blk The block for which the preamble is to be generated.
    */
   void genBlockPreamble(MGrammar_AlternativeBlock_ptr blk);
public:
   void genBody(MGrammar_LexerGrammar_ptr g);
public:
   void genInitFactory( MGrammar_Grammar_ptr g );
public:
public:
   // FIXME: and so why are we passing here a g param while inside
   // we merrily use the global grammar.
   void genBody(MGrammar_ParserGrammar_ptr g);
   void genBody(MGrammar_TreeWalkerGrammar_ptr g);
protected:
   /** Generate a series of case statements that implement a BitSet test.
    * @param p The Bitset for which cases are to be generated
    */
   void genCases(const MParser_BitSet& p);
public:
   /** Generate common code for a block of alternatives; return a postscript
    * that needs to be generated at the end of the block.  Other routines
    * may append else-clauses and such for error checking before the postfix
    * is generated.
    * If the grammar is a lexer, then generate alternatives in an order where
    * alternatives requiring deeper lookahead are generated first, and
    * EOF in the lookahead set reduces the depth of the lookahead.
    * @param blk The block to generate
    * @param noTestForSingle If true, then it does not generate a test for a single alternative.
    */
   MGrammar_CppBlockFinishingInfo genCommonBlock (
      MGrammar_AlternativeBlock_ptr blk,
      bool noTestForSingle
    );

private:
   static bool suitableForCaseExpression(MGrammar_Alternative_ptr a);
   /** Generate code to link an element reference into the AST
    */
private:
   void genElementAST(MGrammar_AlternativeElement_ptr el);
private:
   /** Close the try block and generate catch phrases
    * if the element has a labeled handler in the rule
    */
   void genErrorCatchForElement(MGrammar_AlternativeElement_ptr el);
private:
   /** Generate the catch phrases for a user-specified error handler */
   void genErrorHandler(MGrammar_ExceptionSpec_ptr ex);
private:
   /** Generate a try { opening if the element has a labeled handler in the rule */
   void genErrorTryForElement(MGrammar_AlternativeElement_ptr el);
protected:
   /** Generate a header that is common to all C++ files */
   void genHeader(MLANG_String fileName);

   // these are unique to C++ mode
public:
   void genInclude(MGrammar_LexerGrammar_ptr g);
public:
   void genInclude(MGrammar_ParserGrammar_ptr g);
public:
   void genInclude(MGrammar_TreeWalkerGrammar_ptr g);
   /// for convenience
protected:
   void genASTDeclaration( MGrammar_AlternativeElement_ptr el );
   /// for convenience
protected:
   void genASTDeclaration (
      MGrammar_AlternativeElement_ptr el,
      MLANG_String node_type
    );
   /// Generate (if not already done); a declaration for the AST for el.
protected:
   void genASTDeclaration (
      MGrammar_AlternativeElement_ptr el,
      MLANG_String var_name,
      MLANG_String node_type
    );
private:
   void genLiteralsTest();
private:
   void genLiteralsTestForPartialToken();
protected:
   void genMatch(const MParser_BitSet& b);
protected:
   void genMatch(MGrammar_GrammarAtom_ptr atom);
protected:
   void genMatchUsingAtomText(MGrammar_GrammarAtom_ptr atom);
protected:
   void genMatchUsingAtomTokenType (MGrammar_GrammarAtom_ptr atom);
public:
   /** Generate the nextToken(); rule.
    * nextToken(); is a synthetic lexer rule that is the implicit OR of all
    * user-defined lexer rules.
    * @param RuleBlock
    */
   void genNextToken();
   /** Gen a named rule block.
    * ASTs are generated for each element of an alternative unless
    * the rule or the alternative have a '!' modifier.
    *
    * If an alternative defeats the default tree construction, it
    * must set <rule>_AST to the root of the returned AST.
    *
    * Each alternative that does automatic tree construction, builds
    * up root and child list pointers in an ASTPair structure.
    *
    * A rule finishes by setting the returnAST variable from the
    * ASTPair.
    *
    * @param rule The name of the rule to generate
    * @param startSymbol true if the rule is a start symbol (i.e., not referenced elsewhere);
   */
public:
   void genRule (
      MGrammar_RuleSymbol_ptr s,
      bool startSymbol,
      int ruleNum,
      const MLANG_String& prefix
    );

public:
   void genRuleHeader(MGrammar_RuleSymbol_ptr s, bool startSymbol);
private:
   void GenRuleInvocation(MGrammar_RuleRefElement_ptr rr);
protected:
   void genSemPred(MLANG_String pred, int line);
   /** Write an array of MLANG_Strings which are the semantic predicate
    *  expressions.  The debugger will reference them by number only
    */
protected:
   void genSemPredMap(MLANG_String prefix);
protected:
   void genSynPred(MGrammar_SynPredBlock_ptr blk, const MLANG_String& lookaheadExpr);
public:
   /** Generate a static array containing the names of the tokens,
    * indexed by the token type values.  This static array is used
    * to format error messages so that the token identifers or literal
    * strings are displayed instead of the token numbers.
    *
    * If a lexical rule has a paraphrase, use it rather than the
    * token label.
    */
   void genTokenStrings(const MLANG_String& prefix);

protected:
   /** Generate the token types C++ file */
   void genTokenTypes(MGrammar_TokenManager_ptr tm);

public:
   /** Process a string for an simple expression for use in xx/action.g
    * it is used to cast simple tokens/references to the right type for
    * the generated language. Basically called for every element in
    * the vector to getASTCreateString(vector V);
    * @param str A MLANG_String.
    */
   MLANG_String processStringForASTConstructor(const MLANG_String& str );
public:
   /** Get a string for an expression to generate creation of an AST subtree.
     * @param v A Vector of MLANG_String, where each element is an expression
     * in the target language yielding an AST node.
     */
   MLANG_String getASTCreateString(const MLANG_Vector<MLANG_String>& v) const;

public:
   /** Get a string for an expression to generate creating of an AST node
    * @param str The arguments to the AST constructor
    */
   MLANG_String getASTCreateString (
      MGrammar_GrammarAtom_ptr atom,
      const MLANG_String& str
    );

public:
   /** Get a string for an expression to generate creating of an AST node
    * @param str The arguments to the AST constructor
    */
   MLANG_String getASTCreateString(const MLANG_String& str) const;

protected:
   MLANG_String getLookaheadTestExpression (
      const MLANG_Vector<MGrammar_Lookahead>& look,
      int k
    );

protected:
   /** Generate a lookahead test expression for an alternate.  This
    * will be a series of tests joined by '&&' and enclosed by '();',
    * the number of such tests being determined by the depth of the lookahead.
    */
   MLANG_String getLookaheadTestExpression (
      MGrammar_Alternative_ptr alt, int maxDepth);

protected:
   /**Generate a depth==1 lookahead test expression given the BitSet.
    * This may be one of:
    * 1); a series of 'x==X||' tests
    * 2); a range test using >= && <= where possible,
    * 3); a bitset membership test for complex comparisons
    * @param k The lookahead level
    * @param p The lookahead set for level k
    */
   MLANG_String getLookaheadTestTerm(int k, const MParser_BitSet& p);

public:
   /** Return an expression for testing a contiguous renage of elements
    * @param k The lookahead level
    * @param elems The elements representing the set, usually from BitSet.toArray();.
    * @return MLANG_String containing test expression.
    */
   MLANG_String getRangeExpression(int k, const MLANG_Vector<size_t>& elems);

private:
   /** getValueMLANG_String: get a string representation of a token or char value
    * @param value The token or char value
    */
   MLANG_String getValueString(int value);

protected:
   /**Is the lookahead for this alt empty? */
   bool lookaheadIsEmpty(MGrammar_Alternative_ptr alt, int maxDepth);

private:
   MLANG_String lookaheadString(int k);

private:
   /** Mangle a string literal into a meaningful token name.  This is
     * only possible for literals that are all characters.  The resulting
     * mangled literal name is literalsPrefix with the text of the literal
     * appended.
     * @return A string representing the mangled literal, or null if not possible.
     */
   MLANG_String mangleLiteral(const MLANG_String& s);

public:
   /** Map an identifier to it's corresponding tree-node variable.
     * This is context-sensitive, depending on the rule and alternative
     * being generated
     * @param idParam The identifier name to map
     * @return The mapped id (which may be the same as the input);, or null if the mapping is invalid due to duplicates
     */
   MLANG_String mapTreeId (
       const MLANG_String& idParam,
       MGrammar_ActionTransInfo_ptr transInfo
    );

private:
   /** Given an element and the name of an associated AST variable,
     * create a mapping between the element "name" and the variable name.
     */
   void mapTreeVariable (
      MGrammar_AlternativeElement_ptr e,
      const MLANG_String& name
    );

protected:
   /** Lexically process tree-specifiers in the action.
    * This will replace #id and #(...); with the appropriate
    * function calls and/or variables.
    */
   MLANG_String processActionForSpecialSymbols (
      const MLANG_String& actionStr,
      int line,
      MGrammar_RuleBlock_ptr currentRule,
      MGrammar_ActionTransInfo_ptr tInfo
   );

private:
   MLANG_String fixNameSpaceOption( const MLANG_String& ns );

private:
   void setupGrammarParameters(MGrammar_Grammar_ptr g);
private:
   MGrammar_CppCodeGenerator (const MGrammar_CppCodeGenerator& );
   MGrammar_CppCodeGenerator& operator= (const MGrammar_CppCodeGenerator& );
   friend class __MGrammar_CppCodeGenerator_dict;
};

#endif /* __MGRAMMAR_CPPCODEGENERATOR_H__ */
