#ifndef __MGRAMMAR_DEFAULTTOOLERRORHANDLER_H__
#define __MGRAMMAR_DEFAULTTOOLERRORHANDLER_H__
#include "ace/Vector_T.h"
#include "MLANG/String.h"
#include "MLANG/StringBuffer.h"
#include "MParser/CharFormatter.h"
#include "MGrammar/ToolErrorHandler.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/DefaultToolErrorHandler.java#1 $
 */

class MGrammar_DefaultToolErrorHandler : public MGrammar_ToolErrorHandler

{
public:
   MGrammar_DefaultToolErrorHandler(MGrammar_Tool_ptr tool);
   virtual ~MGrammar_DefaultToolErrorHandler();
private:
   MGrammar_Tool_ptr antlrTool_;

   MParser_CharFormatter_var javaCharFormatter_;

   /** Dump token/character sets to a string array suitable for
     * {@link antlr.Tool.warning(String[], String, int, int)
     * @param output The array that will contain the token/character set dump,
     *               one element per k (lookahead) value
     * @param outputStartIndex The index into <code>output</code> that the
     *                         dump should start at.
     * @param lexicalAnalysis  true for lexical rule
     * @param depth  The depth of the ambiguity
     * @param sets  An array of bitsets containing the ambiguities
     */
   void dumpSets (
      ACE_Vector<MLANG_String>& output,
      int outputStartIndex,
      MGrammar_Grammar_ptr grammar,
      bool lexicalAnalysis,
      int depth,
      const ACE_Vector<MGrammar_Lookahead>& sets
    );

public:
   /** Issue a warning about ambiguity between a alternates
    * @param blk  The block being analyzed
    * @param lexicalAnalysis  true for lexical rule
    * @param depth  The depth of the ambiguity
    * @param sets  An array of bitsets containing the ambiguities
    * @param altIdx1  The zero-based index of the first ambiguous alternative
    * @param altIdx2  The zero-based index of the second ambiguous alternative
    */
   void warnAltAmbiguity (
      MGrammar_Grammar_ptr grammar,
      MGrammar_AlternativeBlock_ptr blk,
      bool lexicalAnalysis,
      int depth,
      const ACE_Vector<MGrammar_Lookahead>& sets,
      int altIdx1,
      int altIdx2
    );

   /** Issue a warning about ambiguity between an alternate and exit path.
    * @param blk  The block being analyzed
    * @param lexicalAnalysis  true for lexical rule
    * @param depth  The depth of the ambiguity
    * @param sets  An array of bitsets containing the ambiguities
    * @param altIdx  The zero-based index of the ambiguous alternative
    */
   void warnAltExitAmbiguity (
      MGrammar_Grammar_ptr grammar,
      MGrammar_BlockWithImpliedExitPath_ptr blk,
      bool lexicalAnalysis,
      int depth,
      const ACE_Vector<MGrammar_Lookahead>& sets,
      int altIdx
    );
};

#endif /* __MGRAMMAR_DEFAULTTOOLERRORHANDLER_H__ */
