#include "MParser/StringUtils.h"
#include "MGrammar/ANTLRTokenTypes.h"
#include "MGrammar/TokenSymbol.h"
#include "MGrammar/SimpleTokenManager.h"
#include "MGrammar/ImportVocabTokenManager.h"
#include "MGrammar/StringLiteralSymbol.h"
#include "MParser/CommonToken.h"
#include "MGrammar/Tool.h"
#include "MGrammar/CodeGenerator.h"
#include "MGrammar/Grammar.h"
#include "MGrammar/LexerGrammar.h"
#include "MGrammar/ParserGrammar.h"
#include "MGrammar/TreeWalkerGrammar.h"
#include "MGrammar/ANTLRParser.h"
#include "MGrammar/DefineGrammarSymbols.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/DefineGrammarSymbols.java#1 $
 */

MLANG_String MGrammar_DefineGrammarSymbols::DEFAULT_TOKENMANAGER_NAME = "*default";
/**DefineGrammarSymbols is a behavior for the ANTLRParser that adds all
 * the token and rule symbols to the grammar symbol table.
 *
 * Token types are assigned to token symbols in this class also.
 * The token type for a token is done in the order seen (lexically).
 */

MGrammar_DefineGrammarSymbols::MGrammar_DefineGrammarSymbols (
   MGrammar_Tool_ptr tool, 
   const ACE_ARGV& args,
   MGrammar_LLkAnalyzer_ptr analyzer
) :
   tool_(tool),
   analyzer_(analyzer),
   args_(args),
   thePreambleAction_ (new MParser_CommonToken(MParser_Token::INVALID_TYPE, "")),
   language_("Cpp"),
   numLexers_(0),
   numParsers_(0),
   numTreeParsers_(0)

{
}

void MGrammar_DefineGrammarSymbols::_refStringLiteral (
   const MParser_Token_var& lit,
   const MParser_Token_var& label,
   int autoGenType,
   bool lastInRule
)
   
{
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (autoGenType);
   ACE_UNUSED_ARG (lastInRule);
   if (!grammar_->isLexer ())
    {
      // String literals are treated like tokens except by the lexer
      MLANG_String str = lit->getText();
      if (grammar_->tokenManager_->getTokenSymbol(str) != 0)
       {
         // string symbol is already defined
         return;
       }
      MGrammar_StringLiteralSymbol_var sl = new MGrammar_StringLiteralSymbol(str);
      int tt = grammar_->tokenManager_->nextTokenType();
      sl->setTokenType(tt);
      grammar_->tokenManager_->define( sl);
    }
}

/** Reference a token */
void MGrammar_DefineGrammarSymbols::_refToken (
   const MParser_Token_var& assignId,
   const MParser_Token_var& t,
   const MParser_Token_var& label,
   const MParser_Token_var& args,
   bool inverted,
   int autoGenType,
   bool lastInRule
)

{
   ACE_UNUSED_ARG (assignId);
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (args);
   ACE_UNUSED_ARG (inverted);
   ACE_UNUSED_ARG (autoGenType);
   ACE_UNUSED_ARG (lastInRule);
   MLANG_String id = t->getText();
   if (!grammar_->tokenManager_->tokenDefined(id))
    {
      /*
      // RK: dish out a warning if the token was not defined before.
         tool.warning("Token '" + id + "' defined outside tokens section",
         tool.grammarFile, t.getLine(), t.getColumn());
      */
      int tt = grammar_->tokenManager_->nextTokenType();
      MGrammar_TokenSymbol_var ts = new MGrammar_TokenSymbol(id);
      ts->setTokenType(tt);
      grammar_->tokenManager_->define(ts);
    }
}

/** Abort the processing of a grammar due to syntax errors */
void MGrammar_DefineGrammarSymbols::abortGrammar()

{
   if (!grammar_.is_null() && grammar_->getClassName().length() != 0)
    {
      grammars_.erase(grammar_->getClassName().c_str());
    }
   grammar_ = MGrammar_Grammar::_nil();
}

void MGrammar_DefineGrammarSymbols::beginAlt(bool doAST)

{
   ACE_UNUSED_ARG (doAST);
}

void MGrammar_DefineGrammarSymbols::beginChildList()

{
}

// Exception handling
void MGrammar_DefineGrammarSymbols::beginExceptionGroup()

{
}

void MGrammar_DefineGrammarSymbols::beginExceptionSpec(const MParser_Token_var& label)

{
   ACE_UNUSED_ARG (label);
}

void MGrammar_DefineGrammarSymbols::beginSubRule (
   const MParser_Token_var& label,
   const MParser_Token_var& start,
   bool no
)

{
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (start);
   ACE_UNUSED_ARG (no);
}

void MGrammar_DefineGrammarSymbols::beginTree(const MParser_Token_var& tok)

{
   ACE_UNUSED_ARG (tok);
}

/** Define a lexer or parser rule */
void MGrammar_DefineGrammarSymbols::defineRuleName (
   const MParser_Token_var& r,
   const MLANG_String& access,
   bool ruleAutoGen,
   const MLANG_String& docComment
)

{
   ACE_UNUSED_ARG (ruleAutoGen);
   MLANG_String id = r->getText();

   //		if ( Character.isUpperCase(id.charAt(0)) ) {
   if (r->getType() == ANTLRTokenTypes::TOKEN_REF)
    {
      // lexer rule
      id = MGrammar_CodeGenerator::encodeLexerRuleName(id);
      // make sure we define it as token identifier also
      if (!grammar_->tokenManager_->tokenDefined(r->getText()))
       {
         int tt = grammar_->tokenManager_->nextTokenType();
         MGrammar_TokenSymbol_var ts = new MGrammar_TokenSymbol(r->getText());
         ts->setTokenType(tt);
         grammar_->tokenManager_->define(ts);
       }
    }

   MGrammar_RuleSymbol_var rs;
   if (grammar_->isDefined(id))
    {
      // symbol seen before?
      rs = MGrammar_RuleSymbol::_narrow(grammar_->getSymbol(id));
      // rule just referenced or has it been defined yet?
      if (rs->isDefined())
       {
         tool_->error("redefinition of rule " + id, grammar_->getFilename(), r->getLine(), r->getColumn());
       }
    }
   else
    {
       rs = new MGrammar_RuleSymbol(id);
       grammar_->define(rs);
    }
   rs->setDefined();
   rs->access(access);
   rs->comment(docComment);
}

/** Define a token from tokens {...}.
 *  Must be label and literal or just label or just a literal.
 */
void MGrammar_DefineGrammarSymbols::defineToken (
   const MParser_Token_var& tokname,
   const MParser_Token_var& tokliteral
)

{
   MLANG_String name;
   MLANG_String literal;
   if (tokname != 0)
    {
      name = tokname->getText();
    }

   if (tokliteral != 0)
    {
      literal = tokliteral->getText();
    }
   // System.out.println("defining " + name + " with literal " + literal);
   //
   if (literal.length() != 0)
    {
      MGrammar_StringLiteralSymbol_var sl = Dynamic_Cast<MGrammar_StringLiteralSymbol>(grammar_->tokenManager_->getTokenSymbol(literal));
      if (!MGrammar_StringLiteralSymbol::is_nil(sl))
       {
         // This literal is known already.
         // If the literal has no label already, but we can provide
         // one here, then no problem, just map the label to the literal
         // and don't change anything else.
         // Otherwise, labels conflict: error.
         if (name.length() == 0 || sl->getLabel().length() != 0)
	  {
            tool_->warning("Redefinition of literal in tokens {...}: " + literal, grammar_->getFilename(), tokliteral->getLine(), tokliteral->getColumn());
            return;
          }
         else if (name.length() != 0)
	  {
            // The literal had no label, but new def does.  Set it.
            sl->setLabel(name);
            // Also, map the label to the literal.
            grammar_->tokenManager_->mapToTokenSymbol(name, sl);
          }
       }
      // if they provide a name/label and that name/label already
      // exists, just hook this literal onto old token.
      if (name.length() != 0)
       {
         MGrammar_TokenSymbol_var ts = Dynamic_Cast<MGrammar_TokenSymbol>(grammar_->tokenManager_->getTokenSymbol(name));
         if (!MGrammar_TokenSymbol::is_nil(ts))
	  {
            // watch out that the label is not more than just a token.
            // If it already has a literal attached, then: conflict.
            if (instanceof<MGrammar_StringLiteralSymbol>(ts))
	     {
               tool_->warning("Redefinition of token in tokens {...}: " + name, grammar_->getFilename(), tokliteral->getLine(), tokliteral->getColumn());
               return;
             }
            // a simple token symbol such as DECL is defined
            // must convert it to a StringLiteralSymbol with a
            // label by co-opting token type and killing old
            // TokenSymbol.  Kill mapping and entry in vector
            // of token manager.
            // First, claim token type.
            int ttype = ts->getTokenType();
            // now, create string literal with label
            sl = new MGrammar_StringLiteralSymbol(literal);
            sl->setTokenType(ttype);
            sl->setLabel(name);
            // redefine this critter as a string literal
            grammar_->tokenManager_->define(sl);
            // make sure the label can be used also.
            grammar_->tokenManager_->mapToTokenSymbol(name, sl);
            return;
          }
         // here, literal was labeled but not by a known token symbol.
       }
      sl = new MGrammar_StringLiteralSymbol(literal);
      int tt = grammar_->tokenManager_->nextTokenType();
      sl->setTokenType(tt);
      sl->setLabel(name);
      grammar_->tokenManager_->define(sl);
      if (name.length() != 0)
       {
         // make the label point at token symbol too
         grammar_->tokenManager_->mapToTokenSymbol(name, sl);
       }
    }

   // create a token in the token manager not a literal
   else
    {
      if (grammar_->tokenManager_->tokenDefined(name)) {
         tool_->warning("Redefinition of token in tokens {...}: " + name, grammar_->getFilename(), tokname->getLine(), tokname->getColumn());
         return;
       }
      int tt = grammar_->tokenManager_->nextTokenType();
      MGrammar_TokenSymbol_var ts = new MGrammar_TokenSymbol(name);
      ts->setTokenType(tt);
      grammar_->tokenManager_->define(ts);
    }
}

void MGrammar_DefineGrammarSymbols::endAlt()

{
}

void MGrammar_DefineGrammarSymbols::endChildList()

{
}

void MGrammar_DefineGrammarSymbols::endExceptionGroup()

{
}

void MGrammar_DefineGrammarSymbols::endExceptionSpec() 

{
}

void MGrammar_DefineGrammarSymbols::endGrammar()

{
}

/** Called after the optional options section, to compensate for
 * options that may not have been set.
 * This method is bigger than it needs to be, but is much more
 * clear if I delineate all the cases.
 */
void MGrammar_DefineGrammarSymbols::endOptions()

{
   // NO VOCAB OPTIONS
   if (grammar_->exportVocab().length() == 0
   &&  grammar_->importVocab().length() == 0)
    {
      grammar_->exportVocab(grammar_->getClassName());
      // Can we get initial vocab from default shared vocab?
      if (tokenManagers_.count(DEFAULT_TOKENMANAGER_NAME.c_str()))
       {
         // Use the already-defined token manager
         grammar_->exportVocab (DEFAULT_TOKENMANAGER_NAME);
         MGrammar_TokenManager_var tm = tokenManagers_.find(DEFAULT_TOKENMANAGER_NAME.c_str())->second;
         // System.out.println("No tokenVocabulary for '" + grammar.getClassName() + "', using default '" + tm->getName() + "'");
         grammar_->setTokenManager(tm);
         return;
       }
      // no shared vocab for file, make new one
      // System.out.println("No exportVocab for '" + grammar_->getClassName() + "', creating default '" + grammar.exportVocab + "'");
      MGrammar_TokenManager_var tm = new MGrammar_SimpleTokenManager(grammar_->exportVocab(), tool_);
      grammar_->setTokenManager(tm);
      // Add the token manager to the list of token managers
      tokenManagers_[grammar_->exportVocab().c_str()] = tm;
            // no default vocab, so make this the default vocab
      tokenManagers_[DEFAULT_TOKENMANAGER_NAME.c_str()] = tm;
      return;
    }

        // NO OUTPUT, BUT HAS INPUT VOCAB
   if (grammar_->exportVocab().length() == 0
   &&  grammar_->importVocab().length() != 0)
    {
      grammar_->exportVocab (grammar_->getClassName());
      // first make sure input!=output
      if (grammar_->importVocab().equals(grammar_->exportVocab()))
       {
         tool_->warning("Grammar " + grammar_->getClassName() +
                        " cannot have importVocab same as default output vocab (grammar name); ignored.");
         // kill importVocab option and try again: use default vocab
         grammar_->importVocab ("");
         endOptions();
         return;
      }
     // check to see if the vocab is already in memory
     // (defined by another grammar in the file).  Not normal situation.
     if (tokenManagers_.containsKey(grammar_->importVocab().c_str()))
      {
        // make a copy since we'll be generating a new output vocab
        // and we don't want to affect this one.  Set the name to
        // the default output vocab==classname.
        MGrammar_TokenManager_var tm = tokenManagers_.get(grammar_->importVocab().c_str());
        // System.out.println("Duping importVocab of " + grammar_->importVocab());
        MGrammar_TokenManager_var dup = tm->clone();
        dup->setName(grammar_->exportVocab());
        // System.out.println("Setting name to " + grammar_->exportVocab());
        dup->setReadOnly(false);
        grammar_->setTokenManager(dup);
        tokenManagers_.put(grammar_->exportVocab().c_str(), dup);
        return;
      }
     // System.out.println("reading in vocab "+grammar.importVocab);
     // Must be a file, go get it.
     MGrammar_ImportVocabTokenManager_var tm =
        new MGrammar_ImportVocabTokenManager(grammar_,
                                    grammar_->importVocab() + MGrammar_CodeGenerator::TokenTypesFileSuffix_ + MGrammar_CodeGenerator::TokenTypesFileExt_,
                                    grammar_->exportVocab(),
                                    tool_);
     tm->setReadOnly(false); // since renamed, can write out
            // Add this token manager to the list so its tokens will be generated
     tokenManagers_.put (grammar_->exportVocab().c_str(), tm);
     // System.out.println("vocab renamed to default output vocab of "+tm.getName());
     // Assign the token manager to this grammar.
     grammar_->setTokenManager(tm);

     // set default vocab if none
     if (!tokenManagers_.containsKey(DEFAULT_TOKENMANAGER_NAME.c_str()))
      {
        tokenManagers_[DEFAULT_TOKENMANAGER_NAME.c_str()] = tm;
      }

     return;
   }

  // OUTPUT VOCAB, BUT NO INPUT VOCAB
  if (grammar_->exportVocab().length() != 0 
  &&  grammar_->importVocab().length() == 0)
   {
     // share with previous vocab if it exists
     if (tokenManagers_.containsKey(grammar_->exportVocab().c_str()))
      {
        // Use the already-defined token manager
        MGrammar_TokenManager_var tm = tokenManagers_[grammar_->exportVocab().c_str()];
        // System.out.println("Sharing exportVocab of " + grammar.exportVocab);
        grammar_->setTokenManager(tm);
        return;
      }
     // create new output vocab
     // System.out.println("Creating exportVocab " + grammar.exportVocab);
     MGrammar_TokenManager_var tm = new MGrammar_SimpleTokenManager(grammar_->exportVocab(), tool_);
     grammar_->setTokenManager(tm);
     // Add the token manager to the list of token managers
     tokenManagers_.put (grammar_->exportVocab(), tm);
     // set default vocab if none
     if (!tokenManagers_.containsKey(DEFAULT_TOKENMANAGER_NAME))
      {
        tokenManagers_.put(DEFAULT_TOKENMANAGER_NAME, tm);
      }
     return;
   }

   // BOTH INPUT AND OUTPUT VOCAB
   if (grammar_->exportVocab().length() != 0
   &&  grammar_->importVocab().length() != 0)
    {
      // don't want input==output
      if (grammar_->importVocab().equals(grammar_->exportVocab()))
       {
         tool_->error("exportVocab of " + grammar_->exportVocab() + " same as importVocab; probably not what you want");
       }
      // does the input vocab already exist in memory?
      if (tokenManagers_.containsKey(grammar_->importVocab()))
       {
         // make a copy since we'll be generating a new output vocab
         // and we don't want to affect this one.
         MGrammar_TokenManager_var tm = 
            tokenManagers_.get(grammar_->importVocab());
         // System.out.println("Duping importVocab of " + grammar.importVocab);
         MGrammar_TokenManager_var dup = tm->clone();
         dup->setName(grammar_->exportVocab());
         // System.out.println("Setting name to " + grammar.exportVocab);
         dup->setReadOnly(false);
         grammar_->setTokenManager(dup);
         tokenManagers_.put(grammar_->exportVocab(), dup);
         return;
       }
      // Must be a file, go get it.
      MGrammar_ImportVocabTokenManager_var tm =
             new MGrammar_ImportVocabTokenManager(grammar_,
                                            grammar_->importVocab() + MGrammar_CodeGenerator::TokenTypesFileSuffix_ + MGrammar_CodeGenerator::TokenTypesFileExt_,
                                            grammar_->exportVocab(),
                                            tool_);
      tm->setReadOnly(false); // write it out as we've changed name
      // Add this token manager to the list so its tokens will be generated
      tokenManagers_.put(grammar_->exportVocab(), tm);
      // Assign the token manager to this grammar.
      grammar_->setTokenManager(tm);

      // set default vocab if none
      if (!tokenManagers_.containsKey(DEFAULT_TOKENMANAGER_NAME))
       {
         tokenManagers_.put(DEFAULT_TOKENMANAGER_NAME, tm);
       }

      return;
   }
}

void MGrammar_DefineGrammarSymbols::endRule(const MLANG_String& r)

{
   ACE_UNUSED_ARG (r);
}

void MGrammar_DefineGrammarSymbols::endSubRule()

{
}

void MGrammar_DefineGrammarSymbols::endTree()

{
}

void MGrammar_DefineGrammarSymbols::hasError()

{
}

void MGrammar_DefineGrammarSymbols::noASTSubRule()

{
}

void MGrammar_DefineGrammarSymbols::oneOrMoreSubRule()

{
}

void MGrammar_DefineGrammarSymbols::optionalSubRule()

{
}

void MGrammar_DefineGrammarSymbols::setUserExceptions(const MLANG_String& thr)

{
   ACE_UNUSED_ARG (thr);
}

void MGrammar_DefineGrammarSymbols::refAction (
   const MParser_Token_var& action
)

{
   ACE_UNUSED_ARG (action);
}

void MGrammar_DefineGrammarSymbols::refArgAction (
   const MParser_Token_var& action
)
   
{
   ACE_UNUSED_ARG (action);
}

void MGrammar_DefineGrammarSymbols::refCharLiteral (
   const MParser_Token_var& lit,
   const MParser_Token_var& label,
   bool inverted,
   int autoGenType,
   bool lastInRule
)

{
   ACE_UNUSED_ARG (lit);
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (inverted);
   ACE_UNUSED_ARG (autoGenType);
   ACE_UNUSED_ARG (lastInRule);
}

void MGrammar_DefineGrammarSymbols::refCharRange (
   const MParser_Token_var& t1, 
   const MParser_Token_var& t2,
   const MParser_Token_var& label,
   int autoGenType,
   bool lastInRule
)

{
   ACE_UNUSED_ARG (t1);
   ACE_UNUSED_ARG (t2);
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (autoGenType);
   ACE_UNUSED_ARG (lastInRule);
}

void MGrammar_DefineGrammarSymbols::refElementOption (
   const MParser_Token_var& option,
   const MParser_Token_var& value
)
   
{
   ACE_UNUSED_ARG (option);
   ACE_UNUSED_ARG (value);
}

void MGrammar_DefineGrammarSymbols::refTokensSpecElementOption (
   const MParser_Token_var& tok,
   const MParser_Token_var& option,
   const MParser_Token_var& value
)

{
   ACE_UNUSED_ARG (tok);
   ACE_UNUSED_ARG (option);
   ACE_UNUSED_ARG (value);
}

void MGrammar_DefineGrammarSymbols::refExceptionHandler (
   const MParser_Token_var& exTypeAndName,
   const MParser_Token_var& action
)

{
   ACE_UNUSED_ARG (exTypeAndName);
   ACE_UNUSED_ARG (action);
}

// Header action applies to all parsers and lexers.
void MGrammar_DefineGrammarSymbols::refHeaderAction (
   const MParser_Token_var& name,
   const MParser_Token_var& act
)

{
   MLANG_String key;

   if (MParser_Token::is_nil(name))
      key = "";
   else
      key = MParser_StringUtils::stripFrontBack(name->getText(), "\"", "\"");

   // FIXME: depending on the mode the inserted header actions should
   // be checked for sanity.
   if (headerActions_.containsKey(key))
    {
      if (key.equals(""))
         tool_->error(act->getLine() + ": header action already defined");
      else
         tool_->error(act->getLine() + ": header action '" + key + "' already defined");
    }
   MParser_Token_var temp = act;
   headerActions_[key] = temp;
}

MLANG_String MGrammar_DefineGrammarSymbols::getHeaderAction (
   const MLANG_String& name
)

{
   MParser_Token_var t = headerActions_.get(name);
   if (MParser_Token::is_nil(t))
    {
      return "";
    }
   return t->getText();
}

void MGrammar_DefineGrammarSymbols::refInitAction (
   const MParser_Token_var& action
)

{
   ACE_UNUSED_ARG (action);
}

void MGrammar_DefineGrammarSymbols::refMemberAction (
   const MParser_Token_var& act
)

{
   ACE_UNUSED_ARG (act);
}

void MGrammar_DefineGrammarSymbols::refPreambleAction (
   const MParser_Token_var& act
)

{
   thePreambleAction_ = act;
}

void MGrammar_DefineGrammarSymbols::refReturnAction (
   const MParser_Token_var& returnAction
)

{
   ACE_UNUSED_ARG (returnAction);
}

void MGrammar_DefineGrammarSymbols::refRule (
   const MParser_Token_var& idAssign,
   const MParser_Token_var& r,
   const MParser_Token_var& label,
   const MParser_Token_var& args,
   int autoGenType
)

{
   ACE_UNUSED_ARG (idAssign);
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (args);
   ACE_UNUSED_ARG (autoGenType);
   MLANG_String id = r->getText();
   //		if ( Character.isUpperCase(id.charAt(0)) ) { // lexer rule?
   if (r->getType() == ANTLRTokenTypes::TOKEN_REF)
    {
      // lexer rule?
      id = MGrammar_CodeGenerator::encodeLexerRuleName(id);
    }
   if (!grammar_->isDefined(id))
    {
      grammar_->define(new MGrammar_RuleSymbol(id));
    }
}

void MGrammar_DefineGrammarSymbols::refSemPred (
   const MParser_Token_var& pred
)

{
   ACE_UNUSED_ARG (pred);
}

void MGrammar_DefineGrammarSymbols::refStringLiteral (
   const MParser_Token_var& lit,
   const MParser_Token_var& label,
   int autoGenType,
   bool lastInRule
)

{
   _refStringLiteral(lit, label, autoGenType, lastInRule);
}

/** Reference a token */
void MGrammar_DefineGrammarSymbols::refToken (
   const MParser_Token_var& assignId, 
   const MParser_Token_var& t, 
   const MParser_Token_var& label, 
   const MParser_Token_var& args,
   bool inverted,
   int autoGenType,
   bool lastInRule
)

{
   _refToken(assignId, t, label, args, inverted, autoGenType, lastInRule);
}

void MGrammar_DefineGrammarSymbols::refTokenRange (
   const MParser_Token_var& t1, 
   const MParser_Token_var& t2, 
   const MParser_Token_var& label, 
   int autoGenType, 
   bool lastInRule
)

{
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (autoGenType);
   // ensure that the DefineGrammarSymbols methods are called; otherwise a range addes more
   // token refs to the alternative by calling MakeGrammar.refToken etc...
   if (t1->getText().charAt(0) == '"')
    {
      refStringLiteral(t1, MParser_Token::_nil(), MGrammar_GrammarElement::AUTO_GEN_NONE, lastInRule);
    }
   else
    {
      _refToken(MParser_Token::_nil(), t1, MParser_Token::_nil(), MParser_Token::_nil(), false, MGrammar_GrammarElement::AUTO_GEN_NONE, lastInRule);
    }
   if (t2->getText().charAt(0) == '"')
    {
      _refStringLiteral(t2, MParser_Token::_nil(), MGrammar_GrammarElement::AUTO_GEN_NONE, lastInRule);
    }
   else
    {
      _refToken(MParser_Token::_nil(), t2, MParser_Token::_nil(), MParser_Token::_nil(), false, MGrammar_GrammarElement::AUTO_GEN_NONE, lastInRule);
    }
}

void MGrammar_DefineGrammarSymbols::refTreeSpecifier (
   const MParser_Token_var& treeSpec
)

{
   ACE_UNUSED_ARG (treeSpec);
}

void MGrammar_DefineGrammarSymbols::refWildcard (
   const MParser_Token_var& t,
   const MParser_Token_var& label,
   int autoGenType
)
   
{
   ACE_UNUSED_ARG (t);
   ACE_UNUSED_ARG (label);
   ACE_UNUSED_ARG (autoGenType);
}

/** Get ready to process a new grammar */
void MGrammar_DefineGrammarSymbols::reset()

{
   grammar_ = 0;
}

void MGrammar_DefineGrammarSymbols::setArgOfRuleRef (
   const MParser_Token_var& argaction
)

{
   ACE_UNUSED_ARG (argaction);
}

/** Set the character vocabulary for a lexer */
void MGrammar_DefineGrammarSymbols::setCharVocabulary(const MParser_BitSet& b)

{
   // grammar should enforce that this is only called for lexer
   MGrammar_LexerGrammar_var grammar = 
        MGrammar_LexerGrammar::_narrow(grammar_);
   grammar->setCharVocabulary(b);
}

/** setFileOption: Associate an option value with a key.
 * This applies to options for an entire grammar file.
 * @param key The token containing the option name
 * @param value The token containing the option value.
 */
void MGrammar_DefineGrammarSymbols::setFileOption (
   const MParser_Token_var& key,
   const MParser_Token_var& value,
   const MLANG_String& filename
)

{
   if (key->getText().equals("language"))
    {
      if (value->getType() == ANTLRParser::STRING_LITERAL)
       {
         language_ = MParser_StringUtils::stripBack(MParser_StringUtils::stripFront(value->getText(), '"'), '"');
       }
      else if (value->getType() == ANTLRParser::TOKEN_REF
           ||  value->getType() == ANTLRParser::RULE_REF)
       {
         language_ = value->getText();
       }
      else
       {
         tool_->error("language option must be string or identifier", filename, value->getLine(), value->getColumn());
       }
    }
   else if (key->getText().equals("mangleLiteralPrefix"))
    {
      if (value->getType() == ANTLRParser::STRING_LITERAL)
       {
         tool_->setLiteralsPrefix (MParser_StringUtils::stripFrontBack(value->getText(), "\"", "\""));
       }
      else
       {
         tool_->error("mangleLiteralPrefix option must be string", filename, value->getLine(), value->getColumn());
       }
    }
   else if (key->getText().equals("upperCaseMangledLiterals"))
    {
      if (value->getText().equals("true"))
       {
         tool_->setUpperCaseMangledLiterals(true);
       }
      else if (value->getText().equals("false"))
       {
         tool_->setUpperCaseMangledLiterals(false);
       }
      else
       {
         grammar_->antlrTool_->error("Value for upperCaseMangledLiterals must be true or false", filename, key->getLine(), key->getColumn());
       }
    }
   else if (key->getText().equals("namespaceStd")  
        ||  key->getText().equals("namespaceAntlr")
        ||  key->getText().equals("genHashLines")
            	  )
    {
      if (!language_.equals("Cpp"))
       {
          tool_->error(key->getText() + " option only valid for C++", filename, key->getLine(), key->getColumn());
       }
      else
       {
          if (key->getText().equals("noConstructors"))
           {
             if (!(value->getText().equals("true")
             ||  value->getText().equals("false")))
                tool_->error("noConstructors option must be true or false", filename, value->getLine(), value->getColumn());
             tool_->noConstructors (value->getText().equals("true"));
           }
          else if (key->getText().equals("genHashLines"))
           {
             if (!(value->getText().equals("true")
             ||    value->getText().equals("false")))
                tool_->error("genHashLines option must be true or false", filename, value->getLine(), value->getColumn());
             tool_->setGenHashLines (value->getText().equals("true"));
           }
          else
           {
             if (value->getType() != ANTLRParser::STRING_LITERAL)
              {
                tool_->error(key->getText() + " option must be a string", filename, value->getLine(), value->getColumn());
              }
             else
              {
                if (key->getText().equals("namespaceStd"))
                   tool_->setNamespaceStd (value->getText());
                else if (key->getText().equals("namespaceAntlr"))
                   tool_->setNamespaceAntlr (value->getText());
              }
           }
         }
    }
   else if ( key->getText().equals("namespace") )
    {
      if ( !language_.equals("Cpp") && !language_.equals("CSharp") )
       {
         tool_->error(key->getText() + " option only valid for C++ and C# (a.k.a CSharp)", filename, key->getLine(), key->getColumn());
       }
      else
       {
         if (value->getType() != ANTLRParser::STRING_LITERAL)
          {
            tool_->error(key->getText() + " option must be a string", filename, value->getLine(), value->getColumn());
          }
         else
          {
            if (key->getText().equals("namespace"))
#if 0
               tool_->setNameSpace(value->getText());
#else
               assert(0);
#endif
          }
       }
    }
   else
    {
      tool_->error("Invalid file-level option: " + key->getText(), filename, key->getLine(), value->getColumn());
    }
}

/** setGrammarOption: Associate an option value with a key.
 * This function forwards to Grammar.setOption for some options.
 * @param key The token containing the option name
 * @param value The token containing the option value.
 */
void MGrammar_DefineGrammarSymbols::setGrammarOption (
   const MParser_Token_var& key,
   const MParser_Token_var& value
)

{
   if (key->getText().equals("tokdef")
   ||  key->getText().equals("tokenVocabulary"))
    {
      tool_->error(MLANG_String("tokdef/tokenVocabulary options are invalid >= ANTLR 2.6.0.\n") +
                   "  Use importVocab/exportVocab instead.  Please see the documentation.\n" +
                   "  The previous options were so heinous that Terence changed the whole\n" +
                   "  vocabulary mechanism; it was better to change the names rather than\n" +
                   "  subtly change the functionality of the known options.  Sorry!", grammar_->getFilename(), value->getLine(), value->getColumn());
    }
   else if (key->getText().equals("literal")
   &&       grammar_->isLexer())
    {
      tool_->error(MLANG_String("the literal option is invalid >= ANTLR 2.6.0.\n") +
                   "  Use the \"tokens {...}\" mechanism instead.",
                   grammar_->getFilename(), value->getLine(), value->getColumn());
    }
   else if (key->getText().equals("exportVocab"))
    {
      // Set the token manager associated with the parser
      if (value->getType() == ANTLRParser::RULE_REF
      ||  value->getType() == ANTLRParser::TOKEN_REF)
       {
         grammar_->exportVocab (value->getText());
       }
      else
       {
         tool_->error("exportVocab must be an identifier", grammar_->getFilename(), value->getLine(), value->getColumn());
       }
    }
   else if (key->getText().equals("importVocab"))
    {
      if (value->getType() == ANTLRParser::RULE_REF
      ||  value->getType() == ANTLRParser::TOKEN_REF)
       {
         grammar_->importVocab (value->getText());
       }
      else
       {
         tool_->error("importVocab must be an identifier", grammar_->getFilename(), value->getLine(), value->getColumn());
       }
    }
   else
    {
      // Forward all unrecognized options to the grammar
      grammar_->setOption(key->getText(), value);
    }
}

void MGrammar_DefineGrammarSymbols::setRuleOption (
   const MParser_Token_var& key, 
   const MParser_Token_var& value
)
   
{
   ACE_UNUSED_ARG (key);
   ACE_UNUSED_ARG (value);
}

void MGrammar_DefineGrammarSymbols::setSubruleOption (
   const MParser_Token_var& key,
   const MParser_Token_var& value
)

{
   ACE_UNUSED_ARG (key);
   ACE_UNUSED_ARG (value);
}

/** Start a new lexer */
void MGrammar_DefineGrammarSymbols::startLexer (
   const MLANG_String& file,
   const MParser_Token_var& name,
   const MLANG_String& superClass,
   const MLANG_String& doc
)

{
   if (numLexers_ > 0)
    {
      tool_->panic("You may only have one lexer per grammar file: class " + name->getText());
    }
   numLexers_++;
   reset();
   //System.out.println("Processing lexer '" + name.getText() + "'");
   // Does the lexer already exist?
   MGrammar_Grammar_var g = grammars_.get(name->getText());
   if (!MGrammar_Grammar::is_nil(g))
    {
      if (!(g->isLexer()))
       {
         tool_->panic("'" + name->getText() + "' is already defined as a non-lexer");
       }
      else
       {
         tool_->panic("Lexer '" + name->getText() + "' is already defined");
       }
    }
   else
    {
       // Create a new lexer grammar
       MGrammar_LexerGrammar_var lg = new MGrammar_LexerGrammar(name->getText(), tool_, superClass);
       lg->comment (doc);
       lg->processArguments(args_);
       lg->setFilename(file);
       grammars_.put(lg->getClassName(), lg);
       // Use any preamble action
       lg->preambleAction (thePreambleAction_);
       thePreambleAction_ = new MParser_CommonToken(MParser_Token::INVALID_TYPE, "");
       // This is now the current grammar
       grammar_ = lg;
    }
}

/** Start a new parser */
void MGrammar_DefineGrammarSymbols::startParser (
   const MLANG_String& file,
   const MParser_Token_var& name,
   const MLANG_String& superClass,
   const MLANG_String& doc)

{
   if (numParsers_ > 0)
    {
      tool_->panic("You may only have one parser per grammar file: class " + name->getText());
    }
   numParsers_++;
   reset();
   //System.out.println("Processing parser '" + name.getText() + "'");
   // Is this grammar already defined?
   MGrammar_Grammar_var g = grammars_.get(name->getText());
   if (!MGrammar_Grammar::is_nil(g))
    {
      if (!(g->isParser()))
       {
         tool_->panic("'" + name->getText() + "' is already defined as a non-parser");
       }
      else
       {
         tool_->panic("Parser '" + name->getText() + "' is already defined");
       }
    }
   else  
    {
       // Create a new grammar
       grammar_ = new MGrammar_ParserGrammar(name->getText(), tool_, superClass);
       grammar_->comment (doc);
       grammar_->processArguments(args_);
       grammar_->setFilename(file);
       grammars_.put(grammar_->getClassName(), grammar_);
       // Use any preamble action
       grammar_->preambleAction (thePreambleAction_);
       thePreambleAction_ = new MParser_CommonToken(MParser_Token::INVALID_TYPE, "");
    }
}

/** Start a new tree-walker */
void MGrammar_DefineGrammarSymbols::startTreeWalker (
   const MLANG_String& file,
   const MParser_Token_var& name,
   const MLANG_String& superClass,
   const MLANG_String& doc
)

{
   if (numTreeParsers_ > 0)
    {
      tool_->panic("You may only have one tree parser per grammar file: class " + name->getText());
    }
   numTreeParsers_++;
   reset();
   //System.out.println("Processing tree-walker '" + name.getText() + "'");
   // Is this grammar already defined?
   MGrammar_Grammar_var g = grammars_.get(name->getText());
   if (!MGrammar_Grammar::is_nil(g))
    {
      if (!(g->isTreeWalker()))
       {
         tool_->panic("'" + name->getText() + "' is already defined as a non-tree-walker");
       }
      else
       {
         tool_->panic("Tree-walker '" + name->getText() + "' is already defined");
       }
    }
   else
    {
      // Create a new grammar
      grammar_ = new MGrammar_TreeWalkerGrammar(name->getText(), tool_, superClass);
      grammar_->comment (doc);
      grammar_->processArguments(args_);
      grammar_->setFilename(file);
      grammars_.put(grammar_->getClassName(), grammar_);
      // Use any preamble action
      grammar_->preambleAction (thePreambleAction_);
      thePreambleAction_ = new MParser_CommonToken(MParser_Token::INVALID_TYPE, "");
    }
}

void MGrammar_DefineGrammarSymbols::synPred()

{
}

void MGrammar_DefineGrammarSymbols::zeroOrMoreSubRule() 

{
}

MGrammar_DefineGrammarSymbols::~MGrammar_DefineGrammarSymbols ()

{
}
