#ifndef __MGRAMMAR_DEFINEGRAMMARSYMBOLS_H__
#define __MGRAMMAR_DEFINEGRAMMARSYMBOLS_H__
#include "ace/ARGV.h"
#include "MLANG/HashMap.h"
#include "MLANG/String.h"
#include "MParser/BitSet.h"
#include "MParser/Token.h"
#include "MGrammar/TokenManager.h"
#include "MGrammar/GrammarParseBehavior.h"
#include "MGrammar/LLkAnalyzer.h"

class MGrammar_DefineGrammarSymbols;
typedef class MGrammar_DefineGrammarSymbols *MGrammar_DefineGrammarSymbols_ptr;
typedef MLANG_SmartPtr<MGrammar_DefineGrammarSymbols> MGrammar_DefineGrammarSymbols_var;

#include "MGrammar/Grammar.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/DefineGrammarSymbols.java#1 $
 */


class MGrammar_Tool;
typedef class MGrammar_Tool *MGrammar_Tool_ptr;
class MGrammar_Grammar;
typedef class MGrammar_Grammar *MGrammar_Grammar_ptr;

/**DefineGrammarSymbols is a behavior for the ANTLRParser that adds all
 * the token and rule symbols to the grammar symbol table.
 *
 * Token types are assigned to token symbols in this class also.
 * The token type for a token is done in the order seen (lexically).
 */
class MGrammar_DefineGrammarSymbols : public MGrammar_GrammarParseBehavior

{
public:
    typedef MLANG_HashMap<MLANG_String, MGrammar_Grammar_var> GRAMMARS;
    typedef MLANG_HashMap<MLANG_String, MGrammar_TokenManager_var> TOKENMANAGERS;
protected:
    // Contains all of the defined parser and lexer Grammar's indexed by name
    GRAMMARS grammars_;
    // Contains all the TokenManagers indexed by name
    TOKENMANAGERS tokenManagers_;
    // Current grammar (parser or lexer)
    MGrammar_Grammar_var grammar_;
    // The tool under which this is invoked
    MGrammar_Tool_ptr tool_;
    // The grammar analyzer object
    MGrammar_LLkAnalyzer_ptr analyzer_;
    // The command-line arguments passed to the tool.
    // This allows each grammar to parse the arguments as it is created
    ACE_ARGV args_;
    // Name for default token manager does not match any valid name
    static MLANG_String DEFAULT_TOKENMANAGER_NAME;
    // Header actions apply to all parsers unless redefined
public:
    // Contains all of the header actions indexed by name
    typedef MLANG_HashMap<MLANG_String, MParser_Token_var> ACTIONS;
    ACTIONS headerActions_;
    // Place where preamble is stored until a grammar is defined
    MParser_Token_var thePreambleAction_;
    // The target language
    MLANG_String language_;

    int numLexers_;
    int numParsers_;
    int numTreeParsers_;

public:

    MGrammar_DefineGrammarSymbols (
       MGrammar_Tool_ptr tool, 
       const ACE_ARGV& args, 
       MGrammar_LLkAnalyzer_ptr analyzer
     );

    virtual ~MGrammar_DefineGrammarSymbols ();

public:

   const GRAMMARS& grammars() const
   { return grammars_; }

    const TOKENMANAGERS& tokenManagers() const
    { return tokenManagers_; }

public:

   static bool is_nil (MGrammar_DefineGrammarSymbols_ptr behavior)
   { return behavior == 0; }

   static MGrammar_DefineGrammarSymbols_ptr _nil()
   { return 0; }

   static MGrammar_DefineGrammarSymbols_ptr _duplicate(MGrammar_DefineGrammarSymbols_ptr behavior)
   { return ::MLANG_acquire(behavior); }

public:

    const MLANG_String& language() const
    { return language_; }

    MGrammar_DefineGrammarSymbols& language(const MLANG_String& value)
    { language_ = value; return *this; }

public:

    void _refStringLiteral(const MParser_Token_var& lit, const MParser_Token_var& label, int autoGenType, bool lastInRule);

    /** Reference a token */
    void _refToken(const MParser_Token_var& assignId,
                          const MParser_Token_var& t,
                          const MParser_Token_var& label,
                          const MParser_Token_var& args,
                          bool inverted,
                          int autoGenType,
                          bool lastInRule);

    /** Abort the processing of a grammar due to syntax errors */
    virtual void abortGrammar();

    virtual void beginAlt(bool doAST);

    virtual void beginChildList();

    // Exception handling
    virtual void beginExceptionGroup();

    void beginExceptionSpec(const MParser_Token_var& label);

    virtual void beginSubRule(const MParser_Token_var& label, const MParser_Token_var& start, bool no);

    virtual void beginTree(const MParser_Token_var& tok);

    /** Define a lexer or parser rule */
    virtual void defineRuleName(const MParser_Token_var& r,
                        const MLANG_String& access,
                        bool ruleAutoGen,
                        const MLANG_String& docComment
      );

    /** Define a token from tokens {...}.
     *  Must be label and literal or just label or just a literal.
     */
    virtual void defineToken(const MParser_Token_var& tokname, const MParser_Token_var& tokliteral);

    virtual void endAlt();

    virtual void endChildList();

    virtual void endExceptionGroup();

    virtual void endExceptionSpec();

    virtual void endGrammar();

    /** Called after the optional options section, to compensate for
     * options that may not have been set.
     * This method is bigger than it needs to be, but is much more
     * clear if I delineate all the cases.
     */
    virtual void endOptions();

    virtual void endRule(const MLANG_String& r);

    virtual void endSubRule();

    virtual void endTree();

    virtual void hasError();

    virtual void noASTSubRule();

    virtual void oneOrMoreSubRule();

    virtual void optionalSubRule();

    virtual void setUserExceptions(const MLANG_String& thr);

    virtual void refAction(const MParser_Token_var& action);

    virtual void refArgAction(const MParser_Token_var& action);

    virtual void refCharLiteral(const MParser_Token_var& lit, const MParser_Token_var& label, bool inverted, int autoGenType, bool lastInRule);

    virtual void refCharRange(const MParser_Token_var& t1, const MParser_Token_var& t2, const MParser_Token_var& label, int autoGenType, bool lastInRule);

    virtual void refElementOption(const MParser_Token_var& option, const MParser_Token_var& value);

    virtual void refTokensSpecElementOption(const MParser_Token_var& tok, const MParser_Token_var& option, const MParser_Token_var& value);

    virtual void refExceptionHandler(const MParser_Token_var& exTypeAndName, const MParser_Token_var& action);

    // Header action applies to all parsers and lexers.
    virtual void refHeaderAction(const MParser_Token_var& name, const MParser_Token_var& act);

    MLANG_String getHeaderAction(const MLANG_String& name);

    virtual void refInitAction(const MParser_Token_var& action);

    virtual void refMemberAction(const MParser_Token_var& act);

    virtual void refPreambleAction(const MParser_Token_var& act);

    virtual void refReturnAction(const MParser_Token_var& returnAction);

    virtual void refRule(const MParser_Token_var& idAssign,
                        const MParser_Token_var& r,
                        const MParser_Token_var& label,
                        const MParser_Token_var& args,
                        int autoGenType);

    virtual void refSemPred(const MParser_Token_var& pred);

    virtual void refStringLiteral(const MParser_Token_var& lit,
                                 const MParser_Token_var& label,
                                 int autoGenType,
                                 bool lastInRule);

    /** Reference a token */
    virtual void refToken(const MParser_Token_var& assignId, const MParser_Token_var& t, const MParser_Token_var& label, const MParser_Token_var& args,
                         bool inverted, int autoGenType, bool lastInRule);

    virtual void refTokenRange(const MParser_Token_var& t1, const MParser_Token_var& t2, const MParser_Token_var& label, int autoGenType, bool lastInRule);

    virtual void refTreeSpecifier(const MParser_Token_var& treeSpec);

    virtual void refWildcard(const MParser_Token_var& t, const MParser_Token_var& label, int autoGenType);

    /** Get ready to process a new grammar */
    virtual void reset();

    virtual void setArgOfRuleRef(const MParser_Token_var& argaction);

    /** Set the character vocabulary for a lexer */
    void setCharVocabulary(const MParser_BitSet& b);

    /** setFileOption: Associate an option value with a key.
     * This applies to options for an entire grammar file.
     * @param key The token containing the option name
     * @param value The token containing the option value.
     */
    void setFileOption(const MParser_Token_var& key, const MParser_Token_var& value, const MLANG_String& filename);

    /** setGrammarOption: Associate an option value with a key.
     * This function forwards to Grammar.setOption for some options.
     * @param key The token containing the option name
     * @param value The token containing the option value.
     */
    virtual void setGrammarOption(const MParser_Token_var& key, const MParser_Token_var& value);

    virtual void setRuleOption(const MParser_Token_var& key, const MParser_Token_var& value);

    virtual void setSubruleOption(const MParser_Token_var& key, const MParser_Token_var& value);

    /** Start a new lexer */
    virtual void startLexer(const MLANG_String& file, const MParser_Token_var& name, const MLANG_String& superClass, const MLANG_String& doc);

    /** Start a new parser */
    virtual void startParser(const MLANG_String& file, const MParser_Token_var& name, const MLANG_String& superClass, const MLANG_String& doc);

    /** Start a new tree-walker */
    virtual void startTreeWalker(const MLANG_String& file, const MParser_Token_var& name, const MLANG_String& superClass, const MLANG_String& doc);

    virtual void synPred();

    virtual void zeroOrMoreSubRule();
};



#endif /* __MGRAMMAR_DEFINEGRAMMARSYMBOLS_H__ */
