#ifndef __MGRAMMAR_GRAMMARANALYZER_H__
#define __MGRAMMAR_GRAMMARANALYZER_H__
#include "ace/Basic_Types.h"
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/GrammarAnalyzer.java#1 $
 */

class MGrammar_GrammarAnalyzer;
typedef class MGrammar_GrammarAnalyzer *MGrammar_GrammarAnalyzer_ptr;

/**A GrammarAnalyzer computes lookahead from Grammar (which contains
 * a grammar symbol table) and can then answer questions about the
 * grammar.
 *
 * To access the RuleBlock for a rule name, the grammar symbol table
 * is consulted.
 *
 * There should be no distinction between static & dynamic analysis.
 * In other words, some of the easy analysis can be done statically
 * and then the part that is hard statically can be deferred to
 * parse-time.  Interestingly, computing LL(k) for k>1 lookahead
 * statically is O(|T|^k) where T is the grammar vocabulary, but,
 * is O(k) at run-time (ignoring the large constant associated with
 * the size of the grammar).  In English, the difference can be
 * described as "find the set of all possible k-sequences of input"
 * versus "does this specific k-sequence match?".
 */
class MGrammar_GrammarAnalyzer : public MLANG_Refcountable<ACE_SYNCH_MUTEX>

{
public:
    /**The epsilon token type is an imaginary type used
     * during analysis.  It indicates an incomplete look() computation.
     * Must be kept consistent with Token constants to be between
     * MIN_USER_TYPE and INVALID_TYPE.
     */
    enum
    {
       EPSILON_TYPE = 2,
       NONDETERMINISTIC = ACE_INT32_MAX, // lookahead depth
       LOOKAHEAD_DEPTH_INIT = -1
    };
protected:
   MGrammar_GrammarAnalyzer();
public:
   virtual ~MGrammar_GrammarAnalyzer();

public:

   static bool is_nil (MGrammar_GrammarAnalyzer_ptr analyzer)
   { return analyzer == 0; }

   static MGrammar_GrammarAnalyzer_ptr _nil()
   { return 0; }

   static MGrammar_GrammarAnalyzer_ptr _duplicate(MGrammar_GrammarAnalyzer_ptr analyzer)
   { return ::MLANG_acquire(analyzer); }
};

typedef MLANG_SmartPtr<MGrammar_GrammarAnalyzer> MGrammar_GrammarAnalyzer_var;

#endif /* __MGRAMMAR_GRAMMARANALYZER_H__ */
