#include "MGrammar/Grammar.h"
#include "MGrammar/Tool.h"
#include "MGrammar/GrammarAtom.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/GrammarAtom.java#1 $
 */

/** A GrammarAtom is either a token ref, a character ref, or string.
 * The analysis doesn't care.
 */

MGrammar_GrammarAtom::MGrammar_GrammarAtom (
   MGrammar_Grammar_ptr g,
   const MParser_Token_var& t,
   int autoGenType
) : MGrammar_AlternativeElement(g, t, autoGenType),
    tokenType_(MParser_Token::INVALID_TYPE),
    no_(false)

{
   atomText_ = t->getText();
}

void MGrammar_GrammarAtom::setLabel(const MLANG_String& label)

{
   label_ = label;
}

const MLANG_String& MGrammar_GrammarAtom::getASTNodeType() const

{
   return ASTNodeType_;
}

void MGrammar_GrammarAtom::setASTNodeType(const MLANG_String& type)

{
   ASTNodeType_ = type;
}

void MGrammar_GrammarAtom::setOption (
   const MParser_Token_var& option,
   const MParser_Token_var& value
)

{
   if (option->getText().equals("AST"))
    {
      setASTNodeType(value->getText());
    }
   else
    {
      grammar_->antlrTool_->error("Invalid element option:" + option->getText(),
                           grammar_->getFilename(), option->getLine(), option->getColumn());
    }
}

MLANG_String MGrammar_GrammarAtom::toString() const

{
   MLANG_String s = " ";
   if (label_.length() != 0) s += label_ + ":";
   if (no_) s += "~";
   return s + atomText_;
}

MGrammar_GrammarAtom::~MGrammar_GrammarAtom ()

{
}
