#ifndef __MGRAMMAR_GRAMMARELEMENT_H__
#define __MGRAMMAR_GRAMMARELEMENT_H__
#include "ace/Synch.h"
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MParser/Token.h"
#include "MGrammar/Lookahead.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/GrammarElement.java#1 $
 */

class MGrammar_Grammar;
typedef class MGrammar_Grammar *MGrammar_Grammar_ptr;
class MGrammar_GrammarElement;
typedef class MGrammar_GrammarElement *MGrammar_GrammarElement_ptr;

/**A GrammarElement is a generic node in our
 * data structure that holds a grammar in memory.
 * This data structure can be used for static
 * analysis or for dynamic analysis (during parsing).
 * Every node must know which grammar owns it, how
 * to generate code, and how to do analysis.
 */
class MGrammar_GrammarElement : public MLANG_Refcountable<ACE_SYNCH_MUTEX>

{
public:
    enum
    {
       AUTO_GEN_NONE = 1,
       AUTO_GEN_CARET = 2,
       AUTO_GEN_BANG = 3
    };

    /*
	 * Note that Java does static argument type matching to
	 * determine which function to execute on the receiver.
	 * Here, that implies that we cannot simply say
	 * grammar.generator.gen(this) in GrammarElement or
	 * only CodeGenerator.gen(GrammarElement ge) would
	 * ever be called.
	 */
protected:
    MGrammar_Grammar_ptr grammar_;
    int line_;
    int column_;

public:

    MGrammar_GrammarElement (
       MGrammar_Grammar_ptr g,
       int line = -1,
       int column = -1
      );

    MGrammar_GrammarElement (
       MGrammar_Grammar_ptr g,
       const MParser_Token_var& token
      );

   virtual ~MGrammar_GrammarElement();

public:

   static bool is_nil (MGrammar_GrammarElement_ptr a)
   { return a == 0; }

   static MGrammar_GrammarElement_ptr _nil()
   { return 0; }
   
   static MGrammar_GrammarElement_ptr _duplicate(MGrammar_GrammarElement_ptr ge)
   { return MLANG_acquire (ge); }

//   static MGrammar_GrammarElement_ptr _duplicate(MGrammar_AlternativeElement_ptr element);

public:

    int getLine() const
    {
        return line_;
    }

    void setLine(int new_line)
    {
       line_ = new_line;
    }

    int getColumn() const
    {
        return column_;
    }

    void setColumn(int new_column)
    { column_ = new_column; }

public:

    virtual void generate() ;

    virtual MGrammar_Lookahead look(int k);

    virtual MLANG_String toString() const = 0;
};

typedef MLANG_SmartPtr<MGrammar_GrammarElement> MGrammar_GrammarElement_var;

#endif /* __MGRAMMAR_GRAMMARELEMENT_H__ */
