#include "MIO/File.h"
#include "MIO/Reader.h"
#include "MIO/FileReader.h"
#include "MIO/BufferedReader.h"
#include "MGrammar/TokenSymbol.h"
#include "MGrammar/StringLiteralSymbol.h"
#include "MIO/IOException.h"
#include "MIO/FileNotFoundException.h"
#include "MParser/RecognitionException.h"
#include "MParser/TokenStreamException.h"
#include "MGrammar/ANTLRTokdefLexer.h"
#include "MGrammar/ANTLRTokdefParser.h"
#include "MGrammar/Tool.h"
#include "MGrammar/Grammar.h"
#include "MGrammar/ImportVocabTokenManager.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/ImportVocabTokenManager.java#1 $
 */

// FIXME: it would be nice if the path to the original grammar file was
// also searched.
MGrammar_ImportVocabTokenManager::MGrammar_ImportVocabTokenManager (
   const MGrammar_Grammar_var& grammar,
   const MLANG_String& filename,
   const MLANG_String& name,
   MGrammar_Tool_ptr tool
) : MGrammar_SimpleTokenManager (name, tool),
    filename_(filename),
    grammar_(grammar)

{
   // Figure out exactly where the file lives.  Check $PWD first,
   // and then search in -o <output_dir>.
   //
   MIO_File_var grammarFile = new MIO_File(filename);

   if (!grammarFile->exists())
    {
      grammarFile = new MIO_File(antlrTool_->getOutputDirectory(), filename_);

      if (!grammarFile->exists())
       {
         antlrTool_->panic("Cannot find importVocab file '" + filename + "'");
       }
    }

   setReadOnly(true);

   // Read a file with lines of the form ID=number
   try
    {
       MIO_Reader_var fileIn = new MIO_BufferedReader(new MIO_FileReader(grammarFile));
       ANTLRTokdefLexer_var tokdefLexer = new ANTLRTokdefLexer(fileIn);
       ANTLRTokdefParser_var tokdefParser = 
           new ANTLRTokdefParser(MParser_TokenStream::_duplicate(tokdefLexer));
       tokdefParser->setTool(antlrTool_);
       tokdefParser->setFilename(filename_);
       tokdefParser->file(this);
    }
   catch (MIO_FileNotFoundException& fnf)
    {
       antlrTool_->panic("Cannot find importVocab file '" + filename_ + "'");
    }
   catch (MParser_RecognitionException& ex)
    {
       antlrTool_->panic("Error parsing importVocab file '" + filename_ + "': " + ex.toString().c_str());
    }
   catch (MParser_TokenStreamException& ex)
    {
       antlrTool_->panic("Error reading importVocab file '" + filename_ + "'");
    }
}

MGrammar_ImportVocabTokenManager::MGrammar_ImportVocabTokenManager (
   const MGrammar_ImportVocabTokenManager& tm
) : MGrammar_SimpleTokenManager (tm.name_, tm.antlrTool_),
    filename_(tm.filename_),
    grammar_(tm.grammar_)


{
}

MGrammar_TokenManager_var MGrammar_ImportVocabTokenManager::clone()

{
   return new MGrammar_ImportVocabTokenManager (*this);
}

/** define a token. */
void MGrammar_ImportVocabTokenManager::define (
   const MGrammar_TokenSymbol_var& ts
)

{
   MGrammar_SimpleTokenManager::define(ts);
}

/** define a token.  Intended for use only when reading the importVocab file. */
void MGrammar_ImportVocabTokenManager::define (
   const MLANG_String& s,
   int ttype
)

{
   MGrammar_TokenSymbol_var ts;
   if (s.startsWith("\""))
    {
      ts = new MGrammar_StringLiteralSymbol(s);
    }
   else
    {
      ts = new MGrammar_TokenSymbol(s);
    }
   ts->setTokenType(ttype);
   MGrammar_SimpleTokenManager::define(ts);
   maxToken_ = (ttype + 1) > maxToken_ ? (ttype + 1) : maxToken_;	// record maximum token type
}

MGrammar_ImportVocabTokenManager::~MGrammar_ImportVocabTokenManager ()

{
}
