#include "MGrammar/JavaCharFormatter.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/JavaCharFormatter.java#1 $
 */

MGrammar_JavaCharFormatter::MGrammar_JavaCharFormatter()

{
}

MLANG_String MGrammar_JavaCharFormatter::escapeChar(int c, bool forCharLiteral) const

{
   switch (c)
   {
   //		case GrammarAnalyzer.EPSILON_TYPE : return "<end-of-token>";
   case '\n':
        return "\\n";
   case '\t':
        return "\\t";
   case '\r':
        return "\\r";
   case '\\':
        return "\\\\";
   case '\'':
        return forCharLiteral ? "\\'" : "'";
   case '"':
        return forCharLiteral ? "\"" : "\\\"";
   default :
        if (c < ' ' || c > 126)
         {
           if ((0x0000 <= c) && (c <= 0x000F))
            {
              return "\\u000" +MLANG_String::valueOf(c, 16);
            }
           else if ((0x0010 <= c) && (c <= 0x00FF))
            {
              return "\\u00" +MLANG_String::valueOf(c, 16);
            }
           else if ((0x0100 <= c) && (c <= 0x0FFF)) {
              return "\\u0" +MLANG_String::valueOf(c, 16);
            }
           else
            {
              return "\\u" +MLANG_String::valueOf(c, 16);
            }
         }
        else
         {
            return MLANG_String::valueOf((char)c);
         }
   }
}

/** Converts a String into a representation that can be use as a literal
 * when surrounded by double-quotes.
 * @param s The String to be changed into a literal
 */
MLANG_String MGrammar_JavaCharFormatter::escapeString(const MLANG_String& s) const

{
   MLANG_String retval;
   for (size_t i = 0; i < s.length(); i++)
    {
       retval += escapeChar(s.charAt(i), false);
    }
   return retval;
}

/** Given a character value, return a string representing the character
 * literal that can be recognized by the target language compiler.
 * This works for languages that use single-quotes for character literals.
 * Code-generators for languages should override this method.
 * @param c   The character of interest.
 */
MLANG_String MGrammar_JavaCharFormatter::literalChar(int c) const

{
   return "'" + escapeChar(c, true) + "'";
}

/** Converts a String into a string literal
 * This works for languages that use double-quotes for string literals.
 * Code-generators for languages should override this method.
 * @param s The String to be changed into a literal
 */
MLANG_String MGrammar_JavaCharFormatter::literalString(const MLANG_String& s) const

{
   return "\"" + escapeString(s) + "\"";
}

MGrammar_JavaCharFormatter::~MGrammar_JavaCharFormatter()

{
}
