#ifndef __MGRAMMAR_JAVACHARFORMATTER_H__
#define __MGRAMMAR_JAVACHARFORMATTER_H__
#include "MLANG/SmartPtr.h"
#include "MParser/CharFormatter.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/JavaCharFormatter.java#1 $
 */

class MGrammar_JavaCharFormatter : public MParser_CharFormatter

{
public:
   MGrammar_JavaCharFormatter();

   virtual ~MGrammar_JavaCharFormatter();

public:

   /** Given a character value, return a string representing the character
    * that can be embedded inside a string literal or character literal
    * This works for Java/C/C++ code-generation and languages with compatible
    * special-character-escapment.
    * Code-generators for languages should override this method.
    * @param c   The character of interest.
    * @param forCharLiteral  true to escape for char literal, false for string literal
    */
   MLANG_String escapeChar(int c, bool forCharLiteral) const;

   /** Converts a String into a representation that can be use as a literal
    * when surrounded by double-quotes.
    * @param s The String to be changed into a literal
    */
   MLANG_String escapeString(const MLANG_String& s) const;

   /** Given a character value, return a string representing the character
    * literal that can be recognized by the target language compiler.
    * This works for languages that use single-quotes for character literals.
    * Code-generators for languages should override this method.
    * @param c   The character of interest.
    */
   MLANG_String literalChar(int c) const;

   /** Converts a String into a string literal
    * This works for languages that use double-quotes for string literals.
    * Code-generators for languages should override this method.
    * @param s The String to be changed into a literal
    */
   MLANG_String literalString(const MLANG_String& s) const;
};

#endif /* __MGRAMMAR_JAVACHARFORMATTER_H__ */
