#include "MGrammar/ANTLRTokenTypes.h"
#include "MGrammar/CodeGenerator.h"
#include "MGrammar/Tool.h"
#include "MGrammar/LexerGrammar.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/LexerGrammar.java#1 $
 */


MGrammar_LexerGrammar::MGrammar_LexerGrammar (
   const MLANG_String& className,
   MGrammar_Tool_ptr tool,
   const MLANG_String& superClass
) : MGrammar_Grammar (className, tool, superClass),
    testLiterals_(true),
    caseSensitiveLiterals_(true),
    caseSensitive_(true),
    filterMode_(false)

{
   // Lexer usually has no default error handling
   defaultErrorHandler_ = false;
}

/** Top-level call to generate the code	 */
void MGrammar_LexerGrammar::generate()

{
   generator_->gen(this);
}

const MLANG_String MGrammar_LexerGrammar::getSuperClass() const

{
   // If debugging, use debugger version of scanner
   if (debuggingOutput_)
      return "debug.DebuggingCharScanner";
   return "CharScanner";
}

/**Process command line arguments.
 * -trace			have all rules call traceIn/traceOut
 * -traceLexer		have lexical rules call traceIn/traceOut
 * -debug			generate debugging output for parser debugger
 */
void MGrammar_LexerGrammar::processArguments(ACE_ARGV& args)

{
   for (int i = 0; i < args.argc(); i++)
    {
       if (MLANG_String(args[i]).equals("-trace"))
        {
          traceRules_ = true;
          antlrTool_->setArgOK(i);
        }
       else if (MLANG_String(args[i]).equals("-traceLexer"))
        {
          traceRules_ = true;
          antlrTool_->setArgOK(i);
        }
       else if (MLANG_String(args[i]).equals("-debug"))
        {
          debuggingOutput_ = true;
          antlrTool_->setArgOK(i);
        }
    }
}

/** Set lexer options */
bool MGrammar_LexerGrammar::setOption (
   const MLANG_String& key, 
   const MParser_Token_var& value
)

{
   MLANG_String s = value->getText();
   if (key.equals("buildAST"))
    {
      antlrTool_->warning("buildAST option is not valid for lexer", getFilename(), value->getLine(), value->getColumn());
      return true;
    }
   if (key.equals("testLiterals"))
    {
      if (s.equals("true"))
       {
         testLiterals_ = true;
       }
      else if (s.equals("false"))
       {
         testLiterals_ = false;
       }
      else
       {
         antlrTool_->warning("testLiterals option must be true or false", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("interactive"))
    {
      if (s.equals("true"))
       {
         interactive_ = true;
       }
      else if (s.equals("false"))
       {
         interactive_ = false;
       }
      else
       {
         antlrTool_->error("interactive option must be true or false", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("caseSensitive"))
    {
      if (s.equals("true"))
       {
         caseSensitive_ = true;
       }
      else if (s.equals("false"))
       {
         caseSensitive_ = false;
       }
      else
       {
         antlrTool_->warning("caseSensitive option must be true or false", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("caseSensitiveLiterals"))
    {
      if (s.equals("true"))
       {
         caseSensitiveLiterals_ = true;
       }
      else if (s.equals("false"))
       {
         caseSensitiveLiterals_ = false;
       }
      else
       {
         antlrTool_->warning("caseSensitiveLiterals option must be true or false", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("filter"))
    {
      if (s.equals("true"))
       {
         filterMode_ = true;
       }
      else if (s.equals("false"))
       {
         filterMode_ = false;
       }
      else if (value->getType() == ANTLRTokenTypes::TOKEN_REF)
       {
         filterMode_ = true;
         filterRule_ = s;
       }
      else
       {
         antlrTool_->warning("filter option must be true, false, or a lexer rule name", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("longestPossible"))
    {
      antlrTool_->warning("longestPossible option has been deprecated; ignoring it...", getFilename(), value->getLine(), value->getColumn());
      return true;
    }
   if (MGrammar_Grammar::setOption(key, value))
    {
      return true;
    }
   antlrTool_->error("Invalid option: " + key, getFilename(), value->getLine(), value->getColumn());
   return false;
}

MGrammar_LexerGrammar::~MGrammar_LexerGrammar ()

{
}
