#ifndef __MGRAMMAR_LEXERGRAMMAR_H__
#define __MGRAMMAR_LEXERGRAMMAR_H__
#include "ace/ARGV.h"
#include "MLANG/SmartPtr.h"
#include "MParser/BitSet.h"
#include "MParser/Token.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/LexerGrammar.java#1 $
 */

class MGrammar_LexerGrammar;
typedef class MGrammar_LexerGrammar *MGrammar_LexerGrammar_ptr;
typedef MLANG_SmartPtr<MGrammar_LexerGrammar> MGrammar_LexerGrammar_var;

#include "MGrammar/Grammar.h"

/** Lexer-specific grammar subclass */
class MGrammar_LexerGrammar : public MGrammar_Grammar

{
public:
   // character set used by lexer
   MParser_BitSet charVocabulary_;
    // true if the lexer generates literal testing code for nextToken
   bool testLiterals_;
   // true if the lexer generates case-sensitive LA(k) testing
   bool caseSensitiveLiterals_;
   /** true if the lexer generates case-sensitive literals testing */
   bool caseSensitive_;
   /** true if lexer is to ignore all unrecognized tokens */
   bool filterMode_;

   /** if filterMode is true, then filterRule can indicate an optional
    *  rule to use as the scarf language.  If null, programmer used
    *  plain "filter=true" not "filter=rule".
    */
   MLANG_String filterRule_;

public:

   MGrammar_LexerGrammar (
      const MLANG_String& className,
      MGrammar_Tool_ptr tool,
      const MLANG_String& superClass
    );

   virtual ~MGrammar_LexerGrammar ();

public:

   static bool is_nil (MGrammar_LexerGrammar_ptr g)
   { return g == 0; }

   static MGrammar_LexerGrammar_ptr _nil()
   { return 0; }
   
   static MGrammar_LexerGrammar_ptr _duplicate(MGrammar_LexerGrammar_ptr g)
   { return MLANG_acquire (g); }

   static MGrammar_LexerGrammar_ptr _narrow(MGrammar_Grammar_ptr g)
   { return MLANG_acquire (ACE_dynamic_cast(MGrammar_LexerGrammar_ptr, g)); }

public:

   bool caseSensitiveLiterals() const
   { return caseSensitiveLiterals_; }

   bool caseSensitive() const
   { return caseSensitive_; }

   bool filterMode() const
   { return filterMode_; }

   const MLANG_String& filterRule() const 
   { return filterRule_; }

   const MParser_BitSet& charVocabulary() const
   { return charVocabulary_; }

public:

   virtual bool isLexer() const { return true; }

    /** Top-level call to generate the code	 */
   void generate();

   const MLANG_String getSuperClass() const;

    // Get the testLiterals option value
    bool getTestLiterals() const
    {
        return testLiterals_;
    }

    /**Process command line arguments.
     * -trace			have all rules call traceIn/traceOut
     * -traceLexer		have lexical rules call traceIn/traceOut
     * -debug			generate debugging output for parser debugger
     */
    void processArguments(ACE_ARGV& args);

    /** Set the character vocabulary used by the lexer */
    void setCharVocabulary(const MParser_BitSet& b)
    {
       charVocabulary_ = b;
    }

    /** Set lexer options */
    bool setOption(const MLANG_String& key, const MParser_Token_var& value);
};


#endif /* __MGRAMMAR_LEXERGRAMMAR_H__ */
