#ifndef __MGRAMMAR_MAKEGRAMMAR_H__
#define __MGRAMMAR_MAKEGRAMMAR_H__
#include <stack>
#include "ace/ARGV.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MLANG/Vector_T.h"
#include "MParser/Token.h"

class MGrammar_MakeGrammar;
typedef class MGrammar_MakeGrammar *MGrammar_MakeGrammar_ptr;

#include "MGrammar/ExceptionSpec.h"
#include "MGrammar/AlternativeElement.h"
#include "MGrammar/AlternativeBlock.h"
#include "MGrammar/RuleSymbol.h"
#include "MGrammar/RuleRefElement.h"
#include "MGrammar/RuleEndElement.h"
#include "MGrammar/RuleBlock.h"
#include "MGrammar/BlockContext.h"
#include "MGrammar/LLkAnalyzer.h"
#include "MGrammar/DefineGrammarSymbols.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/MakeGrammar.java#1 $
 */

class MGrammar_MakeGrammar : public MGrammar_DefineGrammarSymbols

{
protected:
   typedef MGrammar_DefineGrammarSymbols super;
   std::stack<MGrammar_BlockContext_var> blocks_;
   MGrammar_RuleRefElement_var lastRuleRef_;
   MGrammar_RuleEndElement_var ruleEnd_;   // used if not nested
   MGrammar_RuleBlock_var ruleBlock_; // points to block of current rule.
   int nested_;  // nesting inside a subrule
   bool grammarError_;

public:

    MGrammar_ExceptionSpec_var currentExceptionSpec_;

public:

    MGrammar_MakeGrammar (
       MGrammar_Tool_ptr tool,
       ACE_ARGV& args, 
       MGrammar_LLkAnalyzer_ptr analyzer
     );

    virtual ~MGrammar_MakeGrammar ();

public:

   static bool is_nil (MGrammar_MakeGrammar_ptr mg)
   { return mg == 0; }

   static MGrammar_MakeGrammar_ptr _nil()
   { return 0; }

   static MGrammar_MakeGrammar_ptr _duplicate(MGrammar_MakeGrammar_ptr mg)
   { return MLANG_acquire(mg); }

public:

    /** Abort the processing of a grammar (due to syntax errors) */
    void abortGrammar();

protected:

    void addElementToCurrentAlt (
       const MGrammar_AlternativeElement_var& e
      );

    void beginAlt(bool doAutoGen);

    void beginChildList();

    /** Add an exception group to a rule (currently a no-op) */
    void beginExceptionGroup();

    /** Add an exception spec to an exception group or rule block */
    void beginExceptionSpec (
       const MParser_Token_var& label
     );

    void beginSubRule (
       const MParser_Token_var& label, 
       const MParser_Token_var& start,
       bool no
     );

    void beginTree (
       const MParser_Token_var& tok
     );

    MGrammar_BlockContext_var context();

public:
    /**Used to build nextToken() for the lexer.
     * This builds a rule which has every "public" rule in the given Vector of
     * rules as it's alternate.  Each rule ref generates a Token object.
     * @param g  The Grammar that is being processed
     * @param lexRules A vector of lexer rules that will be used to create an alternate block.
     * @param rname The name of the resulting rule.
     */
    static MGrammar_RuleBlock_var createNextTokenRule (
       const MGrammar_Grammar_var& g,
       const MLANG_Vector<MGrammar_RuleSymbol_var>& lexRules,
       const MLANG_String& rname
      );

private:

    /** Return block as if they had typed: "( rule )?" */
    MGrammar_AlternativeBlock_var createOptionalRuleRef (
       const MLANG_String& rule,
       const MParser_Token_var& start
      );

public:

    void defineRuleName (
       const MParser_Token_var& r,
       const MLANG_String& access,
       bool ruleAutoGen,
       const MLANG_String& docComment
     );

    void endAlt();

    void endChildList();

    void endExceptionGroup();

    void endExceptionSpec();

    /** Called at the end of processing a grammar */
    void endGrammar();

    void endRule(const MLANG_String& rule);

    void endSubRule();

    void endTree();

    /** Remember that a major error occured in the grammar */
    void hasError() 
    {
        grammarError_ = true;
    }

private:

    void labelElement (
       const MGrammar_AlternativeElement_var& el, 
       const MParser_Token_var& label
      );

public:

    void noAutoGenSubRule();

    void oneOrMoreSubRule();

    void optionalSubRule();

    void refAction(const MParser_Token_var& action);

    void setUserExceptions(const MLANG_String& thr);

    // Only called for rule blocks
    void refArgAction (
       const MParser_Token_var& action
     );

    void refCharLiteral (
       const MParser_Token_var& lit, 
       const MParser_Token_var& label, 
       bool inverted, 
       int autoGenType, 
       bool lastInRule
     );

    void refCharRange (
       const MParser_Token_var& t1, 
       const MParser_Token_var& t2, 
       const MParser_Token_var& label, 
       int autoGenType, 
       bool lastInRule
     );

    void refTokensSpecElementOption (
       const MParser_Token_var& tok,
       const MParser_Token_var& option,
       const MParser_Token_var& value
      );

    void refElementOption (
       const MParser_Token_var& option, 
       const MParser_Token_var& value
      );

    /** Add an exception handler to an exception spec */
    void refExceptionHandler (
       const MParser_Token_var& exTypeAndName,
       const MParser_Token_var& action
      );

    void refInitAction (
       const MParser_Token_var& action
      );

    void refMemberAction (
       const MParser_Token_var& act
      );

    void refPreambleAction (
       const MParser_Token_var& act
     );

    // Only called for rule blocks
    void refReturnAction (
       const MParser_Token_var& returnAction
     );

    void refRule (
       const MParser_Token_var& idAssign,
       const MParser_Token_var& r,
       const MParser_Token_var& label,
       const MParser_Token_var& args,
       int autoGenType
     );

    void refSemPred (
       const MParser_Token_var& pred
     );

    void refStringLiteral (
       const MParser_Token_var& lit, 
       const MParser_Token_var& label, 
       int autoGenType, 
       bool lastInRule
     );

    void refToken (
       const MParser_Token_var& idAssign, 
       const MParser_Token_var& t, 
       const MParser_Token_var& label, 
       const MParser_Token_var& args,
       bool inverted, 
       int autoGenType, 
       bool lastInRule
      );

    void refTokenRange (
       const MParser_Token_var& t1, 
       const MParser_Token_var& t2, 
       const MParser_Token_var& label, 
       int autoGenType, 
       bool lastInRule
      );

    void refTreeSpecifier (
       const MParser_Token_var& treeSpec
     );

    void refWildcard (
       const MParser_Token_var& t, 
       const MParser_Token_var& label, 
       int autoGenType
      );

    /** Get ready to process a new grammar */
    void reset();

    void setArgOfRuleRef (
       const MParser_Token_var& argAction
     );

    static void setBlock (
       const MGrammar_AlternativeBlock_var& b,
       const MGrammar_AlternativeBlock_var& src
     );

    void setRuleOption (
       const MParser_Token_var& key,
       const MParser_Token_var& value
      );

    void setSubruleOption (
       const MParser_Token_var& key, 
       const MParser_Token_var& value
      );

    void synPred();

    void zeroOrMoreSubRule();
};

typedef MLANG_SmartPtr<MGrammar_MakeGrammar> MGrammar_MakeGrammar_var;

#endif /* __MGRAMMAR_MAKEGRAMMAR_H__ */
