#include "MGrammar/NameSpace.h"

/**
 * ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * Container for a C++ namespace specification.  Namespaces can be
 * nested, so this contains a vector of all the nested names.
 *
 * @author David Wagner (JPL/Caltech) 8-12-00
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/NameSpace.java#1 $
 */

MGrammar_NameSpace::MGrammar_NameSpace(const MLANG_String& name)
   : _name(name)

{
   parse(_name);
}

MLANG_String MGrammar_NameSpace::getName()

{
   return _name;
}
	
/**
 * Parse a C++ namespace declaration into seperate names
 * splitting on ::  We could easily parameterize this to make
 * the delimiter a language-specific parameter, or use subclasses
 * to support C++ namespaces versus java packages. -DAW
 */
void MGrammar_NameSpace::parse (
   const MLANG_String& name
)

{
#if 0
    StringTokenizer tok = new StringTokenizer(name, "::");
    while (tok.hasMoreTokens())
         names.addElement(tok.nextToken());
#else
    assert(0);
#endif
}

/**
 * Method to generate the required C++ namespace declarations
 */
void MGrammar_NameSpace::emitDeclarations (
   const MIO_PrintWriter_var& out
)

{
   for (size_t i = 0; i< names_.size(); i++)
    {
       out->println("ANTLR_BEGIN_NAMESPACE(" + names_[i] + ")");
    }
}

/**
 * Method to generate the required C++ namespace closures
 */
void MGrammar_NameSpace::emitClosures (
   const MIO_PrintWriter_var& out
)

{
   for (size_t i = 0; i < names_.size(); ++i)
       out->println("ANTLR_END_NAMESPACE");
}

MGrammar_NameSpace::~MGrammar_NameSpace()

{
}
