#ifndef __MGRAMMAR_NAMESPACE_H__
#define __MGRAMMAR_NAMESPACE_H__
#include "MLANG/SmartPtr.h"
#include "MLANG/Refcountable.h"
#include "MLANG/String.h"
#include "MLANG/Vector_T.h"
#include "MIO/PrintWriter.h"

class MGrammar_NameSpace;
typedef MGrammar_NameSpace *MGrammar_NameSpace_ptr;
typedef MLANG_SmartPtr<MGrammar_NameSpace> MGrammar_NameSpace_var;

/**
 * ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * Container for a C++ namespace specification.  Namespaces can be
 * nested, so this contains a vector of all the nested names.
 *
 * @author David Wagner (JPL/Caltech) 8-12-00
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/NameSpace.java#1 $
 */

class MGrammar_NameSpace : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
private:
   MLANG_Vector<MLANG_String> names_;
private:
   MLANG_String _name;

public:
   MGrammar_NameSpace(const MLANG_String& name);

   virtual ~MGrammar_NameSpace();

public:

   static bool is_nil (MGrammar_NameSpace_ptr a)
   { return a == 0; }

   static MGrammar_NameSpace_ptr _nil()
   { return 0; }

   static MGrammar_NameSpace_ptr _duplicate(MGrammar_NameSpace_ptr t)
   { return MLANG_acquire(t); }

public:
   const MLANG_Vector<MLANG_String>& names() const
   { return names_; }

   MLANG_String getName();
	
    /**
     * Parse a C++ namespace declaration into seperate names
     * splitting on ::  We could easily parameterize this to make
     * the delimiter a language-specific parameter, or use subclasses
     * to support C++ namespaces versus java packages. -DAW
     */
protected:
   void parse(const MLANG_String& name);

public:
    /**
     * Method to generate the required C++ namespace declarations
     */
    void emitDeclarations(const MIO_PrintWriter_var& out);

public:
    /**
     * Method to generate the required C++ namespace closures
     */
    void emitClosures(const MIO_PrintWriter_var& out);
};

#endif /* __MGRAMMAR_NAMESPACE_H__ */
