#include "MGrammar/CodeGenerator.h"
#include "MGrammar/ParserGrammar.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/ParserGrammar.java#1 $
 */

MGrammar_ParserGrammar::MGrammar_ParserGrammar (
   const MLANG_String& className,
   MGrammar_Tool_ptr tool,
   const MLANG_String& superClass
) : MGrammar_Grammar(className, tool, superClass)

{
}

/** Top-level call to generate the code for this grammar */
void MGrammar_ParserGrammar::generate()

{
   generator_->gen(this);
}

// Get name of class from which generated parser/lexer inherits
const MLANG_String MGrammar_ParserGrammar::getSuperClass() const

{
   // if debugging, choose the debugging version of the parser
   if (debuggingOutput_)
      return "debug.LLkDebuggingParser";
   else
      return "LLkParser";
}

/**Process command line arguments.
 * -trace			have all rules call traceIn/traceOut
 * -traceParser		have parser rules call traceIn/traceOut
 * -debug			generate debugging output for parser debugger
 */
void MGrammar_ParserGrammar::processArguments(ACE_ARGV& args)

{
   for (int i = 1; i < args.argc(); i++)
    {
       if (MLANG_String(args[i]).equals("-trace"))
        {
          traceRules_ = true;
          antlrTool_->setArgOK(i);
        }
       else if (MLANG_String(args[i]).equals("-traceParser"))
        {
          traceRules_ = true;
          antlrTool_->setArgOK(i);
        }
       else if (MLANG_String(args[i]).equals("-debug"))
        {
          debuggingOutput_ = true;
          antlrTool_->setArgOK(i);
        }
    }
}

/** Set parser options -- performs action on the following options:
 */
bool MGrammar_ParserGrammar::setOption (
   const MLANG_String& key,
   const MParser_Token_var& value

)

{
   MLANG_String s = value->getText();
   if (key.equals("buildAST"))
    {
      if (s.equals("true"))
       {
         buildAST_ = true;
       }
      else if (s.equals("false"))
       {
         buildAST_ = false;
       }
      else
       {
         antlrTool_->error("buildAST option must be true or false", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("interactive"))
    {
      if (s.equals("true"))
       {
         interactive_ = true;
       }
      else if (s.equals("false"))
       {
         interactive_ = false;
       }
      else
       {
         antlrTool_->error("interactive option must be true or false", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("ASTLabelType"))
    {
      MGrammar_Grammar::setOption(key, value);
      return true;
    }
   if (MGrammar_Grammar::setOption(key, value))
    {
      return true;
    }
   antlrTool_->error("Invalid option: " + key, getFilename(), value->getLine(), value->getColumn());
   return false;
}

MGrammar_ParserGrammar::~MGrammar_ParserGrammar ()

{
}
