#include "MLANG/String.h"
#include "MParser/BitSet.h"
#include "MParser/Token.h"
#include "MParser/System.h"
#include "MParser/SemanticException.h"
#include "MParser/NoViableAltException.h"
#include "MParser/CharScanner.h"
#include "MGrammar/Preprocessor/Grammar.h"
#include "MGrammar/Preprocessor/Option.h"
#include "MGrammar/Preprocessor/Rule.h"
#include "MGrammar/Preprocessor/Preprocessor.h"

/** In order to make it so existing subclasses don't break, we won't require
 * that the antlr.Tool instance be passed as a constructor element. Instead,
 * the antlr.Tool instance should register itself via {@link #initTool(antlr.Tool)}
 * @throws IllegalStateException if a tool has already been registered
 * @since 2.7.2
 */
void MGrammar_Preprocessor::setTool (
   MGrammar_Tool_ptr tool
)

{
   if (antlrTool_ == 0)

    {
      antlrTool_ = tool;
    }
   else
    {
#if 0
      throw MParser_IllegalStateException("antlr.Tool already registered");
#else
      assert(0);
#endif
    }
}

/** @since 2.7.2 */
MGrammar_Tool_ptr MGrammar_Preprocessor::getTool()

{
   return antlrTool_;
}

/** Delegates the error message to the tool if any was registered via
 *  {@link #initTool(antlr.Tool)}
 *  @since 2.7.2
 */
void MGrammar_Preprocessor::reportError(const MLANG_String& s)

{
   if (getTool() != 0)

    {
      getTool()->error(s, getFilename(), -1, -1);
    }
   else
    {
      super::reportError(s);
    }
}

/** Delegates the error message to the tool if any was registered via
 *  {@link #initTool(antlr.Tool)}
 *  @since 2.7.2
 */
void MGrammar_Preprocessor::reportError (
   MParser_RecognitionException& e
)

{
   if (getTool() != 0)

    {
      getTool()->error(e.getErrorMessage().c_str(), e.getFilename(), e.getLine(), e.getColumn());
    }
   else
    {
      super::reportError(e);
    }
}

/** Delegates the warning message to the tool if any was registered via
 *  {@link #initTool(antlr.Tool)}
 *  @since 2.7.2
 */
void MGrammar_Preprocessor::reportWarning (
   const MLANG_String& s
)

{
   if (getTool() != 0)

    {
       getTool()->warning(s, getFilename(), -1, -1);
    }
   else
    {
       super::reportWarning(s);
    }
}

const char * MGrammar_Preprocessor::_tokenNames[] =

{
   "<0>",
   "EOF",
   "<2>",
   "NULL_TREE_LOOKAHEAD",
   "\"tokens\"",
   "HEADER_ACTION",
   "SUBRULE_BLOCK",
   "ACTION",
   "\"class\"",
   "ID",
   "\"extends\"",
   "SEMI",
   "TOKENS_SPEC",
   "OPTIONS_START",
   "ASSIGN_RHS",
   "RCURLY",
   "\"protected\"",
   "\"private\"",
   "\"public\"",
   "BANG",
   "ARG_ACTION",
   "\"returns\"",
   "RULE_BLOCK",
   "\"throws\"",
   "COMMA",
   "\"exception\"",
   "\"catch\"",
   "ALT",
   "ELEMENT",
   "LPAREN",
   "RPAREN",
   "ID_OR_KEYWORD",
   "CURLY_BLOCK_SCARF",
   "WS",
   "NEWLINE",
   "COMMENT",
   "SL_COMMENT",
   "ML_COMMENT",
   "CHAR_LITERAL",
   "STRING_LITERAL",
   "ESC",
   "DIGIT",
   "XDIGIT"
};

MGrammar_Preprocessor::MGrammar_Preprocessor (
   const MParser_TokenBuffer_var& tokenBuf,
   int k
) : super(tokenBuf,k)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

MGrammar_Preprocessor::MGrammar_Preprocessor (
   const MParser_TokenBuffer_var& tokenBuf
) : super(tokenBuf,1)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

MGrammar_Preprocessor::MGrammar_Preprocessor (
   const MParser_TokenStream_var& lexer,
   int k
) : super(lexer,k)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

MGrammar_Preprocessor::MGrammar_Preprocessor (
   const MParser_TokenStream_var& lexer
) : super(lexer,1)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

MGrammar_Preprocessor::MGrammar_Preprocessor (
   const MParser_ParserSharedInputState_var& state
) : super(state,1)

{
   setTokenNames(_tokenNames, sizeof(_tokenNames)/sizeof(_tokenNames[0]));
}

void MGrammar_Preprocessor::grammarFile (
   const MGrammar_Hierarchy_var& hier,
   const MLANG_String& file
)

{
   MParser_Token_var  hdr;
               
   MGrammar_PreprocessorGrammar_var gr;
   MGrammar_PreprocessorGrammar::OPTIONS opt;
                
                
   try
    {      // for error handling
       {
          LABEL(_loop3)
          do
           {
               if ((LA(1)==HEADER_ACTION))

                {
                  hdr = LT(1);
                  match(HEADER_ACTION);
                  hier->getFile(file)->addHeaderAction(hdr->getText());
                }
               else
                {
                  BREAK(_loop3);
                }
                      
           }
          while (true);
       }
       {
          switch ( LA(1))

          {
          case OPTIONS_START:
               {
                 opt=optionSpec(MGrammar_PreprocessorGrammar::_nil());
                 break;
               }
          case EOF:
          case ACTION:
          case LITERAL_class:
               {
                 break;
               }
          default:
               {
                 throw MParser_NoViableAltException(LT(1), getFilename());
               }
          }
       }
       {
          LABEL(_loop6)
          do
           {
              if ((LA(1)==ACTION||LA(1)==LITERAL_class))

               {
                 gr=class_def(file, hier);
                      	
                 if (!MGrammar_PreprocessorGrammar::is_nil(gr)
                 &&  !opt.is_null())

                  {
                    hier->getFile(file)->setOptions(opt);
                  }
                 if (!MGrammar_PreprocessorGrammar::is_nil(gr))

                  {
                    gr->setFileName(file);
                    hier->addGrammar(gr);
                  }
                                            
               }
              else
               {
                 BREAK(_loop6);
               }
                      
           }
          while (true);
       }
      match(MParser_Token::EOF_TYPE);
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_0);
    }
}
	
MGrammar_PreprocessorGrammar::OPTIONS  MGrammar_Preprocessor::optionSpec (
   const MGrammar_PreprocessorGrammar_var& gr
)

{
    MGrammar_PreprocessorGrammar::OPTIONS options;
                
    MParser_Token_var  op;
    MParser_Token_var  rhs;
                
    try
     {      // for error handling
        match(OPTIONS_START);
        {
           LABEL(_loop18)
           do
            {
              if ((LA(1)==ID))

               {
                 op = LT(1);
                 match(ID);
                 rhs = LT(1);
                 match(ASSIGN_RHS);
                   	
                 MGrammar_PreprocessorOption_var newOp = 
                    new MGrammar_PreprocessorOption(op->getText(),rhs->getText(),gr);
                 options.appendElement(newOp->getName(),newOp);
                 if (!MGrammar_PreprocessorGrammar::is_nil(gr) 
                 &&  op->getText().equals("importVocab"))

                  {
                     gr->specifiedVocabulary(true);
                     gr->importVocab(rhs->getText());
                  }
                 else if (!MGrammar_PreprocessorGrammar::is_nil(gr)
                      &&  op->getText().equals("exportVocab") )

                  {
                     // don't want ';' included in outputVocab.
                     // This is heinously inconsistent!  Ugh.
                     gr->exportVocab(rhs->getText().substring(0,rhs->getText().length()-1));
                     gr->exportVocab (gr->exportVocab().trim());
                  }
                                            	
               }
              else
               {
                 BREAK(_loop18);
               }
                      
            }
           while (true);
        }
       match(RCURLY);
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_1);
    }
   return options;
}
	
MGrammar_PreprocessorGrammar_var  MGrammar_Preprocessor::class_def (
   const MLANG_String& file,
   const MGrammar_Hierarchy_var& hier
)

{
   MGrammar_PreprocessorGrammar_var gr;
                  
   MParser_Token_var  preamble;
   MParser_Token_var  sub;
   MParser_Token_var  sup;
   MParser_Token_var  tk;
   MParser_Token_var  memberA;
                
   MGrammar_PreprocessorGrammar::RULES rules;
   MGrammar_PreprocessorGrammar::OPTIONS  classOptions;
   MLANG_String sc;
 
   try
    {      // for error handling
       {
          switch ( LA(1))

          {
          case ACTION:
               {
                 preamble = LT(1);
                 match(ACTION);
                 break;
               }
          case LITERAL_class:
               {
                 break;
               }
          default:
               {
                 throw MParser_NoViableAltException(LT(1), getFilename());
               }
          }
       }
       match(LITERAL_class);
       sub = LT(1);
       match(ID);
       match(LITERAL_extends);
       sup = LT(1);
       match(ID);
       {
          switch ( LA(1))

          {
          case SUBRULE_BLOCK:
               {
                 sc=superClass();
                 break;
               }
          case SEMI:
               {
                 break;
               }
          default:
               {
                 throw MParser_NoViableAltException(LT(1), getFilename());
               }
          }
       }
       match(SEMI);
                       
       gr = hier->getGrammar(sub->getText());
       if (!MGrammar_PreprocessorGrammar::is_nil(gr))

        {
          //                      antlr.Tool.toolError("redefinition of grammar "+gr.getName()+" ignored");
          gr=MGrammar_PreprocessorGrammar_var();
          throw MParser_SemanticException("redefinition of grammar "+sub->getText(), file, sub->getLine(), sub->getColumn());
        }
       else
        {
          gr = new MGrammar_PreprocessorGrammar(hier->getTool(), sub->getText(), sup->getText(), rules);
          gr->superClass(sc);
          if (!MParser_Token::is_nil(preamble))

           {
             gr->setPreambleAction(preamble->getText());
           }
        }
                      	
       {
          switch ( LA(1))

          {
          case OPTIONS_START:
               {
                  classOptions=optionSpec(MGrammar_PreprocessorGrammar::_nil());
                  break;
               }
          case ACTION:
          case ID:
          case TOKENS_SPEC:
          case LITERAL_protected:
          case LITERAL_private:
          case LITERAL_public:
               {
                 break;
               }
          default:
               {
                 throw MParser_NoViableAltException(LT(1), getFilename());
               }
          }
       }
                       
       if (!MGrammar_PreprocessorGrammar::is_nil(gr))

        {
          gr->setOptions(classOptions);
        }
                      	
       {
          switch ( LA(1))

          {
          case TOKENS_SPEC:
               {
                 tk = LT(1);
                 match(TOKENS_SPEC);
                 gr->setTokenSection(tk->getText());
                 break;
               }
          case ACTION:
          case ID:
          case LITERAL_protected:
          case LITERAL_private:
          case LITERAL_public:
               {
                 break;
               }
          default:
               {
                 throw MParser_NoViableAltException(LT(1), getFilename());
               }
          }
       }
       {
          switch ( LA(1))

          {
          case ACTION:
               {
                 memberA = LT(1);
                 match(ACTION);
                 gr->setMemberAction(memberA->getText());
                 break;
               }
          case ID:
          case LITERAL_protected:
          case LITERAL_private:
          case LITERAL_public:
               {
                 break;
               }
          default:
               {
                 throw MParser_NoViableAltException(LT(1), getFilename());
               }
          }
       }
       {
          int _cnt15=0;
          LABEL(_loop15)
          do
           {
              if ((_tokenSet_2.member(LA(1))))

               {
                 rule(gr);
               }
              else
               {
                 if ( _cnt15>=1 )

                  { BREAK(_loop15); }
                 else {throw MParser_NoViableAltException(LT(1), getFilename());}
               }
                      
              _cnt15++;
           }
          while (true);
       }
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_3);
    }
   return gr;
}
	
MLANG_String MGrammar_Preprocessor::superClass()

{
   MLANG_String sup;
                
   sup=LT(1)->getText();
                
   try
    {      // for error handling
       match(SUBRULE_BLOCK);
    }
   catch (MParser_RecognitionException& ex)

    {
       reportError(ex);
       consume();
       consumeUntil(_tokenSet_4);
    }
   return sup;
}
	
void MGrammar_Preprocessor::rule(
   const MGrammar_PreprocessorGrammar_var& gr
)

{
                
   MParser_Token_var  r;
   MParser_Token_var  arg;
   MParser_Token_var  ret;
   MParser_Token_var  init;
   MParser_Token_var  blk;
                
   MGrammar_PreprocessorRule::OPTIONS o;
   MLANG_String vis;
   bool bang=false;
   MLANG_String eg, thr="";
                
                
   try
    {      // for error handling
      {
         switch ( LA(1))

         {
         case LITERAL_protected:
              {
                match(LITERAL_protected);
                vis="protected";
                break;
              }
         case LITERAL_private:
              {
                match(LITERAL_private);
                vis="private";
                break;
              }
         case LITERAL_public:
              {
                match(LITERAL_public);
                vis="public";
                break;
              }
         case ID:
              {
                break;
              }
         default:
              {
                throw MParser_NoViableAltException(LT(1), getFilename());
              }
         }
      }
      r = LT(1);
      match(ID);
      {
        switch ( LA(1))

        {
        case BANG:
             {
               match(BANG);
               bang=true;
               break;
             }
        case ACTION:
        case OPTIONS_START:
        case ARG_ACTION:
        case LITERAL_returns:
        case RULE_BLOCK:
        case LITERAL_throws:
             {
               break;
             }
        default:
             {
               throw MParser_NoViableAltException(LT(1), getFilename());
             }
        }
      }
      {
        switch ( LA(1))

        {
        case ARG_ACTION:
             {
               arg = LT(1);
               match(ARG_ACTION);
               break;
             }
        case ACTION:
        case OPTIONS_START:
        case LITERAL_returns:
        case RULE_BLOCK:
        case LITERAL_throws:
             {
               break;
             }
        default:
             {
               throw MParser_NoViableAltException(LT(1), getFilename());
             }
        }
      }
      {
         switch ( LA(1))

         {
         case LITERAL_returns:
              {
                match(LITERAL_returns);
                ret = LT(1);
                match(ARG_ACTION);
                break;
              }
         case ACTION:
         case OPTIONS_START:
         case RULE_BLOCK:
         case LITERAL_throws:
              {
                break;
              }
         default:
              {
                throw MParser_NoViableAltException(LT(1), getFilename());
              }
         }
       }
       {
         switch ( LA(1))

         {
         case LITERAL_throws:
              {
                thr=throwsSpec();
                break;
              }
         case ACTION:
         case OPTIONS_START:
         case RULE_BLOCK:
              {
                break;
              }
         default:
              {
                throw MParser_NoViableAltException(LT(1), getFilename());
              }
         }
       }
       {
         switch ( LA(1))

         {
         case OPTIONS_START:
              {
                o=optionSpec(MGrammar_PreprocessorGrammar::_nil());
                break;
              }
         case ACTION:
         case RULE_BLOCK:
              {
                break;
              }
         default:
              {
                throw MParser_NoViableAltException(LT(1), getFilename());
              }
         }
       }
       {
         switch ( LA(1))

         {
         case ACTION:
              {
                 init = LT(1);
                 match(ACTION);
                 break;
              }
         case RULE_BLOCK:
              {
                 break;
              }
         default:
              {
                 throw MParser_NoViableAltException(LT(1), getFilename());
              }
         }
      }
      blk = LT(1);
      match(RULE_BLOCK);
      eg=exceptionGroup();
                       
      MLANG_String rtext = blk->getText()+eg;
      MGrammar_PreprocessorRule_var ppr = 
          new MGrammar_PreprocessorRule(r->getText(),rtext,o,gr);
      ppr->setThrowsSpec(thr);
      if ( !MParser_Token::is_nil(arg))

       {
         ppr->setArgs(arg->getText());
       }
      if ( !MParser_Token::is_nil(ret))

       {
         ppr->setReturnValue(ret->getText());
       }
      if ( !MParser_Token::is_nil(init))

       {
         ppr->setInitAction(init->getText());
       }
      if ( bang )

       {
         ppr->setBang();
       }
      ppr->setVisibility(vis);
      if (!MGrammar_PreprocessorGrammar::is_nil(gr))

       {
         gr->addRule(ppr);
       }
                      	
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_5);
    }
}
	
MLANG_String MGrammar_Preprocessor::throwsSpec()

{
   MLANG_String t;
   
   MParser_Token_var  a;
   MParser_Token_var  b;
   t="throws ";
   
   try
    {      // for error handling
      match(LITERAL_throws);
      a = LT(1);
      match(ID);
      t+=a->getText();
      {
         LABEL(_loop29)
         do
          {
            if ((LA(1)==COMMA))

             {
               match(COMMA);
               b = LT(1);
               match(ID);
               t+=","+b->getText();
             }
            else
             {
               BREAK(_loop29);
             }
            
          }
         while (true);
      }
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_6);
    }
   return t;
}

MLANG_String MGrammar_Preprocessor:: exceptionGroup()

{
   MLANG_String g;
   
   MLANG_String e; g="";
   
   try 
    {  
       // for error handling
       {
          LABEL(_loop32)
          do
           {
              if ((LA(1)==LITERAL_exception))

               {
                 e=exceptionSpec();
                 g += e;
               }
              else
               {
                 BREAK(_loop32);
               }
               
           }
          while (true);
       }
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_5);
    }
   return g;
}

MLANG_String MGrammar_Preprocessor:: exceptionSpec()

{
   MLANG_String es;
   
   MParser_Token_var  aa;
   MLANG_String h;
   es = SYSTEM->getProperty("line.separator")+"exception ";
   
   
   try
    {      // for error handling
      match(LITERAL_exception);
      {
        switch ( LA(1))

        {
        case ARG_ACTION:
             {
                aa = LT(1);
                match(ARG_ACTION);
                es += aa->getText();
                break;
             }
        case EOF:
        case ACTION:
        case LITERAL_class:
        case ID:
        case LITERAL_protected:
        case LITERAL_private:
        case LITERAL_public:
        case LITERAL_exception:
        case LITERAL_catch:
             {
                break;
             }
        default:
             {
                throw MParser_NoViableAltException(LT(1), getFilename());
             }
        }
      }
      {
         LABEL(_loop36)
         do
          {
            if ((LA(1)==LITERAL_catch))

             {
               h=exceptionHandler();
               es += h;
             }
            else
             {
               BREAK(_loop36);
             }
            
          }
         while (true);
      }
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_7);
    }
   return es;
}

MLANG_String MGrammar_Preprocessor:: exceptionHandler()

{
   MLANG_String h;
   
   MParser_Token_var  a1;
   MParser_Token_var  a2;
                
   try 
    {      // for error handling
      match(LITERAL_catch);
      a1 = LT(1);
      match(ARG_ACTION);
      a2 = LT(1);
      match(ACTION);
      h = SYSTEM->getProperty("line.separator")+
                     "catch "+a1->getText()+" "+a2->getText();
    }
   catch (MParser_RecognitionException& ex)

    {
      reportError(ex);
      consume();
      consumeUntil(_tokenSet_8);
    }

   return h;
}

longArray MGrammar_Preprocessor::mk_tokenSet_0()

{
   static ACE_UINT64 data[] = { 2L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_0(MGrammar_Preprocessor::mk_tokenSet_0());
longArray MGrammar_Preprocessor::mk_tokenSet_1()

{
   static ACE_UINT64 data[] = { 4658050L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_1(MGrammar_Preprocessor::mk_tokenSet_1());
longArray MGrammar_Preprocessor::mk_tokenSet_2()

{
   static ACE_UINT64 data[] = { 459264L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_2(MGrammar_Preprocessor::mk_tokenSet_2());
longArray MGrammar_Preprocessor::mk_tokenSet_3()

{
   static ACE_UINT64 data[] = { 386L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_3(MGrammar_Preprocessor::mk_tokenSet_3());
longArray MGrammar_Preprocessor::mk_tokenSet_4()

{
   static ACE_UINT64 data[] = { 2048L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_4(MGrammar_Preprocessor::mk_tokenSet_4());
longArray MGrammar_Preprocessor::mk_tokenSet_5()

{
   static ACE_UINT64 data[] = { 459650L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_5(MGrammar_Preprocessor::mk_tokenSet_5());
longArray MGrammar_Preprocessor::mk_tokenSet_6()

{
   static ACE_UINT64 data[] = { 4202624L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_6(MGrammar_Preprocessor::mk_tokenSet_6());
longArray MGrammar_Preprocessor::mk_tokenSet_7()

{
   static ACE_UINT64 data[] = { 34014082L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_7(MGrammar_Preprocessor::mk_tokenSet_7());
longArray MGrammar_Preprocessor::mk_tokenSet_8()

{
   static ACE_UINT64 data[] = { 101122946L, 0L};
   return longArray(data,sizeof(data)/sizeof(data[0]));
}
MParser_BitSet MGrammar_Preprocessor::_tokenSet_8(MGrammar_Preprocessor::mk_tokenSet_8());

MGrammar_Preprocessor::~MGrammar_Preprocessor ()

{
}
