#ifndef __MGRAMMAR_GRAMMARFILE_H__
#define __MGRAMMAR_GRAMMARFILE_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/HashMap.h"
#include "MLANG/String.h"
#include "MGrammar/Tool.h"
#include "MGrammar/Preprocessor/Option.h"
#include "MGrammar/Preprocessor/Grammar.h"

class MGrammar_GrammarFile;
typedef class MGrammar_GrammarFile *MGrammar_GrammarFile_ptr;
typedef MLANG_SmartPtr<MGrammar_GrammarFile> MGrammar_GrammarFile_var;
class MGrammar_Option;
typedef class MGrammar_Option *MGrammar_Option_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/preprocessor/GrammarFile.java#1 $
 */


/** Stores header action, grammar preamble, file options, and
 *  list of grammars in the file
 */
class MGrammar_GrammarFile : public MLANG_Refcountable<ACE_SYNCH_MUTEX>

{
public:
   typedef MLANG_HashMap<MLANG_String, MGrammar_PreprocessorGrammar_var> GRAMMARS;
   typedef MLANG_HashMap<MLANG_String, MGrammar_PreprocessorOption_var> OPTIONS;
protected:
   MLANG_String fileName_;
   MLANG_String headerAction_;
   OPTIONS options_;
   GRAMMARS grammars_;
   bool expanded_;
   MGrammar_Tool_ptr tool_;

public:

   MGrammar_GrammarFile (
      MGrammar_Tool_ptr tool,
      const MLANG_String& f
    );

   virtual ~MGrammar_GrammarFile ();

public:
   static bool is_nil (MGrammar_GrammarFile_ptr gf)
   { return gf == 0; }

   static MGrammar_GrammarFile_ptr _nil()
   { return 0; }

   static MGrammar_GrammarFile_ptr _duplicate(MGrammar_GrammarFile_ptr gf)
   { return ::MLANG_acquire(gf); }
public:

   void addGrammar(const MGrammar_PreprocessorGrammar_var& g);

   void generateExpandedFile();

   const GRAMMARS& getGrammars() const
   { 
      return grammars_;
   }

   const MLANG_String& getName() const
   {
        return fileName_;
   }

   const MLANG_String nameForExpandedGrammarFile(const MLANG_String& f);

   void setExpanded(bool exp) 
   {
        expanded_ = exp;
   }

   void addHeaderAction(const MLANG_String& a);

   void setOptions(const OPTIONS& o)
   {
        options_ = o;
   }

   MLANG_String toString() const;
};

typedef MLANG_SmartPtr<MGrammar_GrammarFile> MGrammar_GrammarFile_var;

#endif /* __MGRAMMAR_GRAMMARFILE_H__ */
