#ifndef __MPARSER_HIERARCHY_H__
#define __MPARSER_HIERARCHY_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/HashMap.h"
#include "MLANG/String.h"
#include "MGrammar/Preprocessor/Option.h"

class MGrammar_PreprocessorGrammar;
typedef class MGrammar_PreprocessorGrammar *MGrammar_PreprocessorGrammar_ptr;
typedef MLANG_SmartPtr<MGrammar_PreprocessorGrammar> MGrammar_PreprocessorGrammar_var;
class MGrammar_GrammarFile;
typedef class MGrammar_GrammarFile *MGrammar_GrammarFile_ptr;
typedef MLANG_SmartPtr<MGrammar_GrammarFile> MGrammar_GrammarFile_var;
class MGrammar_Hierarchy;
typedef class MGrammar_Hierarchy *MGrammar_Hierarchy_ptr;
typedef MLANG_SmartPtr<MGrammar_Hierarchy> MGrammar_Hierarchy_var;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/preprocessor/Hierarchy.java#1 $
 */
typedef MLANG_HashMap<MLANG_String, MGrammar_PreprocessorOption_var> PREOPTIONS;

#include "MGrammar/Preprocessor/GrammarFile.h"
#include "MGrammar/Preprocessor/Grammar.h"

class MGrammar_Hierarchy : public MLANG_Refcountable<ACE_SYNCH_MUTEX>

{
protected:
    MGrammar_PreprocessorGrammar_var LexerRoot_;
    MGrammar_PreprocessorGrammar_var ParserRoot_;
    MGrammar_PreprocessorGrammar_var TreeParserRoot_;
    typedef MLANG_HashMap<MLANG_String, MGrammar_PreprocessorGrammar_var> SYMBOLS;
    SYMBOLS symbols_;	// table of grammars
    typedef MLANG_HashMap<MLANG_String, MGrammar_GrammarFile_var> FILES;
    FILES files_;	// table of grammar files read in
    MGrammar_Tool_ptr antlrTool_;

public:
    MGrammar_Hierarchy(MGrammar_Tool_ptr tool);
    virtual ~MGrammar_Hierarchy();

public:

   static bool is_nil (MGrammar_Hierarchy_ptr hierarchy)
   { return hierarchy == 0; }

   static MGrammar_Hierarchy_ptr _nil()
   { return 0; }

   static MGrammar_Hierarchy_ptr _duplicate(MGrammar_Hierarchy_ptr hierarchy)
   { return ::MLANG_acquire(hierarchy); }

public:

    void addGrammar(const MGrammar_PreprocessorGrammar_var& gr);

    void addGrammarFile(const MGrammar_GrammarFile_var& gf);

    void expandGrammarsInFile(const MLANG_String& fileName);

    MGrammar_PreprocessorGrammar_var findRoot(const MGrammar_PreprocessorGrammar_var& g);

    MGrammar_GrammarFile_var getFile(const MLANG_String& fileName);

    MGrammar_PreprocessorGrammar_var getGrammar(const MLANG_String& gr);

    static MLANG_String optionsToString(const PREOPTIONS& options);

    void readGrammarFile(const MLANG_String& file);

    /** Return true if hierarchy is complete, false if not */
    bool verifyThatHierarchyIsComplete();

    MGrammar_Tool_ptr getTool() const
    {
        return antlrTool_;
    }

    void setTool(MGrammar_Tool_ptr antlrTool)
    {
        antlrTool_ = antlrTool;
    }
};


#endif /* _MPARSER_HIERARCHY_H__ */
