#ifndef __MGRAMMAR_PREPROCESSORRULE_H__
#define __MGRAMMAR_PREPROCESSORRULE_H__
#include "MLANG/HashMap.h"
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MGrammar/Preprocessor/Option.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/preprocessor/Rule.java#1 $
 */
class MGrammar_PreprocessorGrammar;
typedef MGrammar_PreprocessorGrammar *MGrammar_PreprocessorGrammar_ptr;
class MGrammar_PreprocessorRule;
typedef MGrammar_PreprocessorRule *MGrammar_PreprocessorRule_ptr;

class MGrammar_PreprocessorRule : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
public:
   typedef MLANG_HashMap<MLANG_String, MGrammar_PreprocessorOption_var> OPTIONS;
protected:
   MLANG_String name_;
protected:
   MLANG_String block_;
protected:
   MLANG_String args_;
protected:
   MLANG_String returnValue_;
protected:
   MLANG_String throwsSpec_;
protected:
   MLANG_String initAction_;
protected:
   OPTIONS options_;
protected:
   MLANG_String visibility_;
protected:
   MGrammar_PreprocessorGrammar_ptr enclosingGrammar_;
protected:
   bool bang_;

public:
   MGrammar_PreprocessorRule (
      const MLANG_String& n, 
      const MLANG_String& b, 
      const OPTIONS& options = OPTIONS(), 
      MGrammar_PreprocessorGrammar_ptr gr = 0
    );

   virtual ~MGrammar_PreprocessorRule ();

public:
   static bool is_nil (MGrammar_PreprocessorRule_ptr rule)
   { return rule == 0; }

   static MGrammar_PreprocessorRule_ptr _nil()
   { return 0; }

   static MGrammar_PreprocessorRule_ptr _duplicate(MGrammar_PreprocessorRule_ptr rule)
   { return ::MLANG_acquire(rule); }
public:
   const MLANG_String& getArgs() const
   {
        return args_;
   }

public:
   bool getBang() const
   {
        return bang_;
   }

public:
   const MLANG_String& getName() const
   {
        return name_;
   }

public:
   const MLANG_String& getReturnValue() const
   {
        return returnValue_;
   }

public:
   const MLANG_String& getVisibility() const
   {
        return visibility_;
   }

    /** If 'rule' narrows the visible of 'this', return true;
     *  For example, 'this' is public:
and 'rule' is private,
     *  true is returned.  You cannot narrow the vis. of
     *  a rule.
     */
public:
   bool narrowerVisibility(MGrammar_PreprocessorRule_ptr rule);

    /** Two rules have the same signature if they have:
     *  	same name
     *		same return value
     *		same args
     *	I do a simple string compare now, but later
     *	the type could be pulled out so it is insensitive
     *	to names of args etc...
     */
public:
   bool sameSignature(MGrammar_PreprocessorRule_ptr rule);

public:
   void setArgs(const MLANG_String& a) 
   {
        args_ = a;
   }

public:
   void setBang() 
   {
        bang_ = true;
   }

public:
   MGrammar_PreprocessorGrammar_ptr enclosingGrammar() const
   { return enclosingGrammar_; }

   void setEnclosingGrammar(MGrammar_PreprocessorGrammar_ptr g)
   {
        enclosingGrammar_ = g;
   }

public:
   void setInitAction(const MLANG_String& a)
   {
        initAction_ = a;
   }

public:
   void setOptions(const OPTIONS& options)
   {
        options_ = options;
   }

public:
   void setReturnValue(const MLANG_String& ret)
   {
        returnValue_ = ret;
   }

public:
   void setThrowsSpec(const MLANG_String& t)
   {
        throwsSpec_ = t;
   }

public:
   void setVisibility(const MLANG_String& v)
   {
        visibility_ = v;
   }

public:
   MLANG_String toString() const;
};

typedef MLANG_SmartPtr<MGrammar_PreprocessorRule> MGrammar_PreprocessorRule_var;

#endif /* __MGRAMMAR_PREPROCESSORRULE_H__ */
