#include "ace/ARGV.h"
#include "MParser/System.h"
#include "MIO/File.h"
#include "MIO/IOException.h"
#include "MIO/FileNotFoundException.h"
#include "MGrammar/Preprocessor/Hierarchy.h"
#include "MGrammar/Preprocessor/Tool.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/preprocessor/Tool.java#1 $
 */

MGrammar_PreprocessorTool::MGrammar_PreprocessorTool (
   MGrammar_Tool_ptr t,
   ACE_ARGV& args
) : antlrTool_(t)

{
   args_.add (args[0]);
   processArguments(args);
}

void MGrammar_PreprocessorTool::main(ACE_ARGV& args)

{
   MGrammar_Tool_var antlrTool = new MGrammar_Tool();
   MGrammar_PreprocessorTool_var theTool = new MGrammar_PreprocessorTool(antlrTool, args);
   theTool->preprocess();
   ACE_ARGV a = theTool->preprocessedArgList();
   for (int i = 0; i < a.argc(); i++)
    {
       SYSTEM->out()->print(MLANG_String(" ") + a[i]);
    }
   SYSTEM->out()->println();
}

bool MGrammar_PreprocessorTool::preprocess()

{
   if (grammarFileName_.length()==0)
    {
      antlrTool_->toolError("no grammar file specified");
      return false;
    }
   if (grammars_.size() != 0)
    {
       theHierarchy_ = new MGrammar_Hierarchy(antlrTool_);
       for (size_t i = 0; i<grammars_.size(); i++)
        {
           MLANG_String f = grammars_[i];
           try
            {
               theHierarchy_->readGrammarFile(f);
            }
           catch (MIO_FileNotFoundException& fe)
            {
               antlrTool_->toolError("file " + f + " not found");
               return false;
            }
        }
    }

    // do the actual inheritance stuff
    bool complete = theHierarchy_->verifyThatHierarchyIsComplete();
    if (!complete)
       return false;
    theHierarchy_->expandGrammarsInFile(grammarFileName_);
    MGrammar_GrammarFile_var gf = theHierarchy_->getFile(grammarFileName_);
    MLANG_String expandedFileName = gf->nameForExpandedGrammarFile(grammarFileName_);
    fprintf(stderr,"grammarFileName=%s\n", grammarFileName_.c_str());

    // generate the output file if necessary
    if (expandedFileName.equals(grammarFileName_))
     {
       args_.add (grammarFileName_.c_str()); // add to argument list
     }
    else
     {
       try
        {
          gf->generateExpandedFile(); 				// generate file to feed ANTLR
          args_.add (MLANG_String(antlrTool_->getOutputDirectory() +
                          SYSTEM->getProperty("file.separator") +
                          expandedFileName).c_str());		// add to argument list
        }
       catch (MIO_IOException& io)
        {
          antlrTool_->toolError("cannot write expanded grammar file " + expandedFileName);
          return false;
        }
     }
    return true;
}

/** create new arg list with correct length to pass to ANTLR */
ACE_ARGV MGrammar_PreprocessorTool::preprocessedArgList()

{
   return args_;
}

/** Process -glib options and grammar file.  Create a new args list
 *  that does not contain the -glib option.  The grammar file name
 *  might be modified and, hence, is not added yet to args list.
 */
void MGrammar_PreprocessorTool::processArguments (ACE_ARGV& incomingArgs)

{
   for (int i = 1; i < incomingArgs.argc(); i++)
    {
       if ( MLANG_String(incomingArgs[i]).length() == 0 )
        {
          antlrTool_->warning("Zero length argument ignoring...");
          continue;
        }
       if (MLANG_String(incomingArgs[i]).equals("-glib"))
        {
          // if on a pc and they use a '/', warn them
          if (MIO_File::separator().equals("\\")
          &&  MLANG_String(incomingArgs[i]).indexOf('/') != -1)
           {
              antlrTool_->warning("-glib cannot deal with '/' on a PC: use '\\'; ignoring...");
           }
          else
           {
              grammars_ = antlrTool_->parseSeparatedList(incomingArgs[i + 1], ';');
              i++;
           }
        }
       else if (MLANG_String(incomingArgs[i]).equals("-o"))
        {
          args_.add(incomingArgs[i]);
          if (i + 1 >= incomingArgs.argc())
           {
             antlrTool_->error("missing output directory with -o option; ignoring");
           }
          else
           {
             i++;
             args_.add (incomingArgs[i]);
             antlrTool_->setOutputDirectory(incomingArgs[i]);
           }
        }
       else if (incomingArgs[i][0] == '-')
        {
          args_.add (incomingArgs[i]);
        }
       else
        {
          // Must be the grammar file
          grammarFileName_ = incomingArgs[i];
          grammars_.push_back(grammarFileName_); // process it too
          if ((i + 1) < incomingArgs.argc())
           {
             antlrTool_->warning("grammar file must be last; ignoring other arguments...");
             break;
           }
        }
    }
}

MGrammar_PreprocessorTool::~MGrammar_PreprocessorTool ()

{
}
