#ifndef __MGRAMMAR_PREPROCESSORGRAMMAR_H__
#define __MGRAMMAR_PREPROCESSORGRAMMAR_H__
#include "ace/Vector_T.h"
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MLANG/HashMap.h"
#include "MGrammar/Preprocessor/Rule.h"
#include "MGrammar/Preprocessor/Option.h"
#include "MGrammar/Preprocessor/Hierarchy.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/preprocessor/Grammar.java#1 $
 */

class MGrammar_PreprocessorGrammar;
typedef MGrammar_PreprocessorGrammar *MGrammar_PreprocessorGrammar_ptr;

class MGrammar_PreprocessorGrammar : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
public:
   typedef MLANG_HashMap<MLANG_String, MGrammar_PreprocessorOption_var> OPTIONS;
   typedef MLANG_HashMap<MLANG_String, MGrammar_PreprocessorRule_var> RULES;
protected:
   MLANG_String name_;
protected:
   MLANG_String fileName_; // where does it come from?
protected:
   MLANG_String superGrammar_;	// null if no super class
protected:
   MLANG_String type_; // lexer? parser? tree parser?
protected:
   RULES rules_; // text of rules as they were read in
protected:
   OPTIONS options_;// rule options
protected:
   MLANG_String tokenSection_; // the tokens{} stuff
protected:
   MLANG_String preambleAction_;// action right before grammar
protected:
   MLANG_String memberAction_; // action inside grammar
protected:
   MGrammar_Hierarchy_var hier_; // hierarchy of grammars
protected:
   bool predefined_;	// one of the predefined grammars?
protected:
   bool alreadyExpanded_;
protected:
   bool specifiedVocabulary_;	// found importVocab option?

	/** if not derived from another grammar, might still specify a non-ANTLR
	 *  class to derive from like this "class T extends Parser(MyParserClass);"
	 */
protected:
   MLANG_String superClass_;

protected:
   MLANG_String importVocab_;
protected:
   MLANG_String exportVocab_;
protected:
   MGrammar_Tool_ptr antlrTool_;

public:
   MGrammar_PreprocessorGrammar (
      MGrammar_Tool_ptr tool, 
      const MLANG_String& name, 
      const MLANG_String& superGrammar = "", 
      const RULES& rules = RULES()
    );

   virtual ~MGrammar_PreprocessorGrammar ();

public:
   static bool is_nil (MGrammar_PreprocessorGrammar_ptr grammar)
   { return grammar == 0; }

   static MGrammar_PreprocessorGrammar_ptr _nil()
   { return 0; }

   static MGrammar_PreprocessorGrammar_ptr _duplicate(MGrammar_PreprocessorGrammar_ptr grammar)
   { return ::MLANG_acquire(grammar); }
public:
   void addOption(const MGrammar_PreprocessorOption_var& o);

public:
   void addRule(const MGrammar_PreprocessorRule_var& r);

public:
   void expandInPlace();

public:
   bool specifiedVocabulary() const
   {
      return specifiedVocabulary_;	// found importVocab option?
   }

   void specifiedVocabulary(bool value)
   {
      specifiedVocabulary_ = value;
   }
public:
   const MLANG_String& getFileName() const
   { return fileName_; }

public:
   const MLANG_String& getName() const
   { return name_; }

public:
   const OPTIONS& getOptions() const
   { return options_; }

public:
   const RULES& getRules() const
   { return rules_; }

public:
   MGrammar_PreprocessorGrammar_var getSuperGrammar();

public:
   const MLANG_String& superClass() const
   { return superClass_; }

   void superClass(const MLANG_String& value)
   { superClass_ = value; }

public:
   const MLANG_String& getSuperGrammarName() const
   { return superClass_; }

   void setSuperGrammarName(const MLANG_String& value)
   { superClass_ = value; }

public:
   const MLANG_String& getType() const
   { return type_; }

   const MLANG_String& importVocab() const
   { return importVocab_; }

   void importVocab(const MLANG_String& value) 
   { importVocab_ = value; }

   const MLANG_String& exportVocab() const
   { return exportVocab_; }

   void exportVocab(const MLANG_String& value)
   { exportVocab_ = value; }

public:
   void inherit (
      const MGrammar_PreprocessorOption_var& o, 
      const MGrammar_PreprocessorGrammar_var& superG
    );

public:
   void inherit (
      const MGrammar_PreprocessorRule_var& r,
      const MGrammar_PreprocessorGrammar_var& superG
    );

public:
   void inherit (
      const MLANG_String& memberAction,
      const MGrammar_PreprocessorGrammar_var& superG
    );

public:
   bool isPredefined() const
   { return predefined_; }

public:
   void setFileName(const MLANG_String& f)
   { fileName_ = f; }

public:
   void setHierarchy(const MGrammar_Hierarchy_var& hier);

public:
   void setMemberAction(const MLANG_String& a)
   { memberAction_ = a; }

public:
   void setOptions(const OPTIONS options)
   { options_ = options; }

public:
   void setPreambleAction(const MLANG_String& a)
   { preambleAction_ = a; }

public:
   void setPredefined(bool b)
   { predefined_ = b; }

public:
   void setTokenSection(const MLANG_String& tk)
   { tokenSection_ = tk; }

public:
   void setType(const MLANG_String& t)
   { type_ = t; }

public:
   MLANG_String toString() const;
};

typedef MLANG_SmartPtr<MGrammar_PreprocessorGrammar> MGrammar_PreprocessorGrammar_var;

#endif /* __MGRAMMAR_PREPROCESSORGRAMMAR_H__ */
