#include "MLANG/StringBuffer.h"
#include "MParser/System.h"
#include "MGrammar/Preprocessor/GrammarFile.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/preprocessor/GrammarFile.java#1 $
 */

/** Stores header action, grammar preamble, file options, and
 *  list of grammars in the file
 */

MGrammar_GrammarFile::MGrammar_GrammarFile (
   MGrammar_Tool_ptr tool,
   const MLANG_String& f)
  : fileName_(f),
    expanded_(false),
    tool_ (tool)

{
}

void MGrammar_GrammarFile::addGrammar(const MGrammar_PreprocessorGrammar_var& g)

{
   grammars_[g->getName()] = g;
}

void MGrammar_GrammarFile::generateExpandedFile()

{
   fprintf(stderr,"MGrammar_GrammarFile::generateExpandedFile: enter\n");
   if (!expanded_)
    {
      fprintf(stderr,"MGrammar_GrammarFile::generateExpandedFile: leave already done\n");
      return;	// don't generate if nothing got expanded
    }
   MLANG_String expandedFileName = nameForExpandedGrammarFile(this->getName());

   // create the new grammar file with expanded grammars
   MIO_PrintWriter_var expF = tool_->openOutputFile(expandedFileName);
   expF->println(toString());
   expF->close();
   fprintf(stderr,"MGrammar_GrammarFile::generateExpandedFile: leave OK\n");
}

const MLANG_String MGrammar_GrammarFile::nameForExpandedGrammarFile (
   const MLANG_String& f
)

{
   if (expanded_)
    {
      // strip path to original input, make expanded file in current dir
      return "expanded" + tool_->fileMinusPath(f);
    }
   else
    {
      return f;
    }
}

void MGrammar_GrammarFile::addHeaderAction (
   const MLANG_String& a)

{
  headerAction_ += a + SYSTEM->getProperty("line.separator");
}

MLANG_String MGrammar_GrammarFile::toString() const

{
   MLANG_String h = headerAction_.length() == 0 ? "" : headerAction_;
   MLANG_String o = options_.size() == 0 ? "" : MGrammar_Hierarchy::optionsToString(options_);

   MLANG_StringBuffer s; s.append(h); s.append(o);
   for (GRAMMARS::const_iterator iter = grammars_.begin(); iter!=grammars_.end(); iter++)
    {
       s.append(iter->second->toString());
    }
   return s.toString();
}

MGrammar_GrammarFile::~MGrammar_GrammarFile ()

{
}
