#include "MParser/System.h"
#include "MGrammar/PreprocessorRule.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/preprocessor/Rule.java#1 $
 */


MGrammar_PreprocessorRule::MGrammar_PreprocessorRule (
   const MLANG_String& n, 
   const MLANG_String& b, 
   const OPTIONS& options, 
   MGrammar_PreprocessorGrammar_ptr gr
) : name_(n),
    block_(b),
    options_(options),
    bang_(false)

{
   setEnclosingGrammar(gr);
}

/** If 'rule' narrows the visible of 'this', return true;
 *  For example, 'this' is  and 'rule' is private,
 *  true is returned.  You cannot narrow the vis. of
 *  a rule.
 */
bool MGrammar_PreprocessorRule::narrowerVisibility (
   MGrammar_PreprocessorRule_ptr rule
)

{
   if (visibility_.equals(""))
    {
      if (!rule->toString().equals(""))
       {
         return true;	// everything narrower than 
       }
      return false;
    }
   else if (visibility_.equals("protected"))
    {
      if (rule->toString().equals("private")) 
       {
         return true;	// private narrower than protected
       }
      return false;
    }
   else if (visibility_.equals("private"))
    {
      return false;	// nothing is narrower than private
    }
   return false;
}

/** Two rules have the same signature if they have:
 *  	same name
 *		same return value
 *		same args
 *	I do a simple string compare now, but later
 *	the type could be pulled out so it is insensitive
 *	to names of args etc...
 */
bool MGrammar_PreprocessorRule::sameSignature (
   MGrammar_PreprocessorRule_ptr rule
)

{
   bool nSame = true;
   bool aSame = true;
   bool rSame = true;

   nSame = name_.equals(rule->getName());
   if (args_.length() != 0)
    {
      aSame = args_.equals(rule->getArgs());
    }
   if (returnValue_.length() != 0)
    {
      rSame = returnValue_.equals(rule->getReturnValue());
    }
   return nSame && aSame && rSame;
}

MLANG_String MGrammar_PreprocessorRule::toString() const

{
   MLANG_String s = "";
   MLANG_String retString = returnValue_.length() == 0 ? "" : "returns " + returnValue_;
   MLANG_String argString = args_.length() == 0 ? "" : args_;
   MLANG_String bang = getBang() ? "!" : "";

   s += visibility_.length() == 0 ? "" : visibility_ + " ";
   s += name_ + MLANG_String::valueOf(bang_) + argString + " " + retString + throwsSpec_;
   if (options_.size() != 0)
    {
      s += SYSTEM->getProperty("line.separator") +
            "options {" +
            SYSTEM->getProperty("line.separator");
#if 0
      for (size_t i=0; i<options_.size(); i++)
       {
          s += options_[i]->toString() + SYSTEM->getProperty("line.separator");
       }
      s += "}" + SYSTEM->getProperty("line.separator");
#else
      assert(0);
#endif
    }
   if (initAction_.length() != 0)
    {
      s += initAction_ + SYSTEM->getProperty("line.separator");
    }
   s += block_;
   return s;
}

MGrammar_PreprocessorRule::~MGrammar_PreprocessorRule ()

{
}
