#include "ace/Vector_T.h"
#include "MIO/IOException.h"
#include "MIO/Reader.h"
#include "MIO/FileReader.h"
#include "MIO/BufferedReader.h"
#include "MIO/BufferedWriter.h"
#include "MGrammar/PreservingFileWriter.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/dev/klaren.dev/antlr/PreservingFileWriter.java $
 * @author Ric Klaren <klaren@cs.utwente.nl>
 */

MGrammar_PreservingFileWriter::MGrammar_PreservingFileWriter (
   const MLANG_String& file
) : super(file+".antlr.tmp")

{
   // set up File thingy for target..
   target_file_ = new MIO_File(file);

   MLANG_String parentdirname = target_file_->getParent();
   if (parentdirname.length() != 0)
    {
      MIO_File_var parentdir = new MIO_File(parentdirname);

      if (!parentdir->exists())
         throw MIO_IOException("destination directory of '"+file+"' doesn't exist");
      if (!parentdir->canWrite())
         throw MIO_IOException("destination directory of '"+file+"' isn't writeable");
    }
   if (target_file_->exists() && ! target_file_->canWrite())
      throw MIO_IOException("cannot write to '"+file+"'");

   // and for the temp file
   tmp_file_ = new MIO_File(file+".antlr.tmp");
   // have it nuked at exit
   // RK: this is broken on java 1.4 and
   // is not compatible with java 1.1 (which is a big problem I'm told :) )
   // sigh. Any real language would do this in a destructor ;) ;)
   // tmp_file.deleteOnExit();
}

/** Close the file and see if the actual target is different
 * if so the target file is overwritten by the copy. If not we do nothing
 */
void MGrammar_PreservingFileWriter::close()

{
   MIO_Reader_var source;
   MIO_Writer_var target;

   try
    {
        // close the tmp file so we can access it safely...
        super::close();

        char buffer[1024];
        int cnt;

        // target_file != tmp_file so we have to compare and move it..
        if (target_file_->length() == tmp_file_->length())
         {
           // Do expensive read'n'compare
           MIO_Reader_var tmp;
           char buf2[1024];

           source = new MIO_BufferedReader(new MIO_FileReader(tmp_file_));
           tmp = new MIO_BufferedReader(new MIO_FileReader(target_file_));
           int cnt1, cnt2;
           bool equal = true;

           while (equal)
            {
               cnt1 = source->read(&buffer[0],sizeof(buffer));
               cnt2 = tmp->read(&buf2[0],sizeof(buf2));
               if (cnt1 != cnt2)
                {
                  equal = false;
                  break;
                }
               if (cnt1 == -1) // EOF
                  break;
               for (int i = 0; i < cnt1; i++ )
                {
                   if (buffer[i] != buf2[i])
                    {
                      equal = false;
                      break;
                    }
                }
            }
           // clean up...
           source->close();
           tmp->close();

           source = tmp = MIO_Reader::_nil();

           if (equal)
              return;
         }

        source = new MIO_BufferedReader(new MIO_FileReader(tmp_file_));
        target = new MIO_BufferedWriter(new MIO_FileWriter(target_file_));

        while (true)
         {
            cnt = source->read(&buffer[0],sizeof(buffer));
            if (cnt == -1)
               break;
            target->write(&buffer[0], cnt );
         }
    }
   catch (...)
    {
      if (!MIO_Reader::is_nil(source))
       {
         try { source->close(); }
         catch (MIO_IOException e ) { ; }
       }
      if (!MIO_Writer::is_nil(target))
       {
         try { target->close(); }
         catch (MIO_IOException e ) {;}
       }
      // RK: Now if I'm correct this should be called anytime.
      if (!MIO_File::is_nil(tmp_file_) && tmp_file_->exists() )
       {
          tmp_file_->remove();
//          tmp_file_ = MIO_File::_nil();
       }
    }
}

MGrammar_PreservingFileWriter::~MGrammar_PreservingFileWriter ()

{
   this->close();
}
