#ifndef __MGRAMMAR_PRESERVINGFILEWRITER_H__
#define __MGRAMMAR_PRESERVINGFILEWRITER_H__
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MIO/File.h"
#include "MIO/FileWriter.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/dev/klaren.dev/antlr/PreservingFileWriter.java $
 * @author Ric Klaren <klaren@cs.utwente.nl>
 */

class MGrammar_PreservingFileWriter;
typedef MGrammar_PreservingFileWriter *MGrammar_PreservingFileWriter_ptr;

/** PreservingFileWriter only overwrites target if the new file is different.
 Mainly added in order to prevent big and unnecessary recompiles in C++
 projects.
 I/O is buffered.
*/
class MGrammar_PreservingFileWriter : public MIO_FileWriter
{
public:
   typedef MIO_FileWriter super;
protected:
   MIO_File_var target_file_; /// the file we intend to write to
protected:
   MIO_File_var tmp_file_;  /// the tmp file we create at first

public:
   MGrammar_PreservingFileWriter(const MLANG_String& file);

   virtual ~MGrammar_PreservingFileWriter();

public:
   static bool is_nil (MGrammar_PreservingFileWriter_ptr writer)
   { return writer == 0; }

   static MGrammar_PreservingFileWriter_ptr _nil()
   { return 0; }

   static MGrammar_PreservingFileWriter_ptr _duplicate(MGrammar_PreservingFileWriter_ptr writer)
   { return ::MLANG_acquire(writer); }
   /** Close the file and see if the actual target is different
    * if so the target file is overwritten by the copy. If not we do nothing
    */
public:
   void close();
};

typedef MLANG_SmartPtr<MGrammar_PreservingFileWriter> MGrammar_PreservingFileWriter_var;

#endif /* __MGRAMMAR_PRESERVINGFILEWRITER_H__ */
