#ifndef __MGRAMMAR_RULEBLOCK_H__
#define __MGRAMMAR_RULEBLOCK_H__
#include <map>
#include "ace/SString.h"
#include "ace/Vector_T.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MParser/Token.h"
#include "MGrammar/Lookahead.h"
#include "MGrammar/ExceptionSpec.h"
#include "MGrammar/AlternativeBlock.h"
#include "MGrammar/RuleEndElement.h"

class MGrammar_RuleBlock;
typedef class MGrammar_RuleBlock *MGrammar_RuleBlock_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/RuleBlock.java#1 $
 */

/**A list of alternatives and info contained in
 * the rule definition.
 */
class MGrammar_RuleBlock : public MGrammar_AlternativeBlock

{
public:
    MLANG_String ruleName_;
    MLANG_String argAction_;	// string for rule arguments [...]
    MLANG_String throwsSpec_;
    MLANG_String returnAction_;// string for rule return type(s) <...>
    MGrammar_RuleEndElement_var endNode_;	// which node ends this rule?

    // Generate literal-testing code for lexer rule?
    bool testLiterals_;

    MLANG_Vector<MGrammar_AlternativeElement_var> labeledElements_;	// List of labeled elements found in this rule
    // This is a list of AlternativeElement (or subclass)

    MLANG_Vector<bool> lock_;	// for analysis; used to avoid infinite loops
    // 1..k
    MLANG_Vector<MGrammar_Lookahead> cache_;// Each rule can cache it's lookahead computation.

    // This cache contains an epsilon
    // imaginary token if the FOLLOW is required.  No
    // FOLLOW information is cached here.
    // The FIRST(rule) is stored in this cache; 1..k
    // This set includes FIRST of all alts.

    typedef std::map<MLANG_String, MGrammar_ExceptionSpec_var> EXCEPTIONSPECS;
    EXCEPTIONSPECS exceptionSpecs_;
    // table of String-to-ExceptionSpec.

    // grammar-settable options
    bool defaultErrorHandler_;
    MLANG_String ignoreRule_;

public:

    /** Construct a named rule. */
    MGrammar_RuleBlock (
        MGrammar_Grammar_ptr g,
        const MLANG_String& r
     );

    /** Construct a named rule with line number information */
    MGrammar_RuleBlock (
        MGrammar_Grammar_ptr g,
        const MLANG_String& r,
        int line, 
        bool doAutoGen
      );

    virtual ~MGrammar_RuleBlock ();

public:

   static bool is_nil (MGrammar_RuleBlock_ptr a)
   { return a == 0; }

   static MGrammar_RuleBlock_ptr _nil()
   { return 0; }

   static MGrammar_RuleBlock_ptr _duplicate(MGrammar_RuleBlock_ptr rb)
   { return MLANG_acquire(rb); }

   static MGrammar_RuleBlock_ptr _narrow(MGrammar_AlternativeBlock_ptr rb)
   { return MLANG_acquire(ACE_dynamic_cast(MGrammar_RuleBlock_ptr, rb)); }

public:

    const MLANG_String& argAction() const
    { return argAction_; }

    const MLANG_String& throwsSpec() const
    { return throwsSpec_; }

    const MLANG_String& returnAction() const
    { return returnAction_; }

    const MGrammar_RuleEndElement_var& endNode() const
    { return endNode_; }

    const MLANG_Vector<MGrammar_AlternativeElement_var>& labeledElements() const
    { return labeledElements_; }

    void addExceptionSpec(const MGrammar_ExceptionSpec_var& ex);

    MGrammar_ExceptionSpec_var findExceptionSpec(const MParser_Token_var& label) const;

    MGrammar_ExceptionSpec_var findExceptionSpec(const MLANG_String& label) const;

    void generate();

    bool getDefaultErrorHandler() const
    {
        return defaultErrorHandler_;
    }

    const MGrammar_RuleEndElement_var& getEndElement() const
    {
        return endNode_;
    }

    const MLANG_String& getIgnoreRule() const
    {
        return ignoreRule_;
    }

    const MLANG_String& getRuleName() const
    {
        return ruleName_;
    }

    bool getTestLiterals() const
    {
        return testLiterals_;
    }

    bool isLexerAutoGenRule()  const;

    MGrammar_Lookahead look(int k);

    void prepareForAnalysis();

    // rule option values
    void setDefaultErrorHandler(bool value)
    {
        defaultErrorHandler_ = value;
    }

    void setEndElement(const MGrammar_RuleEndElement_var& re);

    void setOption (
       const MParser_Token_var& key,
       const MParser_Token_var& value
     );

    MLANG_String toString() const;
};

typedef MLANG_SmartPtr<MGrammar_RuleBlock> MGrammar_RuleBlock_var;

#endif /* __MGRAMMAR_RULEBLOCK_H__ */
