#ifndef __MGRAMMAR_RULESYMBOL_H__
#define __MGRAMMAR_RULESYMBOL_H__
#include "ace/Vector_T.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MGrammar/GrammarSymbol.h"
#include "MGrammar/RuleBlock.h"
#include "MGrammar/RuleRefElement.h"

class MGrammar_RuleSymbol;
typedef class MGrammar_RuleSymbol *MGrammar_RuleSymbol_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/RuleSymbol.java#1 $
 */

class MGrammar_RuleSymbol : public MGrammar_GrammarSymbol

{
protected:
    MGrammar_RuleBlock_var block_;	// list of alternatives
    bool defined_;	// has the rule been defined yet?
    MLANG_Vector<MGrammar_RuleRefElement_var> references_;	// list of all nodes referencing this rule
    // not strictly needed by generic symbol table
    // but we will almost always analyze/gen code
    MLANG_String access_;	// access specifier for this rule
    MLANG_String comment_;	// A javadoc comment if any.
public:

    MGrammar_RuleSymbol(const MLANG_String& r);

    virtual ~MGrammar_RuleSymbol();

public:

   static bool is_nil (MGrammar_RuleSymbol_ptr a)
   { return a == 0; }

   static MGrammar_RuleSymbol_ptr _nil()
   { return 0; }

   static MGrammar_RuleSymbol_ptr _duplicate(MGrammar_RuleSymbol_ptr rs)
   { return MLANG_acquire(rs); }

   static MGrammar_RuleSymbol_ptr _narrow(MGrammar_GrammarSymbol_ptr rs)
   { return MLANG_acquire(ACE_dynamic_cast(MGrammar_RuleSymbol_ptr, rs)); }

public:

    const MLANG_Vector<MGrammar_RuleRefElement_var>& references() const
    { return references_; }

    const MLANG_String& access() const
    { return access_; }

    MGrammar_RuleSymbol& access(const MLANG_String& value)
    { access_ = value; return *this; }

    const MLANG_String& comment() const
    { return comment_; }

    MGrammar_RuleSymbol& comment(const MLANG_String& value)
    { comment_ = value; return *this; }

    void addReference(MGrammar_RuleRefElement_ptr e);

    void removeReference(MGrammar_RuleRefElement_ptr e);

    const MGrammar_RuleBlock_var& getBlock() const
    {
        return block_;
    }

    const MGrammar_RuleRefElement_var& getReference(size_t i) const
    {
        return references_[i];
    }

    bool isDefined() const
    {
        return defined_;
    }

    size_t numReferences() const
    {
        return references_.size();
    }

    void setBlock(MGrammar_RuleBlock_ptr rb);

    void setDefined()
    {
        defined_ = true;
    }
};

typedef MLANG_SmartPtr<MGrammar_RuleSymbol> MGrammar_RuleSymbol_var;

#endif /* __MGRAMMAR_RULESYMBOL_H__ */
