#include "MParser/Token.h"
#include "MGrammar/SimpleTokenManager.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/SimpleTokenManager.java#1 $
 */

MGrammar_SimpleTokenManager::MGrammar_SimpleTokenManager (
   const MLANG_String& name,
   MGrammar_Tool_ptr tool
) :
    maxToken_(MParser_Token::MIN_USER_TYPE),
    antlrTool_(tool),
    name_(name),
    readOnly_(false)

{
   // Don't make a bigger vector than we need, because it will show up in output sets.

   // define EOF symbol
   MGrammar_TokenSymbol_var ts = new MGrammar_TokenSymbol("EOF");
   ts->setTokenType(MParser_Token::EOF_TYPE);
   define(ts);

   // define <null-tree-lookahead> but only in the vocabulary vector
   vocabulary_.ensureCapacity(MParser_Token::NULL_TREE_LOOKAHEAD);
   vocabulary_.setElementAt("NULL_TREE_LOOKAHEAD", MParser_Token::NULL_TREE_LOOKAHEAD);
}

MGrammar_SimpleTokenManager::MGrammar_SimpleTokenManager (
   const MGrammar_SimpleTokenManager& tm
) : MGrammar_TokenManager (tm),
    maxToken_(tm.maxToken_),
    vocabulary_(tm.vocabulary_),
    table_(tm.table_),
    antlrTool_(tm.antlrTool_),
    name_(tm.name_)

{
}

MGrammar_TokenManager_var MGrammar_SimpleTokenManager::clone()

{
   return new MGrammar_SimpleTokenManager(*this);
}

/** define a token */
void MGrammar_SimpleTokenManager::define(const MGrammar_TokenSymbol_var& ts)

{
   // Add the symbol to the vocabulary vector
   vocabulary_.ensureCapacity(ts->getTokenType());
   vocabulary_.setElementAt( ts->getId(), ts->getTokenType());
   // add the symbol to the hash table
   mapToTokenSymbol(ts->getId(), ts);
}

/** Get a token symbol by index */
MLANG_String MGrammar_SimpleTokenManager::getTokenStringAt(int idx) const
{
  return vocabulary_[(size_t) idx];
}

/** Get the TokenSymbol for a string */
MGrammar_TokenSymbol_var MGrammar_SimpleTokenManager::getTokenSymbol (
   const MLANG_String& sym
)
   
{
   TABLE::const_iterator iter = table_.find (sym);
   if (iter!=table_.end())
      return iter->second;
   return MGrammar_TokenSymbol_var();
}

/** Get an enumerator over the symbol table */
MGrammar_TokenManager::ELEMENTS
MGrammar_SimpleTokenManager::getTokenSymbolElements() const

{
   ELEMENTS elements;
   for (TABLE::const_iterator iter = table_.begin(); iter!=table_.end(); iter++)
     elements.push_back(iter->second); 
   return elements;
}

MGrammar_TokenManager::KEYS
MGrammar_SimpleTokenManager::getTokenSymbolKeys() const

{
   KEYS keys;
   for (TABLE::const_iterator iter = table_.begin(); iter!=table_.end(); iter++)
     keys.push_back(iter->first); 
   return keys;
}

/** Map a label or string to an existing token symbol */
void MGrammar_SimpleTokenManager::mapToTokenSymbol (
   const MLANG_String& name, 
   const MGrammar_TokenSymbol_var& sym
)

{
   // System.out.println("mapToTokenSymbol("+name+","+sym+")");
   table_[name] = sym;
}

/** Get the next unused token type */
int MGrammar_SimpleTokenManager::nextTokenType()

{
   return maxToken_++;
}

/** Is a token symbol defined? */
bool MGrammar_SimpleTokenManager::tokenDefined (
   const MLANG_String& symbol
)

{
   return table_.count(symbol)!=0;
}

MGrammar_SimpleTokenManager::~MGrammar_SimpleTokenManager ()

{
}
