#ifndef __MGRAMMAR_SIMPLETOKENMANAGER_H__
#define __MGRAMMAR_SIMPLETOKENMANAGER_H__
#include <map>
#include "ace/Vector_T.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MGrammar/TokenManager.h"
#include "MGrammar/Tool.h"

class MGrammar_SimpleTokenManager;
typedef class MGrammar_SimpleTokenManager *MGrammar_SimpleTokenManager_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/SimpleTokenManager.java#1 $
 */

class MGrammar_SimpleTokenManager : public MGrammar_TokenManager

{
protected:
    int maxToken_;
private:
    VOCABULARY vocabulary_;
    // Hash table is a mapping from Strings to TokenSymbol
    TABLE table_;
protected:
    // the ANTLR tool
    MGrammar_Tool_ptr antlrTool_;
    // Name of the token manager
    MLANG_String name_;
protected:
    bool readOnly_;

public:

    MGrammar_SimpleTokenManager (
       const MLANG_String& name,
       MGrammar_Tool_ptr tool
     );

    MGrammar_SimpleTokenManager (
       const MGrammar_SimpleTokenManager& tokenMngr
     );

    virtual ~MGrammar_SimpleTokenManager ();

public:

    MGrammar_TokenManager_var clone();

    /** define a token */
    void define(const MGrammar_TokenSymbol_var& ts);

    /** Simple token manager doesn't have a name -- must be set externally */
    const MLANG_String& getName() const
    {
        return name_;
    }

    /** Get a token symbol by index */
    MLANG_String getTokenStringAt(int idx) const;

    /** Get the TokenSymbol for a string */
    MGrammar_TokenSymbol_var getTokenSymbol(const MLANG_String& sym);

    /** Get a token symbol by index */
    MGrammar_TokenSymbol_var getTokenSymbolAt(int idx) 
    {
        return getTokenSymbol(getTokenStringAt(idx));
    }

    /** Get an enumerator over the symbol table */
    ELEMENTS getTokenSymbolElements() const;

    KEYS getTokenSymbolKeys() const;

    /** Get the token vocabulary (read-only).
     * @return A Vector of TokenSymbol
     */
    const VOCABULARY& getVocabulary() const
    {
        return vocabulary_;
    }

    /** Simple token manager is not read-only */
    bool isReadOnly() const
    {
        return false;
    }

    /** Map a label or string to an existing token symbol */
    void mapToTokenSymbol (
       const MLANG_String& name,
       const MGrammar_TokenSymbol_var& sym
     );

    /** Get the highest token type in use */
    int maxTokenType() const
    {
        return maxToken_ - 1;
    }

    /** Get the next unused token type */
    int nextTokenType();

    /** Set the name of the token manager */
    void setName(const MLANG_String& name)
    {
        name_ = name;
    }

    void setReadOnly(bool ro)
    {
        readOnly_ = ro;
    }

    /** Is a token symbol defined? */
    bool tokenDefined(const MLANG_String& symbol);
};

typedef MLANG_SmartPtr<MGrammar_SimpleTokenManager> MGrammar_SimpleTokenManager_var;

#endif /* __MGRAMMAR_SIMPLETOKENMANAGER_H__ */
