#include "MGrammar/Tool.h"
#include "MGrammar/TokenSymbol.h"
#include "MGrammar/CodeGenerator.h"
#include "MGrammar/LexerGrammar.h"
#include "MGrammar/StringLiteralElement.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/StringLiteralElement.java#1 $
 */

MGrammar_StringLiteralElement::MGrammar_StringLiteralElement (
   MGrammar_Grammar_ptr g,
   const MParser_Token_var& t,
   int autoGenType
) : MGrammar_GrammarAtom(g, t, autoGenType)

{
   if (!g->isLexer())
    {
      // lexer does not have token types for string literals
      MGrammar_TokenSymbol_var ts = grammar_->tokenManager_->getTokenSymbol(atomText_);
      if (MGrammar_TokenSymbol::is_nil(ts)) 
       {
         g->antlrTool_->error("Undefined literal: " + atomText_, grammar_->getFilename(), t->getLine(), t->getColumn());
       }
      else
       {
         tokenType_ = ts->getTokenType();
       }
    }
   line_ = t->getLine();

   // process the string literal text by removing quotes and escaping chars
   // If a lexical grammar, add the characters to the char vocabulary
   processedAtomText_.clear();
   for (int i = 1; i < (int)atomText_.length() - 1; i++)
    {
       char c = atomText_.charAt(i);
       if (c == '\\')
        {
          if (i + 1 < (int) atomText_.length() - 1)
           {
             i++;
             c = atomText_.charAt(i);
             switch (c)
              {
              case 'n':
                   c = '\n';
                   break;
              case 'r':
                   c = '\r';
                   break;
              case 't':
                   c = '\t';
                   break;
              }
           }
        }
       if (g->isLexer())
        {
          ACE_dynamic_cast(MGrammar_LexerGrammar_ptr, g)->charVocabulary_.add(c);
        }
       processedAtomText_ += c;
    }
}

void MGrammar_StringLiteralElement::generate()

{
   grammar_->generator_->gen(this);
}

MGrammar_Lookahead MGrammar_StringLiteralElement::look(int k)

{
   return grammar_->theLLkAnalyzer_->look(k, this);
}

MGrammar_StringLiteralElement::~MGrammar_StringLiteralElement ()

{
}
