#ifndef __MGRAMMAR_TOKENMANAGER_H__
#define __MGRAMMAR_TOKENMANAGER_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/Vector_T.h"
#include "MLANG/HashMap.h"
#include "MLANG/String.h"
#include "MGrammar/TokenSymbol.h"

class MGrammar_TokenManager;
typedef class MGrammar_TokenManager *MGrammar_TokenManager_ptr;
typedef MLANG_SmartPtr<MGrammar_TokenManager> MGrammar_TokenManager_var;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/TokenManager.java#1 $
 */

/** Interface that describes the set of defined tokens */
class MGrammar_TokenManager : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
public:
    typedef MLANG_String KEY;
    typedef MGrammar_TokenSymbol_var ELEMENT;
    typedef MLANG_Vector<KEY> VOCABULARY;
    typedef MLANG_HashMap<KEY, MGrammar_TokenSymbol_var> TABLE;
    typedef MLANG_Vector<KEY> KEYS;
    typedef MLANG_Vector<ELEMENT> ELEMENTS;
protected:
    MGrammar_TokenManager();
    MGrammar_TokenManager(const MGrammar_TokenManager& tm);

public:
  
    virtual ~MGrammar_TokenManager();

public:

   static bool is_nil (MGrammar_TokenManager_ptr a)
   { return a == 0; }

   static MGrammar_TokenManager_ptr _nil()
   { return 0; }

   static MGrammar_TokenManager_ptr _duplicate(MGrammar_TokenManager_ptr tm)
   { return MLANG_acquire(tm); }

public:
    virtual MGrammar_TokenManager_var clone() = 0;

    /** define a token symbol */
    virtual void define(const MGrammar_TokenSymbol_var& ts) = 0;

    /** Get the name of the token manager */
    virtual const MLANG_String& getName() const = 0;

    /** Get a token string by index */
    virtual MLANG_String getTokenStringAt(int idx) const = 0;

    /** Get the TokenSymbol for a string */
    virtual MGrammar_TokenSymbol_var getTokenSymbol(const MLANG_String& sym) = 0;

    virtual MGrammar_TokenSymbol_var getTokenSymbolAt(int idx) = 0;

    /** Get an enumerator over the symbol table */
    virtual ELEMENTS getTokenSymbolElements() const = 0;

    virtual KEYS getTokenSymbolKeys() const = 0;

    /** Get the token vocabulary (read-only).
     * @return A Vector of Strings indexed by token type */
    virtual const VOCABULARY& getVocabulary() const = 0;

    /** Is this token manager read-only? */
    virtual bool isReadOnly() const = 0;

    virtual void mapToTokenSymbol(const MLANG_String& name, const MGrammar_TokenSymbol_var& sym) = 0;

    /** Get the highest token type in use */
    virtual int maxTokenType() const = 0;

    /** Get the next unused token type */
    virtual int nextTokenType() = 0;

    virtual void setName(const MLANG_String& n) = 0;

    virtual void setReadOnly(bool ro) = 0;

    /** Is a token symbol defined? */
    virtual bool tokenDefined(const MLANG_String& symbol) = 0;
};

#endif /* __MGRAMMAR_TOKENMANAGER_H__ */
