#ifndef __MGRAMMAR_TOOLERRORHANDLER_H__
#define __MGRAMMAR_TOOLERRORHANDLER_H__
#include "ace/Vector_T.h"
#include "MParser/BitSet.h"
#include "MGrammar/Lookahead.h"
#include "MGrammar/AlternativeBlock.h"
#include "MGrammar/BlockWithImpliedExitPath.h"

class MGrammar_Tool;
typedef class MGrammar_Tool *MGrammar_Tool_ptr;

class MGrammar_ToolErrorHandler;
typedef class MGrammar_ToolErrorHandler *MGrammar_ToolErrorHandler_ptr;
typedef class MGrammar_ToolErrorHandler *MGrammar_ToolErrorHandler_var;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/ToolErrorHandler.java#1 $
 */

class MGrammar_ToolErrorHandler

{
protected:
    MGrammar_ToolErrorHandler();

public:
    virtual ~MGrammar_ToolErrorHandler();

public:
    /** Issue a warning about ambiguity between a alternates
     * @param blk  The block being analyzed
     * @param lexicalAnalysis  true for lexical rule
     * @param depth  The depth of the ambiguity
     * @param sets  An array of bitsets containing the ambiguities
     * @param altIdx1  The zero-based index of the first ambiguous alternative
     * @param altIdx2  The zero-based index of the second ambiguous alternative
     */
    virtual void warnAltAmbiguity(
        MGrammar_Grammar_ptr grammar,
        MGrammar_AlternativeBlock_ptr blk,
        bool lexicalAnalysis,
        int depth,
        const ACE_Vector<MGrammar_Lookahead>& sets,
        int altIdx1,
        int altIdx2
        ) = 0;

    /** Issue a warning about ambiguity between an alternate and exit path.
     * @param blk  The block being analyzed
     * @param lexicalAnalysis  true for lexical rule
     * @param depth  The depth of the ambiguity
     * @param sets  An array of bitsets containing the ambiguities
     * @param altIdx  The zero-based index of the ambiguous alternative
     */
    virtual void warnAltExitAmbiguity(
        MGrammar_Grammar_ptr grammar,
        MGrammar_BlockWithImpliedExitPath_ptr blk,
        bool lexicalAnalysis,
        int depth,
        const ACE_Vector<MGrammar_Lookahead>& sets,
        int altIdx
        ) = 0;
};

#endif /* __MGRAMMAR_TOOLERRORHANDLER_H__ */
