#include "MGrammar/CodeGenerator.h"
#include "MGrammar/TreeWalkerGrammar.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/TreeWalkerGrammar.java#1 $
 */

MGrammar_TreeWalkerGrammar::MGrammar_TreeWalkerGrammar (
   const MLANG_String& className,
   MGrammar_Tool_ptr tool,
   const MLANG_String& superClass
) : MGrammar_Grammar(className, tool, superClass),
    transform_ (false)

{
}

/** Top-level call to generate the code for this grammar */
void MGrammar_TreeWalkerGrammar::generate()

{ 
   generator_->gen(this);
}

// Get name of class from which generated parser/lexer inherits
const MLANG_String MGrammar_TreeWalkerGrammar::getSuperClass() const

{
   return "TreeParser";
}

/**Process command line arguments.
 * -trace			have all rules call traceIn/traceOut
 * -traceParser		have parser rules call traceIn/traceOut
 * -debug			generate debugging output for parser debugger
 */
void MGrammar_TreeWalkerGrammar::processArguments(ACE_ARGV& args)

{
   for (int i = 0; i < args.argc(); i++)
    {
       if (MLANG_String(args[i]).equals("-trace"))
        {
          traceRules_ = true;
          antlrTool_->setArgOK(i);
        }
       else if (MLANG_String(args[i]).equals("-traceTreeParser"))
        {
          traceRules_ = true;
          antlrTool_->setArgOK(i);
        }
//     else if ( args[i].equals("-debug") ) 
//      {
//	  debuggingOutput = true;
//        superClass = "parseview.DebuggingTreeWalker";
//        Tool.setArgOK(i);
//      }
    }
}

/** Set tree parser options */
bool MGrammar_TreeWalkerGrammar::setOption (
   const MLANG_String& key,
   const MParser_Token_var& value
)

{
   if (key.equals("buildAST"))
    {
      if (value->getText().equals("true"))
       {
         buildAST_ = true;
       }
      else if (value->getText().equals("false"))
       {
         buildAST_ = false;
       }
      else
       {
         antlrTool_->error("buildAST option must be true or false", getFilename(), value->getLine(), value->getColumn());
       }
      return true;
    }
   if (key.equals("ASTLabelType"))
    {
      MGrammar_Grammar::setOption(key, value);
      return true;
    }
   if (MGrammar_Grammar::setOption(key, value))
    {
      return true;
    }
   antlrTool_->error("Invalid option: " + key, getFilename(), value->getLine(), value->getColumn());
   return false;
}

MGrammar_TreeWalkerGrammar::~MGrammar_TreeWalkerGrammar ()

{
}
