#ifndef __MIO_BUFFEREDREADER_H__
#define __MIO_BUFFEREDREADER_H__
#include "MLANG/SmartPtr.h"
#include "MIO/Reader.h"
#include "MLANG/String.h"

class MIO_BufferedReader;
typedef class MIO_BufferedReader *MIO_BufferedReader_ptr;

/**
 * @class MIO_BufferedReader
 *
 * @brief Implementa un lector de un flujo de entrada.
 */

class MIO_BufferedReader : public MIO_Reader
{
public:
   MIO_BufferedReader (
       const MIO_Reader_var& in,
       size_t sz = BUFSIZ
    );
   virtual ~MIO_BufferedReader ();

public:
   static bool is_nil (MIO_BufferedReader_ptr reader)
   { return reader == 0; }

   static MIO_BufferedReader_ptr _nil()
   { return 0; }

   static MIO_BufferedReader_ptr _duplicate(MIO_BufferedReader_ptr reader)
   { return MLANG_acquire(reader); }
public:
   /// Indica si el flujo de entrada se encuentra disponible para
   /// ser leido.
   /// @retval false, subclases deben implementar este metodo
   virtual bool ready();

   /// Marca la posicion actual en el flujo
   ///
   /// @param readAheadLimit Limita el numero de caracteres que pueden ser
   /// leidos mientras se preserva la marca. Despues de leer esta cantidad
   /// de caracteres reintentar reiniciar el flujo podria falla. 
   virtual void mark (int readAheadLimit);

   /// Lee una caracter del flujo de entrada
   ///
   /// @retval el caracter leído
   /// @retval -1, si se encontro el fin de archivo
   /// @throw MIO_IOException Si hubo algun error de entrada/salida
   virtual ssize_t read ();

   /// Lee un buffer de caracteres
   /// 
   /// @param buf apunta al buffer de caracters donde se colocaran los caracteres
   /// leidos.
   /// @param len es el maximo numero de caracteres a ser leidos.
   /// @retval el numero de caracteres leidos, o
   /// @retval -1 si se ha encontrado el fin del flujo.
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   virtual ssize_t read (char buf[], ssize_t len); 

   /// Cierra el flujo de lectura
   ///
   /// @throws MIO_IOException si hubo algun error de entrada
   /// salida
   virtual void close();

   /// Reinicia el flujo a la marca mas reciente
   ///
   virtual void reset();
public:

   virtual MLANG_String readLine();

protected:

   /// Llena el buffer para la proxima lectura con buffering
   ///
   void fill();

protected:
   MIO_Reader_var input_;
   char *buffer_;
   size_t size_;
   size_t off_;
   ssize_t count_;
};

/// El SmartPtr a un objeto de tipo MIO_BufferedReader
typedef MLANG_SmartPtr<MIO_BufferedReader> MIO_BufferedReader_var;

#endif /* __MIO_BUFFEREDREADER_H__ */
