#include "MLANG/StringBuffer.h"
#include "MIO/BufferedWriter.h"

MIO_BufferedWriter::MIO_BufferedWriter (
    const MIO_Writer_var& out,
    size_t sz
) : output_(out),
    buffer_((char*) ACE_OS::malloc(sizeof(char)*sz)),
    size_(sz),
    off_(0),
    count_(0)

{
}

void MIO_BufferedWriter::flush()

{
   if (count_>0)
    {
      output_->write (buffer_, count_);
      off_ = 0;
      count_ = 0;
    }
}

/// Lee una caracter del flujo de entrada
///
/// @retval el caracter leído
/// @retval -1, si se encontro el fin de archivo
/// @throw MIO_IOException Si hubo algun error de entrada/salida
void MIO_BufferedWriter::write (int c)

{
   char ch = (char) c;
   this->write (&ch, 1);
}

void MIO_BufferedWriter::newLine()

{
   write ('\n');
}

void MIO_BufferedWriter::write (const char *buf, size_t len)

{
   if (len >= size_)
    {
      flush();
      output_->write (buf, len);
      return;
    }
   for (size_t i=0; i<len; i++)
    {
       if (count_ == (ssize_t) size_)
        { 
          flush();
        }
       count_++;
       buffer_[off_++] = buf[i];
    }
}

void MIO_BufferedWriter::write (const MLANG_String& s)

{
   write (s.c_str(), s.length());
}

void MIO_BufferedWriter::write (
   const MLANG_String& s,
   int off,
   int len
)

{
   write (s.c_str() + off, len);
}

void MIO_BufferedWriter::close()

{
   flush();
   output_->close();
}


MIO_BufferedWriter::~MIO_BufferedWriter ()

{
   this->close();
   if (buffer_)
    {
      ACE_OS::free(buffer_);
      buffer_ = 0;
    }
}
