#ifndef __MIO_BUFFEREDWRITER_H__
#define __MIO_BUFFEREDWRITER_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MIO/Writer.h"

class MIO_BufferedWriter;
typedef class MIO_BufferedWriter *MIO_BufferedWriter_ptr;

class MIO_BufferedWriter : public MIO_Writer
{
public:
   MIO_BufferedWriter (
     const MIO_Writer_var& output,
     size_t sz = BUFSIZ
    );
public:
   virtual ~MIO_BufferedWriter();

public:
   static bool is_nil (MIO_BufferedWriter_ptr writer)
   { return writer == 0; }

   static MIO_BufferedWriter_ptr _nil()
   { return 0; }

   static MIO_BufferedWriter_ptr _duplicate(MIO_BufferedWriter_ptr writer)
   { return MLANG_acquire(writer); }
public:
   /// Cierra el flujo de salida realizando vaciado de los datos si es necesario
   ///
   virtual void close();

   /// Realiza un vaciado de los datos si es necesario
   ///
   virtual void flush();

   virtual void newLine();

   /// Escribe el @a buffer de @a count bytes al flujo de salida
   ///
   /// @throws IOException si hubo algun error de entrada salida
   virtual void write (const char* buffer, size_t count);

   /// 
   virtual void write (int c);

   virtual void write (const MLANG_String& s);

   virtual void write (const MLANG_String&s, int off, int len);
protected:
   MIO_Writer_var output_;
   char *buffer_;
   size_t size_;
   size_t off_;
   ssize_t count_;
};

typedef MLANG_SmartPtr<MIO_BufferedWriter> MIO_BufferedWriter_var;

#endif /* __MIO_BUFFEREDWRITER_H__ */
