#include "ace/OS.h"
#include "MIO/File.h"

#if defined(WIN32)
#define ACE_stat _stat
#else /* !WIN32 */
#define ACE_stat stat
#endif /* !WIN32 */

MIO_File::MIO_File (
   const MLANG_String& filename
) : filename_ (filename)

{
}

MIO_File::MIO_File (
   const MLANG_String& parent,
   const MLANG_String& child
)

{
   MLANG_String sep = separator();
   filename_ = parent;
   if (filename_.lastIndexOf (sep)!=(int)(filename_.length() - sep.length()))
    {
      filename_ += sep;
    }
   filename_ += child;
}

bool MIO_File::isAbsolute() const

{
   return filename_.startsWith (separator());
}

MLANG_String MIO_File::getParent() const

{
   int pos = filename_.lastIndexOf (separator());
   if (pos < 0)
      return "";
   else
      return filename_.substring (0, pos - 1);
}

bool MIO_File::exists () const

{
   struct ACE_stat st;
   if (ACE_OS::stat (filename_.c_str(), &st)==-1)
      return false;
   else
      return true;
}

MLANG_String MIO_File::separator()

{
   return ACE_DIRECTORY_SEPARATOR_STR;
}

bool MIO_File::isFile() const

{
   struct ACE_stat st;
   int result = ACE_OS::stat (filename_.c_str(), &st);
   if (result < 0)
      return false;
   else
#if defined(WIN32)
      assert(0);
#else
      return S_ISREG(st.st_mode)!=0;
#endif /* !WIN32 */
}


bool MIO_File::canRead() const

{
   struct ACE_stat st;
   int result = ACE_OS::stat (filename_.c_str(), &st);
   if (result < 0)
      return false;
   else
      return (st.st_mode&(S_IRUSR|S_IRGRP|S_IROTH))!=0;
}

bool MIO_File::canWrite() const

{
   struct ACE_stat st;
   int result = ACE_OS::stat (filename_.c_str(), &st);
   if (result < 0)
      return false;
   else
      return (st.st_mode&(S_IRUSR|S_IRGRP|S_IROTH))!=0;
}

long MIO_File::length() const

{
   struct ACE_stat st;
   int result = ACE_OS::stat (filename_.c_str(), &st);
   if (result < 0)
      return 0;
   else
      return st.st_size;
}

bool MIO_File::mkdir () const

{
   int result = ACE_OS::mkdir (filename_.c_str(), 0640);
   if (result < 0 && errno!=EEXIST)
      return false;
   else
      return true;
}

bool MIO_File::mkdirs () const

{
   MLANG_String parent = getParent();
   if (parent.length()!=0)
    {
      MIO_File_var f = new MIO_File (parent);
      if (!f->mkdirs())
         return false;
      return f->mkdir();
    }
   return true; 
}

bool MIO_File::remove() const

{
   int result = ACE_OS::unlink (filename_.c_str());
   if (result < 0)
      return false;
   else
      return true;
}

MIO_File::~MIO_File ()

{
}
