#include "ace/OS.h"
#include "MIO/IOException.h"
#include "MIO/FileNotFoundException.h"
#include "MIO/FileInputStream.h"

MIO_FileInputStream::MIO_FileInputStream (
   ACE_HANDLE fd
) : fd_(fd)

{
}

MIO_FileInputStream::MIO_FileInputStream (
   const MIO_File_var& file
) : fd_(ACE_INVALID_HANDLE)

{
  this->_open (file->filename());
}

MIO_FileInputStream::MIO_FileInputStream (
   const MLANG_String& filename
) : fd_(ACE_INVALID_HANDLE)

{
  this->_open (filename);
}


void MIO_FileInputStream::_open(const MLANG_String& filename)

{
   ACE_HANDLE fd = ACE_OS::open (filename.c_str(), O_RDONLY);
   if (fd == ACE_INVALID_HANDLE)
    {
      if (errno==ENOENT)
       {
         throw MIO_FileNotFoundException (
             filename + ":" + sys_errlist[errno]
          );
       }
      this->_raise_IOException();
    }
   fd_ = fd;
}

void MIO_FileInputStream::_raise_IOException ()

{
   switch (errno)
    {
    case 0:
         break;
    default:
         if (errno > 0 && errno < sys_nerr)
            throw MIO_IOException(sys_errlist[errno]);
         else
          {
            char msg[128+1];
            sprintf(msg, "(errno = %d)", errno);
            throw MIO_IOException(msg);
          }
    }
}

/// Marca la posicion actual en el flujo
///
/// @param readAheadLimit Limita el numero de caracteres que pueden ser
/// leidos mientras se preserva la marca. Despues de leer esta cantidad
/// de caracteres reintentar reiniciar el flujo podria falla. 
void MIO_FileInputStream::mark (int readAheadLimit)

{
   ACE_UNUSED_ARG(readAheadLimit);
}

/// Lee un buffer de caracteres
/// 
/// @param buf apunta al buffer de caracters donde se colocaran los caracteres
/// leidos.
/// @param len es el maximo numero de caracteres a ser leidos.
/// @retval el numero de caracteres leidos, o
/// @retval -1 si se ha encontrado el fin del flujo.
/// @throw MIO_IOException si hubo algun error de entrada/salida
ssize_t MIO_FileInputStream::read (char buf[], ssize_t len)

{
   int result = ACE_OS::read (fd_, buf, len);
   if (result < 0)
    {
      this->_raise_IOException();
    }
   else if (result == 0)
    {
      return -1;
    }    
   return result;
}

/// Cierra el flujo de lectura
///
/// @throws MIO_IOException si hubo algun error de entrada
/// salida
void MIO_FileInputStream::close()

{
   if (fd_!=ACE_INVALID_HANDLE)
    {
      int result = ACE_OS::close(fd_);
      if (result < 0)
       {
         this->_raise_IOException();
       }
    }
   fd_ = ACE_INVALID_HANDLE;
}

/// Reinicia el flujo a la marca mas reciente
///
void MIO_FileInputStream::reset()

{
}

MIO_FileInputStream::~MIO_FileInputStream ()

{
   this->close();
}
