#ifndef __MIO_FILEINPUTSTREAM_H__
#define __MIO_FILEINPUTSTREAM_H__
#include "ace/OS.h"
#include "ace/Synch.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MIO/Export.h"
#include "MIO/File.h"
#include "MIO/InputStream.h"

class MIO_Export MIO_FileInputStream;
typedef class MIO_FileInputStream *MIO_FileInputStream_ptr;

class MIO_Export MIO_FileInputStream : public MIO_InputStream
{
public:
   MIO_FileInputStream (ACE_HANDLE fd);

   MIO_FileInputStream(const MIO_File_var& file);

   MIO_FileInputStream(const MLANG_String& filename);
public:
   virtual ~MIO_FileInputStream();
public:
   static bool is_nil (MIO_FileInputStream_ptr a)
   { return a == 0; }

   static MIO_FileInputStream_ptr _nil()
   { return 0; }

   static MIO_FileInputStream_ptr _duplicate(MIO_FileInputStream_ptr stream)
   { return MLANG_acquire(stream); }

public:
   /// Retorna el descriptor de archivo asociado a este flujo
   ///
   /// @retval fd_, el descriptor del archivo asociado
   virtual ACE_HANDLE getFD() const
   { return fd_; }

   /// Marca la posicion actual en el flujo
   ///
   /// @param readAheadLimit Limita el numero de caracteres que pueden ser
   /// leidos mientras se preserva la marca. Despues de leer esta cantidad
   /// de caracteres reintentar reiniciar el flujo podria falla. 
   virtual void mark (int readAheadLimit);

   /// Lee un buffer de caracteres
   /// 
   /// @param buf apunta al buffer de caracters donde se colocaran los caracteres
   /// leidos.
   /// @param len es el maximo numero de caracteres a ser leidos.
   /// @retval el numero de caracteres leidos, o
   /// @retval -1 si se ha encontrado el fin del flujo.
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   virtual ssize_t read (char buf[], ssize_t len); 

   /// Cierra el flujo de lectura
   ///
   /// @throws MIO_IOException si hubo algun error de entrada
   /// salida
   virtual void close();

   /// Reinicia el flujo a la marca mas reciente
   ///
   virtual void reset();
protected:
   /// Abre el archivo @a filename
   ///
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   void _open (const MLANG_String& filename);
   
   /// Surge una excepcion cuando hay un error de entrada/salida
   ///
   void _raise_IOException();

protected:
   /// Descriptor del archivo asignado por el sistema operativo
   ///
   ACE_HANDLE fd_;
};

typedef MLANG_SmartPtr<MIO_FileInputStream> MIO_FileInputStream_var;

#endif /* __MIO_FILEINPUTSTREAM_H__ */
