#include "ace/OS.h"
#include "MIO/IOException.h"
#include "MIO/FileNotFoundException.h"
#include "MIO/FileOutputStream.h"

MIO_FileOutputStream::MIO_FileOutputStream (
   ACE_HANDLE fd
) : fd_(fd)

{
}

MIO_FileOutputStream::MIO_FileOutputStream (
   const MIO_File_var& file,
   bool append
) : fd_(ACE_INVALID_HANDLE)

{
  this->_open (file->filename(), append);
}

MIO_FileOutputStream::MIO_FileOutputStream (
   const MLANG_String& filename,
   bool append
) : fd_(ACE_INVALID_HANDLE)

{
  this->_open (filename, append);
}


void MIO_FileOutputStream::_open (
   const MLANG_String& filename,
   bool append
)

{
   int flags = O_WRONLY|O_CREAT;
   if (append)
    {
      flags |= O_APPEND;
    }
   else
    {
      flags |= O_TRUNC;
    }
   ACE_HANDLE fd = ACE_OS::open (filename.c_str(), flags, 0660);
   if (fd == ACE_INVALID_HANDLE)
    {
      if (errno==ENOENT)
       {
         throw MIO_FileNotFoundException (
             filename + ":" + sys_errlist[errno]
          );
       }
      this->_raise_IOException();
    }
   fd_ = fd;
}

void MIO_FileOutputStream::_raise_IOException ()

{
   switch (errno)
    {
    case 0:
         break;
    default:
         if (errno > 0 && errno < sys_nerr)
            throw MIO_IOException(sys_errlist[errno]);
         else
          {
            char msg[128+1];
            sprintf(msg, "(errno = %d)", errno);
            throw MIO_IOException(msg);
          }
    }
}

/// Lee un buffer de caracteres
/// 
/// @param buf apunta al buffer de caracters donde se colocaran los caracteres
/// leidos.
/// @param len es el maximo numero de caracteres a ser leidos.
/// @retval el numero de caracteres leidos, o
/// @retval -1 si se ha encontrado el fin del flujo.
/// @throw MIO_IOException si hubo algun error de entrada/salida
void MIO_FileOutputStream::write (const char buf[], ssize_t len)

{
   int result = ACE_OS::write (fd_, buf, len);
   if (result < 0)
    {
      this->_raise_IOException();
    }
}

void MIO_FileOutputStream::write (int c)

{
   char ch = (char) c;
   write (&ch, 1);
}

/// Cierra el flujo de escriture
///
/// @throws MIO_IOException si hubo algun error de entrada
/// salida
void MIO_FileOutputStream::close()

{
   if (fd_!=ACE_INVALID_HANDLE)
    {
      int result = ACE_OS::close(fd_);
      if (result < 0)
       {
         this->_raise_IOException();
       }
    }
   fd_ = ACE_INVALID_HANDLE;
}

MIO_FileOutputStream::~MIO_FileOutputStream ()

{
   this->close();
}
