#ifndef __MIO_FILEOUTPUTSTREAM_H__
#define __MIO_FILEOUTPUTSTREAM_H__
#include "ace/OS.h"
#include "ace/Synch.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MIO/Export.h"
#include "MIO/File.h"
#include "MIO/OutputStream.h"

class MIO_Export MIO_FileOutputStream;
typedef class MIO_FileOutputStream *MIO_FileOutputStream_ptr;

class MIO_Export MIO_FileOutputStream : public MIO_OutputStream
{
public:
   MIO_FileOutputStream (
      ACE_HANDLE fd
    );

   MIO_FileOutputStream (
     const MIO_File_var& fileName,
     bool append = false
   );

   MIO_FileOutputStream (
     const MLANG_String& fileName,
     bool append = false
   );

public:
   virtual ~MIO_FileOutputStream();
public:
   static bool is_nil (MIO_FileOutputStream_ptr a)
   { return a == 0; }

   static MIO_FileOutputStream_ptr _nil()
   { return 0; }

   static MIO_FileOutputStream_ptr _duplicate(MIO_FileOutputStream_ptr stream)
   { return MLANG_acquire(stream); }

public:
   /// Retorna el descriptor de archivo asociado a este flujo
   ///
   /// @retval fd_, el descriptor del archivo asociado
   virtual ACE_HANDLE getFD() const
   { return fd_; }

   /// Escribe un buffer de caracteres
   /// 
   /// @param buf apunta al buffer de caracters donde se colocaran los caracteres
   /// leidos.
   /// @param len es el maximo numero de caracteres a ser leidos.
   /// @retval el numero de caracteres leidos, o
   /// @retval -1 si se ha encontrado el fin del flujo.
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   virtual void write (const char buf[], ssize_t len); 

   /// leidos.
   /// @param len es el maximo numero de caracteres a ser leidos.
   /// @retval el numero de caracteres leidos, o
   /// @retval -1 si se ha encontrado el fin del flujo.
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   virtual void write (int c);

   /// Cierra el flujo de lectura
   ///
   /// @throws MIO_IOException si hubo algun error de entrada
   /// salida
   virtual void close();

protected:
   /// Abre el archivo @a filename
   ///
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   void _open (
      const MLANG_String& filename,
      bool append
    );
   
   /// Surge una excepcion cuando hay un error de entrada/salida
   ///
   void _raise_IOException();

protected:
   /// Descriptor del archivo asignado por el sistema operativo
   ///
   ACE_HANDLE fd_;
};

typedef MLANG_SmartPtr<MIO_FileOutputStream> MIO_FileOutputStream_var;

#endif /* __MIO_FILEOUTPUTSTREAM_H__ */
