#ifndef __MIO_INPUTSTREAM_H__
#define __MIO_INPUTSTREAM_H__
#include "ace/Synch.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/Refcountable.h"
#include "MIO/Export.h"

class MIO_Export MIO_InputStream;
typedef class MIO_InputStream *MIO_InputStream_ptr;

class MIO_Export MIO_InputStream : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
protected:
   MIO_InputStream();
public:
   virtual ~MIO_InputStream();
public:
   static bool is_nil (MIO_InputStream_ptr a)
   { return a == 0; }

   static MIO_InputStream_ptr _nil()
   { return 0; }

   static MIO_InputStream_ptr _duplicate(MIO_InputStream_ptr stream)
   { return MLANG_acquire(stream); }

public:
   /// Retorna el numero de bytes disponibles para lectura
   ///
   /// @retval 0, las subclases deben sobreescribir este metodo
   virtual int available();

   /// Marca la posicion actual en el flujo
   ///
   /// @param readAheadLimit Limita el numero de caracteres que pueden ser
   /// leidos mientras se preserva la marca. Despues de leer esta cantidad
   /// de caracteres reintentar reiniciar el flujo podria falla. 
   virtual void mark (int readAheadLimit);

   /// Lee una caracter del flujo de entrada
   ///
   /// @retval el caracter leído
   /// @retval -1, si se encontro el fin de archivo
   /// @throw MIO_IOException Si hubo algun error de entrada/salida
   virtual ssize_t read ();

   /// Lee un buffer de caracteres
   /// 
   /// @param buf apunta al buffer de caracters donde se colocaran los caracteres
   /// leidos.
   /// @param len es el maximo numero de caracteres a ser leidos.
   /// @retval el numero de caracteres leidos, o
   /// @retval -1 si se ha encontrado el fin del flujo.
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   virtual ssize_t read (char buf[], ssize_t len) = 0; 

   /// Cierra el flujo de lectura
   ///
   /// @throws MIO_IOException si hubo algun error de entrada
   /// salida
   virtual void close();

   /// Reinicia el flujo a la marca mas reciente
   ///
   virtual void reset();
};

typedef MLANG_SmartPtr<MIO_InputStream> MIO_InputStream_var;

#endif /* __MIO_INPUTSTREAM_H__ */
