#ifndef __MIO_INPUTSTREAMREADER_H__
#define __MIO_INPUTSTREAMREADER_H__
#include "MLANG/SmartPtr.h"
#include "MIO/Reader.h"
#include "MIO/InputStream.h"

class MIO_InputStreamReader;
typedef class MIO_InputStreamReader *MIO_InputStreamReader_ptr;

/**
 * @class MIO_InputStreamReader
 *
 * @brief
 */

class MIO_InputStreamReader : public MIO_Reader
{
public:
   MIO_InputStreamReader (
      const MIO_InputStream_var& input
    );
   virtual ~MIO_InputStreamReader ();
public:
   static bool is_nil (MIO_InputStreamReader_ptr a)
   { return a == 0; }

   static MIO_InputStreamReader_ptr _nil()
   { return 0; }

   static MIO_InputStreamReader_ptr _duplicate(MIO_InputStreamReader_ptr reader)
   { return MLANG_acquire(reader); }

public:
   /// Marca la posicion actual en el flujo
   ///
   /// @param readAheadLimit Limita el numero de caracteres que pueden ser
   /// leidos mientras se preserva la marca. Despues de leer esta cantidad
   /// de caracteres reintentar reiniciar el flujo podria falla. 
   virtual void mark (int readAheadLimit);

   /// Lee una caracter del flujo de entrada
   ///
   /// @retval el caracter leído
   /// @retval -1, si se encontro el fin de archivo
   /// @throw MIO_IOException Si hubo algun error de entrada/salida
   virtual ssize_t read ();

   /// Lee hasta @a len caracteres y los coloca en @a buf.
   ///
   /// @param buf apunta al buffer adonde se deben transferir los caracteres
   /// leidos.
   /// @param len es el maximo numero de caracteres a leer
   /// @retval el numero de caracteres leidos o -1 si se produjo el fin de
   /// la entrada.
   /// @throws MIO_IOException si hubo algun error de entrada salida.
   virtual ssize_t read(char buf[], ssize_t len);

   /// Cierra el flujo de entrada
   ///
   /// @throws MIO_IOException si hubo algun error de entrada/salida
   virtual void close();

protected:

   MIO_InputStream_var input_;
};

typedef MLANG_SmartPtr<MIO_InputStreamReader> MIO_InputStreamReader_var;

#endif /* __MIO_INPUTSTREAMREADER_H__ */
