#ifndef __MIO_PRINTWRITER_H__
#define __MIO_PRINTWRITER_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MIO/Writer.h"
#include "MIO/OutputStream.h"

class MIO_PrintWriter;
typedef class MIO_PrintWriter *MIO_PrintWriter_ptr;

class MIO_PrintWriter : public MIO_Writer
{
public:
   MIO_PrintWriter ( 
      const MIO_Writer_var&,
      bool autoflush = false
    );
   MIO_PrintWriter (
      const MIO_OutputStream_var&,
      bool autoflush = false
    );
   virtual ~MIO_PrintWriter ();

public:
   static bool is_nil (MIO_PrintWriter_ptr writer)
   { return writer == 0; }

   static MIO_PrintWriter_ptr _nil()
   { return 0; }

   static MIO_PrintWriter_ptr _duplicate(MIO_PrintWriter_ptr writer)
   { return MLANG_acquire(writer); }
public:
   virtual void close();
   virtual void flush();
   virtual void write (const char *buf, size_t len);
   virtual void write (char c);
public:
   virtual void print(bool b);
   virtual void print(char c);
   virtual void print(const char* buf, ssize_t len);
   virtual void print(double d);
   virtual void print(float f);
   virtual void print(short s);
   virtual void print(long l);
   virtual void print(const MLANG_String& s);
public:
   virtual void println();
   virtual void println(bool b);
   virtual void println(char c);
   virtual void println(const char* buf, ssize_t len);
   virtual void println(double d);
   virtual void println(float f);
   virtual void println(short s);
   virtual void println(long l);
   virtual void println(const MLANG_String& s);
protected:
   MIO_Writer_var output_;
   bool autoflush_;
};

typedef MLANG_SmartPtr<MIO_PrintWriter> MIO_PrintWriter_var;

#endif /* __MIO_PRINTWRITER_H__ */
