#ifndef __MIO_READER_H__
#define __MIO_READER_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"

class MIO_Reader;
typedef class MIO_Reader *MIO_Reader_ptr;

/**
 * @class MIO_Reader
 *
 * @brief Implementa un lector de un flujo de entrada.
 */

class MIO_Reader : public MLANG_Refcountable<ACE_SYNCH_MUTEX>

{
   // = Creacion y destruccion de un flujo de entrada
protected:
   /// Creacion de un flujo de entrada
   MIO_Reader();
public:
   ///Destructor
   virtual ~MIO_Reader();
public:
   static bool is_nil (MIO_Reader_ptr a)
   { return a == 0; }

   static MIO_Reader_ptr _nil()
   { return 0; }

   static MIO_Reader_ptr _duplicate(MIO_Reader_ptr reader)
   { return MLANG_acquire(reader); }
public:
   /// Indica si el flujo de entrada se encuentra disponible para
   /// ser leido.
   /// @retval false, subclases deben implementar este metodo
   virtual bool ready() 
   { return false; }
   
   /// Indica si el flujo de entrada soporta marcas
   ///
   /// @retval false, subclases deben implementar este metodo
   virtual bool markSupported()
   { return false; }

   /// Marca la posicion actual en el flujo
   ///
   /// @param readAheadLimit Limita el numero de caracteres que pueden ser
   /// leidos mientras se preserva la marca. Despues de leer esta cantidad
   /// de caracteres reintentar reiniciar el flujo podria falla. 
   virtual void mark (int readAheadLimit);

   /// Lee una caracter del flujo de entrada
   ///
   /// @retval el caracter leído
   /// @retval -1, si se encontro el fin de archivo
   /// @throw MIO_IOException Si hubo algun error de entrada/salida
   virtual ssize_t read ();

   /// Lee un buffer de caracteres
   /// 
   /// @param buf apunta al buffer de caracters donde se colocaran los caracteres
   /// leidos.
   /// @param len es el maximo numero de caracteres a ser leidos.
   /// @retval el numero de caracteres leidos, o
   /// @retval -1 si se ha encontrado el fin del flujo.
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   virtual ssize_t read (char buf[], ssize_t len) = 0; 

   /// Cierra el flujo de lectura
   ///
   /// @throws MIO_IOException si hubo algun error de entrada
   /// salida
   virtual void close() = 0;

   /// Reinicia el flujo a la marca mas reciente
   ///
   virtual void reset();
};

/// El SmartPtr a un objeto de tipo MIO_Reader
typedef MLANG_SmartPtr<MIO_Reader> MIO_Reader_var;

#endif /* __MIO_READER_H__ */
