#include "MIO/StringReader.h"

MIO_StringReader::MIO_StringReader (const MLANG_String & buf)
  : buf_(buf),
    off_(0),
    mark_(0)

{
}

/// Marca la posicion actual en el flujo
///
/// @param readAheadLimit Limita el numero de caracteres que pueden ser
/// leidos mientras se preserva la marca. Despues de leer esta cantidad
/// de caracteres reintentar reiniciar el flujo podria falla. 
void MIO_StringReader::mark (int readAheadLimit)

{
   ACE_UNUSED_ARG(readAheadLimit);
   mark_ = off_;
}

/// Lee una caracter del flujo de entrada
///
/// @retval el caracter leído
/// @retval -1, si se encontro el fin de archivo
/// @throw MIO_IOException Si hubo algun error de entrada/salida
ssize_t MIO_StringReader::read ()

{
   if (off_<(ssize_t)buf_.length())
    {
      return (unsigned short) buf_[off_++];
    }
   else
    {
      return -1;
    }
}

/// Lee un buffer de caracteres
/// 
/// @param buf apunta al buffer de caracters donde se colocaran los caracteres
/// leidos.
/// @param len es el maximo numero de caracteres a ser leidos.
/// @retval el numero de caracteres leidos, o
/// @retval -1 si se ha encontrado el fin del flujo.
/// @throw MIO_IOException si hubo algun error de entrada/salida
ssize_t MIO_StringReader::read (char buf[], ssize_t len)

{
   ssize_t n = 0;
   ssize_t c;
   while (n<len && (c=read())!=-1)
    {
      buf[n] = (char) c;
      n++;
    }
   if (n==0)
      return -1;
   else
      return n;
}

/// Cierra el flujo de lectura
///
/// @throws MIO_IOException si hubo algun error de entrada
/// salida
void MIO_StringReader::close()

{
}

/// Reinicia el flujo a la marca mas reciente
///
void MIO_StringReader::reset()

{
   off_ = mark_;
}

MIO_StringReader::~MIO_StringReader()

{
}
