#ifndef __MIO_STRINGREADER_H__
#define __MIO_STRINGREADER_H__
#include "MLANG/String.h"
#include "MLANG/SmartPtr.h"
#include "MIO/Reader.h"

class MIO_StringReader;
typedef MIO_StringReader *MIO_StringReader_ptr;

class MIO_StringReader : public MIO_Reader
{
public:
   MIO_StringReader (const MLANG_String & s);
   virtual ~MIO_StringReader();
public:

   static bool is_nil (MIO_StringReader_ptr reader)
   { return reader == 0; }

   static MIO_StringReader_ptr _nil()
   { return 0; }

   static MIO_StringReader_ptr _duplicate(MIO_StringReader_ptr reader)
   { return MLANG_acquire(reader); }

public:
   /// Indica si el flujo de entrada se encuentra disponible para
   /// ser leido.
   /// @retval false, subclases deben implementar este metodo
   virtual bool ready() 
   { return true; }
   
   /// Indica si el flujo de entrada soporta marcas
   ///
   /// @retval false, subclases deben implementar este metodo
   virtual bool markSupported()
   { return true; }

   /// Marca la posicion actual en el flujo
   ///
   /// @param readAheadLimit Limita el numero de caracteres que pueden ser
   /// leidos mientras se preserva la marca. Despues de leer esta cantidad
   /// de caracteres reintentar reiniciar el flujo podria falla. 
   virtual void mark (int readAheadLimit);

   /// Lee una caracter del flujo de entrada
   ///
   /// @retval el caracter leído
   /// @retval -1, si se encontro el fin de archivo
   /// @throw MIO_IOException Si hubo algun error de entrada/salida
   virtual ssize_t read ();

   /// Lee un buffer de caracteres
   /// 
   /// @param buf apunta al buffer de caracters donde se colocaran los caracteres
   /// leidos.
   /// @param len es el maximo numero de caracteres a ser leidos.
   /// @retval el numero de caracteres leidos, o
   /// @retval -1 si se ha encontrado el fin del flujo.
   /// @throw MIO_IOException si hubo algun error de entrada/salida
   virtual ssize_t read (char buf[], ssize_t len); 

   /// Cierra el flujo de lectura
   ///
   /// @throws MIO_IOException si hubo algun error de entrada
   /// salida
   virtual void close();

   /// Reinicia el flujo a la marca mas reciente
   ///
   virtual void reset();
protected:
   MLANG_String buf_;
   int off_;
   int mark_;
};

typedef MLANG_SmartPtr<MIO_StringReader> MIO_StringReader_var;

#endif /* __MIO_STRINGREADER_H__ */
