#ifndef __MLANG_CLASS_H__
#define __MLANG_CLASS_H__
#include "MLANG/Export.h"
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MLANG/Exception.h"
#include "MLANG/Reflection/Type.h"
#include "MLANG/Reflection/Class.h"

class MLANG_Export MLANG_Class;
typedef class MLANG_Export MLANG_Class *MLANG_Class_ptr;
typedef MLANG_SmartPtr<MLANG_Class> MLANG_Class_var;

class MLANG_Export MLANG_ClassNotFoundException : public MLANG_Exception
{
public:
   MLANG_ClassNotFoundException (const char *name);
   virtual ~MLANG_ClassNotFoundException ();
};

class MLANG_Export MLANG_InstantiationException : public MLANG_Exception
{
public:
   MLANG_InstantiationException (const char *message);
   virtual ~MLANG_InstantiationException ();
};

class MLANG_Export MLANG_IllegalAccessException : public MLANG_Exception
{
public:
   MLANG_IllegalAccessException (const char *message);
   virtual ~MLANG_IllegalAccessException ();
};

class MLANG_Export MLANG_IllegalArgumentException : public MLANG_Exception
{
public:
   MLANG_IllegalArgumentException (const char *message);
   virtual ~MLANG_IllegalArgumentException ();
};

class MLANG_Export MLANG_Class : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
protected:
   MLANG_Class (const MLANG_Reflection::Type& t);
   MLANG_Class (const MLANG_Reflection::Class& c);
public:
   virtual ~MLANG_Class ();

public:
   static bool is_nil (MLANG_Class_ptr c)
   { return c == 0; }

   static MLANG_Class_ptr _nil()
   { return 0; }

   static MLANG_Class_ptr _duplicate(MLANG_Class_ptr c)
   { return MLANG_acquire(c); }
public:

   virtual MLANG_String getName() const;

   virtual void* newInstance();

   virtual MLANG_String toString() const;

public:
   static MLANG_Class_var forName (const MLANG_String& name);

   static MLANG_Class_var forTypeId (const std::type_info& ti);

protected:
   MLANG_Reflection::Class impl_;
};

#define DECLARE_CLASS_INFO() \
public: \
\
   MLANG_Class_var getClass(); \
\
   static MLANG_Class_var Class()

#define DEFINE_CLASS_INFO(Base) \
MLANG_Class_var Base::Class() \
\
{ \
   return MLANG_Class::forTypeId(typeid(Base)); \
} \
\
MLANG_Class_var Base::getClass() \
\
{ \
   return MLANG_Class::forTypeId(typeid(*this)); \
}

#endif /* __MLANG_CLASS_H__ */
