// -*- C++ -*-

//=============================================================================
/**
 *  @file   Object.h
 *
 *  Object.h,v 4.3 2002/04/10 17:44:16 ossama Exp
 *
 *  @author Arturo Montes
 */
//=============================================================================
#ifndef MLANG_OBJECT_H
#define MLANG_OBJECT_H
#include "ace/Synch.h"
#include "MLANG/Export.h"
#include "MLANG/SmartPtr.h"

class MLANG_Class;
typedef class MLANG_Class *MLANG_Class_ptr;
typedef MLANG_SmartPtr<MLANG_Class> MLANG_Class_var;

namespace MLANG
{

/**
 * @class Object
 *
 * @brief
 *
 *
 */
class MLANG_Export Object
{
public:
  /// Destructor.
  virtual ~Object (void);

  // = Increment/Decrement refcount
  size_t increment (void);
  size_t decrement (void);

  /// Returns the current refcount.
  size_t refcount (void);

public:

  virtual MLANG_Class_var getClass();

  static MLANG_Class_var Class();

protected:
  /// Protected constructor.
  Object (size_t refcount = 1);

  typedef ACE_SYNCH_MUTEX LOCK;
  /// Lock para acceso al contador
  LOCK lock_;

  /// Current refcount.
  size_t refcount_;
private:
  Object (const Object& o);
  Object& operator=(const Object& o);
};


#if defined (__ACE_INLINE__)
#include "MLANG/Object.i"
#endif /* __ACE_INLINE __ */
}

#define MLANG_Object MLANG::Object

#endif /*MLANG_REFCOUNTABLE_H*/
