// -*- C++ -*-

//=============================================================================
/**
 *  @file   Refcountable.h
 *
 *  Refcountable.h,v 4.3 2002/04/10 17:44:16 ossama Exp
 *
 *  @author Arturo Montes
 */
//=============================================================================
#ifndef MLANG_REFCOUNTABLE_H
#define MLANG_REFCOUNTABLE_H
#include "ace/Synch.h"
#include "ace/Atomic_Op.h"
#include "MLANG/Export.h"

namespace MLANG
{
/**
 * @class Refcountable
 *
 * @brief
 *
 *
 */
template <class LOCK>
class MLANG_Export Refcountable
{
public:
  /// Destructor.
  virtual ~Refcountable (void);

  // = Increment/Decrement refcount
  size_t increment (void);
  size_t decrement (void);

  /// Returns the current refcount.
  size_t refcount (void);

protected:
  /// Protected constructor.
  Refcountable (size_t refcount = 1);

  /// Lock para acceso al contador
  ACE_Atomic_Op <LOCK, long> refcount_;
};

}

#if defined (__ACE_INLINE__)
#include "MLANG/Refcountable.i"
#endif /* __ACE_INLINE __ */

#if defined (ACE_TEMPLATES_REQUIRE_SOURCE)
#include "MLANG/Refcountable.cpp"
#endif /* ACE_TEMPLATES_REQUIRE_SOURCE */

#if defined (ACE_TEMPLATES_REQUIRE_PRAGMA)
#pragma implementation ("Refcountable.cpp")
#endif /* ACE_TEMPLATES_REQUIRE_PRAGMA */

#define MLANG_Refcountable MLANG::Refcountable

#endif /*MLANG_REFCOUNTABLE_H*/
