//$Id: Any.h,v 1.1 2004/07/27 14:22:39 mato Exp $

// See http://www.boost.org/libs/any for Documentation.

// Copyright Kevlin Henney, 2000, 2001, 2002. All rights reserved.
//
// Permission to use, copy, modify, and distribute this software for any
// purpose is hereby granted without fee, provided that this copyright and
// permissions notice appear in all copies and derivatives.
//
// This software is provided "as is" without express or implied warranty.

#ifndef MLANG_REFLECTION_BOOSTANY
#define MLANG_REFLECTION_BOOSTANY

// what:  variant type boost::any
// who:   contributed by Kevlin Henney,
//        with features contributed and bugs found by
//        Ed Brey, Mark Rodgers, Peter Dimov, and James Curran
// when:  July 2001
// where: tested with BCC 5.5, MSVC 6.0, and g++ 2.95

//#include <ostream>
#include <typeinfo>
#include <ostream>

namespace MLANG_Reflection

{
    
   /** 
    * @class Any BoostAny.h Reflection/BoostAny.h
    * @author K. Henney
    */
   class Any
   {
   public: 
      
     /** Constructor */
     Any() 
       : content( 0 ) {}
      
     /** Constructor */
     template< typename ValueType > Any( const ValueType & value ) 
       : content( new holder<ValueType>( value )) {}
      
     /** Copy Constructor */
     Any(const Any & other)
       : content( other.content ? other.content->clone() : 0 ) {}
      
     ~Any()
      {
        delete content;
      }
      
      /** Modifier */
      Any & swap( Any & rhs )
      {
        std::swap( content, rhs.content);
        return *this;
      }
      
      /** Modifier */
      template< typename ValueType > Any & operator=( const ValueType & rhs )
      {
        Any( rhs ).swap( * this );
        return * this;
      }
      
      /** Modifier */
      Any & operator=( const Any & rhs )
      {
        Any( rhs ).swap( * this );
        return * this;
      }
      
      /** Query */
      bool empty() const
      {
        return ! content;
      }
      
      /** Query */
      const std::type_info & type() const
      {
        return content ? content->type() : typeid( void );
      }

      /** stream operator */
      friend std::ostream& operator << ( std::ostream&, const Any& );

    private:  // or public: ?
      
      /**
       * @class placeholder BoostAny.h Reflection/BoostAny.h
       * @author K. Henney
       */
      class placeholder {
      public: 
        
        /** Destructor */
        virtual ~placeholder() {}
        
        /** Query */
        virtual const std::type_info & type() const = 0;
        
        /** Query */
        virtual placeholder * clone() const = 0;
        
      };
      
      /**
       * @class holder BoostAny.h Reflection/BoostAny.h
       * @author K. Henney
       */
      template< typename ValueType > class holder : public placeholder {
      public: 
        
        /** Constructor */
        holder( const ValueType & value )
          : held( value ) {}
        
        /** Query */
        virtual const std::type_info & type() const {
          return typeid( ValueType );
        }
          
        /** Clone */
        virtual placeholder * clone() const {
          return new holder( held );
        }
        
        /** representation */
        ValueType held;
        
      };
      
      
      /** representation */
      template< typename ValueType > friend ValueType * any_cast( Any * );
      
      // or  public:  
      
      /** representation */
      placeholder * content;
      
   };
    
    
   /**
    * @class bad_any_cast BoostAny.h Reflection/BoostAny.h
    * @author K. Henney
    */
   class bad_any_cast : public std::bad_cast
   {
   public:
      
      /** Query */
      virtual const char * what() const throw()
      {
        return "boost::bad_any_cast: failed conversion using boost::any_cast";
      }
   };
    
   /** throw */
   template < class E > void throw_exception( const E & e )
   {
     throw e;
   }
    
   /** value */
   template< typename ValueType > ValueType * any_cast( Any * operand )
   {
      return operand && operand->type() == typeid( ValueType ) 
        ? & static_cast< Any::holder< ValueType > * >( operand->content )->held : 0;
   }
    
    /** value */
   template< typename ValueType > const ValueType * any_cast( const Any * operand ) {
      return any_cast< ValueType >( const_cast< Any * >( operand ));
    }
    
    /** value */
    template< typename ValueType > ValueType any_cast( const Any & operand ) {
      const ValueType * result = any_cast< ValueType >( & operand );
      if ( ! result ) { throw_exception( bad_any_cast()); }
      return * result;
    }

} // namespace MLANG_Reflection

#endif // MLANG_REFLECTION_BOOSTANY
