//$Id: Array.h,v 1.5 2004/11/30 22:57:58 roiser Exp $

#ifndef MLANG_REFLECTION_ARRAY_H
#define MLANG_REFLECTION_ARRAY_H

// Include files
#include "MLANG/Reflection/TypeBase.h"
#include "MLANG/Reflection/Type.h"

namespace MLANG_Reflection
{

    // forward declarations

    /**
     * @class Array Array.h MLANG/Reflection/Array.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class Array : public TypeBase {

    public:

      /** default constructor */
      Array( const Type & arrayType,
             size_t length,
             const std::type_info & typeinfo );


      /** destructor */
      virtual ~Array() {}


      /**
       * fullName will return the string representation of the array type
       * @param  typedefexp expand typedefs or not
       * @return string representation of type
       */
      MLANG_String fullName( bool typedefexp = false ) const;


      /**
       * size returns the size of the array
       * @return size of array
       */
      size_t length() const;


      /**
       * arrayType will return a pointer to the type of the array.
       * @return pointer to Type of member et. al.
       */
      Type toType() const;


      /** static funtion that composes the type name */
      static MLANG_String buildTypeName( const Type & type, 
                                        size_t length );

    private:

      /**
       * type of the array
       * @label array type
       * @link aggregation
       * @supplierCardinality 1
       */
      Type m_arrayType;


      /** the length of the array */
      size_t m_length;

    }; // class Array
} //namespace MLANG_Reflection


//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::Array::length() const { 
//-------------------------------------------------------------------------------
  return m_length; 
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type MLANG_Reflection::Array::toType() const {
//-------------------------------------------------------------------------------
  return m_arrayType;
}

#endif // MLANG_REFLECTION_ARRAY_H
