// $Id: Base.h,v 1.7 2004/11/30 22:57:58 roiser Exp $
#ifndef MLANG_REFLECTION_BASE_H
#define MLANG_REFLECTION_BASE_H 1

// Include files
#include "MLANG/Reflection/Kernel.h"
#include "MLANG/Reflection/Type.h"

namespace MLANG_Reflection
{

    // forward declarations

    /**
     * @class Base Base.h MLANG/Reflection/Base.h
     * @author Stefan Roiser
     * @date   2004-01-28
     * @ingroup Ref
     */
    class Base {

    public:

      /** default constructor */
      Base();


      /** constructor */
      Base( const Type &   baseType,
            OffsetFunction offsetFP,
            unsigned int   modifiers = 0);


      /** destructor */
      virtual ~Base() {}


      /**
       * fullName will return the string represenation of the base class
       * @param  typedefexp expand typedefs or not
       * @return string represenation of base class
       */
      MLANG_String fullName( bool typedefexp = false ) const;


      /**
       * isPrivate will return true if the inheritance is private
       * @return true if inheritance is private
       */
      bool isPrivate() const;


      /**
       * isProtected will return true if the inheritance is protected
       * @return true if inheritance is protected
       */
      bool isProtected() const;


      /**
       * isPublic will return true if the inheritance is public
       * @return true if inheritance is public
       */
      bool isPublic() const;


      /**
       * isVirtual will return true if the inheritance is virtual
       * @return true if inheritance is virtual
       */
      bool isVirtual() const;


      /**
       * offset will return the offset to the base class as int
       * @return offset to base class
       */
      size_t offset(void * mem = 0) const;


      /** 
       * offsetFP will return a pointer to the function which calculates the offset
       * between the two classes
       * @return pointer to offset calculating function
       */
      OffsetFunction offsetFP() const;


      /**
       * baseType will return a pointer to the type of the base class
       * @return pointer to type of base class
       */
      Type toType() const;

    private:

      /** function pointer to stubfunction for offset calculation */
      OffsetFunction m_offsetFP;


      /** modifiers of inheritance relation */
      unsigned int m_modifiers;


      /**
       * pointer to base class
       * @label base bype
       * @link aggregation
       * @supplierCardinality 1
       */
      Type m_baseType;

    }; // class Base
} // namespace MLANG_Reflection

#include "MLANG/Reflection/Type.h"


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Base::Base() 
//-------------------------------------------------------------------------------
  : m_offsetFP( 0 ),
    m_modifiers( 0 ),
    m_baseType( 0 ) {}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type MLANG_Reflection::Base::toType() const {
//-------------------------------------------------------------------------------
  return m_baseType;
}


//-------------------------------------------------------------------------------
inline bool MLANG_Reflection::Base::isPrivate() const {
//-------------------------------------------------------------------------------
  return 0 != (m_modifiers & PRIVATE);
}


//-------------------------------------------------------------------------------
inline bool MLANG_Reflection::Base::isProtected() const {
//-------------------------------------------------------------------------------
  return 0 != (m_modifiers & PROTECTED);
}


//-------------------------------------------------------------------------------
inline bool MLANG_Reflection::Base::isPublic() const {
//-------------------------------------------------------------------------------
  return 0 != (m_modifiers & PUBLIC);
}


//-------------------------------------------------------------------------------
inline bool MLANG_Reflection::Base::isVirtual() const {
//-------------------------------------------------------------------------------
  return 0 != (m_modifiers & VIRTUAL);
}


//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::Base::offset(void * mem) const {
//-------------------------------------------------------------------------------
  return m_offsetFP( mem );
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::OffsetFunction MLANG_Reflection::Base::offsetFP() const {
//-------------------------------------------------------------------------------
  return m_offsetFP;
}

#endif // MLANG_REFLECTION_BASE_H
