//$Id: ClassBuilder.cpp,v 1.7 2004/12/01 14:49:54 mato Exp $

#include "MLANG/Reflection/Type.h"
#include "MLANG/Reflection/Class.h"
#include "MLANG/Reflection/Tools.h"
#include "MLANG/Reflection/ClassBuilder.h"



//-------------------------------------------------------------------------------
MLANG_Reflection::ClassBuilderImpl::ClassBuilderImpl( const char * name, 
                                                  const std::type_info & ti, 
                                                  size_t size, 
                                                  unsigned int modifiers )
//-------------------------------------------------------------------------------
  : m_class(0), m_lastMember( 0 )
{
  Type c = Type::byName(name);
  if ( c ) { 
    // Class already exists. Check if it was a class.
    if (! c.isClass() ) throw RuntimeError("Attempt to replace a non-Class type with a Class type"); 
  }
  else {
    m_class = new Class( name, 
                         size, 
                         ti, 
                         modifiers);
  }
}

    
//-------------------------------------------------------------------------------
MLANG_Reflection::ClassBuilderImpl::~ClassBuilderImpl() {
//-------------------------------------------------------------------------------
  fireClassCallback( m_class->type() );
}
    
//-------------------------------------------------------------------------------
void MLANG_Reflection::ClassBuilderImpl::addBase( const Type & base,
                                              OffsetFunction offsetFP,
                                              unsigned int modifiers ) {
//-------------------------------------------------------------------------------
  m_class->addBase( base, offsetFP, modifiers );
}
    
    
//-------------------------------------------------------------------------------
void MLANG_Reflection::ClassBuilderImpl::addDataMember( const char * name,
                                                    const Type & type,
                                                    size_t offset,
                                                    unsigned int modifiers ) {
//-------------------------------------------------------------------------------
  Member dm = Member(new DataMember( name, type, offset, modifiers ));
  m_class->addDataMember( dm );
  m_lastMember = dm;
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ClassBuilderImpl::addFunctionMember( const char * name,
                                                        const Type & type,
                                                        StubFunction stubFP,
                                                        void*        stubCtx,
                                                        const char * params,
                                                        unsigned int modifiers ) {
//-------------------------------------------------------------------------------
  Member fm = Member(new FunctionMember(name, type, stubFP, stubCtx, params, modifiers));
  m_class->addFunctionMember( fm );
  m_lastMember = fm;
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ClassBuilderImpl::addProperty( const char * key, 
                                                  const char * value ) {
//-------------------------------------------------------------------------------
  addProperty( key, Any(value) );
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ClassBuilderImpl::addProperty( const char * key, 
                                                  Any value ) {
//-------------------------------------------------------------------------------
  if ( m_lastMember ) m_lastMember.propertyList().addProperty( key, value );
  else                m_class->propertyList().addProperty(key, value); 
}

