//$Id: Const.h,v 1.6 2004/10/07 10:03:42 roiser Exp $
#ifndef MLANG_REFLECTION_CONST_H
#define MLANG_REFLECTION_CONST_H

// Include files
#include "MLANG/Reflection/TypeBase.h"

namespace MLANG_Reflection

{


    /**
     * @class Const Const.h Reflection/Const.h
     * @author Stefan Roiser
     * @date 15/06/2004
     */
    class Const : public TypeBase {
    public:

      /** Constructor */
      Const( const Type * constType, const std::type_info & ti );


      /** Destructor */
      virtual ~Const() {}


      /**
       * type will return the pointer to the type the cv is pointing to
       * @return pointer to Type
       */
      const Type * toType() const;

      /** static funtion that composes the type name */
      static MLANG_String buildTypeName(const Type* constType );

    private:
    
      /** 
       * the pointer to the corresponding type 
       * @label const type
       * @link aggregation
       * @supplierCardinality 1
       */
      const Type * m_constType;

    }; // class Const
} //namespace MLANG_Reflection

//-------------------------------------------------------------------------------
inline MLANG_Reflection::Const::Const( const Type * constType, const std::type_info & ti  )
//-------------------------------------------------------------------------------
: TypeBase( buildTypeName(constType).c_str(), constType->sizeOf(), CONST, ti ), 
    m_constType( constType ) {}

//-------------------------------------------------------------------------------
inline const MLANG_Reflection::Type * MLANG_Reflection::Const::toType() const {
//-------------------------------------------------------------------------------
  return m_constType;
}

//-------------------------------------------------------------------------------
inline MLANG_String MLANG_Reflection::Const::buildTypeName( const Type* constType )
//-------------------------------------------------------------------------------
{
  if ( constType->isReference() || constType->isPointer() ) {
    return constType->name() + " const";
  }
  else {
    return MLANG_String("const ") + constType->name();
  }
}
#endif // MLANG_REFLECTION_CONST_H
