//$Id: Enum.h,v 1.5 2004/11/30 22:57:58 roiser Exp $
#ifndef MLANG_REFLECTION_ENUM_H
#define MLANG_REFLECTION_ENUM_H

// Include files
#include "MLANG/Reflection/TypeBase.h"

namespace MLANG_Reflection
{

    // forward declarations
    class EnumInfo;

    /**
     * @class Enum Enum.h MLANG/Reflection/Enum.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class Enum : public TypeBase {

    public:

      /** default constructor */
      Enum( const char * type,
            size_t size,
            const std::type_info & typeInfo );


      /** destructor */
      virtual ~Enum();


      /** addItem will add a new item into enum
       * @param name name of the item 
       * @param value numerical value 
       */
      void addInfo( const char * name, 
                    long value );


      /**
       * enumItem will return the nth enum information
       * @param  nth enum
       * @return nth enum info object
       */
      EnumInfo enumInfo( size_t nth ) const;


      /**
       * enumItemCount will return the number of enums
       * @return number of enums
       */
      size_t enumInfoCount() const;

    private:

      /** enum values */
      std::vector < EnumInfo * > m_enums;

    }; // class Enum

} //namespace MLANG_Reflection

#include "MLANG/Reflection/EnumInfo.h"


//-------------------------------------------------------------------------------
inline void MLANG_Reflection::Enum::addInfo( const char * name , 
                                         long value ) {
//-------------------------------------------------------------------------------
  m_enums.push_back( new EnumInfo( name, value ));
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::EnumInfo MLANG_Reflection::Enum::enumInfo( size_t nth ) const {
//-------------------------------------------------------------------------------
  if ( nth < m_enums.size() ) { return *(m_enums[ nth ]); }
  return EnumInfo();
}


//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::Enum::enumInfoCount() const {
//-------------------------------------------------------------------------------
  return m_enums.size();
}

#endif // MLANG_REFLECTION_ENUM_H

