//$Id: EnumBuilder.h,v 1.3 2004/11/30 22:57:58 roiser Exp $

#ifndef SEAL_REFLECT_ENUMBUILDER_H
#define SEAL_REFLECT_ENUMBUILDER_H

// Include files
#include "MLANG/Reflection/TypeBuilder.h"
#include "MLANG/Reflection/Enum.h"


namespace MLANG_Reflection
{

    /** 
     * @class EnumBuilder EnumBuilder.h MLANG/Reflection/EnumBuilder.h
     * @author Stefan Roiser
     * @ingroup RefBld
     * @date 30/3/2004
     */
    template < typename T >
    class EnumBuilder  {

    public:            

      /** constructor */
      EnumBuilder();

      /** constructor */
      EnumBuilder( const char * name );


			/** destructor */
      virtual ~EnumBuilder() {}

      /** addItem add a new item in the enum
       * @param  name item name
       * @param  value the value of the item
       * @return a reference to the building class
       */
      EnumBuilder & addItem( const char * name, long value );


      /** addProperty will add a property 
       * @param  key the property key
       * @param  value the value of the property
       * @return a reference to the building class
       */
      EnumBuilder & addProperty( const char * key, Any value );
      EnumBuilder & addProperty( const char * key, const char * value );

    private:

      /** the enums and values */
      Enum * m_enum;

    }; // class EnumBuilder

} // namespace MLANG_Reflection

//-------------------------------------------------------------------------------
template < typename T >
inline MLANG_Reflection::EnumBuilder<T>::EnumBuilder() {
//-------------------------------------------------------------------------------
  m_enum = new Enum( Tools::demangle( typeid(T) ).c_str(), sizeof(T), typeid(T) );
}


//-------------------------------------------------------------------------------
template < typename T >
inline MLANG_Reflection::EnumBuilder<T>::EnumBuilder( const char * name ) {
//-------------------------------------------------------------------------------
  m_enum = new Enum( name, sizeof(long), typeid(UnknownType) );
}


//-------------------------------------------------------------------------------
template < typename T >
inline MLANG_Reflection::EnumBuilder<T> & 
MLANG_Reflection::EnumBuilder<T>::addItem( const char * name, long value ) {
//-------------------------------------------------------------------------------
  m_enum->addInfo( name , value );
  return *this;
}


//-------------------------------------------------------------------------------
template < typename T >
inline MLANG_Reflection::EnumBuilder<T> & 
MLANG_Reflection::EnumBuilder<T>::addProperty( const char * key, const char * value ) {
//-------------------------------------------------------------------------------
  m_enum->propertyList().addProperty( key , value );
  return *this;
}


//-------------------------------------------------------------------------------
template < typename T >
inline MLANG_Reflection::EnumBuilder<T> & 
MLANG_Reflection::EnumBuilder<T>::addProperty( const char * key, Any value ) {
//-------------------------------------------------------------------------------
  m_enum->propertyList().addProperty( key , value );
  return *this;
}


#endif //SEAL_REFLECT_ENUMBUILDER_H
