//$Id: Function.h,v 1.9 2004/11/30 22:57:58 roiser Exp $
#ifndef MLANG_REFLECTION_FUNCTION_H
#define MLANG_REFLECTION_FUNCTION_H

// Include files
#include "MLANG/Reflection/TypeBase.h"
#include "MLANG/Reflection/Type.h"

namespace MLANG_Reflection
{

    // forward declarations


    /** 
     * @class Function Function.h MLANG/Reflection/Function.h
     * @author Stefan Roiser
     * @date 24/11/2003 
     * @ingroup Ref
     */
    class Function : public TypeBase {

    public:


      /** default constructor */
      Function( const Type & returnType,
                std::vector< Type > parameters,
                const std::type_info & typeInfo,
                TYPE functionType = FUNCTION );


      /** destructor */
      virtual ~Function() {}


      /**
       * fullName will return the full name of the function containing return
       * type, parameters and modifiers
       * @param  typedefexp expand typedefs or not
       * @return full name of function
       */
      MLANG_String fullName(bool typedefexp = false) const;


      /**
       * isTemplated will return true if the the function is templated
       * @return true if the function is templated
       */
      bool isTemplated() const;


      /**
       * parameter returns the nth parameter
       * @param  nth nth parameter
       * @return pointer to nth parameter type
       */
      Type parameter( size_t nth ) const;


      /**
       * parameterCount will return the number of parameters of this function
       * @return number of parameters
       */
      size_t parameterCount() const;


      /**
       * returnType will return a pointer to the type of the return type.
       * @return pointer to Type of return type
       */
      Type returnType() const;


      /** static funtion that composes the type name */
      static MLANG_String buildTypeName( const Type & ret, 
                                        const std::vector< Type > & param);

    private:

      /** 
       * vector of parameter types 
       * @label parameter types
       * @link aggregation
       */
      std::vector < Type > m_parameters;


      /**
       * characteristics of return type
       * @label return type
       * @link aggregation
       * @supplierCardinality 1
       */
      Type m_returnType;


      /** modifiers of function and return type */
      unsigned int m_modifiers;

    }; // class Function
} // namespace MLANG_Reflection


//-------------------------------------------------------------------------------
inline bool MLANG_Reflection::Function::isTemplated() const {
//-------------------------------------------------------------------------------
  return 0 != (m_modifiers & TEMPLATED);
}



//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::Function::parameterCount() const {
//-------------------------------------------------------------------------------
  return m_parameters.size();
}



//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type 
MLANG_Reflection::Function::parameter( size_t nth ) const {
//------------------------------------------------------------------------------- 
  if (nth < m_parameters.size()) { return m_parameters[nth]; }
  return Type();
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type MLANG_Reflection::Function::returnType() const {
//-------------------------------------------------------------------------------
  return m_returnType;
}


#endif // MLANG_REFLECTION_FUNCTION_H
