//$Id: FunctionBuilder.cpp,v 1.3 2004/10/12 12:56:38 roiser Exp $

#include "Reflection/Builder/FunctionBuilder.h"
#include "Reflection/FunctionMember.h"
#include "Reflection/PropertyList.h"
#include "Reflection/Scope.h"
#include "Reflection/Namespace.h"
#include "Reflection/Tools.h"


//-------------------------------------------------------------------------------
MLANG_Reflection::FunctionBuilderImpl::FunctionBuilderImpl( const char * name, 
                                          const Type * type,
                                          StubFunction stubFP,
                                          void * stubCtx,
                                          const char * params, 
                                          unsigned char modifiers)
//-------------------------------------------------------------------------------
  : m_function(0) {

  MLANG_String fullname(name);
  MLANG_String declScope;
  MLANG_String funcName;
  size_t pos = fullname.rfind( "::" );
  // Name contains declaring scope
  if ( pos != MLANG_String::npos ) {   
    funcName  = fullname.substr( pos + 2 );
    declScope = fullname.substr( 0, pos ); 
  }
  else {
    funcName  = name;
    declScope = "";
  }
  const Scope* scope = Scope::byName(declScope);
  if ( ! scope || scope->isUnresolved() ) {
    // Let's create the namespace here
    scope = (new Namespace(declScope.c_str()))->scope();
  }
  if ( ! scope->isNamespace() ) throw RuntimeError("Declaring scope is not a namespace");
  m_function = new FunctionMember(funcName.c_str(), type, stubFP, stubCtx, 
                                  params, modifiers  | STATIC);
  const_cast<Namespace*>(scope->asNamespace())->addFunctionMember(m_function);
}

//-------------------------------------------------------------------------------
void MLANG_Reflection::FunctionBuilderImpl::addProperty( const char * key, const char * value )
//-------------------------------------------------------------------------------
{
   m_function->propertyList()->addProperty( key , value );
}

//-------------------------------------------------------------------------------
void MLANG_Reflection::FunctionBuilderImpl::addProperty( const char * key, Any value )
//-------------------------------------------------------------------------------
{
  m_function->propertyList()->addProperty( key , value );
}


//-------------------------------------------------------------------------------
MLANG_Reflection::FunctionBuilder::FunctionBuilder( const Type * type,
                                                const char * name, 
                                                StubFunction stubFP,
                                                void * stubCtx,
                                                const char * params, 
                                                unsigned char modifiers)
//-------------------------------------------------------------------------------
  : m_function(0) {

  MLANG_String fullname(name);
  MLANG_String declScope;
  MLANG_String funcName;
  size_t pos = fullname.rfind( "::" );
  // Name contains declaring scope
  if ( pos != MLANG_String::npos ) {   
    funcName  = fullname.substr( pos + 2 );
    declScope = fullname.substr( 0, pos ); 
  }
  else {
    funcName  = name;
    declScope = "";
  }
  const Scope* scope = Scope::byName(declScope);
  if ( ! scope || scope->isUnresolved() ) {
    // Let's create the namespace here
    scope = (new Namespace(declScope.c_str()))->scope();
  }
  if ( ! scope->isNamespace() ) throw RuntimeError("2Declaring scope is not a namespace");
  m_function = new FunctionMember(funcName.c_str(), type, stubFP, stubCtx, 
                                  params, modifiers  | STATIC);
  const_cast<Namespace*>(scope->asNamespace())->addFunctionMember(m_function);
}

//-------------------------------------------------------------------------------
MLANG_Reflection::FunctionBuilder & 
MLANG_Reflection::FunctionBuilder::addProperty( const char * key, const char * value )
//-------------------------------------------------------------------------------
{
   m_function->propertyList()->addProperty( key , value );
   return * this;
}

//-------------------------------------------------------------------------------
MLANG_Reflection::FunctionBuilder & 
MLANG_Reflection::FunctionBuilder::addProperty( const char * key, Any value )
//-------------------------------------------------------------------------------
{
  m_function->propertyList()->addProperty( key , value );
  return * this;
}

