//$Id: FunctionBuilder.h,v 1.6 2004/11/30 22:57:58 roiser Exp $

#ifndef SEAL_REFLECT_FUNCTIONBUILDER_H
#define SEAL_REFLECT_FUNCTIONBUILDER_H

// Include files
#include "MLANG/Reflection/Reflection.h"
#include "MLANG/Reflection/TypeBuilder.h"

namespace MLANG_Reflection
{
  
    // forward declarations
    class FunctionMember;
    class Type;

    /** 
     * @class FunctionBuilder FunctionBuilder.h MLANG/Reflection/FunctionBuilder.h
     * @author Pere Mato
     * @date 1/8/2004
     * @ingroup RefBld
     */
    class FunctionBuilder {

    public:    

      /** constructor */
      FunctionBuilder( const Type & type,
                       const char * name,
                       StubFunction stubFP,
                       void * stubCtx,
                       const char * params, 
                       unsigned char modifiers );
      

      /** destructor */
      virtual ~FunctionBuilder() {}


     /** addProperty will add a property 
       * @param  key the property key
       * @param  value the value of the property
       * @return a reference to the building class
       */
      FunctionBuilder & addProperty( const char * key, 
                                     Any value );
      FunctionBuilder & addProperty( const char * key,
                                     const char * value );

    private:

      /** function member */
      Member m_function;

    }; //class FunctionBuilder
    

    /** 
     * @class FunctionBuilderImpl FunctionBuilder.h MLANG/Reflection/FunctionBuilder.h
     * @author Pere Mato
     * @date 3/8/2004
     * @ingroup RefBld
     */
    class FunctionBuilderImpl {
    
    public:
      
      /** constructor */
      FunctionBuilderImpl( const char * name, 
                           const Type & type,
                           StubFunction stubFP,
                           void * stubCtx,
                           const char * params, 
                           unsigned char modifiers = 0 );
                          

      /** destructor */
      ~FunctionBuilderImpl() {}


      /** addProperty will add a property 
       * @param  key the property key
       * @param  value the value of the property
       * @return a reference to the building class
       */
      void addProperty( const char * key, 
                        Any value );
      void addProperty( const char * key, 
                        const char * value );

      /** string containing the union information */
      Member m_function;

    };


    /** 
     * @class FunctionBuilder FunctionBuilder.h MLANG/Reflection/FunctionBuilder.h
     * @author Pere Mato
     * @date 1/8/2004
     * @ingroup RefBld
     */
    template < typename F > class FunctionBuilderT {

    public:    

      /** constructor */
      FunctionBuilderT( const char * name,
                        StubFunction stubFP,
                        void * stubCtx,
                        const char * params, 
                        unsigned char modifiers );
      
      /** destructor */
      virtual ~FunctionBuilderT() {}

      
      /** addProperty will add a property 
       * @param  key the property key
       * @param  value the value of the property
       * @return a reference to the building class
       */
      template < typename P >
      FunctionBuilderT & addProperty( const char * key, P value );


    private:

      /** function builder implemenation */
      FunctionBuilderImpl m_functionBuilderImpl;

    }; //class FunctionBuilder

} // namespace MLANG_Reflection


//-------------------------------------------------------------------------------
template < typename  F > 
MLANG_Reflection::FunctionBuilderT<F>::FunctionBuilderT( const char * name, 
                                                     StubFunction stubFP,
                                                     void * stubCtx,
                                                     const char * params, 
                                                     unsigned char modifiers )
//-------------------------------------------------------------------------------
  : m_functionBuilderImpl( name,
                           FunctionDistiller<F>::get(),
                           stubFP,
                           stubCtx,
                           params,
                           modifiers ) { }
      

//-------------------------------------------------------------------------------
template <  typename F > template < typename P >
inline MLANG_Reflection::FunctionBuilderT<F> & 
MLANG_Reflection::FunctionBuilderT<F>::addProperty( const char * key, 
                                                P value )
//-------------------------------------------------------------------------------
{ 
  m_functionBuilderImpl.addProperty(key , value);
  return * this;
}
 


#endif //SEAL_REFLECT_UNIONBUILDER_H
