//$Id: FunctionMember.cpp,v 1.7 2004/11/30 22:57:58 roiser Exp $

#include "MLANG/Reflection/FunctionMember.h"
#include "MLANG/Reflection/Scope.h"
#include "MLANG/Reflection/Function.h"
#include "MLANG/Reflection/Object.h"
#include "MLANG/Reflection/Tools.h"
#include "MLANG/Reflection/Member.h"

//-------------------------------------------------------------------------------
MLANG_Reflection::FunctionMember::FunctionMember( const char *  name,
                                              const Type &  type,
                                              StubFunction  stubFP,
                                              void*         stubCtx,
                                              const char *  parameters,
                                              unsigned int  modifiers )
//-------------------------------------------------------------------------------
  : MemberBase( name, type, FUNCTIONMEMBER, modifiers ),
    m_stubFP( stubFP ), m_stubCtx(stubCtx) 
{
  // Obtain the names and default values of the function parameters
  // The "real" number of parameters is obtained from the function type
  size_t type_npar = type.parameterCount();
  std::vector<MLANG_String> params;
  if ( parameters ) Tools::stringSplit(params, parameters, ";");
  size_t npar = std::min(type_npar,params.size());
  for ( size_t i = 0; i < npar ; ++i ) {
    size_t pos = params[i].indexOf( "=" );
    m_parameterNames.push_back(params[i].substring(0,pos));
    if ( (int)pos != MLANG_String::npos ) m_parameterDefaults.push_back(params[i].substring(pos+1));
    else                            m_parameterDefaults.push_back("");
  }
  // padding with blanks
  for ( size_t i = npar; i < type_npar; ++i ) {
    m_parameterNames.push_back("");
    m_parameterDefaults.push_back("");
  }
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::FunctionMember::fullName( bool typedefexp ) const {
//-------------------------------------------------------------------------------
  MLANG_String s = "";
  if ( isPublic())          { s += "public ";    }
  if ( isProtected())       { s += "protected "; }
  if ( isPrivate())         { s += "private ";   }  
  if ( isExtern())          { s += "extern ";    }
  if ( isStatic())          { s += "static ";    }
  if ( isInline())          { s += "inline ";    }
  if ( isVirtual())         { s += "virtual ";   }
  if ( isExplicit())        { s += "explicit ";  }
  Type t = type();
  if ( t ) {
    if ( ! ( isConstructor() || isDestructor())) {
      s += t.returnType().fullName( typedefexp ) + " ";
    }
    s += scope().fullName();
    if ( ! scope().isTopScope()) { s += "::"; }
    s += name() + "(";
    for ( size_t i = 0; i < t.parameterCount(); ++i ) {
      s += t.parameter( i ).fullName( typedefexp );
      if ( parameterName( i ) != "" ) s += " " + parameterName( i );
      if ( parameterDefault( i ) != "" ) s += "=" + parameterDefault( i );
      if ( i < ( t.parameterCount() - 1 ) ) { s += ", "; }
    }
    s += ")";
  }
  return s;
}


//-------------------------------------------------------------------------------
void * 
MLANG_Reflection::FunctionMember::invoke( const Object & obj,
                                      const std::vector < void * > & paramList ) const {
//-------------------------------------------------------------------------------
  void * mem = calculateBaseObject( obj );
  return m_stubFP( mem, paramList, m_stubCtx );
}


//-------------------------------------------------------------------------------
void * 
MLANG_Reflection::FunctionMember::invoke( const std::vector < void * > & paramList ) const {
//-------------------------------------------------------------------------------
  return m_stubFP( 0, paramList, m_stubCtx );
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::FunctionMember::parameterCount() const {
//-------------------------------------------------------------------------------
  return type().parameterCount();
}
