//$Id: FunctionMember.h,v 1.11 2004/11/30 22:57:58 roiser Exp $

#ifndef MLANG_REFLECTION_FUNCTIONMEMBER_H
#define MLANG_REFLECTION_FUNCTIONMEMBER_H

// Include files
#include "MLANG/Reflection/MemberBase.h"

namespace MLANG_Reflection
{

    // forward declarations
    class Type;
    class Object;

    /**
     * @class FunctionMember FunctionMember.h MLANG/Reflection/FunctionMember.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class FunctionMember : public MemberBase {

    public:

      /** default constructor */
      FunctionMember( const char *   name,
                      const Type &   type,
                      StubFunction   stubFP,
                      void *         stubCtx = 0,
                      const char *   params = 0,
                      unsigned int   modifiers = 0 );


      /** destructor */
      virtual ~FunctionMember() {}


      /** return full name of function member */
      MLANG_String fullName( bool typedefexp = false ) const;


      /** invoke the function (if return type as void*) */
      void * invoke( const Object & obj, 
                     const std::vector < void * > & paramList = 
                     std::vector < void * >()) const;


      /** invoke the function (for static functions) */
      void * invoke( const std::vector < void * > & paramList = 
                     std::vector < void * >()) const;


      /** number of parameters */
      size_t parameterCount() const;


      /** parameter nth default value if declared*/
      MLANG_String parameterDefault( size_t nth ) const;


      /** parameter nth name if declared*/
      MLANG_String parameterName( size_t nth ) const;


      /** return the pointer to the stub function */
      StubFunction stubfunction() const;

    private:

      /** pointer to the stub function */
      StubFunction m_stubFP;


      /** user data for the stub function */
      void*  m_stubCtx;


      /** parameter names */
      std::vector < MLANG_String > m_parameterNames;


      /** parameter names */
      std::vector < MLANG_String > m_parameterDefaults;
     
    }; // class FunctionMember
} //namespace MLANG_Reflection


//-------------------------------------------------------------------------------
inline MLANG_String 
MLANG_Reflection::FunctionMember::parameterDefault( size_t nth ) const {
//-------------------------------------------------------------------------------
  return m_parameterDefaults[nth];
}


//-------------------------------------------------------------------------------
inline MLANG_String 
MLANG_Reflection::FunctionMember::parameterName( size_t nth ) const {
//-------------------------------------------------------------------------------
  return m_parameterNames[nth];
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::StubFunction 
MLANG_Reflection::FunctionMember::stubfunction() const {
//-------------------------------------------------------------------------------
  return m_stubFP;
}

#endif // MLANG_REFLECTION_FUNCTIONMEMBER_H
