// $Id: InstantiatedTemplate.h,v 1.4 2004/11/30 22:57:58 roiser Exp $

#ifndef SEAL_REFLECT_INSTANTIATEDTEMPLATE_H
#define SEAL_REFLECT_INSTANTIATEDTEMPLATE_H

// Include files
#include "MLANG/Reflection/Kernel.h"
#include "MLANG/Reflection/Type.h"

namespace MLANG_Reflection
{

    // forward declarations
    class TemplateFamily;
    class InstantiatedTemplateClass;
    class InstantiatedTemplateFunction;


    /**
     * @class InstantiatedTemplate InstantiatedTemplate.h MLANG/Reflection/InstantiatedTemplate.h
     * @author Stefan Roiser
     * @date   2004-01-28
     * @ingroup Ref
     */
    class InstantiatedTemplate {

    public:

      class TemplateArgument {                           // contains information about a template argument
      public:
        TemplateArgument( const Type & type = Type());   // constructor
        MLANG_String fullName( bool typedefexp = false ) const; // return full name of template arg
        Type toType() const;                             // return template arg type
      private:
        Type  m_templateArgumentType;                    // the template argument type
      };                                                 // class TemplateArgument


      /** default constructor */
      InstantiatedTemplate();


      /** constructor */
      InstantiatedTemplate( const TemplateFamily * templateFamily,
                            TYPE instantiatedTemplateType,
                            const std::vector < TemplateArgument > & templateArguments );


      /** destructor */
      virtual ~InstantiatedTemplate() { }


      /**
       * fullName returns the full name of the templated collection
       * @param  typedefexp expand typedefs or not
       * @return full name of template collection
       */
      MLANG_String fullName( bool typedefexp = false ) const;


      /**
       * templateArgument will return a pointer to the nth template argument
       * @param  nth nth template argument
       * @return pointer to nth template argument
       */
      TemplateArgument templateArgument( size_t nth ) const;


      /**
       * templateArgCount will return the number of template arguments
       * @return number of template arguments
       */
      size_t templateArgumentCount() const;


      /**
       * instantiatedTemplatedType will return the actual type of the templated
       * @return type of templated type
       */
      TYPE instantiatedTemplateType() const;


      /**
       * instantiatedTemplateTypeAsString will return the string representation 
       * of the INSTANTIATEDTEMPLATE type
       * @return string representation of INSTANTIATEDTEMPLATE type
       */
      MLANG_String instantiatedTemplateTypeAsString() const;


      /**
       * templateFamily will return a pointer to the template family the
       * templated belongs to
       * @return pointer to templated family
       */
      TemplateFamily & templateFamily() const;

    private:

      /**
       * a pointer to the corresponding template family
       * @label template family
       * @link aggregation
       * @supplierCardinality 1
       */
      const TemplateFamily * m_templateFamily;


      /** 
       * type of templated 
       * @label instantiated templateType
       */
      TYPE m_instantiatedTemplateType;


      /** vector of template arguments */
      std::vector < TemplateArgument > m_templateArguments;

    }; // class Templated
} // namespace MLANG_Reflection

#include "MLANG/Reflection/TemplateFamily.h"

//-------------------------------------------------------------------------------
inline MLANG_Reflection::InstantiatedTemplate::InstantiatedTemplate() 
//-------------------------------------------------------------------------------
  : m_templateFamily( 0 ),
    m_instantiatedTemplateType( UNRESOLVED ),
    m_templateArguments( std::vector<TemplateArgument>()) {}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::TYPE 
MLANG_Reflection::InstantiatedTemplate::instantiatedTemplateType() const {
//-------------------------------------------------------------------------------
  return m_instantiatedTemplateType;
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::InstantiatedTemplate::TemplateArgument 
MLANG_Reflection::InstantiatedTemplate::templateArgument( size_t nth ) const {
//-------------------------------------------------------------------------------
  if ( nth < m_templateArguments.size() ) { return m_templateArguments[ nth ]; }
  return TemplateArgument();
}


//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::InstantiatedTemplate::templateArgumentCount() const {
//-------------------------------------------------------------------------------
  return m_templateArguments.size();
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::TemplateFamily &
MLANG_Reflection::InstantiatedTemplate::templateFamily() const {
//-------------------------------------------------------------------------------
  return * const_cast<TemplateFamily*>(m_templateFamily);
}


//-------------------------------------------------------------------------------
inline MLANG_String
MLANG_Reflection::InstantiatedTemplate::TemplateArgument::fullName( bool typedefexp ) const {
//-------------------------------------------------------------------------------
  return m_templateArgumentType.fullName( typedefexp );
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type
MLANG_Reflection::InstantiatedTemplate::TemplateArgument::toType() const {
//-------------------------------------------------------------------------------
  return m_templateArgumentType;
}


#endif // SEAL_REFLECT_INSTANTIATEDTEMPLATE_H



