//$Id: Kernel.h,v 1.7 2004/11/04 15:37:47 roiser Exp $

#ifndef MLANG_REFLECTION_KERNEL_H
#define MLANG_REFLECTION_KERNEL_H

// Some pragmas to avoid warnings in VisualC
#ifdef _WIN32
// Disable warning C4786: identifier was truncated to '255' characters in the debug information
#pragma warning ( disable : 4786 )
// Disable warning C4291: no matching operator delete found; memory will not be freed if 
// initialization throws an exception
#pragma warning ( disable : 4291 )
// Disable warning C4250: inheritance via dominance
#pragma warning ( disable : 4250 )
#endif

// Large integer definition depends of the platform
#ifdef _WIN32
  typedef __int64 longlong;
  typedef unsigned __int64 ulonglong;
#elif defined(__linux) || defined(sun) || defined(__APPLE__)
  typedef long long int longlong;
  typedef unsigned long long int ulonglong;
#endif
#ifndef LONGLONG_MAX
  #define LONGLONG_MAX 0x7FFFFFFFFFFFFFFFLL
#endif
#ifndef LONGLONG_MIN
  #define LONGLONG_MIN 0x8000000000000000LL
#endif
#ifndef ULONGLONG_MAX
  #define ULONGLONG_MAX 0xFFFFFFFFFFFFFFFFLL
#endif
#ifndef ULONGLONG_MIN
  #define ULONGLONG_MIN 0x0000000000000000LL
#endif

// Include files
#include "MLANG/String.h"
#include <vector>
#if !defined(WIN32)
#include <utility>
#endif /* !WIN32 */
#include <exception>

namespace MLANG_Reflection

{

    // these defines are used for the modifiers they are used in the following 
    // classes
    // BA = BASE
    // CL = CLASS
    // FU = FUNCTION
    // DM = DATAMEMBER
    // FM = FUNCTIONMEMBER 
    //                             BA  CL  FU  DM  FM
#define PUBLIC          (1<<0)  //  X           X   X  
#define TEMPLATED       (1<<0)  //      X   X          

#define PROTECTED       (1<<1)  //  X           X   X
#define STRUCT          (1<<1)  //      X

#define PRIVATE         (1<<2)  //  X           X   X
#define ABSTRACT        (1<<2)  //      X

#define REGISTER        (1<<3)  //              X   X
#define VIRTUAL         (1<<3)  //  X   X

#define STATIC          (1<<4)  //              X   X

#define AUTO            (1<<5)  //              X
#define CONSTRUCTOR     (1<<5)  //                  X

#define MUTABLE         (1<<6)  //              X
#define DESTRUCTOR      (1<<6)  //                  X

#define REF             (1<<7)  //              X
#define EXPLICIT        (1<<7)  //                  X 

#define EXTERN          (1<<8)  //              X   X

#define COPYCONSTRUCTOR (1<<9)  //                  X 

#define OPERATOR        (1<<10) //                  X 

#define INLINE          (1<<11) //                  X 

#define CONVERTER       (1<<12) //                  X 


    /** enum containing all possible types and scopes */
    enum TYPE { CLASS = 0,
                ENUM, 
                FUNCTION, 
                ARRAY, 
                FUNDAMENTAL, 
                POINTER, 
                POINTERTOMEMBER, 
                REFERENCE, 
                TYPEDEF, 
                UNION, 
                INSTANTIATEDTEMPLATECLASS, 
                INSTANTIATEDTEMPLATEFUNCTION, 
                NAMESPACE, 
                DATAMEMBER, 
                FUNCTIONMEMBER,
                MLANG_CONST,
                VOLATILE,
                CONSTVOLATILE,
                UNRESOLVED
    };

    /** all possible status codes of the package */
    enum STATUS { SUCCESS = 0,
                  FAILURE
    };
    
    /** the name of the status code -- don't use this to check */
    #define SRSTATUSNAME rstat
    /** the name of the status code to check for */
    #define SRSTATUS Reflex::SRSTATUSNAME
        

    /**
      * typedef for function member type (necessary for return value of
      * getter function)
      */
    typedef void * (* StubFunction) ( void *, const std::vector < void * > &, void *);

    /** typedef for function for offset calculation */
    typedef size_t( * OffsetFunction ) ( void * );

    /** dummy types for type_info purposes */
    struct NullType {};
    struct UnknownType {};
    
    /** exception classes */
    struct RuntimeError : public std::exception {
      RuntimeError(const MLANG_String& msg) : m_msg(msg) { }
      ~RuntimeError() throw() {}
      virtual const char * what() const throw( ) { return m_msg.c_str();}
      MLANG_String m_msg;
    };

    /** some general information about the Reflex package */
    struct Reflex {
      
      /** default constructor */
      Reflex();
      
      
      /** the name of the package - used for messages */
      static MLANG_String argv0; // "Seal Reflex";
      
    }; // struct Reflex

} // namespace MLANG_Reflection

#endif // MLANG_REFLECTIONION_KERNEL_H

