//$Id: MemberBase.cpp,v 1.2 2004/12/01 16:42:37 roiser Exp $

#include "MLANG/Reflection/Member.h"
#include "MLANG/Reflection/Scope.h"
#include "MLANG/Reflection/Type.h"
#include "MLANG/Reflection/Tools.h"
#include "MLANG/Reflection/Base.h"
#include "MLANG/Reflection/Class.h"
#include "MLANG/Reflection/Object.h"
#include "MLANG/Reflection/PropertyList.h"

#if !defined(WIN32)
#include <iostream>
#endif /* !WIN32 */

//-------------------------------------------------------------------------------
MLANG_Reflection::MemberBase::MemberBase( const char *  name,
                                      const Type &  type,
                                      TYPE          memberType,
                                      unsigned int  modifiers )
//-------------------------------------------------------------------------------
  : m_type( type, modifiers & ( MLANG_CONST | VOLATILE | REFERENCE )),
    m_modifiers( modifiers ),
    m_name( name ),
    m_scope( Scope() ),
    m_memberType( memberType ),
    m_propertyList( PropertyList() ) {}


//-------------------------------------------------------------------------------
MLANG_Reflection::MemberBase::~MemberBase() {
//-------------------------------------------------------------------------------
  m_propertyList.clearProperties();
}


//-------------------------------------------------------------------------------
void * MLANG_Reflection::MemberBase::calculateBaseObject( const Object & obj ) const {
//-------------------------------------------------------------------------------
  char * mem = (char*)obj.address();
  // check if its a dummy object 
  if ( ! obj.type() ) { return mem; }
  if ( obj.type().id() != ((Class)scope()).type().id()) {
    // now we know that the member is an inherited one
    Class cl = obj.type();
    if ( cl ) {
      std::vector < OffsetFunction > basePath = cl.pathToBase( scope());
      if ( basePath.size() ) {
        // there is a path described from the object to the class containing the member
        std::vector < OffsetFunction >::iterator pIter;
        for ( pIter = basePath.begin(); pIter != basePath.end(); ++pIter ) {
          mem += (*pIter)(mem);
        }
      }
      else
       {
#if !defined(WIN32)
         std::cerr << Reflex::argv0 << ": ERROR: There is no path available from class "
                   << cl.fullName() << " to " << fullName() << std::endl;
#else /* WIN32 */
         assert(0);
#endif /* WIN32 */
       }
    }
    else
    {
#if !defined(WIN32)
      std::cerr << Reflex::argv0 << ": ERROR: Object " << obj.type().fullName() 
                << " does not represent a class or is not a complete type " << std::endl;
#else /* WIN32 */
      assert(0);
#endif /* WIN32 */
    }
  }
  return (void*)mem;
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::MemberBase::fullName( bool typedefexp ) const {
//-------------------------------------------------------------------------------
  typedefexp = false;
  return m_name;
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Scope MLANG_Reflection::MemberBase::scope() const {
//-------------------------------------------------------------------------------
  return m_scope;
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::MemberBase::memberTypeAsString() const {
//-------------------------------------------------------------------------------
  switch ( m_memberType ) {
    case DATAMEMBER:
      return "DataMember";
      break;
    case FUNCTIONMEMBER:
      return "FunctionMember";
      break;
    default:
      return Reflex::argv0 + ": ERROR: Member " + fullName() +
         " has no Species associated";
  }
}

//-------------------------------------------------------------------------------
MLANG_Reflection::PropertyList & MLANG_Reflection::MemberBase::propertyList() const {
//-------------------------------------------------------------------------------
  return m_propertyList;
}

