//$Id: Object.h,v 1.5 2004/11/30 22:57:58 roiser Exp $

#ifndef MLANG_REFLECTION_OBJECT_H
#define MLANG_REFLECTION_OBJECT_H

// Include files
#include "MLANG/Reflection/Type.h"
#include <string>
#include <vector>

namespace MLANG_Reflection
{

    // forward declarations

    /** 
     * @class Object Object.h MLANG/Reflection/Object.h
     * @author Stefan Roiser
     * @date 24/06/2004
     * @ingroup Ref
     */
    class Object {
      
    public:

      /** constructor */
      Object( const Type & type = Type(), 
              void * mem = 0 );


      /** constructor */
      Object( const Object & );
      

      /** destructor */
      virtual ~Object() {}

      
      /**
       * type will return the pointer to the type of the object
       * @return pointer to object type
       */
      Type type() const;


      /** 
       * address will return the memory address of the object
       * @return memory address of object
       */
      void * address() const;

      
      /**
       * operator assigment 
       */
       Object operator = ( const Object & obj );

  
      /**
       * operator ==
       */
       bool operator == ( const Object & obj );

       
      /**
       * other utility functions
       */
       void * field( const MLANG_String & data ) const;

       void * invoke( const MLANG_String & method, 
                      std::vector<void*> args = std::vector<void*>(), 
                      const Type & signature = Type() ) const;

    private:

      /** the type of the object */
      Type m_type;


      /** the address of the object */
      void * m_address;

    }; // class Object
} // namespace MLANG_Reflection


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Object::Object( const Type & type,
                                     void * address ) 
//-------------------------------------------------------------------------------
  : m_type( type ),
    m_address( address ) {}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Object::Object( const Object & obj )
//-------------------------------------------------------------------------------
  : m_type( obj.m_type ),
    m_address( obj.m_address ) {}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type MLANG_Reflection::Object::type() const {
//-------------------------------------------------------------------------------
  return m_type;
}


//-------------------------------------------------------------------------------
inline void * MLANG_Reflection::Object::address() const {
//-------------------------------------------------------------------------------
  return m_address;
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Object MLANG_Reflection::Object::operator = ( const Object & obj ) {
//-------------------------------------------------------------------------------
  m_type    = obj.m_type;
  m_address = obj.m_address;
  return * this;
}


//-------------------------------------------------------------------------------
inline bool MLANG_Reflection::Object::operator == ( const Object & obj ) {
//-------------------------------------------------------------------------------
  return ( m_type  == obj.m_type &&  m_address == obj.m_address );
}


#endif // MLANG_REFLECTION_OBJECT_H
