//$Id: Pointer.h,v 1.6 2004/11/30 22:57:58 roiser Exp $
#ifndef MLANG_REFLECTION_POINTER_H
#define MLANG_REFLECTION_POINTER_H

// Include files
#include "MLANG/Reflection/TypeBase.h"
#include "MLANG/Reflection/Type.h"

namespace MLANG_Reflection
{

    // forward declarations

    /**
     * @class Pointer Pointer.h MLANG/Reflection/Pointer.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class Pointer : public TypeBase {
    public:

      /** default constructor */
      Pointer( const Type & pointerType, 
               const std::type_info & typeInfo );

      /** destructor */
      virtual ~Pointer() {}


      /**
       * fullName will return the fully qualified name of the pointer type
       * @param  typedefexp expand typedefs or not
       * @return fully qualified name of pointer type
       */
      MLANG_String fullName( bool typedefexp = false ) const;


      /**
       * type will return a pointer to the type the pointer points to.
       * @return pointer to Type of member et. al.
       */
      Type toType() const;


      /** static funtion that composes the type name */
      static MLANG_String buildTypeName( const Type & pointerType );

    private:

      /**
       * pointer to the Type the Pointer points to
       * @label pointer type
       * @link aggregation
       * @supplierCardinality 1
       */
      Type m_pointerType;

    }; // class Pointer
} //namespace MLANG_Reflection


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type MLANG_Reflection::Pointer::toType() const {
//-------------------------------------------------------------------------------
  return m_pointerType;
}

#endif // MLANG_REFLECTION_POINTER_H

