//$Id: PropertyList.h,v 1.8 2004/12/01 16:42:37 roiser Exp $

#ifndef MLANG_REFLECTION_PROPERTYLIST_H
#define MLANG_REFLECTION_PROPERTYLIST_H


// Include files
#include "MLANG/Reflection/Kernel.h"
#include "MLANG/Reflection/Any.h"
#include <map>
#include <ostream>

namespace MLANG_Reflection
{

    /**
     * @class PropertyList PropertyList.h MLANG/Reflection/PropertyList.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class PropertyList {

    public:

      /** default constructor */
      PropertyList();

     /** copy constructor */
      PropertyList(const PropertyList&);


      /** destructor */
      virtual ~PropertyList();


      /**
       * addProperty will add a key value pair to the property lsit
       * @param key the key of the property
       * @param value the value of the property (as any object)
       */
      void addProperty( const MLANG_String & key,
                        const Any & value );


      /**
       * addProperty will add a key value pair to the property lsit
       * @param key the key of the property
       * @param value the value of the property (as any object)
       */
      void addProperty( const MLANG_String & key,
                        const char * value );

      
      /**
       * clearProperties will remove the internal map of properties if 
       * it is instantiated
       */
      void clearProperties();


      /**
       * removeProperty will remove a key value pair to the property lsit
       * @param key the key of the property
       */
      void removeProperty( const MLANG_String & key );


      /**
       * hasKey will return true if the property list contains the key
       * @param  key the property key
       * @return nth property key
       */
      bool hasKey( const MLANG_String & key ) const;


      /**
       * propertyCount will return the number of properties attached
       * to this item
       * @return number of properties
       */
      size_t propertyCount() const;


      /**
       * propertyKeys will return all keys of this property list
       * @return all property keys
       */
      MLANG_String propertyKeys() const;


      /**
       * propertyNumString will return the nth property as a string if printable
       * @param  key the property key
       * @return nth property value as string
       */
      MLANG_String propertyAsString(const MLANG_String & key) const;


      /**
       * propertyNumValue will return the nth property value 
       * @param  key the property key
       * @return nth property value
       */
      Any propertyValue(const MLANG_String & key) const;

#if 0
      /** 
       * will put the property (key and value) on the ostream if printable
       * @param s the reference to the stream
       * @param p the property to print
       * @return the stream
       */
      friend std::ostream & operator<<( std::ostream & s, const PropertyList & p );
#endif

    public:

      static PropertyList __NOPROPERTYLIST__;


      /** the type of properties */
      typedef std::map< const MLANG_String, Any > Properties;
      
    private:

      /** the properties of the item */
      Properties * m_properties;

    }; // class Propertylist
} //namespace MLANG_Reflection

//-------------------------------------------------------------------------------
inline MLANG_Reflection::PropertyList::PropertyList() 
//-------------------------------------------------------------------------------
  : m_properties( 0 ) {}

//-------------------------------------------------------------------------------
inline MLANG_Reflection::PropertyList::PropertyList(const PropertyList& pl) 
//-------------------------------------------------------------------------------
  : m_properties( pl.m_properties ) {}

  
//-------------------------------------------------------------------------------
inline MLANG_Reflection::PropertyList::~PropertyList() {
//-------------------------------------------------------------------------------
  // FIXME: It need to be deleted by the "owner" of the property list
  //if ( m_properties ) delete m_properties;
}


//-------------------------------------------------------------------------------
inline void MLANG_Reflection::PropertyList::addProperty( const MLANG_String & key,
                                                     const Any & value ) {
//-------------------------------------------------------------------------------
  if ( ! m_properties ) m_properties = new Properties();
  (*m_properties)[ key ] = value;
}


//-------------------------------------------------------------------------------
inline void MLANG_Reflection::PropertyList::addProperty( const MLANG_String & key,
                                                     const char* value ) {
//-------------------------------------------------------------------------------
  if ( ! m_properties ) m_properties = new Properties();
  (*m_properties)[ key ] = value;
}


//-------------------------------------------------------------------------------
inline void MLANG_Reflection::PropertyList::clearProperties() {
//-------------------------------------------------------------------------------
  if ( ! m_properties ) {
    delete m_properties;
    m_properties = 0;
  }
}


//-------------------------------------------------------------------------------
inline void MLANG_Reflection::PropertyList::removeProperty( const MLANG_String & key ) {
//-------------------------------------------------------------------------------
  if ( m_properties ) m_properties->erase( key );
}


//-------------------------------------------------------------------------------
inline bool MLANG_Reflection::PropertyList::hasKey(const MLANG_String & key) const {
//-------------------------------------------------------------------------------
  if ( m_properties && m_properties->find( key ) != m_properties->end() ) 
    return true;
  return false;
}


//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::PropertyList::propertyCount() const {
//-------------------------------------------------------------------------------
  if ( m_properties ) return m_properties->size();
  return 0;
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Any
MLANG_Reflection::PropertyList::propertyValue(const MLANG_String & key) const {
//-------------------------------------------------------------------------------
  if ( m_properties ) return (*m_properties)[ key ];
  return Any();
}

#endif // MLANG_REFLECTION_PROPERTYLIST_H
