//$Id: Reference.h,v 1.6 2004/11/01 10:12:02 roiser Exp $

#ifndef REFLECTION_REFERENCE_H
#define REFLECTION_REFERENCE_H

// Include files
#include "MLANG/Reflection/TypeBase.h"

namespace MLANG_Reflection

{

    // forward declarations
    class Type;

    /**
     * @class Reference Reference.h Reflection/Reference.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class Reference : public TypeBase {
    public:

      /** default constructor */
      Reference(const Type * referencetype,
                const std::type_info & typeInfo );

      /** destructor */
      virtual ~Reference();


      /**
       * fullName will return the fully qualified name of the reference type
       * @param  typedefexp expand typedefs or not
       * @return fully qualified name of reference type
       */
      MLANG_String fullName(bool typedefexp = false) const;


      /**
       * toType will return a pointer to the type of the reference.
       * @return pointer to Type of member et. al.
       */
      const Type * toType() const;


      /** static funtion that composes the type name */
      static MLANG_String buildTypeName(const Type* refType );

    private:
    
      /**
       * pointer to the referenced type
       * @label reference type
       * @link aggregation
       * @supplierCardinality 1
       */
      const Type * m_referenceType;

    }; // class Reference
} //namespace MLANG_Reflection


//-------------------------------------------------------------------------------
inline const MLANG_Reflection::Type * MLANG_Reflection::Reference::toType() const {
//-------------------------------------------------------------------------------
  return m_referenceType;
}

#endif // REFLECTION_REFERENCE_H


