//$Id: ScopeBase.cpp,v 1.13 2004/12/01 16:42:37 roiser Exp $

#include "MLANG/Reflection/Scope.h"
#include "MLANG/Reflection/ScopeBase.h"
#include "MLANG/Reflection/Class.h"
#include "MLANG/Reflection/InstantiatedTemplateClass.h"
#include "MLANG/Reflection/Namespace.h"
#include "MLANG/Reflection/DataMember.h"
#include "MLANG/Reflection/FunctionMember.h"
#include "MLANG/Reflection/Tools.h"
#include "MLANG/Reflection/PropertyList.h"
#include "MLANG/Reflection/Type.h"
#include "MLANG/Reflection/Member.h"
#include "MLANG/Reflection/ScopeName.h"


//-------------------------------------------------------------------------------
MLANG_Reflection::ScopeBase::ScopeBase( const char * scope, 
                                    TYPE scopeType )
//-------------------------------------------------------------------------------
  : m_members( Members()),
    m_dataMembers( Members()),
    m_functionMembers( Members()),
    m_scopeName( 0 ),
    m_name( "" ),
    m_scopeType( scopeType ),
    m_declaringScope( Scope() ),
    m_subScopes( std::vector<Scope>()),
    m_types( std::vector<Type>()),
    m_propertyList( PropertyList() )
{
  MLANG_String sname(scope);

  MLANG_String declScope;
  MLANG_String currScope = sname;

  // Construct Scope
  Scope scopePtr = Scope::byName(sname);
  if ( ! scopePtr ) { 
    // create a new scope
    m_scopeName = new ScopeName(scope, this); 
  }
  else {
    m_scopeName = (ScopeName*)scopePtr.id();
    m_scopeName->m_scopeBase = this;
  }

  Scope declScopePtr = Scope::byName("");

  size_t pos = sname.lastIndexOf( "::" );  
  // Name contains declaring scope
  if ( (int)pos != MLANG_String::npos ) {   
    currScope = sname.substring( pos + 2 );
    declScope = sname.substring( 0, pos - 1 ); 
    declScopePtr = Scope::byName(declScope);
    if ( ! declScopePtr ) {
      if ( scopeType == NAMESPACE ) {
        Namespace * ns = new Namespace(declScope.c_str()); 
        declScopePtr = ns->scope();
      }
      else {
        declScopePtr = (new ScopeName(declScope.c_str(), 0))->scope();
      }
    }
  }

  // scope name within the decalring scope
  m_name = currScope;

  // Set declaring scope and sub-scopes
  m_declaringScope = declScopePtr; 
  if ( m_declaringScope )  m_declaringScope.addSubScope( this->scope());
}


//-------------------------------------------------------------------------------
MLANG_Reflection::ScopeBase::ScopeBase() 
//-------------------------------------------------------------------------------
  : m_members( Members()),
    m_dataMembers( Members()),
    m_functionMembers( Members()),
    m_scopeName( 0 ),
    m_name(""),
    m_scopeType( NAMESPACE ),
    m_declaringScope( Scope::__NIRVANA__ ),
    m_subScopes( std::vector<Scope>()),
    m_types( std::vector<Type>()),
    m_propertyList( PropertyList() ) {
  m_scopeName = new ScopeName("", this);
  m_propertyList.addProperty("Description", "global namespace");
}


//-------------------------------------------------------------------------------
MLANG_Reflection::ScopeBase::~ScopeBase( ) {
//-------------------------------------------------------------------------------
  // Informing Scope that I am going away
  if ( m_scopeName->m_scopeBase == this ) m_scopeName->m_scopeBase = 0;

  // Informing declaring scope that I am going to do away
  if ( m_declaringScope ) {
    m_declaringScope.removeSubScope(this->scope());
  }

  m_propertyList.clearProperties();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::ScopeBase::operator MLANG_Reflection::Scope () const {
//-------------------------------------------------------------------------------
  return Scope( m_scopeName );
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Member
MLANG_Reflection::ScopeBase::dataMember( size_t nth ) const {
//-------------------------------------------------------------------------------
  if ( nth < m_dataMembers.size() ) return m_dataMembers[ nth ];
  return Member();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::ScopeBase::dataMemberCount() const {
//-------------------------------------------------------------------------------
  return m_dataMembers.size();
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::ScopeBase::fullName(bool /* */) const {
//-------------------------------------------------------------------------------
  return m_scopeName->name();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Member
MLANG_Reflection::ScopeBase::functionMember( size_t nth ) const { 
//-------------------------------------------------------------------------------
  if ( nth < m_functionMembers.size() ) return m_functionMembers[ nth ];
  return Member();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::ScopeBase::functionMemberCount() const {
//-------------------------------------------------------------------------------
  return m_functionMembers.size();
}

//-------------------------------------------------------------------------------
bool MLANG_Reflection::ScopeBase::isTopScope() const {
//-------------------------------------------------------------------------------
  if ( m_declaringScope == Scope::__NIRVANA__ ) return true;
  return false;
}

//-------------------------------------------------------------------------------
MLANG_Reflection::Member MLANG_Reflection::ScopeBase::member( size_t nth ) const {
//-------------------------------------------------------------------------------
  if ( nth < m_members.size() ) { return m_members[ nth ]; };
  return Member();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::ScopeBase::memberCount() const {
//-------------------------------------------------------------------------------
  return m_members.size();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Member 
MLANG_Reflection::ScopeBase::member( const MLANG_String & name ) const {
//-------------------------------------------------------------------------------
  for ( size_t i = 0; i < m_members.size() ; i++ ) {
    if ( m_members[i].name() == name ) return m_members[i];
  }
  return Member();
}


//-------------------------------------------------------------------------------
const MLANG_String & MLANG_Reflection::ScopeBase::name() const {
//-------------------------------------------------------------------------------
  return m_scopeName->name();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::PropertyList & MLANG_Reflection::ScopeBase::propertyList() const {
//-------------------------------------------------------------------------------
  return m_propertyList;
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Scope MLANG_Reflection::ScopeBase::scope() const {
//-------------------------------------------------------------------------------
  return m_scopeName->scope();
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::ScopeBase::scopeTypeAsString() const {
//-------------------------------------------------------------------------------
  switch ( m_scopeType ) {
    case CLASS:
      return "CLASS";
      break;
    case INSTANTIATEDTEMPLATECLASS:
      return "INSTANTIATEDTEMPLATECLASS";
      break;
    case NAMESPACE:
      return "NAMESPACE";
      break;
    case UNRESOLVED:
      return "UNRESOLVED";
      break;
    default:
      return "Scope " + fullName() + "is not assigned to a SCOPE";
  }
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::ScopeBase::subType( size_t nth ) const {
//-------------------------------------------------------------------------------
  if ( nth < m_types.size() ) { return m_types[ nth ]; }
  return Type();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::ScopeBase::subTypeCount() const {
//-------------------------------------------------------------------------------
  return m_types.size();
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::addDataMember( const Member & dm ) const {
//-------------------------------------------------------------------------------
  dm.setScope( scope() );
  m_dataMembers.push_back( dm );
  m_members.push_back( dm );
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::removeDataMember( const Member & dm ) const {
//-------------------------------------------------------------------------------
  std::vector< Member >::iterator it;
  for ( it = m_dataMembers.begin(); it != m_dataMembers.end(); ++it) {
    if ( *it == dm ) m_dataMembers.erase(it); break;
  }
  std::vector< Member >::iterator im;
  for ( im = m_members.begin(); im != m_members.end(); ++im) {
    if ( *im == dm ) m_members.erase(im); break;
  }
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::addFunctionMember( const Member & fm ) const {
//-------------------------------------------------------------------------------
  fm.setScope( scope());
  m_functionMembers.push_back( fm );
  m_members.push_back( fm );
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::removeFunctionMember( const Member & fm ) const {
//-------------------------------------------------------------------------------
  std::vector< Member >::iterator it;
  for ( it = m_functionMembers.begin(); it != m_functionMembers.end(); ++it) {
    if ( *it == fm ) m_functionMembers.erase(it); break;
  }
  std::vector< Member >::iterator im;
  for ( im = m_members.begin(); im != m_members.end(); ++im) {
    if ( *im == fm ) m_members.erase(im); break;
  }
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::addSubScope( const Scope & subscope ) {
//-------------------------------------------------------------------------------
  removeSubScope(subscope);
  m_subScopes.push_back(subscope);
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::removeSubScope( const Scope & subscope ) {
//-------------------------------------------------------------------------------
  std::vector< Scope >::iterator it;
  for ( it = m_subScopes.begin(); it != m_subScopes.end(); ++it) {
    if ( *it == subscope ) {
      m_subScopes.erase(it); break;
    }
  }
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::addSubType( const Type & type ) {
//-------------------------------------------------------------------------------
  removeSubType(type);
  m_types.push_back(type);
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::ScopeBase::removeSubType( const Type & type ) {
//-------------------------------------------------------------------------------
  std::vector< Type >::iterator it;
  for ( it = m_types.begin(); it != m_types.end(); ++it) {
    if ( *it == type ) {
      m_types.erase(it); break;
    }
  }
}


