//$Id: ScopeName.h,v 1.1 2004/11/30 22:57:58 roiser Exp $

#ifndef MLANG_REFLECTION_SCOPENAME_H
#define MLANG_REFLECTION_SCOPENAME_H

// Include files
#include "MLANG/Reflection/Kernel.h"
#include "MLANG/String.h"

namespace MLANG_Reflection
{

    // forward declarations
    class ScopeBase;
    class Scope;

    /**
     * @class ScopeName ScopeName.h MLANG/Reflection/ScopeName.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class ScopeName {

      friend class Scope;
      friend class ScopeBase;

    public:

      /** default constructor */
      ScopeName( const char * name, 
                 ScopeBase * scopeBase );


      /**
       * byName will return a pointer to a scope which is given as an argument
       * or 0 if none is found
       * @param  name fully qualified name of scope
       * @return pointer to scope or 0 if none is found
       */
      static Scope byName( const MLANG_String & name );


      /**
       * name will return a string representation of name of the scope (NOT
       * fully qualified, see fullName)
       * @return string representation of scope
       */
      const MLANG_String & name() const;


      /** 
       * scope will return the unqualified Scope object of this ScopeName
       * @return corresponding Scope
       */
      Scope scope() const;


      /**
       * findAll will return a vector of all scopes currently available
       * resolvable scopes
       * @param  nth scope defined in the system
       * @return vector of all available scopes
       */
      static Scope scope( size_t nth );


      /**
       * scopeCount will return the number of currently defined scopes
       * (resolved and unresolved ones)
       * @return number of currently defined scopes
       */
      static size_t scopeCount();

    private:

      /** destructor */
      virtual ~ScopeName();

    private:

      /** pointer to the name of the scope in the static map */
      MLANG_String m_name;


      /**
       * pointer to the resolved Scope
       * @label scope base
       * @link aggregationByValue
       */
      ScopeBase * m_scopeBase;

    }; // class ScopeName
} // namespace MLANG_Reflection


//-------------------------------------------------------------------------------
inline const MLANG_String & MLANG_Reflection::ScopeName::name() const {
//-------------------------------------------------------------------------------
  return m_name;
}

#endif //MLANG_REFLECTION_SCOPENAME_H
