// $Id: TemplateFamily.h,v 1.2 2004/07/27 14:31:44 mato Exp $

#ifndef REFLECTION_TEMPLATEFAMILY_H
#define REFLECTION_TEMPLATEFAMILY_H 1

// Include files
#include <vector>

namespace MLANG_Reflection

{

    // forward declarations
    class TypeBase;
    class Type;
    class InstantiatedTemplate;


    /**
     * @class TemplateFamily TemplateFamily.h Reflection/TemplateFamily.h
     * @author Stefan Roiser
     * @date   2004-01-28
     * @ingroup Ref
     */
    class TemplateFamily {

    public:

      /** default constructor */
      TemplateFamily(const char * name,
                     std::vector < InstantiatedTemplate * > instantiations );


      /** destructor */
      virtual ~TemplateFamily() {}


      /**
       * byName will look up the template family given as argument (argument
       * string without "<>")
       * @param  name of the template family
       * @return pointer to template family
       */
      static const TemplateFamily * byName( const MLANG_String & name );


      /**
       * byNameAndTypes will return a pointer to a templated function or class
       * @param  name of the template family
       * @param  templates pointers to types of template instantiation
       * @return pointer to Templated (class or function)
       */
      static const InstantiatedTemplate * byNameAndTypes(const MLANG_String & name, 
                                                         const std::vector < const Type * > templates);


      /**
       * fullName will return the name of the template family and a list of
       * all currently available instantiations
       * @return template family name with all instantiantion
       */
      const MLANG_String fullName(bool typedefexp = false) const;


      /**
       * instantion will return a pointer to the nth template instantion
       * @param  nth template instantion
       * @return pointer to nth template instantion
       */
      const InstantiatedTemplate * instantiation( size_t nth ) const;


      /**
       * instantionCount will return the number of template instantions for
       * this template family
       * @return number of template instantions
       */
      size_t instantiationCount() const;

    private:

      /** the name of the template family */
      MLANG_String m_name;


      /**
       * template instantions for this family
       * @label instantiations
       * @link aggregationByValue
       * @supplierCardinality 1..*
       */
      std::vector < InstantiatedTemplate * > m_instantiations;

      /// maybe add later the template information (e.g. "<class A, int b>")
    }; // class TemplateFamily
} // namespace MLANG_Reflection

//-------------------------------------------------------------------------------
inline MLANG_Reflection::TemplateFamily::TemplateFamily( const char * name,
                                                      std::vector < InstantiatedTemplate * > instantiations )
//-------------------------------------------------------------------------------
  : m_name( name ),
    m_instantiations( instantiations ) {}

//-------------------------------------------------------------------------------
inline const MLANG_String 
MLANG_Reflection::TemplateFamily::fullName( bool typedefexp ) const {
//-------------------------------------------------------------------------------
  typedefexp = false;
  return m_name;
}

//-------------------------------------------------------------------------------
inline const MLANG_Reflection::InstantiatedTemplate *
   MLANG_Reflection::TemplateFamily::instantiation( size_t nth ) const {
//-------------------------------------------------------------------------------
     if ( nth < m_instantiations.size() ) { return m_instantiations[ nth ]; }
     return 0;
}

//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::TemplateFamily::instantiationCount() const {
//-------------------------------------------------------------------------------
  return m_instantiations.size();
}
#endif // MLANG_REFLECTIONION_TEMPLATEFAMILY_H



