//$Id: Tools.cpp,v 1.9 2004/10/12 12:56:38 roiser Exp $

#include "MLANG/Reflection/Tools.h"
#include "MLANG/Reflection/Function.h"
#include "MLANG/Reflection/Scope.h"
#include "MLANG/Reflection/Type.h"
#include "MLANG/Reflection/Pointer.h"
#include "MLANG/Reflection/Array.h"
#include "FuncHandler.h"

#ifdef __GNUC__
  #include <cxxabi.h>
#endif

//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::stringSplit( std::vector < MLANG_String > & splitValues, 
                                        const MLANG_String & str,
                                        const MLANG_String & delim ) {
//-------------------------------------------------------------------------------
  MLANG_String str2 = str;
  
  size_t pos = 0;
  
  while ( (int) ( pos = str2.find_first_of( delim )) != MLANG_String::npos ) {
    MLANG_String s = str2.substring(0, pos - 1);
    stringStrip( s );
    if ( s.length()) { splitValues.push_back( s ); }
    str2 = str2.substring( pos + delim.length());
  }
  
  stringStrip( str2 );
  if ( str2.length()) { splitValues.push_back( str2 ); }
}



//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::Tools::demangle( const std::type_info & ti ) { 
//-------------------------------------------------------------------------------
#ifdef _WIN32
  static std::vector<MLANG_String> keywords;
  if ( 0 == keywords.size() ) {
    keywords.push_back("class ");
    keywords.push_back("struct ");
    keywords.push_back("enum ");
    keywords.push_back("union ");
    keywords.push_back("__cdecl");
  }
  MLANG_String r = ti.name();
#if 0
  for ( size_t i = 0; i < keywords.size(); i ++ ) {
    while (r.find(keywords[i]) != MLANG_String::npos) 
      r = r.replace(r.find(keywords[i]), keywords[i].size(), "");
    while (r.find(" *") != MLANG_String::npos) 
      r = r.replace(r.find(" *"), 2, "*");
    while (r.find(" &") != MLANG_String::npos) 
      r = r.replace(r.find(" &"), 2, "&");
  }
#else
  assert(0);
#endif
  return r;

#else

  int status = 0;
  bool remove_additional_pointer = false;
  MLANG_String  mangled = ti.name();

#if __GNUC__ <= 3 && __GNUC_MINOR__ <= 2 || defined(__APPLE__)
  // Function types are not decoded at all. We are an extra 'P' to convert it to a pointer
  // and remove it at the end.
  if ( mangled[0] == 'F' ) {
    mangled = "P" + mangled;
    remove_additional_pointer = true;
  }
#endif
  char * c_demangled = abi::__cxa_demangle( mangled.c_str(), 0, 0, & status );
  if ( status == -1 ) {
    throw RuntimeError("Memory allocation failure while demangling ");
  }
  else if ( status == -2 ) {
    throw RuntimeError( MLANG_String(mangled) + " is not a valid name under the C++ ABI");
  }
  else if ( status == -3 )
   {
    throw RuntimeError( MLANG_String("Failure while demangling ") + mangled +
            ". One of the arguments is invalid ");
   }
  else
   {
    int pos;
    MLANG_String demangled = c_demangled;
    free( c_demangled );
    if ( remove_additional_pointer )
     {
      int pos = demangled.indexOf("(*)");
      MLANG_String temp = demangled.substring (0, pos - 1);
      temp += demangled.substring (pos + 3);
      demangled = temp;
     }
    while ( (pos=demangled.indexOf(", ")) != MLANG_String::npos )
     {
      MLANG_String temp;
      if (pos!=0)
       {
         temp += demangled.substring (0, pos - 1);
       }
      temp += ",";
      temp += demangled.substring (pos + 2);
      demangled = temp;
     }
    return demangled;
  }
#endif
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::stringSplitPair( MLANG_String & val1,
                                            MLANG_String & val2,
                                            const MLANG_String & str,
                                            const MLANG_String & delim ) { 
//-------------------------------------------------------------------------------
  MLANG_String str2 = str;
  size_t pos = str2.lastIndexOf( delim );
  if ( (int)pos != MLANG_String::npos ) { 
    val1 = str2.substring( 0, pos - 1 ); 
    val2 = str2.substring( pos + delim.length());
  }
  else { 
    val1 = str2; 
  }
  stringStrip( val1 );
  stringStrip( val2 );
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::stringStrip( MLANG_String & str ) {
//-------------------------------------------------------------------------------
  size_t sPos = 0;
  size_t ePos = str.length();
  while ( str[sPos] == ' ' ) { ++sPos; }
  while ( str[ePos] == ' ' ) { --ePos; }
  str = str.substring( sPos, ePos - 1 );
}
