//$Id: TypeBase.cpp,v 1.11 2004/12/01 16:42:37 roiser Exp $

#include "MLANG/Reflection/TypeBase.h"
#include "MLANG/Reflection/Type.h"
#include "MLANG/Reflection/Array.h"
#include "MLANG/Reflection/Pointer.h"
#include "MLANG/Reflection/PropertyList.h"
#include "MLANG/Reflection/PointerToMember.h"
#include "MLANG/Reflection/Union.h"
#include "MLANG/Reflection/Enum.h"
#include "MLANG/Reflection/Fundamental.h"
#include "MLANG/Reflection/Function.h"
#include "MLANG/Reflection/Class.h"
#include "MLANG/Reflection/Typedef.h"
#include "MLANG/Reflection/InstantiatedTemplateClass.h"
#include "MLANG/Reflection/InstantiatedTemplateFunction.h"
#include "MLANG/Reflection/Object.h"
#include "MLANG/Reflection/Scope.h"
#include "MLANG/Reflection/TypeName.h"
#include <iostream>

//-------------------------------------------------------------------------------
MLANG_Reflection::TypeBase::TypeBase( const char * name, 
                                  size_t size,
                                  TYPE typeType, 
                                  const std::type_info & typeInfo ) 
//-------------------------------------------------------------------------------
  : m_scope( Scope( 0 )),
    m_size( size ),
    m_typeInfo( typeInfo ), 
    m_typeType( typeType ),
    m_propertyList( PropertyList()) {

  Type t = TypeName::byName( name );
  if ( t.id() == 0 ) { 
    m_typeName = new TypeName( name, this, &typeInfo ); 
  }
  else {
    m_typeName = (TypeName*)t.id();
    if ( m_typeName->m_typeBase ) delete m_typeName->m_typeBase;
    m_typeName->m_typeBase = this;
  }

  MLANG_String sname( name );
  size_t pos = sname.lastIndexOf( "::" );
  // Name contains declaring scope
  if ( (int)pos != MLANG_String::npos ) {
    m_scope = Scope::byName(sname.substring( 0, pos ));
    if ( m_scope.id() == 0 ) {
      m_scope = (new ScopeName(sname.substring( 0, pos ).c_str(), 0))->scope();
    }
  }
  else {
    m_scope = Scope::byName("");
  }

  // Set declaring scope
  if ( m_scope ) m_scope.addSubType(type());
}


//-------------------------------------------------------------------------------
MLANG_Reflection::TypeBase::~TypeBase( ) {
//-------------------------------------------------------------------------------
  if( m_typeName->m_typeBase == this ) m_typeName->m_typeBase = 0;
  m_propertyList.clearProperties();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::TypeBase::operator MLANG_Reflection::Type () const {
//-------------------------------------------------------------------------------
  return Type( m_typeName );
}


//-------------------------------------------------------------------------------
void * MLANG_Reflection::TypeBase::allocate() const {
//-------------------------------------------------------------------------------
  return operator new( m_size );
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::TypeBase::deallocate( void * instance ) const {
//-------------------------------------------------------------------------------
  operator delete( instance );
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Object 
MLANG_Reflection::TypeBase::construct( const Type &  /*signature*/,
                                   std::vector < void * > /*values*/, 
                                   void * /*mem*/ ) const {
//-------------------------------------------------------------------------------
  return Object(type(),allocate());
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::TypeBase::destruct( void * instance, 
                                       bool dealloc ) const {
//-------------------------------------------------------------------------------
  if ( dealloc ) deallocate(instance);
}


//-------------------------------------------------------------------------------
MLANG_Reflection::EnumInfo MLANG_Reflection::TypeBase::enumInfo( size_t /* nth */ ) const {
//-------------------------------------------------------------------------------
  return EnumInfo();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::TypeBase::enumInfoCount() const {
//-------------------------------------------------------------------------------
  return 0;
}
  

//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::TypeBase::fullName( bool /* typedefexp */ ) const {
//-------------------------------------------------------------------------------
  return name();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::TypeBase::length() const {
//-------------------------------------------------------------------------------
  return 0;
}


//-------------------------------------------------------------------------------
const MLANG_String & MLANG_Reflection::TypeBase::name() const {
//-------------------------------------------------------------------------------
  return m_typeName->name();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypeBase::parameter( size_t /* nth */ ) const {
//-------------------------------------------------------------------------------
  return Type();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::TypeBase::parameterCount() const {
//-------------------------------------------------------------------------------
  return 0;
}


//-------------------------------------------------------------------------------
MLANG_Reflection::PropertyList & MLANG_Reflection::TypeBase::propertyList() const {
//-------------------------------------------------------------------------------
  return m_propertyList;
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Scope MLANG_Reflection::TypeBase::scope() const {
//-------------------------------------------------------------------------------
  return m_scope;
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypeBase::type() const {
//-------------------------------------------------------------------------------
  return m_typeName->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypeBase::returnType() const {
//-------------------------------------------------------------------------------
  return Type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypeBase::toType() const {
//-------------------------------------------------------------------------------
  return Type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::TYPE MLANG_Reflection::TypeBase::typeType() const {
//-------------------------------------------------------------------------------
  return m_typeType;
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::TypeBase::typeTypeAsString() const {
//-------------------------------------------------------------------------------
  switch ( m_typeType ) {
  case CLASS:
    return "CLASS";
    break;
  case ENUM:
    return "ENUM";
    break;
  case FUNCTION:
    return "FUNCTION";
    break;
  case ARRAY:
    return "ARRAY";
    break;
  case FUNDAMENTAL:
    return "FUNDAMENTAL";
    break;
  case POINTER:
    return "POINTER";
    break;
  case REFERENCE:
    return "REFERENCE";
    break;
  case TYPEDEF:
    return "TYPEDEF";
    break;
  case INSTANTIATEDTEMPLATECLASS:
    return "INSTANTIATEDTEMPLATECLASS";
    break;
  case INSTANTIATEDTEMPLATEFUNCTION:
    return "INSTANTIATEDTEMPLATEFUNCTION";
    break;
  case UNRESOLVED:
    return "UNRESOLVED";
    break;
  default:
    return "Type " + fullName() + "is not assigned to a TYPE";
  }
}


//-------------------------------------------------------------------------------
MLANG_Reflection::UnionInfo MLANG_Reflection::TypeBase::unionInfo( size_t /* nth */ ) const {
//-------------------------------------------------------------------------------
  return UnionInfo();
}


