//$Id: TypeBuilder.cpp,v 1.4 2004/12/02 08:35:09 mato Exp $

#include "MLANG/Reflection/Type.h"
#include "MLANG/Reflection/TypeName.h"
#include "MLANG/Reflection/Pointer.h"
#include "MLANG/Reflection/Function.h"
#include "MLANG/Reflection/Array.h"
#include "MLANG/Reflection/Typedef.h"
#include "MLANG/Reflection/Tools.h"
#include "MLANG/Reflection/TypeBuilder.h"


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypeBuilder(const char * n) {
//-------------------------------------------------------------------------------
  Type ret = Type::byName(n);
  if ( ret ) return ret;
  else       return (new TypeName(n, 0))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::ConstBuilder(const Type & t) {
//-------------------------------------------------------------------------------
  unsigned int mod = MLANG_CONST;
  if ( t.isVolatile() ) mod |= VOLATILE;
  return Type(t,mod);    
}

//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::VolatileBuilder(const Type & t) {
//-------------------------------------------------------------------------------
  unsigned int mod = VOLATILE;
  if ( t.isConst() )    mod |= MLANG_CONST;
  return Type(t,mod);    
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::PointerBuilder(const Type & t) {
//-------------------------------------------------------------------------------
  Type ret = Type::byName( Pointer::buildTypeName(t));
  if ( ret ) return ret;
  else       return (new Pointer(t, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::ReferenceBuilder(const Type & t) {
//-------------------------------------------------------------------------------
  unsigned int mod = REFERENCE;
  if ( t.isConst() )    mod |= MLANG_CONST;
  if ( t.isVolatile() ) mod |= VOLATILE;
  return Type(t,mod);    
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::ArrayBuilder(const Type & t, 
                                              size_t n) {
//-------------------------------------------------------------------------------
  Type ret = Type::byName(Array::buildTypeName(t,n));
  if ( ret ) return ret;
  else       return (new Array(t, n, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypedefTypeBuilder(const char * name, 
                                                    const Type & t) {
//-------------------------------------------------------------------------------
  Type ret = Type::byName(name);
  if ( ret ) return ret;
  else       return (new Typedef( name , t, &t.typeInfo()))->type();        
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r) {
//-------------------------------------------------------------------------------
  std::vector< Type > v;
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}
 

//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8,
                                                     const Type & t9) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8, t9);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}
 

//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8,
                                                     const Type & t9,
                                                     const Type & t10) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8,
                                                     const Type & t9,
                                                     const Type & t10,
                                                     const Type & t11) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}
 

//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8,
                                                     const Type & t9,
                                                     const Type & t10,
                                                     const Type & t11,
                                                     const Type & t12) {
//-------------------------------------------------------------------------------
  std::vector< Type > v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}
 

//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8,
                                                     const Type & t9,
                                                     const Type & t10,
                                                     const Type & t11,
                                                     const Type & t12,
                                                     const Type & t13) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8,
                                                     const Type & t9,
                                                     const Type & t10,
                                                     const Type & t11,
                                                     const Type & t12,
                                                     const Type & t13,
                                                     const Type & t14) {
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}
 

//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::FunctionTypeBuilder(const Type & r, 
                                                     const Type & t0, 
                                                     const Type & t1,
                                                     const Type & t2,
                                                     const Type & t3,
                                                     const Type & t4,
                                                     const Type & t5,
                                                     const Type & t6,
                                                     const Type & t7,
                                                     const Type & t8,
                                                     const Type & t9,
                                                     const Type & t10,
                                                     const Type & t11,
                                                     const Type & t12,
                                                     const Type & t13,
                                                     const Type & t14,
                                                     const Type & t15) { 
//-------------------------------------------------------------------------------
  std::vector<Type> v = makeVector(t0, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14, t15);
  Type ret = Type::byName(Function::buildTypeName(r,v));
  if ( ret ) return ret;
  else       return (new Function(r, v, typeid(UnknownType)))->type();
}

