//$Id: TypeName.cpp,v 1.1 2004/11/30 22:57:59 roiser Exp $

#include "MLANG/Reflection/TypeName.h"
#include "MLANG/Reflection/Type.h"
#ifdef WIN32
#include "MLANG/HashMap.h"
#else /* !WIN32 */
#include "stl_hash.h"
#endif /* !WIN32 */

using namespace MLANG_Reflection;

//-------------------------------------------------------------------------------
#ifdef WIN32
typedef MLANG_HashMap<const char *, TypeName * > Name2Type;
typedef MLANG_HashMap<const char *, TypeName * > TypeId2Type;
#else /* !WIN32 */
typedef __gnu_cxx::hash_map<const char *, TypeName * > Name2Type;
typedef __gnu_cxx::hash_map<const char *, TypeName * > TypeId2Type;
#endif /* !WIN32 */


static Name2Type &   sTypes();
static TypeId2Type & sTypeInfos();


//-------------------------------------------------------------------------------
Name2Type & sTypes() {
//-------------------------------------------------------------------------------
  static Name2Type m;
  return m;
}


//-------------------------------------------------------------------------------
TypeId2Type & sTypeInfos() {
//-------------------------------------------------------------------------------
  static TypeId2Type m;
  return m;
}

#if !defined (__ACE_INLINE__)
#include "MLANG/Reflection/TypeName.i"
#endif /* __ACE_INLINE__ */

//-------------------------------------------------------------------------------
MLANG_Reflection::TypeName::TypeName( const char * name,
                                  TypeBase * typeBase,
                                  const std::type_info * ti )
//-------------------------------------------------------------------------------
  : m_name( name ),
    m_typeBase( typeBase ) {
  sTypes() [ m_name.c_str() ] = this;
  if ( ti ) sTypeInfos() [ ti->name() ] = this;
}


//-------------------------------------------------------------------------------
MLANG_Reflection::TypeName::~TypeName() {
//-------------------------------------------------------------------------------
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type
MLANG_Reflection::TypeName::byName( const MLANG_String & key ) {
//-------------------------------------------------------------------------------
  Name2Type::iterator it = sTypes().find(key.c_str());
  if( it != sTypes().end() ) return Type( it->second );
  else                       return Type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type
MLANG_Reflection::TypeName::byTypeInfo( const std::type_info & tid ) {
//-------------------------------------------------------------------------------
  TypeId2Type::iterator it = sTypeInfos().find(tid.name());
  if( it != sTypeInfos().end() ) return Type( it->second );
  else                           return Type();
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypeName::type() const {
//-------------------------------------------------------------------------------
  return Type( this );
}


//-------------------------------------------------------------------------------
MLANG_Reflection::Type MLANG_Reflection::TypeName::type( size_t nth ) {
//-------------------------------------------------------------------------------
  if ( nth > sTypes().size() ) return Type(); 
  typedef Name2Type::iterator TypeIter;
  size_t n = 0;
  for ( TypeIter tIter = sTypes().begin(); tIter != sTypes().end(); ++tIter ) {
    if ( n == nth ) { return Type( tIter->second ); }
    else { ++n; }
  }
  return Type();
}


//-------------------------------------------------------------------------------
size_t MLANG_Reflection::TypeName::typeCount() {
//-------------------------------------------------------------------------------
  return sTypes().size();
}

