//$Id: TypeName.h,v 1.1 2004/11/30 22:57:58 roiser Exp $

#ifndef MLANG_REFLECTION_TYPENAME_H
#define MLANG_REFLECTION_TYPENAME_H
#include <typeinfo>
// Include files
#include "MLANG/Export.h"
#include "MLANG/String.h"

namespace MLANG_Reflection

{

    // forward declarations 
    class MLANG_Export TypeBase;
    class MLANG_Export Type;

    /** 
     * class TypeName TypeName.h MLANG/Reflection/TypeName.h
     * @author Stefan Roiser
     * @date 06/11/2004
     * @ingroup Ref
     */
    class MLANG_Export TypeName {

      friend class Type;
      friend class TypeBase;

    public:

      /** default constructor */
      TypeName( const char * name,
                TypeBase * typeBase,
                const std::type_info * ti = 0 );

      
      /**
       * byName will look for a type given as a string and return a pointer to
       * its reflexion type
       * @param  key fully qualified name of the type as string
       * @return pointer to type or 0 if none is found
       */
      static Type byName( const MLANG::String & key );
      
      
      /**
       * byTypeId will look for a type given as a string representation of a
       * type_info and return a pointer to its reflexion type
       * @param  tid string representation of the type_info type
       * @return pointer to type or 0 if none is found
       */
      static Type byTypeInfo( const std::type_info & tid );


      /**
       * name will return the string representation of the type (unique)
       * @return type name as a string
       */
      const MLANG::String & name() const;


      /** 
       * type returns the type object of this TypeName
       * @return corresponding Type to this TypeName
       */
      Type type() const;


      /**
       * type will return a pointer to the nth Type in the system
       * @param  nth number of type to return
       * @return pointer to nth Type in the system
       */
      static Type type( size_t nth );


      /**
       * typeCount will return the number of currently defined types in
       * the system
       * @return number of currently defined types
       */
      static size_t typeCount();

    private:

      /** destructor */
      virtual ~TypeName();
      
    private:

      /** the name of the type */
      MLANG::String m_name;


      /**
       * pointer to a TypebeBase if the type is implemented
       * @label type base
       * @link aggrationByValue
       */
      TypeBase * m_typeBase;

    }; // class TypeName

} // namespace reflex


#if defined (__ACE_INLINE__)
#include "MLANG/Reflection/TypeName.i"
#endif /* __ACE_INLINE__ */


#endif // MLANG_REFLECTION_TYPENAME_H
