//$Id: Union.h,v 1.4 2004/11/30 22:57:58 roiser Exp $

#ifndef SEAL_REFLECT_UNION_H
#define SEAL_REFLECT_UNION_H

// Include Files
#include "MLANG/Reflection/TypeBase.h"
#include <vector>

namespace MLANG_Reflection
{

    // forward declarations
    class UnionInfo;
    
    /**
     * @class Union Union.h MLANG/Reflection/Union.h
     * @author Stefan Roiser
     * @date 24/11/2003
     * @ingroup Ref
     */
    class Union : public TypeBase {

    public:

      /** constructor */
      Union( const char * type,
             size_t size,
             const std::type_info & typeInfo,
             const MLANG_String & unionInfos ) ;


      /** destructor */
      virtual ~Union();


      /**
       * unionInfo will return the nth union info
       * @param  nth union info
       * @return nth union info
       */
      UnionInfo unionInfo( size_t nth ) const;


      /**
       * unionInfoCount will return the number of union types
       * @return number of union types
       */
      size_t unionInfoCount() const;


      /**
       * fullName will return the full qualified name and types of the union
       * @param  typedefexp defines whether typedefs shall be expanded
       * @return full name of the union
       */
      MLANG_String fullName( bool typedefexp = false ) const;

    private:

      /** container with infos about different union type possibilities */
      std::vector < UnionInfo * > m_unions;

    }; // class Union

} // namespace MLANG_Reflection

#include "MLANG/Reflection/UnionInfo.h"

//-------------------------------------------------------------------------------
inline size_t MLANG_Reflection::Union::unionInfoCount() const {
//-------------------------------------------------------------------------------
  return m_unions.size();
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::UnionInfo MLANG_Reflection::Union::unionInfo( size_t nth ) const {
//-------------------------------------------------------------------------------
  if ( nth < m_unions.size() ) { return *(m_unions[ nth ]); }
  return UnionInfo();
}


#endif //SEAL_REFLECT_UNION_H

