//$Id: UnionInfo.h,v 1.1 2004/11/30 22:57:58 roiser Exp $

#ifndef SEAL_REFLECT_UNIONINFO_H
#define SEAL_REFLECT_UNIONINFO_H

// Include Files
#include "MLANG/Reflection/Type.h"
#include <string>

namespace MLANG_Reflection
{

    // forward declarations


    /**
     * @class UnionInfo Union.h MLANG/Reflection/Union.h
     * @author Stefan Roiser
     * @date 07/11/2004
     * @ingroup Ref
     */
    class UnionInfo {
      
    private:
      
      struct Info {                       // info about one union
        Info( const Type & type, 
              const MLANG_String & name ); // constructor
        virtual ~Info() {}                // destructor 
        Type m_type;                      // type of union member
        MLANG_String m_name;               // name of union member
      };                                  // class Info

    public:

      /** default constructor */
      UnionInfo();


      /** constructor */
      UnionInfo( const Type & type,
                 const MLANG_String & name );


      /** destructor */
      virtual ~UnionInfo();


      /** return the type of the union member */
      Type & type() const;


      /** return the name of the union member */
      const MLANG_String & name() const;

    private:

      /** pointer to union info object */
      Info * m_unionInfo;

    }; // class UnionInfo

} // namespace MLANG_Reflection

#include "MLANG/Reflection/Union.h"

//-------------------------------------------------------------------------------
inline MLANG_Reflection::UnionInfo::UnionInfo()
//-------------------------------------------------------------------------------
  : m_unionInfo( 0 ) {}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::UnionInfo::UnionInfo( const Type & type,
                                           const MLANG_String & name ) 
//-------------------------------------------------------------------------------
  : m_unionInfo( new Info( type, name )) {}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::UnionInfo::~UnionInfo() {
//-------------------------------------------------------------------------------
  delete m_unionInfo;
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::Type & MLANG_Reflection::UnionInfo::type() const {
//-------------------------------------------------------------------------------
  return m_unionInfo->m_type;
}


//-------------------------------------------------------------------------------
inline const MLANG_String & MLANG_Reflection::UnionInfo::name() const {
//-------------------------------------------------------------------------------
  return m_unionInfo->m_name;
}


//-------------------------------------------------------------------------------
inline MLANG_Reflection::UnionInfo::Info::Info( const Type & type, 
                                            const MLANG_String & name ) 
//-------------------------------------------------------------------------------
  : m_type( type ),
    m_name( name ) {}



#endif //SEAL_REFLECT_UNION_H

