//$Id: Volatile.h,v 1.6 2004/10/07 10:03:42 roiser Exp $
#ifndef MLANG_REFLECTION_VOLATILE_H
#define MLANG_REFLECTION_VOLATILE_H

// Include files

namespace MLANG_Reflection

{


    /**
     * @class Volatile Volatile.h Reflection/Volatile.h
     * @author Stefan Roiser
     * @date 15/06/2004
     */
    class Volatile : public TypeBase {
    public:

      /** Constructor */
      Volatile( const Type * volatileType , const std::type_info & );


      /** Destructor */
      virtual ~Volatile() {}


      /**
       * type will return the pointer to the type the cv is pointing to
       * @return pointer to Type
       */
      const Type * toType() const;


      /** static funtion that composes the type name */
      static MLANG_String buildTypeName(const Type* constType );

    private:

      /** 
       * the pointer to the corresponding type 
       * @label volatile type
       * @link aggregation
       * @supplierCardinality 1
       */
      const Type * m_volatileType;

    }; // class Volatile
} //namespace MLANG_Reflection

//-------------------------------------------------------------------------------
inline MLANG_Reflection::Volatile::Volatile( const Type * volatileType, const std::type_info & ti )
//-------------------------------------------------------------------------------
: TypeBase( buildTypeName(volatileType).c_str(), volatileType->sizeOf(), VOLATILE, ti ), 
    m_volatileType( volatileType ) {}

//-------------------------------------------------------------------------------
inline const MLANG_Reflection::Type * MLANG_Reflection::Volatile::toType() const {
//-------------------------------------------------------------------------------
  return m_volatileType;
}

//-------------------------------------------------------------------------------
inline MLANG_String MLANG_Reflection::Volatile::buildTypeName( const Type* volatileType )
//-------------------------------------------------------------------------------
{
  if ( volatileType->isReference() || volatileType->isPointer() ) {
    return volatileType->name() + " volatile";
  }
  else {
    return MLANG_String("volatile ") + volatileType->name();
  }
}

#endif // MLANG_REFLECTIONION_VOLATILE_H

