#include "mbase/string.h"
#include "mbase/config.h"
#include "MLANG/String.h"
#include "MLANG/StringBuffer.h"

using namespace MLANG;

#if !defined (__ACE_INLINE__)
#include "MLANG/String.i"
#endif /* __ACE_INLINE__ */

int String::npos = -1;

String::String(size_t maxlen, size_t incrlen)
  : maxlen_(maxlen),
    incrlen_(incrlen),
    len_(0)

{
   buf_ = (char*) malloc(sizeof(char)* (maxlen_ + 1));
   buf_[0] = '\0';
}

String::String(const String& ts)
  : maxlen_(ts.len_),
    incrlen_(ts.incrlen_),
    len_(ts.len_)

{
   buf_ = (char*) malloc(sizeof(char)*(maxlen_ + 1));
   memcpy(buf_, ts.buf_, ts.len_);
   buf_[ts.len_] = '\0';
}

String::String(char c)
  : maxlen_(1),
    incrlen_(1),
    len_(1)

{
   buf_ = (char*) malloc(sizeof(char)*(maxlen_ + 1));
   buf_[0] = c;
   buf_[1] = '\0';
}

String::String(const char* s)
  : maxlen_(strlen(s)),
    incrlen_(1),
    len_(maxlen_)
  
{
   buf_ = (char*) malloc(sizeof(char)*(maxlen_ + 1));
   strcpy(buf_, s);
}

String::String(const char* buf, size_t len)
  : maxlen_(len),
    incrlen_(1),
    len_(len)
  
{
   buf_ = (char*) malloc(sizeof(char)*(len_ + 1));
   memcpy(buf_, buf, len);
   buf_[len] = '\0';
}

String& String::operator=(const String& ts)

{
   maxlen_ = ts.len_;
   len_ = ts.len_;
   incrlen_ = ts.incrlen_;
   buf_ = (char*) realloc(buf_,sizeof(char)*(maxlen_ + 1));
   assert(buf_!=0);
   strcpy(buf_, ts.buf_);
   return *this;
}

String& String::operator+=(const String& ts)

{
   int len = this->len_;
   maxlen_ = this->len_ + ts.len_;
   len_ = this->maxlen_;
   buf_ = (char*) realloc(buf_,sizeof(char)*(len_ + 1));
   strcpy(&buf_[len], ts.buf_);
   return *this;
}

String& String::operator=(const char* s)

{
   maxlen_ = strlen(s);
   len_ = maxlen_;
   incrlen_ = 1;
   buf_ = (char*) realloc(buf_,sizeof(char)*(len_ + 1));
   strcpy(buf_, s);
   return *this;
}

void String::clear()

{
   len_ = 0;
   if (buf_)
    {
      buf_[0] = '\0';
    }
}

String String::trim() const

{
   int j;
   StringBuffer sb(len_);
   for (j=0; j<(int)len_ && isspace(buf_[j]); j++)
    {
       sb.append (buf_[j]);  
    }
   int i;
   for (i=len_ - 1; i>=j && isspace(buf_[i]); i--)
    {
       sb.append (buf_[i]);  
    }
   return sb.toString();
}

int String::lastIndexOf (const String& sep) const

{
   if (sep.length() == 0
   ||  sep.length() > length())
    {
      return npos;
    }
   int pos = length() - sep.length();
   const char *p = sep.buf_;
   const char *q = buf_ + pos;
   while (!(pos<0))
    {
       if (memcmp(p, q, sep.length())==0)
          return pos;
       q--;
       pos--;
    }
   return npos;
}

bool String::endsWith (const String& sep) const

{
   if (sep.length() == 0
   ||  sep.length() > length())
    {
      return false;
    }
   int pos = length() - sep.length();
   const char *p = sep.buf_;
   const char *q = buf_ + pos;
   if (memcmp(p, q, sep.length())==0)
      return true;
   return false;
}

bool String::endsWith (const char* sep) const

{
   size_t len = strlen(sep);
   if (len == 0
   ||  len > length())
    {
      return false;
    }
   int pos = length() - len;
   const char *p = sep;
   const char *q = buf_ + pos;
   while (*q==*p && *p!='\0')
    {
      p++; q++;
    }
   if (*p=='\0')
      return true;
   else
    {
      fprintf(stderr,"String::endsWith: \"%s\"==\"%s\" differ '%c'!=%c'\n",
              buf_, sep, *p, *q);
      return false;
    }
}

int String::indexOf (
   const String& sep,
   int pos
) const

{
   if (sep.length() == 0
   ||  sep.length() > length())
    {
      return npos;
    }
   int d = sep.length();
   const char *p = sep.buf_;
   const char *q = buf_ + pos;
   while (pos<d)
    {
       if (memcmp(p, q, d)==0)
          return pos;
       q++;
       pos++;
    }
   return npos;
}

int String::indexOf (
   char c,
   int pos
) const

{
   const char *q = buf_ + pos;
   while (pos<(int)len_)
    {
       if (*q++ == c)
          return pos;
       pos++;
    }
   return npos;
}

int String::find_first_of (const String& s, int pos) const

{
   const char *q = buf_ + pos;
   while (pos<len_)
    {
       for (size_t i=0; i<s.length(); i++)
        {
           if (*q==s[pos])
            {
              return pos;
            }
        }
       q++;
       pos++;
    }
   return npos;
}

int String::find_first_of (const char *s, int pos) const

{
   const char *q = buf_ + pos;
   while (pos<(int)len_)
    {
       for (const char *p = s; *p!='\0'; p++)
        {
           if (*q == *p)
            {
              return pos;
            }
        }
       q++;
       pos++;
    }
   return npos;
}

int String::find_first_of (const char c, int pos) const

{
   return indexOf (c, pos);
}

bool String::startsWith (
   const char *o
) const

{
   const char *p = o;
   const char *q = buf_;
   while (*q == *p && *p !='\0')
    {
     p++; q++;
    }
   if (*p=='\0')
      return true;
   else
    {
      return false;
    }
}

bool String::startsWith (
   const String& s
) const

{
   return startsWith (s);
}


String String::substring (
   int begin,
   int end
) const

{
   int len;
   if (end == -1)
    {
      len = len_ - begin;
    }
   else
    {
      if (end >= (int)len_)
         end = len_ - 1;
      if (begin < 0)
         begin = 0;

     len = end - begin;
     if (end < begin)
        len =0;
   }
   return String (buf_ + begin, (size_t) len);
}

String String::toUpperCase() const

{
   String temp = *this;
   upcase(temp);
   return temp;
}

String String::toLowerCase() const

{
   String temp = *this;
   downcase(temp);
   return temp;
}

String::~String()

{
   if (buf_!=0)
    {
      free(buf_);
    }
}

void String::append(int c)

{
   if (len_==maxlen_)
    {
      char* new_buf = (char*) realloc(buf_, sizeof(char)*(maxlen_ + incrlen_ + 1));
      assert (new_buf!=NULL);
      buf_ = new_buf;
      maxlen_ += incrlen_;
    }
   buf_[len_++] = (char) c;
}

void MLANG::downcase (
  String& ps
)

{
  char* p = ps.buf_;
  for (;*p!='\0';p++)
    if (isupper(*p))
      *p = tolower(*p);
}

void MLANG::upcase (
  String& ps
)

{
  char* p = ps.buf_;
  for (;*p!='\0';p++)
    if (islower(*p))
      *p = toupper(*p);
}

String MLANG::operator+ (
   const String& a,
   const String& b
)

{
   String r (a);
   r += b;  
   return r;
}

#ifndef HPUX
String String::valueOf (size_t i, int base)

{
   char s_i[8+8+1];
   sprintf(s_i,"%u",i);
   return s_i;
}
#endif /* !HPUX */

String String::valueOf (int i, int base)

{
   ACE_UNUSED_ARG (base);
   char s_i[8+8+1];
   sprintf(s_i,"%d",i);
   return s_i;
}

String String::valueOf (ACE_UINT64 l, int base)

{
   char s[8+8+1];
   size_t len=0;
   do
    {
       size_t n = l % base;
       l /= base;
       s[len++] = (n > 9) ? (n - 10 + 'A') : (n + '0');
    }
   while (l);
   for (size_t j=0; j<len/2; j++)
    {
       char c = s[j];
       s[j] = s[len - (j + 1)];
       s[len - (j + 1)] = c;
    }
   s[len] = '\0';
   return s;
}

String String::valueOf (bool b)

{
   if (b)
      return ("true");
   else
      return ("false");
}

String String::valueOf (short d)

{
   char s_d[10+10+1];
   sprintf(s_d,"%d", d);
   return s_d;
}

String String::valueOf (long l, int base)

{
   ACE_UNUSED_ARG (base);
   char s_l[10+10+1];
   sprintf(s_l,"%ld", l);
   return s_l;
}

String String::valueOf (float f)

{
   char s_f[10+10+1];
   sprintf(s_f,"%f", f);
   return s_f;
}

String String::valueOf (double d)

{
   char s_d[10+10+1];
   sprintf(s_d,"%g", d);
   return s_d;
}
