#include "mbase/string.h"
#include "mbase/config.h"
#include "MLANG/String.h"
#include "MLANG/StringBuffer.h"

using namespace MLANG;

StringBuffer::StringBuffer(size_t maxlen, size_t incrlen)
  : maxlen_(maxlen),
    incrlen_(incrlen),
    len_(0)

{
   buf_ = (char*) malloc(sizeof(char)* (maxlen_ + 1));
   buf_[0] = '\0';
}

StringBuffer::StringBuffer(const StringBuffer& ts)
  : maxlen_(ts.len_),
    incrlen_(ts.incrlen_),
    len_(ts.len_)

{
   buf_ = (char*) malloc(sizeof(char)*(maxlen_ + 1));
   memcpy(buf_, ts.buf_, ts.len_);
}

StringBuffer::StringBuffer(char c)
  : maxlen_(1),
    incrlen_(1),
    len_(1)

{
   buf_ = (char*) malloc(sizeof(char)*(maxlen_ + 1));
   buf_[0] = c;
   buf_[1] = '\0';
}

StringBuffer::StringBuffer(const char* s)
  : maxlen_(strlen(s)),
    incrlen_(1),
    len_(maxlen_)
  
{
   buf_ = (char*) malloc(sizeof(char)*(maxlen_ + 1));
   memcpy(buf_, s, strlen(s));
}

StringBuffer& StringBuffer::operator=(const StringBuffer& ts)

{
   maxlen_ = ts.len_;
   len_ = ts.len_;
   incrlen_ = ts.incrlen_;
   buf_ = (char*) realloc(buf_,sizeof(char)*(maxlen_ + 1));
   assert(buf_!=0);
   memcpy(buf_, ts.buf_, ts.len_);
   return *this;
}

StringBuffer& StringBuffer::operator+=(const StringBuffer& ts)

{
   size_t len = this->len_;
   maxlen_ = this->len_ + ts.len_;
   len_ = this->maxlen_;
   buf_ = (char*) realloc(buf_,sizeof(char)*(len_ + 1));
   memcpy(&buf_[len], ts.buf_, len);
   return *this;
}

StringBuffer& StringBuffer::operator=(const char* s)

{
   maxlen_ = strlen(s);
   len_ = maxlen_;
   incrlen_ = 1;
   buf_ = (char*) realloc(buf_,sizeof(char)*(len_ + 1));
   memcpy(buf_, s, len_);
   return *this;
}

StringBuffer::~StringBuffer()

{
   if (buf_!=0)
    {
      free(buf_);
    }
}

StringBuffer& StringBuffer::append(int c)

{
   if (len_==maxlen_)
    {
      char* new_buf = (char*) realloc(buf_, sizeof(char)*(maxlen_ + incrlen_ + 1));
      assert (new_buf!=NULL);
      buf_ = new_buf;
      maxlen_ += incrlen_;
    }
   buf_[len_++] = (char) c;
   return *this;
}


StringBuffer& StringBuffer::append(const char *s)

{
   while (*s)
     this->append (*s++);
   return *this;
}


StringBuffer& StringBuffer::append(const String& s)

{
   for (size_t i=0; i<s.length(); i++)
     this->append (s.charAt(i));
   return *this;
}
