#ifndef __MLANG_VECTOR_T_H__
#define __MLANG_VECTOR_T_H__
#include "ace/Vector_T.h"

namespace MLANG
{
template <class T, size_t DEFAULT_SIZE = ACE_VECTOR_DEFAULT_SIZE>
class Vector : public ACE_Vector<T, DEFAULT_SIZE>
{
public:

  /**
   * General constructor.
   *
   * @param init_size Initial size of the vector with the default
   *                  value of DEFAULT_SIZE
   * @param alloc Pointer to an ACE allocator.  If it is NULL then the
   *              default ACE allocator is used
   */
  Vector (const size_t init_size = DEFAULT_SIZE,
	      ACE_Allocator* alloc = 0);
  Vector (const T *t,
		  const size_t n,
	      ACE_Allocator* alloc = 0);
  ~Vector ();
public:
  void ensureCapacity (size_t minSize);
  bool removeElement(T t);
  void appendElement(const T& t)
  { push_back(t); }
  void setElementAt(T obj, size_t i);
};

#if defined (__ACE_INLINE__)
#include "MLANG/Vector_T.i"
#endif /* __ACE_INLINE__ */
}

#define MLANG_Vector MLANG::Vector

#if defined (ACE_TEMPLATES_REQUIRE_SOURCE)
#include "MLANG/Vector_T.cpp"
#endif /* ACE_TEMPLATES_REQUIRE_SOURCE */

#if defined (ACE_TEMPLATES_REQUIRE_PRAGMA)
#pragma implementation ("MLANG/Vector_T.cpp")
#endif /* ACE_TEMPLATES_REQUIRE_PRAGMA */

#endif /* __LANG_VECTOR_T_H__ */
