#ifndef __MPARSER_AST_H__
#define __MPARSER_AST_H__
#include "ace/Vector_T.h"
#include "MLANG/String.h"
#include "MLANG/Object.h"
#include "MLANG/Class.h"
#include "MParser/Token.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/collections/AST.java#1 $
 */

class MParser_AST;
typedef class MParser_AST *MParser_AST_ptr;
typedef MLANG_SmartPtr<MParser_AST> MParser_AST_var;

typedef ACE_Vector<MParser_AST_var> ASTEnumeration;

/** Minimal AST node interface used by ANTLR AST generation
 * and tree-walker.
 */
class MParser_AST : public MLANG_Object

{
public:
   MParser_AST ();
   virtual ~MParser_AST ();

public:
   DECLARE_CLASS_INFO();

   static bool is_nil (MParser_AST_ptr ast)
   { return ast == 0; }

   static MParser_AST_ptr _nil()
   { return 0; }

   static MParser_AST_ptr _duplicate(MParser_AST_ptr ast)
   { return MLANG_acquire(ast); }
    /** Add a (rightmost) child to this node */
public:
   virtual void addChild(const MParser_AST_var& c) = 0;

public:
   virtual bool equals(const MParser_AST_var& t) = 0;

public:
   virtual bool equalsList(const MParser_AST_var& t) = 0;

public:
   virtual bool equalsListPartial(const MParser_AST_var& t) = 0;

public:
   virtual bool equalsTree(const MParser_AST_var& t) = 0;

public:
   virtual bool equalsTreePartial(const MParser_AST_var& t) = 0;

public:
   virtual ASTEnumeration findAll(const MParser_AST_var& tree) = 0;

public:
   virtual ASTEnumeration findAllPartial(const MParser_AST_var& subtree) = 0;

    /** Get the first child of this node; null if no children */
public:
   virtual MParser_AST_var getFirstChild() = 0;

    /** Get	the next sibling in line after this one */
public:
   virtual MParser_AST_var getNextSibling() = 0;

    /** Get the token text for this node */
public:
   virtual MLANG_String getText() const = 0;

    /** Get the token type for this node */
public:
   virtual int getType() = 0;

    /** @since 2.7.3 Need for error handling */
public:
   virtual int getLine() = 0;

    /** @since 2.7.3 Need for error handling */
public:
   virtual int getColumn() = 0;

	/** Get number of children of this node; if leaf, returns 0 */
public:
   virtual int getNumberOfChildren() = 0;

public:
   virtual void initialize(int t, const MLANG_String& txt) = 0;

public:
   virtual void initialize(const MParser_AST_var& t) = 0;

public:
   virtual void initialize(const MParser_Token_var& t) = 0;

    /** Set the first child of a node. */
public:
   virtual void setFirstChild(const MParser_AST_var& c) = 0;

    /** Set the next sibling after this one. */
public:
   virtual void setNextSibling(const MParser_AST_var& n) = 0;

    /** Set the token text for this node */
public:
   virtual void setText(const MLANG_String& text) = 0;

    /** Set the token type for this node */
public:
   virtual void setType(int ttype) = 0;

public:
   virtual MLANG_String toString() const = 0;

public:
   virtual MLANG_String toStringList() const = 0;

public:
   virtual MLANG_String toStringTree() const = 0;
};

#endif /* __MPARSER_AST_H__ */
