/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/ByteBuffer.java#1 $
 */

/**A Stream of characters fed to the lexer from a InputStream that can
 * be rewound via mark()/rewind() methods.
 * <p>
 * A dynamic array is used to buffer up all the input characters.  Normally,
 * "k" characters are stored in the buffer.  More characters may be stored during
 * guess mode (testing syntactic predicate), or when LT(i>k) is referenced.
 * Consumption of characters is deferred.  In other words, reading the next
 * character is not done by conume(), but deferred until needed by LA or LT.
 * <p>
 *
 * @see antlr.CharQueue
 */
// SAS: added this class to handle Binary input w/ FileInputStream
#include "MIO/IOException.h"
#include "MParser/CharStreamIOException.h"
#include "MParser/ByteBuffer.h"

MParser_ByteBuffer::MParser_ByteBuffer(const MIO_InputStream_var& input)
   : _input(input)

{
}

void MParser_ByteBuffer::fill(int amount)

{
   try
    {
       syncConsume();
       // Fill the buffer sufficiently to hold needed characters
       while (_queue._nbrEntries < amount + _markerOffset)
        {
           // Append the next character
           _queue.append(_input->read());
        }
    }
   catch (MIO_IOException& io)
    {
        throw MParser_CharStreamIOException(io);
    }
}

MParser_ByteBuffer::~MParser_ByteBuffer()

{
}
