#include "ace/Basic_Types.h"
#include "MParser/CharQueue.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/CharQueue.java#1 $
 */

/** A circular buffer object used by CharBuffer */
MParser_CharQueue::MParser_CharQueue (int minSize) 
   : _length(0),
     _buffer(0)

{
    // Find first power of 2 >= to requested size
    int size;
    if ( minSize<0 )
     {
       init(16); // pick some value for them
       return;
     }

    // check for overflow
    if (minSize>=(ACE_INT32_MAX/2) )
     {
        init(ACE_INT32_MAX); // wow that's big.
        return;
     }
    for (size = 2; size < minSize; size *= 2)
     {
     }
    init(size);
}

/** Add token to end of the queue
 * @param tok The token to add
 */
void MParser_CharQueue::append(int tok)

{
    if (_nbrEntries == _length)
     {
        expand();
     }
    _buffer[(_offset + _nbrEntries) & _sizeLessOne] = tok;
    _nbrEntries++;
}

/** Fetch a token from the queue by index
 * @param idx The index of the token to fetch, where zero is the token at the front of the queue
 */
int MParser_CharQueue::elementAt(int idx)

{
    return _buffer[(_offset + idx) & _sizeLessOne];
}

/** Expand the token buffer by doubling its capacity */
void MParser_CharQueue::expand()

{
   int *newBuffer = (int*) malloc (2 * _length * sizeof(int));
   // Re-initialize with new contents, keep old nbrEntries
   for (int i=0; i<_length; i++)
       newBuffer[i] = elementAt(i);

   free(_buffer);
   _buffer = newBuffer;
   _length *= 2;
   _sizeLessOne = _length - 1;
   _offset = 0;
}

/** Initialize the queue.
 * @param size The initial size of the queue
 */
void MParser_CharQueue::init(int size)

{
   // Allocate buffer
   _buffer = (int*) realloc (_buffer, size * sizeof(int));
   // Other initialization
   _length = size;
   _sizeLessOne = size - 1;
   _offset = 0;
   _nbrEntries = 0;
}

/** Clear the queue. Leaving the previous buffer alone.
 */
void MParser_CharQueue::reset()

{
   _offset = 0;
   _nbrEntries = 0;
}

/** Remove char from front of queue */
void MParser_CharQueue::removeFirst()

{
   _offset = (_offset + 1) & _sizeLessOne;
   _nbrEntries--;
}

MParser_CharQueue::~MParser_CharQueue()

{
   if (_buffer!=0)
    {
      free(_buffer);
      _buffer = 0;
    }
}
