
/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/CharQueue.java#1 $
 */

#ifndef __MPARSER_CHARQUEUE_H__
#define __MPARSER_CHARQUEUE_H__

/** A circular buffer object used by CharBuffer */
class MParser_CharQueue

{ 
protected:
   /** Length of buffer */
   int _length;
   /** Physical circular buffer of tokens */
   int *_buffer;
   /** _length-1 for quick modulos */
   int _sizeLessOne;
   /** physical index of front token */
   int _offset;
public:
    /** number of tokens in the queue */
   int _nbrEntries;

public:

   MParser_CharQueue(int minSize = 1);

   ~MParser_CharQueue();

public:

   /** Add token to end of the queue
    * @param tok The token to add
    */
   void append(int tok);

   /** Fetch a token from the queue by index
    * @param idx The index of the token to fetch, where zero is the token at the front of the queue
    */
   int elementAt(int idx);

private:

   /** Expand the token buffer by doubling its capacity */
   void expand();

public:

   /** Initialize the queue.
    * @param size The initial size of the queue
    */
   void init(int size);

    /** Clear the queue. Leaving the previous buffer alone.
     */
   void reset();

    /** Remove char from front of queue */
   void removeFirst();
};

#endif /* __MPARSER_CHARQUEUE_H__ */
