/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/InputBuffer.java#1 $
 */

// SAS: Added this class to genericise the input buffers for scanners
//      This allows a scanner to use a binary (FileInputStream) or
//      text (FileReader) stream of data; the generated scanner
//      subclass will define the input stream
//      There are two subclasses to this: CharBuffer and ByteBuffer

#ifndef __MPARSER_INPUTBUFFER_H__
#define __MPARSER_INPUTBUFFER_H__
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MLANG/Object.h"
#include "MParser/CharQueue.h"


/**A Stream of characters fed to the lexer from a InputStream that can
 * be rewound via mark()/rewind() methods.
 * <p>
 * A dynamic array is used to buffer up all the input characters.  Normally,
 * "k" characters are stored in the buffer.  More characters may be stored during
 * guess mode (testing syntactic predicate), or when LT(i>k) is referenced.
 * Consumption of characters is deferred.  In other words, reading the next
 * character is not done by conume(), but deferred until needed by LA or LT.
 * <p>
 *
 * @see MParser_CharQueue
 */
namespace MParser
{

class InputBuffer;
typedef class InputBuffer *InputBuffer_ptr;
typedef MLANG_SmartPtr<InputBuffer> InputBuffer_var;

class InputBuffer : public MLANG_Object

{
protected:
    // Number of active markers
    int _nMarkers;

    // Additional offset used when markers are active
    int _markerOffset;

    // Number of calls to consume() since last LA() or LT() call
    int _numToConsume;

    // Circular queue
    MParser_CharQueue _queue;

    /** Create an input buffer */
public:

    InputBuffer();

    virtual ~InputBuffer();

public:

   static bool is_nil (InputBuffer_ptr buffer)
   { return buffer == 0; }

   static InputBuffer_ptr _nil()
   { return 0; }

   static InputBuffer_ptr _duplicate(InputBuffer_ptr buffer)
   { return MLANG_acquire(buffer); }

    /** This method updates the state of the input buffer so that
     *  the text matched since the most recent mark() is no longer
     *  held by the buffer.  So, you either do a mark/rewind for
     *  failed predicate or mark/commit to keep on parsing without
     *  rewinding the input.
     */
    void commit();

    /** Mark another character for deferred consumption */
    void consume();

    /** Ensure that the input buffer is sufficiently full */
    virtual void fill(int amount) = 0;

    MLANG_String getLAChars();

    MLANG_String getMarkedChars();

    bool isMarked();

    /** Get a lookahead character */
    int LA(int i);

    /**Return an integer marker that can be used to rewind the buffer to
     * its current state.
     */
    int mark();

    /**Rewind the character buffer to a marker.
     * @param mark Marker returned previously from mark()
     */
    void rewind(int mark);

    /** Reset the input buffer
     */
    void reset();

    /** Sync up deferred consumption */
protected:

    void syncConsume();
};
}

typedef MParser::InputBuffer MParser_InputBuffer;
typedef MParser_InputBuffer *MParser_InputBuffer_ptr;
typedef MLANG_SmartPtr<MParser_InputBuffer> MParser_InputBuffer_var;

#endif /* __MPARSER_INPUTBUFFER_H__ */
