#include "MParser/System.h"
#include "MParser/LLkParser.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/LLkParser.java#1 $
 */

/**An LL(k) parser.
 *
 * @see antlr.Token
 * @see antlr.TokenBuffer
 */
MParser_LLkParser::MParser_LLkParser (
   int k
)
  : k_(k)

{
}

MParser_LLkParser::MParser_LLkParser (
   const MParser_ParserSharedInputState_var& state,
   int k
) : super(state),
    k_(k)

{
}

MParser_LLkParser::MParser_LLkParser (
   const MParser_TokenBuffer_var& tokenBuf,
   int k
) : k_(k)

{
   setTokenBuffer(tokenBuf);
}

MParser_LLkParser::MParser_LLkParser (
   const MParser_TokenStream_var& lexer,
   int k
) : k_(k)

{
   setTokenBuffer(new MParser_TokenBuffer(lexer));
}

/**Consume another token from the input stream.  Can only write sequentially!
 * If you need 3 tokens ahead, you must consume() 3 times.
 * <p>
 * Note that it is possible to overwrite tokens that have not been matched.
 * For example, calling consume() 3 times when k=2, means that the first token
 * consumed will be overwritten with the 3rd.
 */
void MParser_LLkParser::consume()

{
   inputState_->input()->consume();
}

int MParser_LLkParser::LA(int i)

{
   assert(this!=0);
   return inputState_->input()->LA(i);
}

MParser_Token_var MParser_LLkParser::LT(int i)

{
   return inputState_->input()->LT(i);
}

void MParser_LLkParser::trace (
   const MLANG_String& ee,
   const MLANG_String& rname
)

{
   traceIndent();
   SYSTEM->out()->print(ee + rname + ((inputState_->guessing() > 0)?"; [guessing]":"; "));
   for (int i = 1; i <= k(); i++)
    {
       if (i != 1)
        {
          SYSTEM->out()->print(", ");
        }
       if (!MParser_Token::is_nil(LT(i)))
        {
          SYSTEM->out()->print("LA(" +MLANG_String::valueOf(i) + ")==" + LT(i)->getText());
        }
       else
        {
          SYSTEM->out()->print("LA(" +MLANG_String::valueOf(i) + ")==null");
        }
    }
   SYSTEM->out()->println("");
}

void MParser_LLkParser::traceIn(const MLANG_String& rname)

{
   traceDepth_ += 1;
   trace("> ", rname);
}

void MParser_LLkParser::traceOut(const MLANG_String& rname)

{
   trace("< ", rname);
   traceDepth_ -= 1;
}

MParser_LLkParser::~MParser_LLkParser ()

{
}
