#include "MParser/MismatchedCharException.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/MismatchedCharException.java#1 $
 */

/**
 * Returns a clean error message (no line number/column information)
 */
MLANG_String MParser_MismatchedCharException::getMessage() const

{
   MLANG_StringBuffer sb;

   switch (mismatchType_)
    {
    case CHAR:
         sb.append("expecting ");   appendCharName(sb, expecting_);
         sb.append(", found ");     appendCharName(sb, foundChar_);
         break;
    case NOT_CHAR:
         sb.append("expecting anything but '");
         appendCharName(sb, expecting_);
         sb.append("'; got it anyway");
         break;
    case RANGE:
    case NOT_RANGE:
         sb.append("expecting token ");
         if (mismatchType_ == NOT_RANGE)
            sb.append("NOT ");
         sb.append("in range: ");
         appendCharName(sb, expecting_);
         sb.append("..");
         appendCharName(sb, upper_);
         sb.append(", found ");
         appendCharName(sb, foundChar_);
         break;
    case SET:
    case NOT_SET: {
         sb.append("expecting ");
         sb.append((mismatchType_ == NOT_SET) ? "NOT " : "");
         sb.append("one of (");
         ACE_Vector<size_t> elems = set_.toArray();
         for (size_t i = 0; i < elems.size(); i++)
          {
             appendCharName(sb, elems[i]);
          }
         sb.append("), found ");
         appendCharName(sb, foundChar_);
         break;
         }
    default:
         sb.append(this->MParser_RecognitionException::getMessage().c_str());
         break;
    }

    return sb.toString().c_str();
}

/** Append a char to the msg buffer.  If special,
 *  then show escaped version
 */
void MParser_MismatchedCharException::appendCharName (
   MLANG_StringBuffer& sb,
   int c
)

{
   switch (c)
   {
   case 65535:
	    // 65535 = (char) -1 = EOF
            sb.append("'<EOF>'");
            break;
   case '\n':
            sb.append("'\\n'");
            break;
   case '\r':
            sb.append("'\\r'");
            break;
   case '\t':
            sb.append("'\\t'");
            break;
   default:
            sb.append('\'');
            sb.append((char) c);
            sb.append('\'');
            break;
   }
}
