#ifndef __MPARSER_MISMATCHEDCHAREXCEPTION_H__
#define __MPARSER_MISMATCHEDCHAREXCEPTION_H__
#include "MParser/BitSet.h"
#include "MLANG/String.h"
#include "MParser/CharScanner.h"
#include "MParser/RecognitionException.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/MismatchedCharException.java#1 $
 */
class MParser_MismatchedCharException : public MParser_RecognitionException
{
protected:
    enum
    {
       // Types of chars
       CHAR = 1,
       NOT_CHAR = 2,
       RANGE = 3,
       NOT_RANGE = 4,
       SET = 5,
       NOT_SET = 6
    };
public:
    // One of the above
    int mismatchType_;

    // what was found on the input stream
    int foundChar_;

    // For CHAR/NOT_CHAR and RANGE/NOT_RANGE
    int expecting_;

    // For RANGE/NOT_RANGE (expecting is lower bound of range)
    int upper_;

    // For SET/NOT_SET
    MParser_BitSet set_;

    // who knows...they may want to ask scanner questions
    MParser_CharScanner_ptr scanner_;

    /**
     * MismatchedCharException constructor comment.
     */
public:

    // Expected range / not range
    MParser_MismatchedCharException (
       char c, 
       char lower, 
       char upper, 
       bool matchNot, 
       MParser_CharScanner_ptr scanner
     ) : MParser_RecognitionException("Mismatched char", scanner->getFilename().c_str(), scanner->getLine(), scanner->getColumn()),
         mismatchType_ (matchNot ? NOT_RANGE : RANGE),
         foundChar_ (c),
         expecting_ (lower),
         upper_ (upper),
         scanner_ (scanner)
     {
     }

    // Expected token / not token
    MParser_MismatchedCharException (
        char c, 
        char expecting, 
        bool matchNot,
        MParser_CharScanner_ptr scanner
     ) : MParser_RecognitionException("Mismatched char", scanner->getFilename().c_str(), scanner->getLine(), scanner->getColumn()),
         mismatchType_ (matchNot ? NOT_CHAR : CHAR),
         foundChar_ (c),
         expecting_ (expecting),
         scanner_ (scanner)
    {
    }

    // Expected BitSet / not BitSet
    MParser_MismatchedCharException (
        char c, 
        const MParser_BitSet& set,
        bool matchNot, 
        MParser_CharScanner_ptr scanner
     ) : MParser_RecognitionException("Mismatched char", scanner->getFilename().c_str(), scanner->getLine(), scanner->getColumn()),
         mismatchType_ (matchNot ? NOT_SET : SET),
         foundChar_ (c),
         set_ (set),
         scanner_ (scanner)
    {
    }

    /**
     * Returns a clean error message (no line number/column information)
     */
    MLANG_String getMessage() const;

    /** Append a char to the msg buffer.  If special,
     *  then show escaped version
     */
private:
   static void appendCharName(MLANG_StringBuffer& sb, int c);
};

#endif /* __MPARSER_MISMATCHEDCHAREXCEPTION_H__ */
