#include "MLANG/StringBuffer.h"
#include "MParser/MismatchedTokenException.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/MParser_MismatchedTokenException::MParser_MismatchedTokenException.java#1 $
 */

/** Looking for AST wildcard, didn't find it */
MParser_MismatchedTokenException::MParser_MismatchedTokenException()
  : super("Mismatched Token: expecting any AST node", "<AST>", -1, -1)

{
}

// Expected range / not range
MParser_MismatchedTokenException::MParser_MismatchedTokenException (
   const ACE_Vector<MLANG_String>& tokenNames, 
   MParser_AST_ptr node, 
   int lower, 
   int upper, 
   bool matchNot
) : super("Mismatched Token", "<AST>", 
          MParser_AST::is_nil(node)? -1:node->getLine(), 
          MParser_AST::is_nil(node)? -1:node->getColumn())

{
   tokenNames_ = tokenNames;
   node_ = MParser_AST::_duplicate(node);
   if (MParser_AST::is_nil(node))
    {
      tokenText_ = "<empty tree>";
    }
   else
    {
      tokenText_ = node->toString();
    }
   mismatchType_ = matchNot ? NOT_RANGE : RANGE;
   expecting_ = lower;
   upper_ = upper;
}

// Expected token / not token
MParser_MismatchedTokenException::MParser_MismatchedTokenException (
   const ACE_Vector<MLANG_String>& tokenNames, 
   MParser_AST_ptr node, 
   int expecting, 
   bool matchNot
) : super("Mismatched Token", "<AST>",
          MParser_AST::is_nil( node) ? -1:node->getLine(),
          MParser_AST::is_nil(node) ? -1:node->getColumn())

{
   tokenNames_ = tokenNames;
   node_ = MParser_AST::_duplicate(node);
   if (MParser_AST::is_nil(node))
    {
      tokenText_ = "<empty tree>";
    }
   else
    {
      tokenText_ = node->toString();
    }
   mismatchType_ = matchNot ? NOT_TOKEN : TOKEN;
   expecting_ = expecting;
}

// Expected BitSet / not BitSet
MParser_MismatchedTokenException::MParser_MismatchedTokenException (
   const ACE_Vector<MLANG_String>& tokenNames,
   MParser_AST_ptr node, 
   const MParser_BitSet& set, 
   bool matchNot
) : super("Mismatched Token", "<AST>", 
          MParser_AST::is_nil(node)? -1:node->getLine(), 
          MParser_AST::is_nil(node)? -1:node->getColumn())

{
   tokenNames_ = tokenNames;
   node_ = MParser_AST::_duplicate(node);
   if (MParser_AST::is_nil(node))
    {
      tokenText_ = "<empty tree>";
    }
   else
    {
      tokenText_ = node->toString();
    }
   mismatchType_ = matchNot ? NOT_SET : SET;
   set_ = set;
}

// Expected range / not range
MParser_MismatchedTokenException::MParser_MismatchedTokenException (
   const ACE_Vector<MLANG_String>& tokenNames,
   MParser_Token_ptr token, 
   int lower, 
   int upper, 
   bool matchNot, 
   const MLANG_String& fileName
) : super("Mismatched Token", fileName.c_str(), token->getLine(), token->getColumn())

{
   tokenNames_ = tokenNames;
   token_ = MParser_Token::_duplicate(token);
   tokenText_ = token->getText();
   mismatchType_ = matchNot ? NOT_RANGE : RANGE;
   expecting_ = lower;
   upper_ = upper;
}

// Expected token / not token
MParser_MismatchedTokenException::MParser_MismatchedTokenException (
   const ACE_Vector<MLANG_String>& tokenNames,
   MParser_Token_ptr token, 
   int expecting, 
   bool matchNot, 
   const MLANG_String& fileName
) : super("Mismatched Token", fileName.c_str(), token->getLine(), token->getColumn())

{
   tokenNames_ = tokenNames;
   token_ = MParser_Token::_duplicate(token);
   tokenText_ = token_->getText();
   mismatchType_ = matchNot ? NOT_TOKEN : TOKEN;
   expecting_ = expecting;
}

    // Expected BitSet / not BitSet
MParser_MismatchedTokenException::MParser_MismatchedTokenException (
   const ACE_Vector<MLANG_String>& tokenNames, 
   MParser_Token_ptr token, 
   const MParser_BitSet& set, 
   bool matchNot, 
   const MLANG_String& fileName
) : super("Mismatched Token", fileName.c_str(), token->getLine(), token->getColumn())

{
   tokenNames_ = tokenNames;
   token_ = MParser_Token::_duplicate(token);
   tokenText_ = token->getText();
   mismatchType_ = matchNot ? NOT_SET : SET;
   set_ = set;
}

/**
 * Returns a clean error message (no line number/column information)
 */
MLANG_String MParser_MismatchedTokenException::getMessage() const

{
   MLANG_StringBuffer sb;

   switch (mismatchType_)
    {
    case TOKEN:
         sb.append("expecting " + tokenName(expecting_) + ", found '" + tokenText_ + "'");
         break;
    case NOT_TOKEN:
         sb.append("expecting anything but " + tokenName(expecting_) + "; got it anyway");
        break;
   case RANGE:
        sb.append("expecting token in range: " + tokenName(expecting_) + ".." + tokenName(upper_) + ", found '" + tokenText_ + "'");
        break;
   case NOT_RANGE:
        sb.append("expecting token NOT in range: " + tokenName(expecting_) + ".." + tokenName(upper_) + ", found '" + tokenText_ + "'");
        break;
   case SET:
   case NOT_SET:
        sb.append("expecting " + MLANG_String(mismatchType_ == NOT_SET ? "NOT " : "") + "one of ("); {
        ACE_Vector<size_t> elems = set_.toArray();
        for (size_t i = 0; i < elems.size(); i++)
         {
            sb.append(" ");
            sb.append(tokenName(elems[i]));
         }
        }
        sb.append("), found '" + tokenText_ + "'");
        break;
   default :
        sb.append(super::getMessage().c_str());
        break;
   }

  return sb.toString().c_str();
}

MLANG_String MParser_MismatchedTokenException::tokenName(int tokenType) const

{
   if (tokenType == MParser_Token::INVALID_TYPE)
    {
      return "<Set of tokens>";
    }
   else if (tokenType < 0 || (size_t)tokenType >= tokenNames_.size())
    {
      return "<" + MLANG_String::valueOf(tokenType) + ">";
    }
   else
    {
      return tokenNames_[tokenType];
    }
}

MParser_MismatchedTokenException::~MParser_MismatchedTokenException()

{
}
