#ifndef __MPARSER_MISMATCHEDTOKENEXCEPTION_H__
#define __MPARSER_MISMATCHEDTOKENEXCEPTION_H__
#include "ace/Vector_T.h"
#include "MLANG/String.h"
#include "MParser/BitSet.h"
#include "MParser/Token.h"
#include "MParser/AST.h"
#include "MParser/RecognitionException.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/MismatchedTokenException.java#1 $
 */

class MParser_MismatchedTokenException : public MParser_RecognitionException
{
public:
    typedef MParser_RecognitionException super;
protected:
    // Token names array for formatting
    ACE_Vector<MLANG_String> tokenNames_;
    // The token that was encountered
public:
   MParser_Token_var token_;
    // The offending AST node if tree walking
public:
   MParser_AST_var node_;

   MLANG_String tokenText_;

    // Types of tokens
public:
   enum
   {
      TOKEN = 1,
      NOT_TOKEN = 2,
      RANGE = 3,
      NOT_RANGE = 4,
      SET = 5,
      NOT_SET = 6
   };
    // One of the above
public:
   int mismatchType_;

    // For TOKEN/NOT_TOKEN and RANGE/NOT_RANGE
public:
   int expecting_;

    // For RANGE/NOT_RANGE (expecting is lower bound of range)
public:
   int upper_;

    // For SET/NOT_SET
public:
   MParser_BitSet set_;

    /** Looking for AST wildcard, didn't find it */
public:
   MParser_MismatchedTokenException();

    // Expected range / not range
public:
   MParser_MismatchedTokenException (
      const ACE_Vector<MLANG_String>& tokenNames, 
      MParser_AST_ptr node, 
      int lower, 
      int upper, 
      bool matchNot
    );

    // Expected token / not token
public:
   MParser_MismatchedTokenException (
      const ACE_Vector<MLANG_String>& tokenNames, 
      MParser_AST_ptr node, 
      int expecting, 
      bool matchNot
    );

    // Expected BitSet / not BitSet
public:
   MParser_MismatchedTokenException (
      const ACE_Vector<MLANG_String>& tokenNames, 
      MParser_AST_ptr node, 
      const MParser_BitSet& set, 
      bool matchNot
    );

    // Expected range / not range
public:
   MParser_MismatchedTokenException (
      const ACE_Vector<MLANG_String>& tokenNames, 
      MParser_Token_ptr token, 
      int lower,
      int upper, 
      bool matchNot, 
      const MLANG_String& fileName
    );

    // Expected token / not token
public:
   MParser_MismatchedTokenException (
      const ACE_Vector<MLANG_String>& tokenNames, 
      MParser_Token_ptr token, 
      int expecting, 
      bool matchNot, 
      const MLANG_String& fileName
    );

    // Expected BitSet / not BitSet
public:
   MParser_MismatchedTokenException (
      const ACE_Vector<MLANG_String>& tokenNames, 
      MParser_Token_ptr token, 
      const MParser_BitSet& set, 
      bool matchNot, 
      const MLANG_String& fileName
    );

   virtual ~MParser_MismatchedTokenException();
    /**
     * Returns a clean error message (no line number/column information)
     */
public:
   MLANG_String getMessage() const;

private:
   MLANG_String tokenName(int tokenType) const;
};

#endif /* __MPARSER_MISMATCHEDTOKENEXCEPTION_H__ */
