#include "MParser/TreeParser.h"
#include "MParser/NoViableAltException.h"

using namespace MParser;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/NoViableAltException.java#1 $
 */

NoViableAltException::NoViableAltException(MParser_AST_ptr t)
  : super("NoViableAlt", "<AST>", t->getLine(), t->getColumn())
{
  node_ = MParser_AST::_duplicate(t);
}

NoViableAltException::NoViableAltException (
   MParser_Token_ptr t,
   const MLANG_String& fileName
) : super("NoViableAlt", fileName.c_str(), t->getLine(), t->getColumn())

{
   token_ = MParser_Token::_duplicate(t);
}

/**
 * Returns a clean error message (no line number/column information)
 */
MLANG_String NoViableAltException::getMessage() const

{
   if (!MParser_Token::is_nil(token_))
    {
      return ("unexpected token: " + token_->getText()).c_str();
    }

   // must a tree parser error if token==null
   if (node_ == MParser_TreeParser::ASTNULL.in())
    {
      return "unexpected end of subtree";
    }
   return (MLANG_String("unexpected AST node: ") + node_->toString().c_str()).c_str();
}

NoViableAltException::~NoViableAltException ()

{
}
