#ifndef __MPARSER_PARSERSHAREDINPUTSTATE_H__
#define __MPARSER_PARSERSHAREDINPUTSTATE_H__
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MParser/TokenBuffer.h"

class MParser_ParserSharedInputState;
typedef class MParser_ParserSharedInputState *MParser_ParserSharedInputState_ptr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/ParserSharedInputState.java#1 $
 */

/** This object contains the data associated with an
 *  input stream of tokens.  Multiple parsers
 *  share a single ParserSharedInputState to parse
 *  the same stream of tokens.
 */
class MParser_ParserSharedInputState : public MLANG_Refcountable<ACE_SYNCH_MUTEX>
{
    /** Where to get token objects */
protected:
   MParser_TokenBuffer_var input_;

    /** Are we guessing (guessing>0)? */
public:
   int guessing_;

    /** What file (if known) caused the problem? */
protected:
   MLANG_String filename_;

public:

   MParser_ParserSharedInputState ();

   virtual ~MParser_ParserSharedInputState ();

public:

   static bool is_nil (MParser_ParserSharedInputState_ptr a)
   { return a == 0; }

   static MParser_ParserSharedInputState_ptr _nil()
   { return 0; }

   static MParser_ParserSharedInputState_ptr _duplicate(MParser_ParserSharedInputState_ptr reader)
   { return MLANG_acquire(reader); }

public:

   const MParser_TokenBuffer_var& input() const
   { return input_; }
 
   MParser_ParserSharedInputState& input(const MParser_TokenBuffer_var& buf)
   { input_ = buf; return *this; }
 
   int guessing() const
   { return guessing_; }

   MParser_ParserSharedInputState& guessing(int new_guessing) 
   { guessing_ = new_guessing; return *this; }

   const MLANG_String& filename() const
   { return filename_; }

   void filename (const MLANG_String& new_filename)
   { filename_ = new_filename; }

public:
   void reset();
};

typedef MLANG_SmartPtr<MParser_ParserSharedInputState> MParser_ParserSharedInputState_var;

#endif /* __MPARSER_PARSERSHAREDINPUTSTATE_H__ */
