#include "MParser/StringUtils.h"

/** General-purpose utility function for removing
 * characters from back of string
 * @param s The string to process
 * @param c The character to remove
 * @return The resulting string
 */
MLANG_String MParser_StringUtils::stripBack (
   const MLANG_String& s,
   char c
)

{
   MLANG_String r = s;
   while (r.length() > 0 && r.charAt(s.length() - 1) == c)
    {
       r = r.substring(0, r.length() - 1);
    }
   return r;
}

/** General-purpose utility function for removing
 * characters from back of string
 * @param s The string to process
 * @param remove A string containing the set of characters to remove
 * @return The resulting string
 */
MLANG_String MParser_StringUtils::stripBack (
   const MLANG_String& s,
   const MLANG_String& remove
)

{
   bool changed;
   MLANG_String r = s;
   do
    {
       changed = false;
       for (size_t i = 0; i < remove.length(); i++)
        {
           char c = remove.charAt(i);
           while (r.length() > 0 && r.charAt(r.length() - 1) == c)
            {
               changed = true;
               r = r.substring(0, r.length() - 1);
            }
        }
    } while (changed);
   return r;
}

/** General-purpose utility function for removing
 * characters from front of string
 * @param s The string to process
 * @param c The character to remove
 * @return The resulting string
 */
MLANG_String MParser_StringUtils::stripFront (
   const MLANG_String& s,
   char c
)

{
   MLANG_String r = s;
   while (r.length() > 0 && r.charAt(0) == c)
    {
      r = r.substring(1);
    }
   return r;
}

/** General-purpose utility function for removing
 * characters from front of string
 * @param s The string to process
 * @param remove A string containing the set of characters to remove
 * @return The resulting string
 */
MLANG_String MParser_StringUtils::stripFront (
   const MLANG_String& s,
   const MLANG_String& remove
)

{
   bool changed;
   MLANG_String r = s;
   do
    {
      changed = false;
      for (size_t i = 0; i < remove.length(); i++)
       {
          char c = remove.charAt(i);
          while (r.length() > 0 && r.charAt(0) == c)
           {
             changed = true;
             r = r.substring(1);
           }
       }
    } while (changed);
   return r;
}

/** General-purpose utility function for removing
 * characters from the front and back of string
 * @param s The string to process
 * @param head exact string to strip from head
 * @param tail exact string to strip from tail
 * @return The resulting string
 */
MLANG_String MParser_StringUtils::stripFrontBack (
   const MLANG_String& src,
   const MLANG_String& head,
   const MLANG_String& tail
)

{
   int h = src.indexOf(head);
   int t = src.lastIndexOf(tail);
   if (h == -1 || t == -1)
      return src;
   else
      return src.substring(h + 1, t);
}
