#ifndef __MPARSER_TOKEN_H__
#define __MPARSER_TOKEN_H__
#include "MLANG/SmartPtr.h"
#include "MLANG/Object.h"
#include "MLANG/Class.h"
#include "MLANG/String.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/Token.java#1 $
 */


namespace MParser
{

class Token;
typedef Token *Token_ptr;
typedef MLANG_SmartPtr<Token> Token_var;
/** A token is minimally a token type.  Subclasses can add the text matched
 *  for the token and line info.
 */
class Token : public MLANG_Object

{
public:
    // constants
    enum
    {
        MIN_USER_TYPE = 4,
        NULL_TREE_LOOKAHEAD = 3,
        INVALID_TYPE = 0,
        EOF_TYPE = 1,
        SKIP = -1
     };

    // the illegal token object
    static Token_ptr badToken_;

public:

    Token();

    Token(int t);

    Token(int t, const MLANG_String& txt);

    virtual ~Token();

public:

   static bool is_nil (Token_ptr a)
   { return a == 0; }

   static Token_ptr _nil()
   { return 0; }

   static Token_ptr _duplicate(Token_ptr t)
   { return MLANG_acquire(t); }

public:
    DECLARE_CLASS_INFO();

    virtual int getColumn() const;

    virtual int getLine() const;

    virtual MLANG_String getFilename();

    virtual void setFilename(const MLANG_String& name);

    virtual MLANG_String getText() const;

    virtual void setText(const MLANG_String& t);

    virtual void setColumn(int c);

    virtual void setLine(int l);

    virtual int getType() const;

    virtual void setType(int t);

    virtual MLANG_String toString() const;

protected:
    // each Token has at least a token type
    int type_;
};
}

typedef MParser::Token MParser_Token;
typedef MParser_Token *MParser_Token_ptr;
typedef MLANG_SmartPtr<MParser_Token> MParser_Token_var;

#endif /* __MPARSER_TOKEN_H__ */
