#ifndef __MPARSER_TOKENBUFFER_H__ 
#define __MPARSER_TOKENBUFFER_H__ 
#include "MLANG/Refcountable.h"
#include "MLANG/SmartPtr.h"
#include "MParser/Token.h"
#include "MParser/TokenQueue.h"
#include "MParser/TokenStream.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/TokenBuffer.java#1 $
 */

class MParser_TokenBuffer;
typedef class MParser_TokenBuffer *MParser_TokenBuffer_ptr;

/**A Stream of Token objects fed to the parser from a Tokenizer that can
 * be rewound via mark()/rewind() methods.
 * <p>
 * A dynamic array is used to buffer up all the input tokens.  Normally,
 * "k" tokens are stored in the buffer.  More tokens may be stored during
 * guess mode (testing syntactic predicate), or when LT(i>k) is referenced.
 * Consumption of tokens is deferred.  In other words, reading the next
 * token is not done by conume(), but deferred until needed by LA or LT.
 * <p>
 *
 * @see antlr.Token
 * @see antlr.Tokenizer
 * @see antlr.TokenQueue
 */

class MParser_TokenBuffer : public MLANG_Refcountable<ACE_SYNCH_MUTEX>

{
protected:
   // Token source
   MParser_TokenStream_var input_;

   // Number of active markers
   int nMarkers_;

   // Additional offset used when markers are active
   int markerOffset_;

   // Number of calls to consume() since last LA() or LT() call
   int numToConsume_;

   // Circular queue
   MParser_TokenQueue queue_;

public:

   /** Create a token buffer */
   MParser_TokenBuffer(const MParser_TokenStream_var& input);

   virtual ~MParser_TokenBuffer();

public:

   static bool is_nil (MParser_TokenBuffer_ptr tb)
   { return tb == 0; }

   static MParser_TokenBuffer_ptr _nil()
   { return 0; }

   static MParser_TokenBuffer_ptr _duplicate(MParser_TokenBuffer_ptr tb)
   { return MLANG_acquire(tb); }

public:

   /** Reset the input buffer to empty state */
   void reset();

   /** Mark another token for deferred consumption */
   void consume();

private:
   /** Ensure that the token buffer is sufficiently full */
   void fill(int amount);

public:
   /** return the Tokenizer (needed by ParseView) */
   const MParser_TokenStream_var& getInput() const
   {
        return input_;
   }

    /** Get a lookahead token value */
   int LA(int i);

    /** Get a lookahead token */
   MParser_Token_var LT(int i);

   /**Return an integer marker that can be used to rewind the buffer to
    * its current state.
    */
   int mark();

   /**Rewind the token buffer to a marker.
    * @param mark Marker returned previously from mark()
    */
   void rewind(int mark);

   /** Sync up deferred consumption */
   void syncConsume();
};

typedef MLANG_SmartPtr<MParser_TokenBuffer> MParser_TokenBuffer_var;

#endif /* __MPARSER_TOKENBUFFER_H__  */
