#include "MParser/TokenQueue.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/TokenQueue.java#1 $
 */

MParser_TokenQueue::MParser_TokenQueue(int minSize)
  : offset_(0)

{
   // Find first power of 2 >= to requested size
   int size;
   if ( minSize<0 )
    {
      init(16); // pick some value for them
      return;
    }
   // check for overflow
   if ( minSize>=(ACE_INT32_MAX/2) )
    {
      init(ACE_INT32_MAX); // wow that's big.
      return;
    }
   for (size = 2; size < minSize; size *= 2) 
    {
       ;
    }
   init(size);
}

/** Add token to end of the queue
 * @param tok The token to add
 */
void MParser_TokenQueue::append(const MParser_Token_var& tok)

{
   if ((size_t)nbrEntries_ == buffer_.size())
    {
      expand();
    }
   assert(!MParser_Token::is_nil(tok));
   buffer_[(offset_ + nbrEntries_) & sizeLessOne_] = tok;
   nbrEntries_++;
}

/** Expand the token buffer by doubling its capacity */
void MParser_TokenQueue::expand()

{
   ACE_Vector<MParser_Token_var> newBuffer;
   newBuffer.resize (buffer_.size() * 2, MParser_Token_var());
   for (size_t i=0; i<buffer_.size(); i++)
       newBuffer[i] = elementAt(i);
   buffer_ = newBuffer;
   sizeLessOne_ = buffer_.size() - 1;
   offset_ = 0;
}

/** Initialize the queue.
 * @param size The initial size of the queue
 */
void MParser_TokenQueue::init(int size)

{
   // Allocate buffer
   buffer_.resize (size, MParser_Token_var());
   // Other initialization
   sizeLessOne_ = size - 1;
   offset_ = 0;
   nbrEntries_ = 0;
}

MParser_TokenQueue::~MParser_TokenQueue()

{
}
