#ifndef __MPARSER_TOKENQUEUE_H__
#define __MPARSER_TOKENQUEUE_H__
#include "ace/Vector_T.h"
#include "MParser/Token.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/TokenQueue.java#1 $
 */

/** A private circular buffer object used by the token buffer */
class MParser_TokenQueue

{
private:
   /** Physical circular buffer of tokens */
   ACE_Vector<MParser_Token_var> buffer_;
   /** buffer.length-1 for quick modulos */
   int sizeLessOne_;
   /** physical index of front token */
   int offset_;
protected:
   /** number of tokens in the queue */
   int nbrEntries_;

public:

   MParser_TokenQueue(int minSize = 1);

   ~MParser_TokenQueue();

public:

   int offset() const
   { return offset_; }

   int nbrEntries() const
   { return nbrEntries_; }

public:

   /** Add token to end of the queue
    * @param tok The token to add
    */
   void append(const MParser_Token_var& tok);

   /** Fetch a token from the queue by index
    * @param idx The index of the token to fetch, where zero is the token at the front of the queue
    */
   MParser_Token_var elementAt(int idx)
   {
      return buffer_[(offset_ + idx) & sizeLessOne_];
   }
private:

   /** Expand the token buffer by doubling its capacity */
   void expand();

   /** Initialize the queue.
    * @param size The initial size of the queue
    */
   void init(int size);

public:

   /** Clear the queue. Leaving the previous buffer alone.
    */
   void reset()
   {
      offset_ = 0;
      nbrEntries_ = 0;
   }

   /** Remove token from front of queue */
   void removeFirst()
   {
      offset_ = (offset_ + 1) & sizeLessOne_;
      nbrEntries_--;
   }
};

#endif /* __MPARSER_TOKENQUEUE_H__ */
