#include "MParser/System.h"
#include "MParser/ASTNULLType.h"
#include "MParser/MismatchedTokenException.h"
#include "MParser/TreeParser.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/TreeParser.java#1 $
 */


/** The AST Null object; the parsing cursor is set to this when
 *  it is found to be null.  This way, we can test the
 *  token type of a node without having to have tests for null
 *  everywhere.
 */
MParser_AST_var MParser_TreeParser::ASTNULL = new MParser_ASTNULLType();

/** AST support code; parser and treeparser delegate to this object */

/** Used to keep track of indentdepth for traceIn/Out */

MParser_TreeParser::MParser_TreeParser()
 : astFactory_(new MParser_ASTFactory()),
   traceDepth_(0)

{
}

/** Get the AST return value squirreled away in the parser */
const MParser_AST_var& MParser_TreeParser::getAST() const

{
   return returnAST_;
}

const MParser_ASTFactory_var& MParser_TreeParser::getASTFactory() const

{
   return astFactory_;
}

MLANG_String MParser_TreeParser::getTokenName(int num)

{
   return tokenNames_[num];
}

MLANG_Vector<MLANG_String> MParser_TreeParser::getTokenNames()

{
   return tokenNames_;
}

void MParser_TreeParser::match (
   const MParser_AST_var& t,
   int ttype
)

{
   //SYSTEM->out()->println("match("+ttype+"); cursor is "+t);
   if (MParser_AST::is_nil(t) || t.in() == ASTNULL.in() || t->getType() != ttype)

   {
      throw MParser_MismatchedTokenException(getTokenNames(), t, ttype, false);
   }
}

/**Make sure current lookahead symbol matches the given set
 * Throw an exception upon mismatch, which is catch by either the
 * error handler or by the syntactic predicate.
 */
void MParser_TreeParser::match (
   const MParser_AST_var& t,
   const MParser_BitSet& b
)

{
   if (MParser_AST::is_nil(t) || t.in() == ASTNULL.in() || !b.member(t->getType()))

   {
      throw MParser_MismatchedTokenException(getTokenNames(), t, b, false);
   }
}

void MParser_TreeParser::matchNot (
   const MParser_AST_var& t,
   int ttype
)

{
   //SYSTEM->out()->println("match("+ttype+"); cursor is "+t);
   if (MParser_AST::is_nil(t) || t.in() == ASTNULL.in() || t->getType() == ttype)

   {
      throw MParser_MismatchedTokenException(getTokenNames(), t, ttype, true);
   }
}

/** @deprecated as of 2.7.2. This method calls System.exit() and writes
 *  directly to stderr, which is usually not appropriate when
 *  a parser is embedded into a larger application. Since the method is 
 *  <code>static</code>, it cannot be overridden to avoid MParser_TreeParser::these problems.
 *  ANTLR no longer uses this method internally or in generated code.
 */
void MParser_TreeParser::panic()

{
   SYSTEM->err()->println("TreeWalker: panic");
   SYSTEM->exit(1);
}

/** Parser error-reporting function can be overridden in subclass */
void MParser_TreeParser::reportError (
   MParser_RecognitionException& ex
)

{
   SYSTEM->err()->println(ex.toString().c_str());
}

/** Parser error-reporting function can be overridden in subclass */
void MParser_TreeParser::reportError (
   const MLANG_String& s
)

{
   SYSTEM->err()->println("error: " + s);
}

/** Parser warning-reporting function can be overridden in subclass */
void MParser_TreeParser::reportWarning (
   const MLANG_String& s
)

{
   SYSTEM->err()->println("warning: " + s);
}

/** Specify an object with support code (shared by
 *  Parser and TreeParser.  Normally, the programmer
 *  does not play with this, using setASTNodeType instead.
 */
void MParser_TreeParser::setASTFactory (
   const MParser_ASTFactory_var& f
)

{
   astFactory_ = f;
}

/** Specify the type of node to create during tree building.
 * 	@deprecated since 2.7.2
 */
void MParser_TreeParser::setASTNodeType(const MLANG_String& nodeType)

{
   setASTNodeClass(nodeType);
}

/** Specify the type of node to create during tree building */
void MParser_TreeParser::setASTNodeClass(const MLANG_String& nodeType)

{
   astFactory_->setASTNodeType(nodeType);
}

void MParser_TreeParser::traceIndent()

{
   for (int i = 0; i < traceDepth_; i++)
       SYSTEM->out()->print(" ");
}

void MParser_TreeParser::traceIn (
   const MLANG_String& rname,
   const MParser_AST_var& t
)

{
   traceDepth_ += 1;
   traceIndent();
   SYSTEM->out()->println("> " + rname +
      "(" + (!MParser_AST::is_nil(t)?t->toString().c_str():"null") + ")" +
      ((inputState_.guessing() > 0)?" [guessing]":""));
}

void MParser_TreeParser::traceOut (
   const MLANG_String& rname,
   const MParser_AST_var& t
)

{
   traceIndent();
   SYSTEM->out()->println("< " + rname +
      "(" + (!MParser_AST::is_nil(t)?t->toString().c_str():"null") + ")" +
      ((inputState_.guessing() > 0)?" [guessing]":""));
   traceDepth_--;
}

MParser_TreeParser::~MParser_TreeParser()

{
}
