#ifndef __MPARSER_TREEPASER_H__
#define __MPARSER_TREEPASER_H__
#include "MLANG/SmartPtr.h"
#include "MLANG/String.h"
#include "MLANG/Object.h"
#include "MLANG/Vector_T.h"
#include "MParser/BitSet.h"
#include "MParser/RecognitionException.h"
#include "MParser/AST.h"
#include "MParser/ASTFactory.h"
#include "MParser/TreeParserSharedInputState.h"

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.4/antlr/TreeParser.java#1 $
 */

class MParser_TreeParser : public MLANG_Object

{
    /** The AST Null object; the parsing cursor is set to this when
     *  it is found to be null.  This way, we can test the
     *  token type of a node without having to have tests for null
     *  everywhere.
     */
public:
//   static MParser_ASTNULLType_var ASTNULL;
   static MParser_AST_var ASTNULL;

    /** Where did this rule leave off parsing; avoids a return parameter */
protected:
   MParser_AST_var _retTree;

    /** guessing nesting level; guessing==0 implies not guessing */
    // protected:
   int guessing_;

    /** Nesting level of registered handlers */
    // protected:
   int exceptionLevel_;

protected:
   MParser_TreeParserSharedInputState inputState_;

    /** Table of token type to token names */
protected:
   MLANG_Vector<MLANG_String> tokenNames_;

    /** AST return value for a rule is squirreled away here */
protected:
   MParser_AST_var returnAST_;

    /** AST support code; parser and treeparser delegate to this object */
protected:
   MParser_ASTFactory_var astFactory_;

    /** Used to keep track of indentdepth for traceIn/Out */
protected:
   int traceDepth_;

public:
   MParser_TreeParser();
   virtual ~MParser_TreeParser();

    /** Get the AST return value squirreled away in the parser */
public:
   const MParser_AST_var& getAST() const;

public:
   const MParser_ASTFactory_var& getASTFactory() const;

public:
   MLANG_String getTokenName(int num);

public:
   MLANG_Vector<MLANG_String> getTokenNames();

protected:
   void match(const MParser_AST_var& t, int ttype);

    /**Make sure current lookahead symbol matches the given set
     * Throw an exception upon mismatch, which is catch by either the
     * error handler or by the syntactic predicate.
     */
public:
   void match(const MParser_AST_var& t, const MParser_BitSet& b);

protected:
   void matchNot(const MParser_AST_var& t, int ttype) ;

    /** @deprecated as of 2.7.2. This method calls System.exit() and writes
     *  directly to stderr, which is usually not appropriate when
     *  a parser is embedded into a larger application. Since the method is 
     *  <code>static</code>, it cannot be overridden to avoid these problems.
     *  ANTLR no longer uses this method internally or in generated code.
     */
public:
   static void panic();

    /** Parser error-reporting function can be overridden in subclass */
public:
   void reportError(MParser_RecognitionException& ex);

    /** Parser error-reporting function can be overridden in subclass */
public:
   void reportError(const MLANG_String& s);

    /** Parser warning-reporting function can be overridden in subclass */
public:
   void reportWarning(const MLANG_String& s);

    /** Specify an object with support code (shared by
     *  Parser and TreeParser.  Normally, the programmer
     *  does not play with this, using setASTNodeType instead.
     */
public:
   void setASTFactory(const MParser_ASTFactory_var& f);

    /** Specify the type of node to create during tree building.
     * 	@deprecated since 2.7.2
     */
public:
   void setASTNodeType(const MLANG_String& nodeType);

    /** Specify the type of node to create during tree building */
public:
   void setASTNodeClass(const MLANG_String& nodeType);

public:
   void traceIndent();

public:
   void traceIn(const MLANG_String& rname, const MParser_AST_var& t);

public:
   void traceOut(const MLANG_String& rname, const MParser_AST_var& t);
};

#endif /* __MPARSER_TREEPASER_H__ */
