#ifndef __MBASE_BLOB_H__
#define __MBASE_BLOB_H__
#include "mbase/config.h"
#include "mbase/smallint.h"
#include "mbase/integer.h"

/*
 * Structure of the BlobLocation
 */
 

/*
 * blob definitions
 * tblob_t is the data that is stored in the tuple - it describes the blob
 * NOTE: tb_fd is expected to be the first member and TB_FLAGS gives offset
 *       to the tb_flags member.
 */

typedef struct mtblob
    {
    short	tb_fd;		/* blob file descriptor	(must be first) */
    short       tb_coloff;      /* Blob column offset in row    	*/
    long	tb_tblspace;	/* blob table space			*/
    long	tb_start;	/* starting byte			*/
    long	tb_end;		/* ending byte-0 for end of blob 	*/
    long        tb_size;        /* Size of blob                 	*/
    long        tb_addr;        /* Starting Sector or BlobPage  	*/
    long        tb_family;      /* Family ID                    	*/
    long        tb_volume;      /* Family Volume                	*/
    short       tb_medium;      /* Medium - odd if removable    	*/
    short       tb_bstamp;      /* first BlobPage Blob stamp    	*/
    short	tb_sockid;	/* socket id of remote blob		*/
    short	tb_flags;	/* flags - see below			*/
    long	tb_reserved1;	/* reserved for the future		*/
    long	tb_reserved2;	/* reserved for the future		*/
    long	tb_reserved3;	/* reserved for the future		*/
    long	tb_reserved4;	/* reserved for the future		*/
    } mtblob_t;

				/* offest to tb_flags in tblob_t */
#define TB_FLAGS		(2*INTSIZE+7*LONGSIZE+3*INTSIZE) 
#define SIZTBLOB		(11*LONGSIZE + 6*INTSIZE)

/* 'flags' definitions */
#define BLOBISNULL	(0x0001)	/* BLOB is NULL */
#define BLOBALIEN	(0x0002)	/* BLOB is ALIEN */
#define BL_BSBLOB       (0x0004)        /* blob is stored in blobspace */
#define BL_PNBLOB       (0x0008)        /* store in tablespace */

/*
 * this struture is used to pass "useful" information back to
 * the user.
 */

typedef struct mblobinfo
    {
    long	bi_size;		/* Size of blob			*/
    long	bi_addr;		/* Starting Sector or BlobPage	*/
    long 	bi_family;		/* Family ID			*/
    long 	bi_volume;		/* Family Volume		*/
    short	bi_flags;		/* flags 			*/
    short	bi_medium;		/* Medium - odd if removable	*/
    } mblobinfo_t;

EXTERN_C MExport
void mtblobload (
   mtblob_t* tblob,
   const unsigned char* from
);

EXTERN_C MExport
void mtblobstore (
   const mtblob_t* tblob,
   unsigned char* to
);

#ifndef NO_INFORMIX_MAP
#define tblob mtblob
#define tblob_t mtblob_t
#define blobinfo mblobinfo
#define blobinfo_t mblobinfo_t
#define tblobload mtblobload
#define tblobstore mtblobstore
#endif /* NO_INFORMIX_MAP */

#endif  /* __MBASE_BLOB_H__ */
