#include <sys/types.h>
#if defined(__linux__)
#include <sys/timeb.h>
#endif /* __linux__ */
#include <time.h>
#include <memory.h>
#include <stdio.h>
#include <ctype.h>
#include <assert.h>
#include "mbase/debug.h"
#include "mbase/decimal.h"
#include "mbase/rdate.h"
#include "mbase/datetime.h"

void dtunpack (
   register const dec_t* d,
   int qual, 
   ACE_DECIMAL_BASE_T* dgts
)

{
   register int start;
   register int pos;

   start = (ACE_TU_LEN(qual) + (qual&1) + 1) / 2;
   pos = (ACE_TU_LEN(qual) + ACE_TU_SECOND - ACE_TU_END(qual) + 1) / 2;
   memset(dgts,0,start);
   if (d->dec_ndgts>0)
    memcpy(dgts + pos - d->dec_exp,
           &d->dec_dgts[0],
           d->dec_ndgts);
}

static char separ[] = "\0-- ::. \0";

int rdtimestr (
   const dec_t* d,
   int qual,
   char* buf
)

{
   register char* to;
   int end;
   register int next;
   int ndgts;
   ACE_DECIMAL_BASE_T* from;
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];

   to = buf;
   ndgts = ACE_TU_LEN(qual) + 
           (((ACE_TU_END(qual) > ACE_TU_SECOND) ? ACE_TU_FRAC : ACE_TU_END(qual))
         - ACE_TU_START(qual)) / 2;
   if (d->dec_pos==-1)
    {
      memset(to, ' ', ndgts + (ACE_TU_START(qual)==ACE_TU_FRAC) ? 1 : 0);
      return 0;
    }
   next = ACE_TU_START(qual);
   end = ACE_TU_END(qual);
   dtunpack(d, qual, &dgts[0]);
   from = &dgts[0];
   if (next!=ACE_TU_FRAC)
    {
        register int n = ACE_TU_FLEN(qual);

        if (n&1)
         {
           *to++ = *from++ + '0';
           n--;
         }
       
	for (; n > 0; n--)
         {
	  if (n&1)
	   *to++ = *from++ % 10 + '0';
	  else
	   *to++ = *from / 10 + '0';
         }
	next += 2;
    }

   for (;next<=end && next<=ACE_TU_SECOND; next += 2)
    {
      *to++= separ[next / 2];
      *to++ = *from / 10 + '0';
      *to++ = *from++ % 10 + '0';
    }

   if (end>ACE_TU_SECOND)
    {
      *to++ = separ[next / 2];
      for (next = ACE_TU_F1; next<=end; next++)
	if (next&1)
           *to++ = *from / 10 + '0';
         else
	   *to++ = *from++ % 10 + '0';
    }

   *to = '\0';
   return 0;
}

void mdtpack (
   dec_t* d,
   int qual,
   ACE_DECIMAL_BASE_T* dgts,
   int pos 
)

{
   ACE_DECIMAL_BASE_T tmp;
   register int ndgts;
   register int exp;

   ndgts = (ACE_TU_LEN(qual) + (qual&1) + 1) / 2;
   if (ndgts > 0)
    {
      tmp = dgts[ndgts - 1];
      if (qual&1)
       dgts[ndgts - 1] -= (tmp % 10);
    }

   exp = (ACE_TU_LEN(qual) + ACE_TU_SECOND - ACE_TU_END(qual) + 1) / 2;
   decload (d,pos,exp,dgts,ndgts);
   if (ndgts > 0)
    {
      dgts[ndgts - 1] = tmp;
    }
}

void dttoasc (
   const dtime_t* dt,
   char* to
)

{
   rdtimestr(&dt->dt_dec, dt->dt_qual, to);
}

int dtcvasc (
   const char* from,
   dtime_t* dt
)

{
   return rstrdtime(from, &dt->dt_dec, dt->dt_qual);
}

int rtimestampstr (
   const TIMESTAMP_STRUCT* ts,
   int qual,
   char* buf
)

{
   dec_t d;
   register ACE_DECIMAL_BASE_T* to;
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];
   int next;
   int end;

   to = &dgts[0];
   memset(to, 0, sizeof(dgts));
   next = ACE_TU_START(qual);
   end = ACE_TU_END(qual);
   if (next!=ACE_TU_FRAC)
    {

        int n = 0;
        int flen;
	int i;

        switch(next)
         {
         case ACE_TU_YEAR:
              n = ts->year;
              break;
         case ACE_TU_MONTH:
              n = ts->month;
              break;
         case ACE_TU_DAY:
              n = ts->day;
              break;
         case ACE_TU_HOUR:
              n = ts->hour;
              break;
         case ACE_TU_MINUTE:
              n = ts->minute;
              break;
         case ACE_TU_SECOND:
              n = ts->second;
              break;
         default:
              return -1;
         }
        flen = ACE_TU_FLEN(qual);
	for (i=(flen + 1) / 2 - 1; i>=0; i--)
         {
           to[i] =  (ACE_DECIMAL_BASE_T) (n % 100);
           n = n / 100;
         }
        to += (flen + 1) / 2;
        next += 2;
    }
   for (;next<=end && next<=ACE_TU_SECOND; next += 2)
    {
       switch(next)
        {
        case ACE_TU_MONTH:
             *to++ = (ACE_DECIMAL_BASE_T)ts->month;
             break;
        case ACE_TU_DAY:
             *to++ = (ACE_DECIMAL_BASE_T)ts->day;
             break;
        case ACE_TU_HOUR:
             *to++ = (ACE_DECIMAL_BASE_T)ts->hour;
             break;
        case ACE_TU_MINUTE:
             *to++ = (ACE_DECIMAL_BASE_T)ts->minute;
             break;
        case ACE_TU_SECOND:
             *to++ = (ACE_DECIMAL_BASE_T)ts->second;
             break;
        }
    }

#if 0
   if (end>ACE_TU_SECOND)
    {
      int n = ts->fraction;
      for (next = ACE_TU_F1; next<=end; next += 2)
        {
           *to++ = (n % 100);
           n = n / 100;
        }
    }
#endif
   dtpack(&d,qual,&dgts[0],1);
   return rdtimestr(&d, qual, buf);
}


static char maxval[] = { 100, 12, 1, 24, 60, 60, 100, 100, 100, 0 };

extern const char monthdays[];

void rdtcurrent (
   dec_t* d
)

{
   time_t current;
   register struct tm* tm;
   register ACE_DECIMAL_BASE_T* to;
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];

   time(&current);
   tm = localtime(&current);
   to = &dgts[0];
   memset(to, 0, sizeof(dgts)*sizeof(ACE_DECIMAL_BASE_T));
   *to++ = (char) (tm->tm_year / 100 + 19);
   *to++ = (char) (tm->tm_year % 100);
   *to++ = (char) (tm->tm_mon + 1);
   *to++ = (char) tm->tm_mday;
   *to++ = (char) tm->tm_hour;
   *to++ = (char) tm->tm_min;
   *to++ = (char) tm->tm_sec;
   *to++ = (char) (0 / 10);
   *to = (char) (0 % 10);
   dtpack(d,ACE_TU_CURRQUAL,&dgts[0],1);
}

void dtcurrent (
   dtime_t* dt
)

{
   rdtcurrent(&dt->dt_dec);
}

int daysinmonth (
   ACE_DECIMAL_BASE_T* dgts,
   int ndgts
)

{
   int year;
   register int mes;
   ACE_DECIMAL_BASE_T curdgts[ACE_DECSIZE];
   dec_t curdec;

   curdec.dec_exp = 0;
   curdec.dec_pos = -1;
   curdec.dec_ndgts = 0;
   if (ndgts<3)
    {
      rdtcurrent(&curdec);
      dtunpack(&curdec,ACE_TU_CURRQUAL,&curdgts[0]);
    }

   mes = (ndgts>0) ? dgts[ndgts - 1] : curdgts[2];
   year = (ndgts>2) ? (dgts[0] * 100 + dgts[1])
                    : (curdgts[0] * 100 + curdgts[1]);
   return monthdays[mes] + 
         ((mes==2 && rleapyear(year)!=0) ? 1 : 0);
}

static
int strisblank (
  const char* p
)

{
  while (*p!='\0' && *p==' ') p++;
  return *p=='\0';
}

int rstrdtime (
   const char* s,
   dec_t* d,
   int qual
)

{
   return mrstrdinv(s, d, qual, 1);
}

int rstrdtimestamp (
   const char* s,
   TIMESTAMP_STRUCT* ts, 
   int qual,
   int isdtime
)

{
   dec_t d;
   int next;
   int end;
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];
   const ACE_DECIMAL_BASE_T* from;

   d.dec_exp = 0;
   d.dec_pos = -1;
   d.dec_ndgts = 0;
   mrstrdinv(s, &d, qual, isdtime);
   next = ACE_TU_START(qual);
   end = ACE_TU_END(qual);
   dtunpack(&d, qual, &dgts[0]);
   from = &dgts[0];
   memset(ts, 0, sizeof(*ts));
   if (next!=ACE_TU_FRAC)
    {
        register int flen = ACE_TU_FLEN(qual);

        int n = 0;
        if (flen&1)
         {
           n += *from++;
           flen--;
         }

	for (flen = flen / 2; flen > 0; flen--)
          n = n*100 + *from++;
        switch(next)
         {
         case ACE_TU_YEAR:
              ts->year = n;
              break;
         case ACE_TU_MONTH:
              ts->month = n;
              break;
         case ACE_TU_DAY:
              ts->day = n;
              break;
         case ACE_TU_HOUR:
              ts->hour = n;
              break;
         case ACE_TU_MINUTE:
              ts->minute = n;
              break;
         case ACE_TU_SECOND:
              ts->second = n;
              break;
         }
        next += 2;
    }
   for (;next<=end && next<=ACE_TU_SECOND; next += 2)
    {
       switch(next)
        {
        case ACE_TU_MONTH:
             ts->month = *from++;
             break;
        case ACE_TU_DAY:
             ts->day = *from++;
             break;
        case ACE_TU_HOUR:
             ts->hour = *from++;
             break;
        case ACE_TU_MINUTE:
             ts->minute = *from++;
             break;
        case ACE_TU_SECOND:
             ts->second = *from++;
             break;
        }
    }

   if (end>ACE_TU_SECOND)
    {
      int n = 0;
      for (next = ACE_TU_F1; next<=end; next++)
        n = n*100 + *from++;
      ts->fraction = n;
    }
   return 0;
}

int rstrtimestamp (
   const char* s,
   TIMESTAMP_STRUCT* ts,
   int qual
)

{
   return rstrdtimestamp(s, ts, qual, 1);
}

int rstrinv (
   const char* s,
   dec_t* d,
   int qual
)

{
   int retcode;
   int sign;
   register const char* p;

   DPRINTF(("rstrinv: s=%s qual=%d\n", s, qual));
   sign = 0;
   /* Salte los blancos al comienzo
    */
   for (p = s; *p!='\0' && *p==' '; p++);
   /* Es negativo?
    */
   if (*p=='-')
    {
      ++sign;
      ++p;
    }
   /* Es positivo?
    */
   else if (*p=='+')
    {
      ++p;
    }
   DPRINTF(("rstrinv: before mrstrdinv\n"));
   retcode = mrstrdinv(p, d, qual, 0);
   DPRINTF(("rstrinv: after mrstrdinv\n"));
   if (sign!=0)
    d->dec_pos = 0;
   return retcode;
}

int rinvstr (
   const dec_t* d,
   int qual,
   char* buf
)

{
   register char* to;

   to = buf;
   *to++ = ' ';
   rdtimestr(d, qual, to);
   for (;*to=='0' && isdigit((unsigned)to[1]); *to++=' ');
   if (d->dec_pos==0)
    to[-1] = ' ';
   return 0;
}

void tscurrent (
   TIMESTAMP_STRUCT* ts
)

{
#if 0
   struct timeb tb;
#else
   time_t current;
#endif
   register struct tm* tm;

#if 0
   ftime(&tb);
   tm = localtime(&tb.time);
#else
   time(&current);
   tm = localtime(&current);
#endif
   ts->year = tm->tm_year + 1900;
   ts->month = tm->tm_mon + 1;
   ts->day = tm->tm_mday;
   ts->hour = tm->tm_hour;
   ts->minute = tm->tm_min;
   ts->second = tm->tm_sec;
   ts->fraction = 0;
}

int rdttodate (
   const dec_t* d,
   int qual,   
   mjdate* jdate
)

{
   dec_t d2;
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];
   short mdy[3];

   d2.dec_exp = 0;
   d2.dec_pos = -1;
   d2.dec_ndgts = 0;
   rdtextend(d, qual, &d2, ACE_TU_DTENCODE(ACE_TU_YEAR, ACE_TU_DAY));
   dtunpack(&d2,ACE_TU_DTENCODE(ACE_TU_YEAR,ACE_TU_DAY),&dgts[0]);
   mdy[2] = 100*dgts[0] + dgts[1];
   mdy[0] = dgts[2];
   mdy[1] = dgts[3];
   return rmdyjul(&mdy[0], jdate);
}

int rdatedt (
  mjdate jdate,
  dec_t* d,
  int qual
)

{
   short mdy[3];
   dec_t d2; 
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];

   d2.dec_exp = 0;
   d2.dec_pos = -1;
   d2.dec_ndgts = 0;
   rjulmdy(jdate, &mdy[0]);
   dgts[0] = mdy[2] / 100;
   dgts[1] = mdy[2] % 100;
   dgts[2] = (ACE_DECIMAL_BASE_T)mdy[0];
   dgts[3] = (ACE_DECIMAL_BASE_T)mdy[1];
   dtpack(&d2, ACE_TU_DTENCODE(ACE_TU_YEAR, ACE_TU_DAY), &dgts[0], 1);
   return rdtextend(&d2, ACE_TU_DTENCODE(ACE_TU_YEAR, ACE_TU_DAY), d, qual);
}

int rdtextend (
   const dec_t* d1,
   int qual1,
   dec_t* d2,
   int qual2
)

{
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE]; 
   dec_t current;
   int start2;
   int start1;
   int ndgts;

   current.dec_exp = 0;
   current.dec_pos = -1;
   current.dec_ndgts = 0;
   start2 = ACE_TU_START(qual2);
   if (start2==ACE_TU_YEAR)
      start2 -= ACE_TU_LEN(qual2) - (ACE_TU_END(qual2) - ACE_TU_START(qual2)) - 2;

   start1 = ACE_TU_START(qual1);
   if (start1==ACE_TU_YEAR)
      start1 -= ACE_TU_LEN(qual1) - (ACE_TU_END(qual1) - ACE_TU_START(qual1)) - 2;

   if (start2<start1)
    {
      ACE_DECIMAL_BASE_T tempchar;

      rdtcurrent(&current);
      dtunpack(&current, ACE_TU_CURRQUAL, &dgts[0]);
      ndgts = (start1 + 2) / 2;
      tempchar = dgts[ndgts] / 10;
      memset(&dgts[ndgts], 0, sizeof(dgts) - ndgts);
      dtunpack(d1, qual1, &dgts[ndgts]);
      if (start1&1)
       {
         dgts[ndgts] += 10*tempchar;
       }
      ndgts = (start2 + 2) / 2;
    }
   else
    {
      memset(&dgts[0], 0, sizeof(dgts));
      dtunpack(d1, qual1, &dgts[0]);
      ndgts = (start2 - start1) / 2;
    }

   if (ACE_TU_END(qual2)>=ACE_TU_DAY && ACE_TU_END(qual1)<ACE_TU_DAY)
      dgts[(ACE_TU_DAY - start2) / 2] = 1;

   if (ACE_TU_END(qual2)>=ACE_TU_MONTH && ACE_TU_END(qual1)<ACE_TU_MONTH)
      dgts[(ACE_TU_MONTH - start2) / 2] = 1;

   if (start2<start1
   &&  start2<=ACE_TU_DAY
   &&  ACE_TU_END(qual2)>=ACE_TU_DAY
   &&  (dgts[3]<=0 || (dgts[3]>28 && dgts[3]>daysinmonth(&dgts[0], 3)))
      )
    {
      d2->dec_ndgts = 0;
      d2->dec_exp = 0;
      d2->dec_pos = -1;
      return -0x000004f3;
    }

   dtpack(d2, qual2, &dgts[ndgts], d1->dec_pos);
   return 0;
}

int rdtsub (
   const dec_t* d1,
   int qual1,  
   const dec_t* d2,
   int qual2,  
   dec_t* dr, 
   int qualr 
)

{
   dec_t tempd;
   int d;
   int carry;
   int pos;
   ACE_DECIMAL_BASE_T rdgts[ACE_DECSIZE];
   ACE_DECIMAL_BASE_T dgts1[ACE_DECSIZE];
   ACE_DECIMAL_BASE_T dgts2[ACE_DECSIZE];
   int start;
   int end;
   mjdate jdate1;
   mjdate jdate2;
   int sign;
   ACE_DECIMAL_BASE_T* pr;
   const ACE_DECIMAL_BASE_T* p1;
   const ACE_DECIMAL_BASE_T* p2;

   if (d1->dec_pos==-1 || d2->dec_pos==-1)
    {
      dr->dec_ndgts = 0;
      dr->dec_exp = 0;
      dr->dec_pos = -1;
      return 0;
    }

   tempd.dec_exp = 0;
   tempd.dec_pos = -1;
   tempd.dec_ndgts = 0;
   rdtextend(d2, qual2, &tempd, qual1);
   d2 = &tempd;
   sign = deccmp(d1, &tempd);
   if (sign<0)
    {
      d2 = d1;
      d1 = &tempd;
    }

   start = ACE_TU_START(qual1);
   end = ACE_TU_END(qual1);
   if (end&1)
      end++;

   pos = (ACE_TU_LEN(qual1) + (qual1&1) + 1) / 2;
   memset(&rdgts[0], 0, pos);
   dtunpack(d1, qual1, &dgts1[0]);
   dtunpack(d2, qual1, &dgts2[0]);
   pr = &rdgts[pos];
   p1 = &dgts1[pos];
   p2 = &dgts2[pos];
   carry = 0;
   while (end>=start && end!=ACE_TU_DAY)
    {
      d = carry + *--p1 - *--p2;
      carry = 0;
      while (d<0)
       {
         --carry;
         d += maxval[end / 2];
       }
      *--pr = d;
      end -= 2;
    }

   if (end==ACE_TU_DAY)
    {
      rdttodate(d1, qual1, &jdate1);
      rdttodate(d2, qual1, &jdate2);
      for (jdate1 = jdate1 + carry - jdate2;
           jdate1>0;
           jdate1 = jdate1 / 100)
        *--pr = jdate1 % 100;
    }

   dtpack(dr, qualr, &rdgts[0], (sign>=0) ? 1 : 0);
   return 0;
}

int rdtaddinv (
   const dec_t* d1,
   int qual1,     
   const dec_t* d2,
   int qual2,     
   dec_t* dr     
)

{
   int carry;
   int pos;
   ACE_DECIMAL_BASE_T dgts1[ACE_DECSIZE];
   ACE_DECIMAL_BASE_T dgts2[ACE_DECSIZE];
   int start1;
   int start2;
   int end2;
   int end1;
   mjdate jdate;
   int n;
   int m;
   short mdy[3];
   int errcode;
   ACE_DECIMAL_BASE_T* to;
   const ACE_DECIMAL_BASE_T* from;
   const ACE_DECIMAL_BASE_T* p;

   errcode = 0;
   if (d1->dec_pos==-1 || d2->dec_pos==-1)
    {
      dr->dec_ndgts = 0;
      dr->dec_exp = 0;
      dr->dec_pos = -1;
      return 0;
    }

   pos = (ACE_TU_LEN(qual1) + (qual1&1) + 1) / 2;
   end1 = ACE_TU_END(qual1);
   end2 = ACE_TU_END(qual2);
   start1 = ACE_TU_START(qual1);
   start2 = ACE_TU_START(qual2);
   if (end1<end2
   &&  (start1>start2 && start2!=ACE_TU_SECOND))
    {
      errcode = - 0x000003f2;
      dr->dec_ndgts = 0;
      dr->dec_exp = 0;
      dr->dec_pos = -1;
      return errcode;
    }
   if (end1&1)
      end1++;

   if (end2&1)
      end2++;

   if (end1>end2)
    {
      pos -= (end1 - end2) / 2;
      end1 = end2;
    }

   dtunpack(d1, qual1, &dgts1[0]);
   dtunpack(d2, qual2, &dgts2[0]);
   to = &dgts1[pos];
   from = &dgts2[(ACE_TU_LEN(qual2) + (qual2&1) + 1) / 2];
   carry = 0;
   while (end1>=start1)
    {
      n = 0;
      if (end1>=start2)
       {
         if (end1==start2)
          {
            for (p = dgts2; p != from; p++)
               n = 100 * n + *p;
          }
         else
          {
            n = *--from;
          }
       }

      if (d2->dec_pos==0)
         n = - n;

      if (end1==ACE_TU_DAY)
       {
         rdttodate(d1, qual1, &jdate);
         jdate += carry + n;
         rjulmdy(jdate, &mdy[0]);
         *--to = (ACE_DECIMAL_BASE_T)mdy[1];
         if (start1<=ACE_TU_MONTH)
            *--to = (ACE_DECIMAL_BASE_T)mdy[0];
         if (start1<=ACE_TU_YEAR)
          {
            *--to = mdy[2] % 100;
            *--to = mdy[2] / 100;
          }
         carry = 0;
         break;
       }

      n += carry + *--to;
      m = maxval[end1 / 2];
      carry = 0;
      /* Si el valor es inferior al valor minimo esperado para
       * esta posicion, preste una unidad al digito anterior
       * hasta que sea mayor.
       */
      while (n<((end1==ACE_TU_MONTH) ? 1 : 0))
       {
         --carry;
         n += m;
       }
      /* Si el valor es superior al valor maximo esperado para
       * esta posicion, preste una unidad al digito anterior
       * hasta que sea mayor.
       */
      while (n>=(m + ((end1==ACE_TU_MONTH) ? 1 : 0)))
       {
         carry++;
         n -= m;
       }
      *to = n;
      end1 -= 2;
    }

   if (start1==ACE_TU_YEAR && carry!=0)
      dgts1[0] += carry;

   if (start2<=ACE_TU_MONTH && ACE_TU_END(qual1)>=ACE_TU_DAY)
    {
      to = &dgts1[(start1==ACE_TU_YEAR) ? 3 : 1];
      if (*to<=0
      ||  (*to>28 && *to>daysinmonth(&dgts1[0], to - &dgts1[0]))
         )
       {
         errcode = - 0x000003f3;
         dr->dec_ndgts = 0;
         dr->dec_exp = 0;
         dr->dec_pos = -1;
         return errcode;
       }
    }

   dtpack(dr, qual1, &dgts1[0], 1);
   return 0;
}

int rinvcopy (
   const dec_t* d1,
   int qual1,
   dec_t* d2,
   int qual2
)

{
   if (qual2==qual1)
    {
      deccopy(d1, d2);
      return 0;
    }

   // Verifica que los cualificadores no esten fuera de rango
   if (ACE_TU_START(qual1)>ACE_TU_END(qual2)
   ||  ACE_TU_END(qual1)<ACE_TU_START(qual2))
    {
      d2->dec_ndgts = 0;
      d2->dec_exp = 0;
      d2->dec_pos = -1;
      return -0x000004f2;
    }

   rinvtodec(d1, 
            ACE_TU_ENCODE(ACE_TU_END(qual2) - ACE_TU_START(qual1) +
                   ACE_TU_LEN(qual1) - (ACE_TU_END(qual1) - ACE_TU_START(qual1)),
                   ACE_TU_START(qual1), ACE_TU_END(qual2)), d2);
   return rdectoinv(d2, d2, qual2);
}

int rdectoinv (
   const dec_t* d,
   dec_t* in_dec,
   int qual   
)

{
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE]; 
   int start;
   int len;
   long l;
   dec_t temp;
   int i;
   int exp;
   int end;
   ACE_DECIMAL_BASE_T* to;

   temp = *d;
   temp.dec_pos = 1;
   start = ACE_TU_START(qual);
   end = ACE_TU_END(qual);
   len = (ACE_TU_LEN(qual) + (qual&1) + 1) / 2;
   memset(&dgts[0], 0, len);
   to = &dgts[len] - 1;
   if (end>ACE_TU_SECOND)
    {
      len = (end - 9) / 2;
      to -= len;
      exp = 0;
      if (temp.dec_exp>=0)
       {
         exp = temp.dec_exp;
         if ((temp.dec_ndgts - temp.dec_exp)<len)
            len = temp.dec_ndgts - temp.dec_exp;
       }
      else
       {
         to -= temp.dec_exp;
         len = temp.dec_ndgts;
       }

      if (len>0)
       {
         memcpy(to + 1, &temp.dec_dgts[exp], len);
         if ((end&1) && len==(end - 9) / 2)
           to[len] = to[len] % 10;
       }
      end = ACE_TU_SECOND;
    }

   dectrunc(&temp, 0);
   if (temp.dec_ndgts<temp.dec_exp)
    {
      memset(&temp.dec_dgts[temp.dec_ndgts], 0, temp.dec_exp - temp.dec_ndgts);
      temp.dec_ndgts = temp.dec_exp;
    }

   while (end>start && temp.dec_ndgts>0)
    {
      divbyint(&temp, end==ACE_TU_DAY ? 30 : maxval[end / 2], &i);
      *to-- = i;
      end -= 2;
    }

   if (temp.dec_ndgts>0)
    {
      dectolong(&temp, &l);
      while (l>0)
       {
         *to = l % 100;
         l = l / 100;
         if (to==&dgts[0] && l!=0)
          {
            in_dec->dec_ndgts = 0;
            in_dec->dec_exp = 0;
            in_dec->dec_pos = -1;
            return -0x000004f1;
          }
         --to;
       }
    }

   dtpack(in_dec, qual, &dgts[0], d->dec_pos);
   return 0;
}

int rinvtodec (
   const dec_t* in_dec,
   int qual,
   dec_t* d
)

{
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];
   int end;
   int len;
   dec_t t;
   int pos;
   int start;
   ACE_DECIMAL_BASE_T* p;

   dtunpack(in_dec, qual, &dgts[0]);
   pos = in_dec->dec_pos;
   start = ACE_TU_START(qual);
   end = ACE_TU_END(qual);
   len = (ACE_TU_LEN(qual) - (ACE_TU_END(qual) - ACE_TU_START(qual)) + 1) / 2;
   p = &dgts[0];
   if (start<=ACE_TU_SECOND)
    {
      decload(d, 1, len, p, len);
      start += 2;
      p += len;
    }
   else
    {
      deccvlong(0, d);
    }

   while (start<=end && start<=ACE_TU_SECOND)
    {
      deccvlong((start==ACE_TU_DAY) ? 30 : maxval[start/2], &t);
      decmul(&t, d, d);
      decload(&t, 1, 1, p++, 1);
      decadd(&t, d, d);
      start += 2;
    }

   if (end>ACE_TU_SECOND)
    {
      len = (end - ACE_TU_SECOND + 1) / 2;
      decload(&t, 1, 0, p, len);
      decadd(&t, d, d);
    }

   d->dec_pos = pos;
   return 0;
}

void intoasc (
   const intrvl_t* in,
   char* to
)

{
   rinvstr(&in->in_dec, in->in_qual, to);
}

int incvasc (
   const char* from,
   intrvl_t* in
)

{
   return rstrinv(from, &in->in_dec, in->in_qual);
}

int mrstrdinv (
   const char* s,
   dec_t* d, 
   int qual,
   int isdtime
)

{
   register const char* p;
   register int n;
   register int cnt;
   ACE_DECIMAL_BASE_T dgts[ACE_DECSIZE];
   int start;
   int flen;
   int end;
   ACE_DECIMAL_BASE_T* to;
   int check;

   DPRINTF(("mrstrdinv: enter d\n", d));
   /* Inicializa en nulo
    */
   d->dec_exp = 0;
   d->dec_pos = -1;
   d->dec_ndgts = 0;

   /* Salta los blancos al comienzo
    */
   for (p = s; *p!='\0' && *p==' '; p++);

   /* Es nulo?
    */
   if (*p=='\0')
    return 0;

   start = ACE_TU_START(qual);
   end = ACE_TU_END(qual);
   DPRINTF(("mrstrdinv: start=%d end=%d p=%s\n",
		start, end, p));
   memset(&dgts[0],0,sizeof(dgts));
   to = &dgts[0];
   if (start==ACE_TU_FRAC)
    {
      if (*p=='0')
       p++;

      if (*p!='.')
       return -0x000004f7;

      p++;
    }
   else
    {
      /* Encuentra el numero de digitos al comienzo del datetime o
       * interval.
       */
      for (cnt = 0; *p!='\0' && isdigit((unsigned)*p); p++)
       cnt++;
      p -= cnt;
      flen = ACE_TU_FLEN(qual);
      if (cnt>flen)
       return -0x000004ed;

      if (isdtime
      && start==ACE_TU_YEAR
      && cnt<=2)
       {
         *to++ = 19;
       }
      else
       {
         for (;flen>cnt+1; flen -= 2)
          *to++ = 0;
         if (flen>cnt && (flen&1))
          *to++ = 0;
       }

      /* Reserva los 2 primeros digitos del comienzo
       */
      cnt -= 2;

      /* El numero de digitos es impar.
       */
      if (cnt>0 && (cnt&1))
       {
         *to++ = (*p++ - '0');
         cnt--;
       }

      for (;cnt>0; cnt-=2)
       {
         n = (*p++ - '0')*10;
         *to++ = n + (*p++ - '0');
       }
    }

   for (check = isdtime;
        *p!='\0' && start<=end && start<=ACE_TU_SECOND;
         p++, start += 2)
    {
      if (isdigit((unsigned)*p))
       n = *p - '0';
      else if (*p==' ' || *p==':')
       n = 0;
      else
       return -0x000004ee;
      p++;
      if (isdigit((unsigned)*p))
       n = 10*n + (*p++ - '0');
      if (!check)
       {
        check = 1;
       }
      else if (start<=ACE_TU_DAY)
       {
         if (start!=ACE_TU_DAY)
          {
            if (start==ACE_TU_MONTH
            &&  (((n - isdtime)>=maxval[start/2]
            ||   (isdtime && n==0))))
               return -0x000004ef;
          }
         else if (n<=0
         ||       (n>28 &&
                   n>daysinmonth(&dgts[0], to - &dgts[0])))
             return -0x000004ef;
       }
      else if (n>=maxval[start/2])
       {
         return -0x000004ef;
       }
      *to++ = n;
      if (*p==separ[start/2+1])
       continue;
      if (start==end)
       break;
      return -0x000004ee;
    }

   for (start = ACE_TU_F1;
        *p!='\0' && start<=end;
        start++)
    {
      if (!isdigit((unsigned)*p))
       break;
      if (start&1)
       {
         *to = (*p - '0') * 10;
       }
      else
       {
         *to = *to + (*p - '0');
         to++;
       }
      p++;
    }

   if (!strisblank(p))
    return -0x000004f0;

   dtpack(d,qual,&dgts[0],1);
   DPRINTF(("rstrdinv: leave OK\n"));
   return 0;
}

