#if !defined(__MBASE_DATETIME_H__)
#define __MBASE_DATETIME_H__
#include "mbase/decimal.h"
#include "mbase/rdate.h"

#if !defined(ACE_HAS_TIME_STRUCT)
#define ACE_HAS_TIME_STRUCT
/* transfer types for TIME */

typedef struct tagMTIME_STRUCT
{
	unsigned short hour;
	unsigned short minute;
	unsigned short second;
} MTIME_STRUCT;
#endif /* !ACE_HAS_TIME_STRUCT */

#if !defined(ACE_HAS_TIMESTAMP_STRUCT)
/* transfer types for DATE, TIME, TIMESTAMP */

typedef struct tagMTIMESTAMP_STRUCT
{
	signed short year;
	unsigned short month;
	unsigned short day;
	unsigned short hour;
	unsigned short minute;
	unsigned short second;
	unsigned long fraction;
} MTIMESTAMP_STRUCT;

#define ACE_HAS_TIMESTAMP_STRUCT
#endif /* !ACE_HAS_TIMESTAMP_STRUCT */

#if defined(OSF1)
#define MDATETIME_STRUCT MTIMESTAMP_STRUCT
#else /* !OSF1 */
typedef MTIMESTAMP_STRUCT MDATETIME_STRUCT;
#endif /* !OSF1 */

/* time units for datetime qualifier */

#define ACE_TU_YEAR 0
#define ACE_TU_MONTH 2
#define ACE_TU_DAY 4
#define ACE_TU_HOUR 6
#define ACE_TU_MINUTE 8
#define ACE_TU_SECOND 10
#define ACE_TU_FRAC 12
#define ACE_TU_F1 11
#define ACE_TU_F2 12
#define ACE_TU_F3 13
#define ACE_TU_F4 14
#define ACE_TU_F5 15

/* ACE_TU_END - end time unit in datetime qualifier */
/* ACE_TU_START - start time unit in datetime qualifier */
/* ACE_TU_LEN - length in  digits of datetime qualifier */

#define ACE_TU_END(qual) (qual & 0xf)
#define ACE_TU_START(qual) ((qual>>4) & 0xf)
#define ACE_TU_LEN(qual) ((qual>>8) & 0xff)

/* ACE_TU_ENCODE - encode length, start and end time unit to form qualifier */
/* ACE_TU_DTENCODE - encode datetime qual */
/* ACE_TU_IENCODE - encode interval qual */

#define ACE_TU_ENCODE(len,s,e) (((len)<<8) | ((s)<<4) | (e))
#define ACE_TU_DTENCODE(s,e) ACE_TU_ENCODE(((e)-(s)+((s)==ACE_TU_YEAR?4:2)), s, e) 
#define ACE_TU_IENCODE(len,s,e) ACE_TU_ENCODE(((e)-(s)+(len)),s,e)
#define ACE_TU_FLEN(len) (ACE_TU_LEN(len)-(ACE_TU_END(len)-ACE_TU_START(len)))

/* TU_CURRQUAL - default qualifier used by current */

#define ACE_TU_CURRQUAL ACE_TU_ENCODE(17,ACE_TU_YEAR,ACE_TU_F3)

#ifndef WIN32
EXTERN_C
struct mintrvl;
#else /* WIN32 */
EXTERN_C
#endif /* WIN32 */
struct mintrvl
	{
	short in_qual;
	mdec_t in_dec;
	};
typedef struct mintrvl mintrvl_t;

EXTERN_C MExport
void mrintoasc (
   const mintrvl_t* in, 
   char* to
);

EXTERN_C MExport
int mrdectoinv (
   const mdec_t* dec, 
   mdec_t* in_dec,
   int qual
);

EXTERN_C MExport
int mrincvasc (
   const char* s_in, 
   mintrvl_t* in
);

EXTERN_C MExport
int mrinvcopy (
   const mdec_t* d1,
   int qual1,
   mdec_t* d2,
   int qual2
);

// char* ==> (qual,ACE_Decimal)
EXTERN_C MExport
int mrstrdinv (
   const char* buf,
   mdec_t *d,
   int qual,
   int isdtime
);

// (ACE_Decimal,qual) ==> char*
EXTERN_C MExport
int mrdtimestr (
   const mdec_t* d,
   int qual,
   char* buf
);

// (qual,TIMESTAMP_STRUCT) ==> char*
EXTERN_C MExport
int mrtimestampstr (
   const MTIMESTAMP_STRUCT* ts,
   int qual,
   char* varbuf
);

// char* ==> (TIMESTAMP_STRUCT,qual)
EXTERN_C MExport
int mrstrtimestamp (
   const char* s,
   MTIMESTAMP_STRUCT* ts,
   int qual
);

EXTERN_C MExport
int mrstrdtime (
   const char* s,
   mdec_t* d,
   int qual
);

EXTERN_C MExport
int mrstrinv (
   const char* s,
   mdec_t* d,
   int qual
);

EXTERN_C MExport
int mrinvstr (
   const mdec_t* d,
   int qual,
   char* buf
);

EXTERN_C MExport
void mdtpack (
   mdec_t* d,
   int qual,
   ACE_DECIMAL_BASE_T* dgts,
   int pos 
);

EXTERN_C MExport
void mdtunpack (
   register const mdec_t* d,
   int qual, 
   ACE_DECIMAL_BASE_T* dgts
);

EXTERN_C MExport
void mrdtcurrent (
   mdec_t* d
);

EXTERN_C MExport
void mtscurrent (
   MTIMESTAMP_STRUCT* ts
);

#ifndef WIN32
EXTERN_C
struct mdtime;
#else /* WIN32 */
EXTERN_C
#endif /* WIN32 */
struct mdtime
{
   short dt_qual;
   mdec_t dt_dec;
};
typedef struct mdtime mdtime_t;

EXTERN_C MExport
void mdtcurrent (
   mdtime_t* d
);


EXTERN_C MExport
void mdttoasc (
   const mdtime_t* dt,
   char* to
);

EXTERN_C MExport
int mdtcvasc (
   const char* from,
   mdtime_t* dt
);

EXTERN_C MExport
int mrdttodate (
   const mdec_t* d,
   int qual,   
   mjdate* jdate
);

EXTERN_C MExport
int mrdatedt (
  mjdate jdate,
  mdec_t* d,
  int qual
);

EXTERN_C MExport
int mrdtextend (
   const mdec_t* d1,
   int qual1,
   mdec_t* d2,
   int qual2
);

EXTERN_C MExport
int mrdtsub (
   const mdec_t* d1,
   int qual1,  
   const mdec_t* d2,
   int qual2,  
   mdec_t* dr, 
   int qualr 
);

EXTERN_C MExport
int mrdtaddinv (
   const mdec_t* d1,
   int qual1,     
   const mdec_t* d2,
   int qual2,     
   mdec_t* dr     
);

EXTERN_C MExport
int mrdectoinv (
   const mdec_t* d,
   mdec_t* in_dec,
   int qual   
);

EXTERN_C MExport
int mrinvtodec (
   const mdec_t* in_dec,
   int qual,
   mdec_t* d
);

EXTERN_C MExport
void mintoasc (
   const mintrvl_t* in,
   char* to
);

EXTERN_C MExport
int mincvasc (
   const char* from,
   mintrvl_t* in
);

#ifndef NO_INFORMIX_MAP

#define tagTIME_STRUCT tagMTIME_STRUCT
#define TIME_STRUCT MTIME_STRUCT
#define tagTIMESTAMP_STRUCT tagMTIMESTAMP_STRUCT
#define TIMESTAMP_STRUCT MTIMESTAMP_STRUCT
#define DATETIME_STRUCT MDATETIME_STRUCT

#define intrvl mintrvl
#define intrvl_t mintrvl_t
#define rintoasc mrintoasc 
#define rdectoinv mrdectoinv
#define rincvasc mrincvasc
#define rinvcopy mrinvcopy
#define rstrdinv mrstrdinv
#define rdtimestr mrdtimestr
#define rtimestampstr mrtimestampstr
#define rstrtimestamp mrstrtimestamp
#define rstrdtime mrstrdtime
#define rstrinv mrstrinv
#define rinvstr mrinvstr
#define dtpack mdtpack
#define dtunpack mdtunpack
#define rdtcurrent mrdtcurrent
#define tscurrent mtscurrent
#define dtime mdtime
#define dtime_t mdtime_t
#define dtcurrent mdtcurrent
#define dttoasc mdttoasc
#define dtcvasc mdtcvasc
#define rdttodate mrdttodate
#define rdatedt mrdatedt
#define rdtextend mrdtextend
#define rdtsub mrdtsub
#define rdtaddinv mrdtaddinv
#define rdectoinv mrdectoinv
#define rinvtodec mrinvtodec
#define intoasc mintoasc
#define incvasc mincvasc
#endif /* NO_INFORMIX_MAP */

#endif /* __MBASE_DATETIME_H__ */
