#ifndef __MBASE_decimal_H__
#define __MBASE_decimal_H__
#include "mbase/config.h"

// macros to set defaults for ACE_Decimal
// ACE_DECSIZE			Number of digits used in ACE_Decimal
// ACE_DECIMAL_BASE		Base used in representations of digits in ACE_DECIMAL
// ACE_DECIMAL_BASE_T		Type of each digit in ACE_Decimal digits
// ACE_DECIMAL_BASE_PREC	Precision of datatype used for represent a ACE_Decimal digit
// ACE_LONG_PREC		Precision of a long
// ACE_INT_PREC			Precision of an integer
// ACE_SHORT_PREC		Precision of a short

#define ACE_DECSIZE		16
#define ACE_DECIMAL_BASE	100
#define	ACE_DECIMAL_BASE_T	unsigned char
#define	ACE_DECIMAL_BASE_PREC	3
#define	ACE_DECIMAL_BASE_DIGITS	(ACE_DECIMAL_BASE_PREC - 1)
#define	ACE_DECIMAL_DIGITS(prec) ((prec+ACE_DECIMAL_BASE_DIGITS-1)/ACE_DECIMAL_BASE_DIGITS)
#define	ACE_LONG_PREC		10
#define	ACE_ULONG_PREC		10
#define	ACE_INT_PREC		10
#define	ACE_UINT_PREC		10
#define	ACE_SHORT_PREC		5
#define	ACE_USHORT_PREC		5

#define ACE_SHRT_MIN	SHRT_MIN
#define ACE_SHRT_MAX	SHRT_MAX
#define ACE_USHRT_MAX	USHRT_MAX
#define ACE_INT_MIN	INT_MIN
#define ACE_INT_MAX	INT_MAX
#define ACE_UINT_MAX	UINT_MAX
#define ACE_LONG_MAX	LONG_MAX
#define ACE_LONG_MIN	LONG_MIN
#define ACE_ULONG_MAX	ULONG_MAX

// ACE_DECLEN calculates minumum number of bytes
// necessary to hold a decimal(m,n)
// where m = total # significant digits and
//	 n = significant digits to right of decimal

#define ACE_DECLEN(m,n)		(((m)+((n)&1)+3)/2)
#define ACE_DECLENGTH(len)	ACE_DECLEN(ACE_PRECTOT(len),ACE_PRECDEC(len))

// ACE_DECPREC calculates a default precision given
// number of bytes used to store number

#define ACE_DECPREC(size)	(((size-1)<<9)+2)

// macros to look at and make encoded decimal precision
//
//  ACE_PRECTOT(x)		return total precision (digits total)
//  ACE_PRECDEC(x) 		return decimal precision (digits to right)
//  ACE_PRECMAKE(x,y)	make precision from total and decimal

#define ACE_PRECTOT(x)		(((x)>>8) & 0xff)
#define ACE_PRECDEC(x)		((x) & 0xff)
#define ACE_PRECMAKE(x,y)	(((x)<<8) + (y))

/* #ifndef DECSIZE
 */
#ifndef WIN32
EXTERN_C struct mdecimal;
struct mdecimal
   {
    short dec_exp;		// exponent base 100	
    short dec_pos;		// sign: 1=pos, 0=neg, -1=null
    short dec_ndgts;		// number of significant digits
    ACE_DECIMAL_BASE_T dec_dgts[ACE_DECSIZE];	// actual digits base 100
    };
#else /* WIN32 */
EXTERN_C
struct mdecimal
   {
    short dec_exp;		// exponent base 100	
    short dec_pos;		// sign: 1=pos, 0=neg, -1=null
    short dec_ndgts;		// number of significant digits
    ACE_DECIMAL_BASE_T dec_dgts[ACE_DECSIZE];	// actual digits base 100
    };
#endif /* WIN32 */
typedef struct mdecimal mdec_t;
#define DECSIZE ACE_DECSIZE

EXTERN_C MExport
int mdecload (
   mdec_t* d,
   int pos,
   int exp,
   ACE_DECIMAL_BASE_T* dgts,
   int ndgts
);

EXTERN_C MExport
int mdecprec (
   const mdec_t* d
);

EXTERN_C MExport
int mdec2prec (
   const mdec_t* d
);

EXTERN_C MExport
void mdeccopy (
   const mdec_t* in,
   mdec_t* out
);

EXTERN_C MExport
int mdeccmp (
   const mdec_t* d1,
   const mdec_t* d2
);

EXTERN_C MExport
int mdecadd (
   const mdec_t* d1,
   const mdec_t* d2,
   mdec_t* dr
);

EXTERN_C MExport
int mdecsub (
   const mdec_t* md1,
   const mdec_t* md2,
   mdec_t* dr
);

EXTERN_C MExport
int mdecmul (
   const mdec_t* d1,
   const mdec_t* d2,
   mdec_t* dr
);

EXTERN_C MExport
int mdivbyint (
   mdec_t* d,
   int i,
   int* r
);

EXTERN_C MExport
int mdecdiv (
   const mdec_t* d1,
   const mdec_t* d2,
   mdec_t* dr
);

EXTERN_C MExport
int mdectolong (
   const mdec_t*,
   long* l
);

EXTERN_C MExport
int mdectoulong (
   const mdec_t*,
   unsigned long* l
);

EXTERN_C MExport
int mdectoshort (
   const mdec_t*,
   short* s
);

EXTERN_C MExport
int mdectoushort (
   const mdec_t*,
   unsigned short* s
);

EXTERN_C MExport
int mdectoint (
   const mdec_t*,
   int* l
);

EXTERN_C MExport
int mdeccvint (
   int i,
   mdec_t* d
);

EXTERN_C MExport
int mdeccvlong (
   long l,
   mdec_t* d
);

EXTERN_C MExport
int mdeccvulong (
   unsigned long l,
   mdec_t* d
);

EXTERN_C MExport
int mdecround (
   mdec_t* d,
   int prec
);

EXTERN_C MExport
int mdectrunc (
   mdec_t* d,
   int prec
);

EXTERN_C MExport
int mdeccvasc (
   const char* s,
   int len,
   mdec_t* d
);

EXTERN_C MExport
int mrdefmtdec (
   const char* s,
   int len,
   const char* sqlformat,
   mdec_t* d
);

EXTERN_C MExport
char* mdeccvt_r (
   const mdec_t* d,
   int len,
   int* exp,
   int* sign,
   int use_exp,
   char* buf,
   int maxlen
);

// Convert ACE_Decimal to char

EXTERN_C MExport
char* mdececvt_r (
   const mdec_t* d,
   int len,
   int* exp,
   int* sign,
   char* buf,
   int maxlen
);

// Convert ACE_Decimal to char in exponential format

EXTERN_C MExport
char* mdecfcvt_r (
   const mdec_t* d,
   int len,
   int* exp,
   int* sign,
   char* buf,
   int maxlen
);

EXTERN_C MExport
int mdectoexp_r (
   const mdec_t* d,
   char* p,
   int maxlen,
   int exp,
   int sign,
   char* buf,
   int bufsize
);

EXTERN_C MExport
int mdectoasc (
   const mdec_t* d,
   char* to,
   int len,
   int right
);

EXTERN_C MExport
int mdeccvflt (
   double dbl,
   mdec_t* d
);

EXTERN_C MExport
int mdeccvdbl (
   double dbl,
   mdec_t* d
);

EXTERN_C MExport
int mdectodbl (
   const mdec_t* d,
   double* dbl
);

EXTERN_C MExport
int mdeclpow (
   const mdec_t* x,
   long n,
   mdec_t* r
);

EXTERN_C MExport
int mdecpow (
   const mdec_t* x,
   const mdec_t* y,
   mdec_t* r
);

/* Calcula el binomial a de x.
 */

EXTERN_C MExport
int mdecbinomial (
   const mdec_t* x,
   const mdec_t* a,
   mdec_t* r
);

EXTERN_C MExport
int mdecsqrt (
   const mdec_t* x,
   mdec_t* r
);

EXTERN_C MExport
int mdecexp (
   const mdec_t* x,
   mdec_t* r
);

EXTERN_C MExport
int mdecln (
   const mdec_t* x,
   mdec_t* r
);

EXTERN_C MExport
void mstdecimal (
   const mdec_t* d,
   ACE_DECIMAL_BASE_T* dgts,
   int ndgts
);

EXTERN_C MExport
int mlddecimal (
   const ACE_DECIMAL_BASE_T* from,
   int len,
   mdec_t* d
);

/*
#endif DECSIZE 
*/

#ifndef NO_INFORMIX_MAP

#define decimal mdecimal
#define dec_t mdec_t
#define decload mdecload
#define decprec mdecprec
#define dec2prec mdec2prec
#define deccopy mdeccopy
#define deccmp	mdeccmp
#define decadd	mdecadd
#define decsub mdecsub
#define decmul mdecmul
#define divbyint mdivbyint
#define decdiv	mdecdiv
#define dectolong	mdectolong
#define dectoulong	mdectoulong
#define dectoshort	mdectoshort
#define dectoushort	mdectoushort
#define dectoint	mdectoint
#define deccvint mdeccvint
#define deccvlong mdeccvlong
#define deccvulong mdeccvulong
#define decround mdecround
#define dectrunc	mdectrunc
#define deccvasc	mdeccvasc
#define rdefmtdec mrdefmtdec
#define deccvt_r mdeccvt_r
#define dececvt_r mdececvt_r
#define decfcvt_r mdecfcvt_r
#define dectoexp_r mdectoexp_r
#define dectoasc mdectoasc
#define deccvflt mdeccvflt
#define deccvdbl mdeccvdbl
#define dectodbl mdectodbl
#define stdecimal mstdecimal
#define lddecimal mlddecimal
#endif /* NO_INFORMIX_MAP */

#endif /* __MBASE_decimal_H__ */
