#include <sys/types.h>
#if defined(__linux__)
#include <sys/timeb.h>
#endif /* __linux__ */
#include <time.h>
#include <memory.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <assert.h>
#if !defined(WIN32)
#include <libgen.h>
#endif /* WIN32 */
#include "mbase/debug.h"
#include "mbase/rdate.h"
#include "mbase/rutils.h"

const char leapdays[]={0,31,29,31,30,31,30,31,31,30,31,30,31};
const char monthdays[]={0,31,28,31,30,31,30,31,31,30,31,30,31};

#define JDAYS(y) ((mjdate)((y) * 365.2425))
#define JBASE JDAYS(1899)

int rleapyear (
    int year
)

{
    if ((year&3)==0 && (year%400==0 || year%100!=0)) {
       return 1;
    }
    else
       return 0;
}

/*
 * 1 de Enero de 1.900.
 */
int rtoday (
    mjdate *jdate
)

{
#if !defined(WIN32)
   time_t t;
   short mdy[3];
   struct tm tm;

   time(&t);
   localtime_r(&t, &tm);
   mdy[0] = tm.tm_mon + 1;
   mdy[1] = tm.tm_mday;
   mdy[2] = tm.tm_year + 1900;
   return rmdyjul(mdy, jdate);
#else /* WIN32 */
   time_t t;
   short mdy[3];
   struct tm* tm;

   time(&t);
   tm = localtime(&t);
   mdy[0] = tm->tm_mon + 1;
   mdy[1] = tm->tm_mday;
   mdy[2] = tm->tm_year + 1900;
   return rmdyjul(mdy, jdate);
#endif /* WIN32 */
}

int rmdyjul (
   short mdy[],
   mjdate* jdate
)

{
   register int mes;
   register int ano;
   register int i;
   register int dia;
   register MBASE_INT32 nyears;
   register MBASE_INT32 totdias;
   register const char* days;

   mes = mdy[0];
   dia = mdy[1];
   ano = mdy[2];
   if (ano<1 || ano>9999)
      return -1204;

   nyears = ano - 1;
   days = rleapyear(ano) ? leapdays : monthdays;
   if (mes<1 || mes>12)
      return -1205;
   else if (dia<1 || dia>days[mes])
      return -1206;

   totdias = ((nyears / 100) * 146097L) / 4;
   *jdate = dia + totdias + ((nyears % 100) * 1461L) / 4 - JBASE;
   for (i=1; i<mes; i++)
     (*jdate) += days[i];
   return 0;
}

int rjulmdy (
   mjdate jdate,
   short mdy[]
)

{
   register int mes;
   register int dia;
   register int ano;
   register MBASE_INT32 t;
   register const char* days;

   if (jdate<-JBASE || jdate > JDAYS(9999))
    {
      mdy[2] = 0;
      mdy[1] = 0;
      mdy[0] = 0;
      return -0x000004ba;
    }

   jdate += JBASE - 1;
   t = (jdate * 4 + 3) / 146097L;
   jdate -= (t * 146097L) / 4;
   ano = t * 100;
   t = (jdate * 4 + 3) / 1461L;
   jdate -= (t * 1461L) / 4;
   ano += t + 1;
   dia = jdate + 1;
   days = rleapyear(ano) ? leapdays : monthdays;
   mes = 1;
   while (mes<=12 && dia>days[mes])
     dia -= days[mes++];

   mdy[0] = mes;
   mdy[1] = dia;
   mdy[2] = ano;
   return 0;
}

int rdayofweek (
   mjdate jdate
)

{
#if 0
   return (jdate+0xa955b) % 7;
#else
   return (jdate + JBASE) % 7;
#endif
}

int rdatestr (
   mjdate jdate,
   char* s
)

{
  char fmt[20+1];

  *s = '\0';
  if (jdate==(mjdate)0x80000000)
   {
     return 0;
   }

  fmtstr(fmt);
  DPRINTF(("rdatestr: fmt=%s\n", fmt));
  return rfmtdate(jdate,fmt,s);
}

static int initflg;
static char dbdate[20];
static char separator;
char _defdatefmt[MAXDATEFMTLEN+1];
static int datefmtpos[3];
static char datedgts[3];

int datesep(int c)

{
   switch(c)
    {
    case ' ':
    case '-':
    case '.':
    case '/':
         return 1;
    default:
         return 0;
    }
}

/* Get DBDATE format
 */
void getdbdate (
   char* dbdate
)

{
   if (dbdate[0]=='\0' && getenv("DBDATE")==0)
    strcpy(dbdate, "MDY4/");
   else if (dbdate[0]=='\0')
    strcpy(dbdate, getenv("DBDATE"));
}

/* Get default date format
 */
int rgetdefdatefmt (
   char* defdatefmt
)

{
   register char* p;
   register char* to;
   register int i;

   getdbdate (dbdate);
   for (p = dbdate; *p!='\0'; p++)
     if (*p>='A' && *p<='Z')
      *p = *p + ('a' - 'A');

   p = &dbdate[4];
   if (*p!='\0')
    {
      if (datesep(*p))
	separator = *p;
      else
        separator = '/';
    }
   else
    {
	separator = 0;
    }

#if 0
   if (chkfmt()!=0)
    return 0xffffffff;
#endif

   to = &defdatefmt[0];
   p = dbdate;
   for (i = 0; i<3; i++)
    {
       switch (*p) 
        {
        case 'm':
             *to++ = 'm';
             *to++ = 'm';
             datedgts[i] = 2;
             datefmtpos[1] = i;
             break;
    
        case 'd':
             *to++ = 'd';
             *to++ = 'd';
             datedgts[i] = 2;
             datefmtpos[2] = i;
             break;
    
        case 'y':
             datefmtpos[0] = i;
             *to++ = 'y';
             *to++ = 'y';
             ++p;
             if (*p=='4')
              {
	        *to++ = 'y';
	        *to++ = 'y';
	        datedgts[i] = 4;
              }
             else
              {
	        datedgts[i] = 2;
              }
             break;
         }

        p++;
        if (separator!=0 && i!=2)
         *to++ = separator;
    }
   *to = '\0';
   DPRINTF(("rgetdefdate: defmtdate=%s\n", defdatefmt));
   return 0;
}

int getfmt()

{
   if (initflg)
    return 0;

   initflg = 1;
   return rgetdefdatefmt (&_defdatefmt[0]);
}

int rstrdate (
   const char* s,
   mjdate* jdate
)

{
   char fmt[20+1];

   fmtstr(fmt);
   return rdefmtdate(jdate, fmt, s);
}

void rnum2dig (
   int n,
   char* to
)

{
   *to++ = (n / 10) + '0';
   *to++ = (n % 10) + '0';
}
