#include <sys/types.h>
#if defined(__linux__)
#include <sys/timeb.h>
#endif /* __linux__ */
#include <time.h>
#include <memory.h>
#include <stdio.h>
#include <malloc.h>
#include <ctype.h>
#include <stdlib.h>
#include <assert.h>
#if !defined(WIN32)
#include <libgen.h>
#endif /* !WIN32 */
#include "mbase/rdate.h"
#include "mbase/sqltypes.h"
#include "mbase/debug.h"
#include "mbase/rutils.h"
#include "mbase/byutils.h"

extern const char leapdays[];
extern const char monthdays[];

static int initflag = 0; /* intlcnvrt_0x50 */
static char daymsg[7*4+1]; /* intlcnvrt_0x54 */
static char* dayarray[7]; /* intlcnvrt_0x74 */
static char monmsg[12*4+1]; 
static char* monarray[12]; /* intlcnvrt_0xc4 */

void rnum2dig (
   int n,
   char* to
);

static int repcount (
   int c,
   const char* p
)

{
   register int n;

   for (n=0; *p!='\0' && *p==c; n++)
     p++;

   return n;
}

static void initxby (
   short msgnum,
   char* msg,
   short msgsize,
   char** argv,
   int flen
)

{
   int i;
   register int offset;

   if (rgetmsg(msgnum, msg, msgsize)!=0)
    {
    memset(msg, 0, msgsize);
    if (msgnum==0x4b0)
     {
       memcpy(daymsg,"Do Lu Ma Mi Ju Vi Sa",strlen("Do Lu Ma Mi Ju Vi Sa"));
     }
    else if (msgnum==0x4b1)
     {
       memcpy(monmsg,"Ene Feb Mar Abr May Jun Jul Ago Sep Oct Nov Dic",
         strlen("Ene Feb Mar Abr May Jun Jul Ago Sep Oct Nov Dic"));
     }
    }

   for (i = 0, offset = 0;
        offset<(msgsize - flen);
	offset += flen + 1, i++)
    {
	argv[i] = &msg[offset];
        msg[offset + flen] = '\0';
    }
}

static void initarray()

{
    initxby(0x4b0, &daymsg[0], sizeof(daymsg)/sizeof(daymsg[0]), &dayarray[0], 3);
    initxby(0x4b1, &monmsg[0], sizeof(monmsg)/sizeof(monmsg[0]), &monarray[0], 3);
    initflag++;
}

int rfmtmdy (
   short mdy[],
   const char* fmt,
   char* to
)

{
   int cnt;
   int c;
   mjdate jdate;
   const char* p;
   int fmtlen;
   int retcode;
   char deformat[20+1];

   if (initflag==0)
    initarray();

   if (fmt==0)
    {
      fmtstr(deformat);
      fmt = deformat;
    }

   fmtlen = strlen(fmt);
   memset(to, '\0', fmtlen + 1);
   if ((retcode=rmdyjul(mdy, &jdate))!=0)
      return retcode;

   for (p=fmt;*p!='\0'; p += cnt, to += cnt)
    {
      c = *p;
      cnt = repcount(c, p);
      switch(c)
       {
       case 'D':
       case 'd':
	    switch(cnt)
             {
             case 2:
	          rnum2dig(mdy[1], to);
	          continue;
             case 3:
	          memcpy(to, dayarray[rdayofweek(jdate)], 3);
	          continue;
             }
            break;

       case 'M':
       case 'm':
	    switch(cnt)
             {
             case 2:
	          rnum2dig(mdy[0], to);
	          continue;
             case 3:
	          memcpy(to, monarray[mdy[0] - 1], 3);
	          continue;
             }
            break;

       case 'Y':
       case 'y':
	    switch(cnt)
             {
             case 2:
	          rnum2dig(mdy[2] % 100, to);
	          continue;
             case 4:
	          rnum2dig(mdy[2] / 100, to);
	          rnum2dig(mdy[2] % 100, &to[2]);
	          continue;
             }
            break;
       }
     cnt = 1;
     *to = *p;
    }
   return 0;
}

int rfmtdate (
   mjdate jdate,
   const char* fmt,
   char* to
)

{
   short mdy[3];
   int errnum;
   if ((errnum=rjulmdy(jdate, mdy))!=0)
      return errnum;

   return rfmtmdy(mdy, fmt, to);
}

extern char _defdatefmt[];

int getfmt();

int fmtstr(char* format)

{
   getfmt();
   strcpy(format, _defdatefmt);
   return 0;
}

static int gtspace (
   const char* s
)

{
   register const char* p;

   for (p = s; *p==' '; p++);

   if (*p=='\0')
    return 0;
   else
    return -1;
}

int rwildmatch (
   const char* s,
   const char* pat,
   int c1,
   int c2
)

{
#if !defined(_M_UNIX)
   return 0;
#else /* _M_UNIX */
   char* rex = regcmp(pat, (char*) 0);
   if (rex==0)
    return 0;
   if (regex(rex, s)!=0)
    {
      free(rex);
      return 1;
    }
   else
    {
      free(rex);
      return 0;
    }
#endif /* !__linux__ */
}

static int gtday (
   const char* s
)

{
   register int i;
   char* buf;
   char pat[6];

   buf = (char*) malloc(strlen(s)+1);
   strcpy(buf, s);
   strcpy(pat, "*****");
   rdownshift(buf);
   for (i = 0; i<7; i++)
    {
        if (dayarray[i]==0 || strlen(dayarray[i])==0)
           continue;

	memcpy(&pat[1],dayarray[i],3);
	rdownshift(pat);
	if (rwildmatch(buf,pat,'*','?'))
         {
           free(buf);
	   return 0;
         }
    }
   free(buf);
   return -1;
}

static int gtmonth (
   const char* s,
   short* mm
)

{
   register int i;
   char* buf;
   char pat[6];

   buf = (char*) malloc(strlen(s)+1);
   strcpy(buf, s);
   strcpy(pat, "*****");
   rdownshift(buf);
   for (i = 0; i<12; i++)
    {
        if (monarray[i]==0 || strlen(monarray[i])==0)
           continue;

	memcpy(&pat[1],monarray[i],3);
	rdownshift(pat);
	if (rwildmatch(&buf[0],&pat[0],'*','?'))
         {
	   *mm = i + 1;
           free(buf);
	   return 0;
         }
    }
   free(buf);
   return -1;
}

static int gtpart (
   const char* fmt,
   int cnt,
   short* s
)

{
   register int i;
   register int n;
   register const char* p;

   n = 0;
   p = fmt;
   for (i = 0; *p>='0' && *p<='9' && i < cnt; i++)
      n = n*10 + (*p++ - '0');

   *s = n;
   return i;
}

static int xamfmt (
   const char* fmt,
   short* pos,
   int* isyyyy
)

{
   register const char* p;
   register int c;
   register int cnt;
   int i;

   p = fmt;
   for (i = 0; *p!='\0' && i<3;  p+=cnt)
    {
	c = *p;
	cnt = repcount(c, p);
        switch(c)
         {
         case 'D':
	 case 'd':
	      if (cnt==2)
	         pos[i++] = 1;
	     break;

        case 'M':
	case 'm':
             if (cnt==2 || cnt==3)
	        pos[i++] = 0;
	     break;

	case 'Y':
        case 'y':
	    if (cnt==2 || cnt==4)
             {
	       pos[i++] = 2;
	       *isyyyy = cnt == 4 ? 1 : 0;
             }
	    break;
        }
    }

   if (i==3)
     return 0;
   else
     return 0xfffffb44;
}

static int failmsg (int i)

{
   switch(i)
    {
    case 0:
         return 0xfffffb4b;
    case 1:
	 return 0xfffffb4a;
    case 2:
	 return 0xfffffb4c;
    default:
	 return	0xfffffb4c;
    }
}

int rdefmtmdy (
   short mdy[],
   const char* fmt,
   const char* s
)

{
   register int i;
   register int j;
   short pos[3];
   int cnt;
   int isyyyy;
   register const char* p;
   const char* days;
   char deformat[20+1];

   if (initflag==0)
    initarray();

   if (fmt==0)
    {
      fmtstr(deformat);
      fmt = deformat;
    }

   if ((cnt = xamfmt(fmt,&pos[0],&isyyyy)))
    return cnt;

   /* Scan blanks characteres 
    */
   for (p = s; *p==' '; p++);

   /* Is a null date ?
    */
   if (*p=='\0')
    {
      return -1;
    }

   /* Trata de leer el mes si es el primer miembro del formato
    */
   mdy[0] = -1;
   if (gtmonth(p, &mdy[0])!=0)
    mdy[0] = -1;
   for (i=0; i<3; i++)
    {
	if ((j = pos[i])==0 && mdy[0]>0)
	 continue;

        if (*p<'0' || *p>'9')
         {
	   if (*p++=='\0')
	    return failmsg(j);
         }
        /* Realiza el parsing de la fecha.
         */
        cnt = gtpart(p,(j==2 && isyyyy) ? 4 : 2, &mdy[j]);
        /* Si es cero y hubo digitos y es el mes o el dia,
         * Entonces tenemos una fecha invalida.
         */
        if (mdy[j]==0 && cnt>0 && (j==0 || j==1))
         {
	   return failmsg(j);
         }

#if 0
	[ebp-0x24] = cnt;
#endif
	if (j==2 && cnt<=2)
         {
	   if (mdy[2] < 10)
	      mdy[2] += 2000;
           else
	      mdy[2] += 1900;
         }
	p += cnt;
   }

  if (*p!='\0' && gtday(p)==-1 && gtspace(p)==-1)
    return failmsg(j);

 days = rleapyear(mdy[2]) ? leapdays : monthdays;
 if (mdy[0]<1 || mdy[0]>12)
    return -1205;
 else if (mdy[1]<1 || mdy[1]>days[mdy[0]])
    return -1206;
 else
    return 0;
}

int rdefmtdate (
   mjdate* jdate,
   const char* fmt,
   const char* s
)

{
   short mdy[3];
   int errnum;
   if ((errnum=rdefmtmdy(mdy, fmt, s))!=0)
    {
      DPRINTF(("rdefmtdate: errnum=%d fmt=%s s=%s\n", errnum, fmt, s));
      if (errnum==-1)
       {
         rsetnull(CDATETYPE, jdate);
         return 0;
       }
      else
       {
         return errnum;
       }
    }
   return rmdyjul(mdy, jdate);
}
