#include <memory.h>
#include <stdlib.h>
#include "mbase/Formatter.h"
#include "mbase/value.h"

static void init (
   struct Formatter* this
)

{
   this->sign_ = 0;
   this->decpt_pos_ = 0;
   this->fillchar_ = ' ';
   this->parprinted_ = 1;
   this->plusprinted_ = 1;
   this->minprinted_ = 1;
   this->dolprinted_ = 1;
   this->isigna_ = 0;
   this->monsep_ = ',';
   this->mondecm_ = '.';
   this->iprstack_ = 0;
   this->intlcnvrt_ = 0;
   memset(this->signa_,0,sizeof(this->signa_));
   memset(this->monfront_,0,sizeof(this->monfront_));
   memset(this->monback_,0,sizeof(this->monback_));
}

/* Print a character in the stack
 */
static void print (
  struct Formatter* this,
  char  c
)

{
   this->prstack_[this->iprstack_++] = c;
}

static void leading (
   struct Formatter* this,
   char c
)

{
   this->fillchar_ = c;
   if (*this->pfee_=='\0')
    {
      if (this->fillchar_!='<')
       print(this, this->fillchar_);
    }
   else
    {
      print(this, *this->pfee_--);
    }
}

static void prsi (
   struct Formatter* this,
   char* prflag,
   char neg_char,
   char pos_char,
   int sign
)

{
   short monlen;

   if (!*prflag)
    {
      if (neg_char!='$')
       {
         print (this, (char) (sign ? neg_char : pos_char));
       }
      else
       {
         monlen = strlen(&this->monfront_[0]) - 1;
         while (monlen>=0)
           print(this, (char)this->monfront_[monlen--]);
       }

      *prflag = 1;
    }
   else
    {
      if (this->fillchar_!='<')
       print(this, this->fillchar_);
    }
}

static void print_sign (
   struct Formatter* this,
   char* csign,
   int sign
)

{
   this->fillchar_ = ' ';
   if (*this->pfee_=='\0')
    {
      if (*csign=='\0')
       {
         this->isigna_ = this->isigna_ - 1;
         switch(this->signa_[this->isigna_])
          {
          case '-':
               prsi(this, &this->minprinted_,'-',' ',sign);
               break;
          case '+':
               prsi(this, &this->plusprinted_,'-','+',sign);
               break;
          case '(':
               prsi(this, &this->parprinted_,'(',' ',sign);
               break;
          case '$':
               prsi(this, &this->dolprinted_,'$','$',sign);
               break;
          }
       }
      else
       {
          print(this, ' ');
       }
    }
   else
    {
      print(this, *this->pfee_--);
    }
}

static void fmtanalysis (
   struct Formatter* this
)

{
   register char* p;

   this->erdecpoint_ = 0;
   for (p = this->fmter_ + 1; *p!='\0'; p++)
     if (*p=='.')
       this->erdecpoint_ = p;
   this->rightmost_ = p;
}

static void ldsigna (
   struct Formatter* this,
   char c
)

{
   if (this->isigna_==0 || this->signa_[this->isigna_ - 1]!=c)
    this->signa_[this->isigna_++] = c;
}

static void chksigns (
   struct Formatter* this
)

{
   int c;
   register const char* s;

   this->isigna_ = 0;
   for (s = this->fmter_ + 1; *s!='\0'; s++)
   switch(c = *s)
    {
    case '(':
         ldsigna(this, (char)c);
         this->parprinted_ = 0;
         break;
    case '-':
         ldsigna(this, (char)c);
         this->minprinted_ = 0;
         break;
    case '$':
         ldsigna(this, (char)c);
         this->dolprinted_ = 0;
         break;
    case '+':
         ldsigna(this, (char) c);
         this->plusprinted_ = 0;
         break;
    }
}

static void align (
   struct Formatter* this
)

{
   if (this->erdecpoint_!=0)
    {
      this->pfee_ = this->eedecpoint_;
      for (this->pfer_ = this->erdecpoint_ + 1; *this->pfer_!='\0'; this->pfer_++)
       {
          if (*this->pfee_=='\0')
           {
             *this->pfee_ = '0';
             this->pfee_[1] = 0;
           }

          switch(*this->pfer_)
           {
           case '#':
           case '$':
           case '&':
           case '(':
           case '*':
           case '+':
           case '-':
           case '<':
                this->pfee_++;
                break;
           }
       }
    }
   else
    {
      this->pfer_ = this->rightmost_;
      this->pfee_ = this->eedecpoint_;
    }

   this->pfer_--;
   this->pfee_--;
}

static void moninit (
   struct Formatter* this
)

{
   register char* p;

   if ((p = getenv("DBMONEY"))!=0)
    {
      register unsigned i; /* esi */

      for (i = 0; i<(sizeof(this->monfront_)-1) && *p!='\0'; i++)
       {
         if (*p=='.' || *p==',')
          {
            this->mondecm_ = *p++;
            break;
          }
   
         if (*p<'0' || *p>'9')
          {
            this->monfront_[i] = *p++;
          }
         else
          {
            this->monfront_[i] = '*';
            p++;
          }
       }

      if (i==(sizeof(this->monfront_)-1))
       this->mondecm_ = *p++;
   
      this->monfront_[i] = '\0';
      for (i = 0; i<(sizeof(this->monback_)-1); i++)
       {
         if (*p<'0' || *p>'9')
          {
            if ((this->monback_[i] = *p++)=='\0')
             break;
          }
       }

      this->monback_[i] = '\0';
    }

   if (this->mondecm_!='.' ||
       this->monfront_[0]!='$' ||
       strlen(this->monfront_)!=1 ||
       this->monback_[0]!='\0')
    this->intlcnvrt_ = 1;

   if (this->mondecm_==',')
      this->monsep_ = '.';
   else
      this->monsep_ = ',';
}

static void rpfs (
   struct Formatter* this,
   register char* buf
)

{
   if (this->dolprinted_
   &&  this->minprinted_
   &&  this->plusprinted_
   &&  this->parprinted_
   &&  *this->pfee_=='\0')
    {
      while (--this->iprstack_>=0)
        *buf++ = this->prstack_[this->iprstack_];
      *buf = '\0';
    }
   else
    {
      memset(buf, '*', this->iprstack_);
      buf[this->iprstack_] = '\0';
    }
}

int rfmtnum (
   const struct value* v,
   const char* fmt,
   char* buf
)

{
   int ndigit;
   char fmtbuf[0x52];
   char prbuf[0x52];
   char fmteebuf[0x52];
   char prevchar;
   char tmpdgts[0x20+1];
   dec_t d;
   struct Formatter formatter;

   memset(&fmtbuf[0], 0, sizeof(fmtbuf));
   memset(&prbuf[0], 0, sizeof(prbuf));
   memset(&fmteebuf[0], 0, sizeof(fmteebuf));
   init(&formatter);
   formatter.fmter_ = &fmtbuf[0];
   formatter.prstack_ = &prbuf[0];
   strcpy(formatter.fmter_ + 1, fmt);
   formatter.fmter_[0] = '\0';

   /* Encuentra el punto decimal en el formato
    */
   for (formatter.pfer_ = formatter.fmter_ + 1;
        *formatter.pfer_!='\0' && *formatter.pfer_!='.';
        formatter.pfer_++);

   /* Encuentra el numero de digitos requeridos en el formato
    */
   for (ndigit = 0; *formatter.pfer_!='\0'; formatter.pfer_++)
     switch(*formatter.pfer_)
      {
      case '#':
      case '$':
      case '&':
      case '(':
      case '*':
      case '+':
      case '-':
      case '<':
           ndigit++;
           break;
      }

   mtodecimal(v, &d);
   formatter.dgtstr_ = decfcvt_r(&d,ndigit,&formatter.decpt_pos_,&formatter.sign_,&tmpdgts[0],sizeof(tmpdgts));
   if (formatter.dgtstr_==0)
     return 0xfffffb45;

   formatter.fmtee_ = &fmteebuf[0];
   if (formatter.decpt_pos_<0)
    {
      register int nzeroes;

      formatter.eedecpoint_ = formatter.pfee_ = formatter.fmtee_ + 1;
      for (nzeroes =  - formatter.decpt_pos_; nzeroes>0; nzeroes--)
        *formatter.pfee_++ = '0';
      strcpy(formatter.pfee_, formatter.dgtstr_);
    }
   else
    {
      formatter.eedecpoint_ = &formatter.fmtee_[formatter.decpt_pos_ + 1];
      strcpy(formatter.fmtee_ + 1, formatter.dgtstr_);
    }

   fmtanalysis(&formatter);
   chksigns(&formatter);
   align(&formatter);
   moninit(&formatter);
   prevchar = ' ';
   while (*formatter.pfer_!='\0')
    {
      if (*formatter.pfer_==',' && *formatter.pfee_=='\0')
         *formatter.pfer_ = prevchar;

      switch(*formatter.pfer_)
       {
       case ',':
            print(&formatter, formatter.monsep_);
            break;
       case '.':
            print(&formatter, formatter.mondecm_);
            break;
       case '<':
            leading(&formatter, '<');
            break;
       case '*':
            leading(&formatter, '*');
            break;
       case '&':
            leading(&formatter, '0');
            break;
       case '#':
            leading(&formatter, ' ');
            break;
       case '-':
            print_sign(&formatter, &formatter.minprinted_, formatter.sign_);
            break;
       case '+':
            print_sign(&formatter, &formatter.plusprinted_, formatter.sign_);
            break;
       case '(':
            print_sign(&formatter, &formatter.parprinted_, formatter.sign_);
            break;
       case '$':
            print_sign(&formatter, &formatter.dolprinted_, formatter.sign_);
            break;
       case ')':
            print(&formatter, (char) (formatter.sign_ ?  ')' : ' '));
            break;
       default:
            print(&formatter, *formatter.pfer_);
            break;
       }

      prevchar = *formatter.pfer_--;
    }
   rpfs(&formatter, buf);
   return 0;
}
