#include <sys/types.h>
#if !defined(WIN32)
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>
#endif /* WIN32 */
#include <limits.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include "mbase/config.h"
#include "mbase/rutils.h"
#include "mbase/debug.h"


int rsystem (
   const char* cmd
)

{
#if !defined(WIN32)
   int retcode;
   msighandler_t oldint;
   msighandler_t oldquit;
   register int fd;
   register char* p;
   register pid_t pid;
   register pid_t retpid;
   char* progname;
   char* shell = NULL;

   DPRINTF(("rsystem(%s)=", cmd));
   if (shell==NULL && (shell = getenv("SHELL"))==NULL)
     shell = "/bin/sh";
   progname = shell;
   for (p = shell; *p!='\0'; )
     if (*p++=='/')
      progname = p;

   switch((pid = fork()))
    {
    case -1:
         return errno;
    case 0:
         setgid(getgid());
         setuid(getuid());
         for (fd = 3; fd<20; fd++)
           close(fd);
         execl(shell,progname,"-c",cmd,0);
         exit(0x7f);
    }

   oldint = signal(SIGINT, SIG_IGN);
   oldquit = signal(SIGQUIT, SIG_IGN);

   do
    {
      retpid = waitpid(pid, &retcode, 0);
      if (retpid==pid) 
       break;
    }
   while (retpid!=-1);

   if (retpid==-1)
     retcode = -1;
   signal(SIGINT, oldint);
   signal(SIGQUIT, oldquit);
   return retcode;
#else /* WIN32 */
#if 0
    char* progname,*p;
    char* cmdline;
    STARTUPINFO startup_info;
    PROCESS_INFORMATION procinfo;
    /* Arma la nueva linea de comandos
     */
    cmdline = cnewa(strlen(progname) + 1 + 10 + 10 + 1, char);
    memset(&startup_info, 0, sizeof(startup_info));
    startup_info.cb = sizeof(startup_info);
    sprintf(cmdline, "%s -f %ld", progname, (long)  newfd);
    fprintf(stderr, "net_accept: cmdline=%s\n", cmdline);
    if (!CreateProcess (
            0,
            cmdline,
            0, /* Process atributes */
            0, /* Thread atributes */
            TRUE, /* Handle inheritance */
            0, /* Creation flags */
            0, /* inherit the environment */
            0, /* inherit the current working directory */
            &startup_info, /* no startup info */
            &procinfo
        ))
     {
       fprintf(stderr,"netaccept: can't create process\n");
       close(newfd);
       free(cmdline);
       free(progname);
       return -1;
     }
#endif
   return system(cmd);
#endif /* WIN32 */
}
