#include <limits.h>
#include "mbase/sqltypes.h"
#include "mbase/decimal.h"
#include "mbase/float.h"
#include "mbase/datetime.h"
#include "mbase/blob.h"
#include "mbase/value.h"
#include "mbase/byutils.h"
#include "rvaldata.h"

void nullblobval (
   struct value* v,
   int sqltype
)

{
   byfill(v, sizeof(*v), 0);
   v->v_type = sqltype&0xf;
   v->v_ind = -1;
   v->v_tblob.tb_flags |= BLOBISNULL;
}

int chkprec (
   dec_t* d,
   int prec
)

{
   int prec2;

   prec2 = dec2prec(d);
   if (ACE_PRECDEC(prec)==0xff)
    {
      if (ACE_PRECTOT(prec2)>ACE_PRECTOT(prec))
         decround(d, ACE_PRECDEC(prec2) + ACE_PRECTOT(prec) - ACE_PRECTOT(prec2));
    }
   else
    {
      if (ACE_PRECDEC(prec2)>ACE_PRECDEC(prec))
         decround(d, ACE_PRECDEC(prec));

      prec2 = decprec(d);
      if ((ACE_PRECTOT(prec2) - ACE_PRECDEC(prec2))>(ACE_PRECTOT(prec) - ACE_PRECDEC(prec)))
         return -1;
    }

   return 0;
}

double todblnull (
   const struct value* v,
   short* ind
)

{
   char* p; /* esi */
   short len; /* di */

   if (v->v_ind<0)
    {
      *ind = 1;
      return +0.0;
    }

   if (((v->v_type&SQLTYPE)==SQLCHAR)
   ||  ((v->v_type&SQLTYPE)==SQLVCHAR))
    {
      for (len = v->v_len, p = v->v_charp;
           *p==' '; p++, len--);

      if (len==0)
       {
         *ind = 1;
         return +0.0;
       }
    }

   *ind = 0;
   return todouble(v);
}

long rvaldata (
   struct value* v,
   char* to,
   int sqltype,
   int sqllen
)

{
   int i;
   dec_t d;
   intrvl_t in;
   dtime_t dt;
   long l;
   short ind;
   double dbl;
   struct value temp;
   long errcode;

   if (v->v_ind<0 || (sqltype&SQLNONULL))
      return 0xfffffb37;

   if (v->v_ind>0
   &&  ((ISBLOBTYPE(sqltype) && !ISBLOBTYPE(v->v_type))
   ||   (!ISBLOBTYPE(sqltype) && ISBLOBTYPE(v->v_type))))
      return 0xfffffda0;

   toerrno = 0;
   ind = 0;
   switch(sqltype&SQLTYPE)
    {
    case SQLCHAR:
         if (v->v_ind<0)
          {
            *to = 0;
            byfill(to+1, (sqllen>0) ? (sqllen - 1) : 0, ' ');
            break;
          }

         if (v->v_len==0)
            ind = 1;

         if ((v->v_type&SQLTYPE)!=SQLCHAR
         &&  (v->v_type&SQLTYPE)!=SQLVCHAR)
          {
            if ((errcode = rvalstr(v, to, sqllen, 1))!=0)
               return errcode;
            break;
          }
         else
          {
            i = (v->v_len<=sqllen) ? v->v_len : sqllen;
            bycopy(v->v_charp, to, i);
            byfill(to + i, sqllen - i, ' ');
            break;
          }

    case SQLVCHAR:
         i = sqllen&0xff;
         if (v->v_ind<0)
          {
            *to = 1;
            byfill(to + 1, i, 0);
            break;
          }

         if (v->v_len==1
         &&  v->v_charp==0)
            ind = 1;

         if ((v->v_type&SQLTYPE)!=SQLVCHAR
         &&  (v->v_type&SQLTYPE)!=SQLCHAR)
          {
            if ((errcode = rvalstr(v, to + 1, i, 1))!=0)
               return errcode;
          }
         else
          {
            i = (v->v_len<=i) ? v->v_len : i;
            bycopy(v->v_charp, to + 1, i);
          }
         *to = i;
         break;

    case SQLSMINT:
         i = toint(v);
         if (i==(short)0x8000)
            ind = 1;

         to[0] = ((i>>8)&0xff);
         to[1] = i&0xff;
         break;

    case SQLINT:
    case SQLSERIAL:
         l = tolong(v);
         if (l==0x80000000)
            ind = 1;

         stlong(tolong(v), to);
         break;

    case SQLSMFLOAT:
         dbl = todblnull(v, &ind);
         stfltnull(dbl, to, ind);
         break;

    case SQLFLOAT:
         todblnull(v, &ind);
         stdblnull(dbl, to, ind);
         break;

    case SQLDATE:
         l = todate(v);
         if (l==0x80000000)
            ind = 1;
         stlong(l, to);
         break;

    case SQLINTERVAL:
         tointerval(v, &in, sqllen);
         if (toerrno!=0)
            break;

         if (in.in_dec.dec_pos==-1 || v->v_ind<0)
            ind = 1;

         stdecimal(&in.in_dec, to, ACE_DECLENGTH(sqllen));
         break;

    case SQLDTIME:
         todatetime(v, &dt, sqllen);
         if (toerrno!=0)
            break;

         if (dt.dt_dec.dec_pos==-1 || v->v_ind<0)
            ind = 1;

         stdecimal(&dt.dt_dec, to, ACE_DECLENGTH(sqllen));
         break;

    case SQLDECIMAL:
    case SQLMONEY:
         todecimal(v, &d);
         if (toerrno!=0)
            break;

         if (d.dec_pos==-1 || v->v_ind<0)
          {
            ind = 1;
          }
         else if (v->v_ind>=0
         &&  chkprec(&d, sqllen)!=0)
          {
            return 0xfffffb36;
          }
         stdecimal(&d, to, ACE_DECLENGTH(sqllen));
         break;

    case SQLBYTES:
    case SQLTEXT:
         if (v->v_ind<0)
          {
            v = &temp;
            nullblobval(v, sqltype&SQLTYPE);
          }
         tblobstore(&v->v_tblob, to);
         break;
    }

   if (toerrno==0
   &&  ind!=0
   &&  (sqltype&SQLNONULL))
      return 0xfffffb37;
   else
      return toerrno;
}
