#include "mbase/debug.h"
#include "mbase/rdate.h"
#include "mbase/string.h"
#include "mbase/vstring.h"
#include "mbase/integer.h"
#include "mbase/smallint.h"
#include "mbase/decimal.h"
#include "mbase/datetime.h"
#include "mbase/sqltypes.h"
#include "mbase/sqlda.h"

int descalign (
   struct msqlda* sqlda_ptr,
   int pos
)

{
   pos = rtypalign (pos, CLONGTYPE);
   return pos;
}

/************************************************************************/
/* Asigna espacio de la memoria libre para el manejo de los descriptores*/
/************************************************************************/

struct msqlda *allocdesc(int n)

{
   int size = sizeof(struct msqlda) + n * sizeof(struct msqlvar_struct);
   char *p = (char*) malloc(size);
   struct msqlda* sqlda_ptr = (struct msqlda*) p;
   memset(p, 0, size);
   sqlda_ptr->sqld = n;
   sqlda_ptr->sqlvar = (struct msqlvar_struct*) (p + sizeof(struct msqlda));
   return sqlda_ptr;
}

/************************************************************************/
/* Libera un descriptor que habia sido asignado del heap.		*/
/************************************************************************/

int freedesc (
   struct msqlda *sqlda_ptr
)

{
   int i;
   struct msqlvar_struct *sqldvar;

   if (sqlda_ptr->sqld!=0)
    {
      for (i=0,sqldvar=sqlda_ptr->sqlvar; i<sqlda_ptr->sqld; i++,sqldvar++)
        {
	   if (sqldvar->sqlname!=NULL)
            {
              free(sqldvar->sqlname);
            }
           memset (sqldvar, 0, sizeof (struct msqlvar_struct));
        }
    }
   free(sqlda_ptr);
   return 0;
}

void initsqldael (
   struct msqlda *sqlda_ptr,
   int sqld,
   struct msqlvar_struct *sqldvar
)

{
   sqlda_ptr->sqld = sqld;
   sqlda_ptr->sqlvar = sqldvar;
}

struct msqlvar_struct *findsqlname (
   struct msqlda *sqlda_ptr,
   char *sqlname
)

{
	register int i;
	struct msqlvar_struct *sqldvar;

	if (sqlda_ptr!=NULL)
	for (i=0,sqldvar=sqlda_ptr->sqlvar; i<sqlda_ptr->sqld; i++,sqldvar++)
		if (strcmp(sqldvar->sqlname,sqlname)==0)
			return sqldvar;

	return NULL;
}

struct msqlvar_struct *
findsqlstack (
   int nsqlda,
   struct msqlda *sqlda_stack[],
   char colname[]
)

{
	register int i;

	for (i=nsqlda; i!=0; i--) {
		struct msqlvar_struct *sqldvar;

		sqldvar=findsqlname(sqlda_stack[i-1],colname);
		if (sqldvar!=NULL)
			return sqldvar;
	}

	return NULL;
}

void initsqlvar (
   struct msqlvar_struct *sqldvar,
   char sqlname[],
   int sqltype,
   int sqllen,
   void *sqldata
)

{
   memset(sqldvar, 0, sizeof(struct msqlvar_struct));
   sqldvar->sqlname=sqlname;
   sqldvar->sqltype=sqltype;
   sqldvar->sqllen=sqllen;
   sqldvar->sqldata=(char*)sqldata;
}

void dumpcvar (
   char cname[],
   int ctype,
   int clength,
   void *cvar
)

{
   struct msqlvar_struct sqldvar;

   initsqlvar(&sqldvar,cname,ctype,clength,cvar);
   dumpsqlvar(&sqldvar);
}

void dumpsqlvar(struct msqlvar_struct *sqldvar)

{
	char cadena[512];

	if (sqldvar->sqlname!=NULL)
		debug_printf("%s=", sqldvar->sqlname);
	if ((sqldvar->sqltype&0xFF)>15 &&
	    risnull(sqldvar->sqltype, sqldvar->sqldata))
		debug_printf("NULL");
	else
	switch(sqldvar->sqltype&0xFF) {
	case SQLCHAR:
	     debug_printf("%s|", sqldvar->sqldata);
	     break;

	case SQLINT:
	     debug_printf("%ld", mldlong((const unsigned char*)sqldvar->sqldata));
	     break;

	case SQLSMINT:
	     debug_printf("%d", ldshort((short*) sqldvar->sqldata));
	     break;

	case SQLFLOAT:
	     break;

	case SQLSMFLOAT:
	     break;

	case SQLDECIMAL:
	     break;

	case SQLMONEY:
	     break;

	case CCHARTYPE:
	case CSTRINGTYPE:
	     debug_printf("%s|", sqldvar->sqldata);
	     break;

	case CFIXCHARTYPE:
	     mldchar (sqldvar->sqldata, sqldvar->sqllen, cadena);
	     debug_printf("%s|", cadena);
	     break;

	case CSHORTTYPE:
	     debug_printf("%d", *((short*) sqldvar->sqldata));
	     break;

	case CINTTYPE:
	     debug_printf("%d", *((int*) sqldvar->sqldata));
	     break;

	case CLONGTYPE:
	     debug_printf("%ld", *((long*) sqldvar->sqldata));
	     break;

	case CFLOATTYPE:
	     debug_printf("%f", *((float *) sqldvar->sqldata));
	     break;

	case CDOUBLETYPE:
	     debug_printf("%f", *((double *) sqldvar->sqldata));
	     break;

	case CMONEYTYPE:
	     dectoasc((const dec_t*)sqldvar->sqldata, cadena, sizeof(cadena)-1,-1);
	     cadena[sizeof(cadena)-1]='\0';
	     cadena [byleng (cadena, strlen(cadena))] = 0;
	     debug_printf("$%s", cadena);
	     break;

	case CDECIMALTYPE:
	     dectoasc((const dec_t*)sqldvar->sqldata, cadena, sizeof(cadena)-1,-1);
	     cadena[sizeof(cadena)-1]='\0';
	     cadena [byleng (cadena, strlen(cadena))] = 0;
	     debug_printf("%s", cadena);
	     break;

	case CDATETYPE:
	     rfmtdate(*((mjdate*) sqldvar->sqldata), "dd-mm-yyyy", cadena);
	     debug_printf("%s", cadena);
	     break;

	case CDTIMETYPE:
	     dttoasc((dtime_t*)sqldvar->sqldata,cadena);
	     debug_printf("%s", cadena);
	     break;

	case CINVTYPE:
	     intoasc((intrvl_t*)sqldvar->sqldata,cadena);
	     debug_printf("%s", cadena);
	     break;

	case CVSTRINGTYPE:
	     debug_printf("%s", ((mvstring)sqldvar)->data);
	     break;

	default:
	     printf("dumpsqlvar: Tipo de datos no soportado %d\n",
		    sqldvar->sqltype);
		    exit(-1);
	}
}

void dumpsqlda(struct msqlda *sqlda_ptr)

{
	int i;

	for (i=0; i<sqlda_ptr->sqld; i++) {
		dumpsqlvar(&sqlda_ptr->sqlvar[i]);
		if (i+1!=sqlda_ptr->sqld)
			debug_printf((","));
#if 0
                if ((i+1)%4==0)
			DPRINTF(("%c", 10));
#endif
	}
}
