#ifndef __MBASE_SQLTYPES_H__
#define __MBASE_SQLTYPES_H__
#include "mbase/config.h"

#ifdef WIN32
#define MBASE_LACKS_LONGLONG_T
#endif /* !WIN32 */

#if defined(OSF1) || defined(WIN32) || defined(HPUX)
#include <float.h>
#endif /* OSF1 */

/**
 *  @file    Basic_Types.h
 *
 *  Basic_Types.h,v 4.83 2001/12/05 03:41:04 nanbor Exp
 *
 *  @author David L. Levine
 *
 *  #defines the list of preprocessor macros below.  The config.h file can
 *  pre-define any of these to short-cut the definitions.  This is usually
 *  only necessary if the preprocessor does all of its math using integers.
 *
 *  Sizes of built-in types:
 *    - MBASE_SIZEOF_CHAR
 *    - MBASE_SIZEOF_WCHAR
 *    - MBASE_SIZEOF_SHORT
 *    - MBASE_SIZEOF_INT
 *    - MBASE_SIZEOF_LONG
 *    - MBASE_SIZEOF_LONG_LONG
 *    - MBASE_SIZEOF_VOID_P
 *    - MBASE_SIZEOF_FLOAT
 *    - MBASE_SIZEOF_DOUBLE
 *    - MBASE_SIZEOF_LONG_DOUBLE
 *
 *  Wrappers for built-in types of specific sizes:
 *    - MBASE_USHORT16 (For backward compatibility.  Use MBASE_UINT16 instead.)
 *    - MBASE_INT16
 *    - MBASE_UINT16
 *    - MBASE_INT32
 *    - MBASE_UINT32
 *    - MBASE_UINT64
 *  (Note: MBASE_INT64 is not defined, because there is no MBASE_LongLong for
 *   platforms that don't have a native 8-byte integer type.)
 *
 *  Byte-order (endian-ness) determination:
 *    MBASE_BYTE_ORDER, to either MBASE_BIG_ENDIAN or MBASE_LITTLE_ENDIAN
 *
 *
 */


/* A char always has 1 byte, by definition.
 */
# define MBASE_SIZEOF_CHAR 1

# if defined (MBASE_HAS_WCHAR)
#   if !defined (MBASE_SIZEOF_WCHAR)
#     if defined (MBASE_HAS_XPG4_MULTIBYTE_CHAR)
#       define MBASE_SIZEOF_WCHAR sizeof (wchar_t)
#     else
#       define MBASE_SIZEOF_WCHAR 0
#     endif /* MBASE_HAS_XPG4_MULTIBYTE_CHAR */
#   endif /* !MBASE_SIZEOF_WCHAR */
# endif /* MBASE_HAS_WCHAR */

/* The number of bytes in a short.
 */
# if !defined (MBASE_SIZEOF_SHORT)
#   if (USHRT_MAX) == 255U
#     define MBASE_SIZEOF_SHORT 1
#   elif (USHRT_MAX) == 65535U
#     define MBASE_SIZEOF_SHORT 2
#   elif (USHRT_MAX) == 4294967295U
#     define MBASE_SIZEOF_SHORT 4
#   elif (USHRT_MAX) == 18446744073709551615U
#     define MBASE_SIZEOF_SHORT 8
#   else
#     error: unsupported short size, must be updated for this platform!
#   endif /* USHRT_MAX */
# endif /* !defined (MBASE_SIZEOF_SHORT) */

/* The number of bytes in an int.
 */
# if !defined (MBASE_SIZEOF_INT)
#   if (UINT_MAX) == 65535U
#     define MBASE_SIZEOF_INT 2
#   elif (UINT_MAX) == 4294967295U
#     define MBASE_SIZEOF_INT 4
#   elif (UINT_MAX) == 18446744073709551615U
#     define MBASE_SIZEOF_INT 8
#   else
#     error: unsupported int size, must be updated for this platform!
#   endif /* UINT_MAX */
# endif /* !defined (MBASE_SIZEOF_INT) */

/* The number of bytes in a long.
 * NOTE - since preprocessors only need to do integer math, this is a likely
 * place for a preprocessor to not properly support being able to figure out
 * the proper size.  HP aC++ and GNU gcc have this difficulty so watch out.
 */
# if !defined (MBASE_SIZEOF_LONG)
#   if (ULONG_MAX) == 65535UL
#     define MBASE_SIZEOF_LONG 2
#   elif ((ULONG_MAX) == 4294967295UL)
#     define MBASE_SIZEOF_LONG 4
#   elif ((ULONG_MAX) == 18446744073709551615UL)
#     define MBASE_SIZEOF_LONG 8
#   else
#     error: unsupported long size, must be updated for this platform!
#   endif /* ULONG_MAX */
# endif /* !defined (MBASE_SIZEOF_LONG) */

/* The number of bytes in a long long.
 */
# if !defined (MBASE_SIZEOF_LONG_LONG)
#   if defined (MBASE_LACKS_LONGLONG_T)
#     define MBASE_SIZEOF_LONG_LONG 8
#   else  /* ! MBASE_WIN32 && ! MBASE_LACKS_LONGLONG_T */
#     if MBASE_SIZEOF_LONG == 8
#       define MBASE_SIZEOF_LONG_LONG 8
       typedef unsigned long MBASE_UINT64;
#     elif defined (ULLONG_MAX) && !defined (__GNUG__)
       /* Some compilers use ULLONG_MAX and others, e.g. Irix, use
        * ULONGLONG_MAX.
        */
#       if (ULLONG_MAX) == 18446744073709551615ULL
#         define MBASE_SIZEOF_LONG_LONG 8
#       elif (ULLONG_MAX) == 4294967295ULL
#         define MBASE_SIZEOF_LONG_LONG 4
#       else
#         error Unsupported long long size needs to be updated for this platform
#       endif
       typedef unsigned long long MBASE_UINT64;
#     elif defined (ULONGLONG_MAX) && !defined (__GNUG__)
       /* Irix 6.x, for example.
        */
#       if (ULONGLONG_MAX) == 18446744073709551615ULL
#         define MBASE_SIZEOF_LONG_LONG 8
#       elif (ULONGLONG_MAX) == 4294967295ULL
#         define MBASE_SIZEOF_LONG_LONG 4
#       else
#         error Unsupported long long size needs to be updated for this platform
#       endif
       typedef unsigned long long MBASE_UINT64;
#     else
       /* MBASE_SIZEOF_LONG_LONG is not yet known, but the platform doesn't
        * claim MBASE_LACKS_LONGLONG_T, so assume it has 8-byte long longs.
        */
#       define MBASE_SIZEOF_LONG_LONG 8
#       if defined (sun) && !defined (MBASE_LACKS_U_LONGLONG_T)
         /* sun #defines u_longlong_t, maybe other platforms do also.
          * Use it, at least with g++, so that its -pedantic doesn't
          * complain about no ANSI C++ long long.
          */
         typedef u_longlong_t MBASE_UINT64;
#       else
         /* LynxOS 2.5.0 and Linux don't have u_longlong_t.
          */
         typedef unsigned long long MBASE_UINT64;
#       endif /* sun */
#     endif /* ULLONG_MAX && !__GNUG__ */
#   endif /* ! MBASE_WIN32 && ! MBASE_LACKS_LONGLONG_T */
# endif /* !defined (MBASE_SIZEOF_LONG_LONG) */

/* The sizes of the commonly implemented types are now known.  Set up
 * typedefs for whatever we can.  Some of these are needed for certain cases
 * of MBASE_UINT64, so do them before the 64-bit stuff.
 */

# if MBASE_SIZEOF_SHORT == 2
  typedef short MBASE_INT16;
  typedef unsigned short MBASE_UINT16;
# elif MBASE_SIZEOF_INT == 2
  typedef int MBASE_INT16;
  typedef unsigned int MBASE_UINT16;
# elif (MBASE_SIZEOF_SHORT) == 4 && defined(_CRAYMPP)
  /* mpp cray - uses Alpha processors
   *   Use the real 32-bit quantity for MBASE_INT32's, and use a "long"
   *   for MBASE_INT16's.  This gets around conflicts with size_t in some ACE
   *   method signatures, among other things.
   */
  typedef long MBASE_INT16;
  typedef unsigned long MBASE_UINT16;
  typedef short MBASE_INT32;
  typedef unsigned short MBASE_UINT32;
# elif (MBASE_SIZEOF_SHORT) == 8 && defined(_UNICOS)
  /* vector cray - hard 64-bit, all 64 bit types
   */
  typedef short MBASE_INT16;
  typedef unsigned short MBASE_UINT16;
# else
#   error Have to add to the MBASE_UINT16 type setting
# endif

typedef MBASE_UINT16 MBASE_USHORT16;

/* Define a pseudo wide character type when wchar is not supported so we
 * can support basic wide character string operations.
 */

# if defined (MBASE_HAS_WCHAR)
#   define MBASE_WINT_T wint_t
#   define MBASE_WCHAR_T wchar_t
# else
#   define MBASE_WINT_T MBASE_USHORT16
#   define MBASE_WCHAR_T MBASE_USHORT16
# endif /* MBASE_HAS_WCHAR */

# if MBASE_SIZEOF_INT == 4
  typedef int MBASE_INT32;
  typedef unsigned int MBASE_UINT32;
#   if defined (__KCC) && !defined (MBASE_LACKS_LONGLONG_T)
  typedef unsigned long long MBASE_UINT64;
#   endif /* __KCC */
# elif MBASE_SIZEOF_LONG == 4
  typedef long MBASE_INT32;
  typedef unsigned long MBASE_UINT32;
# elif (MBASE_SIZEOF_INT) == 8 && defined(_UNICOS)
  /* vector cray - hard 64-bit, all 64 bit types
   */
#   if !defined(_CRAYMPP)
      typedef int MBASE_INT32;
      typedef unsigned int MBASE_UINT32;
#   endif
  typedef unsigned long long MBASE_UINT64;
# else
#   error Have to add to the MBASE_UINT32 type setting
# endif

/* The number of bytes in a void *.
 */
# ifndef MBASE_SIZEOF_VOID_P
#   define MBASE_SIZEOF_VOID_P MBASE_SIZEOF_LONG
# endif /* MBASE_SIZEOF_VOID_P */

/* Type for doing arithmetic on pointers ... as elsewhere, we assume
 * that unsigned versions of a type are the same size as the signed
 * version of the same type.
 */
#if MBASE_SIZEOF_VOID_P == MBASE_SIZEOF_INT
# if defined (__SUNPRO_CC)
    /* For unknown reasons, Sun CC 5.0 won't allow a reintepret cast
     * of a 64-bit pointer to a 64-bit int.
     */
    typedef u_long mptr_arith_t;
# else  /* ! __SUNPRO_CC */
    typedef u_int mptr_arith_t;
# endif /* ! __SUNPRO_CC */
#elif MBASE_SIZEOF_VOID_P == MBASE_SIZEOF_LONG
  typedef ptrdiff_t mptr_arith_t;
#elif MBASE_SIZEOF_VOID_P == MBASE_SIZEOF_LONG_LONG
  typedef u_long long mptr_arith_t;
#else
# error "Can't find a suitable type for doing pointer arithmetic."
#endif /* MBASE_SIZEOF_VOID_P */

/* Conversions from MBASE_UINT64 to MBASE_UINT32.  MBASE_CU64_TO_CU32 should
 * be used on const MBASE_UINT64's.
 */
# if defined (MBASE_LACKS_LONGLONG_T)
#   define MBASE_U64_TO_U32(n) ((n).lo ())
#   define MBASE_CU64_TO_CU32(n) ((n).lo ())
# else  /* ! MBASE_LACKS_LONGLONG_T */
#   define MBASE_U64_TO_U32(n) (MBASE_static_cast (MBASE_UINT32, (n)))
#   define MBASE_CU64_TO_CU32(n) \
     (MBASE_static_cast (MBASE_CAST_CONST MBASE_UINT32, (n)))
# endif /* ! MBASE_LACKS_LONGLONG_T */

/* 64-bit literals require special marking on some platforms.
 */
# if defined (MBASE_LACKS_LONGLONG_T)
    /* Can only specify 32-bit arguments.
     */
#   define MBASE_UINT64_LITERAL(n) (MBASE_U_LongLong (n))
      /* This one won't really work, but it'll keep
       * some compilers happy until we have better support
       */
#   define MBASE_INT64_LITERAL(n) (MBASE_U_LongLong (n))
# elif defined (MBASE_WIN32)
#  if defined (__IBMCPP__) && (__IBMCPP__ >= 400)
#   define MBASE_UINT64_LITERAL(n) n ## LL
#   define MBASE_INT64_LITERAL(n) n ## LL
#  elif defined (__MINGW32__)
#   define MBASE_UINT64_LITERAL(n) n ## ull
#   define MBASE_INT64_LITERAL(n) n ## ll
#  else
#   define MBASE_UINT64_LITERAL(n) n ## ui64
#   define MBASE_INT64_LITERAL(n) n ## i64
#  endif /* defined (__IBMCPP__) && (__IBMCPP__ >= 400) */
# else  /* ! MBASE_WIN32  &&  ! MBASE_LACKS_LONGLONG_T */
#   define MBASE_UINT64_LITERAL(n) n ## ull
#   define MBASE_INT64_LITERAL(n) n ## ll
# endif /* ! MBASE_WIN32  &&  ! MBASE_LACKS_LONGLONG_T */

#if !defined (MBASE_UINT64_FORMAT_SPECIFIER)
# define MBASE_UINT64_FORMAT_SPECIFIER MBASE_LIB_TEXT ("%llu")
#endif /* MBASE_UINT64_FORMAT_SPECIFIER */

#if !defined (MBASE_INT64_FORMAT_SPECIFIER)
# define MBASE_INT64_FORMAT_SPECIFIER MBASE_LIB_TEXT ("%lld")
#endif /* MBASE_INT64_FORMAT_SPECIFIER */

#if !defined (MBASE_SSIZE_T_FORMAT_SPECIFIER)
# define MBASE_SSIZE_T_FORMAT_SPECIFIER MBASE_LIB_TEXT ("%d")
#endif /* MBASE_SSIZE_T_FORMAT_SPECIFIER */

#if !defined (MBASE_SIZE_T_FORMAT_SPECIFIER)
# define MBASE_SIZE_T_FORMAT_SPECIFIER MBASE_LIB_TEXT ("%u")
#endif /* MBASE_SIZE_T_FORMAT_SPECIFIER */

// Cast from UINT64 to a double requires an intermediate cast to INT64
// on some platforms.
# if defined (MBASE_LACKS_LONGLONG_T)
   // Only use the low 32 bits.
#   define MBASE_UINT64_DBLCAST_ADAPTER(n) MBASE_U64_TO_U32 (n)
# elif defined (MBASE_WIN32)
#   define MBASE_UINT64_DBLCAST_ADAPTER(n) MBASE_static_cast (__int64, n)
# else  /* ! MBASE_WIN32 && ! MBASE_LACKS_LONGLONG_T */
#   define MBASE_UINT64_DBLCAST_ADAPTER(n) (n)
# endif /* ! MBASE_WIN32 && ! MBASE_LACKS_LONGLONG_T */


// The number of bytes in a float.
# ifndef MBASE_SIZEOF_FLOAT
#   if FLT_MAX_EXP == 128
#     define MBASE_SIZEOF_FLOAT 4
#   elif FLT_MAX_EXP == 1024
#     define MBASE_SIZEOF_FLOAT 8
#   else
#     error: unsupported float size, must be updated for this platform!
#   endif /* FLT_MAX_EXP */
# endif /* MBASE_SIZEOF_FLOAT */

/* The number of bytes in a double.
 */
# ifndef MBASE_SIZEOF_DOUBLE
#   if DBL_MAX_EXP == 128
#     define MBASE_SIZEOF_DOUBLE 4
#   elif DBL_MAX_EXP == 1024
#     define MBASE_SIZEOF_DOUBLE 8
#   else
#     error: unsupported double size, must be updated for this platform!
#   endif /* DBL_MAX_EXP */
# endif /* MBASE_SIZEOF_DOUBLE */

/* The number of bytes in a long double.
 */
# ifndef MBASE_SIZEOF_LONG_DOUBLE
#   if LDBL_MAX_EXP == 128
#     define MBASE_SIZEOF_LONG_DOUBLE 4
#   elif LDBL_MAX_EXP == 1024
#     define MBASE_SIZEOF_LONG_DOUBLE 8
#   elif LDBL_MAX_EXP == 16384
#     if defined (LDBL_DIG)  &&  LDBL_DIG == 18
#       if defined (__ia64)
#         define MBASE_SIZEOF_LONG_DOUBLE 16
#       else /* ! __ia64 */
#       define MBASE_SIZEOF_LONG_DOUBLE 12
#       endif /* __ia64 */
#     else  /* ! LDBL_DIG  ||  LDBL_DIG != 18 */
#       define MBASE_SIZEOF_LONG_DOUBLE 16
#     endif /* ! LDBL_DIG  ||  LDBL_DIG != 18 */
#   else
#     error: unsupported double size, must be updated for this platform!
#   endif /* LDBL_MAX_EXP */
# endif /* MBASE_SIZEOF_LONG_DOUBLE */

/* Max and min sizes for the ACE integer types.
 */
#define MBASE_CHAR_MAX 0x7F
#define MBASE_CHAR_MIN -(MBASE_CHAR_MAX)-1
#define MBASE_OCTET_MAX 0xFF
#define MBASE_INT16_MAX 0x7FFF
#define MBASE_INT16_MIN -(MBASE_INT16_MAX)-1
#define MBASE_UINT16_MAX 0xFFFF
#define MBASE_WCHAR_MAX MBASE_UINT16_MAX
#define MBASE_INT32_MAX 0x7FFFFFFF
#define MBASE_INT32_MIN -(MBASE_INT32_MAX)-1
#define MBASE_UINT32_MAX 0xFFFFFFFF
#define MBASE_INT64_MAX MBASE_INT64_LITERAL(0x7FFFFFFFFFFFFFFF)
#define MBASE_INT64_MIN -(MBASE_INT64_MAX)-1
#define MBASE_UINT64_MAX MBASE_UINT64_LITERAL(0xFFFFFFFFFFFFFFFF)
#define MBASE_FLT_MAX 3.402823466e+38F
#define MBASE_DBL_MAX 1.7976931348623158e+308

/* Byte-order (endian-ness) determination.
 */
# if defined (BYTE_ORDER)
#   if (BYTE_ORDER == LITTLE_ENDIAN)
#     define MBASE_LITTLE_ENDIAN 0x0123
#     define MBASE_BYTE_ORDER MBASE_LITTLE_ENDIAN
#   elif (BYTE_ORDER == BIG_ENDIAN)
#     define MBASE_BIG_ENDIAN 0x3210
#     define MBASE_BYTE_ORDER MBASE_BIG_ENDIAN
#   else
#     error: unknown BYTE_ORDER!
#   endif /* BYTE_ORDER */
# elif defined (__BYTE_ORDER)
#   if (__BYTE_ORDER == __LITTLE_ENDIAN)
#     define MBASE_LITTLE_ENDIAN 0x0123
#     define MBASE_BYTE_ORDER MBASE_LITTLE_ENDIAN
#   elif (__BYTE_ORDER == __BIG_ENDIAN)
#     define MBASE_BIG_ENDIAN 0x3210
#     define MBASE_BYTE_ORDER MBASE_BIG_ENDIAN
#   else
#     error: unknown __BYTE_ORDER!
#   endif /* __BYTE_ORDER */
# else /* ! BYTE_ORDER && ! __BYTE_ORDER */
  // We weren't explicitly told, so we have to figure it out . . .
#   if defined (i386) || defined (__i386__) || defined (_M_IX86) || \
     defined (vax) || defined (__alpha) || defined (__LITTLE_ENDIAN__)
    // We know these are little endian.
#     define MBASE_LITTLE_ENDIAN 0x0123
#     define MBASE_BYTE_ORDER MBASE_LITTLE_ENDIAN
#   else
    // Otherwise, we assume big endian.
#     define MBASE_BIG_ENDIAN 0x3210
#     define MBASE_BYTE_ORDER MBASE_BIG_ENDIAN
#   endif
# endif /* ! BYTE_ORDER && ! __BYTE_ORDER */


#ifndef M_CCHARTYPE
 /* C language types */

#define M_CCHARTYPE	100
#define M_CSHORTTYPE	101
#define M_CINTTYPE	102
#define M_CLONGTYPE	103
#define M_CFLOATTYPE	104
#define M_CDOUBLETYPE	105
#define M_CDECIMALTYPE	107
#define M_CFIXCHARTYPE	108
#define M_CSTRINGTYPE	109
#define M_CDATETYPE	110
#define M_CMONEYTYPE	111
#define M_CDTIMETYPE	112
#define M_CLOCATORTYPE    113
#define M_CVCHARTYPE	114
#define M_CINVTYPE	115
#define M_CFILETYPE	116
#define M_CVSTRINGTYPE	117


#define USERCOLL(x)	((x))


/*
 * Define all possible database types
 *   include C-ISAM types here as well as in isam.h
 */

#define M_SQLCHAR		0
#define M_SQLSMINT	1
#define M_SQLINT		2
#define M_SQLFLOAT	3
#define M_SQLSMFLOAT	4
#define M_SQLDECIMAL	5
#define M_SQLSERIAL	6
#define M_SQLDATE		7
#define M_SQLMONEY	8
#define M_SQLNULL		9
#define M_SQLDTIME	10
#define M_SQLBYTES	11
#define M_SQLTEXT		12
#define M_SQLVCHAR	13
#define M_SQLINTERVAL	14
#define M_SQLVSTRING	15
#define M_SQLTYPE		0xFF	/* type mask		*/
#define M_SQLNONULL	0x100	/* disallow nulls	*/
#define M_SQLMAXTYPES	16

/* SQL types used in interpreter
 */
#define SQLADDR		0x4000
#define	SQLFORM		0x4001
#define	SQLCURSOR	0x4002
#define	SQLFUNCTION	0x4003
#define	SQLRECORD	0x4005
#define	SQLCOLUMN	0x4006
#define SQLWINDOW	0x400d
#define SQLREPORT	0x4010
#define	SQLARRAY	0x8000

/* this is not a real type but a flag to show that the
 * value is from a host variable
 */
#define SQLHOST		01000
#define SQLNETFLT	02000

#define SIZCHAR		1
#define SIZSMINT	2
#define SIZINT		4
#define SIZFLOAT	(sizeof(double))
#define SIZSMFLOAT	(sizeof(float))
#define SIZDECIMAL	17	/* decimal(32) */
#define SIZSERIAL	4
#define SIZDATE		4
#define SIZMONEY	17	/* decimal(32) */
#define SIZDTIME	7	/* decimal(12,0) */
#define SIZVCHAR	1

#define ISDECTYPE(t) (ISSQLTYPE(t) && (((t)&M_SQLTYPE)==M_SQLDECIMAL || ((t)&M_SQLTYPE)==M_SQLMONEY || ((t)&M_SQLTYPE) == M_SQLDTIME || ((t)&M_SQLTYPE) == M_SQLINTERVAL)) 
#define ISBLOBTYPE(type)   (ISBYTESTYPE (type) || ISTEXTTYPE(type))
#define ISBYTESTYPE(type)  (ISSQLTYPE(type) && ((type) & M_SQLTYPE) == M_SQLBYTES)
#define ISTEXTTYPE(type)  (ISSQLTYPE(type) && ((type) & M_SQLTYPE) == M_SQLTEXT)
#define ISBLOBCTYPE(type) (ISLOCTYPE(type) || ISFILETYPE(type)) 
#define ISLOCTYPE(type) ((type) == M_CLOCATORTYPE) 
#define ISFILETYPE(type) ((type) == M_CFILETYPE) 
#define ISVCTYPE(t)	(ISSQLTYPE(t) && ((t) & M_SQLTYPE) == M_SQLVCHAR)

#define ISSQLTYPE(t)	((t) >= M_SQLCHAR && ((t)&M_SQLTYPE) < M_SQLMAXTYPES)

#define DEFDECIMAL	9	/* default decimal(16) size */
#define DEFMONEY	9	/* default decimal(16) size */
#define DATENULL	0x80000000L
#define DATEDOOM 	0x80000001L

#define SYSPUBLIC	"public"


#endif /* CCHARTYPE */

EXTERN_C MExport int mrtoctype (
  int sqltype
);

EXTERN_C MExport int mrtosqltype (
  int ctype
);

EXTERN_C MExport int mrtypwidth (
   int sqltype,
   int sqlprec
);

EXTERN_C int MExport mrsetnull (
   int sqltype,
   void* sqldata
);

EXTERN_C int MExport mrisnull (
   int sqltype,
   const void* sqldata
);

EXTERN_C int MExport mrtypalign (
   int pos,
   int sqltype
);

EXTERN_C int MExport mrtypmsize (
   int sqltype,
   int sqlprec
);

EXTERN_C int MExport mrtyplength (
   int sqltype,
   int sqlprec
);

#ifndef NO_INFORMIX_MAP

#define CCHARTYPE	M_CCHARTYPE
#define CSHORTTYPE	M_CSHORTTYPE
#define CINTTYPE	M_CINTTYPE
#define CLONGTYPE	M_CLONGTYPE
#define CFLOATTYPE	M_CFLOATTYPE
#define CDOUBLETYPE	M_CDOUBLETYPE
#define CDECIMALTYPE	M_CDECIMALTYPE
#define CFIXCHARTYPE	M_CFIXCHARTYPE
#define CSTRINGTYPE	M_CSTRINGTYPE
#define CDATETYPE	M_CDATETYPE
#define CMONEYTYPE	M_CMONEYTYPE
#define CDTIMETYPE	M_CDTIMETYPE
#define CLOCATORTYPE    M_CLOCATORTYPE
#define CVCHARTYPE	M_CVCHARTYPE
#define CINVTYPE	M_CINVTYPE
#define CFILETYPE	M_CFILETYPE
#define CVSTRINGTYPE M_CVSTRINGTYPE


#define USERCOLL(x)	((x))


/*
 * Define all possible database types
 *   include C-ISAM types here as well as in isam.h
 */

#define SQLCHAR		M_SQLCHAR
#define SQLSMINT	M_SQLSMINT
#define SQLINT		M_SQLINT
#define SQLFLOAT	M_SQLFLOAT
#define SQLSMFLOAT	M_SQLSMFLOAT
#define SQLDECIMAL	M_SQLDECIMAL
#define SQLSERIAL	M_SQLSERIAL
#define SQLDATE		M_SQLDATE
#define SQLMONEY	M_SQLMONEY
#define SQLNULL		M_SQLNULL
#define SQLDTIME	M_SQLDTIME
#define SQLBYTES	M_SQLBYTES
#define SQLTEXT		M_SQLTEXT
#define SQLVCHAR	M_SQLVCHAR
#define SQLINTERVAL	M_SQLINTERVAL
#define SQLVSTRING	M_SQLVSTRING
#define SQLTYPE		M_SQLTYPE
#define SQLNONULL	M_SQLNONULL
#define SQLMAXTYPES	M_SQLMAXTYPES


#define rtoctype mrtoctype
#define rtosqltype mrtosqltype
#define rtypwidth mrtypwidth
#define rsetnull mrsetnull
#define risnull mrisnull
#define rtypalign mrtypalign
#define rtypmsize mrtypmsize
#define rtyplength mrtyplength
#endif /* NO_INFORMIX_MAP */

#endif /* __MBASE_SQLTYPES_H__ */
