#include <assert.h>
#include <stdio.h>
#include <limits.h>
#include "mbase/byutils.h"
#include "mbase/debug.h"
#include "mbase/rdate.h"
#include "mbase/rmoney.h"
#include "mbase/float.h"
#include "mbase/sqlda.h"
#include "mbase/value.h"

static int toerrnog;

int* mtoerrno_r()

{
   return &toerrnog;
}

int mtointerval (
   const struct value* v,
   intrvl_t* in,
   int qual
)

{
   char* to;

   if (qual==0)
      qual = v->v_prec;

   toerrno = 0;
   if (v->v_ind<0)
    {
      in->in_dec.dec_pos = -1;
      in->in_dec.dec_exp = 0;
      in->in_dec.dec_ndgts = 0;
      return 0;
    }

   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR: 
    case SQLVCHAR: 
         to = (char*)malloc(sizeof(char)*(v->v_len + 1));
         memcpy(to, v->v_charp, v->v_len);
         to[v->v_len] = '\0';
         DPRINTF(("tointerval: from=%s qual=%d\n", to, qual));
         toerrno = mrstrinv(to,&in->in_dec,qual);
         free(to);
         break;

    case SQLINTERVAL: 
         mrinvcopy(&v->v_decimal,v->v_prec,&in->in_dec,qual);
         break;

    case SQLDECIMAL: 
         mdeccopy(&v->v_decimal,&in->in_dec);
         break;

    case SQLBYTES: 
    case SQLTEXT: 
         in->in_dec.dec_pos = -1;
         toerrno = 0xfffffda0;
         break;

    default:
         toerrno = -0x000004ec;
         break;
    }
   in->in_qual = qual;
   return 0;
}

int mtointerval_r (
   const struct value* v,
   intrvl_t* in,
   int qual,
   int *err
)

{
   char* to;

   if (qual==0)
      qual = v->v_prec;

   *err = 0;
   if (v->v_ind<0)
    {
      in->in_dec.dec_pos = -1;
      in->in_dec.dec_exp = 0;
      in->in_dec.dec_ndgts = 0;
      return 0;
    }

   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR: 
    case SQLVCHAR: 
         to = (char*)malloc(sizeof(char)*(v->v_len + 1));
         memcpy(to, v->v_charp, v->v_len);
         to[v->v_len] = '\0';
         DPRINTF(("tointerval: from=%s qual=%d\n", to, qual));
         *err = mrstrinv(to,&in->in_dec,qual);
         free(to);
         break;

    case SQLINTERVAL: 
         mrinvcopy(&v->v_decimal,v->v_prec,&in->in_dec,qual);
         break;

    case SQLDECIMAL: 
         mdeccopy(&v->v_decimal,&in->in_dec);
         break;

    case SQLBYTES: 
    case SQLTEXT: 
         in->in_dec.dec_pos = -1;
         *err = 0xfffffda0;
         break;

    default:
         *err = -0x000004ec;
         break;
    }
   in->in_qual = qual;
   return 0;
}

int mtodatetime (
   const struct value* v,
   dtime_t* dt,
   int qual
)

{
   char* to;
   int err = 0;

   if (qual==0)
      qual = v->v_prec;

   if (v->v_ind<0)
    {
      dt->dt_dec.dec_pos = -1;
      dt->dt_dec.dec_exp = 0;
      dt->dt_dec.dec_ndgts = 0;
      return 0;
    }

   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR:
    case SQLVCHAR: {
            to = (char*)malloc(sizeof(char)*(v->v_len + 1));
            memcpy(to, v->v_charp, v->v_len);
            to[v->v_len] = '\0';
            DPRINTF(("todatetime: to=%s\n", to));
            err = mrstrdtime(to, &dt->dt_dec, qual);
            free(to);
            break;
         }

    case SQLDTIME:
         mrdtextend(&v->v_decimal, v->v_prec, &dt->dt_dec, qual);
         break;

    case SQLDATE:
         mrdatedt(v->v_long, &dt->dt_dec, qual);
         break;

    case SQLBYTES:
    case SQLTEXT:
         dt->dt_dec.dec_pos = -1;
         err = -0x00000260;
         break;

    default:
         err = -0x000004ec;
         break;
    }

   dt->dt_qual = qual;
   return err;
}

int mtodecimal (
   const struct value* v,
   dec_t* d
)

{
   char* from = 0;
   int err = 0;
   if (v->v_ind<0)
    {
      d->dec_pos = -1;
      d->dec_exp = 0;
      d->dec_ndgts = 0;
      return 0;
    }

   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR:
    case SQLVCHAR:
         if (v->v_len>0)
          {
            from = cnewa(v->v_len, char);
          }
         memcpy(from, v->v_charp, v->v_len);
         mrstrdec(from, v->v_len);
         if (mdeccvasc(from, v->v_len, d)!=0)
            err = -0x000004bd;
         if (from) free(from);
         break;

    case SQLVSTRING:
         from = v->v_string->data;
         if (mdeccvasc(from, v->v_string->len, d)!=0)
            err = -0x000004bd;
         break;

    case SQLSMINT:
         mdeccvint(v->v_int, d);
         break;

    case SQLINT:
    case SQLSERIAL:
    case SQLDATE:
         mdeccvlong(v->v_long, d);
         break;

    case SQLSMFLOAT:
         mdeccvflt((double) v->v_float, d);
         break;

    case SQLFLOAT:
         mdeccvdbl(v->v_double, d);
         break;

    case SQLDECIMAL:
    case SQLMONEY:
         mdeccopy(&v->v_decimal, d);
         break;

    case SQLDTIME:
    case SQLINTERVAL:
         d->dec_pos = -1;
         d->dec_exp = 0;
         d->dec_ndgts = 0;
         err = -0x00000260;
         break;

    default:
         err = -0x000004ec;
         break;
    }

   return err;
}

double matodub (
   const struct value* v
)

{
   dec_t d;
   double dbl;
   char* from = 0;

   toerrno = 0;
   if (v->v_len> 0)
    {
      from = cnewa(v->v_len, char);
      memcpy(from, v->v_charp, v->v_len);
      mrstrdec(from, v->v_len);
    }
   if (mdeccvasc(from, v->v_len, &d)!=0)
      toerrno = -0x000004bd;
   if (from) free(from);

   if (mdectodbl(&d, &dbl)!=0)
      toerrno = -0x000004c7;

   return dbl;
}

double matodub_r (
   const struct value* v,
   int *err
)

{
   dec_t d;
   double dbl;
   char* from = 0;

   *err = 0;
   if (v->v_len> 0)
    {
      from = cnewa(v->v_len, char);
      memcpy(from, v->v_charp, v->v_len);
      mrstrdec(from, v->v_len);
    }
   if (mdeccvasc(from, v->v_len, &d)!=0)
      *err = -0x000004bd;
   if (from) free(from);

   if (mdectodbl(&d, &dbl)!=0)
      *err = -0x000004c7;

   return dbl;
}

double mtodouble ( 
   const struct value* v
)

{
   double dbl;

   toerrno = 0;
   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR:
    case SQLVCHAR:
         dbl = matodub(v);
         break;

    case SQLSMINT:
         dbl = v->v_int;
         break;

    case SQLINT:
    case SQLSERIAL:
    case SQLDATE:
         dbl = v->v_long;
         break;

    case SQLSMFLOAT:
         dbl = v->v_float;
         break;

    case SQLFLOAT:
         dbl = v->v_double;
         break;

    case SQLDECIMAL:
    case SQLMONEY:
         if (mdectodbl(&v->v_decimal, &dbl)!=0)
            toerrno = -0x000004c7;
         break;

    case SQLBYTES:
    case SQLTEXT:
         toerrno = -0x00000260;
         break;

   default:
         toerrno = -0x000004ec;
         break;
    }

   return dbl;
}

double mtodouble_r ( 
   const struct value* v,
   int *err
)

{
   double dbl;

   *err = 0;
   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR:
    case SQLVCHAR:
         dbl = matodub_r(v, err);
         break;

    case SQLSMINT:
         dbl = v->v_int;
         break;

    case SQLINT:
    case SQLSERIAL:
    case SQLDATE:
         dbl = v->v_long;
         break;

    case SQLSMFLOAT:
         dbl = v->v_float;
         break;

    case SQLFLOAT:
         dbl = v->v_double;
         break;

    case SQLDECIMAL:
    case SQLMONEY:
         if (mdectodbl(&v->v_decimal, &dbl)!=0)
            *err = -0x000004c7;
         break;

    case SQLBYTES:
    case SQLTEXT:
         *err = -0x00000260;
         break;

   default:
         *err = -0x000004ec;
         break;
    }

   return dbl;
}

long mtodate (
   const struct value* v
)

{
   mjdate jdate;
   int errcode;

   toerrno = 0;
   if (v->v_ind<0)
    {
      return 0x80000000;
    }
   else if (ISBYTESTYPE(v->v_type) || ISTEXTTYPE(v->v_type))
    {
      jdate = 0x80000000;
      toerrno = -0x00000260;
    }
   else if ((v->v_type&SQLTYPE)==SQLCHAR || (v->v_type&SQLTYPE)==SQLVCHAR)
    {
      if (stcmpr(v->v_charp,"today")==0)
       {
         mrtoday(&jdate);
       }
      else
       {
         char* to = (char*)malloc(sizeof(char)*(v->v_len + 1));
         memcpy(to, v->v_charp, v->v_len);
         to[v->v_len] = '\0';
         if ((errcode = mrstrdate(to, &jdate))!=0)
            toerrno = errcode;
         free(to);
       }
    }
   else if ((v->v_type&SQLTYPE)==SQLDTIME)
    {
      mrdttodate(&v->v_decimal, v->v_prec, &jdate);
    }
   else
    {
      jdate = tolong(v);
    }

   return jdate;
}

long mtodate_r (
   const struct value* v,
   int *err
)

{
   mjdate jdate;
   int errcode;

   *err = 0;
   if (v->v_ind<0)
    {
      return 0x80000000;
    }
   else if (ISBYTESTYPE(v->v_type) || ISTEXTTYPE(v->v_type))
    {
      jdate = 0x80000000;
      *err = -0x00000260;
    }
   else if ((v->v_type&SQLTYPE)==SQLCHAR || (v->v_type&SQLTYPE)==SQLVCHAR)
    {
      if (stcmpr(v->v_charp,"today")==0)
       {
         mrtoday(&jdate);
       }
      else
       {
         char* to = (char*)malloc(sizeof(char)*(v->v_len + 1));
         memcpy(to, v->v_charp, v->v_len);
         to[v->v_len] = '\0';
         if ((errcode = mrstrdate(to, &jdate))!=0)
            *err = errcode;
         free(to);
       }
    }
   else if ((v->v_type&SQLTYPE)==SQLDTIME)
    {
      mrdttodate(&v->v_decimal, v->v_prec, &jdate);
    }
   else
    {
      jdate = mtolong_r(v, err);
    }

   return jdate;
}

long mtolong (
   const struct value* v
)

{
   long l;
   dec_t d;

   toerrno = 0;
   if (v->v_ind<0)
    {
      l = 0x80000000;
      return l;
    }

   switch(v->v_type&SQLTYPE)
    {
    case SQLSMINT:
         l = v->v_int;
         break;

    case SQLINT:
    case SQLSERIAL:
    case SQLDATE:
         l = v->v_long;
         if (risnull(CLONGTYPE, &l))
            toerrno = - 0x000004bf;
         break;

    case SQLBYTES:
    case SQLTEXT:
         l = 0x80000000;
         toerrno = 0xfffffda0;
         break;

    default:
        toerrno = mtodecimal(v, &d);
        if (dectolong(&d, &l)!=0)
           toerrno = -0x000004bf;
        break;
    }

   return l;
}

long mtolong_r (
   const struct value* v,
   int* err
)

{
   long l;
   dec_t d;

   *err = 0;
   if (v->v_ind<0)
    {
      l = 0x80000000;
      return l;
    }

   switch(v->v_type&SQLTYPE)
    {
    case SQLSMINT:
         l = v->v_int;
         break;

    case SQLINT:
    case SQLSERIAL:
    case SQLDATE:
         l = v->v_long;
         if (risnull(CLONGTYPE, &l))
            *err = - 0x000004bf;
         break;

    case SQLBYTES:
    case SQLTEXT:
         l = 0x80000000;
         *err = 0xfffffda0;
         break;

    default:
        *err = mtodecimal(v, &d);
        if (dectolong(&d, &l)!=0)
           *err = -0x000004bf;
        break;
    }

   return l;
}

int mtoint (
  const struct value* v
)

{
   register int i;
   register long l;

   toerrno = 0;
   if (v->v_ind<0)
    {
      return (short)0x8000;
    }

   if (ISBLOBTYPE(v->v_type))
    {
      i = (short)0x8000;
      toerrno = 0xfffffda0;
      return i;
    }

   if ((v->v_type&SQLTYPE)==SQLSMINT)
    {
      i = v->v_int;
      if (i==(short)0x8000)
       {
         toerrno = -0x000004be;
       }
      return i;
    }

   l = mtolong(v);
   if ((unsigned long)l==0x80000000)
    {
      l = (short)0x8000;
      i = l;
      return i;
    }

   if (l<SHRT_MIN || l>SHRT_MAX)
    {
      toerrno = -0x000004be;
    }

   i = l;
   return i;
}

int mtoint_r (
  const struct value* v,
  int* err
)

{
   register int i;
   register long l;

   *err = 0;
   if (v->v_ind<0)
    {
      return (short)0x8000;
    }

   if (ISBLOBTYPE(v->v_type))
    {
      i = (short)0x8000;
      *err = 0xfffffda0;
      return i;
    }

   if ((v->v_type&SQLTYPE)==SQLSMINT)
    {
      i = v->v_int;
      if (i==(short)0x8000)
       {
         *err = -0x000004be;
       }
      return i;
    }

   l = mtolong_r(v, err);
   if ((unsigned long)l==0x80000000)
    {
      l = (short)0x8000;
      i = l;
      return i;
    }

   if (l<SHRT_MIN || l>SHRT_MAX)
    {
      *err = -0x000004be;
    }

   i = l;
   return i;
}

int mrvalstr (
   const struct mvalue* v,
   char* to,
   int tolen,
   int prec
)

{
   int templen;
   int precision;
   int errcode;
   char tempbuf[80];
   char tempbuf2[80];
   dec_t tempdec;

   errcode = 0;
   if (v->v_ind<0)
    {
      if (tolen!=0)
       {
         memset(to, ' ', tolen);
         return 0;
       }
      *to = '\0';
      return 0;
    }

   templen = 0;
   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR:
    case SQLVCHAR:
         templen = (tolen!=0 && tolen<v->v_len) ? tolen : v->v_len;
         memcpy(to, v->v_charp, templen);
         if (tolen!=0)
          {
            memset(&to[templen], ' ', tolen - templen);
          }
         else
          {
            to[templen] = '\0';
          }
         return errcode;

    case SQLVSTRING:
         templen = (tolen!=0 && tolen<v->v_string->len) ? tolen : v->v_string->len;
         memcpy (to, v->v_string->data, templen);
         if (tolen!=0)
          {
            memset(&to[templen], ' ', tolen - templen);
          }
         else
          {
            to[templen] = '\0';
          }
         return errcode;

    case SQLMONEY:	
         templen = (tolen - mrfrontlen()) - mrbacklen();
         if ((unsigned)templen>sizeof(tempbuf))
            templen = sizeof(tempbuf);
         precision = ACE_PRECDEC(v->v_prec);
         if (precision==0xff)
            precision = 2;

         if (mdectoasc(&v->v_decimal, &tempbuf[0], templen, precision)!=0)
            errcode = 0xfffffb49;

         mrdecmon(&tempbuf[0], &tempbuf[0], templen);
         strcpy(&tempbuf[0], &tempbuf2[0]);
         break;

    case SQLDECIMAL:	
         templen = ((tolen>(int)sizeof(tempbuf)) || tolen==0) ? (int)sizeof(tempbuf) : tolen;
         precision = ACE_PRECDEC(v->v_prec);
         if (precision==0xff)
            precision = 2;
      
         if (mdectoasc(&v->v_decimal, &tempbuf[0], templen, precision)!=0)
            errcode = 0xfffffb49;
         mrdecstr(&tempbuf[0], templen);
         ldchar(&tempbuf[0], templen, &tempbuf[0]);
         break;

    case SQLSMINT:
         sprintf(&tempbuf[0], "%d", v->v_int);
         break;

    case SQLDTIME:
         errcode = rdtimestr(&v->v_decimal, v->v_prec, &tempbuf[0]);
         DPRINTF(("rvalstr: SQLDTIME %s errcode=%d\n", &tempbuf[0], errcode));
         break;

    case SQLINTERVAL:
         errcode = rinvstr(&v->v_decimal, v->v_prec, &tempbuf[0]);
         break;

    case SQLDATE:
         errcode = rdatestr(v->v_long, &tempbuf[0]);
         break;

    case SQLINT:	
    case SQLSERIAL:
         sprintf(&tempbuf[0], "%ld", v->v_long);
         break;

    case SQLSMFLOAT:
         errcode = mdeccvflt((double) v->v_float, &tempdec);
         templen = ((tolen>(int)sizeof(tempbuf))) ? (int)sizeof(tempbuf) : tolen;
         if (errcode==0 && dectoasc(&tempdec,&tempbuf[0], templen, 2)!=0)
            errcode = -0x000004b7;

         mrdecstr(&tempbuf[0], templen);
         ldchar(&tempbuf[0], templen, &tempbuf[0]);
         break;

    case SQLFLOAT:	
         errcode = mdeccvdbl(v->v_double, &tempdec);
         templen = (tolen>(int)sizeof(tempbuf)) ? (int)sizeof(tempbuf) : tolen;
         if (errcode==0 && mdectoasc(&tempdec,&tempbuf[0], templen, 2)!=0)
            errcode = 0xfffffb49;

         mrdecstr(&tempbuf[0], templen);
         ldchar(&tempbuf[0], templen, &tempbuf[0]);
         break;
    }

   templen = strlen(&tempbuf[0]);
   if (tolen!=0 && templen>tolen)
    {
      memset(to, '*', tolen);
      return -0x000004b7;
    }

   if (errcode==0)
    {
      if (tolen!=0)
       {
         memcpy(prec!=0 ? to : (to + tolen - templen), &tempbuf[0], templen);
         memset(to + ((prec!=0) ? templen : 0), ' ', tolen - templen);
       }
      else
       {
         strcpy(to, &tempbuf[0]);
       }
    }
   return errcode;
}

int mtovstring (
   const struct mvalue* v,
   mvstring to
)

{
   int templen;
   int precision;
   int errcode;
   char tempbuf[80];
   char tempbuf2[80];
   dec_t tempdec;

   errcode = 0;
   if (v->v_ind<0)
    {
      mvstring_setnull (to);
      return 0;
    }

   templen = 0;
   switch(v->v_type&SQLTYPE)
    {
    case SQLCHAR: 
    case SQLVCHAR:
         mvstring_memcpy (to, v->v_charp, v->v_len);
         return 0;

    case SQLVSTRING:
         mvstring_assign (to, v->v_string);
         return 0;

    case SQLMONEY:	
         templen = (sizeof(tempbuf) - mrfrontlen()) - mrbacklen();
         if ((unsigned)templen>sizeof(tempbuf))
            templen = sizeof(tempbuf);
         precision = ACE_PRECDEC(v->v_prec);
         if (precision==0xff)
            precision = 2;

         if (mdectoasc(&v->v_decimal, &tempbuf[0], templen, precision)!=0)
            errcode = 0xfffffb49;

         mrdecmon(&tempbuf[0], &tempbuf[0], templen);
         strcpy(&tempbuf[0], &tempbuf2[0]);
         break;

    case SQLDECIMAL:	
         templen = sizeof(tempbuf);
         precision = ACE_PRECDEC(v->v_prec);
         if (precision==0xff)
            precision = 2;
      
         if (mdectoasc(&v->v_decimal, &tempbuf[0], templen, precision)!=0)
            errcode = 0xfffffb49;
         mrdecstr(&tempbuf[0], templen);
         ldchar(&tempbuf[0], templen, &tempbuf[0]);
         break;

    case SQLSMINT:
         sprintf(&tempbuf[0], "%d", v->v_int);
         break;

    case SQLDTIME:
         errcode = rdtimestr(&v->v_decimal, v->v_prec, &tempbuf[0]);
         DPRINTF(("rvalstr: SQLDTIME %s errcode=%d\n", &tempbuf[0], errcode));
         break;

    case SQLINTERVAL:
         errcode = rinvstr(&v->v_decimal, v->v_prec, &tempbuf[0]);
         break;

    case SQLDATE:
         errcode = rdatestr(v->v_long, &tempbuf[0]);
         break;

    case SQLINT:	
    case SQLSERIAL:
         sprintf(&tempbuf[0], "%ld", v->v_long);
         break;

    case SQLSMFLOAT:
         errcode = mdeccvflt((double) v->v_float, &tempdec);
         templen = sizeof(tempbuf);
         if (errcode==0 && dectoasc(&tempdec,&tempbuf[0], templen, 2)!=0)
            errcode = -0x000004b7;

         mrdecstr(&tempbuf[0], templen);
         ldchar(&tempbuf[0], templen, &tempbuf[0]);
         break;

    case SQLFLOAT:	
         errcode = mdeccvdbl(v->v_double, &tempdec);
         templen = sizeof(tempbuf);
         if (errcode==0 && mdectoasc(&tempdec,&tempbuf[0], templen, 2)!=0)
            errcode = -0x000004b7;

         mrdecstr(&tempbuf[0], templen);
         ldchar(&tempbuf[0], templen, &tempbuf[0]);
         break;
    }

   if (errcode==0)
    {
      mvstring_copy (to, &tempbuf[0]);
    }
   return errcode;
}

int rcvarval (
   void* cvar,
   int ctype,
   struct value* v,
   short prec,
   short* ind
)

{
   int len;
   const char* p;

   if (ctype==CINTTYPE && sizeof(int)==sizeof(long))
      ctype = CLONGTYPE;

   v->v_ind = 0;
   v->v_prec = 0;
   if (ctype==CINTTYPE && sizeof(int)==sizeof(long))
      ctype = CLONGTYPE;

   if (ctype>=CCHARTYPE && ctype<=CVSTRINGTYPE)
    {
      v->v_type = rtosqltype(ctype);
    }
   else if (ISSQLTYPE(ctype))
    {
      v->v_type = ctype;
    }
   else
    {
      return -1;
    }

   if ((ind!=0 && *ind<0)
   ||  risnull(ctype, cvar))
    {
      v->v_ind = -1;
      if (v->v_type==SQLCHAR || v->v_type==SQLVCHAR)
       {
         v->v_charp = cvar;
         v->v_len = prec;
         return 0;
       }
      else if (v->v_type==SQLVSTRING)
	  {
		v->v_string = (mvstring) cvar;
		v->v_prec = 0;
		return 0;
	  }
      else if (ISBLOBTYPE(v->v_type))
       {
#if 0
         v->w0x2e = 1;
#else
	 assert(0);
#endif
       }

      return 0;
    }

   if (ctype==CINTTYPE && sizeof(int)==sizeof(long))
      ctype = CLONGTYPE;

   if (ISSQLTYPE(ctype))
      ctype = rtoctype(ctype);

   switch(ctype)
    {
    case CSHORTTYPE:
         v->v_int = *((const short*) cvar);
         v->v_prec = ACE_PRECMAKE(5, 0);
         break;

    case CINTTYPE:
         v->v_int = *((const int*) cvar);
         v->v_prec = ACE_PRECMAKE(5, 0);
         break;

    case CLONGTYPE:
         v->v_long = *((const long*) cvar);
         v->v_prec = ACE_PRECMAKE(10, 0);
         break;

    case CDATETYPE:
         v->v_long = *((const mjdate*) cvar);
         v->v_prec = ACE_PRECMAKE(10, 0);
         break;

    case CFLOATTYPE:
         v->v_float = *((const float*) cvar);
         v->v_prec = ACE_PRECMAKE(8,0xff);
         break;

    case CDOUBLETYPE:
         v->v_double = *((const double*) cvar);
         v->v_prec = ACE_PRECMAKE(16,0xff);
         break;

    case CCHARTYPE:
    case CVCHARTYPE:
         if (prec<=0)
          {
            len = stleng(cvar);
          }
         else
          {
            for (p = (const char*) cvar, len = 0;
                 *p!='\0' && len<prec; len++, p++);
          }

         v->v_charp = (char*) cvar;
         v->v_len = len;
         v->v_ind = len==0?-1:0;
         v->v_prec = 0;
         break;

    case CSTRINGTYPE:
         v->v_charp = (char*) cvar;
         v->v_len = stleng(cvar);
         v->v_ind = v->v_len==0?-1:0;
         v->v_prec = 0;
         break;

    case CFIXCHARTYPE:
         v->v_charp = (char*) cvar;
         v->v_len = prec;
         v->v_prec = 0;
         break;

    case CDTIMETYPE:
         if (((const dtime_t*)cvar)->dt_dec.dec_ndgts>ACE_DECSIZE
         ||  ((const dtime_t*)cvar)->dt_dec.dec_ndgts<0)
          {
            return -1;
          }
         else
          {
            deccopy(&((const dtime_t*)cvar)->dt_dec, &v->v_decimal);
            v->v_prec = ((const dtime_t*)cvar)->dt_qual;
            break;
          }

    case CINVTYPE:
         if (((const intrvl_t*)cvar)->in_dec.dec_ndgts>ACE_DECSIZE
         ||  ((const intrvl_t*)cvar)->in_dec.dec_ndgts<0)
          {
            return -1;
          }
         else
          {
            deccopy(&((const intrvl_t*)cvar)->in_dec, &v->v_decimal);
            v->v_prec = ((const intrvl_t*)cvar)->in_qual;
            break;
          }

    case CDECIMALTYPE:
    case CMONEYTYPE:
         if (((const dec_t*)cvar)->dec_ndgts>ACE_DECSIZE
         ||  ((const dec_t*)cvar)->dec_ndgts<0)
          {
            return -1;
          }
         else
          {
            deccopy((const dec_t*)cvar, &v->v_decimal);
            v->v_prec = prec;
            break;
          }

    case CLOCATORTYPE:
#if 0
         v->w0x2e = 0;
#else
	 assert(0);
#endif
         v->v_blocator = (mbase_loc_t*) cvar;
         break;

    case CFILETYPE:
#if 0
         v->w0x2e = 0;
#else
	 assert(0);
#endif
         v->v_charp = (char*) cvar;
         v->v_len = prec;
         v->v_prec = 0;
         break;

    case CVSTRINGTYPE:
         v->v_string = (mvstring) cvar;
         v->v_prec = 0;
         break;
    }

   return 0;
}

int rvalcvar (
   const struct value* v,
   void* cvar,
   int ctype,
   int len,
   short* ind
)

{
   int err = 0;
   if (ctype==CINTTYPE && sizeof(int)==sizeof(long))
      ctype = CLONGTYPE;

   if (ISSQLTYPE(ctype))
      ctype = rtoctype(ctype);

   if (v->v_ind<0)
    {
      rsetnull(ctype, cvar);
      if (ind!=0)
         *ind = -1;
      return 0;
    }

   switch(ctype)
    {
    case CCHARTYPE:
    case CSTRINGTYPE:
    case CVCHARTYPE:
         if (len>1)
          {
            --len;
          }
         else if (len==1)
          {
            *((char*) cvar) = '\0';
            len = -1;
          }
    case CFIXCHARTYPE:
         if (len<0)
          {
            err = -0x000004d3;
          }
         else
          {
            err = rvalstr(v, cvar, len, 1);
          }

         if (ind!=0)
          {
            if ((v->v_type==SQLCHAR || v->v_type==SQLVCHAR)
            &&  (len!=0 && v->v_len>len))
             {
               *ind = v->v_len;
             }
            else
             {
               *ind = (toerrno!=0) ? 1 : 0;
             }
          }

         if (ctype!=CFIXCHARTYPE && len>=0)
          {
            if (ctype==CSTRINGTYPE)
             {
               len = byleng(cvar, len);
               if (len==0)
                  len++;
               ((char*)cvar)[len] = '\0';
             }
            else if (ctype==CVCHARTYPE && v->v_len<len)
             {
                len = v->v_len;
             }
            if (len!=0)
               ((char*)cvar)[len] = '\0';
          }
         return err;

    case CSHORTTYPE:
         *((short*) cvar) = mtoint_r(v, &err);
         break;

    case CINTTYPE:
         *((int*) cvar) = mtoint_r(v, &err);
         break;

    case CLONGTYPE:
         *((long*) cvar) = mtolong_r(v, &err);
         break;

    case CDATETYPE:
         *((mjdate*) cvar) = mtodate_r(v, &err);
         break;

    case CFLOATTYPE:
         *((float*) cvar) = (float)mtodouble_r(v, &err);
         break;

    case CDOUBLETYPE:
         *((double*) cvar) = mtodouble_r(v, &err);
         break;

    case CDTIMETYPE:
         mtodatetime(v, cvar, len);
         break;

    case CINVTYPE:
         mtointerval(v, cvar, len);
         break;

    case CDECIMALTYPE:
    case CMONEYTYPE:
         err = mtodecimal(v, (dec_t*)cvar);
         break;

    case CLOCATORTYPE:
         if (!ISBLOBTYPE(mrtosqltype(v->v_type)))
          {
            err = 0xfffffb0b;
          }
         else
          {
            memcpy(cvar, v->v_blocator, 0x10);
          }
         break;

    case CFILETYPE:
         if (!ISBLOBTYPE(mrtosqltype(v->v_type)))
          {
            err = 0xfffffb0b;
          }
         else
          {
            bycopy(v->v_charp, cvar, v->v_len);
          }
         break;

    case CVSTRINGTYPE:
         mtovstring (v, (mvstring) cvar);
         break;

    default:
         EPRINTF(("rvalcvar: Invalido tipo de datos %d\n", ctype));
         break;
    }

   if (ind!=0)
    {
      *ind = (err!=0) ? 1 : 0;
    }

   return err;
}

void nullblobval (
   struct value* v,
   int sqltype
)

{
   byfill(v, sizeof(*v), 0);
   v->v_type = sqltype&0xf;
   v->v_ind = -1;
   v->v_tblob.tb_flags |= BLOBISNULL;
}

int mchkprec (
   dec_t* d,
   int prec
)

{
   int prec2;

   prec2 = mdec2prec(d);
   if (ACE_PRECDEC(prec)==0xff)
    {
      if (ACE_PRECTOT(prec2)>ACE_PRECTOT(prec))
         mdecround(d, ACE_PRECDEC(prec2) + ACE_PRECTOT(prec) - ACE_PRECTOT(prec2));
    }
   else
    {
      if (ACE_PRECDEC(prec2)>ACE_PRECDEC(prec))
         mdecround(d, ACE_PRECDEC(prec));

      prec2 = mdecprec(d);
      if ((ACE_PRECTOT(prec2) - ACE_PRECDEC(prec2))>(ACE_PRECTOT(prec) - ACE_PRECDEC(prec)))
         return -1;
    }

   return 0;
}

double mtodblnull (
   const struct mvalue* v,
   short* ind
)

{
   char* p;
   short len;

   if (v->v_ind<0)
    {
      *ind = 1;
      return +0.0;
    }

   if (((v->v_type&SQLTYPE)==SQLCHAR)
   ||  ((v->v_type&SQLTYPE)==SQLVCHAR))
    {
      for (len = v->v_len, p = v->v_charp;
           *p==' '; p++, len--);

      if (len==0)
       {
         *ind = 1;
         return +0.0;
       }
    }

   *ind = 0;
   return mtodouble(v);
}

long mrvaldata (
   const struct value* v,
   char* to,
   int sqltype,
   int sqllen
)

{
   int i;
   dec_t d;
   intrvl_t in;
   dtime_t dt;
   long l;
   mjdate jdate;
   short ind;
   double dbl;
   struct value temp;
   int errcode;

   if (v->v_ind<0 && (sqltype&SQLNONULL))
      return 0xfffffb37;

   if (v->v_ind>0
   &&  ((ISBLOBTYPE(sqltype) && !ISBLOBTYPE(v->v_type))
   ||   (!ISBLOBTYPE(sqltype) && ISBLOBTYPE(v->v_type))))
      return 0xfffffda0;

   errcode = 0;
   ind = 0;
   switch(sqltype&SQLTYPE)
    {
    case SQLCHAR:
         if (v->v_ind<0)
          {
            *to = 0;
            byfill(to+1, (sqllen>0) ? (sqllen - 1) : 0, ' ');
            break;
          }

         if (v->v_len==0)
            ind = 1;

         if ((v->v_type&SQLTYPE)!=SQLCHAR
         &&  (v->v_type&SQLTYPE)!=SQLVCHAR)
          {
            if ((errcode = rvalstr(v, to, sqllen, 1))!=0)
               return errcode;
            break;
          }
         else
          {
            i = (v->v_len<sqllen) ? v->v_len : sqllen;
            bycopy(v->v_charp, to, i);
            byfill(to + i, sqllen - i, ' ');
            break;
          }

    case SQLVCHAR:
         i = sqllen&0xff;
         if (v->v_ind<0)
          {
            *to = 1;
            byfill(to + 1, i, 0);
            break;
          }

         if (v->v_len==1
         &&  v->v_charp==0)
            ind = 1;

         if ((v->v_type&SQLTYPE)!=SQLVCHAR
         &&  (v->v_type&SQLTYPE)!=SQLCHAR)
          {
            if ((errcode = rvalstr(v, to + 1, i, 1))!=0)
               return errcode;
          }
         else
          {
            i = (v->v_len<=i) ? v->v_len : i;
            bycopy(v->v_charp, to + 1, i);
          }
         *to = i;
         break;

    case SQLSMINT:
         i = mtoint_r(v, &errcode);
         if (i==(short)0x8000)
            ind = 1;

         to[0] = ((i>>8)&0xff);
         to[1] = i&0xff;
         break;

    case SQLINT:
    case SQLSERIAL:
         l = mtolong_r(v, &errcode);
         if ((unsigned long)l==0x80000000)
            ind = 1;

         mstlong(mtolong_r(v, &errcode), (unsigned char*) to);
         break;

    case SQLSMFLOAT:
         dbl = mtodblnull(v, &ind);
         stfltnull(dbl, (unsigned char*)to, ind);
         break;

    case SQLFLOAT:
         dbl = mtodblnull(v, &ind);
         mstdblnull(dbl, (unsigned char*)to, ind);
         break;

    case SQLDATE:
         jdate = mtodate_r(v, &errcode);
         if (risnull(CDATETYPE, &jdate))
            ind = 1;
         mstlong(jdate, (unsigned char*) to);
         break;

    case SQLINTERVAL:
         errcode = mtointerval(v, &in, sqllen);
         if (errcode!=0)
            break;

         if (in.in_dec.dec_pos==-1 || v->v_ind<0)
            ind = 1;

         mstdecimal(&in.in_dec, (unsigned char*) to, ACE_DECLENGTH(sqllen));
         break;

    case SQLDTIME:
         errcode = mtodatetime(v, &dt, sqllen);
         if (errcode!=0)
            break;

         if (dt.dt_dec.dec_pos==-1 || v->v_ind<0)
            ind = 1;

         mstdecimal(&dt.dt_dec, (unsigned char*) to, ACE_DECLENGTH(sqllen));
         break;

    case SQLDECIMAL:
    case SQLMONEY:
         errcode = mtodecimal(v, &d);
         if (errcode!=0)
            break;

         if (d.dec_pos==-1 || v->v_ind<0)
          {
            ind = 1;
          }
         else if (v->v_ind>=0
         &&  mchkprec(&d, sqllen)!=0)
          {
            return 0xfffffb36;
          }
         mstdecimal(&d, (unsigned char*) to, ACE_DECLENGTH(sqllen));
         break;

    case SQLBYTES:
    case SQLTEXT:
         if (v->v_ind<0)
          {
            nullblobval(&temp, sqltype&SQLTYPE);
            v = &temp;
          }
         tblobstore(&v->v_tblob, (unsigned char*) to);
         break;
    }

   if (errcode==0
   &&  ind!=0
   &&  (sqltype&SQLNONULL))
      return 0xfffffb37;
   else
      return errcode;
}

long mrdataval (
   unsigned char* from,
   int sqltype,
   int sqllen,
   struct mvalue* v
)

{
   short ind;
   int len;

   v->v_type = sqltype&SQLTYPE;
   v->v_ind = 0;
   switch(v->v_type)
    {
    case SQLCHAR:
         v->v_charp = (char*) from;
         v->v_len = sqllen;
         v->v_prec = 0;
         if (*from!='\0')
            return 0;
         break;

    case SQLVCHAR:
         v->v_charp = (char*)(from + 1);
         len = *from&0xff;
         v->v_len = (sqllen==0 || sqllen>len) ? len : sqllen;
         v->v_prec = 0;
         if (v->v_len!=1 || from[1]!='\0')
            return 0;
         break;

    case SQLSMINT:
         v->v_prec = ACE_PRECMAKE(5,0);
         v->v_int = ldshort(from);
         if (v->v_int!=(short)0x8000)
            return 0;
         break;

    case SQLINT:
    case SQLSERIAL:
    case SQLDATE:
         v->v_prec = ACE_PRECMAKE(10,0);
         v->v_long = mldlong(from);
         if (v->v_long!=0x80000000)
            return 0;
         break;

    case SQLSMFLOAT:
         v->v_prec = ACE_PRECMAKE(8,0xff);
         v->v_float = (float)mldfltnull(from, &ind);
         if (ind==0)
            return 0;
         break;

    case SQLFLOAT:
         v->v_prec = ACE_PRECMAKE(16, 0xff);
         v->v_double = mlddblnull(from, &ind);
         if (ind==0)
            return 0;
         break;

    case SQLDECIMAL:
    case SQLMONEY:
    case SQLDTIME:
    case SQLINTERVAL:
         v->v_prec = sqllen;
         mlddecimal(from, ACE_DECLENGTH(sqllen), &v->v_decimal);
         if (v->v_decimal.dec_pos!=-1)
            return 0;
         break;

    case SQLBYTES:
    case SQLTEXT:
         tblobload(&v->v_tblob, from);
         if ((v->v_tblob.tb_flags&1)==0)
            return 0;
         break;

    default:
         return 0;
    }

   v->v_ind = -1;
   return 0;
}

void bindvalue (
   struct mvalue *v,
   struct msqlvar_struct *sqldvar
)

{
   switch(v->v_type) {
   case SQLCHAR:
	initsqlvar(sqldvar,
		NULL,
		CCHARTYPE,
		v->v_len + 1,
		v->v_charp);
	break;
	
   case SQLVSTRING:
	initsqlvar(sqldvar,
		NULL,
		CVSTRINGTYPE,
		0,
		v->v_string);
	break;
	
   case SQLSMINT:
	initsqlvar(sqldvar,
		NULL,
		CINTTYPE,
		sizeof(int),
		(char*) &v->v_int);
	break;

   case SQLINT:
   case SQLSERIAL:
	initsqlvar(sqldvar,
		NULL,
		CLONGTYPE,
		sizeof(long),
		(char*) &v->v_long);
	break;
   
   case SQLDECIMAL:
	initsqlvar(sqldvar,
		NULL,
		CDECIMALTYPE,
		v->v_prec,
		(char*) &v->v_decimal);
		break;

    case SQLDTIME:
	DPRINTF(("bindvalue: SQLDTIME(%d,%d)=%d\n",
		ACE_TU_START(v->v_prec),ACE_TU_END(v->v_prec),
		v->v_prec));
		
	initsqlvar(sqldvar,
		NULL,	
		CDTIMETYPE,
		v->v_prec,
		(char*) &v->v_prec);
	break;

   case SQLINTERVAL:

	DPRINTF(("bindvalue: SQLINTERVAL(%d,%d)=%d\n",
		ACE_TU_START(v->v_prec),ACE_TU_END(v->v_prec),
		v->v_prec));
	
	initsqlvar(sqldvar,
		NULL,	
		CINVTYPE,
		v->v_prec,
		(char*) &v->v_prec);
	break;

   case SQLMONEY:
	initsqlvar(sqldvar,
		NULL,
		CMONEYTYPE,
		v->v_prec,
		(char*) &v->v_decimal);
	break;

   case SQLSMFLOAT:
	initsqlvar(sqldvar,
		NULL,
		CFLOATTYPE,
		rtypmsize(CFLOATTYPE, sqldvar->sqllen),
		(char*) &v->v_float);
	break;

   case SQLFLOAT:
	initsqlvar(sqldvar,
		NULL,
		CDOUBLETYPE,
		rtypmsize(CDOUBLETYPE, sqldvar->sqllen),
		(char*) &v->v_double);
	break;

   case SQLDATE:
	initsqlvar(sqldvar,
		NULL,
		CDATETYPE,
		rtypmsize(CDATETYPE, sqldvar->sqllen),
		(char*) &v->v_long);
	break;

   default:
	printf("bindvalue: %d Tipo de datos no soportado\n",
		v->v_type);
   }
  if (v->v_ind<0)
     mrsetnull (sqldvar->sqltype, sqldvar->sqldata);

/*	sqldvar->sqlind = &v->v_ind;*/
}

void dumpval (
   struct mvalue *v
)

{
   mdtime_t dt;
   mintrvl_t in;
   int column;
   struct msqlvar_struct sqlovar;

   switch(v->v_type)
   {
   case SQLVSTRING:
	bindvalue (v, &sqlovar);
	break;

   case SQLCHAR:
	bindvalue (v, &sqlovar);
	break;

   case SQLSMINT:
	bindvalue (v, &sqlovar);
	break;

   case SQLINT:
	bindvalue (v, &sqlovar);
	break;

   case SQLDATE:
	bindvalue (v, &sqlovar);
	break;

   case SQLDECIMAL:
	bindvalue (v, &sqlovar);
	break;

   case SQLMONEY:
	bindvalue (v, &sqlovar);
	break;

   case SQLSMFLOAT:
	bindvalue (v, &sqlovar);
	break;

   case SQLFLOAT:
	bindvalue (v, &sqlovar);
	break;

   case SQLDTIME:
	/* v a dt */
	dt.dt_qual=v->v_prec;
	mrvalcvar (v,
		&dt,
		SQLDTIME,
		v->v_prec,
		0);
	initsqlvar (&sqlovar, NULL, CDTIMETYPE, dt.dt_qual, &dt);
	break;

   case SQLINTERVAL:
	/* v a in */
	in.in_qual=v->v_prec;
	rvalcvar (v,
		&in,
		SQLINTERVAL,
		v->v_prec,
		0);
	initsqlvar (&sqlovar, NULL, CINVTYPE, in.in_qual, &in);
	break;

   case SQLCOLUMN:
        column = v->v_int;
        initsqlvar (&sqlovar, NULL, CINTTYPE, sizeof(column), &column);
        break;

   default:
	printf("dumpval: %d Tipo de datos no soportado\n",v->v_type);
	exit(-1);
   }
  dumpsqlvar(&sqlovar);
}

