#ifndef __MBASE_VALUE_H__
#define __MBASE_VALUE_H__
#include "mbase/config.h"
#include "mbase/sqltypes.h"
#include "mbase/smallint.h"
#include "mbase/integer.h"
#include "mbase/decimal.h"
#include "mbase/datetime.h"
#include "mbase/locator.h"
#include "mbase/blob.h"
#include "mbase/vstring.h"
#include "mbase/sqlda.h"

#if __cplusplus
#include "DBBASE/Date.h"
#include "DBBASE/Decimal.h"
#include "DBBASE/DateTime.h"
#endif /* __cplusplus */

#define MAXADDR		14

EXTERN_C struct mvalue;
struct mvalue
{
   short v_type;
   short v_ind;
   short v_prec;
   union {
	struct {
	    char *vcp;		/* string start 		*/
	    short vidx;		
	    short vlen;		/* string length 		*/
	    short vflgs;	/* flags - see below 		*/
	    short vsstart;	/* substring start for 4GL-RDS 	*/
	    short vsend;	/* substring end for 4GL-RDS 	*/
	} vchar;

	int vint;
	long vlng;
	float vflo;
	double vdub;
	mdec_t vdec;
        mvstring vstring;
	short vaddr[MAXADDR];	/* 4GL address modifiers	*/
        mtblob_t vtblob;
	mbase_loc_t *vlocator;	/* blobs locator 		*/
   } v_val;		
};

/*
 * defines to make the union transparent
 */
#define v_charp		v_val.vchar.vcp
#define v_index		v_val.vchar.vidx
#define v_len		v_val.vchar.vlen
#define v_flags		v_val.vchar.vflgs
#define v_sstart	v_val.vchar.vsstart
#define v_send		v_val.vchar.vsend
#define v_int		v_val.vint
#define v_long		v_val.vlng
#define v_float		v_val.vflo
#define v_double	v_val.vdub
#ifndef v_decimal
#define v_decimal	v_val.vdec
#endif /* v_decimal */
#define v_decimal	v_val.vdec
#define v_string	v_val.vstring
#define v_idesc		v_ind
#define v_naddr		v_prec
#define v_addr		v_val.vaddr
#define v_blocator	v_val.vlocator
#define v_tblob		v_val.vtblob

/*
 * flags for v_flags
 * used by the 4GL Debugger and Pcode Run Time
 */
#ifndef V_SUBSTR
#define	V_BREAK		01		/* break when variable is updated */ 
#define	V_SUBSTR	02		/* char value is a substring */
#define	V_QUOTED	04		/* char value is from quoted string */
#endif /* V_SUBSTR */

#define toerrno (*mtoerrno_r())

#if __cplusplus
MExport int is_null(const struct mvalue* v);
MExport int set_null(const struct mvalue* v);
MExport int to_int(const struct mvalue* v, int* i);
MExport int to_smallint(const struct mvalue* v, Char& c);
MExport int to_smallint(const struct mvalue* v, SmallInt& s);
MExport int to_integer(const struct mvalue* v, Integer& i);
MExport int to_date(const struct mvalue* v, Date& d);
MExport int to_decimal(const struct mvalue* v, Decimal& d);
MExport int to_datetime(const struct mvalue* v, DateTime& dt);
MExport int to_interval(const struct mvalue* v, Interval& in);
MExport int to_value(int i, struct mvalue* v);
MExport int to_value(const Char& c, struct mvalue* v);
MExport int to_value(const SmallInt& s, struct mvalue* v);
MExport int to_value(const Integer& i, struct mvalue* v);
MExport int to_value(const Date& d, struct mvalue* v);
MExport int to_value(const Decimal& d, struct mvalue* v);
MExport int to_value(const Interval& d, struct mvalue* v);
MExport int to_value(const DateTime& d, struct mvalue* v);
MExport int to_str(const struct mvalue* v, char* to, int len, int prec = 0);
MExport int to_char(const struct mvalue* v, Char& c);
MExport int to_long(const struct mvalue* v, long* l);
MExport int to_value(long l, struct mvalue* v);
MExport int to_date(const struct mvalue* v, mjdate* l);
MExport int to_decimal(const struct mvalue* v, dec_t* d);
MExport int to_value(const dec_t* d, struct mvalue* v);
MExport int to_float(const struct mvalue* v, float* d);
MExport int to_value(float d, struct mvalue* v);
MExport int to_double(const struct mvalue* v, double* d);
MExport int to_value(double d, struct mvalue* v);
MExport int to_datetime(const struct mvalue* v, dtime_t* dt, short qual);
MExport int to_value(const dtime_t* dt, short qual, struct mvalue* v);
MExport int to_interval(const struct mvalue* v, intrvl_t* in, short qual);
MExport int to_value(const intrvl_t* in, short qual, struct mvalue* v);
MExport int fmtdate(const struct mvalue* v, const char* fmt, char* to, int len);
MExport int fmtnum(const struct mvalue* v, const char* fmt, char* to, int len);
#endif /* __cplusplus */

EXTERN_C MExport void cvtosmint(struct mvalue*);

#undef toint

EXTERN_C MExport 
int* mtoerrno_r(void);

EXTERN_C MExport 
int mtoint (
  const struct mvalue* v
);

EXTERN_C MExport 
int mtoint_r (
  const struct mvalue* v,
  int *err
);

EXTERN_C MExport 
long mtolong (
   const struct mvalue* v
);

EXTERN_C MExport 
long mtolong_r (
   const struct mvalue* v,
   int* err
);

EXTERN_C MExport 
long mtodate (
   const struct mvalue* v
);

EXTERN_C MExport 
long mtodate_r (
   const struct mvalue* v,
   int* err
);

EXTERN_C MExport 
double matodub (
   const struct mvalue* v
);

EXTERN_C MExport 
double matodub_r (
   const struct mvalue* v,
   int *err
);

EXTERN_C MExport 
int mtointerval (
   const struct mvalue* v,
   mintrvl_t* in,
   int qual
);

EXTERN_C MExport 
int mtointerval_r (
   const struct mvalue* v,
   mintrvl_t* in,
   int qual,
   int *err
);

EXTERN_C MExport 
int mtodatetime (
   const struct mvalue* v,
   mdtime_t* dt,
   int qual
);

EXTERN_C MExport 
int mtodecimal (
   const struct mvalue* v,
   mdec_t* d
);

EXTERN_C MExport 
int mtovstring (
   const struct mvalue* v,
   mvstring vs
);

EXTERN_C MExport 
double mtodouble ( 
   const struct mvalue* v
);

EXTERN_C MExport 
double mtodouble_r ( 
   const struct mvalue* v,
   int* err
);

EXTERN_C MExport 
int mrcvarval (
   void* cvar,
   int ctype,
   struct mvalue* v,
   short prec,
   short* ind
);

EXTERN_C MExport 
int mrvalcvar (
   const struct mvalue* v,
   void* cvar,
   int ctype,
   int len,
   short* ind
);

EXTERN_C MExport 
int mrvalstr (
   const struct mvalue* v,
   char* to,
   int len,
   int prec
);

EXTERN_C MExport 
int mrfmtnum (
   const struct mvalue* v,
   const char* format,
   char* result
);

EXTERN_C MExport 
void mnullblobval (
   struct mvalue* v,
   int sqltype
);

EXTERN_C MExport 
int mchkprec (
   mdec_t* d,
   int prec
);

EXTERN_C MExport 
long mrvaldata (
   const struct mvalue* v,
   char* to,
   int sqltype,
   int sqllen
);

EXTERN_C MExport
long mrdataval (
   unsigned char* from,
   int sqltype,
   int sqllen,
   struct mvalue* v
);

EXTERN_C MExport 
void bindvalue (
   struct mvalue *v,
   struct msqlvar_struct *sqldvar
);

EXTERN_C MExport 
void dumpval (
   struct mvalue *v
);

#ifndef FRCBOOL
#define FRCBOOL(x)	if (x->v_ind >= 0)\
			    switch (x->v_type) \
				{\
				case SQLSMINT: break;\
				case SQLDATE:\
				case SQLSERIAL:\
				case SQLINT: if (x->v_long != 0)\
						x->v_int = 1;\
					     else\
						x->v_int = 0;\
					     x->v_type = SQLSMINT;\
					     x->v_prec = ACE_PRECMAKE(5, 0);\
					     break;\
			        default:     cvtosmint(x);\
					     break;\
				}
#endif /* FRCBOOL */

#if !defined(NO_INFORMIX_MAP)
#define value mvalue
#define toint mtoint
#define tolong mtolong
#define todate mtodate
#define atodub matodub
#define tointerval mtointerval
#define todatetime mtodatetime
#define todecimal mtodecimal
#define todouble mtodouble
#define rcvarval mrcvarval
#define rvalcvar mrvalcvar
#define rvalstr mrvalstr
#define rfmtnum mrfmtnum
#define nullblobval mnullblobval
#define chkprec mchkprec
#define rvaldata mrvaldata
#define rdataval mrdataval
#endif /* NO_INFORMIX_MAP */

#endif /* __MBASE_VALUE_H__ */
