//$Id: Tools.cpp,v 1.9 2004/10/12 12:56:38 roiser Exp $

#include "Reflection/Tools.h"
#include "Reflection/Function.h"
#include "Reflection/Scope.h"
#include "Reflection/Type.h"
#include "Reflection/Reference.h"
#include "Reflection/Pointer.h"
#include "Reflection/Array.h"
#include "FuncHandler.h"

#ifdef __GNUC__
  #include <cxxabi.h>
#endif

//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::stringSplit( std::vector < MLANG_String > & splitValues, 
                                        const MLANG_String & str,
                                        const MLANG_String & delim ) {
//-------------------------------------------------------------------------------
  MLANG_String str2 = str;
  
  size_t pos = 0;
  
  while ( (int) ( pos = str2.find_first_of( delim )) != MLANG_String::npos ) {
    MLANG_String s = str2.substring(0, pos - 1);
    stringStrip( s );
    if ( s.length()) { splitValues.push_back( s ); }
    str2 = str2.substring( pos + delim.length());
  }
  
  stringStrip( str2 );
  if ( str2.length()) { splitValues.push_back( str2 ); }
}



//-------------------------------------------------------------------------------
bool MLANG_Reflection::Tools::argsEqual( const Function * mf,
                                      const MLANG_String & parString ) {
//-------------------------------------------------------------------------------
  std::vector < MLANG_String > sParVec;
  stringSplit(sParVec, parString);
  size_t sParLen = sParVec.size();
  size_t fParLen = mf->parameterCount();
  
  if (( sParLen == 0 )) { return true; }
  if (( sParLen > fParLen )) { return false; }
  
  for ( size_t i = 0; i < fParLen; ++i ) {
    const Type * fType = mf->parameter( i );
    const Type * sType = Type::byName( sParVec[ i ] );
    if ( sType != fType ) { return false; }
  }
  
  return true;
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::checkParamName( MLANG_String & type,
                                           MLANG_String & name,
                                           MLANG_String & defaultVal,
                                           MLANG_String & paramString ) {
//-------------------------------------------------------------------------------
  std::vector< MLANG_String > pVec = std::vector< MLANG_String >();
  stringSplit(pVec, paramString, "^");
  size_t pVecLen = pVec.size();

  type = pVec[0];

  if ( pVecLen > 1 ) {
    stringStrip( pVec[ 1 ]);
    name = pVec[ 1 ]; 
  }
  if ( pVecLen > 2 ) { 
    stringStrip( pVec[ 2 ]);
    defaultVal = pVec[ 2 ]; 
  }
}



//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::constructType( const Type * & type, 
                                          const MLANG_String & typeStr ) {
//-------------------------------------------------------------------------------
  MLANG_String typeStr2 = typeStr;
  unsigned int lchar = typeStr2.length() - 1;

  stringStrip( typeStr2 );

  const Type * t = Type::byName( typeStr2 );

  if ( t ) {
    type = t;
  }
  else if ( typeStr2[ lchar ] == '&' ) {
    // is it a reference ?

    MLANG_String refName = typeStr2.substring( 0, lchar - 1 );
    constructType( t, refName );

    Reference * r = new Reference( t,  
                                   t->typeInfo());
    type = r->type();

  }
  else if ( typeStr2[ lchar ] == '*' ) {
    // is it a pointer ?

    MLANG_String pntName = typeStr2.substring( 0, lchar - 1);
    constructType( t, pntName );

    Pointer * p =  new Pointer( t,
                                typeid(void*)); // fixme how to retrieve the pointer typeinfo?
    type = p->type();
  }

  else if ( typeStr2[ lchar ] == ']' ) {
    // is it an array ?

    unsigned int pos1 = typeStr2.lastIndexOf( "[" ) + 1;
    unsigned int pos2 = typeStr2.length() - 1;
    MLANG_String arrSizeStr = typeStr2.substring( pos1, pos2 );

    stringStrip( arrSizeStr );
    size_t arrSize = atoi( arrSizeStr.c_str());

    MLANG_String arrName = typeStr2.substring( 0, pos1-2 );
    constructType( t, arrName );

    Array * a = new Array( t,
                           arrSize ,
                           typeid(char[1]) ); // fixme how to retrieve the array typeinfo
    type = a->type();
                          
  }
  else {
    // if everything failes let's at least construct a Type
    type = new Type( typeStr.c_str(), 0 );
  }
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::Tools::demangle( const std::type_info & ti ) { 
//-------------------------------------------------------------------------------
#ifdef _WIN32
  static std::vector<MLANG_String> keywords;
  if ( 0 == keywords.size() ) {
    keywords.push_back("class ");
    keywords.push_back("struct ");
    keywords.push_back("enum ");
    keywords.push_back("union ");
    keywords.push_back("__cdecl");
  }
  MLANG_String r = ti.name();
  for ( size_t i = 0; i < keywords.size(); i ++ ) {
    while (r.find(keywords[i]) != MLANG_String::npos) 
      r = r.replace(r.find(keywords[i]), keywords[i].size(), "");
    while (r.find(" *") != MLANG_String::npos) 
      r = r.replace(r.find(" *"), 2, "*");
    while (r.find(" &") != MLANG_String::npos) 
      r = r.replace(r.find(" &"), 2, "&");
  }
  return r;

#else

  int status = 0;
  bool remove_additional_pointer = false;
  MLANG_String  mangled = ti.name();

#if __GNUC__ <= 3 && __GNUC_MINOR__ <= 2 || defined(__APPLE__)
  // Function types are not decoded at all. We are an extra 'P' to convert it to a pointer
  // and remove it at the end.
  if ( mangled[0] == 'F' ) {
    mangled = "P" + mangled;
    remove_additional_pointer = true;
  }
#endif
  char * c_demangled = abi::__cxa_demangle( mangled.c_str(), 0, 0, & status );
  if ( status == -1 ) {
    throw RuntimeError("Memory allocation failure while demangling ");
  }
  else if ( status == -2 ) {
    throw RuntimeError( MLANG_String(mangled) + " is not a valid name under the C++ ABI");
  }
  else if ( status == -3 )
   {
    throw RuntimeError( MLANG_String("Failure while demangling ") + mangled +
            ". One of the arguments is invalid ");
   }
  else
   {
    int pos;
    MLANG_String demangled = c_demangled;
    free( c_demangled );
    if ( remove_additional_pointer )
     {
      int pos = demangled.indexOf("(*)");
      MLANG_String temp = demangled.substring (0, pos - 1);
      temp += demangled.substring (pos + 3);
      demangled = temp;
     }
    while ( (pos=demangled.indexOf(", ")) != MLANG_String::npos )
     {
      MLANG_String temp;
      if (pos!=0)
       {
         temp += demangled.substring (0, pos - 1);
       }
      temp += ",";
      temp += demangled.substring (pos + 2);
      demangled = temp;
     }
    return demangled;
  }
#endif
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::stringSplitPair( MLANG_String & val1,
                                            MLANG_String & val2,
                                            const MLANG_String & str,
                                            const MLANG_String & delim ) { 
//-------------------------------------------------------------------------------
  MLANG_String str2 = str;
  size_t pos = str2.lastIndexOf( delim );
  if ( (int)pos != MLANG_String::npos ) { 
    val1 = str2.substring( 0, pos - 1 ); 
    val2 = str2.substring( pos + delim.length());
  }
  else { 
    val1 = str2; 
  }
  stringStrip( val1 );
  stringStrip( val2 );
}


//-------------------------------------------------------------------------------
void MLANG_Reflection::Tools::stringStrip( MLANG_String & str ) {
//-------------------------------------------------------------------------------
  size_t sPos = 0;
  size_t ePos = str.length();
  while ( str[sPos] == ' ' ) { ++sPos; }
  while ( str[ePos] == ' ' ) { --ePos; }
  str = str.substring( sPos, ePos - 1 );
}


//-------------------------------------------------------------------------------
MLANG_String MLANG_Reflection::Tools::joinTypeNames(std::vector<const Type*>& types) {
//-------------------------------------------------------------------------------
  std::vector<const Type*>::iterator it;
  MLANG_String result;
  for ( it = types.begin(); it != types.end(); ) {
    result += (*it)->fullName();
    if ( ++it != types.end() ) result += ",";
  }
  return result;
}
