//$Id: Tools.h,v 1.4 2004/10/07 10:03:42 roiser Exp $

#ifndef MLANG_REFLECTION_TOOLS
#define MLANG_REFLECTION_TOOLS

// Include files
#include "MLANG/String.h"
#include <vector>
#include <typeinfo>

namespace MLANG_Reflection

{

    // forward declarations
    class Function;
    class Type;

   namespace Tools {

      /**
       * argsEqual will check the equality of the parameters of a function with
       * a string containing representations of parameters
       * @param mf pointer to function
       * @param argstring string containing parameter representations
       * @return true if arguments are equal
       */
      bool argsEqual( const Function * mf,
                      const MLANG_String & argstring );


      /**
       * checkParamName will check a parameter whether it contains a
       * default value
       * @param  type the string representation of the type
       * @param  name the name of the parameter
       * @param  defaultVal returns the default value if any
       * @param  paramString the string contatining the parameter
       */
      void checkParamName( MLANG_String & type,
                           MLANG_String & name,
                                  MLANG_String & defaultVal,
                                  MLANG_String & paramString );


      /**
       * constructType will construct a type from a string
       * @param  type returns the pointer to the outermost type
       * @param  typeStr is teh string representation of the type
       */
      void constructType( const Type * & type, 
                          const MLANG_String & typeStr );


      /**
       * demangle will call the internal demangling routines and
       * return the demangled string of a type 
       * @param ti the mangled type string
       * @return the demangled string
       */
      MLANG_String demangle( const std::type_info & ti );


      /**
       * joinTypeNames will join the types names with a comma
       * separation
       * @types vector of Types
       * @return joined string
       */

      MLANG_String joinTypeNames(std::vector<const Type*>& types);
 


      /**
       * stringSplit will return a vector of splitted strings
       * @param  splitValues returns the vector with splitted strings
       * @param  str the input string
       * @param  delim the delimiter on which to split 
       */
      void stringSplit(std::vector <MLANG_String> & splitValues,
                       const MLANG_String & str, 
                       const MLANG_String & delim = ",");


      /** 
       * stringSplitPair will return two values which are split
       * @param  val1 returns the first value
       * @param  val2 returns the second value
       * @param  str the string to be split
       * @param  delim the delimiter on which to split
       */
      void stringSplitPair(MLANG_String & val1,
                           MLANG_String & val2,
                           const MLANG_String & str,
                           const MLANG_String & delim = ",");


      /**
       * stringStrip will strip off empty spaces of a string
       * @param str a reference to a string to be stripped
       */
      void stringStrip(MLANG_String & str);

   
    } // namespace Tools
} // namespace MLANG_Reflection

#endif // MLANG_REFLECTION_TOOLS
